@@ -24,26 +24,31 @@
 #include "dropshortcut.h"
 #include "emoteshortcut.h"
 #include "game.h"
+#include "gamemodifiers.h"
 #include "itemshortcut.h"
+#include "settings.h"
 #include "soundmanager.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 #include "being/playerrelations.h"
 
-#include "input/inputevent.h"
 #include "input/inputmanager.h"
 
+#include "gui/dialogsmanager.h"
+#include "gui/gui.h"
+#include "gui/popupmanager.h"
 #include "gui/sdlinput.h"
 #include "gui/viewport.h"
 
 #include "gui/windows/skilldialog.h"
 #include "gui/windows/socialwindow.h"
 #include "gui/windows/statuswindow.h"
+#include "gui/windows/tradewindow.h"
 #include "gui/windows/questswindow.h"
 #include "gui/windows/quitdialog.h"
 #include "gui/windows/whoisonline.h"
-
 #include "gui/windows/botcheckerwindow.h"
 #include "gui/windows/buyselldialog.h"
 #include "gui/windows/chatwindow.h"
@@ -54,12 +59,12 @@
 #include "gui/windows/inventorywindow.h"
 #include "gui/windows/killstats.h"
 #include "gui/windows/minimap.h"
-#include "gui/windows/ministatuswindow.h"
 #include "gui/windows/npcdialog.h"
 #include "gui/windows/outfitwindow.h"
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 #include "gui/windows/shopwindow.h"
 #include "gui/windows/shortcutwindow.h"
+#include "gui/windows/updaterwindow.h"
 
 #include "gui/widgets/tabs/chattab.h"
 
@@ -67,6 +72,11 @@
 
 #include "net/net.h"
 #include "net/playerhandler.h"
+#include "net/tradehandler.h"
+
+#include "listeners/updatestatuslistener.h"
+
+#include "resources/map/map.h"
 
 #include "utils/gettext.h"
 
@@ -81,7 +91,22 @@
 #define impHandler(name) bool name(const InputEvent &event)
 #define impHandler0(name) bool name(const InputEvent &event A_UNUSED)
 
+#define callYellowBar(name) \
+    GameModifiers::name(!inputManager.isActionActive( \
+        InputAction::STOP_ATTACK)); \
+    return true;
+
+#define callYellowBarCond(name) \
+    if (!settings.disableGameModifiers) \
+    { \
+        GameModifiers::name(!inputManager.isActionActive( \
+            InputAction::STOP_ATTACK)); \
+        return true; \
+    } \
+    return false;
+
 extern ShortcutWindow *spellShortcutWindow;
+extern std::string tradePartnerName;
 extern QuitDialog *quitDialog;
 
 namespace ActionManager
@@ -107,56 +132,56 @@ static bool closeMoveNpcDialog(bool focus)
 
 impHandler(moveUp)
 {
-    if (inputManager.isActionActive(Input::KEY_EMOTE))
+    if (inputManager.isActionActive(InputAction::EMOTE))
         return directUp(event);
     return closeMoveNpcDialog(false);
 }
 
 impHandler(moveDown)
 {
-    if (inputManager.isActionActive(Input::KEY_EMOTE))
+    if (inputManager.isActionActive(InputAction::EMOTE))
         return directDown(event);
     return closeMoveNpcDialog(false);
 }
 
 impHandler(moveLeft)
 {
-    if (outfitWindow && inputManager.isActionActive(Input::KEY_WEAR_OUTFIT))
+    if (outfitWindow && inputManager.isActionActive(InputAction::WEAR_OUTFIT))
     {
         outfitWindow->wearPreviousOutfit();
         if (Game::instance())
             Game::instance()->setValidSpeed();
         return true;
     }
-    if (inputManager.isActionActive(Input::KEY_EMOTE))
+    if (inputManager.isActionActive(InputAction::EMOTE))
         return directLeft(event);
     return closeMoveNpcDialog(false);
 }
 
 impHandler(moveRight)
 {
-    if (outfitWindow && inputManager.isActionActive(Input::KEY_WEAR_OUTFIT))
+    if (outfitWindow && inputManager.isActionActive(InputAction::WEAR_OUTFIT))
     {
         outfitWindow->wearNextOutfit();
         if (Game::instance())
             Game::instance()->setValidSpeed();
         return true;
     }
-    if (inputManager.isActionActive(Input::KEY_EMOTE))
+    if (inputManager.isActionActive(InputAction::EMOTE))
         return directRight(event);
     return closeMoveNpcDialog(false);
 }
 
 impHandler(moveForward)
 {
-    if (inputManager.isActionActive(Input::KEY_EMOTE))
+    if (inputManager.isActionActive(InputAction::EMOTE))
         return directRight(event);
     return closeMoveNpcDialog(false);
 }
 
 impHandler(emote)
 {
-    const int emotion = 1 + event.action - Input::KEY_EMOTE_1;
+    const int emotion = 1 + event.action - InputAction::EMOTE_1;
     if (emotion > 0)
     {
         if (emoteShortcut)
@@ -171,7 +196,7 @@ impHandler(emote)
 
 impHandler(moveToPoint)
 {
-    const int num = event.action - Input::KEY_MOVE_TO_POINT_1;
+    const int num = event.action - InputAction::MOVE_TO_POINT_1;
     if (socialWindow && num >= 0)
     {
         socialWindow->selectPortal(num);
@@ -183,9 +208,9 @@ impHandler(moveToPoint)
 
 impHandler(outfit)
 {
-    if (inputManager.isActionActive(Input::KEY_WEAR_OUTFIT))
+    if (inputManager.isActionActive(InputAction::WEAR_OUTFIT))
     {
-        const int num = event.action - Input::KEY_OUTFIT_1;
+        const int num = event.action - InputAction::OUTFIT_1;
         if (outfitWindow && num >= 0)
         {
             outfitWindow->wearOutfit(num);
@@ -194,9 +219,9 @@ impHandler(outfit)
             return true;
         }
     }
-    else if (inputManager.isActionActive(Input::KEY_COPY_OUTFIT))
+    else if (inputManager.isActionActive(InputAction::COPY_OUTFIT))
     {
-        const int num = event.action - Input::KEY_OUTFIT_1;
+        const int num = event.action - InputAction::OUTFIT_1;
         if (outfitWindow && num >= 0)
         {
             outfitWindow->copyOutfit(num);
@@ -211,12 +236,12 @@ impHandler(outfit)
 
 impHandler0(mouseClick)
 {
-    if (!guiInput)
+    if (!guiInput || !gui)
         return false;
 
     int mouseX, mouseY;
-    SDL_GetMouseState(&mouseX, &mouseY);
-    guiInput->simulateMouseClick(mouseX, mouseY, gcn::MouseEvent::RIGHT);
+    Gui::getMouseState(&mouseX, &mouseY);
+    guiInput->simulateMouseClick(mouseX, mouseY, MouseButton::RIGHT);
     return true;
 }
 
@@ -231,12 +256,12 @@ impHandler0(ok)
     // Close the config window, cancelling changes if opened
     else if (setupWindow && setupWindow->isWindowVisible())
     {
-        setupWindow->action(gcn::ActionEvent(nullptr, "cancel"));
+        setupWindow->action(ActionEvent(nullptr, "cancel"));
         return true;
     }
     else if (NpcDialog *const dialog = NpcDialog::getActive())
     {
-        dialog->action(gcn::ActionEvent(nullptr, "ok"));
+        dialog->action(ActionEvent(nullptr, "ok"));
         return true;
     }
     return false;
@@ -252,7 +277,7 @@ impHandler(shortcut)
             if (itemShortcut[num])
             {
                 itemShortcut[num]->useItem(event.action
-                    - Input::KEY_SHORTCUT_1);
+                    - InputAction::SHORTCUT_1);
             }
         }
         return true;
@@ -339,9 +364,9 @@ impHandler0(quit)
 {
     if (!Game::instance())
         return false;
-    if (viewport && viewport->isPopupMenuVisible())
+    if (popupManager && popupManager->isPopupMenuVisible())
     {
-        viewport->closePopupMenu();
+        popupManager->closePopupMenu();
         return true;
     }
     else if (!quitDialog)
@@ -376,27 +401,22 @@ impHandler0(dropItem)
 
 impHandler0(switchQuickDrop)
 {
-    if (player_node && !player_node->getDisableGameModifiers())
-    {
-        player_node->changeQuickDropCounter();
-        return true;
-    }
-    return false;
+    callYellowBarCond(changeQuickDropCounter)
 }
 
 impHandler0(heal)
 {
     if (actorManager)
     {
-        if (inputManager.isActionActive(Input::KEY_MOD))
+        if (inputManager.isActionActive(InputAction::STOP_ATTACK))
         {
-            Being *target = player_node->getTarget();
-            if (!target || target->getType() != ActorSprite::PLAYER)
+            Being *target = localPlayer->getTarget();
+            if (!target || target->getType() != ActorType::PLAYER)
             {
                 target = actorManager->findNearestLivingBeing(
-                    player_node, 10, ActorSprite::PLAYER);
+                    localPlayer, 10, ActorType::PLAYER, true);
                 if (target)
-                    player_node->setTarget(target);
+                    localPlayer->setTarget(target);
             }
         }
 
@@ -410,9 +430,9 @@ impHandler0(heal)
 
 impHandler0(crazyMoves)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->crazyMove();
+        localPlayer->crazyMove();
         return true;
     }
     return false;
@@ -423,7 +443,7 @@ impHandler0(itenplz)
     if (actorManager)
     {
         if (Net::getPlayerHandler() && Net::getPlayerHandler()->canUseMagic()
-            && PlayerInfo::getAttribute(PlayerInfo::MP) >= 3)
+            && PlayerInfo::getAttribute(Attributes::MP) >= 3)
         {
             actorManager->itenplz();
         }
@@ -434,30 +454,20 @@ impHandler0(itenplz)
 
 impHandler0(changeCrazyMove)
 {
-    if (player_node)
-    {
-        player_node->changeCrazyMoveType();
-        return true;
-    }
-    return false;
+    callYellowBar(changeCrazyMoveType);
 }
 
 impHandler0(changePickupType)
 {
-    if (player_node)
-    {
-        player_node->changePickUpType();
-        return true;
-    }
-    return false;
+    callYellowBar(changePickUpType);
 }
 
 impHandler0(moveToTarget)
 {
-    if (player_node && !inputManager.isActionActive(Input::KEY_TARGET_ATTACK)
-        && !inputManager.isActionActive(Input::KEY_ATTACK))
+    if (localPlayer && !inputManager.isActionActive(InputAction::TARGET_ATTACK)
+        && !inputManager.isActionActive(InputAction::ATTACK))
     {
-        player_node->moveToTarget();
+        localPlayer->moveToTarget();
         return true;
     }
     return false;
@@ -465,10 +475,10 @@ impHandler0(moveToTarget)
 
 impHandler0(moveToHome)
 {
-    if (player_node && !inputManager.isActionActive(Input::KEY_TARGET_ATTACK)
-        && !inputManager.isActionActive(Input::KEY_ATTACK))
+    if (localPlayer && !inputManager.isActionActive(InputAction::TARGET_ATTACK)
+        && !inputManager.isActionActive(InputAction::ATTACK))
     {
-        player_node->moveToHome();
+        localPlayer->moveToHome();
         if (Game::instance())
             Game::instance()->setValidSpeed();
         return true;
@@ -478,9 +488,9 @@ impHandler0(moveToHome)
 
 impHandler0(setHome)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->setHome();
+        localPlayer->setHome();
         return true;
     }
     return false;
@@ -488,92 +498,52 @@ impHandler0(setHome)
 
 impHandler0(changeMoveType)
 {
-    if (player_node)
-    {
-        player_node->invertDirection();
-        return true;
-    }
-    return false;
+    callYellowBar(changeMoveType);
 }
 
 impHandler0(changeAttackWeaponType)
 {
-    if (player_node)
-    {
-        player_node->changeAttackWeaponType();
-        return true;
-    }
-    return false;
+    callYellowBar(changeAttackWeaponType);
 }
 
 impHandler0(changeAttackType)
 {
-    if (player_node)
-    {
-        player_node->changeAttackType();
-        return true;
-    }
-    return false;
+    callYellowBar(changeAttackType);
 }
 
 impHandler0(changeFollowMode)
 {
-    if (player_node)
-    {
-        player_node->changeFollowMode();
-        return true;
-    }
-    return false;
+    callYellowBar(changeFollowMode);
 }
 
 impHandler0(changeImitationMode)
 {
-    if (player_node)
-    {
-        player_node->changeImitationMode();
-        return true;
-    }
-    return false;
+    callYellowBar(changeImitationMode);
 }
 
 impHandler0(magicAttack)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->magicAttack();
+        localPlayer->magicAttack();
         return true;
     }
     return false;
 }
 
-impHandler0(changeMagicAttack)
+impHandler0(changeMagicAttackType)
 {
-    if (player_node)
-    {
-        player_node->switchMagicAttack();
-        return true;
-    }
-    return false;
+    callYellowBar(changeMagicAttackType);
 }
 
 impHandler0(changePvpMode)
 {
-    if (player_node)
-    {
-        player_node->switchPvpAttack();
-        return true;
-    }
-    return false;
+    callYellowBar(changePvpAttackType);
 }
 
 impHandler0(changeMoveToTarget)
 {
-    if (player_node)
-    {
-        player_node->changeMoveToTargetType();
-        return true;
-    }
-    return false;
+    callYellowBar(changeMoveToTargetType);
 }
 
 impHandler0(copyEquippedToOutfit)
@@ -588,9 +558,9 @@ impHandler0(copyEquippedToOutfit)
 
 impHandler0(changeGameModifier)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->switchGameModifiers();
+        GameModifiers::changeGameModifiers();
         return true;
     }
     return false;
@@ -599,15 +569,17 @@ impHandler0(changeGameModifier)
 impHandler0(changeAudio)
 {
     soundManager.changeAudio();
+    if (localPlayer)
+        localPlayer->updateMusic();
     return true;
 }
 
 impHandler0(away)
 {
-    if (player_node)
+    GameModifiers::changeAwayMode();
+    if (localPlayer)
     {
-        player_node->changeAwayMode();
-        player_node->updateStatus();
+        localPlayer->updateStatus();
         if (Game::instance())
             Game::instance()->setValidSpeed();
         return true;
@@ -647,22 +619,27 @@ impHandler0(setupWindowShow)
 
 impHandler0(pickup)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->pickUpItems();
+        localPlayer->pickUpItems();
         return true;
     }
     return false;
 }
 
+static void doSit()
+{
+    if (inputManager.isActionActive(InputAction::EMOTE))
+        localPlayer->updateSit();
+    else
+        localPlayer->toggleSit();
+}
+
 impHandler0(sit)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        if (inputManager.isActionActive(Input::KEY_EMOTE))
-            player_node->updateSit();
-        else
-            player_node->toggleSit();
+        doSit();
         return true;
     }
     return false;
@@ -670,48 +647,22 @@ impHandler0(sit)
 
 impHandler0(hideWindows)
 {
-    if (statusWindow)
-        statusWindow->setVisible(false);
-    if (inventoryWindow)
-        inventoryWindow->setVisible(false);
-    if (shopWindow)
-        shopWindow->setVisible(false);
-    if (skillDialog)
-        skillDialog->setVisible(false);
     if (setupWindow)
-        setupWindow->setVisible(false);
-    if (equipmentWindow)
-        equipmentWindow->setVisible(false);
-    if (helpWindow)
-        helpWindow->setVisible(false);
-    if (debugWindow)
-        debugWindow->setVisible(false);
-    if (outfitWindow)
-        outfitWindow->setVisible(false);
-    if (dropShortcutWindow)
-        dropShortcutWindow->setVisible(false);
-    if (spellShortcutWindow)
-        spellShortcutWindow->setVisible(false);
-    if (botCheckerWindow)
-        botCheckerWindow->setVisible(false);
-    if (socialWindow)
-        socialWindow->setVisible(false);
-    if (questsWindow)
-        questsWindow->setVisible(false);
+        setupWindow->hideWindows();
     return true;
 }
 
-impHandler0(helpWindowShow)
+static bool showHelpPage(const std::string &page, const bool showHide)
 {
     if (helpWindow)
     {
-        if (helpWindow->isWindowVisible())
+        if (showHide && helpWindow->isWindowVisible())
         {
             helpWindow->setVisible(false);
         }
         else
         {
-            helpWindow->loadHelp("index");
+            helpWindow->loadHelp(page);
             helpWindow->requestMoveToTop();
         }
         return true;
@@ -719,6 +670,16 @@ impHandler0(helpWindowShow)
     return false;
 }
 
+impHandler0(helpWindowShow)
+{
+    return showHelpPage("index", true);
+}
+
+impHandler0(aboutWindowShow)
+{
+    return showHelpPage("about", false);
+}
+
 static void showHideWindow(Window *const window)
 {
     if (window)
@@ -847,12 +808,20 @@ impHandler0(questsWindowShow)
     return true;
 }
 
+impHandler0(updaterWindowShow)
+{
+    if (updaterWindow)
+        updaterWindow->deleteSelf();
+    else
+        DialogsManager::createUpdaterWindow();
+    return true;
+}
+
 impHandler0(changeMapMode)
 {
     if (viewport)
-        viewport->toggleDebugPath();
-    if (miniStatusWindow)
-        miniStatusWindow->updateStatus();
+        viewport->toggleMapDrawType();
+    UpdateStatusListener::distributeEvent();
     if (Game::instance())
     {
         if (Map *const map = Game::instance()->getCurrentMap())
@@ -870,7 +839,7 @@ impHandler0(changeTrade)
         {
             // TRANSLATORS: disable trades message
             localChatTab->chatLog(_("Ignoring incoming trade requests"),
-                BY_SERVER);
+                ChatMsgType::BY_SERVER);
         }
         deflt &= ~PlayerRelation::TRADE;
     }
@@ -880,7 +849,7 @@ impHandler0(changeTrade)
         {
             // TRANSLATORS: enable trades message
             localChatTab->chatLog(_("Accepting incoming trade requests"),
-                BY_SERVER);
+                ChatMsgType::BY_SERVER);
         }
         deflt |= PlayerRelation::TRADE;
     }
@@ -902,15 +871,15 @@ impHandler0(ignoreInput)
 
 impHandler0(directUp)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        if (player_node->getDirection() != Being::UP)
+        if (localPlayer->getDirection() != BeingDirection::UP)
         {
-//            if (client->limitPackets(PACKET_DIRECTION))
+//            if (PacketLimiter::limitPackets(PACKET_DIRECTION))
             {
-                player_node->setDirection(Being::UP);
+                localPlayer->setDirection(BeingDirection::UP);
                 if (Net::getPlayerHandler())
-                    Net::getPlayerHandler()->setDirection(Being::UP);
+                    Net::getPlayerHandler()->setDirection(BeingDirection::UP);
             }
         }
         return true;
@@ -920,15 +889,18 @@ impHandler0(directUp)
 
 impHandler0(directDown)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        if (player_node->getDirection() != Being::DOWN)
+        if (localPlayer->getDirection() != BeingDirection::DOWN)
         {
-//            if (client->limitPackets(PACKET_DIRECTION))
+//            if (PacketLimiter::limitPackets(PACKET_DIRECTION))
             {
-                player_node->setDirection(Being::DOWN);
+                localPlayer->setDirection(BeingDirection::DOWN);
                 if (Net::getPlayerHandler())
-                    Net::getPlayerHandler()->setDirection(Being::DOWN);
+                {
+                    Net::getPlayerHandler()->setDirection(
+                        BeingDirection::DOWN);
+                }
             }
         }
         return true;
@@ -938,15 +910,18 @@ impHandler0(directDown)
 
 impHandler0(directLeft)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        if (player_node->getDirection() != Being::LEFT)
+        if (localPlayer->getDirection() != BeingDirection::LEFT)
         {
-//            if (client->limitPackets(PACKET_DIRECTION))
+//            if (PacketLimiter::limitPackets(PACKET_DIRECTION))
             {
-                player_node->setDirection(Being::LEFT);
+                localPlayer->setDirection(BeingDirection::LEFT);
                 if (Net::getPlayerHandler())
-                    Net::getPlayerHandler()->setDirection(Being::LEFT);
+                {
+                    Net::getPlayerHandler()->setDirection(
+                        BeingDirection::LEFT);
+                }
             }
         }
         return true;
@@ -956,15 +931,18 @@ impHandler0(directLeft)
 
 impHandler0(directRight)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        if (player_node->getDirection() != Being::RIGHT)
+        if (localPlayer->getDirection() != BeingDirection::RIGHT)
         {
-//            if (client->limitPackets(PACKET_DIRECTION))
+//            if (PacketLimiter::limitPackets(PACKET_DIRECTION))
             {
-                player_node->setDirection(Being::RIGHT);
+                localPlayer->setDirection(BeingDirection::RIGHT);
                 if (Net::getPlayerHandler())
-                    Net::getPlayerHandler()->setDirection(Being::RIGHT);
+                {
+                    Net::getPlayerHandler()->setDirection(
+                        BeingDirection::RIGHT);
+                }
             }
         }
         return true;
@@ -974,18 +952,18 @@ impHandler0(directRight)
 
 impHandler0(talk)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        Being *target = player_node->getTarget();
+        Being *target = localPlayer->getTarget();
         if (!target && actorManager)
         {
             target = actorManager->findNearestLivingBeing(
-                player_node, 1, ActorSprite::NPC);
+                localPlayer, 1, ActorType::NPC, true);
             // ignore closest target if distance in each direction more than 1
             if (target)
             {
-                if (abs(target->getTileX() - player_node->getTileX()) > 1
-                    || abs(target->getTileY() - player_node->getTileY()) > 1)
+                if (abs(target->getTileX() - localPlayer->getTileX()) > 1
+                    || abs(target->getTileY() - localPlayer->getTileY()) > 1)
                 {
                     return true;
                 }
@@ -995,7 +973,7 @@ impHandler0(talk)
         {
             if (target->canTalk())
                 target->talkTo();
-            else if (target->getType() == Being::PLAYER)
+            else if (target->getType() == ActorType::PLAYER)
                 new BuySellDialog(target->getName());
         }
         return true;
@@ -1005,11 +983,11 @@ impHandler0(talk)
 
 impHandler0(stopAttack)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->stopAttack();
+        localPlayer->stopAttack();
         // not consume if target attack key pressed
-        if (inputManager.isActionActive(Input::KEY_TARGET_ATTACK))
+        if (inputManager.isActionActive(InputAction::TARGET_ATTACK))
             return false;
         return true;
     }
@@ -1018,9 +996,9 @@ impHandler0(stopAttack)
 
 impHandler0(untarget)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->untarget();
+        localPlayer->untarget();
         return true;
     }
     return false;
@@ -1028,11 +1006,11 @@ impHandler0(untarget)
 
 impHandler0(attack)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        Being *const target = player_node->getTarget();
+        Being *const target = localPlayer->getTarget();
         if (target)
-            player_node->attack(target, true);
+            localPlayer->attack(target, true);
         return true;
     }
     return false;
@@ -1040,39 +1018,39 @@ impHandler0(attack)
 
 impHandler0(targetAttack)
 {
-    if (player_node && actorManager)
+    if (localPlayer && actorManager)
     {
         Being *target = nullptr;
 
         const bool newTarget = !inputManager.isActionActive(
-            Input::KEY_STOP_ATTACK);
+            InputAction::STOP_ATTACK);
         // A set target has highest priority
-        if (!player_node->getTarget())
+        if (!localPlayer->getTarget())
         {
             // Only auto target Monsters
             target = actorManager->findNearestLivingBeing(
-                player_node, 90, ActorSprite::MONSTER);
+                localPlayer, 90, ActorType::MONSTER, true);
         }
         else
         {
-            target = player_node->getTarget();
+            target = localPlayer->getTarget();
         }
 
-        player_node->attack2(target, newTarget);
+        localPlayer->attack2(target, newTarget);
         return true;
     }
     return false;
 }
 
-static bool setTarget(const ActorSprite::Type type)
+static bool setTarget(const ActorType::Type type, const bool allowSort)
 {
-    if (actorManager && player_node)
+    if (actorManager && localPlayer)
     {
         Being *const target = actorManager->findNearestLivingBeing(
-            player_node, 20, type);
+            localPlayer, 20, type, allowSort);
 
-        if (target && target != player_node->getTarget())
-            player_node->setTarget(target);
+        if (target && target != localPlayer->getTarget())
+            localPlayer->setTarget(target);
 
         return true;
     }
@@ -1081,17 +1059,22 @@ static bool setTarget(const ActorSprite::Type type)
 
 impHandler0(targetPlayer)
 {
-    return setTarget(ActorSprite::PLAYER);
+    return setTarget(ActorType::PLAYER, true);
 }
 
 impHandler0(targetMonster)
 {
-    return setTarget(ActorSprite::MONSTER);
+    return setTarget(ActorType::MONSTER, true);
+}
+
+impHandler0(targetClosestMonster)
+{
+    return setTarget(ActorType::MONSTER, false);
 }
 
 impHandler0(targetNPC)
 {
-    return setTarget(ActorSprite::NPC);
+    return setTarget(ActorType::NPC, true);
 }
 
 impHandler0(safeVideoMode)
@@ -1104,18 +1087,16 @@ impHandler0(safeVideoMode)
 
 impHandler0(stopSit)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->stopAttack();
+        localPlayer->stopAttack();
         // not consume if target attack key pressed
-        if (inputManager.isActionActive(Input::KEY_TARGET_ATTACK))
+        if (inputManager.isActionActive(InputAction::TARGET_ATTACK))
             return false;
-        if (!player_node->getTarget())
+        if (!localPlayer->getTarget())
         {
-            if (inputManager.isActionActive(Input::KEY_EMOTE))
-                player_node->updateSit();
-            else
-                player_node->toggleSit();
+            doSit();
+            return true;
         }
         return true;
     }
@@ -1141,9 +1122,63 @@ impHandler0(showKeyboard)
 
 impHandler0(showWindows)
 {
-    if (viewport)
+    if (popupManager)
     {
-        viewport->showWindowsPopup();
+        popupManager->showWindowsPopup();
+        return true;
+    }
+    return false;
+}
+
+impHandler0(nextShortcutsTab)
+{
+    if (itemShortcutWindow)
+    {
+        itemShortcutWindow->nextTab();
+        return true;
+    }
+    return false;
+}
+
+impHandler0(prevShortcutsTab)
+{
+    if (itemShortcutWindow)
+    {
+        itemShortcutWindow->prevTab();
+        return true;
+    }
+    return false;
+}
+
+impHandler0(nextCommandsTab)
+{
+    if (spellShortcutWindow)
+    {
+        spellShortcutWindow->nextTab();
+        return true;
+    }
+    return false;
+}
+
+impHandler0(prevCommandsTab)
+{
+    if (spellShortcutWindow)
+    {
+        spellShortcutWindow->prevTab();
+        return true;
+    }
+    return false;
+}
+
+impHandler0(openTrade)
+{
+    const Being *const being = localPlayer->getTarget();
+    if (being && being->getType() == ActorType::PLAYER)
+    {
+        Net::getTradeHandler()->request(being);
+        tradePartnerName = being->getName();
+        if (tradeWindow)
+            tradeWindow->clear();
         return true;
     }
     return false;
