@@ -64,7 +64,7 @@ namespace ActionManager
     decHandler(changeFollowMode);
     decHandler(changeImitationMode);
     decHandler(magicAttack);
-    decHandler(changeMagicAttack);
+    decHandler(changeMagicAttackType);
     decHandler(changePvpMode);
     decHandler(changeMoveToTarget);
     decHandler(copyEquippedToOutfit);
@@ -89,14 +89,21 @@ namespace ActionManager
     decHandler(targetAttack);
     decHandler(targetPlayer);
     decHandler(targetMonster);
+    decHandler(targetClosestMonster);
     decHandler(targetNPC);
     decHandler(safeVideoMode);
     decHandler(stopSit);
     decHandler(showKeyboard);
     decHandler(showWindows);
+    decHandler(nextShortcutsTab);
+    decHandler(prevShortcutsTab);
+    decHandler(nextCommandsTab);
+    decHandler(prevCommandsTab);
+    decHandler(openTrade);
 
     decHandler(hideWindows);
     decHandler(helpWindowShow);
+    decHandler(aboutWindowShow);
     decHandler(setupWindowShow);
     decHandler(statusWindowShow);
     decHandler(inventoryWindowShow);
@@ -117,6 +124,7 @@ namespace ActionManager
     decHandler(whoIsOnlineWindowShow);
     decHandler(didYouKnowWindowShow);
     decHandler(questsWindowShow);
+    decHandler(updaterWindowShow);
 }  // namespace ActionManager
 
 #undef decHandler
