@@ -25,10 +25,13 @@
 
 #include "flooritem.h"
 
-#include "being/being.h"
+#include "listeners/configlistener.h"
+
+#include "utils/stringvector.h"
 
 #include "localconsts.h"
 
+class Being;
 class LocalPlayer;
 class Map;
 
@@ -58,9 +61,14 @@ class ActorManager final: public ConfigListener
         /**
          * Create a Being and add it to the list of ActorSprites.
          */
-        Being *createBeing(const int id, const ActorSprite::Type type,
+        Being *createBeing(const int id,
+                           const ActorType::Type type,
                            const uint16_t subtype) A_WARN_UNUSED;
 
+        Being *cloneBeing(const Being *const srcBeing,
+                          const int dx, const int dy,
+                          const int id);
+
         /**
          * Create a FloorItem and add it to the list of ActorSprites.
          */
@@ -87,8 +95,8 @@ class ActorManager final: public ConfigListener
         /**
          * Returns a being at specific coordinates.
          */
-        Being *findBeing(const int x, const int y, const ActorSprite::Type
-                         type = ActorSprite::UNKNOWN) const A_WARN_UNUSED;
+        Being *findBeing(const int x, const int y, const ActorType::Type
+                         type = ActorType::UNKNOWN) const A_WARN_UNUSED;
 
         /**
          * Returns a being at the specific pixel.
@@ -130,10 +138,9 @@ class ActorManager final: public ConfigListener
          */
         Being *findNearestLivingBeing(const int x, const int y,
                                       int maxTileDist,
-                                      const ActorSprite::Type
-                                      type = Being::UNKNOWN,
-                                      const Being *const
-                                      excluded = nullptr) const A_WARN_UNUSED;
+                                      const ActorType::Type type,
+                                      const Being *const excluded)
+                                      const A_WARN_UNUSED;
 
         /**
          * Returns a being nearest to another being.
@@ -145,23 +152,24 @@ class ActorManager final: public ConfigListener
          */
         Being *findNearestLivingBeing(const Being *const aroundBeing,
                                       const int maxTileDist,
-                                      const ActorSprite::Type
-                                      type = Being::UNKNOWN)
+                                      const ActorType::Type type,
+                                      const bool allowSort)
                                       const A_WARN_UNUSED;
 
         /**
          * Finds a being by name and (optionally) by type.
          */
         Being *findBeingByName(const std::string &name,
-                               const ActorSprite::Type
-                               type = Being::UNKNOWN) const A_WARN_UNUSED;
+                               const ActorType::Type
+                               type = ActorType::UNKNOWN)
+                               const A_WARN_UNUSED;
 
        /**
         * Finds a nearest being by name and (optionally) by type.
         */
         Being *findNearestByName(const std::string &name,
-                                 const Being::Type &type = Being::UNKNOWN)
-                                 const A_WARN_UNUSED;
+                                 const ActorType::Type &type
+                                 = ActorType::UNKNOWN) const A_WARN_UNUSED;
 
        /**
         * Heal all players in distance.
@@ -170,7 +178,7 @@ class ActorManager final: public ConfigListener
         *                no being is returned
         */
 //        void HealAllTargets(Being *aroundBeing, int maxdist,
-//                                            Being::Type type) const;
+//                                            ActorType::Type type) const;
 
         void healTarget() const;
 
@@ -211,11 +219,11 @@ class ActorManager final: public ConfigListener
 
         void printAllToChat() const;
 
-        void printBeingsToChat(const ActorSprites &beings,
-                               const std::string &header) const;
+        static void printBeingsToChat(const ActorSprites &beings,
+                                      const std::string &header);
 
-        void printBeingsToChat(const std::vector<Being*> &beings,
-                               const std::string &header) const;
+        static void printBeingsToChat(const std::vector<Being*> &beings,
+                                      const std::string &header);
 
         void getPlayerNames(StringVect &names,
                             const bool npcNames) const;
@@ -251,10 +259,10 @@ class ActorManager final: public ConfigListener
 
         void addIgnorePickupItem(const std::string &name);
 
-        void setPriorityAttackMobs(std::list<std::string> mobs)
+        void setPriorityAttackMobs(const std::list<std::string> &mobs)
         { mPriorityAttackMobs = mobs; }
 
-        void setAttackMobs(std::list<std::string> mobs)
+        void setAttackMobs(const std::list<std::string> &mobs)
         { mAttackMobs = mobs; }
 
         int getPriorityAttackMobsSize() const A_WARN_UNUSED
@@ -296,8 +304,9 @@ class ActorManager final: public ConfigListener
 
         int getPickupItemIndex(const std::string &name) const A_WARN_UNUSED;
 
-        int getIndexByName(const std::string &name, const std::map<std::string,
-                           int> &map) const A_WARN_UNUSED;
+        static int getIndexByName(const std::string &name,
+                                  const std::map<std::string, int> &map)
+                                  A_WARN_UNUSED;
 
         bool checkForPickup(const FloorItem *const item) const A_WARN_UNUSED;
 
@@ -307,16 +316,17 @@ class ActorManager final: public ConfigListener
     protected:
         bool validateBeing(const Being *const aroundBeing,
                            Being *const being,
-                           const Being::Type &type,
+                           const ActorType::Type &type,
                            const Being *const excluded = nullptr,
                            const int maxCost = 20) const A_WARN_UNUSED;
 
         Being *findNearestLivingBeing(const Being *const aroundBeing,
                                       const int maxdist,
-                                      const Being::Type &type,
+                                      const ActorType::Type &type,
                                       const int x, const int y,
-                                      const Being *const
-                                      excluded = nullptr) const A_WARN_UNUSED;
+                                      const Being *const excluded,
+                                      const bool allowSort)
+                                      const A_WARN_UNUSED;
 
         void loadAttackList();
 
