@@ -22,29 +22,28 @@
 
 #include "avatar.h"
 
-#include "being/being.h"
-
 #include "debug.h"
 
-Avatar::Avatar(const std::string &name):
-        mId(0),
-        mCharId(0),
-        mName(name),
-        mOriginalName(name),
-        mHp(0),
-        mMaxHp(0),
-        mDamageHp(0),
-        mLevel(1),
-        mMap(),
-        mX(-1),
-        mY(-1),
-        mType(AVATAR_PLAYER),
-        mExp(0),
-        mGender(GENDER_UNSPECIFIED),
-        mRace(-1),
-        mIp(),
-        mOnline(false),
-        mDisplayBold(false)
+Avatar::Avatar(const std::string &name) :
+    mId(0),
+    mCharId(0),
+    mName(name),
+    mOriginalName(name),
+    mHp(0),
+    mMaxHp(0),
+    mDamageHp(0),
+    mLevel(1),
+    mMap(),
+    mX(-1),
+    mY(-1),
+    mType(AVATAR_PLAYER),
+    mExp(0),
+    mGender(Gender::UNSPECIFIED),
+    mRace(-1),
+    mIp(),
+    mOnline(false),
+    mDisplayBold(false),
+    mPoison(false)
 {
 }
 
