@@ -21,23 +21,41 @@
 
 #include "being/actorsprite.h"
 
+#include "animatedsprite.h"
 #include "configuration.h"
 #include "imagesprite.h"
 #include "statuseffect.h"
 
-#include "being/actorspritelistener.h"
 #include "being/localplayer.h"
 
 #include "gui/theme.h"
 
+#include "listeners/actorspritelistener.h"
+
+#include "particle/particle.h"
+
 #include "resources/resourcemanager.h"
+#include "resources/spritereference.h"
 
 #include "utils/checkutils.h"
+#include "utils/delete2.h"
 #include "utils/timer.h"
 
 #include "debug.h"
 
-AnimatedSprite *ActorSprite::targetCursor[NUM_TCT][NUM_TC];
+#define for_each_cursors() \
+    for (int size = TargetCursorSize::SMALL; \
+         size < TargetCursorSize::NUM_TC; \
+         size ++) \
+    { \
+        for (int type = TargetCursorType::NORMAL; \
+             type < TargetCursorType::NUM_TCT; \
+             type ++) \
+
+#define end_foreach }
+
+AnimatedSprite *ActorSprite::targetCursor[TargetCursorType::NUM_TCT]
+    [TargetCursorSize::NUM_TC];
 bool ActorSprite::loaded = false;
 
 ActorSprite::ActorSprite(const int id) :
@@ -53,7 +71,8 @@ ActorSprite::ActorSprite(const int id) :
     mActorSpriteListeners(),
     mCursorPaddingX(0),
     mCursorPaddingY(0),
-    mMustResetParticles(false)
+    mMustResetParticles(false),
+    mPoison(false)
 {
 }
 
@@ -64,8 +83,8 @@ ActorSprite::~ActorSprite()
 
     mUsedTargetCursor = nullptr;
 
-    if (player_node && player_node->getTarget() == this)
-        player_node->setTarget(nullptr);
+    if (localPlayer && localPlayer->getTarget() == this)
+        localPlayer->setTarget(nullptr);
 
     // Notify listeners of the destruction.
     FOR_EACH (ActorSpriteListenerIterator, iter, mActorSpriteListeners)
@@ -130,24 +149,25 @@ void ActorSprite::controlParticle(Particle *const particle)
     mChildParticleEffects.addLocally(particle);
 }
 
-void ActorSprite::setTargetType(const TargetCursorType type)
+void ActorSprite::setTargetType(const TargetCursorType::Type type)
 {
-    static const int targetWidths[ActorSprite::NUM_TC] = {0, 0, 0};
-    static const int targetHeights[ActorSprite::NUM_TC]
-        = {-mapTileSize / 2, -mapTileSize / 2, -mapTileSize};
-
-    if (type == TCT_NONE)
+    if (type == TargetCursorType::NONE)
     {
         untarget();
     }
     else
     {
-        const TargetCursorSize sz = getTargetCursorSize();
-        mUsedTargetCursor = targetCursor[type][sz];
+        const TargetCursorSize::Size sz = getTargetCursorSize();
+        mUsedTargetCursor = targetCursor[static_cast<int>(type)][sz];
         if (mUsedTargetCursor)
         {
-            mCursorPaddingX = targetWidths[sz];
-            mCursorPaddingY = targetHeights[sz];
+            static const int targetWidths[TargetCursorSize::NUM_TC]
+                = {0, 0, 0};
+            static const int targetHeights[TargetCursorSize::NUM_TC]
+                = {-mapTileSize / 2, -mapTileSize / 2, -mapTileSize};
+
+            mCursorPaddingX = static_cast<int>(targetWidths[sz]);
+            mCursorPaddingY = static_cast<int>(targetHeights[sz]);
         }
     }
 }
@@ -193,10 +213,16 @@ void ActorSprite::updateStunMode(const int oldMode, const int newMode)
 
 void ActorSprite::updateStatusEffect(const int index, const bool newStatus)
 {
-    handleStatusEffect(StatusEffect::getStatusEffect(index, newStatus), index);
+    StatusEffect *const effect = StatusEffect::getStatusEffect(
+        index, newStatus);
+    if (!effect)
+        return;
+    if (effect->isPoison() && getType() == ActorType::PLAYER)
+        setPoison(newStatus);
+    handleStatusEffect(effect, index);
 }
 
-void ActorSprite::handleStatusEffect(StatusEffect *const effect,
+void ActorSprite::handleStatusEffect(const StatusEffect *const effect,
                                      const int effectId)
 {
     if (!effect)
@@ -321,10 +347,10 @@ static const char *cursorType(const int type)
 {
     switch (type)
     {
-        case ActorSprite::TCT_IN_RANGE:
+        case TargetCursorType::IN_RANGE:
             return "in-range";
         default:
-        case ActorSprite::TCT_NORMAL:
+        case TargetCursorType::NORMAL:
             return "normal";
     }
 }
@@ -333,12 +359,12 @@ static const char *cursorSize(const int size)
 {
     switch (size)
     {
-        case ActorSprite::TC_LARGE:
+        case TargetCursorSize::LARGE:
             return "l";
-        case ActorSprite::TC_MEDIUM:
+        case TargetCursorSize::MEDIUM:
             return "m";
         default:
-        case ActorSprite::TC_SMALL:
+        case TargetCursorSize::SMALL:
             return "s";
     }
 }
@@ -348,30 +374,23 @@ void ActorSprite::initTargetCursor()
     static const std::string targetCursorFile("target-cursor-%s-%s.xml");
 
     // Load target cursors
-    for (int size = TC_SMALL; size < NUM_TC; size++)
+    for_each_cursors()
     {
-        for (int type = TCT_NORMAL; type < NUM_TCT; type++)
-        {
-            targetCursor[type][size] = AnimatedSprite::load(
-                Theme::resolveThemePath(strprintf(
-                targetCursorFile.c_str(),
-                cursorType(type),
-                cursorSize(size))));
-        }
+        targetCursor[type][size] = AnimatedSprite::load(
+            Theme::resolveThemePath(strprintf(
+            targetCursorFile.c_str(),
+            cursorType(type),
+            cursorSize(size))));
     }
+    end_foreach
 }
 
 void ActorSprite::cleanupTargetCursors()
 {
-    for (int size = TC_SMALL; size < NUM_TC; size++)
+    for_each_cursors()
     {
-        for (int type = TCT_NORMAL; type < NUM_TCT; type++)
-        {
-            if (targetCursor[type][size])
-            {
-                delete targetCursor[type][size];
-                targetCursor[type][size] = nullptr;
-            }
-        }
+        if (targetCursor[type][size])
+            delete2(targetCursor[type][size])
     }
+    end_foreach
 }
