@@ -23,11 +23,17 @@
 #ifndef BEING_BEING_H
 #define BEING_BEING_H
 
-#include "equipment.h"
-
 #include "resources/beinginfo.h"
 
-#include <guichan/color.hpp>
+#include "position.h"
+
+#include "listeners/configlistener.h"
+
+#include "being/beingaction.h"
+#include "being/beingdirection.h"
+#include "being/gender.h"
+
+#include "being/actorsprite.h"
 
 #include <map>
 
@@ -45,6 +51,8 @@ static const int DEFAULT_BEING_HEIGHT = 32;
 
 class AnimatedSprite;
 class BeingCacheEntry;
+class Color;
+class Equipment;
 class FlashText;
 class Guild;
 class ItemInfo;
@@ -57,16 +65,7 @@ struct ParticleInfo;
 
 extern volatile int cur_time;
 
-enum Gender
-{
-    GENDER_MALE = 0,
-    GENDER_FEMALE = 1,
-    GENDER_UNSPECIFIED = 2,
-    GENDER_OTHER = 3
-};
-
-
-struct NextSoundInfo
+struct NextSoundInfo final
 {
     NextSoundInfo() :
         sound(nullptr),
@@ -82,47 +81,13 @@ struct NextSoundInfo
     int time;
 };
 
-class Being : public ActorSprite, public ConfigListener
+class Being notfinal : public ActorSprite,
+                       public ConfigListener
 {
     public:
         friend class BeingEquipBackend;
         friend class LocalPlayer;
 
-        enum FLAGS
-        {
-            FLAG_SHOP = 1,
-            FLAG_AWAY = 2,
-            FLAG_INACTIVE = 4,
-            FLAG_GENDER_OTHER = 32,
-            FLAG_GM = 64,
-            FLAG_GENDER_MALE = 128,
-            FLAG_SPECIAL = 128 + 64
-        };
-
-        /**
-         * Action the being is currently performing
-         * WARNING: Has to be in sync with the same enum in the Being class
-         * of the server!
-         */
-        enum Action
-        {
-            STAND = 0,
-            MOVE,
-            ATTACK,
-            SIT,
-            DEAD,
-            HURT,
-            SPAWN
-        };
-
-        enum Speech
-        {
-            NO_SPEECH = 0,
-            TEXT_OVERHEAD,
-            NO_NAME_IN_BUBBLE,
-            NAME_IN_BUBBLE
-        };
-
         enum AttackType
         {
             HIT = 0x00,
@@ -141,17 +106,6 @@ class Being : public ActorSprite, public ConfigListener
             REACH_NO = 2
         };
 
-        /**
-         * Directions, to be used as bitmask values
-         */
-        enum BeingDirection
-        {
-            DOWN = 1,
-            LEFT = 2,
-            UP = 4,
-            RIGHT = 8
-        };
-
         /**
          * Constructor.
          *
@@ -159,14 +113,16 @@ class Being : public ActorSprite, public ConfigListener
          * @param subtype partly determines the type of the being
          * @param map     the map the being is on
          */
-        Being(const int id, const Type type, const uint16_t subtype,
+        Being(const int id,
+              const ActorType::Type type,
+              const uint16_t subtype,
               Map *const map);
 
         A_DELETE_COPY(Being)
 
         virtual ~Being();
 
-        Type getType() const A_WARN_UNUSED
+        ActorType::Type getType() const override final A_WARN_UNUSED
         { return mType; }
 
         /**
@@ -198,14 +154,14 @@ class Being : public ActorSprite, public ConfigListener
          * TODO: Used by eAthena only?
          */
         int getXOffset() const A_WARN_UNUSED
-        { return getOffset(LEFT, RIGHT); }
+        { return getOffset(BeingDirection::LEFT, BeingDirection::RIGHT); }
 
         /**
          * Get the current Y pixel offset.
          * TODO: Used by eAthena only?
          */
         int getYOffset() const A_WARN_UNUSED
-        { return getOffset(UP, DOWN); }
+        { return getOffset(BeingDirection::UP, BeingDirection::DOWN); }
 
         /**
          * Creates a path for the being from current position to ex and ey
@@ -436,8 +392,8 @@ class Being : public ActorSprite, public ConfigListener
         const BeingInfo *getInfo() const A_WARN_UNUSED
         { return mInfo; }
 
-        TargetCursorSize getTargetCursorSize() const override final
-                                             A_WARN_UNUSED;
+        TargetCursorSize::Size getTargetCursorSize() const override final
+                                                     A_WARN_UNUSED;
 
         int getTargetOffsetX() const A_WARN_UNUSED
         {
@@ -466,25 +422,21 @@ class Being : public ActorSprite, public ConfigListener
         /**
          * Gets the way the monster blocks pathfinding for other objects
          */
-        Map::BlockType getBlockType() const A_WARN_UNUSED
+        BlockType::BlockType getBlockType() const A_WARN_UNUSED
         {
             if (!mInfo)
-                return Map::BLOCKTYPE_NONE;
+                return BlockType::NONE;
             return mInfo->getBlockType();
         }
 
         /**
-         * Sets the walk speed.
-         * in pixels per second for eAthena,
-         * in tiles per second for Manaserv.
+         * Sets the walk speed in pixels per second.
          */
-        void setWalkSpeed(Vector speed)
+        void setWalkSpeed(const Vector &speed)
         { mWalkSpeed = speed; mSpeed = speed.x; }
 
         /**
-         * Gets the walk speed.
-         * in pixels per second for eAthena,
-         * in tiles per second for Manaserv (0.1 precision).
+         * Gets the walk speed in pixels per second.
          */
         Vector getWalkSpeed() const A_WARN_UNUSED
         { return mWalkSpeed; }
@@ -506,19 +458,20 @@ class Being : public ActorSprite, public ConfigListener
         /**
          * Sets the current action.
          */
-        virtual void setAction(const Action &action, const int attackType);
+        virtual void setAction(const BeingAction::Action &action,
+                               const int attackType);
 
         /**
          * Get the being's action currently performed.
          */
-        Action getCurrentAction() const A_WARN_UNUSED
+        BeingAction::Action getCurrentAction() const A_WARN_UNUSED
         { return mAction; }
 
         /**
          * Returns whether this being is still alive.
          */
         bool isAlive() const A_WARN_UNUSED
-        { return mAction != DEAD; }
+        { return mAction != BeingAction::DEAD; }
 
         /**
          * Returns the current direction.
@@ -540,10 +493,10 @@ class Being : public ActorSprite, public ConfigListener
         /**
          * Returns the direction the being is facing.
          */
-        SpriteDirection getSpriteDirection() const A_WARN_UNUSED
-        { return static_cast<SpriteDirection>(mSpriteDirection); }
+        SpriteDirection::Type getSpriteDirection() const A_WARN_UNUSED
+        { return static_cast<SpriteDirection::Type>(mSpriteDirection); }
 
-        void setPosition(const Vector &pos);
+        void setPosition(const Vector &pos) override;
 
         /**
          * Overloaded method provided for convenience.
@@ -647,9 +600,9 @@ class Being : public ActorSprite, public ConfigListener
         /**
          * Sets the gender of this being.
          */
-        virtual void setGender(const Gender gender);
+        virtual void setGender(const Gender::Type gender);
 
-        Gender getGender() const A_WARN_UNUSED
+        Gender::Type getGender() const A_WARN_UNUSED
         { return mGender; }
 
         /**
@@ -681,7 +634,7 @@ class Being : public ActorSprite, public ConfigListener
         void setGM(const bool gm);
 
         bool canTalk() const A_WARN_UNUSED
-        { return mType == NPC; }
+        { return mType == ActorType::NPC; }
 
         void talkTo() const;
 
@@ -731,7 +684,7 @@ class Being : public ActorSprite, public ConfigListener
         const std::string &getIp() const A_WARN_UNUSED
         { return mIp; }
 
-        void setIp(std::string ip)
+        void setIp(const std::string &ip)
         { mIp = ip; }
 
         unsigned int getPvpRank() const A_WARN_UNUSED
@@ -797,7 +750,7 @@ class Being : public ActorSprite, public ConfigListener
         const std::string getComment() const A_WARN_UNUSED
         { return mComment; }
 
-        void setComment(std::string n)
+        void setComment(const std::string &n)
         { mComment = n; }
 
         static void clearCache();
@@ -835,7 +788,7 @@ class Being : public ActorSprite, public ConfigListener
 
         void updatePercentHP();
 
-        void setRaceName(std::string name)
+        void setRaceName(const std::string &name)
         { mRaceName = name; }
 
         std::string getRaceName() const A_WARN_UNUSED
@@ -843,6 +796,8 @@ class Being : public ActorSprite, public ConfigListener
 
         int getSpriteID(const int slot) const A_WARN_UNUSED;
 
+        unsigned char getSpriteColor(const int slot) const A_WARN_UNUSED;
+
         void setHairStyle(const unsigned int slot, const int id);
 
         void setHairColor(const unsigned int slot,
@@ -901,14 +856,14 @@ class Being : public ActorSprite, public ConfigListener
         void playSfx(const SoundInfo &sound, Being *const being,
                      const bool main, const int x, const int y) const;
 
-        int getLook() const
+        uint8_t getLook() const
         { return mLook; }
 
-        void setLook(const int look);
+        void setLook(const uint8_t look);
 
-        static uint8_t genderToInt(const Gender sex) A_WARN_UNUSED;
+        static uint8_t genderToInt(const Gender::Type sex) A_WARN_UNUSED;
 
-        static Gender intToGender(const uint8_t sex) A_WARN_UNUSED;
+        static Gender::Type intToGender(const uint8_t sex) A_WARN_UNUSED;
 
         NextSoundInfo mNextSound;
 
@@ -919,6 +874,7 @@ class Being : public ActorSprite, public ConfigListener
 
         int getSortPixelY() const A_WARN_UNUSED
         { return static_cast<int>(mPos.y) - mYDiff - mSortOffsetY; }
+//        { return static_cast<int>(mPos.y) - mYDiff - mSortOffsetY + 16; }
 
         void setMap(Map *const map);
 
@@ -930,6 +886,12 @@ class Being : public ActorSprite, public ConfigListener
         int decUsage()
         { return --mUsageCounter; }
 
+        virtual int getLastAttackX() const
+        { return mLastAttackX; }
+
+        virtual int getLastAttackY() const
+        { return mLastAttackY; }
+
     protected:
         /**
          * Updates name's location.
@@ -944,6 +906,8 @@ class Being : public ActorSprite, public ConfigListener
 
         void removeItemParticles(const int id);
 
+        void createSpeechBubble();
+
         static int getDefaultEffectId(const int type);
 
         BeingInfo *mInfo;
@@ -961,7 +925,7 @@ class Being : public ActorSprite, public ConfigListener
          * Holds a text object when the being displays it's name, 0 otherwise
          */
         FlashText *mDispName;
-        const gcn::Color *mNameColor;
+        const Color *mNameColor;
 
         /** Engine-related infos about weapon. */
         const ItemInfo *mEquippedWeapon;
@@ -971,7 +935,7 @@ class Being : public ActorSprite, public ConfigListener
 
         Path mPath;
         Text *mText;
-        const gcn::Color *mTextColor;
+        const Color *mTextColor;
 
         Vector mDest;  /**< destination coordinates. */
 
@@ -980,7 +944,7 @@ class Being : public ActorSprite, public ConfigListener
 
         StringVect mSpriteColors;
         std::vector<int> mSpriteIDs;
-        std::vector<int> mSpriteColorsIds;
+        std::vector<unsigned char> mSpriteColorsIds;
         SpriteParticleInfo mSpriteParticles;
 
         // Character guild information
@@ -996,8 +960,10 @@ class Being : public ActorSprite, public ConfigListener
 
         int mLevel;
         int mAttackRange;
-        Gender mGender;
-        Action mAction;       /**< Action the being is performing */
+        int mLastAttackX;
+        int mLastAttackY;
+        Gender::Type mGender;
+        BeingAction::Action mAction;
         uint16_t mSubType;      /**< Subtype (graphical view, basically) */
         uint8_t mDirection;               /**< Facing direction */
         uint8_t mDirectionDelayed;        /**< Facing direction */
@@ -1017,21 +983,21 @@ class Being : public ActorSprite, public ConfigListener
         int searchSlotValue(const std::vector<int> &slotRemap,
                             const int val) const A_WARN_UNUSED;
 
-        void searchSlotValueItr(std::vector<int>::iterator &it, int &idx,
-                                std::vector<int> &slotRemap,
-                                const int val) const;
+        static void searchSlotValueItr(std::vector<int>::iterator &it,
+                                       int &idx,
+                                       std::vector<int> &slotRemap,
+                                       const int val);
 
         void dumpSprites() const;
 
-        const Type mType;
+        const ActorType::Type mType;
 
         /** Speech Bubble components */
         SpeechBubble *mSpeechBubble;
 
         /**
          * Walk speed for x and y movement values.
-         * In pixels per second for eAthena,
-         * In pixels per ticks for Manaserv.
+         * In pixels per second.
          * @see MILLISECONDS_IN_A_TICK
          */
         Vector mWalkSpeed;
@@ -1045,10 +1011,12 @@ class Being : public ActorSprite, public ConfigListener
         Being *mOwner;
         Particle *mSpecialParticle;
 
-        int mX;            // position in tiles
-        int mY;            // position in tiles
-        int mSortOffsetY;  // caculated offset in pixels based on mOffsetY
-        int mOffsetY;      // fixed tile height offset in pixels
+        int mX;             // position in tiles
+        int mY;             // position in tiles
+        int mSortOffsetY;   // caculated offset in pixels based on mOffsetY
+        int mOffsetY;       // tile height offset in pixels
+                            // calculated between tiles
+        int mFixedOffsetY;  // fixed tile height offset in pixels for tile
         uint8_t mOldHeight;
 
         int mDamageTaken;
@@ -1088,8 +1056,8 @@ class Being : public ActorSprite, public ConfigListener
         int mCriticalHit;
         unsigned int mPvpRank;
         unsigned int mNumber;
-        int mLook;
         int mUsageCounter;
+        uint8_t mLook;
         unsigned char mHairColor;
         bool mErased;
         bool mEnemy;
@@ -1098,6 +1066,7 @@ class Being : public ActorSprite, public ConfigListener
         bool mShop;
         bool mAway;
         bool mInactive;
+        bool mNeedPosUpdate;
 };
 
 extern std::list<BeingCacheEntry*> beingInfoCache;
