@@ -23,56 +23,38 @@
 #ifndef BEING_LOCALPLAYER_H
 #define BEING_LOCALPLAYER_H
 
-#include "depricatedlistener.h"
-
-#include "being/actorspritelistener.h"
 #include "being/being.h"
 
 #include "gui/userpalette.h"
 
-#include <guichan/actionlistener.hpp>
+#include "listeners/actorspritelistener.h"
+#include "listeners/attributelistener.h"
+#include "listeners/statlistener.h"
 
 #include <vector>
 
 #include "localconsts.h"
 
+class AwayListener;
 class ChatTab;
 class FloorItem;
 class Map;
 class OkDialog;
 
-class AwayListener final : public gcn::ActionListener
-{
-    public:
-        void action(const gcn::ActionEvent &event) override final;
-};
-
-/**
- * Reasons an item can fail to be picked up.
- */
-enum
-{
-    PICKUP_OKAY = 0,
-    PICKUP_BAD_ITEM,
-    PICKUP_TOO_HEAVY,
-    PICKUP_TOO_FAR,
-    PICKUP_INV_FULL,
-    PICKUP_STACK_FULL,
-    PICKUP_DROP_STEAL
-};
-
 /**
  * The local player character.
  */
 class LocalPlayer final : public Being,
                           public ActorSpriteListener,
-                          public DepricatedListener
+                          public AttributeListener,
+                          public StatListener
 {
     public:
         /**
          * Constructor.
          */
-        explicit LocalPlayer(const int id = 65535, const int subtype = 0);
+        explicit LocalPlayer(const int id = 65535,
+                             const uint16_t subtype = 0U);
 
         A_DELETE_COPY(LocalPlayer)
 
@@ -85,7 +67,7 @@ class LocalPlayer final : public Being,
 
         void slowLogic();
 
-        void setAction(const Action &action,
+        void setAction(const BeingAction::Action &action,
                        const int attackType = 0) override final;
 
         /**
@@ -204,79 +186,22 @@ class LocalPlayer final : public Being,
         bool isPathSetByMouse() const A_WARN_UNUSED
         { return mPathSetByMouse; }
 
-        int getInvertDirection() const A_WARN_UNUSED
-        { return mInvertDirection; }
-
-        void setInvertDirection(const int n)
-        { mInvertDirection = n; }
-
-        void invertDirection();
-
-        int getAttackWeaponType() const A_WARN_UNUSED
-        { return mAttackWeaponType; }
-
-        int getAttackType() const A_WARN_UNUSED
-        { return mAttackType; }
-
-        int getFollowMode() const A_WARN_UNUSED
-        { return mFollowMode; }
-
-        int getImitationMode() const A_WARN_UNUSED
-        { return mImitationMode; }
-
-        void changeAttackWeaponType();
-
-        void changeAttackType();
-
-        void changeFollowMode();
-
-        void changeImitationMode();
-
-        void changePickUpType();
-
-        int getCrazyMoveType() const A_WARN_UNUSED
-        { return mCrazyMoveType; }
-
-        int getPickUpType() const A_WARN_UNUSED
-        { return mPickUpType; }
-
-        int getQuickDropCounter() const A_WARN_UNUSED
-        { return mQuickDropCounter; }
-
-        void setQuickDropCounter(const int n);
-
-        void changeQuickDropCounter();
-
-        int getMoveState() const A_WARN_UNUSED
+        unsigned int getMoveState() const A_WARN_UNUSED
         { return mMoveState; }
 
-        void setMoveState(const int n)
+        void setMoveState(const unsigned int n)
         { mMoveState = n; }
 
-        void switchMagicAttack();
+        unsigned int getCrazyMoveState() const A_WARN_UNUSED
+        { return mCrazyMoveState; }
 
-        void switchPvpAttack();
-
-        int getMagicAttackType() const A_WARN_UNUSED
-        { return mMagicAttackType; }
-
-        int getPvpAttackType() const A_WARN_UNUSED
-        { return mPvpAttackType; }
-
-        int getMoveToTargetType() const A_WARN_UNUSED
-        { return mMoveToTargetType; }
-
-        int getDisableGameModifiers() const A_WARN_UNUSED
-        { return mDisableGameModifiers; }
+        void setCrazyMoveState(const unsigned int n)
+        { mCrazyMoveState = n; }
 
         std::string getPingTime() const A_WARN_UNUSED;
 
         void tryPingRequest();
 
-        void changeMoveToTargetType();
-
-        void switchGameModifiers();
-
         void magicAttack() const;
 
         void specialMove(const unsigned char direction);
@@ -285,8 +210,6 @@ class LocalPlayer final : public Being,
 
         bool pickUpItems(int pickUpType = 0);
 
-        void changeCrazyMoveType();
-
         void crazyMove();
 
         void moveTo(const int x, const int y);
@@ -297,8 +220,6 @@ class LocalPlayer final : public Being,
 
         void moveToHome();
 
-        void debugMsg(const std::string &str) const;
-
         bool isReachable(Being *const being,
                          const int maxCost = 0) A_WARN_UNUSED;
 
@@ -311,18 +232,10 @@ class LocalPlayer final : public Being,
 
         void pingResponse();
 
-        void changeAwayMode();
-
         void setAway(const std::string &message);
 
         void setPseudoAway(const std::string &message);
 
-        bool getAway() const A_WARN_UNUSED
-        { return mAwayMode; }
-
-        bool getPseudoAway() const A_WARN_UNUSED
-        { return mPseudoAwayMode; }
-
         void setHalfAway(const bool n)
         { mInactive = n; }
 
@@ -331,7 +244,7 @@ class LocalPlayer final : public Being,
 
         void afkRespond(ChatTab *const tab, const std::string &nick);
 
-        void setAfkMessage(std::string message);
+        static void setAfkMessage(std::string message);
 
         bool navigateTo(const int x, const int y);
 
@@ -343,12 +256,13 @@ class LocalPlayer final : public Being,
                           const unsigned char emote) const;
 
         void imitateAction(const Being *const being,
-                           const Being::Action &action);
+                           const BeingAction::Action &action);
 
         void imitateDirection(const Being *const being,
                               const unsigned char dir);
 
-        void imitateOutfit(Being *const player, const int sprite = -1) const;
+        void imitateOutfit(const Being *const player,
+                           const int sprite = -1) const;
 
         void followMoveTo(const Being *const being, const int x, const int y);
 
@@ -377,9 +291,6 @@ class LocalPlayer final : public Being,
          */
         void optionChanged(const std::string &value) override final;
 
-        void processEvent(const Channels channel,
-                          const DepricatedEvent &event) override final;
-
         /**
          * set a following player.
          */
@@ -455,8 +366,6 @@ class LocalPlayer final : public Being,
 
         void checkNewName(Being *const being);
 
-        void resetYellowBar();
-
         unsigned char getWalkMask() const override final A_WARN_UNUSED;
 
         void saveHomes();
@@ -473,41 +382,36 @@ class LocalPlayer final : public Being,
         void setTestParticle(const std::string &fileName,
                              const bool updateHash = true);
 
-        std::string getInvertDirectionString();
+        int getLastAttackX() const override final
+        { return mTarget ? mTarget->getTileX() : mLastAttackX; }
 
-        std::string getCrazyMoveTypeString();
+        int getLastAttackY() const override final
+        { return mTarget ? mTarget->getTileY() : mLastAttackY; }
 
-        std::string getMoveToTargetTypeString();
+        void attributeChanged(const int id,
+                              const int oldVal,
+                              const int newVal) override final;
 
-        std::string getFollowModeString();
+        void statChanged(const int id,
+                         const int oldVal1,
+                         const int oldVal2) override final;
 
-        std::string getAttackWeaponTypeString();
+        void updateMusic();
 
-        std::string getAttackTypeString();
+        void setAfkTime(const int v)
+        { mAfkTime = v; }
 
-        std::string getQuickDropCounterString();
+        void setAwayDialog(OkDialog *const dialog)
+        { mAwayDialog = dialog; }
 
-        std::string getPickUpTypeString();
-
-        std::string getDebugPathString() const;
-
-        std::string getMagicAttackString();
-
-        std::string getPvpAttackString();
-
-        std::string getImitationModeString();
-
-        std::string getAwayModeString();
-
-        std::string getCameraModeString() const;
-
-        std::string getGameModifiersString();
+        AwayListener *getAwayListener() const A_WARN_UNUSED
+        { return mAwayListener; }
 
     protected:
         void updateCoords() override final;
 
-        void handleStatusEffect(StatusEffect *const effect,
-                                const int effectId);
+        void handleStatusEffect(const StatusEffect *const effect,
+                                const int effectId) override final;
 
         void startWalking(const unsigned char dir);
 
@@ -516,15 +420,9 @@ class LocalPlayer final : public Being,
         static void tryMagic(const std::string &spell, const int baseMagic,
                              const int schoolMagic, const int mana);
 
-        const char *getVarItem(const char *const *const arr,
-                               const unsigned index,
-                               const unsigned sz) const A_WARN_UNUSED;
-
-        void changeMode(unsigned *restrict const var, const unsigned limit,
-                        const char *restrict const conf,
-                        std::string (LocalPlayer::*const func)(),
-                        const unsigned def = 0,
-                        const bool save = true);
+        static const char *getVarItem(const char *const *const arr,
+                                      const unsigned index,
+                                      const unsigned sz) A_WARN_UNUSED;
 
         void crazyMove1();
         void crazyMove2();
@@ -541,29 +439,10 @@ class LocalPlayer final : public Being,
 
         int mGMLevel;
 
-        // move type
-        unsigned int mInvertDirection;
-        // crazy move type
-        unsigned int mCrazyMoveType;
         // crazy move state
         unsigned int mCrazyMoveState;
-        // attack weapon type
-        unsigned int mAttackWeaponType;
-        // quick drop counter
-        unsigned int mQuickDropCounter;
-        // move state. used if mInvertDirection == 2
+        // move state. used if mMoveType == 2
         unsigned int mMoveState;
-        // pick up type 1x1, normal aka 2x1, forward aka 2x3, 3x3, 3x3 + 1
-        unsigned int mPickUpType;
-        // magic attack type
-        unsigned int mMagicAttackType;
-        // pvp attack type
-        unsigned int mPvpAttackType;
-        // type how move to target
-        unsigned int mMoveToTargetType;
-        unsigned int mAttackType;
-        unsigned int mFollowMode;
-        unsigned int mImitationMode;
 
         int mLastTargetX;
         int mLastTargetY;
@@ -625,13 +504,13 @@ class LocalPlayer final : public Being,
         bool mEnableAdvert;
         bool mTradebot;
         bool mTargetOnlyReachable;
-        bool mDisableGameModifiers;
         bool mIsServerBuggy;
         bool mSyncPlayerMove;
         bool mDrawPath;
         bool mAttackMoving;
         bool mAttackNext;
         bool mShowJobExp;
+        bool mShowServerPos;
         bool mNextStep;
         // temporary disable crazy moves in moves
         bool mDisableCrazyMove;
@@ -641,11 +520,9 @@ class LocalPlayer final : public Being,
         // Tells if the path was set using mouse
         bool mPathSetByMouse;
         bool mWaitPing;
-        bool mAwayMode;
-        bool mPseudoAwayMode;
         bool mShowNavigePath;
 };
 
-extern LocalPlayer *player_node;
+extern LocalPlayer *localPlayer;
 
 #endif  // BEING_LOCALPLAYER_H
