@@ -23,15 +23,14 @@
 #define BEING_PLAYERINFO_H
 
 #include "equipment.h"
+#include "state.h"
 
 #include <map>
-#include <string>
-
 
 /**
  * Stat information storage structure.
  */
-struct Stat
+struct Stat final
 {
     int base;
     int mod;
@@ -63,18 +62,6 @@ class FloorItem;
 class Inventory;
 class Item;
 
-/**
- * Special information storage structure.
- */
-struct Special final
-{
-    int currentMana;
-    int neededMana;
-    int recharge;
-};
-
-typedef std::map<int, Special> SpecialsMap;
-
 /**
  * A database like namespace which holds global info about the localplayer
  *
@@ -82,30 +69,6 @@ typedef std::map<int, Special> SpecialsMap;
  */
 namespace PlayerInfo
 {
-    /**
-     * Standard attributes for players.
-     */
-    enum Attribute
-    {
-        LEVEL = 0,
-        HP,
-        MAX_HP,
-        MP,
-        MAX_MP,
-        EXP,
-        EXP_NEEDED,
-        MONEY,
-        TOTAL_WEIGHT,
-        MAX_WEIGHT,
-        SKILL_POINTS,
-        CHAR_POINTS,
-        CORR_POINTS,
-        ATTACK_DELAY = 100,
-        ATTACK_RANGE = 101,
-        WALK_SPEED = 102,
-        ATTACK_SPEED = 103
-    };
-
 // --- Attributes -------------------------------------------------------------
 
     /**
@@ -189,7 +152,7 @@ namespace PlayerInfo
      * Changes the inventory item at the given slot.
      */
     void setInventoryItem(const int index, const int id,
-                          const int amount, const int refine);
+                          const int amount, const uint8_t refine);
 
     /**
      * Returns the player's equipment.
@@ -243,16 +206,11 @@ namespace PlayerInfo
 
     void gameDestroyed();
 
-    void stateChange(const int state);
-
-    void triggerAttr(int id);
+    void stateChange(const State state);
 
     void triggerAttr(const int id, const int old);
 
-    void triggerStat(int id);
-
-    void triggerStat(const int id, const std::string &changed,
-                     const int old1, const int old2 = 0);
+    void triggerStat(const int id, const int old1, const int old2);
 
     void setEquipmentBackend(Equipment::Backend *const backend);
 
