@@ -22,6 +22,8 @@
 
 #include "commandhandler.h"
 
+#include "commands.h"
+
 #include "input/inputmanager.h"
 
 #include "gui/widgets/tabs/chattab.h"
@@ -31,6 +33,8 @@
 
 #include "debug.h"
 
+CommandHandler *commandHandler = nullptr;
+
 CommandHandler::CommandHandler() :
     mCommands()
 {
@@ -61,14 +65,14 @@ void CommandHandler::addChatCommands(std::list<std::string> &arr)
 }
 
 void CommandHandler::handleCommands(const std::string &command,
-                                    ChatTab *const tab)
+                                    ChatTab *const tab) const
 {
     // here need add splitting commands
     handleCommand(command, tab);
 }
 
 void CommandHandler::handleCommand(const std::string &command,
-                                   ChatTab *const tab)
+                                   ChatTab *const tab) const
 {
     const size_t pos = command.find(' ');
     const std::string type(command, 0, pos);
