@@ -27,14 +27,17 @@
 #include "client.h"
 #include "configuration.h"
 #include "game.h"
+#include "gamemodifiers.h"
 #include "graphicsmanager.h"
 #include "guildmanager.h"
 #include "main.h"
 #include "party.h"
+#include "settings.h"
 
 #include "being/localplayer.h"
 #include "being/playerrelations.h"
 
+#include "gui/chatconsts.h"
 #include "gui/gui.h"
 
 #include "gui/windows/buydialog.h"
@@ -58,9 +61,12 @@
 #include "net/adminhandler.h"
 #include "net/beinghandler.h"
 #include "net/chathandler.h"
+#include "net/download.h"
 #include "net/gamehandler.h"
 #include "net/guildhandler.h"
+#include "net/ipc.h"
 #include "net/net.h"
+#include "net/uploadcharinfo.h"
 #include "net/partyhandler.h"
 #include "net/pethandler.h"
 #include "net/tradehandler.h"
@@ -76,6 +82,7 @@
 
 #include "resources/db/itemdb.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 #include "utils/process.h"
 #include "utils/timer.h"
@@ -96,7 +103,8 @@ extern char **environ;
 namespace Commands
 {
 
-static void outString(ChatTab *const tab, const std::string &str,
+static void outString(const ChatTab *const tab,
+                      const std::string &str,
                       const std::string &def)
 {
     if (!tab)
@@ -107,16 +115,16 @@ static void outString(ChatTab *const tab, const std::string &str,
 
     switch (tab->getType())
     {
-        case ChatTab::TAB_PARTY:
+        case ChatTabType::PARTY:
         {
             Net::getPartyHandler()->chat(str);
             break;
         }
-        case ChatTab::TAB_GUILD:
+        case ChatTabType::GUILD:
         {
-            if (!player_node)
+            if (!localPlayer)
                 return;
-            const Guild *const guild = player_node->getGuild();
+            const Guild *const guild = localPlayer->getGuild();
             if (guild)
             {
                 if (guild->getServerGuild())
@@ -149,7 +157,7 @@ static void changeRelation(const std::string &args,
     if (args.empty())
     {
         // TRANSLATORS: change relation
-        tab->chatLog(_("Please specify a name."), BY_SERVER);
+        tab->chatLog(_("Please specify a name."), ChatMsgType::BY_SERVER);
         return;
     }
 
@@ -157,7 +165,7 @@ static void changeRelation(const std::string &args,
     {
         // TRANSLATORS: change relation
         tab->chatLog(strprintf(_("Player already %s!"),
-                     relationText.c_str()), BY_SERVER);
+                     relationText.c_str()), ChatMsgType::BY_SERVER);
         return;
     }
     else
@@ -169,13 +177,13 @@ static void changeRelation(const std::string &args,
     {
         // TRANSLATORS: change relation
         tab->chatLog(strprintf(_("Player successfully %s!"),
-                     relationText.c_str()), BY_SERVER);
+                     relationText.c_str()), ChatMsgType::BY_SERVER);
     }
     else
     {
         // TRANSLATORS: change relation
         tab->chatLog(strprintf(_("Player could not be %s!"),
-                     relationText.c_str()), BY_SERVER);
+                     relationText.c_str()), ChatMsgType::BY_SERVER);
     }
 }
 
@@ -199,7 +207,7 @@ static void outStringNormal(ChatTab *const tab,
                             const std::string &str,
                             const std::string &def)
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     if (!tab)
@@ -210,14 +218,14 @@ static void outStringNormal(ChatTab *const tab,
 
     switch (tab->getType())
     {
-        case ChatTab::TAB_PARTY:
+        case ChatTabType::PARTY:
         {
             Net::getPartyHandler()->chat(str);
             break;
         }
-        case ChatTab::TAB_GUILD:
+        case ChatTabType::GUILD:
         {
-            const Guild *const guild = player_node->getGuild();
+            const Guild *const guild = localPlayer->getGuild();
             if (guild)
             {
                 if (guild->getServerGuild())
@@ -233,11 +241,11 @@ static void outStringNormal(ChatTab *const tab,
             }
             break;
         }
-        case ChatTab::TAB_WHISPER:
+        case ChatTabType::WHISPER:
         {
             const WhisperTab *const whisper
-                = static_cast<WhisperTab *const>(tab);
-            tab->chatLog(player_node->getName(), str);
+                = static_cast<const WhisperTab *const>(tab);
+            tab->chatLog(localPlayer->getName(), str);
             Net::getChatHandler()->privateMessage(whisper->getNick(), str);
             break;
         }
@@ -266,12 +274,12 @@ impHandler2(help)
     }
     switch (tab->getType())
     {
-        case ChatTab::TAB_PARTY:
+        case ChatTabType::PARTY:
         {
             helpWindow->loadHelp("chatparty");
             break;
         }
-        case ChatTab::TAB_GUILD:
+        case ChatTabType::GUILD:
         {
             helpWindow->loadHelp("chatguild");
             break;
@@ -283,13 +291,38 @@ impHandler2(help)
     helpWindow->requestMoveToTop();
 }
 
+impHandler0(ipcToggle)
+{
+    if (ipc)
+    {
+        IPC::stop();
+        if (!ipc)
+            debugChatTab->chatLog("IPC service stopped.");
+        else
+            debugChatTab->chatLog("Unable to stop IPC service.");
+    }
+    else
+    {
+        IPC::start();
+        if (ipc)
+        {
+            debugChatTab->chatLog(strprintf("IPC service available on port %d",
+                ipc->getPort()));
+        }
+        else
+        {
+            debugChatTab->chatLog("Unable to start IPC service");
+        }
+    }
+}
+
 impHandler2(where)
 {
     std::ostringstream where;
     where << Game::instance()->getCurrentMapName() << ", coordinates: "
-        << ((player_node->getPixelX() - mapTileSize / 2) / mapTileSize)
-        << ", " << ((player_node->getPixelY() - mapTileSize) / mapTileSize);
-    tab->chatLog(where.str(), BY_SERVER);
+        << ((localPlayer->getPixelX() - mapTileSize / 2) / mapTileSize)
+        << ", " << ((localPlayer->getPixelY() - mapTileSize) / mapTileSize);
+    tab->chatLog(where.str(), ChatMsgType::BY_SERVER);
 }
 
 impHandler0(who)
@@ -332,7 +365,7 @@ impHandler(msg)
 
     if (msg.length() > 0)
     {
-        std::string playerName = player_node->getName();
+        std::string playerName = localPlayer->getName();
         std::string tempNick = recvnick;
 
         toLower(playerName);
@@ -341,12 +374,12 @@ impHandler(msg)
         if (tempNick.compare(playerName) == 0 || args.empty())
             return;
 
-        chatWindow->addWhisper(recvnick, msg, BY_PLAYER);
+        chatWindow->addWhisper(recvnick, msg, ChatMsgType::BY_PLAYER);
     }
     else
     {
         // TRANSLATORS: whisper send
-        tab->chatLog(_("Cannot send empty whispers!"), BY_SERVER);
+        tab->chatLog(_("Cannot send empty whispers!"), ChatMsgType::BY_SERVER);
     }
 }
 
@@ -366,7 +399,7 @@ impHandler(query)
         // TRANSLATORS: new whisper query
         tab->chatLog(strprintf(_("Cannot create a whisper tab for nick "
             "\"%s\"! It either already exists, or is you."),
-            args.c_str()), BY_SERVER);
+            args.c_str()), ChatMsgType::BY_SERVER);
     }
 }
 
@@ -406,7 +439,7 @@ impHandler(createParty)
     if (args.empty())
     {
         // TRANSLATORS: create party message
-        tab->chatLog(_("Party name is missing."), BY_SERVER);
+        tab->chatLog(_("Party name is missing."), ChatMsgType::BY_SERVER);
     }
     else
     {
@@ -422,7 +455,7 @@ impHandler(createGuild)
     if (args.empty())
     {
         // TRANSLATORS: create guild message
-        tab->chatLog(_("Guild name is missing."), BY_SERVER);
+        tab->chatLog(_("Guild name is missing."), ChatMsgType::BY_SERVER);
     }
     else
     {
@@ -442,7 +475,7 @@ impHandler(party)
     else
     {
         // TRANSLATORS: party invite message
-        tab->chatLog(_("Please specify a name."), BY_SERVER);
+        tab->chatLog(_("Please specify a name."), ChatMsgType::BY_SERVER);
     }
 }
 
@@ -529,7 +562,7 @@ impHandler(unignore)
         if (tab)
         {
             // TRANSLATORS: unignore command
-            tab->chatLog(_("Please specify a name."), BY_SERVER);
+            tab->chatLog(_("Please specify a name."), ChatMsgType::BY_SERVER);
         }
         return;
     }
@@ -544,7 +577,7 @@ impHandler(unignore)
         if (tab)
         {
             // TRANSLATORS: unignore command
-            tab->chatLog(_("Player wasn't ignored!"), BY_SERVER);
+            tab->chatLog(_("Player wasn't ignored!"), ChatMsgType::BY_SERVER);
         }
         return;
     }
@@ -554,12 +587,14 @@ impHandler(unignore)
         if (player_relations.getRelation(args) == PlayerRelation::NEUTRAL)
         {
             // TRANSLATORS: unignore command
-            tab->chatLog(_("Player no longer ignored!"), BY_SERVER);
+            tab->chatLog(_("Player no longer ignored!"),
+                ChatMsgType::BY_SERVER);
         }
         else
         {
             // TRANSLATORS: unignore command
-            tab->chatLog(_("Player could not be unignored!"), BY_SERVER);
+            tab->chatLog(_("Player could not be unignored!"),
+                ChatMsgType::BY_SERVER);
         }
     }
 }
@@ -583,7 +618,7 @@ impHandler(erase)
         if (tab)
         {
             // TRANSLATORS: erase command
-            tab->chatLog(_("Please specify a name."), BY_SERVER);
+            tab->chatLog(_("Please specify a name."), ChatMsgType::BY_SERVER);
         }
         return;
     }
@@ -593,7 +628,7 @@ impHandler(erase)
         if (tab)
         {
             // TRANSLATORS: erase command
-            tab->chatLog(_("Player already erased!"), BY_SERVER);
+            tab->chatLog(_("Player already erased!"), ChatMsgType::BY_SERVER);
         }
         return;
     }
@@ -607,12 +642,14 @@ impHandler(erase)
         if (player_relations.getRelation(args) == PlayerRelation::ERASED)
         {
             // TRANSLATORS: erase command
-            tab->chatLog(_("Player successfully erased!"), BY_SERVER);
+            tab->chatLog(_("Player successfully erased!"),
+                ChatMsgType::BY_SERVER);
         }
         else
         {
             // TRANSLATORS: erase command
-            tab->chatLog(_("Player could not be erased!"), BY_SERVER);
+            tab->chatLog(_("Player could not be erased!"),
+                ChatMsgType::BY_SERVER);
         }
     }
 }
@@ -633,47 +670,47 @@ impHandler1(move)
     int x = 0;
     int y = 0;
 
-    if (player_node && parse2Int(args, x, y))
-        player_node->moveTo(x, y);
+    if (localPlayer && parse2Int(args, x, y))
+        localPlayer->moveTo(x, y);
 }
 
 impHandler1(navigate)
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     int x = 0;
     int y = 0;
 
     if (parse2Int(args, x, y))
-        player_node->navigateTo(x, y);
+        localPlayer->navigateTo(x, y);
     else
-        player_node->navigateClean();
+        localPlayer->navigateClean();
 }
 
 impHandler1(target)
 {
-    if (!actorManager || !player_node)
+    if (!actorManager || !localPlayer)
         return;
 
     Being *const target = actorManager->findNearestByName(args);
     if (target)
-        player_node->setTarget(target);
+        localPlayer->setTarget(target);
 }
 
 impHandler0(attackHuman)
 {
-    if (!actorManager || !player_node)
+    if (!actorManager || !localPlayer)
         return;
 
     Being *const target = actorManager->findNearestLivingBeing(
-        player_node, 10, ActorSprite::PLAYER);
+        localPlayer, 10, ActorType::PLAYER, true);
     if (target)
     {
-        if (player_node->checAttackPermissions(target))
+        if (localPlayer->checAttackPermissions(target))
         {
-            player_node->setTarget(target);
-            player_node->attack2(target, true);
+            localPlayer->setTarget(target);
+            localPlayer->attack2(target, true);
         }
     }
 }
@@ -720,8 +757,8 @@ impHandler1(outfit)
 
 impHandler1(emote)
 {
-    if (player_node)
-        player_node->emote(static_cast<uint8_t>(atoi(args.c_str())));
+    if (localPlayer)
+        localPlayer->emote(static_cast<uint8_t>(atoi(args.c_str())));
 }
 
 impHandler1(emotePet)
@@ -732,44 +769,44 @@ impHandler1(emotePet)
 
 impHandler1(away)
 {
-    if (player_node)
-        player_node->setAway(args);
+    if (localPlayer)
+        localPlayer->setAway(args);
 }
 
 impHandler1(pseudoAway)
 {
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->setPseudoAway(args);
-        player_node->updateStatus();
+        localPlayer->setPseudoAway(args);
+        localPlayer->updateStatus();
     }
 }
 
 impHandler(follow)
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     if (!features.getBoolValue("allowFollow"))
         return;
 
     if (!args.empty())
-        player_node->setFollow(args);
-    else if (tab && tab->getType() == ChatTab::TAB_WHISPER)
-        player_node->setFollow(static_cast<WhisperTab*>(tab)->getNick());
+        localPlayer->setFollow(args);
+    else if (tab && tab->getType() == ChatTabType::WHISPER)
+        localPlayer->setFollow(static_cast<WhisperTab*>(tab)->getNick());
 }
 
 impHandler(imitation)
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     if (!args.empty())
-        player_node->setImitate(args);
-    else if (tab && tab->getType() == ChatTab::TAB_WHISPER)
-        player_node->setImitate(static_cast<WhisperTab*>(tab)->getNick());
+        localPlayer->setImitate(args);
+    else if (tab && tab->getType() == ChatTabType::WHISPER)
+        localPlayer->setImitate(static_cast<WhisperTab*>(tab)->getNick());
     else
-        player_node->setImitate("");
+        localPlayer->setImitate("");
 }
 
 impHandler1(heal)
@@ -780,13 +817,13 @@ impHandler1(heal)
     if (!args.empty())
     {
         const Being *const being = actorManager->findBeingByName(
-            args, Being::PLAYER);
+            args, ActorType::PLAYER);
         if (being)
             actorManager->heal(being);
     }
     else
     {
-        actorManager->heal(player_node);
+        actorManager->heal(localPlayer);
     }
 }
 
@@ -830,13 +867,13 @@ impHandler1(undress)
 
 impHandler1(attack)
 {
-    if (!player_node || !actorManager)
+    if (!localPlayer || !actorManager)
         return;
 
     Being *const target = actorManager->findNearestByName(args);
     if (target)
-        player_node->setTarget(target);
-    player_node->attack2(player_node->getTarget(), true);
+        localPlayer->setTarget(target);
+    localPlayer->attack2(localPlayer->getTarget(), true);
 }
 
 impHandler1(trade)
@@ -845,7 +882,7 @@ impHandler1(trade)
         return;
 
     const Being *const being = actorManager->findBeingByName(
-        args, Being::PLAYER);
+        args, ActorType::PLAYER);
     if (being)
     {
         Net::getTradeHandler()->request(being);
@@ -857,29 +894,29 @@ impHandler1(trade)
 
 impHandler0(dirs)
 {
-    if (!player_node || !debugChatTab)
+    if (!localPlayer || !debugChatTab)
         return;
 
     debugChatTab->chatLog("config directory: "
-        + client->getConfigDirectory());
+        + settings.configDir);
     debugChatTab->chatLog("logs directory: "
-        + client->getLocalDataDirectory());
+        + settings.localDataDir);
     debugChatTab->chatLog("screenshots directory: "
-        + client->getScreenshotDirectory());
+        + settings.screenshotDir);
     debugChatTab->chatLog("temp directory: "
-        + client->getTempDirectory());
+        + settings.tempDir);
 }
 
 impHandler2(info)
 {
-    if (!tab || !player_node || tmwServerVersion > 0)
+    if (!tab || !localPlayer || tmwServerVersion > 0)
         return;
 
     switch (tab->getType())
     {
-        case ChatTab::TAB_GUILD:
+        case ChatTabType::GUILD:
         {
-            const Guild *const guild = player_node->getGuild();
+            const Guild *const guild = localPlayer->getGuild();
             if (guild)
                 Net::getGuildHandler()->info(guild->getId());
             break;
@@ -891,8 +928,8 @@ impHandler2(info)
 
 impHandler1(wait)
 {
-    if (player_node)
-        player_node->waitFor(args);
+    if (localPlayer)
+        localPlayer->waitFor(args);
 }
 
 impHandler0(uptime)
@@ -1022,7 +1059,7 @@ impHandler0(cacheInfo)
         return;
 
 /*
-    SDLFont *const font = dynamic_cast<SDLFont *const>(chatWindow->getFont());
+    Font *const font = chatWindow->getFont();
     if (!font)
         return;
 
@@ -1090,17 +1127,17 @@ impHandler2(dumpGraphics)
     str.append(",0");
 #endif
 
-    str.append(strprintf(",%f,", static_cast<double>(client->getGuiAlpha())))
-    .append(config.getBoolValue("adjustPerfomance") ? "1" : "0")
-    .append(config.getBoolValue("alphaCache") ? "1" : "0")
-    .append(config.getBoolValue("enableMapReduce") ? "1" : "0")
-    .append(config.getBoolValue("beingopacity") ? "1" : "0")
-    .append(",")
-    .append(config.getBoolValue("enableAlphaFix") ? "1" : "0")
-    .append(config.getBoolValue("disableAdvBeingCaching") ? "1" : "0")
-    .append(config.getBoolValue("disableBeingCaching") ? "1" : "0")
-    .append(config.getBoolValue("particleeffects") ? "1" : "0")
-    .append(strprintf(",%d-%d", fps, config.getIntValue("fpslimit")));
+    str.append(strprintf(",%f,", static_cast<double>(settings.guiAlpha)))
+        .append(config.getBoolValue("adjustPerfomance") ? "1" : "0")
+        .append(config.getBoolValue("alphaCache") ? "1" : "0")
+        .append(config.getBoolValue("enableMapReduce") ? "1" : "0")
+        .append(config.getBoolValue("beingopacity") ? "1" : "0")
+        .append(",")
+        .append(config.getBoolValue("enableAlphaFix") ? "1" : "0")
+        .append(config.getBoolValue("disableAdvBeingCaching") ? "1" : "0")
+        .append(config.getBoolValue("disableBeingCaching") ? "1" : "0")
+        .append(config.getBoolValue("particleeffects") ? "1" : "0")
+        .append(strprintf(",%d-%d", fps, config.getIntValue("fpslimit")));
     outStringNormal(tab, str, str);
 }
 
@@ -1125,14 +1162,14 @@ impHandler2(dumpTests)
 
 impHandler1(setDrop)
 {
-    if (player_node)
-        player_node->setQuickDropCounter(atoi(args.c_str()));
+    GameModifiers::setQuickDropCounter(atoi(args.c_str()));
 }
 
 impHandler0(error)
 {
     const int *const ptr = nullptr;
     logger->log("test %d", *ptr);
+    exit(1);
 }
 
 impHandler(url)
@@ -1222,8 +1259,8 @@ impHandler2(enableAway)
 
 impHandler1(testParticle)
 {
-    if (player_node)
-        player_node->setTestParticle(args);
+    if (localPlayer)
+        localPlayer->setTestParticle(args);
 }
 
 impHandler0(createItems)
@@ -1237,14 +1274,17 @@ impHandler0(createItems)
         if (id <= 500)
             continue;
 
-        const int colors = info->getColorsSize();
-        if (!colors || serverVersion < 1)
+        int colors = info->getColorsSize();
+        if (colors >= 255)
+            colors = 254;
+
+        if (!colors)
         {
             dialog->addItem(id, 1, 100, 0);
         }
         else
         {
-            for (int f = 0; f < colors; f ++)
+            for (unsigned char f = 0; f < colors; f ++)
             {
                 if (!info->getColor(f).empty())
                     dialog->addItem(id, f, 100, 0);
@@ -1262,16 +1302,95 @@ impHandler1(talkRaw)
 impHandler1(talkPet)
 {
     // in future probably need add channel detection
-    if (!player_node->getPets().empty())
+    if (!localPlayer->getPets().empty())
         Net::getChatHandler()->talkPet(args, GENERAL_CHANNEL);
     else
         Net::getChatHandler()->talk(args, GENERAL_CHANNEL);
 }
 
+impHandler1(gm)
+{
+    Net::getChatHandler()->talk("@wgm " + args, GENERAL_CHANNEL);
+}
+
+static int uploadUpdate(void *ptr,
+                        DownloadStatus::Type status,
+                        size_t total A_UNUSED,
+                        size_t remaining A_UNUSED)
+{
+    if (status == DownloadStatus::Idle || status == DownloadStatus::Starting)
+        return 0;
+
+    UploadChatInfo *const info = reinterpret_cast<UploadChatInfo*>(ptr);
+    if (status == DownloadStatus::Complete)
+    {
+        ChatTab *const tab = info->tab;
+        if (chatWindow && (!tab || chatWindow->isTabPresent(tab)))
+        {
+            std::string str = Net::Download::getUploadResponse();
+            const size_t sz = str.size();
+            if (sz > 0)
+            {
+                if (str[sz - 1] == '\n')
+                    str = str.substr(0, sz - 1);
+                str.append(info->addStr);
+                str = strprintf("%s [@@%s |%s@@]",
+                    info->text.c_str(), str.c_str(), str.c_str());
+                outStringNormal(tab, str, str);
+            }
+        }
+    }
+    delete2(info->upload);
+    delete info;
+    return 0;
+}
+
+static void uploadFile(const std::string &str,
+                       const std::string &fileName,
+                       const std::string &addStr,
+                       ChatTab *const tab)
+{
+    UploadChatInfo *const info = new UploadChatInfo();
+    Net::Download *const upload = new Net::Download(info,
+        "http://sprunge.us",
+        &uploadUpdate,
+        false, true, false);
+    info->upload = upload;
+    info->text = str;
+    info->addStr = addStr;
+    info->tab = tab;
+    upload->setFile(fileName);
+    upload->start();
+}
+
+impHandler2(uploadConfig)
+{
+    uploadFile(_("Uploaded config into:"),
+        config.getFileName(),
+        "?xml",
+        tab);
+}
+
+impHandler2(uploadServerConfig)
+{
+    uploadFile(_("Uploaded server config into:"),
+        serverConfig.getFileName(),
+        "?xml",
+        tab);
+}
+
+impHandler2(uploadLog)
+{
+    uploadFile(_("Uploaded log into:"),
+        settings.logFileName,
+        "?txt",
+        tab);
+}
+
 impHandler0(testsdlfont)
 {
 #if defined USE_OPENGL && defined DEBUG_SDLFONT
-    SDLFont *font = new SDLFont("fonts/dejavusans.ttf", 18);
+    Font *font = new Font("fonts/dejavusans.ttf", 18);
     timespec time1;
     timespec time2;
     NullOpenGLGraphics *nullGraphics = new NullOpenGLGraphics;
@@ -1280,7 +1399,7 @@ impHandler0(testsdlfont)
 
     for (int f = 0; f < 300; f ++)
         data.push_back("test " + toString(f) + "string");
-    nullGraphics->_beginDraw();
+    nullGraphics->beginDraw();
 
     clock_gettime(CLOCK_MONOTONIC, &time1);
     for (int f = 0; f < 500; f ++)
@@ -1311,6 +1430,12 @@ impHandler0(testsdlfont)
 #endif
 }
 
+impHandler2(dumpMods)
+{
+    std::string str = "enabled mods: " + serverConfig.getValue("mods", "");
+    outStringNormal(tab, str, str);
+}
+
 #ifdef USE_OPENGL
 impHandler2(dumpGL)
 {
@@ -1413,18 +1538,31 @@ impHandler0(dumpOGL)
 #endif
 }
 
+impHandler0(debugSpawn)
+{
+    int cnt = atoi(args.c_str());
+    if (cnt < 1)
+        cnt = 1;
+    const int half = cnt / 2;
+    for (int x = -half; x < cnt - half; x ++)
+    {
+        for (int y =  -half; y < cnt - half; y ++)
+            actorManager->cloneBeing(localPlayer, x, y, cnt);
+    }
+}
+
 void replaceVars(std::string &str)
 {
-    if (!player_node || !actorManager)
+    if (!localPlayer || !actorManager)
         return;
 
     if (str.find("<PLAYER>") != std::string::npos)
     {
-        const Being *target = player_node->getTarget();
-        if (!target || target->getType() != ActorSprite::PLAYER)
+        const Being *target = localPlayer->getTarget();
+        if (!target || target->getType() != ActorType::PLAYER)
         {
             target = actorManager->findNearestLivingBeing(
-                player_node, 20, ActorSprite::PLAYER);
+                localPlayer, 20, ActorType::PLAYER, true);
         }
         if (target)
             replaceAll(str, "<PLAYER>", target->getName());
@@ -1433,11 +1571,11 @@ void replaceVars(std::string &str)
     }
     if (str.find("<MONSTER>") != std::string::npos)
     {
-        const Being *target = player_node->getTarget();
-        if (!target || target->getType() != ActorSprite::MONSTER)
+        const Being *target = localPlayer->getTarget();
+        if (!target || target->getType() != ActorType::MONSTER)
         {
             target = actorManager->findNearestLivingBeing(
-                player_node, 20, ActorSprite::MONSTER);
+                localPlayer, 20, ActorType::MONSTER, true);
         }
         if (target)
             replaceAll(str, "<MONSTER>", target->getName());
@@ -1451,7 +1589,7 @@ void replaceVars(std::string &str)
         actorManager->getPlayerNames(names, false);
         FOR_EACH (StringVectCIter, it, names)
         {
-            if (*it != player_node->getName())
+            if (*it != localPlayer->getName())
                 newStr.append(*it).append(",");
         }
         if (newStr[newStr.size() - 1] == ',')
@@ -1466,12 +1604,12 @@ void replaceVars(std::string &str)
         StringVect names;
         std::string newStr;
         const Party *party = nullptr;
-        if (player_node->isInParty() && (party = player_node->getParty()))
+        if (localPlayer->isInParty() && (party = localPlayer->getParty()))
         {
             party->getNames(names);
             FOR_EACH (StringVectCIter, it, names)
             {
-                if (*it != player_node->getName())
+                if (*it != localPlayer->getName())
                     newStr.append(*it).append(",");
             }
             if (newStr[newStr.size() - 1] == ',')
