@@ -22,10 +22,11 @@
 #include "defaults.h"
 
 #include "graphicsmanager.h"
+#include "variabledata.h"
 
-#include "being/being.h"
+#include "being/beingspeech.h"
 
-#include "input/keydata.h"
+#include "input/inputaction.h"
 
 #include "render/graphics.h"
 
@@ -78,7 +79,7 @@ DefaultsData* getConfigDefaults()
     AddDEF("speechBubbleAlpha", 1.0F);
     AddDEF("MostUsedServerName0", "server.themanaworld.org");
     AddDEF("visiblenames", true);
-    AddDEF("speech", static_cast<int>(Being::NO_NAME_IN_BUBBLE));
+    AddDEF("speech", static_cast<int>(BeingSpeech::NO_NAME_IN_BUBBLE));
     AddDEF("showgender", true);
     AddDEF("showlevel", false);
     AddDEF("showMonstersTakedDamage", true);
@@ -98,7 +99,7 @@ DefaultsData* getConfigDefaults()
     AddDEF("imitationMode", 0);
     AddDEF("syncPlayerMove", false);
     AddDEF("drawPath", false);
-    AddDEF("moveToTargetType", 6);
+    AddDEF("moveToTargetType", 10);
     AddDEF("crazyMoveProgram", "mumrsonmdmlon");
     AddDEF("disableGameModifiers", false);
     AddDEF("targetDeadPlayers", false);
@@ -123,6 +124,7 @@ DefaultsData* getConfigDefaults()
     AddDEF("showChatColorsList", false);
     AddDEF("customcursor", false);
     AddDEF("showDidYouKnow", false);
+    AddDEF("longmouseclick", true);
 #else
     AddDEF("screenwidth", defaultScreenWidth);
     AddDEF("screenheight", defaultScreenHeight);
@@ -140,6 +142,7 @@ DefaultsData* getConfigDefaults()
     AddDEF("customcursor", true);
 #endif
     AddDEF("showDidYouKnow", true);
+    AddDEF("longmouseclick", false);
 #endif
     AddDEF("showEmotesButton", true);
     AddDEF("screen", false);
@@ -152,7 +155,7 @@ DefaultsData* getConfigDefaults()
     AddDEF("lastCharacter", "");
     AddDEF("altfpslimit", 5);
     AddDEF("updatehost", "");
-    AddDEF("screenshotDirectory2", "");
+    AddDEF("screenshotDirectory3", "");
     AddDEF("useScreenshotDirectorySuffix", true);
     AddDEF("screenshotDirectorySuffix", "");
     AddDEF("EnableSync", false);
@@ -162,7 +165,7 @@ DefaultsData* getConfigDefaults()
     AddDEF("leftTolerance", 100);
     AddDEF("rightTolerance", 100);
     AddDEF("logNpcInGui", true);
-    AddDEF("download-music", false);
+    AddDEF("download-music", true);
     AddDEF("guialpha", 0.8F);
     AddDEF("ChatLogLength", 0);
     AddDEF("enableChatLog", true);
@@ -225,7 +228,7 @@ DefaultsData* getConfigDefaults()
     AddDEF("showBattleEvents", false);
     AddDEF("showMobHP", true);
     AddDEF("showOwnHP", true);
-    AddDEF("usePersistentIP", false);
+    AddDEF("usePersistentIP", true);
     AddDEF("showJobExp", true);
     AddDEF("showExtMinimaps", false);
     AddDEF("hideChatInput", true);
@@ -294,14 +297,18 @@ DefaultsData* getConfigDefaults()
     AddDEF("repeateInterval2", SDL_DEFAULT_REPEAT_DELAY);
 #endif
     AddDEF("compresstextures", 0);
-    AddDEF("rectangulartextures", true);
+    AddDEF("rectangulartextures", false);
     AddDEF("networksleep", 0);
     AddDEF("newtextures", true);
     AddDEF("videodetected", false);
     AddDEF("hideErased", false);
     AddDEF("enableDelayedAnimations", true);
     AddDEF("enableCompoundSpriteDelay", true);
+#ifdef ANDROID
+    AddDEF("useAtlases", false);
+#else
     AddDEF("useAtlases", true);
+#endif
     AddDEF("useTextureSampler", false);
     AddDEF("ministatussaved", 0);
     AddDEF("allowscreensaver", false);
@@ -318,19 +325,19 @@ DefaultsData* getConfigDefaults()
     AddDEF("selfMouseHeal", true);
     AddDEF("serverslistupdate", "");
     AddDEF("fadeoutmusic", true);
-    AddDEF("screenActionKeyboard", Input::KEY_SHOW_KEYBOARD);
-    AddDEF("screenActionButton0", Input::KEY_STOP_SIT);
-    AddDEF("screenActionButton1", Input::KEY_TARGET_ATTACK);
-    AddDEF("screenActionButton2", Input::KEY_PICKUP);
-    AddDEF("screenActionButton3", Input::KEY_PATHFIND);
-    AddDEF("screenActionButton4", Input::KEY_STOP_ATTACK);
-    AddDEF("screenActionButton5", Input::KEY_TALK);
-    AddDEF("screenActionButton6", Input::KEY_TARGET_NPC);
-    AddDEF("screenActionButton7", Input::KEY_TARGET_MONSTER);
-    AddDEF("screenActionButton8", Input::KEY_WINDOW_CHAT);
-    AddDEF("screenActionButton9", Input::KEY_WINDOW_DEBUG);
-    AddDEF("screenActionButton10", Input::KEY_TRADE);
-    AddDEF("screenActionButton11", Input::KEY_DIRECT_DOWN);
+    AddDEF("screenActionKeyboard", InputAction::SHOW_KEYBOARD);
+    AddDEF("screenActionButton0", InputAction::TALK);
+    AddDEF("screenActionButton1", InputAction::TARGET_ATTACK);
+    AddDEF("screenActionButton2", InputAction::PICKUP);
+    AddDEF("screenActionButton3", InputAction::STOP_SIT);
+    AddDEF("screenActionButton4", InputAction::TARGET_NPC);
+    AddDEF("screenActionButton5", InputAction::WINDOW_STATUS);
+    AddDEF("screenActionButton6", InputAction::WINDOW_INVENTORY);
+    AddDEF("screenActionButton7", InputAction::WINDOW_SKILL);
+    AddDEF("screenActionButton8", InputAction::WINDOW_SOCIAL);
+    AddDEF("screenActionButton9", InputAction::WINDOW_DEBUG);
+    AddDEF("screenActionButton10", InputAction::TRADE);
+    AddDEF("screenActionButton11", InputAction::DIRECT_DOWN);
     AddDEF("screenButtonsFormat", 0);
     AddDEF("autoresizeminimaps", false);
     AddDEF("showGuildOnline", false);
@@ -345,7 +352,7 @@ DefaultsData* getConfigDefaults()
     AddDEF("uselonglivesprites", false);
     AddDEF("uselonglivesounds", true);
     AddDEF("screenDensity", 0);
-    AddDEF("cfgver", 4);
+    AddDEF("cfgver", 9);
     AddDEF("enableDebugLog", false);
     AddDEF("doubleClick", true);
     AddDEF("useDiagonalSpeed", true);
@@ -356,6 +363,11 @@ DefaultsData* getConfigDefaults()
     AddDEF("showmotd", false);
     AddDEF("playMapAnimations", true);
     AddDEF("usepets", true);
+    AddDEF("scale", 1);
+    AddDEF("addwatermark", true);
+    AddDEF("hidesupport", false);
+    AddDEF("showserverpos", false);
+    AddDEF("textureSize", "1024,1024,1024,1024,1024,1024");
     return configData;
 }
 
@@ -441,6 +453,22 @@ DefaultsData* getBrandingDefaults()
 
     AddDEF("wallpaperFile", "");
     AddDEF("dataPath", "");
+
+    AddDEF("androidDownloadUrl", "https://play.google.com/store/apps/details"
+        "?id=org.evolonline.beta.manaplus");
+    AddDEF("androidDownloadText", "Google Play");
+    AddDEF("otherDownloadUrl", "http://manaplus.org/");
+    AddDEF("otherDownloadText", "http://manaplus.org/");
+    AddDEF("windowsDownloadUrl", "http://download.evolonline.org/"
+        "manaplus/download/manaplus-win32.exe");
+    AddDEF("windowsDownloadText", "download here");
+    AddDEF("updateMirror1", "http://manaplus.org/update/");
+    AddDEF("updateMirror2", "http://www.manaplus.org/update/");
+    AddDEF("updateMirror3", "http://www2.manaplus.org/update/");
+    AddDEF("updateMirror4", "http://www3.manaplus.org/update/");
+    AddDEF("updateMirror5", "");
+    AddDEF("updateMirror6", "");
+    AddDEF("updateMirror7", "");
     return configData;
 }
 
@@ -452,6 +480,11 @@ DefaultsData* getPathsDefaults()
     AddDEF("sprites", "graphics/sprites/");
     AddDEF("spriteErrorFile", "error.xml");
     AddDEF("guiIcons", "graphics/guiicons/");
+    AddDEF("shaders", "graphics/shaders/");
+    AddDEF("help", "help/");
+
+    AddDEF("simpleVertexShader", "simple_vertex.glsl");
+    AddDEF("simpleFragmentShader", "simple_frag.glsl");
 
     AddDEF("particles", "graphics/particles/");
     AddDEF("portalEffectFile", "warparea.particle.xml");
@@ -472,8 +505,6 @@ DefaultsData* getPathsDefaults()
     AddDEF("wallpapers", "graphics/images/");
     AddDEF("wallpaperFile", "login_wallpaper.png");
 
-    AddDEF("help", "help/");
-
     AddDEF("statusEffectsFile", "status-effects.xml");
     AddDEF("statusEffectsPatchFile", "status-effects_patch.xml");
     AddDEF("statusEffectsPatchDir", "status-effects.d");
@@ -524,12 +555,15 @@ DefaultsData* getPathsDefaults()
     AddDEF("monstersPatchFile", "monsters_patch.xml");
     AddDEF("monstersPatchDir", "monsters.d");
     AddDEF("mapsRemapFile", "maps/remap.xml");
+    AddDEF("mapsRemapPatchFile", "maps/remap_patch.xml");
+    AddDEF("mapsRemapPatchDir", "maps/remap.d");
     AddDEF("mapsFile", "maps.xml");
     AddDEF("mapsPatchFile", "maps_patch.xml");
     AddDEF("mapsPatchDir", "maps.d");
     AddDEF("deadMessagesFile", "deadmessages.xml");
     AddDEF("deadMessagesPatchFile", "deadmessages_patch.xml");
     AddDEF("deadMessagesPatchDir", "deadmessages.d");
+    AddDEF("poisonEffectName", "poison");
 
     return configData;
 }
