@@ -22,14 +22,16 @@
 
 #include "dropshortcut.h"
 
-#include "client.h"
 #include "configuration.h"
 #include "inventory.h"
 #include "item.h"
+#include "settings.h"
 
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 
+#include "net/packetlimiter.h"
+
 #include "debug.h"
 
 DropShortcut *dropShortcut = nullptr;
@@ -91,10 +93,10 @@ void DropShortcut::save() const
 
 void DropShortcut::dropFirst() const
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
-    if (!client->limitPackets(PACKET_DROP))
+    if (!PacketLimiter::limitPackets(PACKET_DROP))
         return;
 
     const int itemId = getItem(0);
@@ -108,8 +110,8 @@ void DropShortcut::dropFirst() const
             ->findItem(itemId, itemColor);
         if (item && item->getQuantity())
         {
-            const int cnt = player_node->getQuickDropCounter();
-            if (player_node->isServerBuggy())
+            const int cnt = settings.quickDropCounter;
+            if (localPlayer->isServerBuggy())
             {
                 PlayerInfo::dropItem(item, cnt, true);
             }
@@ -124,22 +126,22 @@ void DropShortcut::dropFirst() const
 
 void DropShortcut::dropItems(const int cnt)
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
-    if (player_node->isServerBuggy())
+    if (localPlayer->isServerBuggy())
     {
-        dropItem(player_node->getQuickDropCounter());
+        dropItem(settings.quickDropCounter);
         return;
     }
 
     int n = 0;
-    const int sz = player_node->getQuickDropCounter();
+    const int sz = settings.quickDropCounter;
     for (int f = 0; f < 9; f++)
     {
         for (int i = 0; i < sz; i++)
         {
-            if (!client->limitPackets(PACKET_DROP))
+            if (!PacketLimiter::limitPackets(PACKET_DROP))
                 return;
             if (dropItem())
                 n++;
