@@ -71,7 +71,7 @@ class DropShortcut final
         /**
          * Returns the amount of shortcut items.
          */
-        int getItemCount() const A_WARN_UNUSED
+        static int getItemCount() A_WARN_UNUSED
         { return DROP_SHORTCUT_ITEMS; }
 
         /**
