@@ -23,16 +23,23 @@
 #include "configuration.h"
 #include "client.h"
 #include "game.h"
-#ifdef USE_SDL2
-#include "render/graphics.h"
-#endif
 #include "logger.h"
+#ifdef USE_MUMBLE
 #include "mumblemanager.h"
+#endif
+#include "sdlshared.h"
+#include "settings.h"
+
+#include "gui/windowmanager.h"
 
 #include "being/localplayer.h"
 
 #include "input/inputmanager.h"
 
+#ifdef USE_SDL2
+#include "render/graphics.h"
+#endif
+
 #include "utils/process.h"
 
 #include "debug.h"
@@ -60,8 +67,9 @@ void EventsManager::shutdown()
    config.removeListeners(this);
 }
 
-bool EventsManager::handleCommonEvents(const SDL_Event &event)
+bool EventsManager::handleCommonEvents(const SDL_Event &event) const
 {
+    BLOCK_START("EventsManager::handleCommonEvents")
     if (mLogInput)
         logEvent(event);
 
@@ -70,21 +78,28 @@ bool EventsManager::handleCommonEvents(const SDL_Event &event)
         case SDL_QUIT:
             client->setState(STATE_EXIT);
             logger->log1("force exit");
+            BLOCK_END("EventsManager::handleCommonEvents")
             return true;
 #ifdef USE_SDL2
         case SDL_WINDOWEVENT:
             handleSDL2WindowEvent(event);
+            BLOCK_END("EventsManager::handleCommonEvents")
             return true;
 #else
         case SDL_VIDEORESIZE:
-            client->resizeVideo(event.resize.w, event.resize.h);
+            WindowManager::resizeVideo(event.resize.w,
+                event.resize.h,
+                false);
+            BLOCK_END("EventsManager::handleCommonEvents")
             return true;
         case SDL_ACTIVEEVENT:
             handleActive(event);
+            BLOCK_END("EventsManager::handleCommonEvents")
             return true;
 #ifdef ANDROID
         case SDL_KEYBOARDSHOW:
-            client->updateScreenKeyboard(event.user.code);
+            WindowManager::updateScreenKeyboard(event.user.code);
+            BLOCK_END("EventsManager::handleCommonEvents")
             return true;
         case SDL_ACCELEROMETER:
             break;
@@ -95,13 +110,18 @@ bool EventsManager::handleCommonEvents(const SDL_Event &event)
     }
 
     if (inputManager.handleEvent(event))
+    {
+        BLOCK_END("EventsManager::handleCommonEvents")
         return true;
+    }
 
+    BLOCK_END("EventsManager::handleCommonEvents")
     return false;
 }
 
-bool EventsManager::handleEvents()
+bool EventsManager::handleEvents() const
 {
+    BLOCK_START("EventsManager::handleEvents")
     if (Game::instance())
     {
         // Let the game handle the events while it is active
@@ -110,13 +130,8 @@ bool EventsManager::handleEvents()
     else
     {
         SDL_Event event;
-
         // Handle SDL events
-#ifdef USE_SDL2
         while (SDL_WaitEventTimeout(&event, 0))
-#else
-        while (SDL_PollEvent(&event))
-#endif
         {
             if (!handleCommonEvents(event))
             {
@@ -144,30 +159,31 @@ bool EventsManager::handleEvents()
             }
 
 #ifdef USE_MUMBLE
-            if (player_node && mumbleManager)
+            if (localPlayer && mumbleManager)
             {
-                mumbleManager->setPos(player_node->getTileX(),
-                    player_node->getTileY(), player_node->getDirection());
+                mumbleManager->setPos(localPlayer->getTileX(),
+                    localPlayer->getTileY(), localPlayer->getDirection());
             }
 #endif
         }
         if (client->getState() == STATE_EXIT)
+        {
+            BLOCK_END("EventsManager::handleEvents")
             return true;
+        }
     }
+    BLOCK_END("EventsManager::handleEvents")
     return false;
 }
 
-void EventsManager::handleGameEvents()
+void EventsManager::handleGameEvents() const
 {
+    BLOCK_START("EventsManager::handleGameEvents")
     Game *const game = Game::instance();
 
     // Events
     SDL_Event event;
-#ifdef USE_SDL2
     while (SDL_WaitEventTimeout(&event, 0))
-#else
-    while (SDL_PollEvent(&event))
-#endif
     {
         if (event.type == SDL_KEYDOWN || event.type == SDL_KEYUP)
             game->updateHistory(event);
@@ -176,6 +192,7 @@ void EventsManager::handleGameEvents()
         if (handleCommonEvents(event))
             break;
     }  // End while
+    BLOCK_END("EventsManager::handleGameEvents")
 }
 
 void EventsManager::optionChanged(const std::string &name)
@@ -188,16 +205,16 @@ void EventsManager::logEvent(const SDL_Event &event)
 {
     switch (event.type)
     {
-        case SDL_MOUSEMOTION:
-            logger->log("event: SDL_MOUSEMOTION: %d,%d,%d",
-                event.motion.state, event.motion.x, event.motion.y);
-            break;
 #ifdef USE_SDL2
 #define winEventLog(name, name2) \
     case name: \
         str = name2; \
         break
 
+        case SDL_MOUSEMOTION:
+            logger->log("event: SDL_MOUSEMOTION: %u,%u,%u",
+                event.motion.state, event.motion.x, event.motion.y);
+            break;
         case SDL_FINGERDOWN:
         {
             const SDL_TouchFingerEvent &touch = event.tfinger;
@@ -297,8 +314,8 @@ void EventsManager::logEvent(const SDL_Event &event)
         {
             const char *const text = event.text.text;
             logger->log("event: SDL_TEXTINPUT: %s", text);
-            const int sz = strlen(event.text.text);
-            for (int f = 0; f < sz; f ++)
+            const size_t sz = strlen(event.text.text);
+            for (size_t f = 0; f < sz; f ++)
                 logger->log("dec: %d", text[f]);
             break;
         }
@@ -321,6 +338,10 @@ void EventsManager::logEvent(const SDL_Event &event)
             logger->log("SDL_APP_DIDENTERBACKGROUND");
             break;
 #else
+        case SDL_MOUSEMOTION:
+            logger->log("event: SDL_MOUSEMOTION: %u,%d,%d",
+                event.motion.state, event.motion.x, event.motion.y);
+            break;
         case SDL_KEYDOWN:
             logger->log("event: SDL_KEYDOWN: %d,%d,%d", event.key.state,
                 event.key.keysym.scancode, event.key.keysym.unicode);
@@ -390,7 +411,7 @@ void EventsManager::logEvent(const SDL_Event &event)
 #endif
 #endif
         default:
-            logger->log("event: other: %d", event.type);
+            logger->log("event: other: %u", event.type);
             break;
     };
 }
@@ -404,42 +425,44 @@ void EventsManager::handleSDL2WindowEvent(const SDL_Event &event)
     switch (eventType)
     {
         case SDL_WINDOWEVENT_RESIZED:
-            client->resizeVideo(event.window.data1, event.window.data2, false);
+            WindowManager::resizeVideo(event.window.data1,
+                event.window.data2,
+                false);
             break;
         case SDL_WINDOWEVENT_ENTER:
-            client->setMouseFocused(true);
+            settings.mouseFocused = true;
             break;
         case SDL_WINDOWEVENT_LEAVE:
-            client->setMouseFocused(false);
+            settings.mouseFocused = false;
             break;
         case SDL_WINDOWEVENT_FOCUS_GAINED:
-            client->setInputFocused(true);
+            settings.inputFocused = true;
             break;
         case SDL_WINDOWEVENT_FOCUS_LOST:
-            client->setInputFocused(false);
+            settings.inputFocused = false;
             break;
         case SDL_WINDOWEVENT_MINIMIZED:
-            client->setIsMinimized(true);
+            WindowManager::setIsMinimized(true);
             if (inGame)
             {
-                if (player_node && !player_node->getAway())
+                if (localPlayer && !settings.awayMode)
                 {
                     fpsLimit = config.getIntValue("altfpslimit");
-                    player_node->setHalfAway(true);
+                    localPlayer->setHalfAway(true);
                 }
             }
             setPriority(false);
             break;
         case SDL_WINDOWEVENT_RESTORED:
         case SDL_WINDOWEVENT_MAXIMIZED:
-            client->setIsMinimized(false);
+            WindowManager::setIsMinimized(false);
             if (inGame)
             {
-                if (player_node)
+                if (localPlayer)
                 {
-                    if (!player_node->getAway())
+                    if (!settings.awayMode)
                         fpsLimit = config.getIntValue("fpslimit");
-                    player_node->setHalfAway(false);
+                    localPlayer->setHalfAway(false);
                 }
             }
             setPriority(true);
@@ -455,10 +478,10 @@ void EventsManager::handleSDL2WindowEvent(const SDL_Event &event)
         || eventType == SDL_WINDOWEVENT_RESTORED
         || eventType == SDL_WINDOWEVENT_MAXIMIZED)
     {
-        if (player_node)
+        if (localPlayer)
         {
-            player_node->updateStatus();
-            player_node->updateName();
+            localPlayer->updateStatus();
+            localPlayer->updateName();
         }
         Game::instance()->updateFrameRate(fpsLimit);
     }
@@ -472,12 +495,12 @@ void EventsManager::handleActive(const SDL_Event &event)
     {
         if (event.active.gain)
         {   // window restore
-            client->setIsMinimized(false);
-            if (inGame && player_node)
+            WindowManager::setIsMinimized(false);
+            if (inGame && localPlayer)
             {
-                if (!player_node->getAway())
+                if (!settings.awayMode)
                     fpsLimit = config.getIntValue("fpslimit");
-                player_node->setHalfAway(false);
+                localPlayer->setHalfAway(false);
             }
             setPriority(true);
         }
@@ -486,25 +509,25 @@ void EventsManager::handleActive(const SDL_Event &event)
 #ifdef ANDROID
             client->setState(STATE_EXIT);
 #else
-            client->setIsMinimized(true);
-            if (inGame && player_node && !player_node->getAway())
+            WindowManager::setIsMinimized(true);
+            if (inGame && localPlayer && !settings.awayMode)
             {
                 fpsLimit = config.getIntValue("altfpslimit");
-                player_node->setHalfAway(true);
+                localPlayer->setHalfAway(true);
             }
             setPriority(false);
 #endif
         }
-        if (inGame && player_node)
-            player_node->updateStatus();
+        if (inGame && localPlayer)
+            localPlayer->updateStatus();
     }
-    if (inGame && player_node)
-        player_node->updateName();
+    if (inGame && localPlayer)
+        localPlayer->updateName();
 
     if (event.active.state & SDL_APPINPUTFOCUS)
-        client->setInputFocused(event.active.gain);
+        settings.inputFocused = event.active.gain;
     if (event.active.state & SDL_APPMOUSEFOCUS)
-        client->setMouseFocused(event.active.gain);
+        settings.mouseFocused = event.active.gain;
     if (inGame)
         Game::instance()->updateFrameRate(fpsLimit);
 }
