@@ -27,16 +27,20 @@
 #include "render/graphics.h"
 
 #include "gui/gui.h"
-#include "gui/sdlfont.h"
 #include "gui/userpalette.h"
 
+#include "gui/fonts/font.h"
+
 #include "resources/iteminfo.h"
 
 #include "resources/db/itemdb.h"
 
+#include "resources/map/map.h"
+
 #include "debug.h"
 
 extern int serverVersion;
+extern volatile int cur_time;
 
 FloorItem::FloorItem(const int id, const int itemId, const int x, const int y,
                      const int amount, const unsigned char color) :
@@ -109,7 +113,7 @@ void FloorItem::draw(Graphics *const graphics,
     BLOCK_START("FloorItem::draw")
     const int x = mX * mMap->getTileWidth() + offsetX;
     const int y = mY * mMap->getTileHeight() + offsetY;
-    gcn::Font *font = nullptr;
+    Font *font = nullptr;
 
     if (mHighlight)
     {
@@ -122,24 +126,23 @@ void FloorItem::draw(Graphics *const graphics,
 
             if (curTime > mDropTime + 28 && curTime < mDropTime + 50)
             {
-                graphics->setColor(gcn::Color(80, 200, 20, 200));
-                graphics->fillRectangle(gcn::Rectangle(
+                graphics->setColor(Color(80, 200, 20, 200));
+                graphics->fillRectangle(Rect(
                                         x, y, dx, dy));
             }
             else if (curTime > mDropTime + 19
                      && curTime < mDropTime + 28)
             {
-                graphics->setColor(gcn::Color(200, 80, 20,
+                graphics->setColor(Color(200, 80, 20,
                     80 + 10 * (curTime - mDropTime - 18)));
-                graphics->fillRectangle(gcn::Rectangle(
+                graphics->fillRectangle(Rect(
                     x, y, dx, dy));
             }
             else if (curTime > mDropTime && curTime < mDropTime + 20)
             {
-                graphics->setColor(gcn::Color(20, 20, 255,
+                graphics->setColor(Color(20, 20, 255,
                     7 * (curTime - mDropTime)));
-                graphics->fillRectangle(gcn::Rectangle(
-                                        x, y, dx, dy));
+                graphics->fillRectangle(Rect(x, y, dx, dy));
             }
         }
     }
@@ -153,7 +156,7 @@ void FloorItem::draw(Graphics *const graphics,
     {
         if (font && mAmount > 1)
         {
-//            graphics->setColor(gcn::Color(255, 255, 255, 100));
+//            graphics->setColor(Color(255, 255, 255, 100));
             graphics->setColor(userPalette->getColor(
                 UserPalette::FLOOR_ITEM_TEXT));
             font->drawString(graphics, toString(mAmount), x, y);
