@@ -33,9 +33,11 @@
 #include "effectmanager.h"
 #include "emoteshortcut.h"
 #include "eventsmanager.h"
+#include "gamemodifiers.h"
 #include "guildmanager.h"
 #include "itemshortcut.h"
 #include "soundmanager.h"
+#include "settings.h"
 #include "spellshortcut.h"
 #include "touchmanager.h"
 
@@ -48,27 +50,29 @@
 #include "input/joystick.h"
 #include "input/keyboardconfig.h"
 
+#include "gui/chatconsts.h"
+#include "gui/dialogsmanager.h"
 #include "gui/gui.h"
-#ifdef MANASERV_SUPPORT
-#include "gui/specialswindow.h"
-#endif
+#include "gui/popupmanager.h"
 #include "gui/viewport.h"
+#include "gui/windowmanager.h"
 #include "gui/windowmenu.h"
 
+#include "gui/fonts/font.h"
+
 #include "gui/windows/botcheckerwindow.h"
+#include "gui/windows/chatwindow.h"
 #include "gui/windows/debugwindow.h"
 #include "gui/windows/didyouknowwindow.h"
 #include "gui/windows/emotewindow.h"
 #include "gui/windows/equipmentwindow.h"
-#include "gui/windows/helpwindow.h"
 #include "gui/windows/inventorywindow.h"
 #include "gui/windows/killstats.h"
 #include "gui/windows/minimap.h"
 #include "gui/windows/ministatuswindow.h"
 #include "gui/windows/npcdialog.h"
-#include "gui/windows/okdialog.h"
 #include "gui/windows/outfitwindow.h"
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 #include "gui/windows/shopwindow.h"
 #include "gui/windows/shortcutwindow.h"
 #include "gui/windows/skilldialog.h"
@@ -94,83 +98,40 @@
 #include "net/net.h"
 #include "net/packetcounters.h"
 
+#include "resources/delayedmanager.h"
 #include "resources/imagewriter.h"
 #include "resources/mapreader.h"
 #include "resources/resourcemanager.h"
 
 #include "resources/db/mapdb.h"
 
+#include "resources/map/map.h"
+
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 #include "utils/langs.h"
 #include "utils/mkdir.h"
 #include "utils/physfstools.h"
-#include "utils/process.h"
 #include "utils/sdlcheckutils.h"
 #include "utils/timer.h"
 
+#include "listeners/errorlistener.h"
+
 #include <sstream>
 #include <string>
 
+#ifdef USE_MUMBLE
 #include "mumblemanager.h"
+#endif
 
 #include "debug.h"
 
-Joystick *joystick = nullptr;
-
-OkDialog *weightNotice = nullptr;
-int weightNoticeTime = 0;
-OkDialog *deathNotice = nullptr;
 QuitDialog *quitDialog = nullptr;
 Window *disconnectedDialog = nullptr;
 
-ChatWindow *chatWindow = nullptr;
-EmoteWindow *emoteWindow = nullptr;
-StatusWindow *statusWindow = nullptr;
-MiniStatusWindow *miniStatusWindow = nullptr;
-InventoryWindow *inventoryWindow = nullptr;
-ShopWindow *shopWindow = nullptr;
-SkillDialog *skillDialog = nullptr;
-Minimap *minimap = nullptr;
-EquipmentWindow *equipmentWindow = nullptr;
-EquipmentWindow *beingEquipmentWindow = nullptr;
-TradeWindow *tradeWindow = nullptr;
-HelpWindow *helpWindow = nullptr;
-DebugWindow *debugWindow = nullptr;
-ShortcutWindow *itemShortcutWindow = nullptr;
-ShortcutWindow *emoteShortcutWindow = nullptr;
-OutfitWindow *outfitWindow = nullptr;
-#ifdef MANASERV_SUPPORT
-SpecialsWindow *specialsWindow = nullptr;
-#endif
-ShortcutWindow *dropShortcutWindow = nullptr;
-ShortcutWindow *spellShortcutWindow = nullptr;
-WhoIsOnline *whoIsOnline = nullptr;
-DidYouKnowWindow *didYouKnowWindow = nullptr;
-KillStats *killStats = nullptr;
-BotCheckerWindow *botCheckerWindow = nullptr;
-SocialWindow *socialWindow = nullptr;
-QuestsWindow *questsWindow = nullptr;
-WindowMenu *windowMenu = nullptr;
-
-ActorManager *actorManager = nullptr;
-CommandHandler *commandHandler = nullptr;
-#ifdef USE_MUMBLE
-MumbleManager *mumbleManager = nullptr;
-#endif
-Particle *particleEngine = nullptr;
-EffectManager *effectManager = nullptr;
-SpellManager *spellManager = nullptr;
-Viewport *viewport = nullptr;                    /**< Viewport on the map. */
-GuildManager *guildManager = nullptr;
-AuctionManager *auctionManager = nullptr;
-
-ChatTab *localChatTab = nullptr;
-ChatTab *debugChatTab = nullptr;
-TradeTab *tradeChatTab = nullptr;
-BattleTab *battleChatTab = nullptr;
-GmTab *gmChatTab = nullptr;
-LangTab *langChatTab = nullptr;
+int weightNoticeTime = 0;
 
+bool mStatsReUpdated = false;
 const unsigned adjustDelay = 10;
 
 /**
@@ -194,6 +155,9 @@ static void initEngines()
     keyboard.update();
     if (joystick)
         joystick->update();
+
+    GameModifiers::init();
+    UpdateStatusListener::distributeEvent();
 }
 
 /**
@@ -214,7 +178,7 @@ static void createGuiWindows()
     chatWindow->postInit();
     tradeWindow = new TradeWindow;
     equipmentWindow = new EquipmentWindow(PlayerInfo::getEquipment(),
-        player_node);
+        localPlayer);
     equipmentWindow->postInit();
     beingEquipmentWindow = new EquipmentWindow(nullptr, nullptr, true);
     beingEquipmentWindow->postInit();
@@ -236,7 +200,7 @@ static void createGuiWindows()
     for (unsigned f = 0; f < SHORTCUT_TABS; f ++)
     {
         itemShortcutWindow->addTab(toString(f + 1),
-            new ItemShortcutContainer(f));
+            new ItemShortcutContainer(nullptr, f));
     }
     if (config.getBoolValue("showDidYouKnow"))
     {
@@ -245,20 +209,20 @@ static void createGuiWindows()
     }
 
     emoteShortcutWindow = new ShortcutWindow("EmoteShortcut",
-        new EmoteShortcutContainer, "emotes.xml", 130, 480);
+        new EmoteShortcutContainer(nullptr),
+        "emotes.xml",
+        130, 480);
     outfitWindow = new OutfitWindow();
-#ifdef MANASERV_SUPPORT
-    specialsWindow = new SpecialsWindow();
-#endif
     dropShortcutWindow = new ShortcutWindow("DropShortcut",
-        new DropShortcutContainer, "drops.xml");
+        new DropShortcutContainer(nullptr),
+        "drops.xml");
 
     spellShortcutWindow = new ShortcutWindow("SpellShortcut", "spells.xml",
                                              265, 328);
     for (unsigned f = 0; f < SPELL_SHORTCUT_TABS; f ++)
     {
         spellShortcutWindow->addTab(toString(f + 1),
-            new SpellShortcutContainer(f));
+            new SpellShortcutContainer(nullptr, f));
     }
 
     botCheckerWindow = new BotCheckerWindow;
@@ -267,8 +231,7 @@ static void createGuiWindows()
     killStats = new KillStats;
     socialWindow = new SocialWindow;
     socialWindow->postInit();
-    if (serverVersion >= 6)
-        questsWindow = new QuestsWindow;
+    questsWindow = new QuestsWindow;
 
     // TRANSLATORS: chat tab header
     localChatTab = new ChatTab(chatWindow, _("General"), GENERAL_CHANNEL);
@@ -310,8 +273,8 @@ static void createGuiWindows()
         battleChatTab = nullptr;
     }
 
-    if (player_node && !gmChatTab && config.getBoolValue("enableGmTab")
-        && player_node->getGMLevel() > 0)
+    if (localPlayer && !gmChatTab && config.getBoolValue("enableGmTab")
+        && localPlayer->getGMLevel() > 0)
     {
         gmChatTab = new GmTab(chatWindow);
     }
@@ -322,14 +285,12 @@ static void createGuiWindows()
     if (setupWindow)
         setupWindow->externalUpdate();
 
-    if (player_node)
-        player_node->updateStatus();
+    if (localPlayer)
+        localPlayer->updateStatus();
 
     Net::getGeneralHandler()->gameStarted();
 }
 
-#define del_0(X) { delete X; X = nullptr; }
-
 /**
  * Destroy all the globally accessible gui windows
  */
@@ -346,40 +307,37 @@ static void destroyGuiWindows()
     if (guildManager)
         guildManager->clear();
 
-    del_0(windowMenu);
-    del_0(localChatTab)  // Need to do this first, so it can remove itself
-    del_0(debugChatTab)
-    del_0(tradeChatTab)
-    del_0(battleChatTab)
-    del_0(langChatTab)
-    del_0(gmChatTab);
+    delete2(windowMenu);
+    delete2(localChatTab)  // Need to do this first, so it can remove itself
+    delete2(debugChatTab)
+    delete2(tradeChatTab)
+    delete2(battleChatTab)
+    delete2(langChatTab)
+    delete2(gmChatTab);
     logger->log("start deleting");
-    del_0(emoteWindow);
-    del_0(chatWindow)
+    delete2(emoteWindow);
+    delete2(chatWindow)
     logger->log("end deleting");
-    del_0(statusWindow)
-    del_0(miniStatusWindow)
-    del_0(inventoryWindow)
-    del_0(shopWindow)
-    del_0(skillDialog)
-    del_0(minimap)
-    del_0(equipmentWindow)
-    del_0(beingEquipmentWindow)
-    del_0(tradeWindow)
-    del_0(debugWindow)
-    del_0(itemShortcutWindow)
-    del_0(emoteShortcutWindow)
-    del_0(outfitWindow)
-#ifdef MANASERV_SUPPORT
-    del_0(specialsWindow)
-#endif
-    del_0(socialWindow)
-    del_0(dropShortcutWindow);
-    del_0(spellShortcutWindow);
-    del_0(botCheckerWindow);
-    del_0(questsWindow);
-    del_0(whoIsOnline);
-    del_0(killStats);
+    delete2(statusWindow)
+    delete2(miniStatusWindow)
+    delete2(inventoryWindow)
+    delete2(shopWindow)
+    delete2(skillDialog)
+    delete2(minimap)
+    delete2(equipmentWindow)
+    delete2(beingEquipmentWindow)
+    delete2(tradeWindow)
+    delete2(debugWindow)
+    delete2(itemShortcutWindow)
+    delete2(emoteShortcutWindow)
+    delete2(outfitWindow)
+    delete2(socialWindow)
+    delete2(dropShortcutWindow);
+    delete2(spellShortcutWindow);
+    delete2(botCheckerWindow);
+    delete2(questsWindow);
+    delete2(whoIsOnline);
+    delete2(killStats);
 
     if (auctionManager && AuctionManager::getEnableAuctionBot())
         auctionManager->reload();
@@ -390,7 +348,7 @@ static void destroyGuiWindows()
 
 Game *Game::mInstance = nullptr;
 
-Game::Game():
+Game::Game() :
     mCurrentMap(nullptr),
     mMapName(""),
     mValidSpeed(true),
@@ -399,7 +357,8 @@ Game::Game():
     mAdjustPerfomance(config.getBoolValue("adjustPerfomance")),
     mLowerCounter(0),
     mPing(0),
-    mTime(cur_time + 1)
+    mTime(cur_time + 1),
+    mTime2(cur_time + 10)
 {
     touchManager.setInGame(true);
     spellManager = new SpellManager;
@@ -419,7 +378,7 @@ Game::Game():
     viewport->setSize(mainGraphics->mWidth, mainGraphics->mHeight);
     PlayerInfo::clear();
 
-    gcn::Container *const top = static_cast<gcn::Container*>(gui->getTop());
+    BasicContainer2 *const top = static_cast<BasicContainer2*>(gui->getTop());
     if (top)
         top->add(viewport);
     viewport->requestMoveToBottom();
@@ -440,7 +399,7 @@ Game::Game():
 
     // Initialize beings
     if (actorManager)
-        actorManager->setPlayer(player_node);
+        actorManager->setPlayer(localPlayer);
 
     Net::getGameHandler()->ping(tick_time);
 
@@ -451,8 +410,8 @@ Game::Game():
     if (guildManager && GuildManager::getEnableGuildBot())
         guildManager->requestGuildInfo();
 
-    if (player_node)
-        player_node->updatePets();
+    if (localPlayer)
+        localPlayer->updatePets();
 }
 
 Game::~Game()
@@ -465,20 +424,20 @@ Game::~Game()
 
     AnimatedSprite::setEnableCache(false);
 
-    del_0(actorManager)
+    delete2(actorManager)
     if (client->getState() != STATE_CHANGE_MAP)
-        del_0(player_node)
-    del_0(commandHandler)
-    del_0(effectManager)
-    del_0(particleEngine)
-    del_0(viewport)
-    del_0(mCurrentMap)
-    del_0(spellManager)
-    del_0(spellShortcut)
-    del_0(auctionManager)
-    del_0(guildManager)
+        delete2(localPlayer)
+    delete2(commandHandler)
+    delete2(effectManager)
+    delete2(particleEngine)
+    delete2(viewport)
+    delete2(mCurrentMap)
+    delete2(spellManager)
+    delete2(spellShortcut)
+    delete2(auctionManager)
+    delete2(guildManager)
 #ifdef USE_MUMBLE
-    del_0(mumbleManager)
+    delete2(mumbleManager)
 #endif
 
     Being::clearCache();
@@ -486,6 +445,15 @@ Game::~Game()
     PlayerInfo::gameDestroyed();
 }
 
+void Game::addWatermark()
+{
+    if (!boldFont || !config.getBoolValue("addwatermark"))
+        return;
+    mainGraphics->setColorAll(theme->getColor(Theme::TEXT, 255),
+        theme->getColor(Theme::TEXT_OUTLINE, 255));
+    boldFont->drawString(mainGraphics, settings.serverName, 100, 50);
+}
+
 bool Game::createScreenshot()
 {
     if (!mainGraphics)
@@ -498,11 +466,13 @@ bool Game::createScreenshot()
         mainGraphics->setSecure(true);
         mainGraphics->prepareScreenshot();
         gui->draw();
+        addWatermark();
         screenshot = mainGraphics->getScreenshot();
         mainGraphics->setSecure(false);
     }
     else
     {
+        addWatermark();
         screenshot = mainGraphics->getScreenshot();
     }
 
@@ -514,7 +484,7 @@ bool Game::createScreenshot()
 
 bool Game::saveScreenshot(SDL_Surface *const screenshot)
 {
-    std::string screenshotDirectory = client->getScreenshotDirectory();
+    std::string screenshotDirectory = settings.screenshotDir;
     if (mkdir_r(screenshotDirectory.c_str()) != 0)
     {
         logger->log("Directory %s doesn't exist and can't be created! "
@@ -524,7 +494,6 @@ bool Game::saveScreenshot(SDL_Surface *const screenshot)
     }
 
     // Search for an unused screenshot name
-    std::stringstream filenameSuffix;
     std::stringstream filename;
     std::fstream testExists;
     bool found = false;
@@ -536,7 +505,7 @@ bool Game::saveScreenshot(SDL_Surface *const screenshot)
     struct tm *const timeinfo = localtime(&rawtime);
     strftime(buffer, 99, "%Y-%m-%d_%H-%M-%S", timeinfo);
 
-    const std::string serverName = client->getServerName();
+    const std::string serverName = settings.serverName;
     std::string screenShortStr;
     if (serverName.empty())
     {
@@ -572,7 +541,7 @@ bool Game::saveScreenshot(SDL_Surface *const screenshot)
             // TRANSLATORS: save file message
             std::string str = strprintf(_("Screenshot saved as %s"),
                 fileNameStr.c_str());
-            localChatTab->chatLog(str, BY_SERVER);
+            localChatTab->chatLog(str, ChatMsgType::BY_SERVER);
         }
     }
     else
@@ -581,7 +550,7 @@ bool Game::saveScreenshot(SDL_Surface *const screenshot)
         {
             // TRANSLATORS: save file message
             localChatTab->chatLog(_("Saving screenshot failed!"),
-                                  BY_SERVER);
+                                  ChatMsgType::BY_SERVER);
         }
         logger->log1("Error: could not save screenshot.");
     }
@@ -604,18 +573,20 @@ void Game::logic()
     if (mCurrentMap)
         mCurrentMap->update();
 
-    cur_time = static_cast<int>(time(nullptr));
     BLOCK_END("Game::logic")
 }
 
 void Game::slowLogic()
 {
     BLOCK_START("Game::slowLogic")
-    if (player_node)
-        player_node->slowLogic();
+    if (localPlayer)
+        localPlayer->slowLogic();
     const int time = cur_time;
     if (mTime != time)
     {
+        if (valTest(Updated))
+            mValidSpeed = false;
+
         mTime = time + 1;
         if (botCheckerWindow)
             botCheckerWindow->slowLogic();
@@ -630,12 +601,19 @@ void Game::slowLogic()
         Being::reReadConfig();
         if (killStats)
             killStats->recalcStats();
+
+        if (time > mTime2 || mTime2 - time > 10)
+        {
+            mTime2 = time + 10;
+            config.writeUpdated();
+            serverConfig.writeUpdated();
+        }
     }
 
     if (shopWindow)
         shopWindow->updateTimes();
     if (mainGraphics->getOpenGL())
-        ResourceManager::delayedLoad();
+        DelayedManager::delayedLoad();
     if (guildManager)
         guildManager->slowLogic();
     PacketCounters::update();
@@ -652,7 +630,7 @@ void Game::slowLogic()
             {
                 // TRANSLATORS: error message text
                 errorMessage = _("The connection to the server was lost.");
-                disconnectedDialog = client->openErrorDialog(
+                disconnectedDialog = DialogsManager::openErrorDialog(
                     // TRANSLATORS: error message header
                     _("Network Error"), errorMessage, false);
                 disconnectedDialog->addActionListener(&errorListener);
@@ -669,8 +647,8 @@ void Game::slowLogic()
                 map->saveExtraLayer();
             }
         }
-        closeDialogs();
-        client->setFramerate(config.getIntValue("fpslimit"));
+        DialogsManager::closeDialogs();
+        WindowManager::setFramerate(config.getIntValue("fpslimit"));
         mNextAdjustTime = cur_time + adjustDelay;
         if (client->getState() != STATE_ERROR)
             errorMessage.clear();
@@ -707,13 +685,13 @@ void Game::adjustPerfomance()
     {
         mNextAdjustTime = time + adjustDelay;
 
-        if (mAdjustLevel > 3 || !player_node || player_node->getHalfAway()
-            || player_node->getAway())
+        if (mAdjustLevel > 3 || !localPlayer || localPlayer->getHalfAway()
+            || settings.awayMode)
         {
             return;
         }
 
-        int maxFps = client->getFramerate();
+        int maxFps = WindowManager::getFramerate();
         if (maxFps != config.getIntValue("fpslimit"))
             return;
 
@@ -743,7 +721,7 @@ void Game::adjustPerfomance()
                         if (localChatTab)
                         {
                             localChatTab->chatLog("Auto disable Show "
-                                "beings transparency", BY_SERVER);
+                                "beings transparency", ChatMsgType::BY_SERVER);
                         }
                     }
                     else
@@ -760,7 +738,7 @@ void Game::adjustPerfomance()
                         if (localChatTab)
                         {
                             localChatTab->chatLog("Auto lower Particle "
-                                "effects", BY_SERVER);
+                                "effects", ChatMsgType::BY_SERVER);
                         }
                     }
                     else
@@ -777,7 +755,7 @@ void Game::adjustPerfomance()
                         if (localChatTab)
                         {
                             localChatTab->chatLog("Auto enable opacity cache",
-                                BY_SERVER);
+                                ChatMsgType::BY_SERVER);
                         }
                     }
                     break;
@@ -821,11 +799,15 @@ void Game::resetAdjustLevel()
 
 void Game::handleMove()
 {
-    if (!player_node)
+    BLOCK_START("Game::handleMove")
+    if (!localPlayer)
+    {
+        BLOCK_END("Game::handleMove")
         return;
+    }
 
     // Moving player around
-    if (player_node->isAlive()
+    if (localPlayer->isAlive()
         && chatWindow
         && !chatWindow->isInputFocused()
         && !InventoryWindow::isAnyInputFocused()
@@ -833,59 +815,64 @@ void Game::handleMove()
     {
         NpcDialog *const dialog = NpcDialog::getActive();
         if (dialog)
+        {
+            BLOCK_END("Game::handleMove")
             return;
+        }
 
         // Ignore input if either "ignore" key is pressed
         // Stops the character moving about if the user's window manager
         // uses "ignore+arrow key" to switch virtual desktops.
-        if (inputManager.isActionActive(Input::KEY_IGNORE_INPUT_1) ||
-            inputManager.isActionActive(Input::KEY_IGNORE_INPUT_2))
+        if (inputManager.isActionActive(InputAction::IGNORE_INPUT_1) ||
+            inputManager.isActionActive(InputAction::IGNORE_INPUT_2))
         {
+            BLOCK_END("Game::handleMove")
             return;
         }
 
         unsigned char direction = 0;
 
         // Translate pressed keys to movement and direction
-        if (inputManager.isActionActive(Input::KEY_MOVE_UP) ||
+        if (inputManager.isActionActive(InputAction::MOVE_UP) ||
             (joystick && joystick->isUp()))
         {
-            direction |= Being::UP;
+            direction |= BeingDirection::UP;
             setValidSpeed();
-            player_node->cancelFollow();
+            localPlayer->cancelFollow();
         }
-        else if (inputManager.isActionActive(Input::KEY_MOVE_DOWN) ||
+        else if (inputManager.isActionActive(InputAction::MOVE_DOWN) ||
                  (joystick && joystick->isDown()))
         {
-            direction |= Being::DOWN;
+            direction |= BeingDirection::DOWN;
             setValidSpeed();
-            player_node->cancelFollow();
+            localPlayer->cancelFollow();
         }
 
-        if (inputManager.isActionActive(Input::KEY_MOVE_LEFT) ||
+        if (inputManager.isActionActive(InputAction::MOVE_LEFT) ||
             (joystick && joystick->isLeft()))
         {
-            direction |= Being::LEFT;
+            direction |= BeingDirection::LEFT;
             setValidSpeed();
-            player_node->cancelFollow();
+            localPlayer->cancelFollow();
         }
-        else if (inputManager.isActionActive(Input::KEY_MOVE_RIGHT) ||
+        else if (inputManager.isActionActive(InputAction::MOVE_RIGHT) ||
                  (joystick && joystick->isRight()))
         {
-            direction |= Being::RIGHT;
+            direction |= BeingDirection::RIGHT;
             setValidSpeed();
-            player_node->cancelFollow();
+            localPlayer->cancelFollow();
         }
-        else if (inputManager.isActionActive(Input::KEY_MOVE_FORWARD))
+        else if (inputManager.isActionActive(InputAction::MOVE_FORWARD))
         {
-            direction = player_node->getDirection();
+            direction = localPlayer->getDirection();
             setValidSpeed();
-            player_node->cancelFollow();
+            localPlayer->cancelFollow();
         }
 
-        if (!inputManager.isActionActive(Input::KEY_EMOTE) || direction == 0)
+        if (!inputManager.isActionActive(InputAction::EMOTE) || direction == 0)
             moveInDirection(direction);
     }
+    BLOCK_END("Game::handleMove")
 }
 
 void Game::moveInDirection(const unsigned char direction)
@@ -893,23 +880,23 @@ void Game::moveInDirection(const unsigned char direction)
     if (!viewport)
         return;
 
-    if (!viewport->getCameraMode())
+    if (!settings.cameraMode)
     {
-        if (player_node)
-            player_node->specialMove(direction);
+        if (localPlayer)
+            localPlayer->specialMove(direction);
     }
     else
     {
         int dx = 0;
         int dy = 0;
-        if (direction & Being::LEFT)
+        if (direction & BeingDirection::LEFT)
             dx = -5;
-        else if (direction & Being::RIGHT)
+        else if (direction & BeingDirection::RIGHT)
             dx = 5;
 
-        if (direction & Being::UP)
+        if (direction & BeingDirection::UP)
             dy = -5;
-        else if (direction & Being::DOWN)
+        else if (direction & BeingDirection::DOWN)
             dy = 5;
         viewport->moveCamera(dx, dy);
     }
@@ -919,9 +906,9 @@ void Game::updateFrameRate(int fpsLimit)
 {
     if (!fpsLimit)
     {
-        if (player_node && player_node->getAway())
+        if (settings.awayMode)
         {
-            if (client->getInputFocused() || client->getMouseFocused())
+            if (settings.inputFocused || settings.mouseFocused)
                 fpsLimit = config.getIntValue("fpslimit");
             else
                 fpsLimit = config.getIntValue("altfpslimit");
@@ -931,7 +918,7 @@ void Game::updateFrameRate(int fpsLimit)
             fpsLimit = config.getIntValue("fpslimit");
         }
     }
-    client->setFramerate(fpsLimit);
+    WindowManager::setFramerate(fpsLimit);
     mNextAdjustTime = cur_time + adjustDelay;
 }
 
@@ -947,15 +934,15 @@ void Game::handleInput()
     eventsManager.handleGameEvents();
 
     // If the user is configuring the keys then don't respond.
-    if (!player_node || !keyboard.isEnabled() || player_node->getAway())
+    if (!keyboard.isEnabled() || settings.awayMode)
     {
         BLOCK_END("Game::handleInput 1")
         return;
     }
 
     // If pressed outfits keys, stop processing keys.
-    if (inputManager.isActionActive(Input::KEY_WEAR_OUTFIT)
-        || inputManager.isActionActive(Input::KEY_COPY_OUTFIT)
+    if (inputManager.isActionActive(InputAction::WEAR_OUTFIT)
+        || inputManager.isActionActive(InputAction::COPY_OUTFIT)
         || (setupWindow && setupWindow->isWindowVisible()))
     {
         BLOCK_END("Game::handleInput 1")
@@ -963,7 +950,7 @@ void Game::handleInput()
     }
 
     handleMove();
-    inputManager.handleRepeat();
+    InputManager::handleRepeat();
     BLOCK_END("Game::handleInput 1")
 }
 
@@ -979,8 +966,11 @@ void Game::changeMap(const std::string &mapPath)
     ResourceManager *const resman = ResourceManager::getInstance();
     resman->cleanProtected();
 
-    if (viewport)
-        viewport->clearPopup();
+    if (popupManager)
+    {
+        popupManager->clearPopup();
+        popupManager->closePopupMenu();
+    }
 
     // Clean up floor items, beings and particles
     if (actorManager)
@@ -989,15 +979,12 @@ void Game::changeMap(const std::string &mapPath)
     // Close the popup menu on map change so that invalid options can't be
     // executed.
     if (viewport)
-    {
-        viewport->closePopupMenu();
         viewport->cleanHoverItems();
-    }
 
     // Unset the map of the player so that its particles are cleared before
     // being deleted in the next step
-    if (player_node)
-        player_node->setMap(nullptr);
+    if (localPlayer)
+        localPlayer->setMap(nullptr);
 
     if (particleEngine)
         particleEngine->clear();
@@ -1009,7 +996,7 @@ void Game::changeMap(const std::string &mapPath)
     std::string realFullMap = paths.getValue("maps", "maps/").append(
         MapDB::getMapName(mMapName)).append(".tmx");
 
-    if (!resman->exists(realFullMap))
+    if (!PhysFs::exists(realFullMap.c_str()))
         realFullMap.append(".gz");
 
     // Attempt to load the new map
@@ -1065,8 +1052,8 @@ void Game::changeMap(const std::string &mapPath)
         mumbleManager->setMap(mapPath);
 #endif
 
-    if (player_node)
-        player_node->recreateItemParticles();
+    if (localPlayer)
+        localPlayer->recreateItemParticles();
 
     Net::getGameHandler()->mapLoadedEvent();
     BLOCK_END("Game::changeMap")
@@ -1074,10 +1061,10 @@ void Game::changeMap(const std::string &mapPath)
 
 void Game::updateHistory(const SDL_Event &event)
 {
-    if (!player_node || !player_node->getAttackType())
+    if (!localPlayer || !settings.attackType)
         return;
 
-    if (event.key.keysym.sym != -1)
+    if (static_cast<int>(event.key.keysym.sym) != -1)
     {
         bool old = false;
 
@@ -1131,7 +1118,7 @@ void Game::checkKeys()
     const int timeRange = 120;
     const int cntInTime = 130;
 
-    if (!player_node || !player_node->getAttackType())
+    if (!localPlayer || !settings.attackType)
         return;
 
     const int time = cur_time;
@@ -1166,17 +1153,7 @@ void Game::clearKeysArray()
     }
 }
 
-void Game::closeDialogs()
-{
-    Client::closeDialogs();
-    if (deathNotice)
-    {
-        deathNotice->scheduleDelete();
-        deathNotice = nullptr;
-    }
-}
-
-void Game::videoResized(const int width, const int height) const
+void Game::videoResized(const int width, const int height)
 {
     if (viewport)
         viewport->setSize(width, height);
