@@ -23,27 +23,31 @@
 #ifdef USE_OPENGL
 
 #ifndef WIN32
+
 #ifdef ANDROID
 #include <GLES2/gl2.h>
 #include <GLES/glext.h>
 #include <EGL/egl.h>
 #ifndef USE_SDL2
 #include <SDL_android.h>
+#endif  // USE_SDL2
+#else  // ANDROID
+#include <GL/glx.h>
+#endif  // ANDROID
+#else  // WIN32
 #endif
-#else
-#include "GL/glx.h"
-#endif
-#else
-#endif
-
-#endif
+#endif  // USE_OPENGL
 
 #include "configuration.h"
 #include "logger.h"
+#include "settings.h"
 
 #include "render/graphics.h"
+#include "render/mglcheck.h"
 #include "render/mgl.h"
+#include "render/mglemu.h"
 #include "render/mobileopenglgraphics.h"
+#include "render/modernopenglgraphics.h"
 #include "render/normalopenglgraphics.h"
 #include "render/renderers.h"
 #include "render/safeopenglgraphics.h"
@@ -54,6 +58,8 @@
 #include "resources/openglimagehelper.h"
 #include "resources/sdlimagehelper.h"
 
+#include "render/mglfunctions.h"
+
 #ifdef USE_SDL2
 #include "render/sdl2softwaregraphics.h"
 
@@ -61,6 +67,7 @@
 #include "resources/surfaceimagehelper.h"
 #endif
 
+#include "utils/delete2.h"
 #include "utils/sdlhelper.h"
 #include "utils/stringutils.h"
 
@@ -74,24 +81,12 @@
 #ifndef GL_MAX_RENDERBUFFER_SIZE
 #define GL_MAX_RENDERBUFFER_SIZE 0x84E8
 #endif
-
-#ifdef WIN32
-#define getFunction(name) wglGetProcAddress(name)
-#elif defined ANDROID
-#define getFunction(name) eglGetProcAddress(name)
-#elif defined __APPLE__
-#define getFunction(name) nullptr
-#else
-#define getFunction(name) glXGetProcAddress(\
-    reinterpret_cast<const GLubyte*>(name))
-#endif
-
-#define assignFunction(func, name) m##func \
-    = reinterpret_cast<func##_t>(getFunction(name))
 #endif
 
 GraphicsManager graphicsManager;
 
+RenderType openGLMode = RENDER_SOFTWARE;
+
 const int densitySize = 6;
 
 const std::string densityNames[] =
@@ -104,14 +99,21 @@ const std::string densityNames[] =
     "xxhigh"
 };
 
+#ifdef USE_OPENGL
+GLenum GraphicsManager::mLastError(GL_NO_ERROR);
+#endif
+
 GraphicsManager::GraphicsManager() :
     mExtensions(),
     mPlatformExtensions(),
     mGlVersionString(),
     mGlVendor(),
     mGlRenderer(),
+    mGlShaderVersionString(),
     mMinor(0),
     mMajor(0),
+    mSLMinor(0),
+    mSLMajor(0),
     mPlatformMinor(0),
     mPlatformMajor(0),
     mMaxVertices(500),
@@ -125,6 +127,7 @@ GraphicsManager::GraphicsManager() :
     mUseTextureSampler(true),
     mTextureSampler(0),
     mSupportDebug(0),
+    mSupportModernOpengl(false),
 #endif
     mUseAtlases(false)
 {
@@ -133,7 +136,7 @@ GraphicsManager::GraphicsManager() :
 GraphicsManager::~GraphicsManager()
 {
 #ifdef USE_OPENGL
-    if (mglGenSamplers && mTextureSampler)
+    if (isGLNotNull(mglGenSamplers) && mTextureSampler)
         mglDeleteSamplers(1, &mTextureSampler);
 #endif
 }
@@ -158,6 +161,7 @@ int GraphicsManager::detectGraphics()
     SDL_Window *const window = createWindow(100, 100, 0,
         SDL_ANYFORMAT | SDL_OPENGL);
     mainGraphics->setWindow(window, 100, 100);
+    mainGraphics->createGLContext();
 
     initOpenGL();
     logVersion();
@@ -217,7 +221,8 @@ int GraphicsManager::detectGraphics()
         mode = RENDER_SOFTWARE;
     }
 
-    if (mode > 0 && findI(mGlVersionString, "Mesa") != std::string::npos)
+    if (mode != RENDER_SOFTWARE && findI(mGlVersionString, "Mesa")
+        != std::string::npos)
     {
         // Mesa detected. In latest Mesa look like compression broken.
         config.setValue("compresstextures", false);
@@ -229,14 +234,25 @@ int GraphicsManager::detectGraphics()
     config.write();
 
     logger->log("detection complete");
-    return mode | (1024 * textureSampler) | (2048 * compressTextures);
+    return static_cast<unsigned int>(mode)
+        | (1024 * textureSampler) | (2048 * compressTextures);
 }
 
-void GraphicsManager::initGraphics(const bool noOpenGL)
+void GraphicsManager::createRenderers()
 {
     RenderType useOpenGL = RENDER_SOFTWARE;
-    if (!noOpenGL)
-        useOpenGL = intToRenderType(config.getIntValue("opengl"));
+    if (!settings.options.noOpenGL)
+    {
+        if (settings.options.renderer < 0)
+        {
+            useOpenGL = intToRenderType(config.getIntValue("opengl"));
+            settings.options.renderer = useOpenGL;
+        }
+        else
+        {
+            useOpenGL = intToRenderType(settings.options.renderer);
+        }
+    }
 
     // Setup image loading for the right image format
     ImageHelper::setOpenGlMode(useOpenGL);
@@ -275,6 +291,12 @@ void GraphicsManager::initGraphics(const bool noOpenGL)
             mainGraphics = new SafeOpenGLGraphics;
             mUseTextureSampler = false;
             break;
+        case RENDER_MODERN_OPENGL:
+            imageHelper = new OpenGLImageHelper;
+            surfaceImageHelper = new SurfaceImageHelper;
+            mainGraphics = new ModernOpenGLGraphics;
+            mUseTextureSampler = true;
+            break;
 #endif
         case RENDER_GLES_OPENGL:
             imageHelper = new OpenGLImageHelper;
@@ -293,15 +315,17 @@ void GraphicsManager::initGraphics(const bool noOpenGL)
 #endif
     };
     mUseAtlases = (useOpenGL == RENDER_NORMAL_OPENGL
-        || useOpenGL == RENDER_SAFE_OPENGL || useOpenGL == RENDER_GLES_OPENGL)
+        || useOpenGL == RENDER_SAFE_OPENGL
+        || useOpenGL == RENDER_MODERN_OPENGL
+        || useOpenGL == RENDER_GLES_OPENGL)
         && config.getBoolValue("useAtlases");
 
 #else  // USE_OPENGL
 
-void GraphicsManager::initGraphics(const bool noOpenGL)
+void GraphicsManager::createRenderers()
 {
     RenderType useOpenGL = RENDER_SOFTWARE;
-    if (!noOpenGL)
+    if (!settings.options.noOpenGL)
         useOpenGL = intToRenderType(config.getIntValue("opengl"));
 
     // Setup image loading for the right image format
@@ -313,6 +337,7 @@ void GraphicsManager::initGraphics(const bool noOpenGL)
         case RENDER_SOFTWARE:
         case RENDER_SAFE_OPENGL:
         case RENDER_GLES_OPENGL:
+        case RENDER_MODERN_OPENGL:
         case RENDER_NORMAL_OPENGL:
         case RENDER_NULL:
         case RENDER_LAST:
@@ -342,6 +367,15 @@ void GraphicsManager::initGraphics(const bool noOpenGL)
 #endif  // USE_OPENGL
 }
 
+void GraphicsManager::deleteRenderers()
+{
+    delete2(mainGraphics);
+    if (imageHelper != surfaceImageHelper)
+        delete surfaceImageHelper;
+    surfaceImageHelper = nullptr;
+    delete2(imageHelper);
+}
+
 void GraphicsManager::setVideoMode()
 {
     const int bpp = 0;
@@ -376,9 +410,10 @@ void GraphicsManager::setVideoMode()
     int width = config.getIntValue("screenwidth");
     int height = config.getIntValue("screenheight");
 #endif
+    const int scale = config.getIntValue("scale");
 
     // Try to set the desired video mode
-    if (!mainGraphics->setVideoMode(width, height, bpp,
+    if (!mainGraphics->setVideoMode(width, height, scale, bpp,
         fullscreen, hwaccel, enableResize, noFrame))
     {
         logger->log(strprintf("Couldn't set %dx%dx%d video mode: %s",
@@ -396,7 +431,7 @@ void GraphicsManager::setVideoMode()
             config.setValueInt("screenwidth", oldWidth);
             config.setValueInt("screenheight", oldHeight);
             config.setValue("screen", oldFullscreen == 1);
-            if (!mainGraphics->setVideoMode(oldWidth, oldHeight, bpp,
+            if (!mainGraphics->setVideoMode(oldWidth, oldHeight, scale, bpp,
                 oldFullscreen, hwaccel, enableResize, noFrame))
             {
                 logger->safeError(strprintf("Couldn't restore %dx%dx%d "
@@ -407,18 +442,73 @@ void GraphicsManager::setVideoMode()
     }
 }
 
-SDL_Window *GraphicsManager::createWindow(const int w, const int h,
-                                          const int bpp, const int flags)
+void GraphicsManager::initGraphics()
 {
-#ifdef USE_SDL2
-    return SDL_CreateWindow("ManaPlus", SDL_WINDOWPOS_UNDEFINED,
-        SDL_WINDOWPOS_UNDEFINED, w, h, flags);
+    openGLMode = intToRenderType(config.getIntValue("opengl"));
+#ifdef USE_OPENGL
+    OpenGLImageHelper::setBlur(config.getBoolValue("blur"));
+    SurfaceImageHelper::SDLSetEnableAlphaCache(
+        config.getBoolValue("alphaCache") && !openGLMode);
+    ImageHelper::setEnableAlpha(config.getFloatValue("guialpha") != 1.0F
+        || openGLMode);
 #else
-    return SDL_SetVideoMode(w, h, bpp, flags);
+    SurfaceImageHelper::SDLSetEnableAlphaCache(
+        config.getBoolValue("alphaCache"));
+    ImageHelper::setEnableAlpha(config.getFloatValue("guialpha") != 1.0F);
+#endif
+    createRenderers();
+    detectPixelSize();
+    setVideoMode();
+#ifdef USE_OPENGL
+    const RenderType oldOpenGLMode = openGLMode;
+    if (openGLMode == RENDER_MODERN_OPENGL)
+    {
+        if (!mSupportModernOpengl || !checkGLVersion(3, 0))
+        {
+            logger->log("Fallback to normal OpenGL mode");
+            openGLMode = RENDER_NORMAL_OPENGL;
+        }
+    }
+    if (openGLMode == RENDER_NORMAL_OPENGL || openGLMode == RENDER_GLES_OPENGL)
+    {
+        if (!checkGLVersion(2, 0))
+        {
+            logger->log("Fallback to safe OpenGL mode");
+            openGLMode = RENDER_SAFE_OPENGL;
+        }
+    }
+    if (openGLMode != oldOpenGLMode && !settings.options.test.empty())
+    {
+        deleteRenderers();
+        settings.options.renderer = static_cast<int>(openGLMode);
+        config.setValue("opengl", settings.options.renderer);
+        createRenderers();
+        detectPixelSize();
+        setVideoMode();
+    }
+#if !defined(ANDROID) && !defined(__APPLE__)
+    const std::string str = config.getStringValue("textureSize");
+    std::vector<int> sizes;
+    splitToIntVector(sizes, str, ',');
+    const size_t pos = static_cast<size_t>(openGLMode);
+    if (sizes.size() <= pos)
+        settings.textureSize = 1024;
+    else
+        settings.textureSize = sizes[pos];
+    logger->log("Detected max texture size: %u", settings.textureSize);
+#endif  // !defined(ANDROID) && !defined(__APPLE__)
 #endif
 }
 
 #ifdef USE_SDL2
+SDL_Window *GraphicsManager::createWindow(const int w, const int h,
+                                          const int bpp A_UNUSED,
+                                          const int flags)
+{
+    return SDL_CreateWindow("ManaPlus", SDL_WINDOWPOS_UNDEFINED,
+        SDL_WINDOWPOS_UNDEFINED, w, h, flags);
+}
+
 SDL_Renderer *GraphicsManager::createRenderer(SDL_Window *const window,
                                               const int flags)
 {
@@ -427,6 +517,12 @@ SDL_Renderer *GraphicsManager::createRenderer(SDL_Window *const window,
     SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_BLEND);
     return renderer;
 }
+#else
+SDL_Window *GraphicsManager::createWindow(const int w, const int h,
+                                          const int bpp, const int flags)
+{
+    return SDL_SetVideoMode(w, h, bpp, flags);
+}
 #endif
 
 #ifdef USE_OPENGL
@@ -436,7 +532,7 @@ void GraphicsManager::updateExtensions()
     logger->log1("opengl extensions: ");
     if (checkGLVersion(3, 0))
     {   // get extensions in new way
-        assignFunction(glGetStringi, "glGetStringi");
+        assignFunction2(glGetStringi, "glGetStringi");
         std::string extList;
         int num = 0;
         glGetIntegerv(GL_NUM_EXTENSIONS, &num);
@@ -545,25 +641,26 @@ bool GraphicsManager::supportExtension(const std::string &ext) const
     return mExtensions.find(ext) != mExtensions.end();
 }
 
-void GraphicsManager::updateTextureFormat()
+void GraphicsManager::updateTextureFormat() const
 {
     const int compressionFormat = config.getIntValue("compresstextures");
-    if (compressionFormat)
+    // using extensions if can
+    if (checkGLVersion(3, 1) || supportExtension("GL_ARB_texture_compression"))
     {
-        // using extensions if can
-        if (supportExtension("GL_ARB_texture_compression"))
+        GLint num = 0;
+        glGetIntegerv(GL_NUM_COMPRESSED_TEXTURE_FORMATS, &num);
+        logger->log("support %d compressed formats", num);
+        GLint *const formats = new GLint[num > 10
+            ? static_cast<size_t>(num) : 10];
+        glGetIntegerv(GL_COMPRESSED_TEXTURE_FORMATS, formats);
+        for (int f = 0; f < num; f ++)
+            logger->log(" 0x%x", static_cast<unsigned int>(formats[f]));
+
+        if (compressionFormat)
         {
             if (supportExtension("GL_EXT_texture_compression_s3tc")
                 || supportExtension("3DFX_texture_compression_FXT1"))
             {
-                GLint num = 0;
-                glGetIntegerv(GL_NUM_COMPRESSED_TEXTURE_FORMATS, &num);
-                logger->log("support %d compressed formats", num);
-                GLint *const formats = new GLint[num > 10 ? num : 10];
-                glGetIntegerv(GL_COMPRESSED_TEXTURE_FORMATS, formats);
-                for (int f = 0; f < num; f ++)
-                    logger->log(" 0x%x", formats[f]);
-
                 for (int f = 0; f < num; f ++)
                 {
                     if (formats[f] == GL_COMPRESSED_RGBA_S3TC_DXT5_EXT
@@ -598,6 +695,7 @@ void GraphicsManager::updateTextureFormat()
             {
                 if (compressionFormat == 3)
                 {
+                    delete []formats;
                     OpenGLImageHelper::setInternalTextureType(
                         GL_COMPRESSED_RGBA_ARB);
                     logger->log1("using ARB texture compression");
@@ -605,14 +703,18 @@ void GraphicsManager::updateTextureFormat()
                 }
             }
         }
-        else
-        {
-            logger->log1("no correct compression format found");
-        }
+    }
+    else
+    {
+        logger->log1("no correct compression format found");
     }
 
+    const int renderer = settings.options.renderer;
+
     // using default formats
-    if (config.getBoolValue("newtextures"))
+    if (renderer == RENDER_MODERN_OPENGL
+        || renderer == RENDER_GLES_OPENGL
+        || config.getBoolValue("newtextures"))
     {
         OpenGLImageHelper::setInternalTextureType(GL_RGBA);
         logger->log1("using RGBA texture format");
@@ -623,9 +725,6 @@ void GraphicsManager::updateTextureFormat()
         logger->log1("using 4 texture format");
     }
 }
-#endif
-
-#ifdef USE_OPENGL
 
 void GraphicsManager::logString(const char *const format, const int num)
 {
@@ -648,6 +747,8 @@ void GraphicsManager::setGLVersion()
     sscanf(mGlVersionString.c_str(), "%5d.%5d", &mMajor, &mMinor);
     mGlVendor = getGLString(GL_VENDOR);
     mGlRenderer = getGLString(GL_RENDERER);
+    mGlShaderVersionString = getGLString(GL_SHADING_LANGUAGE_VERSION);
+    sscanf(mGlShaderVersionString.c_str(), "%5d.%5d", &mSLMajor, &mSLMinor);
 }
 
 void GraphicsManager::logVersion() const
@@ -655,6 +756,7 @@ void GraphicsManager::logVersion() const
     logger->log("gl vendor: " + mGlVendor);
     logger->log("gl renderer: " + mGlRenderer);
     logger->log("gl version: " + mGlVersionString);
+    logger->log("glsl version: " + mGlShaderVersionString);
 }
 
 bool GraphicsManager::checkGLVersion(const int major, const int minor) const
@@ -662,6 +764,11 @@ bool GraphicsManager::checkGLVersion(const int major, const int minor) const
     return mMajor > major || (mMajor == major && mMinor >= minor);
 }
 
+bool GraphicsManager::checkSLVersion(const int major, const int minor) const
+{
+    return mSLMajor > major || (mSLMajor == major && mSLMinor >= minor);
+}
+
 bool GraphicsManager::checkPlatformVersion(const int major,
                                            const int minor) const
 {
@@ -739,18 +846,38 @@ void GraphicsManager::deleteFBO(FBOInfo *const fbo)
 
 void GraphicsManager::initOpenGLFunctions()
 {
+    const bool is10 = checkGLVersion(1, 0);
+    const bool is11 = checkGLVersion(1, 1);
+    const bool is12 = checkGLVersion(1, 2);
+    const bool is15 = checkGLVersion(1, 5);
+    const bool is20 = checkGLVersion(2, 0);
+    const bool is21 = checkGLVersion(2, 1);
+    const bool is30 = checkGLVersion(3, 0);
+    const bool is33 = checkGLVersion(3, 3);
+    const bool is41 = checkGLVersion(4, 1);
+    const bool is42 = checkGLVersion(4, 2);
+    const bool is43 = checkGLVersion(4, 3);
+    const bool is44 = checkGLVersion(4, 4);
+    const bool isDao = supportExtension("GL_EXT_direct_state_access");
+    mSupportModernOpengl = true;
+
     // Texture sampler
-    if (checkGLVersion(1, 0) && supportExtension("GL_ARB_sampler_objects"))
+    if (is10 && (is33 || supportExtension("GL_ARB_sampler_objects")))
     {
         logger->log1("found GL_ARB_sampler_objects");
-        assignFunction(glGenSamplers, "glGenSamplers");
-        assignFunction(glDeleteSamplers, "glDeleteSamplers");
-        assignFunction(glBindSampler, "glBindSampler");
-        assignFunction(glSamplerParameteri, "glSamplerParameteri");
-        if (mglGenSamplers && config.getBoolValue("useTextureSampler"))
+        assignFunction(glGenSamplers);
+        assignFunction(glDeleteSamplers);
+        assignFunction(glBindSampler);
+        assignFunction(glSamplerParameteri);
+        if (isGLNotNull(mglGenSamplers)
+            && config.getBoolValue("useTextureSampler"))
+        {
             mUseTextureSampler &= true;
+        }
         else
+        {
             mUseTextureSampler = false;
+        }
     }
     else
     {
@@ -758,55 +885,83 @@ void GraphicsManager::initOpenGLFunctions()
         mUseTextureSampler = false;
     }
 
-    if (!checkGLVersion(1, 1))
+    if (!is11)
+    {
+        mSupportModernOpengl = false;
+        emulateFunction(glTextureSubImage2D);
         return;
+    }
 
-    if (supportExtension("GL_ARB_framebuffer_object"))
+    if (isDao)
+    {
+        logger->log1("found GL_EXT_direct_state_access");
+        assignFunctionEmu2(glTextureSubImage2D, "glTextureSubImage2DEXT");
+    }
+    else
+    {
+        logger->log1("GL_EXT_direct_state_access not found");
+        emulateFunction(glTextureSubImage2D);
+    }
+
+    if (is12 && (is42 || supportExtension("GL_ARB_texture_storage")))
+    {
+        logger->log1("found GL_ARB_texture_storage");
+        assignFunction(glTexStorage2D);
+    }
+    else
+    {
+        logger->log1("GL_ARB_texture_storage not found");
+    }
+    if (is30 || supportExtension("GL_ARB_framebuffer_object"))
     {   // frame buffer supported
         logger->log1("found GL_ARB_framebuffer_object");
-        assignFunction(glGenRenderbuffers, "glGenRenderbuffers");
-        assignFunction(glBindRenderbuffer, "glBindRenderbuffer");
-        assignFunction(glRenderbufferStorage, "glRenderbufferStorage");
-        assignFunction(glGenFramebuffers, "glGenFramebuffers");
-        assignFunction(glBindFramebuffer, "glBindFramebuffer");
-        assignFunction(glFramebufferTexture2D, "glFramebufferTexture2D");
-        assignFunction(glFramebufferRenderbuffer, "glFramebufferRenderbuffer");
-        assignFunction(glDeleteFramebuffers, "glDeleteFramebuffers");
-        assignFunction(glDeleteRenderbuffers, "glDeleteRenderbuffers");
+        assignFunction(glGenRenderbuffers);
+        assignFunction(glBindRenderbuffer);
+        assignFunction(glRenderbufferStorage);
+        assignFunction(glGenFramebuffers);
+        assignFunction(glBindFramebuffer);
+        assignFunction(glFramebufferTexture2D);
+        assignFunction(glFramebufferRenderbuffer);
+        assignFunction(glDeleteFramebuffers);
+        assignFunction(glDeleteRenderbuffers);
+        assignFunction(glCheckFramebufferStatus);
     }
     else if (supportExtension("GL_EXT_framebuffer_object"))
     {   // old frame buffer extension
         logger->log1("found GL_EXT_framebuffer_object");
-        assignFunction(glGenRenderbuffers, "glGenRenderbuffersEXT");
-        assignFunction(glBindRenderbuffer, "glBindRenderbufferEXT");
-        assignFunction(glRenderbufferStorage, "glRenderbufferStorageEXT");
-        assignFunction(glGenFramebuffers, "glGenFramebuffersEXT");
-        assignFunction(glBindFramebuffer, "glBindFramebufferEXT");
-        assignFunction(glFramebufferTexture2D, "glFramebufferTexture2DEXT");
-        assignFunction(glFramebufferRenderbuffer,
-            "glFramebufferRenderbufferEXT");
-        assignFunction(glDeleteFramebuffers, "glDeleteFramebuffersEXT");
-        assignFunction(glDeleteRenderbuffers, "glDeleteRenderbuffersEXT");
+        assignFunctionEXT(glGenRenderbuffers);
+        assignFunctionEXT(glBindRenderbuffer);
+        assignFunctionEXT(glRenderbufferStorage);
+        assignFunctionEXT(glGenFramebuffers);
+        assignFunctionEXT(glBindFramebuffer);
+        assignFunctionEXT(glFramebufferTexture2D);
+        assignFunctionEXT(glFramebufferRenderbuffer);
+        assignFunctionEXT(glDeleteFramebuffers);
+        assignFunctionEXT(glDeleteRenderbuffers);
     }
     else
     {   // no frame buffer support
-        logger->log1("frame buffer not found");
+        logger->log1("GL_ARB_framebuffer_object or "
+            "GL_EXT_framebuffer_object not found");
         config.setValue("usefbo", false);
     }
 
     // debug extensions
-    if (supportExtension("GL_KHR_debug"))
+    if (is43 || supportExtension("GL_KHR_debug"))
     {
         logger->log1("found GL_KHR_debug");
-        assignFunction(glDebugMessageControl, "glDebugMessageControl");
-        assignFunction(glDebugMessageCallback, "glDebugMessageCallback");
+        assignFunction(glDebugMessageControl);
+        assignFunction(glDebugMessageCallback);
+        assignFunction(glPushDebugGroup);
+        assignFunction(glPopDebugGroup);
+        assignFunction(glObjectLabel);
         mSupportDebug = 2;
     }
     else if (supportExtension("GL_ARB_debug_output"))
     {
         logger->log1("found GL_ARB_debug_output");
-        assignFunction(glDebugMessageControl, "glDebugMessageControlARB");
-        assignFunction(glDebugMessageCallback, "glDebugMessageCallbackARB");
+        assignFunctionARB(glDebugMessageControl);
+        assignFunctionARB(glDebugMessageCallback);
         mSupportDebug = 1;
     }
     else
@@ -815,8 +970,186 @@ void GraphicsManager::initOpenGLFunctions()
         mSupportDebug = 0;
     }
 
+    if (supportExtension("GL_GREMEDY_frame_terminator"))
+    {
+        logger->log1("found GL_GREMEDY_frame_terminator");
+        assignFunction2(glFrameTerminator, "glFrameTerminatorGREMEDY");
+    }
+    else
+    {
+        logger->log1("GL_GREMEDY_frame_terminator not found");
+    }
+    if (supportExtension("GL_EXT_debug_label"))
+    {
+        logger->log1("found GL_EXT_debug_label");
+        assignFunctionEXT(glLabelObject);
+        if (isGLNull(mglLabelObject))
+            assignFunction2(glLabelObject, "glObjectLabel");
+        assignFunctionEXT(glGetObjectLabel);
+    }
+    else
+    {
+        logger->log1("GL_EXT_debug_label not found");
+    }
+    if (supportExtension("GL_GREMEDY_string_marker"))
+    {
+        logger->log1("found GL_GREMEDY_string_marker");
+        assignFunction2(glPushGroupMarker, "glStringMarkerGREMEDY");
+    }
+    else
+    {
+        logger->log1("GL_GREMEDY_string_marker not found");
+    }
+    if (supportExtension("GL_EXT_debug_marker"))
+    {
+        logger->log1("found GL_EXT_debug_marker");
+        assignFunctionEXT(glInsertEventMarker);
+        assignFunctionEXT(glPushGroupMarker);
+        assignFunctionEXT(glPopGroupMarker);
+    }
+    else
+    {
+        logger->log1("GL_EXT_debug_marker not found");
+    }
+    if (is15 && (is30 || supportExtension("GL_EXT_timer_query")))
+    {
+        logger->log1("found GL_EXT_timer_query");
+        assignFunction(glGenQueries);
+        assignFunction(glBeginQuery);
+        assignFunction(glEndQuery);
+        assignFunction(glDeleteQueries);
+        assignFunction(glGetQueryObjectiv);
+        assignFunctionEXT(glGetQueryObjectui64v);
+    }
+    else
+    {
+        logger->log1("GL_EXT_timer_query not supported");
+    }
+    if (is20 && (is43 || supportExtension("GL_ARB_invalidate_subdata")))
+    {
+        logger->log1("found GL_ARB_invalidate_subdata");
+        assignFunction(glInvalidateTexImage);
+    }
+    else
+    {
+        logger->log1("GL_ARB_invalidate_subdata not supported");
+    }
+    if (is21 && (is30 || supportExtension("GL_ARB_vertex_array_object")))
+    {
+        logger->log1("found GL_ARB_vertex_array_object");
+        assignFunction(glGenVertexArrays);
+        assignFunction(glBindVertexArray);
+        assignFunction(glDeleteVertexArrays);
+        assignFunction(glVertexAttribPointer);
+        assignFunction(glEnableVertexAttribArray);
+        assignFunction(glDisableVertexAttribArray);
+        assignFunction(glVertexAttribIPointer);
+    }
+    else
+    {
+        mSupportModernOpengl = false;
+        logger->log1("GL_ARB_vertex_array_object not found");
+    }
+    if (is20 || supportExtension("GL_ARB_vertex_buffer_object"))
+    {
+        assignFunction(glGenBuffers);
+        assignFunction(glDeleteBuffers);
+        assignFunction(glBindBuffer);
+        assignFunction(glBufferData);
+        assignFunction(glIsBuffer);
+    }
+    else
+    {
+        mSupportModernOpengl = false;
+        logger->log1("buffers extension not found");
+    }
+    if (is43 || supportExtension("GL_ARB_copy_image"))
+    {
+        assignFunction(glCopyImageSubData);
+    }
+    else
+    {
+        logger->log1("GL_ARB_copy_image not found");
+    }
+    if (is20 || supportExtension("GL_ARB_shader_objects"))
+    {
+        assignFunction(glCreateShader);
+        assignFunction(glDeleteShader);
+        assignFunction(glGetShaderiv);
+        assignFunction(glGetShaderInfoLog);
+        assignFunction(glGetShaderSource);
+        assignFunction(glShaderSource);
+        assignFunction(glCompileShader);
+        assignFunction(glLinkProgram);
+        assignFunction(glGetProgramInfoLog);
+        assignFunction(glDeleteProgram);
+        assignFunction(glCreateProgram);
+        assignFunction(glAttachShader);
+        assignFunction(glDetachShader);
+        assignFunction(glGetAttachedShaders);
+        assignFunction(glGetUniformLocation);
+        assignFunction(glGetActiveUniform);
+        assignFunction(glGetProgramiv);
+        assignFunction(glUseProgram);
+        assignFunction(glValidateProgram);
+        assignFunction(glGetAttribLocation);
+        assignFunction(glUniform1f);
+        assignFunction(glUniform2f);
+        assignFunction(glUniform3f);
+        assignFunction(glUniform4f);
+
+        if (is30 || supportExtension("GL_EXT_gpu_shader4"))
+        {
+            assignFunction(glBindFragDataLocation);
+        }
+        else
+        {
+            mSupportModernOpengl = false;
+            logger->log1("GL_EXT_gpu_shader4 not supported");
+        }
+        if (is41 || supportExtension("GL_ARB_separate_shader_objects"))
+        {
+            logger->log1("found GL_ARB_separate_shader_objects");
+            assignFunction(glProgramUniform1f);
+            assignFunction(glProgramUniform2f);
+            assignFunction(glProgramUniform3f);
+            assignFunction(glProgramUniform4f);
+        }
+        else
+        {
+            logger->log1("GL_ARB_separate_shader_objects not supported");
+        }
+        if (is43 || supportExtension("GL_ARB_vertex_attrib_binding"))
+        {
+            logger->log1("found GL_ARB_vertex_attrib_binding");
+            assignFunction(glBindVertexBuffer);
+            assignFunction(glVertexAttribBinding);
+            assignFunction(glVertexAttribFormat);
+            assignFunction(glVertexAttribIFormat);
+        }
+        else
+        {
+            mSupportModernOpengl = false;
+            logger->log1("GL_ARB_vertex_attrib_binding not supported");
+        }
+        if (is44 || supportExtension("GL_ARB_multi_bind"))
+        {
+            logger->log1("found GL_ARB_multi_bind");
+            assignFunction(glBindVertexBuffers);
+        }
+        else
+        {
+            logger->log1("GL_ARB_multi_bind not supported");
+        }
+    }
+    else
+    {
+        mSupportModernOpengl = false;
+        logger->log1("shaders not supported");
+    }
+
 #ifdef WIN32
-    assignFunction(wglGetExtensionsString, "wglGetExtensionsStringARB");
+    assignFunctionARB(wglGetExtensionsString);
 #endif
 }
 
@@ -885,6 +1218,7 @@ GLenum GraphicsManager::getLastError()
     while (tmp != GL_NO_ERROR)
     {
         error = tmp;
+        mLastError = tmp;
         tmp = glGetError();
     }
     return error;
@@ -923,6 +1257,13 @@ std::string GraphicsManager::errorToString(const GLenum error)
     return "";
 }
 
+void GraphicsManager::logError()
+{
+    const GLenum error = GraphicsManager::getLastError();
+    if (error != GL_NO_ERROR)
+        logger->log(errorToString(error));
+}
+
 void GraphicsManager::detectVideoSettings()
 {
     config.setValue("videodetected", true);
@@ -932,7 +1273,7 @@ void GraphicsManager::detectVideoSettings()
     {
         const Configuration &conf = test->getConfig();
         int val = conf.getValueInt("opengl", -1);
-        if (val >= 0 && val <= 2)
+        if (val >= 0 && val < static_cast<int>(RENDER_LAST))
         {
             config.setValue("opengl", val);
             val = conf.getValue("useTextureSampler", -1);
@@ -942,6 +1283,9 @@ void GraphicsManager::detectVideoSettings()
             if (val != -1)
                 config.setValue("compresstextures", val);
         }
+        config.setValue("textureSize", conf.getValue("textureSize",
+            "1024,1024,1024,1024,1024,1024"));
+        config.setValue("testInfo", conf.getValue("testInfo", ""));
         delete test;
     }
 }
@@ -998,6 +1342,12 @@ static CALLBACK void debugCallback(GLenum source, GLenum type, GLuint id,
         case GL_DEBUG_TYPE_MARKER:
             message.append(" MARKER");
             break;
+        case GL_DEBUG_TYPE_PUSH_GROUP:
+            message.append(" PUSH GROUP");
+            break;
+        case GL_DEBUG_TYPE_POP_GROUP:
+            message.append(" POP GROUP");
+            break;
         default:
             message.append(" ?").append(toString(type));
             break;
@@ -1021,7 +1371,7 @@ static CALLBACK void debugCallback(GLenum source, GLenum type, GLuint id,
             message.append(" ?").append(toString(type));
             break;
     }
-    char *const buf = new char[length + 1];
+    char *const buf = new char[static_cast<size_t>(length + 1)];
     memcpy(buf, text, length);
     buf[length] = 0;
     message.append(" ").append(buf);
@@ -1101,8 +1451,8 @@ void GraphicsManager::detectPixelSize()
         &mWidthMM, &mHeightMM, &mDensity);
 #endif
 #endif
-    logger->log("screen size in pixels: %dx%d", mMaxWidth, mMaxHeight);
-    logger->log("screen size in millimeters: %dx%d", mWidthMM, mHeightMM);
+    logger->log("screen size in pixels: %ux%u", mMaxWidth, mMaxHeight);
+    logger->log("screen size in millimeters: %ux%u", mWidthMM, mHeightMM);
     logger->log("actual screen density: " + getDensityString());
     const int density = config.getIntValue("screenDensity");
     if (density > 0 && density <= densitySize)
