@@ -24,15 +24,15 @@
 #include "being/being.h"
 #include "being/playerrelations.h"
 
-#include "gui/sdlfont.h"
+#include "gui/gui.h"
+
+#include "gui/fonts/font.h"
 
 #include "gui/widgets/label.h"
 
 #include "utils/gettext.h"
 #include "utils/stringutils.h"
 
-#include <guichan/font.hpp>
-
 #include "debug.h"
 
 BeingPopup::BeingPopup() :
@@ -95,7 +95,7 @@ void BeingPopup::show(const int x, const int y, Being *const b)
 
     b->updateComment();
 
-    if (b->getType() == Being::NPC && b->getComment().empty())
+    if (b->getType() == ActorType::NPC && b->getComment().empty())
     {
         setVisible(false);
         return;
