@@ -27,7 +27,9 @@
 #include "item.h"
 #include "units.h"
 
-#include "gui/sdlfont.h"
+#include "gui/gui.h"
+
+#include "gui/fonts/font.h"
 
 #include "gui/widgets/icon.h"
 #include "gui/widgets/label.h"
@@ -36,10 +38,9 @@
 #include "utils/gettext.h"
 
 #include "resources/image.h"
+#include "resources/iteminfo.h"
 #include "resources/resourcemanager.h"
 
-#include <guichan/font.hpp>
-
 #include "debug.h"
 
 extern int serverVersion;
@@ -50,7 +51,7 @@ ItemPopup::ItemPopup() :
     mItemDesc(new TextBox(this)),
     mItemEffect(new TextBox(this)),
     mItemWeight(new TextBox(this)),
-    mItemType(ITEM_UNUSABLE),
+    mItemType(ItemType::UNUSABLE),
     mIcon(new Icon(this, nullptr)),
     mLastName(),
     mLastId(0),
@@ -116,13 +117,17 @@ void ItemPopup::setItem(const Item *const item, const bool showImage)
         mLastId = item->getId();
         if (serverVersion > 0)
         {
-            mItemName->setCaption(strprintf("%s (+%d), %d", ii.getName(
-                item->getColor()).c_str(), item->getRefine(), ii.getId()));
+            mItemName->setCaption(strprintf("%s (+%u), %d",
+                ii.getName(item->getColor()).c_str(),
+                static_cast<unsigned int>(item->getRefine()),
+                ii.getId()));
         }
         else
         {
-            mItemName->setCaption(strprintf("%s (+%d), %d",
-                ii.getName().c_str(), item->getRefine(), ii.getId()));
+            mItemName->setCaption(strprintf("%s (+%u), %d",
+                ii.getName().c_str(),
+                static_cast<unsigned int>(item->getRefine()),
+                ii.getId()));
         }
         mItemName->adjustSize();
         const unsigned minWidth = mItemName->getWidth() + 8;
@@ -235,26 +240,26 @@ void ItemPopup::setItem(const ItemInfo &item, const unsigned char color,
         return label->setForegroundColorAll(getThemeColor(name2), \
         getThemeColor(name2##_OUTLINE)); \
     }
-void ItemPopup::setLabelColor(Label *label, const ItemType type) const
+void ItemPopup::setLabelColor(Label *label, const ItemType::Type type) const
 {
     switch (type)
     {
-        caseSetColor(ITEM_UNUSABLE, Theme::GENERIC)
-        caseSetColor(ITEM_USABLE, Theme::USABLE)
-        caseSetColor(ITEM_EQUIPMENT_ONE_HAND_WEAPON, Theme::ONEHAND)
-        caseSetColor(ITEM_EQUIPMENT_TWO_HANDS_WEAPON, Theme::TWOHAND)
-        caseSetColor(ITEM_EQUIPMENT_TORSO, Theme::TORSO)
-        caseSetColor(ITEM_EQUIPMENT_ARMS, Theme::ARMS)
-        caseSetColor(ITEM_EQUIPMENT_HEAD, Theme::HEAD)
-        caseSetColor(ITEM_EQUIPMENT_LEGS, Theme::LEGS)
-        caseSetColor(ITEM_EQUIPMENT_SHIELD, Theme::SHIELD)
-        caseSetColor(ITEM_EQUIPMENT_RING, Theme::RING)
-        caseSetColor(ITEM_EQUIPMENT_NECKLACE, Theme::NECKLACE)
-        caseSetColor(ITEM_EQUIPMENT_FEET, Theme::FEET)
-        caseSetColor(ITEM_EQUIPMENT_AMMO, Theme::AMMO)
-        caseSetColor(ITEM_EQUIPMENT_CHARM, Theme::CHARM)
-        caseSetColor(ITEM_SPRITE_RACE, Theme::UNKNOWN_ITEM)
-        caseSetColor(ITEM_SPRITE_HAIR, Theme::UNKNOWN_ITEM)
+        caseSetColor(ItemType::UNUSABLE, Theme::GENERIC)
+        caseSetColor(ItemType::USABLE, Theme::USABLE)
+        caseSetColor(ItemType::EQUIPMENT_ONE_HAND_WEAPON, Theme::ONEHAND)
+        caseSetColor(ItemType::EQUIPMENT_TWO_HANDS_WEAPON, Theme::TWOHAND)
+        caseSetColor(ItemType::EQUIPMENT_TORSO, Theme::TORSO)
+        caseSetColor(ItemType::EQUIPMENT_ARMS, Theme::ARMS)
+        caseSetColor(ItemType::EQUIPMENT_HEAD, Theme::HEAD)
+        caseSetColor(ItemType::EQUIPMENT_LEGS, Theme::LEGS)
+        caseSetColor(ItemType::EQUIPMENT_SHIELD, Theme::SHIELD)
+        caseSetColor(ItemType::EQUIPMENT_RING, Theme::RING)
+        caseSetColor(ItemType::EQUIPMENT_NECKLACE, Theme::NECKLACE)
+        caseSetColor(ItemType::EQUIPMENT_FEET, Theme::FEET)
+        caseSetColor(ItemType::EQUIPMENT_AMMO, Theme::AMMO)
+        caseSetColor(ItemType::EQUIPMENT_CHARM, Theme::CHARM)
+        caseSetColor(ItemType::SPRITE_RACE, Theme::UNKNOWN_ITEM)
+        caseSetColor(ItemType::SPRITE_HAIR, Theme::UNKNOWN_ITEM)
         default:
         {
             return label->setForegroundColorAll(getThemeColor(
@@ -265,7 +270,7 @@ void ItemPopup::setLabelColor(Label *label, const ItemType type) const
 }
 #undef caseSetColor
 
-void ItemPopup::mouseMoved(gcn::MouseEvent &event)
+void ItemPopup::mouseMoved(MouseEvent &event)
 {
     Popup::mouseMoved(event);
 
