@@ -26,9 +26,11 @@
 
 #include "gui/widgets/popup.h"
 
-#include "resources/iteminfo.h"
+#include "resources/itemtype.h"
 
 class Icon;
+class Item;
+class ItemInfo;
 class Label;
 class TextBox;
 
@@ -60,20 +62,20 @@ class ItemPopup final : public Popup
 
         void setItem(const Item *const item, const bool showImage = false);
 
-        void mouseMoved(gcn::MouseEvent &mouseEvent) override final;
+        void mouseMoved(MouseEvent &event) override final;
 
     private:
         Label *mItemName;
         TextBox *mItemDesc;
         TextBox *mItemEffect;
         TextBox *mItemWeight;
-        ItemType mItemType;
+        ItemType::Type mItemType;
         Icon *mIcon;
         std::string mLastName;
         int mLastId;
         unsigned char mLastColor;
 
-        void setLabelColor(Label *label, const ItemType type) const;
+        void setLabelColor(Label *label, const ItemType::Type type) const;
 };
 
 #endif  // GUI_POPUPS_ITEMPOPUP_H
