@@ -26,19 +26,19 @@
 #include "gui/widgets/linkhandler.h"
 #include "gui/widgets/popup.h"
 
-#include "being/actorsprite.h"
-
-#include <guichan/actionlistener.hpp>
-#include <guichan/listmodel.hpp>
+#include "listeners/playerlistener.h"
+#include "listeners/renamelistener.h"
 
 #include "localconsts.h"
 
+class ActorSprite;
 class Being;
 class BrowserBox;
 class Button;
 class ChatTab;
 class FloorItem;
 class Item;
+class ListModel;
 class MapItem;
 class ScrollArea;
 class TextCommand;
@@ -47,50 +47,6 @@ class TextField;
 class ProgressBar;
 class Window;
 
-class RenameListener final : public gcn::ActionListener
-{
-    public:
-        RenameListener();
-
-        A_DELETE_COPY(RenameListener)
-
-        void action(const gcn::ActionEvent &event) override final;
-
-        void setMapItem(MapItem *const mapItem);
-
-        void setDialog(TextDialog *dialog)
-        { mDialog = dialog; }
-
-    private:
-        int mMapItemX;
-        int mMapItemY;
-        TextDialog *mDialog;
-};
-
-class PlayerListener : public gcn::ActionListener
-{
-    public:
-        PlayerListener();
-
-        A_DELETE_COPY(PlayerListener)
-
-        void action(const gcn::ActionEvent &event) override final;
-
-        void setNick(std::string name)
-        { mNick = name; }
-
-        void setDialog(TextDialog *dialog)
-        { mDialog = dialog; }
-
-        void setType(int type)
-        { mType = type; }
-
-    private:
-        std::string mNick;
-        TextDialog *mDialog;
-        int mType;
-};
-
 /**
  * Window showing popup menu.
  */
@@ -115,7 +71,7 @@ class PopupMenu final : public Popup, public LinkHandler
          * Shows the beings related popup menu at the specified mouse coords.
          */
         void showPopup(const int x, const int y,
-                       std::vector<ActorSprite*> &beings);
+                       const std::vector<ActorSprite*> &beings);
 
         void showPlayerPopup(const int x, const int y,
                              const std::string &nick);
@@ -173,7 +129,7 @@ class PopupMenu final : public Popup, public LinkHandler
 
         void showChangePos(const int x, const int y);
 
-        void showPopup(const int x, const int y, gcn::ListModel *const model);
+        void showPopup(const int x, const int y, ListModel *const model);
 
         void showTextFieldPopup(int x, int y, TextField *const input);
 
@@ -183,11 +139,13 @@ class PopupMenu final : public Popup, public LinkHandler
 
         void showNpcDialogPopup(const int npcId, const int x, const int y);
 
+        void showWindowPopup(Window *const window, const int x, const int y);
+
         /**
          * Handles link action.
          */
         void handleLink(const std::string &link,
-                        gcn::MouseEvent *event A_UNUSED) override final;
+                        MouseEvent *event A_UNUSED) override final;
 
         void clear();
 
@@ -212,6 +170,8 @@ class PopupMenu final : public Popup, public LinkHandler
 
         void addGmCommands();
 
+        void addNormalRelations();
+
         void showGMPopup();
 
         BrowserBox *mBrowserBox;
