@@ -23,13 +23,15 @@
 
 #include "gui/popups/spellpopup.h"
 
-#include "gui/sdlfont.h"
+#include "textcommand.h"
+
+#include "gui/gui.h"
 
 #include "gui/widgets/label.h"
 
 #include "debug.h"
 
-SpellPopup::SpellPopup():
+SpellPopup::SpellPopup() :
     Popup("SpellPopup", "spellpopup.xml"),
     mItemName(new Label(this)),
     mItemComment(new Label(this))
@@ -88,7 +90,7 @@ void SpellPopup::view(const int x, const int y)
     int posX = std::max(0, x - getWidth() / 2);
     int posY = y + distance;
 
-    const gcn::Rectangle &rect = mDimension;
+    const Rect &rect = mDimension;
     const int w = rect.width;
     const int h = rect.height;
     if (posX + w > mainGraphics->mWidth)
@@ -109,7 +111,7 @@ void SpellPopup::view(const int x, const int y)
     requestMoveToTop();
 }
 
-void SpellPopup::mouseMoved(gcn::MouseEvent &event)
+void SpellPopup::mouseMoved(MouseEvent &event)
 {
     Popup::mouseMoved(event);
 
