@@ -26,9 +26,8 @@
 
 #include "gui/widgets/popup.h"
 
-#include "textcommand.h"
-
 class Label;
+class TextCommand;
 
 /**
  * A popup that displays information about an item.
@@ -60,7 +59,7 @@ class SpellPopup final : public Popup
          */
         void view(const int x, const int y);
 
-        void mouseMoved(gcn::MouseEvent &mouseEvent) override final;
+        void mouseMoved(MouseEvent &event) override final;
 
     private:
         Label *mItemName;
