@@ -21,8 +21,8 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef GUI_TEXTPOPUP_H
-#define GUI_TEXTPOPUP_H
+#ifndef GUI_POPUPS_TEXTPOPUP_H
+#define GUI_POPUPS_TEXTPOPUP_H
 
 #include "gui/widgets/popup.h"
 
@@ -72,10 +72,10 @@ class TextPopup final : public Popup
         void show(const int x, const int y, const std::string &str1,
                   const std::string &str2, const std::string &str3);
 
-        void mouseMoved(gcn::MouseEvent &mouseEvent) override final;
+        void mouseMoved(MouseEvent &event) override final;
 
     private:
         Label *mText[TEXTPOPUPCOUNT];
 };
 
-#endif  // GUI_TEXTPOPUP_H
+#endif  // GUI_POPUPS_TEXTPOPUP_H
