@@ -1,5 +1,5 @@
 /*
- *  Configurable text colors
+ *  The ManaPlus Client
  *  Copyright (C) 2008  Douglas Boffey <dougaboffey@netscape.net>
  *  Copyright (C) 2009  The Mana World Development Team
  *  Copyright (C) 2009-2010  The Mana Developers
@@ -29,6 +29,8 @@
 
 #include "debug.h"
 
+UserPalette *userPalette = nullptr;
+
 const std::string ColorTypeNames[] =
 {
     "ColorBeing",
@@ -97,8 +99,8 @@ std::string UserPalette::getConfigName(const std::string &typeName)
     return res;
 }
 
-UserPalette::UserPalette():
-        Palette(USER_COLOR_LAST)
+UserPalette::UserPalette() :
+    Palette(USER_COLOR_LAST)
 {
     mColors[BEING] = ColorElem();
     mColors[PC] = ColorElem();
@@ -234,10 +236,12 @@ UserPalette::~UserPalette()
     }
 }
 
-void UserPalette::setColor(const int type, const int r,
-                           const int g, const int b)
+void UserPalette::setColor(const int type,
+                           const int r,
+                           const int g,
+                           const int b)
 {
-    gcn::Color &color = mColors[type].color;
+    Color &color = mColors[type].color;
     color.r = r;
     color.g = g;
     color.b = b;
@@ -296,14 +300,14 @@ void UserPalette::rollback()
         if (i->grad != i->committedGrad)
             setGradient(i->type, i->committedGrad);
 
-        const gcn::Color &committedColor = i->committedColor;
+        const Color &committedColor = i->committedColor;
         setGradientDelay(i->type, i->committedDelay);
         setColor(i->type, committedColor.r,
                  committedColor.g, committedColor.b);
 
         if (i->grad == PULSE)
         {
-            gcn::Color &testColor = i->testColor;
+            Color &testColor = i->testColor;
             testColor.r = committedColor.r;
             testColor.g = committedColor.g;
             testColor.b = committedColor.b;
@@ -311,7 +315,7 @@ void UserPalette::rollback()
     }
 }
 
-int UserPalette::getColorTypeAt(int i)
+int UserPalette::getColorTypeAt(const int i)
 {
     if (i < 0 || i >= getNumberOfElements())
         return BEING;
@@ -319,7 +323,8 @@ int UserPalette::getColorTypeAt(int i)
     return mColors[i].type;
 }
 
-void UserPalette::addColor(const unsigned type, const unsigned rgb,
+void UserPalette::addColor(const unsigned type,
+                           const unsigned rgb,
                            Palette::GradientType grad,
                            const std::string &text,
                            int delay)
@@ -342,7 +347,7 @@ void UserPalette::addColor(const unsigned type, const unsigned rgb,
         rgbValue = atox(rgbString);
     else
         rgbValue = atoi(rgbString.c_str());
-    const gcn::Color &trueCol = gcn::Color(rgbValue);
+    const Color &trueCol = Color(rgbValue);
     grad = static_cast<GradientType>(config.getValue(configName + "Gradient",
                                      static_cast<int>(grad)));
     delay = config.getValueInt(configName + "Delay", delay);
