@@ -1,5 +1,5 @@
 /*
- *  Configurable text colors
+ *  The ManaPlus Client
  *  Copyright (C) 2008  Douglas Boffey <dougaboffey@netscape.net>
  *  Copyright (C) 2009  The Mana World Development Team
  *  Copyright (C) 2009-2010  The Mana Developers
@@ -26,12 +26,12 @@
 
 #include "gui/palette.h"
 
-#include <guichan/listmodel.hpp>
+#include "gui/models/listmodel.h"
 
 /**
  * Class controlling the game's color palette.
  */
-class UserPalette final : public Palette, public gcn::ListModel
+class UserPalette final : public Palette, public ListModel
 {
     public:
         /** List of all colors that are configurable. */
@@ -100,8 +100,8 @@ class UserPalette final : public Palette, public gcn::ListModel
          *
          * @return the requested committed color
          */
-        inline const gcn::Color &getCommittedColor(const int type)
-                                                   const A_WARN_UNUSED
+        inline const Color &getCommittedColor(const int type)
+                                              const A_WARN_UNUSED
         {
             return mColors[type].committedColor;
         }
@@ -113,8 +113,7 @@ class UserPalette final : public Palette, public gcn::ListModel
          *
          * @return the requested test color
          */
-        inline const gcn::Color &getTestColor(const int type)
-                                              const A_WARN_UNUSED
+        inline const Color &getTestColor(const int type) const A_WARN_UNUSED
         { return mColors[type].testColor; }
 
         /**
@@ -123,7 +122,7 @@ class UserPalette final : public Palette, public gcn::ListModel
          * @param type the color type requested
          * @param color the color that should be tested
          */
-        inline void setTestColor(const int type, const gcn::Color &color)
+        inline void setTestColor(const int type, const Color &color)
         { mColors[type].testColor = color; }
 
         /**
@@ -187,7 +186,7 @@ class UserPalette final : public Palette, public gcn::ListModel
          *
          * @return the color type of the color with the given index
          */
-        int getColorTypeAt(int i) A_WARN_UNUSED;
+        int getColorTypeAt(const int i) A_WARN_UNUSED;
 
     private:
         /**
