@@ -23,29 +23,20 @@
 #ifndef GUI_VIEWPORT_H
 #define GUI_VIEWPORT_H
 
-#include "configlistener.h"
 #include "position.h"
 
+#include "listeners/configlistener.h"
+
 #include "gui/widgets/windowcontainer.h"
 
-#include <guichan/mouselistener.hpp>
+#include "listeners/mouselistener.h"
 
 class ActorSprite;
-class Button;
 class Being;
-class BeingPopup;
-class ChatTab;
 class FloorItem;
 class Graphics;
-class Item;
 class Map;
 class MapItem;
-class PopupMenu;
-class ProgressBar;
-class TextCommand;
-class TextField;
-class TextPopup;
-class Window;
 
 /** Delay between two mouse calls when dragging mouse and move the player */
 const int walkingMouseDelay = 500;
@@ -59,7 +50,7 @@ const int walkingMouseDelay = 500;
  * coordinates.
  */
 class Viewport final : public WindowContainer,
-                       public gcn::MouseListener,
+                       public MouseListener,
                        public ConfigListener
 {
     public:
@@ -83,7 +74,7 @@ class Viewport final : public WindowContainer,
         /**
          * Draws the viewport.
          */
-        void draw(gcn::Graphics *graphics) override final;
+        void draw(Graphics *graphics) override final;
 
         /**
          * Implements player to keep following mouse.
@@ -95,104 +86,29 @@ class Viewport final : public WindowContainer,
          * debug with all images and grid, debug with out big images
          * and with out grid.
          */
-        void toggleDebugPath();
+        void toggleMapDrawType();
 
         void toggleCameraMode();
 
         /**
          * Handles mouse press on map.
          */
-        void mousePressed(gcn::MouseEvent &event) override final;
+        void mousePressed(MouseEvent &event) override final;
 
         /**
          * Handles mouse move on map
          */
-        void mouseDragged(gcn::MouseEvent &event) override final;
+        void mouseDragged(MouseEvent &event) override final;
 
         /**
          * Handles mouse button release on map.
          */
-        void mouseReleased(gcn::MouseEvent &event) override final;
+        void mouseReleased(MouseEvent &event) override final;
 
         /**
          * Handles mouse move on map.
          */
-        void mouseMoved(gcn::MouseEvent &event) override final;
-
-        /**
-         * Shows a popup for an item.
-         * TODO Find some way to get rid of Item here
-         */
-        void showPopup(Window *const parent, const int x, const int y,
-                       Item *const item, const bool isInventory = true);
-
-        /**
-         * Shows a popup for an item.
-         * TODO Find some way to get rid of Item here
-         */
-        void showPopup(Window *const parent, Item *const item,
-                       const bool isInventory = true);
-
-        void showPopup(const int x, const int y, Button *const button);
-
-        void showPopup(const int x, const int y, const ProgressBar *const bar);
-
-        void showPopup(MapItem *const item);
-
-        void showItemPopup(Item *const item);
-
-        void showItemPopup(const int itemId, const unsigned char color = 1);
-
-        void showDropPopup(Item *const item);
-
-        /**
-         * Shows a popup for being.
-         */
-        void showPopup(const int x, const int y, const Being *const being);
-
-        void showPopup(const Being *const being);
-
-        void showPlayerPopup(const std::string &nick);
-
-        void showOutfitsPopup(const int x, const int y);
-
-        void showOutfitsPopup();
-
-        void showSpellPopup(TextCommand *const cmd);
-
-        void showAttackMonsterPopup(const std::string &name, const int type);
-
-        void showPickupItemPopup(const std::string &name);
-
-        /**
-         * Shows the related popup menu when right click on the chat
-         * at the specified mouse coordinates.
-         */
-        void showChatPopup(const int x, const int y, ChatTab *const tab);
-
-        /**
-         * Shows the related popup menu when right click on the chat
-         */
-        void showChatPopup(ChatTab *const tab);
-
-        void showUndressPopup(const int x, const int y,
-                              const Being *const being, Item *const item);
-
-        void showMapPopup(const int x, const int y);
-
-        void showTextFieldPopup(TextField *const input);
-
-        void showLinkPopup(const std::string &link);
-
-        void showWindowsPopup();
-
-        void showNpcDialogPopup(const int npcId);
-
-        /**
-         * Closes the popup menu. Needed for when the player dies or switching
-         * maps.
-         */
-        void closePopupMenu();
+        void mouseMoved(MouseEvent &event) override final;
 
         /**
          * A relevant config option changed.
@@ -229,20 +145,6 @@ class Viewport final : public WindowContainer,
         void scrollBy(const int x, const int y)
         { mPixelViewX += x; mPixelViewY += y; }
 
-        int getDebugPath() const A_WARN_UNUSED
-        { return mShowDebugPath; }
-
-        void setDebugPath(const int n)
-        { mShowDebugPath = n; }
-
-        int getCameraMode() const A_WARN_UNUSED
-        { return mCameraMode; }
-
-        /**
-         * Hides the BeingPopup.
-         */
-        void hideBeingPopup();
-
         /**
          * Clear all hover item\being etc
          */
@@ -265,9 +167,8 @@ class Viewport final : public WindowContainer,
         void setCameraRelativeY(const int n)
         { mCameraRelativeY = n; }
 
-        bool isPopupMenuVisible() const A_WARN_UNUSED;
-
-        void moveCameraToActor(const int actorId, const int x = 0,
+        void moveCameraToActor(const int actorId,
+                               const int x = 0,
                                const int y = 0);
 
         void moveCameraToPosition(const int x, const int y);
@@ -276,64 +177,68 @@ class Viewport final : public WindowContainer,
 
         void returnCamera();
 
-        void clearPopup();
-
     protected:
         friend class ActorManager;
 
         /// Clears any matching hovers
         void clearHover(const ActorSprite *const actor);
 
-        void validateSpeed() const;
+        static void validateSpeed();
 
     private:
         /**
          * Finds a path from the player to the mouse, and draws it. This is for
          * debug purposes.
          */
-        void _drawDebugPath(Graphics *const graphics);
+        void drawDebugPath(Graphics *const graphics);
 
         /**
          * Draws the given path.
          */
-        void _drawPath(Graphics *const graphics, const Path &path,
-                       const gcn::Color &color = gcn::Color(255, 0, 0)) const;
+        void drawPath(Graphics *const graphics,
+                      const Path &path,
+                      const Color &color = Color(255, 0, 0)) const;
+
+        bool leftMouseAction();
+
+        bool openContextMenu(const MouseEvent &event);
+
+        void walkByMouse(const MouseEvent &event);
 
         /**
          * Make the player go to the mouse position.
          */
-        void _followMouse();
+        void followMouse();
 
         Map *mMap;                   /**< The current map. */
 
-        int mScrollRadius;
-        int mScrollLaziness;
-        bool mShowBeingPopup;
-        bool mSelfMouseHeal;
-        bool mEnableLazyScrolling;
-        int mScrollCenterOffsetX;
-        int mScrollCenterOffsetY;
-        bool mMouseDirectionMove;
-        int mMouseX;                /**< Current mouse position in pixels. */
-        int mMouseY;                /**< Current mouse position in pixels. */
-        int mPixelViewX;            /**< Current viewpoint in pixels. */
-        int mPixelViewY;            /**< Current viewpoint in pixels. */
-        int mShowDebugPath;         /**< Show a path from player to pointer. */
-        int mCameraMode;            /**< Camera mode. */
-
-        bool mPlayerFollowMouse;
-
-        int mLocalWalkTime; /**< Timestamp before the next walk can be sent. */
-
-        PopupMenu *mPopupMenu;       /**< Popup menu. */
         Being *mHoverBeing;          /**< Being mouse is currently over. */
         FloorItem *mHoverItem;       /**< FloorItem mouse is currently over. */
         MapItem *mHoverSign;         /**< Map sign mouse is currently over. */
-        BeingPopup *mBeingPopup;     /**< Being information popup. */
-        TextPopup *mTextPopup;       /**< Map Item information popup. */
+
+        int mScrollRadius;
+        int mScrollLaziness;
+        int mScrollCenterOffsetX;
+        int mScrollCenterOffsetY;
+        int mMouseX;                /**< Current mouse position in pixels. */
+        int mMouseY;                /**< Current mouse position in pixels. */
+        int mMousePressX;
+        int mMousePressY;
+        int mPixelViewX;            /**< Current viewpoint in pixels. */
+        int mPixelViewY;            /**< Current viewpoint in pixels. */
+
+        int mLocalWalkTime; /**< Timestamp before the next walk can be sent. */
 
         int mCameraRelativeX;
         int mCameraRelativeY;
+
+        bool mShowBeingPopup;
+        bool mSelfMouseHeal;
+        bool mEnableLazyScrolling;
+        bool mMouseDirectionMove;
+        bool mLongMouseClick;
+        bool mMouseClicked;
+        bool mPlayerFollowMouse;
 };
 
 extern Viewport *viewport;           /**< The viewport. */
