@@ -24,19 +24,24 @@
 #include "actormanager.h"
 #include "configuration.h"
 #include "graphicsvertexes.h"
-#include "maplayer.h"
 
 #include "being/localplayer.h"
 
 #include "gui/gui.h"
-#include "gui/sdlfont.h"
+#include "gui/popupmanager.h"
+#include "gui/skin.h"
 #include "gui/viewport.h"
 
+#include "gui/fonts/font.h"
+
+#include "gui/models/avatarlistmodel.h"
+
 #include "gui/windows/chatwindow.h"
 
 #include "resources/image.h"
+#include "resources/mapitemtype.h"
 
-#include <guichan/font.hpp>
+#include "resources/map/map.h"
 
 #include "debug.h"
 
@@ -90,25 +95,26 @@ AvatarListBox::~AvatarListBox()
     }
 }
 
-void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
+void AvatarListBox::draw(Graphics *graphics)
 {
     BLOCK_START("AvatarListBox::draw")
-    if (!mListModel || !player_node)
+    if (!mListModel || !localPlayer)
     {
         BLOCK_END("AvatarListBox::draw")
         return;
     }
 
+    const Widget *const parent = mParent;
+    if (!parent)
+        return;
+
     AvatarListModel *const model = static_cast<AvatarListModel *const>(
         mListModel);
     updateAlpha();
-    Graphics *const graphics = static_cast<Graphics *const>(gcnGraphics);
-
-    gcn::Font *const font = getFont();
 
+    Font *const font = getFont();
     const int fontHeight = getFont()->getHeight();
-    const gcn::Widget *const parent = mParent;
-    const std::string name = player_node->getName();
+    const std::string &name = localPlayer->getName();
 
     // Draw the list elements
     graphics->setColorAll(mForegroundColor, mForegroundColor2);
@@ -123,7 +129,7 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
         if (!a)
             continue;
 
-        if (a->getType() != MapItem::SEPARATOR)
+        if (a->getType() != MapItemType::SEPARATOR)
         {
             // Draw online status
             const Image *const icon = a->getOnline()
@@ -137,10 +143,27 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
                 }
                 else
                 {
-                    graphics->drawImage2(icon, mImagePadding, y + mPadding);
+                    graphics->drawImage(icon, mImagePadding, y + mPadding);
                 }
             }
         }
+    }
+
+    if (useCaching)
+    {
+        graphics->finalize(&vertexes);
+        graphics->drawTileCollection(&vertexes);
+    }
+
+    graphics->setColorAll(mForegroundColor, mForegroundColor2);
+
+    for (int i = 0, y = 0;
+         i < model->getNumberOfElements();
+         ++i, y += fontHeight)
+    {
+        const Avatar *const a = model->getAvatarAt(i);
+        if (!a)
+            continue;
 
         std::string text;
 
@@ -156,15 +179,17 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
                 text = strprintf("%s %d/%d", a->getComplexName().c_str(),
                                  a->getHp(), a->getMaxHp());
             }
-            if (parent && a->getMaxHp())
+            const bool isPoison = a->getPoison();
+            if (a->getMaxHp())
             {
-                gcn::Color color = Theme::getProgressColor(
-                        Theme::PROG_HP, static_cast<float>(a->getHp())
-                        / static_cast<float>(a->getMaxHp()));
+                const int themeColor = (isPoison
+                    ? Theme::PROG_HP_POISON : Theme::PROG_HP);
+                Color color = Theme::getProgressColor(
+                    themeColor, static_cast<float>(a->getHp())
+                    / static_cast<float>(a->getMaxHp()));
                 color.a = 80;
                 graphics->setColor(color);
-
-                graphics->fillRectangle(gcn::Rectangle(mPadding, y + mPadding,
+                graphics->fillRectangle(Rect(mPadding, y + mPadding,
                     parent->getWidth() * a->getHp() / a->getMaxHp()
                     - 2 * mPadding, fontHeight));
             }
@@ -182,28 +207,25 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
                                  a->getDamageHp());
             }
 
-            if (parent)
+            const int themeColor = (a->getPoison()
+                ? Theme::PROG_HP_POISON : Theme::PROG_HP);
+            Color color = Theme::getProgressColor(themeColor, 1);
+            color.a = 80;
+            graphics->setColor(color);
+            graphics->fillRectangle(Rect(mPadding, y + mPadding,
+                parent->getWidth() * a->getDamageHp() / 1024
+                - 2 * mPadding, fontHeight));
+
+            if (a->getLevel() > 1)
             {
-                gcn::Color color = Theme::getProgressColor(Theme::PROG_HP,
-                        1);
+                graphics->setColor(mForegroundColor);
+                int minHp = 40 + ((a->getLevel() - 1) * 5);
+                if (minHp < 0)
+                    minHp = 40;
 
-                color.a = 80;
-                graphics->setColor(color);
-                graphics->fillRectangle(gcn::Rectangle(mPadding, y + mPadding,
-                    parent->getWidth() * a->getDamageHp() / 1024
-                    - 2 * mPadding, fontHeight));
-
-                if (a->getLevel() > 1)
-                {
-                    graphics->setColor(mForegroundColor);
-                    int minHp = 40 + ((a->getLevel() - 1) * 5);
-                    if (minHp < 0)
-                        minHp = 40;
-
-                    graphics->drawLine(parent->getWidth()*minHp / 1024
-                        + mPadding, y + mPadding,
-                        parent->getWidth() * minHp / 1024, y + fontHeight);
-                }
+                graphics->drawLine(parent->getWidth()*minHp / 1024
+                    + mPadding, y + mPadding,
+                    parent->getWidth() * minHp / 1024, y + fontHeight);
             }
         }
         else
@@ -238,13 +260,15 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
             {
                 switch (a->getGender())
                 {
-                    case GENDER_FEMALE:
+                    case Gender::FEMALE:
                         text.append(" \u2640 ");
                         break;
-                    case GENDER_MALE:
+                    case Gender::MALE:
                         text.append(" \u2642 ");
                         break;
                     default:
+                    case Gender::UNSPECIFIED:
+                    case Gender::OTHER:
                         break;
                 }
             }
@@ -255,15 +279,17 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
             {
                 switch (a->getGender())
                 {
-                    case GENDER_FEMALE:
+                    case Gender::FEMALE:
                         text.append(strprintf(" \u2640 %s",
                             a->getAdditionString().c_str()));
                         break;
-                    case GENDER_MALE:
+                    case Gender::MALE:
                         text.append(strprintf(" \u2642 %s",
                             a->getAdditionString().c_str()));
                         break;
                     default:
+                    case Gender::UNSPECIFIED:
+                    case Gender::OTHER:
                         break;
                 }
             }
@@ -273,12 +299,12 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
             }
         }
 
-        graphics->setColorAll(mForegroundColor, mForegroundColor2);
+        graphics->setColor(mForegroundColor);
 
         // Draw Name
         if (a->getDisplayBold())
         {
-            if (a->getType() == MapItem::SEPARATOR)
+            if (a->getType() == MapItemType::SEPARATOR)
             {
                 boldFont->drawString(graphics, text,
                     mImagePadding + mPadding, y + mPadding);
@@ -291,7 +317,7 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
         }
         else
         {
-            if (a->getType() == MapItem::SEPARATOR)
+            if (a->getType() == MapItemType::SEPARATOR)
             {
                 font->drawString(graphics, text, mImagePadding + mPadding,
                     y + mPadding);
@@ -304,17 +330,13 @@ void AvatarListBox::draw(gcn::Graphics *gcnGraphics)
         }
     }
 
-    if (useCaching)
-        graphics->drawTileCollection(&vertexes);
-
-    if (parent)
-        setWidth(parent->getWidth() - 10);
+    setWidth(parent->getWidth() - 10);
     BLOCK_END("AvatarListBox::draw")
 }
 
-void AvatarListBox::mousePressed(gcn::MouseEvent &event)
+void AvatarListBox::mousePressed(MouseEvent &event)
 {
-    if (!actorManager || !player_node || !viewport
+    if (!actorManager || !localPlayer || !popupManager
         || !getFont()->getHeight())
     {
         return;
@@ -335,43 +357,44 @@ void AvatarListBox::mousePressed(gcn::MouseEvent &event)
     if (!ava)
         return;
 
+    event.consume();
     const unsigned int eventButton = event.getButton();
-    if (eventButton == gcn::MouseEvent::LEFT)
+    if (eventButton == MouseButton::LEFT)
     {
         if (ava->getType() == AVATAR_PLAYER)
         {
             const Being *const being = actorManager->findBeingByName(
-                ava->getName(), Being::PLAYER);
+                ava->getName(), ActorType::PLAYER);
             if (being)
                 actorManager->heal(being);
         }
         else
         {
-            player_node->navigateTo(ava->getX(), ava->getY());
+            localPlayer->navigateTo(ava->getX(), ava->getY());
         }
     }
-    else if (eventButton == gcn::MouseEvent::RIGHT)
+    else if (eventButton == MouseButton::RIGHT)
     {
         switch (ava->getType())
         {
             // AVATAR_PLAYER
-            case MapItem::EMPTY:
+            case MapItemType::EMPTY:
             {
                 const Avatar *const avatar = model->getAvatarAt(selected);
                 if (avatar)
                 {
-                    const Being *const being = actorManager
-                        ->findBeingByName(avatar->getName(), Being::PLAYER);
+                    const Being *const being = actorManager->findBeingByName(
+                        avatar->getName(), ActorType::PLAYER);
                     if (being)
-                        viewport->showPopup(being);
+                        popupManager->showPopup(being);
                     else
-                        viewport->showPlayerPopup(avatar->getName());
+                        popupManager->showPlayerPopup(avatar->getName());
                 }
                 break;
             }
-            case MapItem::ATTACK:
-            case MapItem::PRIORITY:
-            case MapItem::IGNORE_:
+            case MapItemType::ATTACK:
+            case MapItemType::PRIORITY:
+            case MapItemType::IGNORE_:
             {
                 std::string name;
                 if (model->getAvatarAt(selected)->getLevel() == 0)
@@ -379,12 +402,12 @@ void AvatarListBox::mousePressed(gcn::MouseEvent &event)
                 else
                     name = model->getAvatarAt(selected)->getName();
 
-                viewport->showAttackMonsterPopup(name,
+                popupManager->showAttackMonsterPopup(name,
                     model->getAvatarAt(selected)->getType());
                 break;
             }
-            case MapItem::PICKUP:
-            case MapItem::NOPICKUP:
+            case MapItemType::PICKUP:
+            case MapItemType::NOPICKUP:
             {
                 std::string name;
                 if (model->getAvatarAt(selected)->getLevel() == 0)
@@ -392,7 +415,7 @@ void AvatarListBox::mousePressed(gcn::MouseEvent &event)
                 else
                     name = model->getAvatarAt(selected)->getName();
 
-                viewport->showPickupItemPopup(name);
+                popupManager->showPickupItemPopup(name);
                 break;
             }
             default:
@@ -403,13 +426,13 @@ void AvatarListBox::mousePressed(gcn::MouseEvent &event)
                 {
                     MapItem *const mapItem = map->findPortalXY(
                         ava->getX(), ava->getY());
-                    viewport->showPopup(mapItem);
+                    popupManager->showPopup(mapItem);
                 }
                 break;
             }
         }
     }
-    else if (eventButton == gcn::MouseEvent::MIDDLE)
+    else if (eventButton == MouseButton::MIDDLE)
     {
         if (ava->getType() == AVATAR_PLAYER && chatWindow)
         {
@@ -421,7 +444,7 @@ void AvatarListBox::mousePressed(gcn::MouseEvent &event)
     }
 }
 
-void AvatarListBox::mouseReleased(gcn::MouseEvent &event A_UNUSED)
+void AvatarListBox::mouseReleased(MouseEvent &event A_UNUSED)
 {
 }
 
