@@ -22,53 +22,45 @@
 #ifndef GUI_WIDGETS_AVATARLISTBOX_H
 #define GUI_WIDGETS_AVATARLISTBOX_H
 
-#include "avatar.h"
-
-#include "configlistener.h"
+#include "listeners/configlistener.h"
 
 #include "gui/widgets/listbox.h"
 
 #include <string>
 
+class AvatarListModel;
 class Image;
 
-class AvatarListModel : public gcn::ListModel
+class AvatarListBox final : public ListBox,
+                            public ConfigListener
 {
-public:
-    virtual Avatar *getAvatarAt(const int i) A_WARN_UNUSED = 0;
+    public:
+        AvatarListBox(const Widget2 *const widget,
+                      AvatarListModel *const model);
 
-    std::string getElementAt(int i) override final A_WARN_UNUSED
-    { return getAvatarAt(i)->getName(); }
-};
+        A_DELETE_COPY(AvatarListBox)
 
-class AvatarListBox final : public ListBox, public ConfigListener
-{
-public:
-    AvatarListBox(const Widget2 *const widget, AvatarListModel *const model);
+        ~AvatarListBox();
 
-    A_DELETE_COPY(AvatarListBox)
+        /**
+         * Draws the list box.
+         */
+        void draw(Graphics *gcnGraphics) override final;
 
-    ~AvatarListBox();
+        void mousePressed(MouseEvent &event) override final;
 
-    /**
-     * Draws the list box.
-     */
-    void draw(gcn::Graphics *gcnGraphics) override final;
+        void mouseReleased(MouseEvent &event A_UNUSED) override final;
 
-    void mousePressed(gcn::MouseEvent &event) override final;
+        void optionChanged(const std::string &value) override final;
 
-    void mouseReleased(gcn::MouseEvent &event A_UNUSED) override final;
+    private:
+        int mImagePadding;
+        bool mShowGender;
+        bool mShowLevel;
 
-    void optionChanged(const std::string &value) override final;
-
-private:
-    int mImagePadding;
-    bool mShowGender;
-    bool mShowLevel;
-
-    static int instances;
-    static Image *onlineIcon;
-    static Image *offlineIcon;
+        static int instances;
+        static Image *onlineIcon;
+        static Image *offlineIcon;
 };
 
 #endif  // GUI_WIDGETS_AVATARLISTBOX_H
