@@ -26,7 +26,9 @@
 #include "input/inputmanager.h"
 
 #include "gui/gui.h"
-#include "gui/sdlfont.h"
+#include "gui/skin.h"
+
+#include "gui/fonts/font.h"
 
 #include "gui/widgets/linkhandler.h"
 
@@ -37,14 +39,31 @@
 #include "utils/stringutils.h"
 #include "utils/timer.h"
 
-#include <guichan/graphics.hpp>
-#include <guichan/font.hpp>
-#include <guichan/cliprectangle.hpp>
+#include "gui/cliprect.h"
+
+#include "render/graphics.h"
 
 #include <algorithm>
 
 #include "debug.h"
 
+namespace
+{
+    struct MouseOverLink final
+    {
+        MouseOverLink(const int x, const int y) :
+            mX(x), mY(y)
+        { }
+
+        bool operator() (const BrowserLink &link) const
+        {
+            return (mX >= link.x1 && mX < link.x2 &&
+                    mY >= link.y1 && mY < link.y2);
+        }
+        int mX, mY;
+    };
+}  // namespace
+
 ImageSet *BrowserBox::mEmotes = nullptr;
 int BrowserBox::mInstances = 0;
 
@@ -52,9 +71,8 @@ BrowserBox::BrowserBox(const Widget2 *const widget,
                        const unsigned int mode,
                        const bool opaque,
                        const std::string &skin) :
-    gcn::Widget(),
-    Widget2(widget),
-    gcn::MouseListener(),
+    Widget(widget),
+    MouseListener(),
     mTextRows(),
     mTextRowLinksCount(),
     mLineParts(),
@@ -84,12 +102,13 @@ BrowserBox::BrowserBox(const Widget2 *const widget,
     mEnableKeys(false),
     mEnableTabs(false)
 {
+    mAllowLogic = false;
+
     setFocusable(true);
     addMouseListener(this);
 
     mBackgroundColor = getThemeColor(Theme::BACKGROUND);
 
-    Theme *const theme = Theme::instance();
     if (theme)
         mSkin = theme->load(skin, "browserbox.xml");
     if (mInstances == 0)
@@ -141,7 +160,6 @@ BrowserBox::~BrowserBox()
     if (gui)
         gui->removeDragged(this);
 
-    Theme *const theme = Theme::instance();
     if (theme)
     {
         theme->unload(mSkin);
@@ -174,7 +192,7 @@ void BrowserBox::addRow(const std::string &row, const bool atTop)
     std::string tmp = row;
     std::string newRow;
     size_t idx1;
-    const gcn::Font *const font = getFont();
+    const Font *const font = getFont();
     int linksCount = 0;
 
     if (getWidth() < 0)
@@ -290,7 +308,7 @@ void BrowserBox::addRow(const std::string &row, const bool atTop)
     // discard older rows when a row limit has been set
     if (mMaxRows > 0 && !mTextRows.empty())
     {
-        while (mTextRows.size() > mMaxRows)
+        while (mTextRows.size() > static_cast<size_t>(mMaxRows))
         {
             mTextRows.pop_front();
             int cnt = mTextRowLinksCount.front();
@@ -408,21 +426,7 @@ void BrowserBox::clearRows()
     updateHeight();
 }
 
-struct MouseOverLink
-{
-    MouseOverLink(const int x, const int y) :
-        mX(x), mY(y)
-    { }
-
-    bool operator() (const BrowserLink &link) const
-    {
-        return (mX >= link.x1 && mX < link.x2 &&
-                mY >= link.y1 && mY < link.y2);
-    }
-    int mX, mY;
-};
-
-void BrowserBox::mousePressed(gcn::MouseEvent &event)
+void BrowserBox::mousePressed(MouseEvent &event)
 {
     if (!mLinkHandler)
         return;
@@ -437,7 +441,7 @@ void BrowserBox::mousePressed(gcn::MouseEvent &event)
     }
 }
 
-void BrowserBox::mouseMoved(gcn::MouseEvent &event)
+void BrowserBox::mouseMoved(MouseEvent &event)
 {
     const LinkIterator i = std::find_if(mLinks.begin(), mLinks.end(),
         MouseOverLink(event.getX(), event.getY()));
@@ -446,13 +450,19 @@ void BrowserBox::mouseMoved(gcn::MouseEvent &event)
         ? static_cast<int>(i - mLinks.begin()) : -1;
 }
 
-void BrowserBox::draw(gcn::Graphics *graphics)
+void BrowserBox::mouseExited(MouseEvent &event A_UNUSED)
+{
+    mSelectedLink = -1;
+}
+
+void BrowserBox::draw(Graphics *graphics)
 {
     BLOCK_START("BrowserBox::draw")
-    const gcn::ClipRectangle &cr = graphics->getCurrentClipArea();
-    Graphics *const graphics2 = static_cast<Graphics *const>(graphics);
-    mYStart = cr.y - cr.yOffset;
-    const int yEnd = mYStart + cr.height;
+    const ClipRect *const cr = graphics->getCurrentClipArea();
+    if (!cr)
+        return;
+    mYStart = cr->y - cr->yOffset;
+    const int yEnd = mYStart + cr->height;
     if (mYStart < 0)
         mYStart = 0;
 
@@ -462,7 +472,7 @@ void BrowserBox::draw(gcn::Graphics *graphics)
     if (mOpaque)
     {
         graphics->setColor(mBackgroundColor);
-        graphics->fillRectangle(gcn::Rectangle(0, 0, getWidth(), getHeight()));
+        graphics->fillRectangle(Rect(0, 0, getWidth(), getHeight()));
     }
 
     if (mSelectedLink >= 0 && mSelectedLink
@@ -471,7 +481,7 @@ void BrowserBox::draw(gcn::Graphics *graphics)
         if ((mHighMode & BACKGROUND))
         {
             graphics->setColor(mHighlightColor);
-            graphics->fillRectangle(gcn::Rectangle(
+            graphics->fillRectangle(Rect(
                 mLinks[mSelectedLink].x1,
                 mLinks[mSelectedLink].y1,
                 mLinks[mSelectedLink].x2 - mLinks[mSelectedLink].x1,
@@ -489,7 +499,7 @@ void BrowserBox::draw(gcn::Graphics *graphics)
         }
     }
 
-    gcn::Font *const font = getFont();
+    Font *const font = getFont();
 
     FOR_EACH (LinePartCIter, i, mLineParts)
     {
@@ -500,7 +510,7 @@ void BrowserBox::draw(gcn::Graphics *graphics)
             break;
         if (!part.mType)
         {
-            graphics2->setColorAll(part.mColor, part.mColor2);
+            graphics->setColorAll(part.mColor, part.mColor2);
             if (part.mBold)
                 boldFont->drawString(graphics, part.mText, part.mX, part.mY);
             else
@@ -508,7 +518,7 @@ void BrowserBox::draw(gcn::Graphics *graphics)
         }
         else if (part.mImage)
         {
-            graphics2->drawImage2(part.mImage, part.mX, part.mY);
+            graphics->drawImage(part.mImage, part.mX, part.mY);
         }
     }
 
@@ -528,14 +538,14 @@ int BrowserBox::calcHeight()
     if (maxWidth < 0)
         return 1;
 
-    const gcn::Font *const font = getFont();
+    const Font *const font = getFont();
     const int fontHeight = font->getHeight() + 2 * mItemPadding;
     const int fontWidthMinus = font->getWidth("-");
     const char *const hyphen = "~";
     const int hyphenWidth = font->getWidth(hyphen);
 
-    gcn::Color selColor[2] = {mForegroundColor, mForegroundColor2};
-    const gcn::Color textColor[2] = {mForegroundColor, mForegroundColor2};
+    Color selColor[2] = {mForegroundColor, mForegroundColor2};
+    const Color textColor[2] = {mForegroundColor, mForegroundColor2};
     ResourceManager *const resman = ResourceManager::getInstance();
 
     mLineParts.clear();
@@ -581,7 +591,7 @@ int BrowserBox::calcHeight()
             continue;
         }
 
-        gcn::Color prevColor[2];
+        Color prevColor[2];
         prevColor[0] = selColor[0];
         prevColor[1] = selColor[1];
         bold = false;
@@ -621,10 +631,11 @@ int BrowserBox::calcHeight()
                     const signed char c = row.at(start + 2);
 
                     bool valid(false);
-                    const gcn::Color col[2] =
+                    const Color col[2] =
                     {
                         getThemeCharColor(c, valid),
-                        getThemeCharColor(c | 0x80, valid)
+                        getThemeCharColor(static_cast<signed char>(
+                            c | 0x80), valid)
                     };
 
                     if (c == '>')
@@ -734,7 +745,7 @@ int BrowserBox::calcHeight()
                             if (mEmotes)
                             {
                                 const size_t sz = mEmotes->size();
-                                if (static_cast<unsigned>(cid) < sz)
+                                if (static_cast<size_t>(cid) < sz)
                                 {
                                     Image *const img = mEmotes->get(cid);
                                     if (img)
@@ -897,16 +908,9 @@ std::string BrowserBox::getTextAtPos(const int x, const int y) const
     return str;
 }
 
-void BrowserBox::setForegroundColorAll(const gcn::Color &color1,
-                                       const gcn::Color &color2)
+void BrowserBox::setForegroundColorAll(const Color &color1,
+                                       const Color &color2)
 {
     mForegroundColor = color1;
     mForegroundColor2 = color2;
 }
-
-LinePart::~LinePart()
-{
-    if (mImage)
-        mImage->decRef();
-    mImage = nullptr;
-}
