@@ -24,24 +24,30 @@
 
 #include "units.h"
 
+#include "being/attributes.h"
+
+#include "gui/gui.h"
+
 #include "gui/windows/charselectdialog.h"
 
 #include "gui/popups/textpopup.h"
 
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layouthelper.h"
 
 #include "utils/gettext.h"
+#include "utils/stringutils.h"
 
-#include <SDL_mouse.h>
+#include "net/character.h"
 
 #include "debug.h"
 
 CharacterDisplay::CharacterDisplay(const Widget2 *const widget,
                                    CharSelectDialog *const charSelectDialog) :
     Container(widget),
-    gcn::MouseListener(),
-    gcn::WidgetListener(),
+    MouseListener(),
+    WidgetListener(),
     mCharacter(nullptr),
     mPlayerBox(new PlayerBox(nullptr)),
     mName(new Label(this, "wwwwwwwwwwwwwwwwwwwwwwww")),
@@ -76,8 +82,7 @@ CharacterDisplay::CharacterDisplay(const Widget2 *const widget,
 
 CharacterDisplay::~CharacterDisplay()
 {
-    delete mPopup;
-    mPopup = nullptr;
+    delete2(mPopup);
 }
 
 void CharacterDisplay::setCharacter(Net::Character *const character)
@@ -110,28 +115,32 @@ void CharacterDisplay::update()
     distributeResizedEvent();
 }
 
-void CharacterDisplay::widgetHidden(const gcn::Event &event A_UNUSED)
+void CharacterDisplay::widgetHidden(const Event &event A_UNUSED)
 {
     mPopup->setVisible(false);
 }
 
-void CharacterDisplay::mouseExited(gcn::MouseEvent &event A_UNUSED)
+void CharacterDisplay::mouseExited(MouseEvent &event A_UNUSED)
 {
     mPopup->setVisible(false);
 }
 
-void CharacterDisplay::mouseMoved(gcn::MouseEvent &event A_UNUSED)
+void CharacterDisplay::mouseMoved(MouseEvent &event A_UNUSED)
 {
+    if (!gui)
+        return;
+
     int mouseX = 0;
     int mouseY = 0;
-    SDL_GetMouseState(&mouseX, &mouseY);
+    Gui::getMouseState(&mouseX, &mouseY);
     const std::string &name = mName->getCaption();
     if (!name.empty())
     {
         mPopup->show(mouseX, mouseY, name, strprintf(_("Level: %u"),
-            mCharacter->data.mAttributes[PlayerInfo::LEVEL]),
+            static_cast<uint32_t>(
+            mCharacter->data.mAttributes[Attributes::LEVEL])),
             strprintf(_("Money: %s"), Units::formatCurrency(
-            mCharacter->data.mAttributes[PlayerInfo::MONEY]).c_str()));
+            mCharacter->data.mAttributes[Attributes::MONEY]).c_str()));
     }
     else
     {
@@ -139,8 +148,9 @@ void CharacterDisplay::mouseMoved(gcn::MouseEvent &event A_UNUSED)
     }
 }
 
-void CharacterDisplay::mousePressed(gcn::MouseEvent &event)
+void CharacterDisplay::mousePressed(MouseEvent &event)
 {
+    event.consume();
     if (event.getClickCount() == 2)
         distributeActionEvent();
 }
