@@ -22,17 +22,23 @@
 
 #include "gui/widgets/dropdown.h"
 
-#include "client.h"
+#include "settings.h"
 
-#include "input/keydata.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
+
+#include "input/inputaction.h"
+
+#include "gui/gui.h"
+#include "gui/skin.h"
+
+#include "gui/fonts/font.h"
+
+#include "gui/models/extendedlistmodel.h"
 
-#include "gui/widgets/extendedlistmodel.h"
 #include "gui/widgets/popuplist.h"
 
 #include "resources/image.h"
-
-#include <guichan/font.hpp>
+#include "resources/imagerect.h"
 
 #include <algorithm>
 
@@ -51,18 +57,17 @@ static std::string const dropdownFiles[2] =
 };
 
 DropDown::DropDown(const Widget2 *const widget,
-                   gcn::ListModel *const listModel,
+                   ListModel *const listModel,
                    const bool extended,
                    const bool modal,
-                   gcn::ActionListener *const listener,
-                   const std::string &eventId):
-    gcn::ActionListener(),
-    gcn::BasicContainer(),
-    gcn::KeyListener(),
-    gcn::MouseListener(),
-    gcn::FocusListener(),
-    gcn::SelectionListener(),
-    Widget2(widget),
+                   ActionListener *const listener,
+                   const std::string &eventId) :
+    ActionListener(),
+    BasicContainer(widget),
+    KeyListener(),
+    MouseListener(),
+    FocusListener(),
+    SelectionListener(),
     mPopup(new PopupList(this, listModel, extended, modal)),
     mShadowColor(getThemeColor(Theme::DROPDOWN_SHADOW)),
     mHighlightColor(getThemeColor(Theme::HIGHLIGHT)),
@@ -76,6 +81,7 @@ DropDown::DropDown(const Widget2 *const widget,
     mPushed(false),
     mIsDragged(false)
 {
+    mAllowLogic = false;
     mPopup->postInit();
     mFrameSize = 2;
     mForegroundColor2 = getThemeColor(Theme::DROPDOWN_OUTLINE);
@@ -83,13 +89,12 @@ DropDown::DropDown(const Widget2 *const widget,
     mPopup->setHeight(100);
 
     // Initialize graphics
-    if (instances == 0)
+    if (instances == 0 && theme)
     {
         // Load the background skin
         for (int i = 0; i < 2; i ++)
         {
-            Skin *const skin = Theme::instance()->load(
-                dropdownFiles[i], "dropdown.xml");
+            Skin *const skin = theme->load(dropdownFiles[i], "dropdown.xml");
             if (skin)
             {
                 if (!i)
@@ -109,7 +114,7 @@ DropDown::DropDown(const Widget2 *const widget,
                     }
                 }
                 if (i)
-                    Theme::instance()->unload(skin);
+                    theme->unload(skin);
             }
             else
             {
@@ -119,11 +124,8 @@ DropDown::DropDown(const Widget2 *const widget,
         }
 
         // get the border skin
-        if (Theme::instance())
-        {
-            Theme::instance()->loadRect(skinRect,
-                "dropdown_background.xml", "");
-        }
+        if (theme)
+            theme->loadRect(skinRect, "dropdown_background.xml", "");
     }
 
     instances++;
@@ -179,19 +181,18 @@ DropDown::~DropDown()
                     buttons[f][i]->decRef();
             }
         }
-        Theme *const theme = Theme::instance();
         if (theme)
         {
             theme->unload(mSkin);
-            theme->unloadRect(skinRect);
+            Theme::unloadRect(skinRect);
         }
     }
 }
 
 void DropDown::updateAlpha()
 {
-    const float alpha = std::max(client->getGuiAlpha(),
-        Theme::instance()->getMinimumOpacity());
+    const float alpha = std::max(settings.guiAlpha,
+        theme->getMinimumOpacity());
 
     if (mAlpha != alpha)
     {
@@ -214,7 +215,7 @@ void DropDown::updateAlpha()
     }
 }
 
-void DropDown::draw(gcn::Graphics* graphics)
+void DropDown::draw(Graphics* graphics)
 {
     BLOCK_START("DropDown::draw")
     int h;
@@ -231,12 +232,11 @@ void DropDown::draw(gcn::Graphics* graphics)
     mHighlightColor.a = alpha;
     mShadowColor.a = alpha;
 
-    gcn::ListModel *const model = mPopup->getListModel();
+    ListModel *const model = mPopup->getListModel();
     if (model && mPopup->getSelected() >= 0)
     {
-        gcn::Font *const font = getFont();
-        static_cast<Graphics *const>(graphics)->setColorAll(
-            mForegroundColor, mForegroundColor2);
+        Font *const font = getFont();
+        graphics->setColorAll(mForegroundColor, mForegroundColor2);
         if (mExtended)
         {
             const int sel = mPopup->getSelected();
@@ -250,9 +250,9 @@ void DropDown::draw(gcn::Graphics* graphics)
             }
             else
             {
-                static_cast<Graphics*>(graphics)->drawImage2(
-                    image, mImagePadding, (mDimension.height
-                    - image->getHeight()) / 2 + mPadding);
+                graphics->drawImage(image,
+                    mImagePadding,
+                    (mDimension.height - image->getHeight()) / 2 + mPadding);
                 font->drawString(graphics, model->getElementAt(sel),
                     image->getWidth() + mImagePadding + mSpacing, mPadding);
             }
@@ -267,7 +267,7 @@ void DropDown::draw(gcn::Graphics* graphics)
     if (isFocused())
     {
         graphics->setColor(mHighlightColor);
-        graphics->drawRectangle(gcn::Rectangle(mPadding, mPadding,
+        graphics->drawRectangle(Rect(mPadding, mPadding,
             mDimension.width - h - pad, h - pad));
     }
 
@@ -286,55 +286,56 @@ void DropDown::draw(gcn::Graphics* graphics)
     BLOCK_END("DropDown::draw")
 }
 
-void DropDown::drawFrame(gcn::Graphics *graphics)
+void DropDown::drawFrame(Graphics *graphics)
 {
     BLOCK_START("DropDown::drawFrame")
     const int bs2 = getFrameSize();
-    const gcn::Rectangle &rect = mDimension;
-    static_cast<Graphics*>(graphics)->drawImageRect(
-        0, 0, rect.width + bs2, rect.height + bs2, skinRect);
+    const Rect &rect = mDimension;
+    graphics->drawImageRect(0, 0,
+        rect.width + bs2, rect.height + bs2,
+        skinRect);
     BLOCK_END("DropDown::drawFrame")
 }
 
-void DropDown::drawButton(gcn::Graphics *graphics)
+void DropDown::drawButton(Graphics *graphics)
 {
     const int height = mDroppedDown ? mFoldedUpHeight : mDimension.height;
 
     Image *image = buttons[mDroppedDown][mPushed];
     if (image)
     {
-        static_cast<Graphics*>(graphics)->drawImage2(image,
+        graphics->drawImage(image,
             mDimension.width - image->getWidth() - mImagePadding,
             (height - image->getHeight()) / 2);
     }
 }
 
-void DropDown::keyPressed(gcn::KeyEvent& keyEvent)
+void DropDown::keyPressed(KeyEvent& event)
 {
-    if (keyEvent.isConsumed())
+    if (event.isConsumed())
         return;
 
-    const int actionId = static_cast<KeyEvent*>(&keyEvent)->getActionId();
+    const int actionId = event.getActionId();
     switch (actionId)
     {
-        case Input::KEY_GUI_SELECT:
-        case Input::KEY_GUI_SELECT2:
+        case InputAction::GUI_SELECT:
+        case InputAction::GUI_SELECT2:
             dropDown();
             break;
 
-        case Input::KEY_GUI_UP:
+        case InputAction::GUI_UP:
             setSelected(getSelected() - 1);
             break;
 
-        case Input::KEY_GUI_DOWN:
+        case InputAction::GUI_DOWN:
             setSelected(getSelected() + 1);
             break;
 
-        case Input::KEY_GUI_HOME:
+        case InputAction::GUI_HOME:
             setSelected(0);
             break;
 
-        case Input::KEY_GUI_END:
+        case InputAction::GUI_END:
             if (mPopup->getListModel())
             {
                 setSelected(mPopup->getListModel()->
@@ -346,7 +347,7 @@ void DropDown::keyPressed(gcn::KeyEvent& keyEvent)
             return;
     }
 
-    keyEvent.consume();
+    event.consume();
 }
 
 void DropDown::hideDrop(bool event)
@@ -356,11 +357,12 @@ void DropDown::hideDrop(bool event)
     mPopup->setVisible(false);
 }
 
-void DropDown::mousePressed(gcn::MouseEvent& mouseEvent)
+void DropDown::mousePressed(MouseEvent& event)
 {
+    event.consume();
     // If we have a mouse press on the widget.
-    if (mouseEvent.getButton() == gcn::MouseEvent::LEFT
-        && !mDroppedDown && mouseEvent.getSource() == this)
+    if (event.getButton() == MouseButton::LEFT
+        && !mDroppedDown && event.getSource() == this)
     {
         mPushed = true;
         dropDown();
@@ -373,23 +375,23 @@ void DropDown::mousePressed(gcn::MouseEvent& mouseEvent)
     }
 }
 
-void DropDown::mouseReleased(gcn::MouseEvent &mouseEvent)
+void DropDown::mouseReleased(MouseEvent &event)
 {
     if (mIsDragged)
         mPushed = false;
 
-    const int button = mouseEvent.getButton();
-    const int x = mouseEvent.getX();
-    const int y = mouseEvent.getY();
+    const int button = event.getButton();
+    const int x = event.getX();
+    const int y = event.getY();
     // Released outside of widget. Can happen when we have modal
     // input focus.
     if ((0 > y || y >= mDimension.height || x < 0 || x >= mDimension.width)
-        && button == gcn::MouseEvent::LEFT)
+        && button == MouseButton::LEFT)
     {
         if (mIsDragged)
             foldUp();
     }
-    else if (button == gcn::MouseEvent::LEFT)
+    else if (button == MouseButton::LEFT)
     {
         mPushed = false;
     }
@@ -397,27 +399,27 @@ void DropDown::mouseReleased(gcn::MouseEvent &mouseEvent)
     mIsDragged = false;
 }
 
-void DropDown::mouseDragged(gcn::MouseEvent &mouseEvent)
+void DropDown::mouseDragged(MouseEvent &event)
 {
     mIsDragged = true;
-    mouseEvent.consume();
+    event.consume();
 }
 
-void DropDown::mouseWheelMovedUp(gcn::MouseEvent& mouseEvent)
+void DropDown::mouseWheelMovedUp(MouseEvent& event)
 {
     setSelected(getSelected() - 1);
-    mouseEvent.consume();
+    event.consume();
 }
 
-void DropDown::mouseWheelMovedDown(gcn::MouseEvent& mouseEvent)
+void DropDown::mouseWheelMovedDown(MouseEvent& event)
 {
     setSelected(getSelected() + 1);
-    mouseEvent.consume();
+    event.consume();
 }
 
 void DropDown::setSelectedString(const std::string &str)
 {
-    gcn::ListModel *const listModel = mPopup->getListModel();
+    ListModel *const listModel = mPopup->getListModel();
     if (!listModel)
         return;
 
@@ -433,7 +435,7 @@ void DropDown::setSelectedString(const std::string &str)
 
 std::string DropDown::getSelectedString() const
 {
-    gcn::ListModel *const listModel = mPopup->getListModel();
+    ListModel *const listModel = mPopup->getListModel();
     if (!listModel)
         return "";
 
@@ -490,7 +492,7 @@ void DropDown::setSelected(int selected)
         mPopup->setSelected(selected);
 }
 
-void DropDown::setListModel(gcn::ListModel *const listModel)
+void DropDown::setListModel(ListModel *const listModel)
 {
     mPopup->setListModel(listModel);
 
@@ -500,30 +502,30 @@ void DropDown::setListModel(gcn::ListModel *const listModel)
     adjustHeight();
 }
 
-gcn::ListModel *DropDown::getListModel()
+ListModel *DropDown::getListModel()
 {
     return mPopup->getListModel();
 }
 
-void DropDown::action(const gcn::ActionEvent &actionEvent A_UNUSED)
+void DropDown::action(const ActionEvent &actionEvent A_UNUSED)
 {
     foldUp();
     distributeActionEvent();
 }
 
-gcn::Rectangle DropDown::getChildrenArea()
+Rect DropDown::getChildrenArea()
 {
     if (mDroppedDown)
     {
         // Calculate the children area (with the one pixel border in mind)
-        return gcn::Rectangle(1, mFoldedUpHeight + 1,
+        return Rect(1, mFoldedUpHeight + 1,
             mDimension.width - 2, mDimension.height - mFoldedUpHeight - 2);
     }
 
-    return gcn::Rectangle();
+    return Rect();
 }
 
-void DropDown::valueChanged(const gcn::SelectionEvent& event A_UNUSED)
+void DropDown::valueChanged(const SelectionEvent& event A_UNUSED)
 {
 }
 
@@ -551,7 +553,7 @@ void DropDown::distributeValueChangedEvent()
           iter != mSelectionListeners.end();
           ++iter)
     {
-        gcn::SelectionEvent event(this);
+        SelectionEvent event(this);
         (*iter)->valueChanged(event);
     }
 }
