@@ -22,26 +22,30 @@
 
 #include "gui/widgets/dropshortcutcontainer.h"
 
-#include "client.h"
 #include "dragdrop.h"
 #include "dropshortcut.h"
+#include "settings.h"
 
 #include "being/playerinfo.h"
 
+#include "gui/popupmanager.h"
 #include "gui/viewport.h"
 
+#include "gui/fonts/font.h"
+
 #include "gui/popups/itempopup.h"
 
 #include "gui/windows/inventorywindow.h"
 
 #include "resources/image.h"
 
-#include <guichan/font.hpp>
+#include "utils/delete2.h"
+#include "utils/stringutils.h"
 
 #include "debug.h"
 
-DropShortcutContainer::DropShortcutContainer():
-    ShortcutContainer(),
+DropShortcutContainer::DropShortcutContainer(Widget2 *const widget) :
+    ShortcutContainer(widget),
     mItemClicked(false),
     mItemPopup(new ItemPopup),
     mEquipedColor(getThemeColor(Theme::ITEM_EQUIPPED)),
@@ -51,38 +55,15 @@ DropShortcutContainer::DropShortcutContainer():
 {
     mItemPopup->postInit();
 
-    addMouseListener(this);
-    addWidgetListener(this);
-
-    mBackgroundImg = Theme::getImageFromThemeXml(
-        "item_shortcut_background.xml", "background.xml");
     if (dropShortcut)
         mMaxItems = dropShortcut->getItemCount();
     else
         mMaxItems = 0;
-
-    if (mBackgroundImg)
-    {
-        mBackgroundImg->setAlpha(client->getGuiAlpha());
-        mBoxHeight = mBackgroundImg->getHeight();
-        mBoxWidth = mBackgroundImg->getWidth();
-    }
-    else
-    {
-        mBoxHeight = 1;
-        mBoxWidth = 1;
-    }
 }
 
 DropShortcutContainer::~DropShortcutContainer()
 {
-    if (mBackgroundImg)
-    {
-        mBackgroundImg->decRef();
-        mBackgroundImg = nullptr;
-    }
-    delete mItemPopup;
-    mItemPopup = nullptr;
+    delete2(mItemPopup);
 }
 
 void DropShortcutContainer::setWidget2(const Widget2 *const widget)
@@ -94,21 +75,20 @@ void DropShortcutContainer::setWidget2(const Widget2 *const widget)
     mUnEquipedColor2 = getThemeColor(Theme::ITEM_NOT_EQUIPPED_OUTLINE);
 }
 
-void DropShortcutContainer::draw(gcn::Graphics *graphics)
+void DropShortcutContainer::draw(Graphics *graphics)
 {
     if (!dropShortcut)
         return;
 
     BLOCK_START("DropShortcutContainer::draw")
-    if (client->getGuiAlpha() != mAlpha)
+    if (settings.guiAlpha != mAlpha)
     {
-        mAlpha = client->getGuiAlpha();
+        mAlpha = settings.guiAlpha;
         if (mBackgroundImg)
             mBackgroundImg->setAlpha(mAlpha);
     }
 
-    Graphics *const g = static_cast<Graphics*>(graphics);
-    drawBackground(g);
+    drawBackground(graphics);
 
     const Inventory *const inv = PlayerInfo::getInventory();
     if (!inv)
@@ -117,7 +97,7 @@ void DropShortcutContainer::draw(gcn::Graphics *graphics)
         return;
     }
 
-    gcn::Font *const font = getFont();
+    Font *const font = getFont();
 
     for (unsigned i = 0; i < mMaxItems; i++)
     {
@@ -144,12 +124,12 @@ void DropShortcutContainer::draw(gcn::Graphics *graphics)
                     caption = "Eq.";
 
                 image->setAlpha(1.0F);
-                g->drawImage2(image, itemX, itemY);
+                graphics->drawImage(image, itemX, itemY);
                 if (item->isEquipped())
-                    g->setColorAll(mEquipedColor, mEquipedColor2);
+                    graphics->setColorAll(mEquipedColor, mEquipedColor2);
                 else
-                    g->setColorAll(mUnEquipedColor, mUnEquipedColor2);
-                font->drawString(g, caption,
+                    graphics->setColorAll(mUnEquipedColor, mUnEquipedColor2);
+                font->drawString(graphics, caption,
                     itemX + (mBoxWidth - font->getWidth(caption)) / 2,
                     itemY + mBoxHeight - 14);
             }
@@ -158,12 +138,12 @@ void DropShortcutContainer::draw(gcn::Graphics *graphics)
     BLOCK_END("DropShortcutContainer::draw")
 }
 
-void DropShortcutContainer::mouseDragged(gcn::MouseEvent &event)
+void DropShortcutContainer::mouseDragged(MouseEvent &event)
 {
     if (!dropShortcut)
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         if (dragDrop.isEmpty() && mItemClicked)
         {
@@ -197,7 +177,7 @@ void DropShortcutContainer::mouseDragged(gcn::MouseEvent &event)
     }
 }
 
-void DropShortcutContainer::mousePressed(gcn::MouseEvent &event)
+void DropShortcutContainer::mousePressed(MouseEvent &event)
 {
     if (!dropShortcut || !inventoryWindow)
         return;
@@ -207,8 +187,10 @@ void DropShortcutContainer::mousePressed(gcn::MouseEvent &event)
     if (index == -1)
         return;
 
+    event.consume();
+
     const int eventButton = event.getButton();
-    if (eventButton == gcn::MouseEvent::LEFT)
+    if (eventButton == MouseButton::LEFT)
     {
         if (dropShortcut->getItem(index) > 0)
         {
@@ -224,7 +206,7 @@ void DropShortcutContainer::mousePressed(gcn::MouseEvent &event)
             }
         }
     }
-    else if (eventButton == gcn::MouseEvent::RIGHT)
+    else if (eventButton == MouseButton::RIGHT)
     {
         const Inventory *const inv = PlayerInfo::getInventory();
         if (!inv)
@@ -233,17 +215,17 @@ void DropShortcutContainer::mousePressed(gcn::MouseEvent &event)
         Item *const item = inv->findItem(dropShortcut->getItem(index),
             dropShortcut->getItemColor(index));
 
-        if (viewport)
-            viewport->showDropPopup(item);
+        if (popupManager)
+            popupManager->showDropPopup(item);
     }
 }
 
-void DropShortcutContainer::mouseReleased(gcn::MouseEvent &event)
+void DropShortcutContainer::mouseReleased(MouseEvent &event)
 {
     if (!dropShortcut)
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         if (dropShortcut->isItemSelected())
             dropShortcut->setItemSelected(-1);
@@ -270,7 +252,7 @@ void DropShortcutContainer::mouseReleased(gcn::MouseEvent &event)
 }
 
 // Show ItemTooltip
-void DropShortcutContainer::mouseMoved(gcn::MouseEvent &event)
+void DropShortcutContainer::mouseMoved(MouseEvent &event)
 {
     if (!dropShortcut)
         return;
@@ -303,13 +285,13 @@ void DropShortcutContainer::mouseMoved(gcn::MouseEvent &event)
     }
 }
 
-void DropShortcutContainer::mouseExited(gcn::MouseEvent &event A_UNUSED)
+void DropShortcutContainer::mouseExited(MouseEvent &event A_UNUSED)
 {
     if (mItemPopup)
         mItemPopup->setVisible(false);
 }
 
-void DropShortcutContainer::widgetHidden(const gcn::Event &event A_UNUSED)
+void DropShortcutContainer::widgetHidden(const Event &event A_UNUSED)
 {
     if (mItemPopup)
         mItemPopup->setVisible(false);
