@@ -1,5 +1,5 @@
 /*
- *  Extended support for activating emotes
+ *  The ManaPlus Client
  *  Copyright (C) 2009  Aethyra Development Team
  *  Copyright (C) 2011-2014  The ManaPlus Developers
  *
@@ -24,12 +24,12 @@
 
 #include "gui/widgets/shortcutcontainer.h"
 
-#include "resources/db/emotedb.h"
-
 #include <vector>
 
 class TextPopup;
 
+struct EmoteSprite;
+
 /**
  * An emote shortcut container. Used to quickly use emoticons.
  *
@@ -41,7 +41,7 @@ class EmoteShortcutContainer final : public ShortcutContainer
         /**
          * Constructor. Initializes the graphic.
          */
-        EmoteShortcutContainer();
+        explicit EmoteShortcutContainer(Widget2 *const widget);
 
         A_DELETE_COPY(EmoteShortcutContainer)
 
@@ -53,28 +53,28 @@ class EmoteShortcutContainer final : public ShortcutContainer
         /**
          * Draws the items.
          */
-        void draw(gcn::Graphics *graphics) override final;
+        void draw(Graphics *graphics) override final;
 
         /**
          * Handles mouse when dragged.
          */
-        void mouseDragged(gcn::MouseEvent &event) override final;
+        void mouseDragged(MouseEvent &event) override final;
 
         /**
          * Handles mouse when pressed.
          */
-        void mousePressed(gcn::MouseEvent &event) override final;
+        void mousePressed(MouseEvent &event) override final;
 
         /**
          * Handles mouse release.
          */
-        void mouseReleased(gcn::MouseEvent &event) override final;
+        void mouseReleased(MouseEvent &event) override final;
 
-        void mouseMoved(gcn::MouseEvent &event) override final;
+        void mouseMoved(MouseEvent &event) override final;
 
-        void mouseExited(gcn::MouseEvent &event) override final;
+        void mouseExited(MouseEvent &event) override final;
 
-        void widgetHidden(const gcn::Event &event) override final;
+        void widgetHidden(const Event &event) override final;
 
         void setWidget2(const Widget2 *const widget) override final;
 
