@@ -30,8 +30,11 @@
 #include "being/playerinfo.h"
 
 #include "gui/gui.h"
+#include "gui/skin.h"
 #include "gui/viewport.h"
 
+#include "gui/fonts/font.h"
+
 #include "gui/popups/itempopup.h"
 
 #include "gui/windows/chatwindow.h"
@@ -45,9 +48,11 @@
 #include "utils/gettext.h"
 
 #include "resources/image.h"
+#include "resources/iteminfo.h"
 
-#include <guichan/font.hpp>
-#include <guichan/selectionlistener.hpp>
+#include "listeners/selectionlistener.h"
+
+#include "utils/delete2.h"
 
 #include <algorithm>
 
@@ -143,12 +148,13 @@ namespace
                 if (!pair1 || !pair2)
                     return false;
 
-                const int t1 = pair1->mItem->getInfo().getType();
-                const int t2 = pair2->mItem->getInfo().getType();
+                const ItemInfo &info1 = pair1->mItem->getInfo();
+                const ItemInfo &info2 = pair2->mItem->getInfo();
+                const ItemType::Type t1 = info1.getType();
+                const ItemType::Type t2 = info2.getType();
                 if (t1 == t2)
                 {
-                    return (pair1->mItem->getInfo().getName()
-                            < pair2->mItem->getInfo().getName());
+                    return (info1.getName() < info2.getName());
                 }
                 return t1 < t2;
             }
@@ -158,19 +164,17 @@ namespace
 ItemContainer::ItemContainer(const Widget2 *const widget,
                              Inventory *const inventory,
                              const bool forceQuantity) :
-    gcn::Widget(),
-    Widget2(widget),
-    gcn::KeyListener(),
-    gcn::MouseListener(),
-    gcn::WidgetListener(),
+    Widget(widget),
+    KeyListener(),
+    MouseListener(),
+    WidgetListener(),
     mInventory(inventory),
     mSelImg(Theme::getImageFromThemeXml("item_selection.xml", "")),
     mProtectedImg(Theme::getImageFromTheme("lock.png")),
     mName(),
     mItemPopup(new ItemPopup),
     mShowMatrix(nullptr),
-    mSkin(Theme::instance() ? Theme::instance()->load(
-          "itemcontainer.xml", "") : nullptr),
+    mSkin(theme ? theme->load("itemcontainer.xml", "") : nullptr),
     mEquipedColor(getThemeColor(Theme::ITEM_EQUIPPED)),
     mEquipedColor2(getThemeColor(Theme::ITEM_EQUIPPED_OUTLINE)),
     mUnEquipedColor(getThemeColor(Theme::ITEM_NOT_EQUIPPED)),
@@ -198,6 +202,7 @@ ItemContainer::ItemContainer(const Widget2 *const widget,
     addKeyListener(this);
     addMouseListener(this);
     addWidgetListener(this);
+    mAllowLogic = false;
 }
 
 ItemContainer::~ItemContainer()
@@ -216,18 +221,17 @@ ItemContainer::~ItemContainer()
         mProtectedImg = nullptr;
     }
 
-    if (Theme::instance())
-        Theme::instance()->unload(mSkin);
+    if (theme)
+        theme->unload(mSkin);
 
-    delete mItemPopup;
-    mItemPopup = nullptr;
+    delete2(mItemPopup);
     delete []mShowMatrix;
 }
 
 void ItemContainer::logic()
 {
     BLOCK_START("ItemContainer::logic")
-    gcn::Widget::logic();
+    Widget::logic();
 
     if (!mInventory)
     {
@@ -245,14 +249,13 @@ void ItemContainer::logic()
     BLOCK_END("ItemContainer::logic")
 }
 
-void ItemContainer::draw(gcn::Graphics *graphics)
+void ItemContainer::draw(Graphics *graphics)
 {
     if (!mInventory || !mShowMatrix)
         return;
 
     BLOCK_START("ItemContainer::draw")
-    Graphics *const g = static_cast<Graphics *const>(graphics);
-    gcn::Font *const font = getFont();
+    Font *const font = getFont();
 
     for (int j = 0; j < mGridRows; j++)
     {
@@ -278,16 +281,18 @@ void ItemContainer::draw(gcn::Graphics *graphics)
                 if (mShowMatrix[itemIndex] == mSelectedIndex)
                 {
                     if (mSelImg)
-                        g->drawImage2(mSelImg, itemX, itemY);
+                        graphics->drawImage(mSelImg, itemX, itemY);
                 }
                 image->setAlpha(1.0F);  // ensure the image if fully drawn...
-                g->drawImage2(image, itemX + mPaddingItemX,
+                graphics->drawImage(image,
+                    itemX + mPaddingItemX,
                     itemY + mPaddingItemY);
                 if (mProtectedImg && PlayerInfo::isItemProtected(
                     item->getId()))
                 {
-                    g->drawImage2(mProtectedImg,
-                        itemX + mPaddingItemX, itemY + mPaddingItemY);
+                    graphics->drawImage(mProtectedImg,
+                        itemX + mPaddingItemX,
+                        itemY + mPaddingItemY);
                 }
             }
         }
@@ -324,11 +329,11 @@ void ItemContainer::draw(gcn::Graphics *graphics)
             }
 
             if (item->isEquipped())
-                g->setColorAll(mEquipedColor, mEquipedColor2);
+                graphics->setColorAll(mEquipedColor, mEquipedColor2);
             else
-                g->setColorAll(mUnEquipedColor, mUnEquipedColor2);
+                graphics->setColorAll(mUnEquipedColor, mUnEquipedColor2);
 
-            font->drawString(g, caption,
+            font->drawString(graphics, caption,
                 itemX + (mBoxWidth - font->getWidth(caption)) / 2,
                 itemY + mEquippedTextPadding);
         }
@@ -373,7 +378,7 @@ void ItemContainer::distributeValueChangedEvent()
     {
         if (*i)
         {
-            gcn::SelectionEvent event(this);
+            SelectionEvent event(this);
             (*i)->valueChanged(event);
         }
     }
@@ -385,15 +390,15 @@ void ItemContainer::hidePopup()
         mItemPopup->setVisible(false);
 }
 
-void ItemContainer::keyPressed(gcn::KeyEvent &event A_UNUSED)
+void ItemContainer::keyPressed(KeyEvent &event A_UNUSED)
 {
 }
 
-void ItemContainer::keyReleased(gcn::KeyEvent &event A_UNUSED)
+void ItemContainer::keyReleased(KeyEvent &event A_UNUSED)
 {
 }
 
-void ItemContainer::mousePressed(gcn::MouseEvent &event)
+void ItemContainer::mousePressed(MouseEvent &event)
 {
     if (!mInventory)
         return;
@@ -401,8 +406,9 @@ void ItemContainer::mousePressed(gcn::MouseEvent &event)
     const int button = event.getButton();
     mClicks = event.getClickCount();
 
-    if (button == gcn::MouseEvent::LEFT || button == gcn::MouseEvent::RIGHT)
+    if (button == MouseButton::LEFT || button == MouseButton::RIGHT)
     {
+        event.consume();
         const int index = getSlotIndex(event.getX(), event.getY());
         if (index == Inventory::NO_SLOT_INDEX)
             return;
@@ -467,13 +473,13 @@ void ItemContainer::mousePressed(gcn::MouseEvent &event)
     }
 }
 
-void ItemContainer::mouseDragged(gcn::MouseEvent &event A_UNUSED)
+void ItemContainer::mouseDragged(MouseEvent &event A_UNUSED)
 {
     if (mSelectionStatus != SEL_NONE)
         mSelectionStatus = SEL_DRAGGING;
 }
 
-void ItemContainer::mouseReleased(gcn::MouseEvent &event)
+void ItemContainer::mouseReleased(MouseEvent &event)
 {
     if (mClicks == 2)
         return;
@@ -505,7 +511,7 @@ void ItemContainer::mouseReleased(gcn::MouseEvent &event)
         Net::getInventoryHandler()->moveItem(mSelectedIndex, index);
         selectNone();
     }
-    else
+    else if (mInventory)
     {
         const DragDropSource src = dragDrop.getSource();
         DragDropSource dst = DRAGDROP_SOURCE_EMPTY;
@@ -593,7 +599,7 @@ void ItemContainer::mouseReleased(gcn::MouseEvent &event)
     }
 }
 
-void ItemContainer::mouseMoved(gcn::MouseEvent &event)
+void ItemContainer::mouseMoved(MouseEvent &event)
 {
     if (!mInventory)
         return;
@@ -612,12 +618,12 @@ void ItemContainer::mouseMoved(gcn::MouseEvent &event)
     }
 }
 
-void ItemContainer::mouseExited(gcn::MouseEvent &event A_UNUSED)
+void ItemContainer::mouseExited(MouseEvent &event A_UNUSED)
 {
     mItemPopup->setVisible(false);
 }
 
-void ItemContainer::widgetResized(const gcn::Event &event A_UNUSED)
+void ItemContainer::widgetResized(const Event &event A_UNUSED)
 {
     mGridColumns = std::max(1, mDimension.width / mBoxWidth);
     adjustHeight();
@@ -643,7 +649,7 @@ void ItemContainer::updateMatrix()
         return;
 
     delete []mShowMatrix;
-    mShowMatrix = new int[mGridRows * mGridColumns];
+    mShowMatrix = new int[static_cast<size_t>(mGridRows * mGridColumns)];
 
     std::vector<ItemIdPair*> sortedItems;
     int i = 0;
