@@ -22,6 +22,7 @@
 
 #include "gui/widgets/itemlinkhandler.h"
 
+#include "gui/popupmanager.h"
 #include "gui/viewport.h"
 
 #include "gui/popups/itempopup.h"
@@ -29,41 +30,25 @@
 #include "gui/windows/confirmdialog.h"
 #include "gui/windows/helpwindow.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
-#include "utils/process.h"
 #include "utils/stringutils.h"
 
 #include "resources/db/itemdb.h"
 
-#include <string>
+#include "listeners/openurllistener.h"
 
-#include <guichan/actionlistener.hpp>
-#include <guichan/mouseinput.hpp>
+#include <string>
 
 #include "debug.h"
 
 namespace
 {
-    struct OpenUrlListener : public gcn::ActionListener
-    {
-        OpenUrlListener() :
-            url()
-        {
-        }
-
-        A_DELETE_COPY(OpenUrlListener)
-
-        void action(const gcn::ActionEvent &event) override final
-        {
-            if (event.getId() == "yes")
-                openBrowser(url);
-        }
-
-        std::string url;
-    } listener;
+    OpenUrlListener listener;
 }  // namespace
 
 ItemLinkHandler::ItemLinkHandler() :
+    LinkHandler(),
     mItemPopup(new ItemPopup)
 {
     mItemPopup->postInit();
@@ -71,12 +56,10 @@ ItemLinkHandler::ItemLinkHandler() :
 
 ItemLinkHandler::~ItemLinkHandler()
 {
-    delete mItemPopup;
-    mItemPopup = nullptr;
+    delete2(mItemPopup);
 }
 
-void ItemLinkHandler::handleLink(const std::string &link,
-                                 gcn::MouseEvent *event)
+void ItemLinkHandler::handleLink(const std::string &link, MouseEvent *event)
 {
     if (strStartWith(link, "http://") || strStartWith(link, "https://"))
     {
@@ -86,7 +69,7 @@ void ItemLinkHandler::handleLink(const std::string &link,
         replaceAll(url, " ", "");
         listener.url = url;
         const int button = event->getButton();
-        if (button == gcn::MouseInput::LEFT)
+        if (button == MouseButton::LEFT)
         {
             ConfirmDialog *const confirmDlg = new ConfirmDialog(
                 // TRANSLATORS: dialog message
@@ -94,10 +77,10 @@ void ItemLinkHandler::handleLink(const std::string &link,
             confirmDlg->postInit();
             confirmDlg->addActionListener(&listener);
         }
-        else if (button == gcn::MouseInput::RIGHT)
+        else if (button == MouseButton::RIGHT)
         {
-            if (viewport)
-                viewport->showLinkPopup(url);
+            if (popupManager)
+                popupManager->showLinkPopup(url);
         }
     }
     else if (!link.empty() && link[0] == '?')
@@ -129,9 +112,9 @@ void ItemLinkHandler::handleLink(const std::string &link,
         splitToIntVector(str, link, ',');
         if (str.empty())
             return;
-        int color = 1;
+        unsigned char color = 1;
         if (str.size() > 1)
-            color = str[1];
+            color = static_cast<unsigned char>(str[1]);
         const int id = str[0];
         if (id > 0)
         {
