@@ -22,19 +22,22 @@
 
 #include "gui/widgets/itemshortcutcontainer.h"
 
-#include "client.h"
 #include "dragdrop.h"
 #include "inventory.h"
 #include "item.h"
 #include "itemshortcut.h"
+#include "settings.h"
 #include "spellshortcut.h"
 
 #include "being/playerinfo.h"
 
 #include "input/inputmanager.h"
 
+#include "gui/popupmanager.h"
 #include "gui/viewport.h"
 
+#include "gui/fonts/font.h"
+
 #include "gui/popups/itempopup.h"
 #include "gui/popups/spellpopup.h"
 
@@ -44,13 +47,16 @@
 #include "gui/widgets/skillinfo.h"
 
 #include "resources/image.h"
+#include "resources/skillconsts.h"
 
-#include <guichan/font.hpp>
+#include "utils/delete2.h"
+#include "utils/stringutils.h"
 
 #include "debug.h"
 
-ItemShortcutContainer::ItemShortcutContainer(const unsigned number) :
-    ShortcutContainer(),
+ItemShortcutContainer::ItemShortcutContainer(Widget2 *const widget,
+                                             const unsigned number) :
+    ShortcutContainer(widget),
     mItemClicked(false),
     mNumber(number),
     mItemPopup(new ItemPopup),
@@ -63,42 +69,16 @@ ItemShortcutContainer::ItemShortcutContainer(const unsigned number) :
     mItemPopup->postInit();
     mSpellPopup->postInit();
 
-    addMouseListener(this);
-    addWidgetListener(this);
-
-    mForegroundColor2 = getThemeColor(Theme::TEXT_OUTLINE);
-    mBackgroundImg = Theme::getImageFromThemeXml(
-        "item_shortcut_background.xml", "background.xml");
     if (itemShortcut[mNumber])
         mMaxItems = itemShortcut[mNumber]->getItemCount();
     else
         mMaxItems = 0;
-
-    if (mBackgroundImg)
-    {
-        mBackgroundImg->setAlpha(client->getGuiAlpha());
-        mBoxHeight = mBackgroundImg->getHeight();
-        mBoxWidth = mBackgroundImg->getWidth();
-    }
-    else
-    {
-        mBoxHeight = 1;
-        mBoxWidth = 1;
-    }
-    mForegroundColor = getThemeColor(Theme::TEXT);
 }
 
 ItemShortcutContainer::~ItemShortcutContainer()
 {
-    if (mBackgroundImg)
-    {
-        mBackgroundImg->decRef();
-        mBackgroundImg = nullptr;
-    }
-    delete mItemPopup;
-    mItemPopup = nullptr;
-    delete mSpellPopup;
-    mSpellPopup = nullptr;
+    delete2(mItemPopup);
+    delete2(mSpellPopup);
 }
 
 void ItemShortcutContainer::setWidget2(const Widget2 *const widget)
@@ -112,7 +92,7 @@ void ItemShortcutContainer::setWidget2(const Widget2 *const widget)
     mForegroundColor2 = getThemeColor(Theme::TEXT_OUTLINE);
 }
 
-void ItemShortcutContainer::draw(gcn::Graphics *graphics)
+void ItemShortcutContainer::draw(Graphics *graphics)
 {
     BLOCK_START("ItemShortcutContainer::draw")
     const ItemShortcut *const selShortcut = itemShortcut[mNumber];
@@ -122,16 +102,15 @@ void ItemShortcutContainer::draw(gcn::Graphics *graphics)
         return;
     }
 
-    if (client->getGuiAlpha() != mAlpha)
+    if (settings.guiAlpha != mAlpha)
     {
         if (mBackgroundImg)
             mBackgroundImg->setAlpha(mAlpha);
-        mAlpha = client->getGuiAlpha();
+        mAlpha = settings.guiAlpha;
     }
 
-    Graphics *const g = static_cast<Graphics*>(graphics);
-    gcn::Font *const font = getFont();
-    drawBackground(g);
+    Font *const font = getFont();
+    drawBackground(graphics);
 
     const Inventory *const inv = PlayerInfo::getInventory();
     if (!inv)
@@ -148,9 +127,9 @@ void ItemShortcutContainer::draw(gcn::Graphics *graphics)
 
         // Draw item keyboard shortcut.
         const std::string key = inputManager.getKeyValueString(
-            Input::KEY_SHORTCUT_1 + i);
-        g->setColorAll(mForegroundColor, mForegroundColor);
-        font->drawString(g, key, itemX + 2, itemY + 2);
+            InputAction::SHORTCUT_1 + i);
+        graphics->setColorAll(mForegroundColor, mForegroundColor);
+        font->drawString(graphics, key, itemX + 2, itemY + 2);
 
         const int itemId = selShortcut->getItem(i);
         const unsigned char itemColor = selShortcut->getItemColor(i);
@@ -175,12 +154,17 @@ void ItemShortcutContainer::draw(gcn::Graphics *graphics)
                     caption = "Eq.";
 
                     image->setAlpha(1.0F);
-                    g->drawImage2(image, itemX, itemY);
+                    graphics->drawImage(image, itemX, itemY);
                     if (item->isEquipped())
-                        g->setColorAll(mEquipedColor, mEquipedColor2);
+                    {
+                        graphics->setColorAll(mEquipedColor, mEquipedColor2);
+                    }
                     else
-                        g->setColorAll(mUnEquipedColor, mUnEquipedColor2);
-                    font->drawString(g, caption,
+                    {
+                        graphics->setColorAll(mUnEquipedColor,
+                            mUnEquipedColor2);
+                    }
+                    font->drawString(graphics, caption,
                         itemX + (mBoxWidth - font->getWidth(caption)) / 2,
                         itemY + mBoxHeight - 14);
                 }
@@ -199,11 +183,11 @@ void ItemShortcutContainer::draw(gcn::Graphics *graphics)
                     if (image)
                     {
                         image->setAlpha(1.0F);
-                        g->drawImage2(image, itemX, itemY);
+                        graphics->drawImage(image, itemX, itemY);
                     }
                 }
 
-                font->drawString(g, spell->getSymbol(),
+                font->drawString(graphics, spell->getSymbol(),
                     itemX + 2, itemY + mBoxHeight / 2);
             }
         }
@@ -218,10 +202,10 @@ void ItemShortcutContainer::draw(gcn::Graphics *graphics)
                 if (image)
                 {
                     image->setAlpha(1.0F);
-                    g->drawImage2(image, itemX, itemY);
+                    graphics->drawImage(image, itemX, itemY);
                 }
 
-                font->drawString(g, skill->data->shortName, itemX + 2,
+                font->drawString(graphics, skill->data->shortName, itemX + 2,
                     itemY + mBoxHeight / 2);
             }
         }
@@ -229,13 +213,13 @@ void ItemShortcutContainer::draw(gcn::Graphics *graphics)
     BLOCK_END("ItemShortcutContainer::draw")
 }
 
-void ItemShortcutContainer::mouseDragged(gcn::MouseEvent &event)
+void ItemShortcutContainer::mouseDragged(MouseEvent &event)
 {
     ItemShortcut *const selShortcut = itemShortcut[mNumber];
     if (!selShortcut)
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         if (dragDrop.isEmpty() && mItemClicked)
         {
@@ -317,7 +301,7 @@ void ItemShortcutContainer::mouseDragged(gcn::MouseEvent &event)
     }
 }
 
-void ItemShortcutContainer::mousePressed(gcn::MouseEvent &event)
+void ItemShortcutContainer::mousePressed(MouseEvent &event)
 {
     ItemShortcut *const selShortcut = itemShortcut[mNumber];
     if (!selShortcut)
@@ -328,8 +312,9 @@ void ItemShortcutContainer::mousePressed(gcn::MouseEvent &event)
     if (index == -1)
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
+        event.consume();
         // Stores the selected item if theirs one.
         if (selShortcut->isItemSelected() && inventoryWindow &&
             (inventoryWindow->isWindowVisible()
@@ -346,23 +331,24 @@ void ItemShortcutContainer::mousePressed(gcn::MouseEvent &event)
             mItemClicked = true;
         }
     }
-    else if (event.getButton() == gcn::MouseEvent::RIGHT)
+    else if (event.getButton() == MouseButton::RIGHT)
     {
-        if (viewport && selShortcut)
+        event.consume();
+        if (popupManager && selShortcut)
         {
-            viewport->showItemPopup(selShortcut->getItem(index),
+            popupManager->showItemPopup(selShortcut->getItem(index),
                 selShortcut->getItemColor(index));
         }
     }
 }
 
-void ItemShortcutContainer::mouseReleased(gcn::MouseEvent &event)
+void ItemShortcutContainer::mouseReleased(MouseEvent &event)
 {
     ItemShortcut *const selShortcut = itemShortcut[mNumber];
     if (!selShortcut)
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         if (selShortcut->isItemSelected())
             selShortcut->setItemSelected(-1);
@@ -398,7 +384,7 @@ void ItemShortcutContainer::mouseReleased(gcn::MouseEvent &event)
     }
 }
 
-void ItemShortcutContainer::mouseMoved(gcn::MouseEvent &event)
+void ItemShortcutContainer::mouseMoved(MouseEvent &event)
 {
     const ItemShortcut *const selShortcut = itemShortcut[mNumber];
     if (!selShortcut)
@@ -455,7 +441,7 @@ void ItemShortcutContainer::mouseMoved(gcn::MouseEvent &event)
 }
 
 // Hide ItemTooltip
-void ItemShortcutContainer::mouseExited(gcn::MouseEvent &event A_UNUSED)
+void ItemShortcutContainer::mouseExited(MouseEvent &event A_UNUSED)
 {
     if (mItemPopup)
         mItemPopup->setVisible(false);
@@ -463,7 +449,7 @@ void ItemShortcutContainer::mouseExited(gcn::MouseEvent &event A_UNUSED)
         mSpellPopup->setVisible(false);
 }
 
-void ItemShortcutContainer::widgetHidden(const gcn::Event &event A_UNUSED)
+void ItemShortcutContainer::widgetHidden(const Event &event A_UNUSED)
 {
     if (mItemPopup)
         mItemPopup->setVisible(false);
