@@ -20,10 +20,62 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/*      _______   __   __   __   ______   __   __   _______   __   __
+ *     / _____/\ / /\ / /\ / /\ / ____/\ / /\ / /\ / ___  /\ /  |\/ /\
+ *    / /\____\// / // / // / // /\___\// /_// / // /\_/ / // , |/ / /
+ *   / / /__   / / // / // / // / /    / ___  / // ___  / // /| ' / /
+ *  / /_// /\ / /_// / // / // /_/_   / / // / // /\_/ / // / |  / /
+ * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /
+ * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/
+ *
+ * Copyright (c) 2004 - 2008 Olof Naessén and Per Larsson
+ *
+ *
+ * Per Larsson a.k.a finalman
+ * Olof Naessén a.k.a jansem/yakslem
+ *
+ * Visit: http://guichan.sourceforge.net
+ *
+ * License: (BSD)
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ * 3. Neither the name of Guichan nor the names of its contributors may
+ *    be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+ * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
 #include "gui/widgets/scrollarea.h"
 
-#include "client.h"
 #include "graphicsvertexes.h"
+#include "settings.h"
+
+#include "gui/gui.h"
+#include "gui/skin.h"
+
+#include "resources/image.h"
+#include "resources/imagerect.h"
+
+#include "utils/delete2.h"
+#include "utils/stringutils.h"
 
 #include "debug.h"
 
@@ -45,44 +97,47 @@ static std::string const buttonFiles[2] =
     "scrollbuttons_pressed.xml"
 };
 
-ScrollArea::ScrollArea(const bool opaque, const std::string &skin) :
-    gcn::ScrollArea(),
-    gcn::WidgetListener(),
-    mX(0),
-    mY(0),
-    mClickX(0),
-    mClickY(0),
-    mVertexes(new ImageCollection),
-    mVertexes2(new ImageCollection),
-    mXOffset(0),
-    mYOffset(0),
-    mDrawWidth(0),
-    mDrawHeight(0),
-    mHasMouse(false),
-    mRedraw(true)
-{
-    mOpaque = opaque;
-    addWidgetListener(this);
-    init(skin);
-}
-
-ScrollArea::ScrollArea(gcn::Widget *const widget, const bool opaque,
+ScrollArea::ScrollArea(Widget2 *const widget2,
+                       Widget *const widget,
+                       const bool opaque,
                        const std::string &skin) :
-    gcn::ScrollArea(widget),
-    gcn::WidgetListener(),
+    BasicContainer(widget2),
+    MouseListener(),
+    WidgetListener(),
+    mVertexes(new ImageCollection),
+    mVertexes2(new ImageCollection),
+    mHPolicy(SHOW_AUTO),
+    mVPolicy(SHOW_AUTO),
+    mVScroll(0),
+    mHScroll(0),
+    mScrollbarWidth(12),
+    mUpButtonScrollAmount(10),
+    mDownButtonScrollAmount(10),
+    mLeftButtonScrollAmount(10),
+    mRightButtonScrollAmount(10),
+    mHorizontalMarkerDragOffset(0),
+    mVerticalMarkerDragOffset(0),
     mX(0),
     mY(0),
     mClickX(0),
     mClickY(0),
-    mVertexes(new ImageCollection),
-    mVertexes2(new ImageCollection),
     mXOffset(0),
     mYOffset(0),
     mDrawWidth(0),
     mDrawHeight(0),
-    mHasMouse(false),
-    mRedraw(true)
+    mVBarVisible(false),
+    mHBarVisible(false),
+    mUpButtonPressed(false),
+    mDownButtonPressed(false),
+    mLeftButtonPressed(false),
+    mRightButtonPressed(false),
+    mIsVerticalMarkerDragged(false),
+    mIsHorizontalMarkerDragged(false),
+    mOpaque(true),
+    mHasMouse(false)
 {
+    setContent(widget);
+    addMouseListener(this);
     mOpaque = opaque;
     init(skin);
 }
@@ -96,31 +151,27 @@ ScrollArea::~ScrollArea()
     delete getContent();
 
     instances--;
-    const Theme *const theme = Theme::instance();
-    if (theme)
+    if (instances == 0)
     {
-        if (instances == 0)
+        Theme::unloadRect(background);
+        Theme::unloadRect(vMarker);
+        Theme::unloadRect(vMarkerHi);
+        Theme::unloadRect(vBackground);
+        Theme::unloadRect(hBackground);
+        for (int i = 0; i < 2; i ++)
         {
-            theme->unloadRect(background);
-            theme->unloadRect(vMarker);
-            theme->unloadRect(vMarkerHi);
-            theme->unloadRect(vBackground);
-            theme->unloadRect(hBackground);
-            for (int i = 0; i < 2; i ++)
+            for (int f = UP; f < BUTTONS_DIR; f ++)
             {
-                for (int f = UP; f < BUTTONS_DIR; f ++)
-                {
-                    if (buttons[f][i])
-                        buttons[f][i]->decRef();
-                }
+                if (buttons[f][i])
+                    buttons[f][i]->decRef();
             }
         }
     }
 
-    delete mVertexes;
-    mVertexes = nullptr;
-    delete mVertexes2;
-    mVertexes2 = nullptr;
+    delete2(mVertexes);
+    delete2(mVertexes2);
+
+    setContent(nullptr);
 }
 
 void ScrollArea::init(std::string skinName)
@@ -146,7 +197,6 @@ void ScrollArea::init(std::string skinName)
         // +++ here probably need move background from static
         if (skinName == "")
             skinName = "scroll_background.xml";
-        Theme *const theme = Theme::instance();
         if (theme)
         {
             theme->loadRect(background, skinName, "scroll_background.xml");
@@ -158,8 +208,9 @@ void ScrollArea::init(std::string skinName)
 
         for (int i = 0; i < 2; i ++)
         {
-            Skin *const skin = Theme::instance()->load(
-                buttonFiles[i], "scrollbuttons.xml");
+            Skin *skin = nullptr;
+            if (theme)
+                skin = theme->load(buttonFiles[i], "scrollbuttons.xml");
             if (skin)
             {
                 const ImageRect &rect = skin->getBorder();
@@ -181,7 +232,8 @@ void ScrollArea::init(std::string skinName)
                 for (int f = UP; f < BUTTONS_DIR; f ++)
                     buttons[f][i] = nullptr;
             }
-            Theme::instance()->unload(skin);
+            if (theme)
+                theme->unload(skin);
         }
     }
     mScrollbarWidth = mScrollbarSize;
@@ -197,20 +249,27 @@ void ScrollArea::logic()
         return;
     }
 
-    gcn::ScrollArea::logic();
-    gcn::Widget *const content = getContent();
+    checkPolicies();
 
-    // When no scrollbar in a certain direction, adapt content size to match
-    // the content dimension exactly.
+    setVerticalScrollAmount(getVerticalScrollAmount());
+    setHorizontalScrollAmount(getHorizontalScrollAmount());
+
+    Widget *const content = getContent();
     if (content)
     {
-        const unsigned int frameSize = 2 * content->getFrameSize();
-        if (mHPolicy == gcn::ScrollArea::SHOW_NEVER)
+        unsigned int frameSize = content->getFrameSize();
+        content->setPosition(-mHScroll + frameSize, -mVScroll + frameSize);
+        content->logic();
+
+        // When no scrollbar in a certain direction,
+        // adapt content size to match the content dimension exactly.
+        frameSize = 2 * content->getFrameSize();
+        if (mHPolicy == ScrollArea::SHOW_NEVER)
         {
             content->setWidth((mVBarVisible ? (mDimension.width
                 - mScrollbarWidth) : mDimension.width) - frameSize);
         }
-        if (mVPolicy == gcn::ScrollArea::SHOW_NEVER)
+        if (mVPolicy == ScrollArea::SHOW_NEVER)
         {
             content->setHeight((mHBarVisible ? (mDimension.height
                 - mScrollbarWidth) : mDimension.height) - frameSize);
@@ -230,8 +289,8 @@ void ScrollArea::logic()
 
 void ScrollArea::updateAlpha()
 {
-    const float alpha = std::max(client->getGuiAlpha(),
-        Theme::instance()->getMinimumOpacity());
+    const float alpha = std::max(settings.guiAlpha,
+        theme->getMinimumOpacity());
 
     if (alpha != mAlpha)
     {
@@ -252,7 +311,7 @@ void ScrollArea::updateAlpha()
     }
 }
 
-void ScrollArea::draw(gcn::Graphics *graphics)
+void ScrollArea::draw(Graphics *graphics)
 {
     BLOCK_START("ScrollArea::draw")
     if (mVBarVisible || mHBarVisible)
@@ -261,6 +320,7 @@ void ScrollArea::draw(gcn::Graphics *graphics)
         {
             if (!mOpaque)
                 updateCalcFlag(graphics);
+            // need add caching or remove calc calls.
 //            if (mRedraw)
             {
                 mVertexes->clear();
@@ -285,9 +345,9 @@ void ScrollArea::draw(gcn::Graphics *graphics)
                     calcHBar(graphics);
                     calcHMarker(graphics);
                 }
+                graphics->finalize(mVertexes);
             }
-            static_cast<Graphics *const>(graphics)->drawTileCollection(
-                mVertexes);
+            graphics->drawTileCollection(mVertexes);
         }
         else
         {
@@ -319,11 +379,10 @@ void ScrollArea::draw(gcn::Graphics *graphics)
 
     if (mRedraw)
     {
-        Graphics *g = static_cast<Graphics *const>(graphics);
-        const bool redraw = g->getRedraw();
-        g->setRedraw(true);
+        const bool redraw = graphics->getRedraw();
+        graphics->setRedraw(true);
         drawChildren(graphics);
-        g->setRedraw(redraw);
+        graphics->setRedraw(redraw);
     }
     else
     {
@@ -333,14 +392,13 @@ void ScrollArea::draw(gcn::Graphics *graphics)
     BLOCK_END("ScrollArea::draw")
 }
 
-void ScrollArea::updateCalcFlag(gcn::Graphics *const graphics)
+void ScrollArea::updateCalcFlag(Graphics *const graphics)
 {
     if (!mRedraw)
     {
         // because we don't know where parent windows was moved,
         // need recalc vertexes
-        const gcn::ClipRectangle &rect = static_cast<Graphics*>(
-            graphics)->getTopClip();
+        const ClipRect &rect = graphics->getTopClip();
         if (rect.xOffset != mXOffset || rect.yOffset != mYOffset)
         {
             mRedraw = true;
@@ -353,14 +411,14 @@ void ScrollArea::updateCalcFlag(gcn::Graphics *const graphics)
             mDrawWidth = rect.width;
             mDrawHeight = rect.height;
         }
-        else if (static_cast<Graphics*>(graphics)->getRedraw())
+        else if (graphics->getRedraw())
         {
             mRedraw = true;
         }
     }
 }
 
-void ScrollArea::drawFrame(gcn::Graphics *graphics)
+void ScrollArea::drawFrame(Graphics *graphics)
 {
     BLOCK_START("ScrollArea::drawFrame")
     if (mOpaque)
@@ -376,15 +434,19 @@ void ScrollArea::drawFrame(gcn::Graphics *graphics)
             if (mRedraw)
             {
                 mVertexes2->clear();
-                static_cast<Graphics*>(graphics)->calcWindow(
-                    mVertexes2, 0, 0, w, h, background);
+                graphics->calcWindow(mVertexes2,
+                    0, 0,
+                    w, h,
+                    background);
+                graphics->finalize(mVertexes2);
             }
-            static_cast<Graphics*>(graphics)->drawTileCollection(mVertexes2);
+            graphics->drawTileCollection(mVertexes2);
         }
         else
         {
-            static_cast<Graphics*>(graphics)->drawImageRect(
-                0, 0, w, h, background);
+            graphics->drawImageRect(0, 0,
+                w, h,
+                background);
         }
     }
     BLOCK_END("ScrollArea::drawFrame")
@@ -396,11 +458,9 @@ void ScrollArea::setOpaque(bool opaque)
     setFrameSize(mOpaque ? 2 : 0);
 }
 
-void ScrollArea::drawButton(gcn::Graphics *const graphics,
-                            const BUTTON_DIR dir)
+Image *ScrollArea::getImageByState(Rect &dim, const BUTTON_DIR dir)
 {
     int state = 0;
-    gcn::Rectangle dim;
 
     switch (dir)
     {
@@ -424,235 +484,241 @@ void ScrollArea::drawButton(gcn::Graphics *const graphics,
         default:
             logger->log("ScrollArea::drawButton unknown dir: "
                         + toString(static_cast<unsigned>(dir)));
-            return;
-    }
-
-    if (buttons[dir][state])
-    {
-        static_cast<Graphics*>(graphics)->drawImage2(
-            buttons[dir][state], dim.x, dim.y);
+            return nullptr;
     }
+    return buttons[dir][state];
 }
 
-void ScrollArea::calcButton(gcn::Graphics *const graphics,
+void ScrollArea::drawButton(Graphics *const graphics,
                             const BUTTON_DIR dir)
 {
-    int state = 0;
-    gcn::Rectangle dim;
+    Rect dim;
+    const Image *const image = getImageByState(dim, dir);
 
-    switch (dir)
-    {
-        case UP:
-            state = mUpButtonPressed ? 1 : 0;
-            dim = getUpButtonDimension();
-            break;
-        case DOWN:
-            state = mDownButtonPressed ? 1 : 0;
-            dim = getDownButtonDimension();
-            break;
-        case LEFT:
-            state = mLeftButtonPressed ? 1 : 0;
-            dim = getLeftButtonDimension();
-            break;
-        case RIGHT:
-            state = mRightButtonPressed ? 1 : 0;
-            dim = getRightButtonDimension();
-            break;
-        case BUTTONS_DIR:
-        default:
-            logger->log("ScrollArea::drawButton unknown dir: "
-                        + toString(static_cast<unsigned>(dir)));
-            return;
-    }
+    if (image)
+        graphics->drawImage(image, dim.x, dim.y);
+}
 
-    if (buttons[dir][state])
+void ScrollArea::calcButton(Graphics *const graphics,
+                            const BUTTON_DIR dir)
+{
+    Rect dim;
+    const Image *const image = getImageByState(dim, dir);
+
+    if (image)
     {
         static_cast<Graphics*>(graphics)->calcTileCollection(
-            mVertexes, buttons[dir][state], dim.x, dim.y);
+            mVertexes, image, dim.x, dim.y);
     }
 }
 
-void ScrollArea::drawVBar(gcn::Graphics *const graphics)
+void ScrollArea::drawVBar(Graphics *const graphics) const
 {
-    const gcn::Rectangle &dim = getVerticalBarDimension();
-    Graphics *const g = static_cast<Graphics*>(graphics);
+    const Rect &dim = getVerticalBarDimension();
 
     if (vBackground.grid[4])
     {
-        g->drawPattern(vBackground.grid[4],
+        graphics->drawPattern(vBackground.grid[4],
             dim.x, dim.y, dim.width, dim.height);
     }
     if (vBackground.grid[1])
     {
-        g->drawPattern(vBackground.grid[1],
-            dim.x, dim.y, dim.width, vBackground.grid[1]->getHeight());
+        graphics->drawPattern(vBackground.grid[1],
+            dim.x, dim.y,
+            dim.width, vBackground.grid[1]->getHeight());
     }
     if (vBackground.grid[7])
     {
-        g->drawPattern(vBackground.grid[7],
+        graphics->drawPattern(vBackground.grid[7],
             dim.x, dim.height - vBackground.grid[7]->getHeight() + dim.y,
             dim.width, vBackground.grid[7]->getHeight());
     }
 }
 
-void ScrollArea::calcVBar(gcn::Graphics *const graphics)
+void ScrollArea::calcVBar(const Graphics *const graphics)
 {
-    const gcn::Rectangle &dim = getVerticalBarDimension();
-    Graphics *const g = static_cast<Graphics *const>(graphics);
+    const Rect &dim = getVerticalBarDimension();
 
     if (vBackground.grid[4])
     {
-        g->calcPattern(mVertexes, vBackground.grid[4],
-            dim.x, dim.y, dim.width, dim.height);
+        graphics->calcPattern(mVertexes,
+            vBackground.grid[4],
+            dim.x, dim.y,
+            dim.width, dim.height);
     }
     if (vBackground.grid[1])
     {
-        g->calcPattern(mVertexes, vBackground.grid[1],
-            dim.x, dim.y, dim.width, vBackground.grid[1]->getHeight());
+        graphics->calcPattern(mVertexes,
+            vBackground.grid[1],
+            dim.x, dim.y,
+            dim.width, vBackground.grid[1]->getHeight());
     }
     if (vBackground.grid[7])
     {
-        g->calcPattern(mVertexes, vBackground.grid[7],
+        graphics->calcPattern(mVertexes,
+            vBackground.grid[7],
             dim.x, dim.height - vBackground.grid[7]->getHeight() + dim.y,
             dim.width, vBackground.grid[7]->getHeight());
     }
 }
 
-void ScrollArea::drawHBar(gcn::Graphics *const graphics)
+void ScrollArea::drawHBar(Graphics *const graphics) const
 {
-    const gcn::Rectangle &dim = getHorizontalBarDimension();
-    Graphics *const g = static_cast<Graphics*>(graphics);
+    const Rect &dim = getHorizontalBarDimension();
 
     if (hBackground.grid[4])
     {
-        g->drawPattern(hBackground.grid[4],
-            dim.x, dim.y, dim.width, dim.height);
+        graphics->drawPattern(hBackground.grid[4],
+            dim.x, dim.y,
+            dim.width, dim.height);
     }
 
     if (hBackground.grid[3])
     {
-        g->drawPattern(hBackground.grid[3],
-            dim.x, dim.y, hBackground.grid[3]->getWidth(), dim.height);
+        graphics->drawPattern(hBackground.grid[3],
+            dim.x, dim.y,
+            hBackground.grid[3]->getWidth(), dim.height);
     }
 
     if (hBackground.grid[5])
     {
-        g->drawPattern(hBackground.grid[5],
-            dim.x + dim.width - hBackground.grid[5]->getWidth(), dim.y,
-            hBackground.grid[5]->getWidth(), dim.height);
+        graphics->drawPattern(hBackground.grid[5],
+            dim.x + dim.width - hBackground.grid[5]->getWidth(),
+            dim.y,
+            hBackground.grid[5]->getWidth(),
+            dim.height);
     }
 }
 
-void ScrollArea::calcHBar(gcn::Graphics *const graphics)
+void ScrollArea::calcHBar(const Graphics *const graphics)
 {
-    const gcn::Rectangle &dim = getHorizontalBarDimension();
-    Graphics *const g = static_cast<Graphics*>(graphics);
+    const Rect &dim = getHorizontalBarDimension();
 
     if (hBackground.grid[4])
     {
-        g->calcPattern(mVertexes, hBackground.grid[4],
-            dim.x, dim.y, dim.width, dim.height);
+        graphics->calcPattern(mVertexes,
+            hBackground.grid[4],
+            dim.x, dim.y,
+            dim.width, dim.height);
     }
 
     if (hBackground.grid[3])
     {
-        g->calcPattern(mVertexes, hBackground.grid[3],
-            dim.x, dim.y, hBackground.grid[3]->getWidth(), dim.height);
+        graphics->calcPattern(mVertexes,
+            hBackground.grid[3],
+            dim.x, dim.y,
+            hBackground.grid[3]->getWidth(), dim.height);
     }
 
     if (hBackground.grid[5])
     {
-        g->calcPattern(mVertexes, hBackground.grid[5],
-            dim.x + dim.width - hBackground.grid[5]->getWidth(), dim.y,
-            hBackground.grid[5]->getWidth(), dim.height);
+        graphics->calcPattern(mVertexes,
+            hBackground.grid[5],
+            dim.x + dim.width - hBackground.grid[5]->getWidth(),
+            dim.y,
+            hBackground.grid[5]->getWidth(),
+            dim.height);
     }
 }
 
-void ScrollArea::drawVMarker(gcn::Graphics *const graphics)
+void ScrollArea::drawVMarker(Graphics *const graphics)
 {
-    const gcn::Rectangle &dim = getVerticalMarkerDimension();
+    const Rect &dim = getVerticalMarkerDimension();
 
     if ((mHasMouse) && (mX > (mDimension.width - mScrollbarWidth)))
     {
-        static_cast<Graphics*>(graphics)->
-            drawImageRect(dim.x, dim.y, dim.width, dim.height, vMarkerHi);
+        graphics->drawImageRect(dim.x, dim.y,
+            dim.width, dim.height,
+            vMarkerHi);
     }
     else
     {
-        static_cast<Graphics*>(graphics)->
-            drawImageRect(dim.x, dim.y, dim.width, dim.height, vMarker);
+        graphics->drawImageRect(dim.x, dim.y,
+            dim.width, dim.height,
+            vMarker);
     }
 }
 
-void ScrollArea::calcVMarker(gcn::Graphics *const graphics)
+void ScrollArea::calcVMarker(Graphics *const graphics)
 {
-    const gcn::Rectangle &dim = getVerticalMarkerDimension();
+    const Rect &dim = getVerticalMarkerDimension();
 
     if ((mHasMouse) && (mX > (mDimension.width - mScrollbarWidth)))
     {
-        static_cast<Graphics*>(graphics)->calcWindow(
-            mVertexes, dim.x, dim.y, dim.width, dim.height, vMarkerHi);
+        graphics->calcWindow(mVertexes,
+            dim.x, dim.y,
+            dim.width, dim.height,
+            vMarkerHi);
     }
     else
     {
-        static_cast<Graphics*>(graphics)->calcWindow(
-            mVertexes, dim.x, dim.y, dim.width, dim.height, vMarker);
+        graphics->calcWindow(mVertexes,
+            dim.x, dim.y,
+            dim.width, dim.height,
+            vMarker);
     }
 }
 
-void ScrollArea::drawHMarker(gcn::Graphics *const graphics)
+void ScrollArea::drawHMarker(Graphics *const graphics)
 {
-    const gcn::Rectangle dim = getHorizontalMarkerDimension();
+    const Rect dim = getHorizontalMarkerDimension();
 
     if ((mHasMouse) && (mY > (mDimension.height - mScrollbarWidth)))
     {
-        static_cast<Graphics*>(graphics)->
-            drawImageRect(dim.x, dim.y, dim.width, dim.height, vMarkerHi);
+        graphics->drawImageRect(dim.x, dim.y,
+            dim.width, dim.height,
+            vMarkerHi);
     }
     else
     {
-        static_cast<Graphics*>(graphics)->
-            drawImageRect(dim.x, dim.y, dim.width, dim.height, vMarker);
+        graphics->drawImageRect(
+            dim.x, dim.y,
+            dim.width, dim.height,
+            vMarker);
     }
 }
 
-void ScrollArea::calcHMarker(gcn::Graphics *const graphics)
+void ScrollArea::calcHMarker(Graphics *const graphics)
 {
-    const gcn::Rectangle dim = getHorizontalMarkerDimension();
+    const Rect dim = getHorizontalMarkerDimension();
 
     if ((mHasMouse) && (mY > (mDimension.height - mScrollbarWidth)))
     {
-        static_cast<Graphics*>(graphics)->calcWindow(
-            mVertexes, dim.x, dim.y, dim.width, dim.height, vMarkerHi);
+        graphics->calcWindow(mVertexes,
+            dim.x, dim.y,
+            dim.width, dim.height,
+            vMarkerHi);
     }
     else
     {
-        static_cast<Graphics*>(graphics)->calcWindow(
-            mVertexes, dim.x, dim.y, dim.width, dim.height, vMarker);
+        graphics->calcWindow(mVertexes,
+            dim.x, dim.y,
+            dim.width, dim.height,
+            vMarker);
     }
 }
 
-void ScrollArea::mouseMoved(gcn::MouseEvent& event)
+void ScrollArea::mouseMoved(MouseEvent& event)
 {
     mX = event.getX();
     mY = event.getY();
 }
 
-void ScrollArea::mouseEntered(gcn::MouseEvent& event A_UNUSED)
+void ScrollArea::mouseEntered(MouseEvent& event A_UNUSED)
 {
     mHasMouse = true;
 }
 
-void ScrollArea::mouseExited(gcn::MouseEvent& event A_UNUSED)
+void ScrollArea::mouseExited(MouseEvent& event A_UNUSED)
 {
     mHasMouse = false;
 }
 
-void ScrollArea::widgetResized(const gcn::Event &event A_UNUSED)
+void ScrollArea::widgetResized(const Event &event A_UNUSED)
 {
     mRedraw = true;
     const unsigned int frameSize = 2 * mFrameSize;
-    gcn::Widget *const content = getContent();
+    Widget *const content = getContent();
     if (content)
     {
         content->setSize(mDimension.width - frameSize,
@@ -660,12 +726,12 @@ void ScrollArea::widgetResized(const gcn::Event &event A_UNUSED)
     }
 }
 
-void ScrollArea::widgetMoved(const gcn::Event& event A_UNUSED)
+void ScrollArea::widgetMoved(const Event& event A_UNUSED)
 {
     mRedraw = true;
 }
 
-void ScrollArea::mousePressed(gcn::MouseEvent& event)
+void ScrollArea::mousePressed(MouseEvent& event)
 {
     const int x = event.getX();
     const int y = event.getY();
@@ -675,24 +741,28 @@ void ScrollArea::mousePressed(gcn::MouseEvent& event)
         setVerticalScrollAmount(mVScroll
                                 - mUpButtonScrollAmount);
         mUpButtonPressed = true;
+        event.consume();
     }
     else if (getDownButtonDimension().isPointInRect(x, y))
     {
         setVerticalScrollAmount(mVScroll
                                 + mDownButtonScrollAmount);
         mDownButtonPressed = true;
+        event.consume();
     }
     else if (getLeftButtonDimension().isPointInRect(x, y))
     {
         setHorizontalScrollAmount(mHScroll
                                   - mLeftButtonScrollAmount);
         mLeftButtonPressed = true;
+        event.consume();
     }
     else if (getRightButtonDimension().isPointInRect(x, y))
     {
         setHorizontalScrollAmount(mHScroll
                                   + mRightButtonScrollAmount);
         mRightButtonPressed = true;
+        event.consume();
     }
     else if (getVerticalMarkerDimension().isPointInRect(x, y))
     {
@@ -700,6 +770,7 @@ void ScrollArea::mousePressed(gcn::MouseEvent& event)
         mIsVerticalMarkerDragged = true;
 
         mVerticalMarkerDragOffset = y - getVerticalMarkerDimension().y;
+        event.consume();
     }
     else if (getVerticalBarDimension().isPointInRect(x, y))
     {
@@ -713,13 +784,14 @@ void ScrollArea::mousePressed(gcn::MouseEvent& event)
             setVerticalScrollAmount(mVScroll
                 + static_cast<int>(getChildrenArea().height * 0.95));
         }
+        event.consume();
     }
     else if (getHorizontalMarkerDimension().isPointInRect(x, y))
     {
         mIsHorizontalMarkerDragged = true;
         mIsVerticalMarkerDragged = false;
-
         mHorizontalMarkerDragOffset = x - getHorizontalMarkerDimension().x;
+        event.consume();
     }
     else if (getHorizontalBarDimension().isPointInRect(x, y))
     {
@@ -733,18 +805,19 @@ void ScrollArea::mousePressed(gcn::MouseEvent& event)
             setHorizontalScrollAmount(mHScroll
                 + static_cast<int>(getChildrenArea().width * 0.95));
         }
+        event.consume();
     }
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         mClickX = event.getX();
         mClickY = event.getY();
     }
 }
 
-void ScrollArea::mouseReleased(gcn::MouseEvent& event)
+void ScrollArea::mouseReleased(MouseEvent& event)
 {
-    if (event.getButton() == gcn::MouseEvent::LEFT && mClickX && mClickY)
+    if (event.getButton() == MouseButton::LEFT && mClickX && mClickY)
     {
         if (!event.isConsumed())
         {
@@ -795,19 +868,26 @@ void ScrollArea::mouseReleased(gcn::MouseEvent& event)
             }
             mClickX = 0;
             mClickY = 0;
-            if (dx || dy)
+            if (mMouseConsume && (dx || dy))
                 event.consume();
         }
     }
-    gcn::ScrollArea::mouseReleased(event);
+    mUpButtonPressed = false;
+    mDownButtonPressed = false;
+    mLeftButtonPressed = false;
+    mRightButtonPressed = false;
+    mIsHorizontalMarkerDragged = false;
+    mIsVerticalMarkerDragged = false;
+    if (mMouseConsume)
+        event.consume();
     mRedraw = true;
 }
 
-void ScrollArea::mouseDragged(gcn::MouseEvent &event)
+void ScrollArea::mouseDragged(MouseEvent &event)
 {
     if (mIsVerticalMarkerDragged)
     {
-        const gcn::Rectangle barDim = getVerticalBarDimension();
+        const Rect barDim = getVerticalBarDimension();
 
         const int pos = event.getY() - barDim.y
               - mVerticalMarkerDragOffset;
@@ -826,7 +906,7 @@ void ScrollArea::mouseDragged(gcn::MouseEvent &event)
 
     if (mIsHorizontalMarkerDragged)
     {
-        const gcn::Rectangle barDim = getHorizontalBarDimension();
+        const Rect barDim = getHorizontalBarDimension();
 
         const int pos = event.getX() - barDim.x
             - mHorizontalMarkerDragOffset;
@@ -847,56 +927,56 @@ void ScrollArea::mouseDragged(gcn::MouseEvent &event)
     mRedraw = true;
 }
 
-gcn::Rectangle ScrollArea::getVerticalBarDimension() const
+Rect ScrollArea::getVerticalBarDimension() const
 {
     if (!mVBarVisible)
-        return gcn::Rectangle(0, 0, 0, 0);
+        return Rect(0, 0, 0, 0);
 
     const int height = (mVBarVisible && mShowButtons) ? mScrollbarWidth : 0;
     if (mHBarVisible)
     {
-        return gcn::Rectangle(mDimension.width - mScrollbarWidth,
+        return Rect(mDimension.width - mScrollbarWidth,
             height,
             mScrollbarWidth,
             mDimension.height - 2 * height - mScrollbarWidth);
     }
 
-    return gcn::Rectangle(mDimension.width - mScrollbarWidth,
+    return Rect(mDimension.width - mScrollbarWidth,
         height,
         mScrollbarWidth,
         mDimension.height - 2 * height);
 }
 
-gcn::Rectangle ScrollArea::getHorizontalBarDimension() const
+Rect ScrollArea::getHorizontalBarDimension() const
 {
     if (!mHBarVisible)
-        return gcn::Rectangle(0, 0, 0, 0);
+        return Rect(0, 0, 0, 0);
 
     const int width = mShowButtons ? mScrollbarWidth : 0;
     if (mVBarVisible)
     {
-        return gcn::Rectangle(width,
+        return Rect(width,
             mDimension.height - mScrollbarWidth,
             mDimension.width - 2 * width - mScrollbarWidth,
             mScrollbarWidth);
     }
 
-    return gcn::Rectangle(width,
+    return Rect(width,
                       mDimension.height - mScrollbarWidth,
                       mDimension.width - 2 * width,
                       mScrollbarWidth);
 }
 
-gcn::Rectangle ScrollArea::getVerticalMarkerDimension()
+Rect ScrollArea::getVerticalMarkerDimension()
 {
     if (!mVBarVisible)
-        return gcn::Rectangle(0, 0, 0, 0);
+        return Rect(0, 0, 0, 0);
 
     int length, pos;
     int height;
     const int h2 = mShowButtons
         ? mScrollbarWidth : mMarkerSize / 2;
-    const gcn::Widget *content;
+    const Widget *content;
     if (!mWidgets.empty())
         content = *mWidgets.begin();
     else
@@ -941,20 +1021,20 @@ gcn::Rectangle ScrollArea::getVerticalMarkerDimension()
             pos = 0;
     }
 
-    return gcn::Rectangle(mDimension.width - mScrollbarWidth, h2 + pos,
+    return Rect(mDimension.width - mScrollbarWidth, h2 + pos,
         mScrollbarWidth, length);
 }
 
-gcn::Rectangle ScrollArea::getHorizontalMarkerDimension()
+Rect ScrollArea::getHorizontalMarkerDimension()
 {
     if (!mHBarVisible)
-        return gcn::Rectangle(0, 0, 0, 0);
+        return Rect(0, 0, 0, 0);
 
     int length, pos;
     int width;
     const int w2 = mShowButtons
         ? mScrollbarWidth : mMarkerSize / 2;
-    const gcn::Widget *content;
+    const Widget *content;
     if (!mWidgets.empty())
         content = *mWidgets.begin();
     else
@@ -1003,62 +1083,349 @@ gcn::Rectangle ScrollArea::getHorizontalMarkerDimension()
         }
     }
 
-    return gcn::Rectangle(w2 + pos, mDimension.height - mScrollbarWidth,
+    return Rect(w2 + pos, mDimension.height - mScrollbarWidth,
         length, mScrollbarWidth);
 }
 
-gcn::Rectangle ScrollArea::getUpButtonDimension() const
+Rect ScrollArea::getUpButtonDimension() const
 {
     if (!mVBarVisible || !mShowButtons)
-        return gcn::Rectangle(0, 0, 0, 0);
+        return Rect(0, 0, 0, 0);
 
-    return gcn::Rectangle(mDimension.width - mScrollbarWidth, 0,
+    return Rect(mDimension.width - mScrollbarWidth, 0,
         mScrollbarWidth, mScrollbarWidth);
 }
 
-gcn::Rectangle ScrollArea::getDownButtonDimension() const
+Rect ScrollArea::getDownButtonDimension() const
 {
     if (!mVBarVisible || !mShowButtons)
-        return gcn::Rectangle(0, 0, 0, 0);
+        return Rect(0, 0, 0, 0);
 
     if (mVBarVisible && mHBarVisible)
     {
-        return gcn::Rectangle(mDimension.width - mScrollbarWidth,
+        return Rect(mDimension.width - mScrollbarWidth,
             mDimension.height - mScrollbarWidth*2,
             mScrollbarWidth,
             mScrollbarWidth);
     }
 
-    return gcn::Rectangle(mDimension.width - mScrollbarWidth,
+    return Rect(mDimension.width - mScrollbarWidth,
         mDimension.height - mScrollbarWidth,
         mScrollbarWidth,
         mScrollbarWidth);
 }
 
-gcn::Rectangle ScrollArea::getLeftButtonDimension() const
+Rect ScrollArea::getLeftButtonDimension() const
 {
     if (!mHBarVisible || !mShowButtons)
-        return gcn::Rectangle(0, 0, 0, 0);
+        return Rect(0, 0, 0, 0);
 
-    return gcn::Rectangle(0, mDimension.height - mScrollbarWidth,
+    return Rect(0, mDimension.height - mScrollbarWidth,
         mScrollbarWidth, mScrollbarWidth);
 }
 
-gcn::Rectangle ScrollArea::getRightButtonDimension() const
+Rect ScrollArea::getRightButtonDimension() const
 {
     if (!mHBarVisible || !mShowButtons)
-        return gcn::Rectangle(0, 0, 0, 0);
+        return Rect(0, 0, 0, 0);
 
     if (mVBarVisible && mHBarVisible)
     {
-        return gcn::Rectangle(mDimension.width - mScrollbarWidth*2,
+        return Rect(mDimension.width - mScrollbarWidth*2,
             mDimension.height - mScrollbarWidth,
             mScrollbarWidth,
             mScrollbarWidth);
     }
 
-    return gcn::Rectangle(mDimension.width - mScrollbarWidth,
+    return Rect(mDimension.width - mScrollbarWidth,
         mDimension.height - mScrollbarWidth,
         mScrollbarWidth,
         mScrollbarWidth);
 }
+
+void ScrollArea::setContent(Widget* widget)
+{
+    if (widget)
+    {
+        clear();
+        add(widget);
+        widget->setPosition(0, 0);
+    }
+    else
+    {
+        clear();
+    }
+
+    checkPolicies();
+}
+
+Widget* ScrollArea::getContent()
+{
+    if (!mWidgets.empty())
+        return *mWidgets.begin();
+
+    return nullptr;
+}
+
+void ScrollArea::setHorizontalScrollPolicy(const ScrollPolicy hPolicy)
+{
+    mHPolicy = hPolicy;
+    checkPolicies();
+}
+
+void ScrollArea::setVerticalScrollPolicy(const ScrollPolicy vPolicy)
+{
+    mVPolicy = vPolicy;
+    checkPolicies();
+}
+
+void ScrollArea::setScrollPolicy(const ScrollPolicy hPolicy,
+                                 const ScrollPolicy vPolicy)
+{
+    mHPolicy = hPolicy;
+    mVPolicy = vPolicy;
+    checkPolicies();
+}
+
+void ScrollArea::setVerticalScrollAmount(const int vScroll)
+{
+    const int max = getVerticalMaxScroll();
+
+    mVScroll = vScroll;
+
+    if (vScroll > max)
+        mVScroll = max;
+
+    if (vScroll < 0)
+        mVScroll = 0;
+}
+
+void ScrollArea::setHorizontalScrollAmount(int hScroll)
+{
+    const int max = getHorizontalMaxScroll();
+
+    mHScroll = hScroll;
+
+    if (hScroll > max)
+        mHScroll = max;
+    else if (hScroll < 0)
+        mHScroll = 0;
+}
+
+void ScrollArea::setScrollAmount(const int hScroll, const int vScroll)
+{
+    setHorizontalScrollAmount(hScroll);
+    setVerticalScrollAmount(vScroll);
+}
+
+int ScrollArea::getHorizontalMaxScroll()
+{
+    checkPolicies();
+
+    const Widget *const content = getContent();
+    if (!content)
+        return 0;
+
+    const int value = content->getWidth() - getChildrenArea().width +
+        2 * content->getFrameSize();
+
+    if (value < 0)
+        return 0;
+
+    return value;
+}
+
+int ScrollArea::getVerticalMaxScroll()
+{
+    checkPolicies();
+
+    const Widget *const content = getContent();
+    if (!content)
+        return 0;
+
+    int value;
+
+    value = content->getHeight() - getChildrenArea().height +
+        2 * content->getFrameSize();
+
+    if (value < 0)
+        return 0;
+
+    return value;
+}
+
+void ScrollArea::setScrollbarWidth(const int width)
+{
+    if (width > 0)
+        mScrollbarWidth = width;
+}
+
+void ScrollArea::showWidgetPart(Widget *const widget, Rect area)
+{
+    const Widget *const content = getContent();
+    if (widget != content)
+        return;
+
+    BasicContainer::showWidgetPart(widget, area);
+
+    setHorizontalScrollAmount(content->getFrameSize()
+        - content->getX());
+    setVerticalScrollAmount(content->getFrameSize()
+        - content->getY());
+}
+
+Rect ScrollArea::getChildrenArea()
+{
+    const Rect area = Rect(0, 0,
+        mVBarVisible ? (getWidth() - mScrollbarWidth) : getWidth(),
+        mHBarVisible ? (getHeight() - mScrollbarWidth) : getHeight());
+
+    if (area.width < 0 || area.height < 0)
+        return Rect();
+
+    return area;
+}
+
+Widget *ScrollArea::getWidgetAt(int x, int y)
+{
+    if (getChildrenArea().isPointInRect(x, y))
+        return getContent();
+
+    return nullptr;
+}
+
+void ScrollArea::setWidth(int width)
+{
+    Widget::setWidth(width);
+    checkPolicies();
+}
+
+void ScrollArea::setHeight(int height)
+{
+    Widget::setHeight(height);
+    checkPolicies();
+}
+
+void ScrollArea::setDimension(const Rect& dimension)
+{
+    Widget::setDimension(dimension);
+    checkPolicies();
+}
+
+void ScrollArea::mouseWheelMovedUp(MouseEvent& event)
+{
+    if (event.isConsumed())
+        return;
+
+    setVerticalScrollAmount(getVerticalScrollAmount()
+        - getChildrenArea().height / 8);
+
+    event.consume();
+}
+
+void ScrollArea::mouseWheelMovedDown(MouseEvent& event)
+{
+    if (event.isConsumed())
+        return;
+
+    setVerticalScrollAmount(getVerticalScrollAmount()
+        + getChildrenArea().height / 8);
+
+    event.consume();
+}
+
+void ScrollArea::checkPolicies()
+{
+    const int w = getWidth();
+    const int h = getHeight();
+
+    mHBarVisible = false;
+    mVBarVisible = false;
+
+    const Widget *const content = getContent();
+    if (!content)
+    {
+        mHBarVisible = (mHPolicy == SHOW_ALWAYS);
+        mVBarVisible = (mVPolicy == SHOW_ALWAYS);
+        return;
+    }
+
+    if (mHPolicy == SHOW_AUTO &&
+        mVPolicy == SHOW_AUTO)
+    {
+        if (content->getWidth() <= w
+            && content->getHeight() <= h)
+        {
+            mHBarVisible = false;
+            mVBarVisible = false;
+        }
+
+        if (content->getWidth() > w)
+        {
+            mHBarVisible = true;
+        }
+
+        if ((content->getHeight() > h)
+            || (mHBarVisible && content->getHeight()
+            > h - mScrollbarWidth))
+        {
+            mVBarVisible = true;
+        }
+
+        if (mVBarVisible && content->getWidth() > w - mScrollbarWidth)
+            mHBarVisible = true;
+
+        return;
+    }
+
+    switch (mHPolicy)
+    {
+        case SHOW_NEVER:
+            mHBarVisible = false;
+            break;
+
+        case SHOW_ALWAYS:
+            mHBarVisible = true;
+            break;
+
+        case SHOW_AUTO:
+            if (mVPolicy == SHOW_NEVER)
+            {
+                mHBarVisible = (content->getWidth() > w);
+            }
+            else  // (mVPolicy == SHOW_ALWAYS)
+            {
+                mHBarVisible = (content->getWidth()
+                    > w - mScrollbarWidth);
+            }
+            break;
+
+        default:
+            break;
+    }
+
+    switch (mVPolicy)
+    {
+        case SHOW_NEVER:
+            mVBarVisible = false;
+            break;
+
+        case SHOW_ALWAYS:
+            mVBarVisible = true;
+            break;
+
+        case SHOW_AUTO:
+            if (mHPolicy == SHOW_NEVER)
+            {
+                mVBarVisible = (content->getHeight() > h);
+            }
+            else  // (mHPolicy == SHOW_ALWAYS)
+            {
+                mVBarVisible = (content->getHeight()
+                    > h - mScrollbarWidth);
+            }
+            break;
+        default:
+            break;
+    }
+}
