@@ -22,28 +22,30 @@
 
 #include "gui/widgets/shoplistbox.h"
 
-#include "client.h"
+#include "settings.h"
 #include "shopitem.h"
 
 #include "being/playerinfo.h"
 
+#include "gui/popupmanager.h"
 #include "gui/viewport.h"
 
+#include "gui/fonts/font.h"
+
 #include "gui/popups/itempopup.h"
 
-#include "gui/widgets/shopitems.h"
+#include "gui/models/shopitems.h"
 
 #include "resources/image.h"
 
-#include <guichan/font.hpp>
-#include <guichan/listmodel.hpp>
+#include "gui/models/listmodel.h"
 
 #include "debug.h"
 
 const int ITEM_ICON_SIZE = 32;
 
 ShopListBox::ShopListBox(const Widget2 *const widget,
-                         gcn::ListModel *const listModel) :
+                         ListModel *const listModel) :
     ListBox(widget, listModel, "shoplistbox.xml"),
     mPlayerMoney(0),
     mShopItems(nullptr),
@@ -60,7 +62,7 @@ ShopListBox::ShopListBox(const Widget2 *const widget,
 }
 
 ShopListBox::ShopListBox(const Widget2 *const widget,
-                         gcn::ListModel *const listModel,
+                         ListModel *const listModel,
                          ShopItems *const shopListModel) :
     ListBox(widget, listModel, "shoplistbox.xml"),
     mPlayerMoney(0),
@@ -82,7 +84,7 @@ void ShopListBox::setPlayersMoney(const int money)
     mPlayerMoney = money;
 }
 
-void ShopListBox::draw(gcn::Graphics *gcnGraphics)
+void ShopListBox::draw(Graphics *graphics)
 {
     BLOCK_START("ShopListBox::draw")
     if (!mListModel || !mShopItems)
@@ -91,12 +93,11 @@ void ShopListBox::draw(gcn::Graphics *gcnGraphics)
         return;
     }
 
-    if (client->getGuiAlpha() != mAlpha)
-        mAlpha = client->getGuiAlpha();
+    if (settings.guiAlpha != mAlpha)
+        mAlpha = settings.guiAlpha;
 
     const int alpha = static_cast<int>(mAlpha * 255.0F);
-    Graphics *graphics = static_cast<Graphics*>(gcnGraphics);
-    gcn::Font *const font = getFont();
+    Font *const font = getFont();
 
     const int sz = mListModel->getNumberOfElements();
     const int fontHeigh = getFont()->getHeight();
@@ -107,8 +108,8 @@ void ShopListBox::draw(gcn::Graphics *gcnGraphics)
          ++i, y += mRowHeight)
     {
         bool needDraw(false);
-        gcn::Color temp;
-        gcn::Color* backgroundColor = &mBackgroundColor;
+        Color temp;
+        Color* backgroundColor = &mBackgroundColor;
 
         ShopItem *const item = mShopItems->at(i);
         if (item && ((mShopItems && mPlayerMoney < item->getPrice()
@@ -145,7 +146,7 @@ void ShopListBox::draw(gcn::Graphics *gcnGraphics)
         if (needDraw)
         {
             graphics->setColor(*backgroundColor);
-            graphics->fillRectangle(gcn::Rectangle(mPadding, y + mPadding,
+            graphics->fillRectangle(Rect(mPadding, y + mPadding,
                 width, mRowHeight));
         }
 
@@ -155,7 +156,7 @@ void ShopListBox::draw(gcn::Graphics *gcnGraphics)
             if (icon)
             {
                 icon->setAlpha(1.0F);
-                graphics->drawImage2(icon, mPadding, y + mPadding);
+                graphics->drawImage(icon, mPadding, y + mPadding);
             }
         }
         if (mSelected == i)
@@ -190,7 +191,7 @@ void ShopListBox::setPriceCheck(const bool check)
     mPriceCheck = check;
 }
 
-void ShopListBox::mouseMoved(gcn::MouseEvent &event)
+void ShopListBox::mouseMoved(MouseEvent &event)
 {
     if (!mItemPopup || !mRowHeight)
         return;
@@ -222,22 +223,23 @@ void ShopListBox::mouseMoved(gcn::MouseEvent &event)
     }
 }
 
-void ShopListBox::mouseReleased(gcn::MouseEvent& mouseEvent)
+void ShopListBox::mouseReleased(MouseEvent& event)
 {
-    ListBox::mouseReleased(mouseEvent);
-    if (mouseEvent.getButton() == gcn::MouseEvent::RIGHT)
+    ListBox::mouseReleased(event);
+    if (event.getButton() == MouseButton::RIGHT)
     {
-        setSelected(std::max(0, getSelectionByMouse(mouseEvent.getY())));
+        setSelected(std::max(0, getSelectionByMouse(event.getY())));
 
         if (mSelected < 0 || mSelected >= mShopItems->getNumberOfElements())
             return;
 
         Item *const item = mShopItems->at(mSelected);
-        viewport->showItemPopup(item);
+        if (popupManager)
+            popupManager->showItemPopup(item);
     }
 }
 
-void ShopListBox::mouseExited(gcn::MouseEvent& mouseEvent A_UNUSED)
+void ShopListBox::mouseExited(MouseEvent& event A_UNUSED)
 {
     if (!mItemPopup)
         return;
