@@ -27,7 +27,8 @@
 #include "gui/theme.h"
 
 #include "gui/widgets/skilldata.h"
-#include "gui/widgets/skillmodel.h"
+
+#include "gui/models/skillmodel.h"
 
 #include "utils/gettext.h"
 #include "utils/stringutils.h"
@@ -37,6 +38,7 @@
 SkillInfo::SkillInfo() :
     skillLevel(),
     skillExp(),
+    useButton(),
     progress(0.0F),
     color(),
     dataMap(),
