@@ -22,65 +22,44 @@
 
 #include "gui/widgets/spellshortcutcontainer.h"
 
-#include "client.h"
 #include "dragdrop.h"
 #include "itemshortcut.h"
+#include "settings.h"
 #include "spellshortcut.h"
 
+#include "gui/popupmanager.h"
 #include "gui/viewport.h"
 
+#include "gui/fonts/font.h"
+
 #include "gui/popups/spellpopup.h"
 
-#include "gui/windows/inventorywindow.h"
 #include "gui/windows/shortcutwindow.h"
 
 #include "resources/image.h"
 
-#include <guichan/font.hpp>
+#include "utils/delete2.h"
 
 #include "debug.h"
 
-SpellShortcutContainer::SpellShortcutContainer(const unsigned number) :
-    ShortcutContainer(),
+SpellShortcutContainer::SpellShortcutContainer(Widget2 *const widget,
+                                               const unsigned number) :
+    ShortcutContainer(widget),
     mSpellPopup(new SpellPopup),
     mNumber(number),
     mSpellClicked(false)
 {
     mSpellPopup->postInit();
 
-    addMouseListener(this);
-    addWidgetListener(this);
-
-    mForegroundColor2 = getThemeColor(Theme::TEXT_OUTLINE);
-
-    mBackgroundImg = Theme::getImageFromThemeXml(
-        "item_shortcut_background.xml", "background.xml");
     if (spellShortcut)
         mMaxItems = spellShortcut->getSpellsCount();
     else
         mMaxItems = 0;
-
-    if (mBackgroundImg)
-    {
-        mBackgroundImg->setAlpha(client->getGuiAlpha());
-        mBoxHeight = mBackgroundImg->getHeight();
-        mBoxWidth = mBackgroundImg->getWidth();
-    }
-    else
-    {
-        mBoxHeight = 1;
-        mBoxWidth = 1;
-    }
-    mForegroundColor = getThemeColor(Theme::TEXT);
 }
 
 SpellShortcutContainer::~SpellShortcutContainer()
 {
-    if (mBackgroundImg)
-        mBackgroundImg->decRef();
-    mBackgroundImg = nullptr;
-    delete mSpellPopup;
-    mSpellPopup = nullptr;
+    delete2(mSpellPopup);
 }
 
 void SpellShortcutContainer::setWidget2(const Widget2 *const widget)
@@ -90,26 +69,26 @@ void SpellShortcutContainer::setWidget2(const Widget2 *const widget)
     mForegroundColor2 = getThemeColor(Theme::TEXT_OUTLINE);
 }
 
-void SpellShortcutContainer::draw(gcn::Graphics *graphics)
+void SpellShortcutContainer::draw(Graphics *graphics)
 {
     if (!spellShortcut)
         return;
 
     BLOCK_START("SpellShortcutContainer::draw")
-    if (client->getGuiAlpha() != mAlpha)
+    if (settings.guiAlpha != mAlpha)
     {
-        mAlpha = client->getGuiAlpha();
+        mAlpha = settings.guiAlpha;
         if (mBackgroundImg)
             mBackgroundImg->setAlpha(mAlpha);
     }
 
-    Graphics *const g = static_cast<Graphics *const>(graphics);
-    gcn::Font *const font = getFont();
+    Font *const font = getFont();
 
     const int selectedId = spellShortcut->getSelectedItem();
-    g->setColorAll(mForegroundColor, mForegroundColor2);
-    drawBackground(g);
+    graphics->setColorAll(mForegroundColor, mForegroundColor2);
+    drawBackground(graphics);
 
+    // +++ in future need reorder images and string drawing.
     for (unsigned i = 0; i < mMaxItems; i++)
     {
         const int itemX = (i % mGridWidth) * mBoxWidth;
@@ -118,7 +97,7 @@ void SpellShortcutContainer::draw(gcn::Graphics *graphics)
         const int itemId = getItemByIndex(i);
         if (selectedId >= 0 && itemId == selectedId)
         {
-            g->drawRectangle(gcn::Rectangle(itemX + 1, itemY + 1,
+            graphics->drawRectangle(Rect(itemX + 1, itemY + 1,
                 mBoxWidth - 1, mBoxHeight - 1));
         }
 
@@ -135,11 +114,11 @@ void SpellShortcutContainer::draw(gcn::Graphics *graphics)
                 if (image)
                 {
                     image->setAlpha(1.0F);
-                    g->drawImage2(image, itemX, itemY);
+                    graphics->drawImage(image, itemX, itemY);
                 }
             }
 
-            font->drawString(g, spell->getSymbol(),
+            font->drawString(graphics, spell->getSymbol(),
                 itemX + 2, itemY + mBoxHeight / 2);
         }
     }
@@ -147,9 +126,9 @@ void SpellShortcutContainer::draw(gcn::Graphics *graphics)
     BLOCK_END("SpellShortcutContainer::draw")
 }
 
-void SpellShortcutContainer::mouseDragged(gcn::MouseEvent &event)
+void SpellShortcutContainer::mouseDragged(MouseEvent &event)
 {
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         if (dragDrop.isEmpty() && mSpellClicked)
         {
@@ -178,7 +157,7 @@ void SpellShortcutContainer::mouseDragged(gcn::MouseEvent &event)
     }
 }
 
-void SpellShortcutContainer::mousePressed(gcn::MouseEvent &event)
+void SpellShortcutContainer::mousePressed(MouseEvent &event)
 {
     const int index = getIndexFromGrid(event.getX(), event.getY());
 
@@ -186,26 +165,28 @@ void SpellShortcutContainer::mousePressed(gcn::MouseEvent &event)
         return;
 
     const unsigned int eventButton = event.getButton();
-    if (eventButton == gcn::MouseEvent::LEFT)
+    if (eventButton == MouseButton::LEFT)
     {
         const int itemId = getItemByIndex(index);
         if (itemId > 0)
             mSpellClicked = true;
+        event.consume();
     }
-    else if (eventButton == gcn::MouseEvent::RIGHT)
+    else if (eventButton == MouseButton::RIGHT)
     {
     }
-    else if (eventButton == gcn::MouseEvent::MIDDLE)
+    else if (eventButton == MouseButton::MIDDLE)
     {
         if (!spellShortcut || !spellManager)
             return;
 
+        event.consume();
         const int itemId = getItemByIndex(index);
         spellManager->invoke(itemId);
     }
 }
 
-void SpellShortcutContainer::mouseReleased(gcn::MouseEvent &event)
+void SpellShortcutContainer::mouseReleased(MouseEvent &event)
 {
     if (!spellShortcut || !spellManager)
         return;
@@ -221,7 +202,7 @@ void SpellShortcutContainer::mouseReleased(gcn::MouseEvent &event)
     const int itemId = getItemByIndex(index);
     const unsigned int eventButton = event.getButton();
 
-    if (eventButton == gcn::MouseEvent::LEFT)
+    if (eventButton == MouseButton::LEFT)
     {
         mSpellClicked = false;
 
@@ -273,19 +254,19 @@ void SpellShortcutContainer::mouseReleased(gcn::MouseEvent &event)
             }
         }
     }
-    else if (eventButton == gcn::MouseEvent::RIGHT)
+    else if (eventButton == MouseButton::RIGHT)
     {
         TextCommand *spell = nullptr;
         if (itemId >= 0)
             spell = spellManager->getSpell(itemId);
 
-        if (spell && viewport)
-            viewport->showSpellPopup(spell);
+        if (spell && popupManager)
+            popupManager->showSpellPopup(spell);
     }
 }
 
 // Show ItemTooltip
-void SpellShortcutContainer::mouseMoved(gcn::MouseEvent &event)
+void SpellShortcutContainer::mouseMoved(MouseEvent &event)
 {
     if (!mSpellPopup || !spellShortcut || !spellManager)
         return;
@@ -309,19 +290,19 @@ void SpellShortcutContainer::mouseMoved(gcn::MouseEvent &event)
     }
 }
 
-void SpellShortcutContainer::mouseExited(gcn::MouseEvent &event A_UNUSED)
+void SpellShortcutContainer::mouseExited(MouseEvent &event A_UNUSED)
 {
     if (mSpellPopup)
         mSpellPopup->setVisible(false);
 }
 
-void SpellShortcutContainer::widgetHidden(const gcn::Event &event A_UNUSED)
+void SpellShortcutContainer::widgetHidden(const Event &event A_UNUSED)
 {
     if (mSpellPopup)
         mSpellPopup->setVisible(false);
 }
 
-int SpellShortcutContainer::getItemByIndex(const int index)
+int SpellShortcutContainer::getItemByIndex(const int index) const
 {
     return spellShortcut->getItem(
         (mNumber * SPELL_SHORTCUT_ITEMS) + index);
