@@ -38,7 +38,8 @@ class SpellShortcutContainer final : public ShortcutContainer
         /**
          * Constructor. Initializes the graphic.
          */
-        explicit SpellShortcutContainer(const unsigned number);
+        SpellShortcutContainer(Widget2 *const widget,
+                               const unsigned number);
 
         A_DELETE_COPY(SpellShortcutContainer)
 
@@ -50,32 +51,32 @@ class SpellShortcutContainer final : public ShortcutContainer
         /**
          * Draws the items.
          */
-        void draw(gcn::Graphics *graphics) override final;
+        void draw(Graphics *graphics) override final;
 
         /**
          * Handles mouse when dragged.
          */
-        void mouseDragged(gcn::MouseEvent &event) override final;
+        void mouseDragged(MouseEvent &event) override final;
 
         /**
          * Handles mouse when pressed.
          */
-        void mousePressed(gcn::MouseEvent &event) override final;
+        void mousePressed(MouseEvent &event) override final;
 
         /**
          * Handles mouse release.
          */
-        void mouseReleased(gcn::MouseEvent &event) override final;
+        void mouseReleased(MouseEvent &event) override final;
 
-        void widgetHidden(const gcn::Event &event) override final;
+        void widgetHidden(const Event &event) override final;
 
-        void mouseExited(gcn::MouseEvent &event) override final;
+        void mouseExited(MouseEvent &event) override final;
 
-        void mouseMoved(gcn::MouseEvent &event) override final;
+        void mouseMoved(MouseEvent &event) override final;
 
         void setWidget2(const Widget2 *const widget) override final;
 
-        int getItemByIndex(const int index);
+        int getItemByIndex(const int index) const A_WARN_UNUSED;
 
     private:
         SpellPopup *mSpellPopup;
