@@ -1,5 +1,5 @@
 /*
- *  Custom keyboard shortcuts configuration
+ *  The ManaPlus Client
  *  Copyright (C) 2007  Joshua Langley <joshlangley@optusnet.com.au>
  *  Copyright (C) 2009  The Mana World Development Team
  *  Copyright (C) 2009-2010  The Mana Developers
@@ -28,68 +28,25 @@
 #include "input/inputmanager.h"
 #include "input/keyboardconfig.h"
 
-#include "gui/setupactiondata.h"
+#include "gui/gui.h"
 
 #include "gui/windows/okdialog.h"
 
 #include "gui/widgets/button.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/layouthelper.h"
 #include "gui/widgets/listbox.h"
 #include "gui/widgets/scrollarea.h"
 #include "gui/widgets/tabstrip.h"
 
-#include <guichan/listmodel.hpp>
+#include "gui/models/keylistmodel.h"
+
+#include "utils/delete2.h"
 
 #include "debug.h"
 
-static int selectedData = 0;
 static const int setupGroups = 9;
 
-/**
- * The list model for key function list.
- *
- * \ingroup Interface
- */
-class KeyListModel final : public gcn::ListModel
-{
-    public:
-        KeyListModel() :
-            mDataNum(0),
-            mSize(0)
-        {
-        }
-
-        A_DELETE_COPY(KeyListModel)
-
-        /**
-         * Returns the number of elements in container.
-         */
-        int getNumberOfElements() override final
-        { return mSize; }
-
-        /**
-         * Returns element from container.
-         */
-        std::string getElementAt(int i) override final
-        { return setupActionData[selectedData][i].text; }
-
-        /**
-         * Sets element from container.
-         */
-        void setElementAt(const int i, const std::string &caption)
-        { setupActionData[selectedData][i].text = caption; }
-
-        void setSize(const int size)
-        { mSize = size; }
-
-        void setDataNum(const int num)
-        { mDataNum = num; }
-
-    private:
-        int mDataNum;
-        int mSize;
-};
-
 Setup_Input::Setup_Input(const Widget2 *const widget) :
     SetupTab(widget),
     mKeyListModel(new KeyListModel),
@@ -103,7 +60,8 @@ Setup_Input::Setup_Input(const Widget2 *const widget) :
     // TRANSLATORS: button in input settings tab
     mResetKeysButton(new Button(this, _("Reset all keys"), "resetkeys", this)),
     mTabs(new TabStrip(this, config.getIntValue("fontSize") + 10)),
-    mScrollArea(new ScrollArea(mKeyList, true, "setup_input_background.xml")),
+    mScrollArea(new ScrollArea(this, mKeyList,
+        true, "setup_input_background.xml")),
     mKeySetting(false),
     mActionDataSize(new int [9])
 {
@@ -112,7 +70,7 @@ Setup_Input::Setup_Input(const Widget2 *const widget) :
     // TRANSLATORS: setting tab name
     setName(_("Input"));
 
-    selectedData = 0;
+    mKeyListModel->setSelectedData(0);
 
     for (int f = 0; f < setupGroups; f ++)
     {
@@ -125,7 +83,7 @@ Setup_Input::Setup_Input(const Widget2 *const widget) :
     mKeyListModel->setSize(mActionDataSize[0]);
     refreshKeys();
     if (gui)
-        mKeyList->setFont(reinterpret_cast<gcn::Font*>(gui->getHelpFont()));
+        mKeyList->setFont(gui->getHelpFont());
     mKeyList->addActionListener(this);
 
     mScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
@@ -162,26 +120,19 @@ Setup_Input::Setup_Input(const Widget2 *const widget) :
     if (config.getIntValue("screenwidth") >= 730)
         width += 100;
 
-    setDimension(gcn::Rectangle(0, 0, width, 350));
+    setDimension(Rect(0, 0, width, 350));
 }
 
 Setup_Input::~Setup_Input()
 {
-    delete mKeyList;
-    mKeyList = nullptr;
-    delete mKeyListModel;
-    mKeyListModel = nullptr;
-
-    delete mAssignKeyButton;
-    mAssignKeyButton = nullptr;
-    delete mUnassignKeyButton;
-    mUnassignKeyButton = nullptr;
-    delete mResetKeysButton;
-    mResetKeysButton = nullptr;
+    delete2(mKeyList);
+    delete2(mKeyListModel);
+    delete2(mAssignKeyButton);
+    delete2(mUnassignKeyButton);
+    delete2(mResetKeysButton);
     delete [] mActionDataSize;
     mActionDataSize = nullptr;
-    delete mScrollArea;
-    mScrollArea = nullptr;
+    delete2(mScrollArea);
 }
 
 void Setup_Input::apply()
@@ -199,7 +150,7 @@ void Setup_Input::apply()
             // TRANSLATORS: input settings error
             strprintf(_("Conflict \"%s\" and \"%s\" keys. "
             "Resolve them, or gameplay may result in strange behaviour."),
-            gettext(str1.c_str()), gettext(str2.c_str())), DIALOG_ERROR);
+            gettext(str1.c_str()), gettext(str2.c_str())), DialogType::ERROR);
     }
     keyboard.setEnabled(true);
     inputManager.store();
@@ -213,9 +164,10 @@ void Setup_Input::cancel()
     refreshKeys();
 }
 
-void Setup_Input::action(const gcn::ActionEvent &event)
+void Setup_Input::action(const ActionEvent &event)
 {
-    const std::string id = event.getId();
+    const std::string &id = event.getId();
+    const int selectedData = mKeyListModel->getSelectedData();
 
     if (event.getSource() == mKeyList)
     {
@@ -225,7 +177,7 @@ void Setup_Input::action(const gcn::ActionEvent &event)
             if (i >= 0 && i < mActionDataSize[selectedData])
             {
                 if (setupActionData[selectedData][i].actionId
-                    == static_cast<int>(Input::KEY_NO_VALUE))
+                    == static_cast<int>(InputAction::NO_VALUE))
                 {
                     mAssignKeyButton->setEnabled(false);
                     mUnassignKeyButton->setEnabled(false);
@@ -263,14 +215,15 @@ void Setup_Input::action(const gcn::ActionEvent &event)
             inputManager.setNewKeyIndex(ik);
             refreshAssignedKey(mKeyList->getSelected());
             inputManager.unassignKey();
-            inputManager.setNewKeyIndex(static_cast<int>(Input::KEY_NO_VALUE));
+            inputManager.setNewKeyIndex(static_cast<int>(
+                InputAction::NO_VALUE));
         }
         mAssignKeyButton->setEnabled(true);
     }
     else if (id == "resetkeys")
     {
         inputManager.resetKeys();
-        inputManager.update();
+        InputManager::update();
         refreshKeys();
     }
     else if (id == "default")
@@ -296,7 +249,7 @@ void Setup_Input::action(const gcn::ActionEvent &event)
         }
         if (pages[k] && str + pages[k] == id)
         {
-            selectedData = k;
+            mKeyListModel->setSelectedData(k);
             mKeyListModel->setSize(mActionDataSize[k]);
             refreshKeys();
             mKeyList->setSelected(0);
@@ -306,8 +259,9 @@ void Setup_Input::action(const gcn::ActionEvent &event)
 
 void Setup_Input::refreshAssignedKey(const int index)
 {
+    const int selectedData = mKeyListModel->getSelectedData();
     const SetupActionData &key = setupActionData[selectedData][index];
-    if (key.actionId == static_cast<int>(Input::KEY_NO_VALUE))
+    if (key.actionId == static_cast<int>(InputAction::NO_VALUE))
     {
         const std::string str(" \342\200\225\342\200\225\342\200\225"
             "\342\200\225\342\200\225 ");
@@ -338,6 +292,7 @@ void Setup_Input::newKeyCallback(const int index)
 
 int Setup_Input::keyToSetupData(const int index) const
 {
+    const int selectedData = mKeyListModel->getSelectedData();
     for (int i = 0; i < mActionDataSize[selectedData]; i++)
     {
         const SetupActionData &key = setupActionData[selectedData][i];
@@ -364,6 +319,7 @@ std::string Setup_Input::keyToString(const int index) const
 
 void Setup_Input::refreshKeys()
 {
+    const int selectedData = mKeyListModel->getSelectedData();
     for (int i = 0; i < mActionDataSize[selectedData]; i++)
         refreshAssignedKey(i);
 }
@@ -373,13 +329,14 @@ void Setup_Input::keyUnresolved()
     if (mKeySetting)
     {
         newKeyCallback(inputManager.getNewKeyIndex());
-        inputManager.setNewKeyIndex(static_cast<int>(Input::KEY_NO_VALUE));
+        inputManager.setNewKeyIndex(static_cast<int>(InputAction::NO_VALUE));
     }
 }
 
 void Setup_Input::fixTranslation(SetupActionData *const actionDatas,
-                                 const int actionStart, const int actionEnd,
-                                 const std::string &text) const
+                                 const int actionStart,
+                                 const int actionEnd,
+                                 const std::string &text)
 {
     int k = 0;
 
@@ -397,19 +354,19 @@ void Setup_Input::fixTranslation(SetupActionData *const actionDatas,
     }
 }
 
-void Setup_Input::fixTranslations() const
+void Setup_Input::fixTranslations()
 {
-    fixTranslation(setupActionData1, static_cast<int>(Input::KEY_SHORTCUT_1),
-        static_cast<int>(Input::KEY_SHORTCUT_20), "Item Shortcut %d");
+    fixTranslation(setupActionData1, static_cast<int>(InputAction::SHORTCUT_1),
+        static_cast<int>(InputAction::SHORTCUT_20), "Item Shortcut %d");
 
-    fixTranslation(setupActionData3, static_cast<int>(Input::KEY_EMOTE_1),
-        static_cast<int>(Input::KEY_EMOTE_48), "Emote Shortcut %d");
+    fixTranslation(setupActionData3, static_cast<int>(InputAction::EMOTE_1),
+        static_cast<int>(InputAction::EMOTE_48), "Emote Shortcut %d");
 
-    fixTranslation(setupActionData4, static_cast<int>(Input::KEY_OUTFIT_1),
-        static_cast<int>(Input::KEY_OUTFIT_48), "Outfit Shortcut %d");
+    fixTranslation(setupActionData4, static_cast<int>(InputAction::OUTFIT_1),
+        static_cast<int>(InputAction::OUTFIT_48), "Outfit Shortcut %d");
 
     fixTranslation(setupActionData7, static_cast<int>(
-        Input::KEY_MOVE_TO_POINT_1),
-        static_cast<int>(Input::KEY_MOVE_TO_POINT_48),
+        InputAction::MOVE_TO_POINT_1),
+        static_cast<int>(InputAction::MOVE_TO_POINT_48),
         "Move to point Shortcut %d");
 }
