@@ -22,14 +22,18 @@
 
 #include "gui/widgets/tabs/setup_other.h"
 
+#include "gui/models/namesmodel.h"
+
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/layouthelper.h"
-#include "gui/widgets/namesmodel.h"
 #include "gui/widgets/setupitem.h"
 #include "gui/widgets/scrollarea.h"
 
 #include "configuration.h"
-#include "map.h"
 
+#include "resources/map/mapconsts.h"
+
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include "debug.h"
@@ -180,6 +184,10 @@ Setup_Other::Setup_Other(const Widget2 *const widget) :
     new SetupItemCheckBox(_("Auto fix position"), "",
         "autofixPos", this, "autofixPosEvent");
 
+    // TRANSLATORS: settings option
+    new SetupItemCheckBox(_("Show server side position"), "",
+        "showserverpos", this, "showserverposEvent");
+
     // TRANSLATORS: settings option
     new SetupItemCheckBox(_("Attack while moving"), "",
         "attackMoving", this, "attackMovingEvent");
@@ -332,6 +340,10 @@ Setup_Other::Setup_Other(const Widget2 *const widget) :
     new SetupItemCheckBox(_("Enable server side attack"), "",
         "serverAttack", this, "serverAttackEvent");
 
+    // TRANSLATORS: settings option
+    new SetupItemCheckBox(_("Hide support page link on error"), "",
+        "hidesupport", this, "hidesupportEvent");
+
     // TRANSLATORS: settings option
     new SetupItemCheckBox(_("Enable double clicks"), "",
         "doubleClick", this, "doubleClickEvent");
@@ -374,7 +386,7 @@ Setup_Other::Setup_Other(const Widget2 *const widget) :
 #ifndef WIN32
     // TRANSLATORS: settings option
     new SetupItemTextField(_("Screenshot directory"), "",
-        "screenshotDirectory2", this, "screenshotDirectory2Event", true, true);
+        "screenshotDirectory3", this, "screenshotDirectory3Event", true, true);
 #endif
 
     // TRANSLATORS: settings option
@@ -390,17 +402,14 @@ Setup_Other::Setup_Other(const Widget2 *const widget) :
     new SetupItemDropDown(_("Screen density override"), "",
         "screenDensity", this, "screenDensityEvent", mDensityList, 100);
 
-    setDimension(gcn::Rectangle(0, 0, 550, 350));
+    setDimension(Rect(0, 0, 550, 350));
 }
 
 Setup_Other::~Setup_Other()
 {
-    delete mProxyTypeList;
-    mProxyTypeList = nullptr;
-    delete mShortcutsList;
-    mShortcutsList = nullptr;
-    delete mDensityList;
-    mDensityList = nullptr;
+    delete2(mProxyTypeList);
+    delete2(mShortcutsList);
+    delete2(mDensityList);
 }
 
 void Setup_Other::apply()
