@@ -22,11 +22,14 @@
 
 #include "gui/widgets/tabs/setup_perfomance.h"
 
+#include "gui/models/namesmodel.h"
+
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/layouthelper.h"
-#include "gui/widgets/namesmodel.h"
 #include "gui/widgets/scrollarea.h"
 #include "gui/widgets/setupitem.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include "debug.h"
@@ -103,8 +106,8 @@ Setup_Perfomance::Setup_Perfomance(const Widget2 *const widget) :
         "beingopacity", this, "beingopacityEvent");
 
     // TRANSLATORS: settings option
-    new SetupItemCheckBox(_("Enable reorder sprites."), "",
-        "enableReorderSprites", this, "enableReorderSpritesEvent");
+    new SetupItemCheckBox(_("Enable reorder sprites (need for mods support)."),
+        "", "enableReorderSprites", this, "enableReorderSpritesEvent");
 
 
     // TRANSLATORS: settings option
@@ -145,7 +148,7 @@ Setup_Perfomance::Setup_Perfomance(const Widget2 *const widget) :
 
     // TRANSLATORS: settings option
     new SetupItemCheckBox(_("Cache all sprites per map (can use "
-        "additinal memory)"), "", "uselonglivesprites", this,
+        "additional memory)"), "", "uselonglivesprites", this,
         "uselonglivespritesEvent");
 
     // TRANSLATORS: settings option
@@ -153,11 +156,10 @@ Setup_Perfomance::Setup_Perfomance(const Widget2 *const widget) :
         "", "uselonglivesounds", this,
         "uselonglivesoundsEvent");
 
-    setDimension(gcn::Rectangle(0, 0, 550, 350));
+    setDimension(Rect(0, 0, 550, 350));
 }
 
 Setup_Perfomance::~Setup_Perfomance()
 {
-    delete mTexturesList;
-    mTexturesList = nullptr;
+    delete2(mTexturesList);
 }
