@@ -22,21 +22,27 @@
 
 #include "gui/widgets/tabs/setup_theme.h"
 
+#include "gui/gui.h"
+#include "gui/themeinfo.h"
+
 #include "gui/windows/okdialog.h"
 
+#include "gui/models/fontsizechoicelistmodel.h"
+#include "gui/models/fontsmodel.h"
+#include "gui/models/langlistmodel.h"
+#include "gui/models/themesmodel.h"
+
 #include "gui/widgets/button.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/dropdown.h"
-#include "gui/widgets/extendedlistmodel.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layouthelper.h"
-#include "gui/widgets/namesmodel.h"
 
 #include "configuration.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
-#include "resources/resourcemanager.h"
-
 #include "debug.h"
 
 const char* ACTION_THEME = "theme";
@@ -51,180 +57,6 @@ const char* ACTION_JAPAN_FONT = "japanese font";
 const char* ACTION_CHINA_FONT = "chinese font";
 const char* ACTION_INFO = "info";
 
-class ThemesModel final : public NamesModel
-{
-public:
-    ThemesModel() :
-        NamesModel()
-    {
-        mNames.push_back(gettext("(default)"));
-        Theme::fillSkinsList(mNames);
-    }
-
-    ~ThemesModel()
-    { }
-};
-
-class FontsModel final : public NamesModel
-{
-public:
-    FontsModel() :
-        NamesModel()
-    { Theme::fillFontsList(mNames); }
-
-    ~FontsModel()
-    { }
-};
-
-const int maxFontSizes = 16;
-
-const char *SIZE_NAME[maxFontSizes] =
-{
-    // TRANSLATORS: font size
-    N_("Very small (8)"),
-    // TRANSLATORS: font size
-    N_("Very small (9)"),
-    // TRANSLATORS: font size
-    N_("Tiny (10)"),
-    // TRANSLATORS: font size
-    N_("Small (11)"),
-    // TRANSLATORS: font size
-    N_("Medium (12)"),
-    // TRANSLATORS: font size
-    N_("Normal (13)"),
-    // TRANSLATORS: font size
-    N_("Large (14)"),
-    // TRANSLATORS: font size
-    N_("Large (15)"),
-    // TRANSLATORS: font size
-    N_("Large (16)"),
-    // TRANSLATORS: font size
-    N_("Big (17)"),
-    // TRANSLATORS: font size
-    N_("Big (18)"),
-    // TRANSLATORS: font size
-    N_("Big (19)"),
-    // TRANSLATORS: font size
-    N_("Very big (20)"),
-    // TRANSLATORS: font size
-    N_("Very big (21)"),
-    // TRANSLATORS: font size
-    N_("Very big (22)"),
-    // TRANSLATORS: font size
-    N_("Huge (23)"),
-};
-
-class FontSizeChoiceListModel final : public gcn::ListModel
-{
-public:
-    ~FontSizeChoiceListModel()
-    { }
-
-    int getNumberOfElements() override final A_WARN_UNUSED
-    { return maxFontSizes; }
-
-    std::string getElementAt(int i) override final A_WARN_UNUSED
-    {
-        if (i >= getNumberOfElements() || i < 0)
-            return "???";
-
-        return gettext(SIZE_NAME[i]);
-    }
-};
-
-struct Language final
-{
-    std::string name;
-    std::string value;
-    std::string icon;
-};
-
-const int langs_count = 17;
-
-const Language LANG_NAME[langs_count] =
-{
-    // TRANSLATORS: language
-    {N_("(default)"), "", ""},
-    // TRANSLATORS: language
-    {N_("Chinese (China)"), "zh_CN", "cn.png"},
-    // TRANSLATORS: language
-    {N_("Czech"), "cs_CZ", "cz.png"},
-    // TRANSLATORS: language
-    {N_("English"), "C", "en.png"},
-    // TRANSLATORS: language
-    {N_("Finnish"), "fi_FI", "fi.png"},
-    // TRANSLATORS: language
-    {N_("French"), "fr_FR", "fr.png"},
-    // TRANSLATORS: language
-    {N_("German"), "de_DE", "de.png"},
-    // TRANSLATORS: language
-    {N_("Indonesian"), "id_ID", "id.png"},
-    // TRANSLATORS: language
-    {N_("Italian"), "it_IT", "it.png"},
-    // TRANSLATORS: language
-    {N_("Polish"), "pl_PL", "pl.png"},
-    // TRANSLATORS: language
-    {N_("Japanese"), "ja_JP", "jp.png"},
-    // TRANSLATORS: language
-    {N_("Dutch (Belgium/Flemish)"), "nl_BE", "nl_BE.png"},
-    // TRANSLATORS: language
-    {N_("Portuguese"), "pt_PT", "pt.png"},
-    // TRANSLATORS: language
-    {N_("Portuguese (Brazilian)"), "pt_BR", "pt_BR.png"},
-    // TRANSLATORS: language
-    {N_("Russian"), "ru_RU", "ru.png"},
-    // TRANSLATORS: language
-    {N_("Spanish (Castilian)"), "es_ES", "es.png"},
-    // TRANSLATORS: language
-    {N_("Turkish"), "tr_TR", "tr.png"}
-};
-
-class LangListModel final : public ExtendedListModel
-{
-public:
-    LangListModel()
-    {
-        ResourceManager *const resman = ResourceManager::getInstance();
-        for (int f = 0; f < langs_count; f ++)
-        {
-            mIcons[f] = resman->getImage("graphics/flags/"
-                + LANG_NAME[f].icon);
-        }
-    }
-
-    A_DELETE_COPY(LangListModel)
-
-    ~LangListModel()
-    {
-        for (int f = 0; f < langs_count; f ++)
-        {
-            Image *const img = mIcons[f];
-            if (img)
-                img->decRef();
-        }
-    }
-
-    int getNumberOfElements() override final A_WARN_UNUSED
-    { return langs_count; }
-
-    std::string getElementAt(int i) override final A_WARN_UNUSED
-    {
-        if (i >= getNumberOfElements() || i < 0)
-            return "???";
-
-        return gettext(LANG_NAME[i].name.c_str());
-    }
-
-    const Image *getImageAt(int i) override final A_WARN_UNUSED
-    {
-        if (i >= getNumberOfElements() || i < 0)
-            return nullptr;
-        return mIcons[i];
-    }
-
-    Image *mIcons[langs_count];
-};
-
 Setup_Theme::Setup_Theme(const Widget2 *const widget) :
     SetupTab(widget),
     // TRANSLATORS: theme settings label
@@ -387,31 +219,18 @@ Setup_Theme::Setup_Theme(const Widget2 *const widget) :
     else if (size > maxWidth)
         size = maxWidth;
 
-    setDimension(gcn::Rectangle(0, 0, size, 500));
+    setDimension(Rect(0, 0, size, 500));
 }
 
 Setup_Theme::~Setup_Theme()
 {
-    delete mInfo;
-    mInfo = nullptr;
-
-    delete mThemesModel;
-    mThemesModel = nullptr;
-
-    delete mFontsModel;
-    mFontsModel = nullptr;
-
-    delete mFontSizeListModel;
-    mFontSizeListModel = nullptr;
-
-    delete mNpcFontSizeListModel;
-    mNpcFontSizeListModel = nullptr;
-
-    delete mLangListModel;
-    mLangListModel = nullptr;
-
-    delete mInfo;
-    mInfo = nullptr;
+    delete2(mInfo);
+    delete2(mThemesModel);
+    delete2(mFontsModel);
+    delete2(mFontSizeListModel);
+    delete2(mNpcFontSizeListModel);
+    delete2(mLangListModel);
+    delete2(mInfo);
 }
 
 void Setup_Theme::updateInfo()
@@ -433,7 +252,7 @@ void Setup_Theme::updateInfo()
     mInfoButton->setEnabled(!mThemeInfo.empty());
 }
 
-void Setup_Theme::action(const gcn::ActionEvent &event)
+void Setup_Theme::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == ACTION_THEME)
@@ -487,7 +306,7 @@ void Setup_Theme::action(const gcn::ActionEvent &event)
     else if (eventId == ACTION_INFO)
     {
         // TRANSLATORS: theme info dialog header
-        new OkDialog(_("Theme info"), mThemeInfo, DIALOG_OK,
+        new OkDialog(_("Theme info"), mThemeInfo, DialogType::OK,
             false, true, nullptr, 600);
     }
 }
