@@ -22,16 +22,18 @@
 
 #include "gui/widgets/tabs/setup_video.h"
 
-#include "client.h"
 #include "configuration.h"
 
 #include "graphicsmanager.h"
 
+#include "gui/windowmanager.h"
+
 #include "gui/windows/okdialog.h"
 #include "gui/windows/textdialog.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/checkbox.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layouthelper.h"
 #include "gui/widgets/listbox.h"
@@ -41,140 +43,21 @@
 
 #include "render/rendererslistsdl.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
-#include "utils/sdlhelper.h"
 
 #include "test/testmain.h"
 
-#include <guichan/listmodel.hpp>
-
-#include <algorithm>
+#include "gui/models/modelistmodel.h"
+#include "gui/models/opengllistmodel.h"
 
 #include "debug.h"
 
 extern Graphics *mainGraphics;
 
-class ModeListModel final : public gcn::ListModel
-{
-    public:
-        ModeListModel();
-
-        A_DELETE_COPY(ModeListModel)
-
-        ~ModeListModel()
-        { }
-
-        /**
-         * Returns the number of elements in container.
-         */
-        int getNumberOfElements() override final
-        { return static_cast<int>(mVideoModes.size()); }
-
-        /**
-         * Returns element from container.
-         */
-        std::string getElementAt(int i)
-        { return mVideoModes[i]; }
-
-        /**
-         * Returns the index corresponding to the given video mode.
-         * E.g.: "800x600".
-         * or -1 if not found.
-         */
-        int getIndexOf(const std::string &widthXHeightMode);
-
-    private:
-        void addCustomMode(const std::string &mode);
-
-        StringVect mVideoModes;
-};
-
-#ifndef ANDROID
-static bool modeSorter(const std::string &mode1, const std::string &mode2)
-{
-    const int width1 = atoi(mode1.substr(0, mode1.find("x")).c_str());
-    const int height1 = atoi(mode1.substr(mode1.find("x") + 1).c_str());
-    if (!width1 || !height1)
-        return false;
-
-    const int width2 = atoi(mode2.substr(0, mode2.find("x")).c_str());
-    const int height2 = atoi(mode2.substr(mode2.find("x") + 1).c_str());
-    if (!width2 || !height2)
-        return false;
-    if (width1 != width2)
-        return width1 < width2;
-
-    if (height1 != height2)
-        return height1 < height2;
-
-    return false;
-}
-#endif
-
-ModeListModel::ModeListModel() :
-    mVideoModes()
-{
-    SDL::getAllVideoModes(mVideoModes);
-#ifndef ANDROID
-    addCustomMode("640x480");
-    addCustomMode("800x600");
-    addCustomMode("1024x768");
-    addCustomMode("1280x1024");
-    addCustomMode("1400x900");
-    addCustomMode("1500x990");
-    addCustomMode(toString(mainGraphics->mWidth).append("x")
-        .append(toString(mainGraphics->mHeight)));
-
-    std::sort(mVideoModes.begin(), mVideoModes.end(), &modeSorter);
-    mVideoModes.push_back("custom");
-#endif
-}
-
-void ModeListModel::addCustomMode(const std::string &mode)
-{
-    StringVectCIter it = mVideoModes.begin();
-    const StringVectCIter it_end = mVideoModes.end();
-    while (it != it_end)
-    {
-        if (*it == mode)
-            return;
-        ++ it;
-    }
-    mVideoModes.push_back(mode);
-}
-
-int ModeListModel::getIndexOf(const std::string &widthXHeightMode)
-{
-    std::string currentMode("");
-    for (int i = 0; i < getNumberOfElements(); i++)
-    {
-        currentMode = getElementAt(i);
-        if (currentMode == widthXHeightMode)
-            return i;
-    }
-    return -1;
-}
-
-class OpenGLListModel final : public gcn::ListModel
-{
-public:
-    ~OpenGLListModel()
-    { }
-
-    int getNumberOfElements() override final
-    { return renderModesListSize; }
-
-    std::string getElementAt(int i) override final
-    {
-        if (i >= getNumberOfElements() || i < 0)
-            return "???";
-        return gettext(OPENGL_NAME[i]);
-    }
-};
-
 Setup_Video::Setup_Video(const Widget2 *const widget) :
     SetupTab(widget),
-    gcn::KeyListener(),
+    KeyListener(),
     mFullScreenEnabled(config.getBoolValue("screen")),
     mOpenGLEnabled(intToRenderType(config.getIntValue("opengl"))),
     mFps(config.getIntValue("fpslimit")),
@@ -187,9 +70,9 @@ Setup_Video::Setup_Video(const Widget2 *const widget) :
     mOpenGLDropDown(new DropDown(widget, mOpenGLListModel)),
     // TRANSLATORS: video settings checkbox
     mFpsCheckBox(new CheckBox(this, _("FPS limit:"))),
-    mFpsSlider(new Slider(2, 160)),
+    mFpsSlider(new Slider(this, 2.0, 160.0, 1.0)),
     mFpsLabel(new Label(this)),
-    mAltFpsSlider(new Slider(2, 160)),
+    mAltFpsSlider(new Slider(this, 2.0, 160.0, 1.0)),
     // TRANSLATORS: video settings label
     mAltFpsLabel(new Label(this, _("Alt FPS limit: "))),
 #if !defined(ANDROID) && !defined(__APPLE__)
@@ -220,10 +103,10 @@ Setup_Video::Setup_Video(const Widget2 *const widget) :
     // TRANSLATORS: video settings tab name
     setName(_("Video"));
 
-    ScrollArea *const scrollArea = new ScrollArea(mModeList,
+    ScrollArea *const scrollArea = new ScrollArea(this, mModeList,
         true, "setup_video_background.xml");
     scrollArea->setWidth(150);
-    scrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    scrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
 
     mOpenGLDropDown->setSelected(renderToIndex[mOpenGLEnabled]);
 
@@ -237,14 +120,15 @@ Setup_Video::Setup_Video(const Widget2 *const widget) :
                              ? toString(mAltFps) : _("None")));
     mAltFpsLabel->setWidth(150);
     mFpsSlider->setEnabled(mFps > 0);
-    mFpsSlider->setValue2(mFps);
+    mFpsSlider->setValue(mFps);
     mAltFpsSlider->setEnabled(mAltFps > 0);
-    mAltFpsSlider->setValue2(mAltFps);
+    mAltFpsSlider->setValue(mAltFps);
     mFpsCheckBox->setSelected(mFps > 0);
 
     // Pre-select the current video mode.
-    const std::string videoMode = toString(mainGraphics->mWidth).append("x")
-        .append(toString(mainGraphics->mHeight));
+    const std::string videoMode = toString(
+        mainGraphics->mActualWidth).append("x").append(
+        toString(mainGraphics->mActualHeight));
     mModeList->setSelected(mModeListModel->getIndexOf(videoMode));
 
     mModeList->setActionEventId("videomode");
@@ -299,19 +183,15 @@ Setup_Video::Setup_Video(const Widget2 *const widget) :
     if (config.getIntValue("screenwidth") >= 730)
         width += 100;
 
-    setDimension(gcn::Rectangle(0, 0, width, 300));
+    setDimension(Rect(0, 0, width, 300));
 }
 
 Setup_Video::~Setup_Video()
 {
-    delete mModeListModel;
-    mModeListModel = nullptr;
-    delete mModeList;
-    mModeList = nullptr;
-    delete mOpenGLListModel;
-    mOpenGLListModel = nullptr;
-    delete mDialog;
-    mDialog = nullptr;
+    delete2(mModeListModel);
+    delete2(mModeList);
+    delete2(mOpenGLListModel);
+    delete2(mDialog);
 }
 
 void Setup_Video::apply()
@@ -414,9 +294,9 @@ void Setup_Video::cancel()
     mOpenGLDropDown->setSelected(renderToIndex[mOpenGLEnabled]);
     mCustomCursorCheckBox->setSelected(mCustomCursorEnabled);
     mFpsSlider->setEnabled(mFps > 0);
-    mFpsSlider->setValue2(mFps);
+    mFpsSlider->setValue(mFps);
     mAltFpsSlider->setEnabled(mAltFps > 0);
-    mAltFpsSlider->setValue2(mAltFps);
+    mAltFpsSlider->setValue(mAltFps);
     mFpsLabel->setCaption(mFpsCheckBox->isSelected()
     // TRANSLATORS: video settings label
                           ? toString(mFps) : _("None"));
@@ -428,11 +308,11 @@ void Setup_Video::cancel()
     config.setValue("screen", mFullScreenEnabled);
 
     // Set back to the current video mode.
-    std::string videoMode = toString(mainGraphics->mWidth).append("x")
-        .append(toString(mainGraphics->mHeight));
+    std::string videoMode = toString(mainGraphics->mActualWidth).append("x")
+        .append(toString(mainGraphics->mActualHeight));
     mModeList->setSelected(mModeListModel->getIndexOf(videoMode));
-    config.setValue("screenwidth", mainGraphics->mWidth);
-    config.setValue("screenheight", mainGraphics->mHeight);
+    config.setValue("screenwidth", mainGraphics->mActualWidth);
+    config.setValue("screenheight", mainGraphics->mActualHeight);
 
     config.setValue("customcursor", mCustomCursorEnabled);
     config.setValue("opengl", static_cast<int>(mOpenGLEnabled));
@@ -440,7 +320,7 @@ void Setup_Video::cancel()
     config.setValue("noframe", mNoFrame);
 }
 
-void Setup_Video::action(const gcn::ActionEvent &event)
+void Setup_Video::action(const ActionEvent &event)
 {
     const std::string &id = event.getId();
 
@@ -474,18 +354,19 @@ void Setup_Video::action(const gcn::ActionEvent &event)
         if (!width || !height)
             return;
 
-        if (width != mainGraphics->mWidth || height != mainGraphics->mHeight)
+        if (width != mainGraphics->mActualWidth
+            || height != mainGraphics->mActualHeight)
         {
 #if defined(WIN32) || defined(__APPLE__) || defined(ANDROID)
             if (intToRenderType(config.getIntValue("opengl"))
                 == RENDER_SOFTWARE)
             {
-                client->resizeVideo(width, height);
+                WindowManager::resizeVideo(width, height, false);
             }
             else
             {
-                if (width < mainGraphics->mWidth
-                    || height < mainGraphics->mHeight)
+                if (width < mainGraphics->mActualWidth
+                    || height < mainGraphics->mActualHeight)
                 {
                     // TRANSLATORS: video settings warning
                     new OkDialog(_("Screen Resolution Changed"),
@@ -505,13 +386,13 @@ void Setup_Video::action(const gcn::ActionEvent &event)
             }
 #else
             mainGraphics->setWindowSize(width, height);
-            client->resizeVideo(width, height);
+            WindowManager::resizeVideo(width, height, false);
 #endif
         }
 
         config.setValue("oldscreen", config.getBoolValue("screen"));
-        config.setValue("oldscreenwidth", mainGraphics->mWidth);
-        config.setValue("oldscreenheight", mainGraphics->mHeight);
+        config.setValue("oldscreenwidth", mainGraphics->mActualWidth);
+        config.setValue("oldscreenheight", mainGraphics->mActualHeight);
         config.setValue("screenwidth", width);
         config.setValue("screenheight", height);
     }
@@ -536,7 +417,7 @@ void Setup_Video::action(const gcn::ActionEvent &event)
 
         mFpsLabel->setCaption(text);
         mFpsSlider->setEnabled(mFps > 0);
-        mFpsSlider->setValue2(mFps);
+        mFpsSlider->setValue(mFps);
     }
     else if (id == "altfpslimitslider")
     {
@@ -550,7 +431,7 @@ void Setup_Video::action(const gcn::ActionEvent &event)
         // TRANSLATORS: video settings label
         mAltFpsLabel->setCaption(_("Alt FPS limit: ") + text);
         mAltFpsSlider->setEnabled(mAltFps > 0);
-        mAltFpsSlider->setValue2(mAltFps);
+        mAltFpsSlider->setValue(mAltFps);
     }
     else if (id == "enableresize")
     {
@@ -566,12 +447,16 @@ void Setup_Video::action(const gcn::ActionEvent &event)
         TestMain *test = graphicsManager.startDetection();
         if (test)
         {
-            const int val = test->getConfig().getValueInt("opengl", -1);
+            Configuration &conf = test->getConfig();
+            const int val = conf.getValueInt("opengl", -1);
             if (val >= 0 && static_cast<unsigned int>(val)
                 < sizeof(renderToIndex) / sizeof(int))
             {
                 mOpenGLDropDown->setSelected(renderToIndex[val]);
             }
+            config.setValue("textureSize",
+                conf.getValue("textureSize", "1024,1024,1024,1024,1024,1024"));
+            config.setValue("testInfo", conf.getValue("testInfo", ""));
             delete test;
         }
     }
