@@ -20,27 +20,71 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/*      _______   __   __   __   ______   __   __   _______   __   __
+ *     / _____/\ / /\ / /\ / /\ / ____/\ / /\ / /\ / ___  /\ /  |\/ /\
+ *    / /\____\// / // / // / // /\___\// /_// / // /\_/ / // , |/ / /
+ *   / / /__   / / // / // / // / /    / ___  / // ___  / // /| ' / /
+ *  / /_// /\ / /_// / // / // /_/_   / / // / // /\_/ / // / |  / /
+ * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /
+ * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/
+ *
+ * Copyright (c) 2004 - 2008 Olof Naessén and Per Larsson
+ *
+ *
+ * Per Larsson a.k.a finalman
+ * Olof Naessén a.k.a jansem/yakslem
+ *
+ * Visit: http://guichan.sourceforge.net
+ *
+ * License: (BSD)
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ * 3. Neither the name of Guichan nor the names of its contributors may
+ *    be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+ * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
 #include "gui/widgets/textfield.h"
 
-#include "client.h"
+#include "settings.h"
 
 #include "input/inputmanager.h"
 
-#include "input/keyevent.h"
+#include "gui/gui.h"
+#include "gui/popupmanager.h"
+#include "gui/skin.h"
+#ifdef ANDROID
+#include "gui/windowmanager.h"
+#endif
 
-#include "gui/sdlinput.h"
-
-#include "gui/viewport.h"
-
-#include "gui/popups/popupmenu.h"
+#include "gui/fonts/font.h"
 
 #include "resources/image.h"
+#include "resources/imagerect.h"
 
 #include "utils/copynpaste.h"
+#include "utils/stringutils.h"
 #include "utils/timer.h"
 
-#include <guichan/font.hpp>
-
 #undef DELETE  // Win32 compatibility hack
 
 #include "debug.h"
@@ -53,22 +97,31 @@ ImageRect TextField::skin;
 TextField::TextField(const Widget2 *restrict const widget,
                      const std::string &restrict text,
                      const bool loseFocusOnTab,
-                     gcn::ActionListener *restrict const listener,
+                     ActionListener *restrict const listener,
                      const std::string &restrict eventId,
-                     const bool sendAlwaysEvents):
-    gcn::TextField(text),
-    gcn::FocusListener(),
-    Widget2(widget),
-    mSendAlwaysEvents(sendAlwaysEvents),
+                     const bool sendAlwaysEvents) :
+    Widget(widget),
+    FocusListener(),
+    KeyListener(),
+    MouseListener(),
+    mText(text),
+    mCaretPosition(0),
+    mXScroll(0),
     mCaretColor(&getThemeColor(Theme::CARET)),
-    mPopupMenu(nullptr),
     mMinimum(0),
     mMaximum(0),
     mLastEventPaste(false),
     mPadding(1),
     mNumeric(false),
-    mLoseFocusOnTab(loseFocusOnTab)
+    mLoseFocusOnTab(loseFocusOnTab),
+    mAllowSpecialActions(true),
+    mSendAlwaysEvents(sendAlwaysEvents)
 {
+    mAllowLogic = false;
+    setFocusable(true);
+    addMouseListener(this);
+    addKeyListener(this);
+
     setFrameSize(2);
     mForegroundColor = getThemeColor(Theme::TEXTFIELD);
     mForegroundColor2 = getThemeColor(Theme::TEXTFIELD_OUTLINE);
@@ -77,7 +130,6 @@ TextField::TextField(const Widget2 *restrict const widget,
 
     if (instances == 0)
     {
-        Theme *const theme = Theme::instance();
         if (theme)
         {
             mSkin = theme->loadSkinRect(skin, "textfield.xml",
@@ -106,25 +158,21 @@ TextField::~TextField()
     if (gui)
         gui->removeDragged(this);
 
-    delete mPopupMenu;
-    mPopupMenu = nullptr;
-
     instances--;
     if (instances == 0)
     {
-        Theme *const theme = Theme::instance();
         if (theme)
         {
             theme->unload(mSkin);
-            theme->unloadRect(skin);
+            Theme::unloadRect(skin);
         }
     }
 }
 
 void TextField::updateAlpha()
 {
-    const float alpha = std::max(client->getGuiAlpha(),
-        Theme::instance()->getMinimumOpacity());
+    const float alpha = std::max(settings.guiAlpha,
+        theme->getMinimumOpacity());
 
     if (alpha != mAlpha)
     {
@@ -137,30 +185,32 @@ void TextField::updateAlpha()
     }
 }
 
-void TextField::draw(gcn::Graphics *graphics)
+void TextField::draw(Graphics *graphics)
 {
     BLOCK_START("TextField::draw")
     updateAlpha();
 
-    gcn::Font *const font = getFont();
+    Font *const font = getFont();
     if (isFocused())
     {
         drawCaret(graphics,
             font->getWidth(mText.substr(0, mCaretPosition)) - mXScroll);
     }
 
-    static_cast<Graphics*>(graphics)->setColorAll(
-        mForegroundColor, mForegroundColor2);
+    graphics->setColorAll(mForegroundColor, mForegroundColor2);
     font->drawString(graphics, mText, mPadding - mXScroll, mPadding);
     BLOCK_END("TextField::draw")
 }
 
-void TextField::drawFrame(gcn::Graphics *graphics)
+void TextField::drawFrame(Graphics *graphics)
 {
     BLOCK_START("TextField::drawFrame")
     const int bs = 2 * mFrameSize;
-    static_cast<Graphics*>(graphics)->drawImageRect(0, 0,
-        mDimension.width + bs, mDimension.height + bs, skin);
+    graphics->drawImageRect(0,
+        0,
+        mDimension.width + bs,
+        mDimension.height + bs,
+        skin);
     BLOCK_END("TextField::drawFrame")
 }
 
@@ -196,16 +246,16 @@ int TextField::getValue() const
     return value;
 }
 
-void TextField::keyPressed(gcn::KeyEvent &keyEvent)
+void TextField::keyPressed(KeyEvent &event)
 {
-    const int val = keyEvent.getKey().getValue();
+    const int val = event.getKey().getValue();
 #ifdef USE_SDL2
     if (val == Key::TEXTINPUT)
     {
-        std::string str = static_cast<KeyEvent*>(&keyEvent)->getText();
+        std::string str = event.getText();
         mText.insert(mCaretPosition, str);
-        mCaretPosition += str.size();
-        keyEvent.consume();
+        mCaretPosition += static_cast<unsigned int>(str.size());
+        event.consume();
         fixScroll();
         if (mSendAlwaysEvents)
             distributeActionEvent();
@@ -224,7 +274,7 @@ void TextField::keyPressed(gcn::KeyEvent &keyEvent)
                 buf[1] = 0;
                 mText.insert(mCaretPosition, std::string(buf));
                 mCaretPosition += 1;
-                keyEvent.consume();
+                event.consume();
                 fixScroll();
                 if (mSendAlwaysEvents)
                     distributeActionEvent();
@@ -252,11 +302,11 @@ void TextField::keyPressed(gcn::KeyEvent &keyEvent)
             }
 
             if (len > 1)
-                buf[0] |= static_cast<char>(255 << (8 - len));
+                buf[0] |= static_cast<char>(255U << (8 - len));
 
             mText.insert(mCaretPosition, std::string(buf, buf + len));
             mCaretPosition += len;
-            keyEvent.consume();
+            event.consume();
             fixScroll();
             if (mSendAlwaysEvents)
                 distributeActionEvent();
@@ -272,47 +322,29 @@ void TextField::keyPressed(gcn::KeyEvent &keyEvent)
         mLastEventPaste = 0;
 
     bool consumed(false);
-    handleSDLKeys(val, consumed);
-
-    if (consumed)
-    {
-        if (mSendAlwaysEvents)
-            distributeActionEvent();
-
-        keyEvent.consume();
-        fixScroll();
-        return;
-    }
 #endif
 
-    if (consumed)
+    const int action = event.getActionId();
+    if (!inputManager.isActionActive(static_cast<int>(
+        InputAction::GUI_CTRL)))
     {
-        keyEvent.consume();
+        if (!handleNormalKeys(action, consumed))
+        {
+            if (consumed)
+                event.consume();
+            return;
+        }
     }
     else
     {
-        const int action = static_cast<KeyEvent*>(&keyEvent)->getActionId();
-        if (!inputManager.isActionActive(static_cast<int>(
-            Input::KEY_GUI_CTRL)))
-        {
-            if (!handleNormalKeys(action, consumed))
-            {
-                if (consumed)
-                    keyEvent.consume();
-                return;
-            }
-        }
-        else
-        {
-            handleCtrlKeys(action, consumed);
-        }
+        handleCtrlKeys(action, consumed);
     }
 
     if (mSendAlwaysEvents)
         distributeActionEvent();
 
     if (consumed)
-        keyEvent.consume();
+        event.consume();
     fixScroll();
 }
 
@@ -320,7 +352,7 @@ bool TextField::handleNormalKeys(const int action, bool &consumed)
 {
     switch (action)
     {
-        case Input::KEY_GUI_LEFT:
+        case InputAction::GUI_LEFT:
         {
             consumed = true;
             while (mCaretPosition > 0)
@@ -332,7 +364,7 @@ bool TextField::handleNormalKeys(const int action, bool &consumed)
             break;
         }
 
-        case Input::KEY_GUI_RIGHT:
+        case InputAction::GUI_RIGHT:
         {
             consumed = true;
             const unsigned sz = static_cast<unsigned>(mText.size());
@@ -348,7 +380,7 @@ bool TextField::handleNormalKeys(const int action, bool &consumed)
             break;
         }
 
-        case Input::KEY_GUI_DELETE:
+        case InputAction::GUI_DELETE:
         {
             consumed = true;
             unsigned sz = static_cast<unsigned>(mText.size());
@@ -365,28 +397,28 @@ bool TextField::handleNormalKeys(const int action, bool &consumed)
             break;
         }
 
-        case Input::KEY_GUI_BACKSPACE:
+        case InputAction::GUI_BACKSPACE:
             consumed = true;
             deleteCharLeft(mText, &mCaretPosition);
             break;
 
-        case Input::KEY_GUI_SELECT2:
+        case InputAction::GUI_SELECT2:
             distributeActionEvent();
             consumed = true;
             fixScroll();
             return false;
 
-        case Input::KEY_GUI_HOME:
+        case InputAction::GUI_HOME:
             mCaretPosition = 0;
             consumed = true;
             break;
 
-        case Input::KEY_GUI_END:
+        case InputAction::GUI_END:
             mCaretPosition = static_cast<unsigned>(mText.size());
             consumed = true;
             break;
 
-        case Input::KEY_GUI_TAB:
+        case InputAction::GUI_TAB:
             if (mLoseFocusOnTab)
                 return false;
             consumed = true;
@@ -402,155 +434,93 @@ void TextField::handleCtrlKeys(const int action, bool &consumed)
 {
     switch (action)
     {
-        case Input::KEY_GUI_LEFT:
+        case InputAction::GUI_LEFT:
         {
             moveCaretWordBack();
             consumed = true;
             break;
         }
-        case Input::KEY_GUI_RIGHT:
+        case InputAction::GUI_RIGHT:
         {
             moveCaretWordForward();
             consumed = true;
             break;
         }
-#ifdef USE_SDL2
-        case Input::KEY_GUI_B:
+        case InputAction::GUI_B:
         {
-            moveCaretBack();
-            consumed = true;
+            if (mAllowSpecialActions)
+            {
+                moveCaretBack();
+                consumed = true;
+            }
             break;
         }
-        case Input::KEY_GUI_C:
-        {
-            handleCopy();
-            consumed = true;
-            break;
-        }
-        case Input::KEY_GUI_D:
-        {
-            caretDelete();
-            consumed = true;
-            break;
-        }
-        case Input::KEY_GUI_E:
-        {
-            mCaretPosition = static_cast<int>(mText.size());
-            consumed = true;
-            break;
-        }
-        case Input::KEY_GUI_F:
-        {
-            moveCaretBack();
-            consumed = true;
-            break;
-        }
-        case Input::KEY_GUI_H:
-        {
-            deleteCharLeft(mText, &mCaretPosition);
-            consumed = true;
-            break;
-        }
-        case Input::KEY_GUI_U:
-        {
-            caretDeleteToStart();
-            consumed = true;
-            break;
-        }
-        case Input::KEY_GUI_K:
-        {
-            mText = mText.substr(0, mCaretPosition);
-            consumed = true;
-            break;
-        }
-        case Input::KEY_GUI_V:
-        {
-            handlePaste();
-            consumed = true;
-            break;
-        }
-        case Input::KEY_GUI_W:
-        {
-            caretDeleteWord();
-            consumed = true;
-            break;
-        }
-#endif
-        default:
-            break;
-    }
-}
-
-#ifndef USE_SDL2
-void TextField::handleSDLKeys(const int val, bool &consumed)
-{
-    switch (val)
-    {
-        case 2:  // Ctrl+b
-        {
-            moveCaretBack();
-            consumed = true;
-            break;
-        }
-
-        case 6:  // Ctrl+f
+        case InputAction::GUI_F:
         {
             moveCaretForward();
             consumed = true;
             break;
         }
-
-        case 4:  // Ctrl+d
+        case InputAction::GUI_D:
         {
             caretDelete();
             consumed = true;
             break;
         }
-
-        case 8:  // Ctrl+h
-            deleteCharLeft(mText, &mCaretPosition);
-            consumed = true;
-            break;
-
-        case 5:  // Ctrl+e
+        case InputAction::GUI_E:
+        {
             mCaretPosition = static_cast<int>(mText.size());
             consumed = true;
             break;
-
-        case 11:  // Ctrl+k
+        }
+        case InputAction::GUI_H:
+        {
+            deleteCharLeft(mText, &mCaretPosition);
+            consumed = true;
+            break;
+        }
+        case InputAction::GUI_K:
+        {
             mText = mText.substr(0, mCaretPosition);
             consumed = true;
             break;
-
-        case 21:  // Ctrl+u
+        }
+        case InputAction::GUI_U:
+        {
             caretDeleteToStart();
             consumed = true;
             break;
-
-        case 3:  // Ctrl+c
+        }
+        case InputAction::GUI_C:
+        {
             handleCopy();
             consumed = true;
             break;
-
-        case 22:  // Control code 22, SYNCHRONOUS IDLE, sent on Ctrl+v
+        }
+        case InputAction::GUI_V:
+        {
+#ifdef USE_SDL2
+            handlePaste();
+#else
             // hack to prevent paste key sticking
             if (mLastEventPaste && mLastEventPaste > cur_time)
                 break;
             handlePaste();
             mLastEventPaste = cur_time + 2;
+#endif
             consumed = true;
             break;
-
-        case 23:  // Ctrl+w
+        }
+        case InputAction::GUI_W:
+        {
             caretDeleteWord();
             consumed = true;
             break;
-
+        }
         default:
             break;
     }
 }
-#endif
 
 void TextField::moveCaretBack()
 {
@@ -588,12 +558,12 @@ void TextField::caretDelete()
 void TextField::handlePaste()
 {
     std::string text = getText();
-    size_t caretPos = getCaretPosition();
+    size_t caretPos = static_cast<size_t>(getCaretPosition());
 
     if (retrieveBuffer(text, caretPos))
     {
         setText(text);
-        setCaretPosition(static_cast<unsigned>(caretPos));
+        setCaretPosition(static_cast<unsigned int>(caretPos));
     }
 }
 
@@ -661,12 +631,14 @@ void TextField::handleCopy() const
     sendBuffer(text);
 }
 
-void TextField::drawCaret(gcn::Graphics* graphics, int x)
+void TextField::drawCaret(Graphics* graphics, int x)
 {
-    const gcn::Rectangle &clipArea = graphics->getCurrentClipArea();
+    const Rect *const clipArea = graphics->getCurrentClipArea();
+    if (!clipArea)
+        return;
 
     graphics->setColor(*mCaretColor);
-    graphics->drawLine(x + mPadding, clipArea.height - mPadding,
+    graphics->drawLine(x + mPadding, clipArea->height - mPadding,
         x + mPadding, mPadding);
 }
 
@@ -708,7 +680,7 @@ void TextField::fixScroll()
 
 void TextField::setCaretPosition(unsigned int position)
 {
-    const unsigned int sz = mText.size();
+    const unsigned int sz = static_cast<unsigned int>(mText.size());
     if (position > sz)
         mCaretPosition = static_cast<int>(sz);
     else
@@ -722,45 +694,47 @@ void TextField::fontChanged()
     fixScroll();
 }
 
-void TextField::mousePressed(gcn::MouseEvent &mouseEvent)
+void TextField::mousePressed(MouseEvent &event)
 {
 #ifdef ANDROID
-    if (!client->isKeyboardVisible())
-        inputManager.executeAction(Input::KEY_SHOW_KEYBOARD);
+    if (!WindowManager::isKeyboardVisible())
+        inputManager.executeAction(InputAction::SHOW_KEYBOARD);
 #endif
-    if (mouseEvent.getButton() == gcn::MouseEvent::RIGHT)
+    event.consume();
+    if (event.getButton() == MouseButton::RIGHT)
     {
-        if (viewport)
-        {
-            viewport->showTextFieldPopup(this);
-        }
-        else
-        {
-            if (!mPopupMenu)
-            {
-                mPopupMenu = new PopupMenu();
-                mPopupMenu->postInit();
-            }
-            int x = 0;
-            int y = 0;
-            SDL_GetMouseState(&x, &y);
-            mPopupMenu->showTextFieldPopup(x, y, this);
-        }
+        if (popupManager)
+            popupManager->showTextFieldPopup(this);
     }
-    else
+    else if (event.getButton() == MouseButton::LEFT)
     {
-        gcn::TextField::mousePressed(mouseEvent);
+        mCaretPosition = getFont()->getStringIndexAt(
+            mText, event.getX() + mXScroll);
+        fixScroll();
     }
 }
 
-void TextField::focusGained(const gcn::Event &event A_UNUSED)
+void TextField::focusGained(const Event &event A_UNUSED)
 {
 #ifdef ANDROID
-    if (!client->isKeyboardVisible())
-        inputManager.executeAction(Input::KEY_SHOW_KEYBOARD);
+    if (!WindowManager::isKeyboardVisible())
+        inputManager.executeAction(InputAction::SHOW_KEYBOARD);
 #endif
 }
 
-void TextField::focusLost(const gcn::Event &event A_UNUSED)
+void TextField::focusLost(const Event &event A_UNUSED)
 {
 }
+
+void TextField::setText(const std::string& text)
+{
+    const unsigned int sz = static_cast<unsigned int>(text.size());
+    if (sz < mCaretPosition)
+        mCaretPosition = sz;
+    mText = text;
+}
+
+void TextField::mouseDragged(MouseEvent& event)
+{
+    event.consume();
+}
