@@ -26,15 +26,18 @@
 
 #include "input/inputmanager.h"
 
-#include "gui/viewport.h"
+#include "gui/buttoninfo.h"
+#include "gui/buttontext.h"
+#include "gui/popupmanager.h"
+#include "gui/skin.h"
 
 #include "gui/popups/textpopup.h"
 
 #include "gui/windows/skilldialog.h"
-#ifdef MANASERV_SUPPORT
-#include "gui/specialswindow.h"
-#endif
 
+#include "gui/widgets/button.h"
+
+#include "utils/delete2.h"
 #include "utils/dtor.h"
 #include "utils/gettext.h"
 
@@ -42,13 +45,14 @@
 
 #include "debug.h"
 
+WindowMenu *windowMenu = nullptr;
+
 WindowMenu::WindowMenu(const Widget2 *const widget) :
     Container(widget),
-    gcn::ActionListener(),
-    gcn::SelectionListener(),
-    gcn::MouseListener(),
-    mSkin(Theme::instance() ? Theme::instance()->load("windowmenu.xml", "")
-          : nullptr),
+    ActionListener(),
+    SelectionListener(),
+    MouseListener(),
+    mSkin(theme ? theme->load("windowmenu.xml", "") : nullptr),
     mPadding(mSkin ? mSkin->getPadding() : 1),
     mSpacing(mSkin ? mSkin->getOption("spacing", 3) : 3),
     mTextPopup(new TextPopup),
@@ -66,24 +70,24 @@ WindowMenu::WindowMenu(const Widget2 *const widget) :
     setFocusable(false);
     // TRANSLATORS: short button name for who is online window.
     addButton(N_("ONL"),
-        _("Who is online"), x, h, Input::KEY_WINDOW_ONLINE);
+        _("Who is online"), x, h, InputAction::WINDOW_ONLINE);
     // TRANSLATORS: short button name for help window.
     addButton(N_("HLP"),
-        _("Help"), x, h, Input::KEY_WINDOW_HELP);
+        _("Help"), x, h, InputAction::WINDOW_HELP);
     // TRANSLATORS: short button name for quests window.
     addButton(N_("QE"),
-        _("Quests"), x, h, Input::KEY_WINDOW_QUESTS);
+        _("Quests"), x, h, InputAction::WINDOW_QUESTS);
     // TRANSLATORS: short button name for bot checker window.
     addButton(N_("BC"),
-        _("Bot checker"), x, h, Input::KEY_WINDOW_BOT_CHECKER, false);
+        _("Bot checker"), x, h, InputAction::WINDOW_BOT_CHECKER, false);
     // TRANSLATORS: short button name for kill stats window.
     addButton(N_("KS"),
-        _("Kill stats"), x, h, Input::KEY_WINDOW_KILLS);
+        _("Kill stats"), x, h, InputAction::WINDOW_KILLS);
     addButton(":-)",
-        _("Smilies"), x, h, Input::KEY_WINDOW_EMOTE_SHORTCUT);
+        _("Smilies"), x, h, InputAction::WINDOW_EMOTE_SHORTCUT);
     // TRANSLATORS: short button name for chat window.
     addButton(N_("CH"),
-        _("Chat"), x, h, Input::KEY_WINDOW_CHAT,
+        _("Chat"), x, h, InputAction::WINDOW_CHAT,
 #ifdef ANDROID
         true);
 #else
@@ -91,65 +95,60 @@ WindowMenu::WindowMenu(const Widget2 *const widget) :
 #endif
     // TRANSLATORS: short button name for status window.
     addButton(N_("STA"),
-        _("Status"), x, h, Input::KEY_WINDOW_STATUS);
+        _("Status"), x, h, InputAction::WINDOW_STATUS);
     // TRANSLATORS: short button name for equipment window.
     addButton(N_("EQU"),
-        _("Equipment"), x, h, Input::KEY_WINDOW_EQUIPMENT);
+        _("Equipment"), x, h, InputAction::WINDOW_EQUIPMENT);
     // TRANSLATORS: short button name for inventory window.
     addButton(N_("INV"),
-        _("Inventory"), x, h, Input::KEY_WINDOW_INVENTORY);
+        _("Inventory"), x, h, InputAction::WINDOW_INVENTORY);
     // TRANSLATORS: short button name for map window.
     addButton(N_("MAP"),
-        _("Map"), x, h, Input::KEY_WINDOW_MINIMAP, false);
+        _("Map"), x, h, InputAction::WINDOW_MINIMAP, false);
 
     if (skillDialog->hasSkills())
     {
         // TRANSLATORS: short button name for skills window.
         addButton(N_("SKI"),
-            _("Skills"), x, h, Input::KEY_WINDOW_SKILL);
+            _("Skills"), x, h, InputAction::WINDOW_SKILL);
     }
 
-#ifdef MANASERV_SUPPORT
-    if (Net::getNetworkType() == ServerInfo::MANASERV)
-    {
-        // TRANSLATORS: short button name for specials window.
-        addButton(N_("SPE"),
-            _("Specials"), x, h, Input::KEY_NO_VALUE);
-    }
-#endif
-
     // TRANSLATORS: short button name for social window.
     addButton(N_("SOC"),
     // TRANSLATORS: full button name
-        _("Social"), x, h, Input::KEY_WINDOW_SOCIAL);
+        _("Social"), x, h, InputAction::WINDOW_SOCIAL);
     // TRANSLATORS: short button name for shortcuts window.
     addButton(N_("SH"),
         // TRANSLATORS: full button name
-        _("Shortcuts"), x, h, Input::KEY_WINDOW_SHORTCUT);
+        _("Shortcuts"), x, h, InputAction::WINDOW_SHORTCUT);
     // TRANSLATORS: short button name for spells window.
     addButton(N_("SP"),
         // TRANSLATORS: full button name
-        _("Spells"), x, h, Input::KEY_WINDOW_SPELLS);
+        _("Spells"), x, h, InputAction::WINDOW_SPELLS);
     // TRANSLATORS: short button name for drops window.
     addButton(N_("DR"),
         // TRANSLATORS: full button name
-        _("Drop"), x, h, Input::KEY_WINDOW_DROP, false);
+        _("Drop"), x, h, InputAction::WINDOW_DROP, false);
     // TRANSLATORS: short button name for did you know window.
     addButton(N_("YK"),
         // TRANSLATORS: full button name
-        _("Did you know"), x, h, Input::KEY_WINDOW_DIDYOUKNOW, false);
+        _("Did you know"), x, h, InputAction::WINDOW_DIDYOUKNOW, false);
     // TRANSLATORS: short button name for shop window.
     addButton(N_("SHP"),
         // TRANSLATORS: full button name
-        _("Shop"), x, h, Input::KEY_WINDOW_SHOP, false);
+        _("Shop"), x, h, InputAction::WINDOW_SHOP, false);
     // TRANSLATORS: short button name for outfits window.
     addButton(N_("OU"),
         // TRANSLATORS: full button name
-        _("Outfits"), x, h, Input::KEY_WINDOW_OUTFIT, false);
+        _("Outfits"), x, h, InputAction::WINDOW_OUTFIT, false);
+    // TRANSLATORS: short button name for updates window.
+    addButton(N_("UP"),
+        // TRANSLATORS: full button name
+        _("Updates"), x, h, InputAction::WINDOW_UPDATER, false);
     // TRANSLATORS: short button name for debug window.
     addButton(N_("DBG"),
         // TRANSLATORS: full button name
-        _("Debug"), x, h, Input::KEY_WINDOW_DEBUG,
+        _("Debug"), x, h, InputAction::WINDOW_DEBUG,
 #ifdef ANDROID
         true);
 #else
@@ -158,15 +157,15 @@ WindowMenu::WindowMenu(const Widget2 *const widget) :
     // TRANSLATORS: short button name for windows list menu.
     addButton(N_("WIN"),
         // TRANSLATORS: full button name
-        _("Windows"), x, h, Input::KEY_SHOW_WINDOWS, false);
+        _("Windows"), x, h, InputAction::SHOW_WINDOWS, false);
     // TRANSLATORS: short button name for setup window.
     addButton(N_("SET"),
         // TRANSLATORS: full button name
-        _("Setup"), x, h, Input::KEY_WINDOW_SETUP);
+        _("Setup"), x, h, InputAction::WINDOW_SETUP);
 
     x += mPadding - mSpacing;
     if (mainGraphics)
-        setDimension(gcn::Rectangle(mainGraphics->mWidth - x, 0, x, h));
+        setDimension(Rect(mainGraphics->mWidth - x, 0, x, h));
 
     loadButtons();
 
@@ -182,8 +181,7 @@ WindowMenu::~WindowMenu()
     config.removeListener("autohideButtons", this);
     CHECKLISTENERS
 
-    delete mTextPopup;
-    mTextPopup = nullptr;
+    delete2(mTextPopup);
     for (std::map <std::string, ButtonInfo*>::iterator
          it = mButtonNames.begin(),
          it_end = mButtonNames.end(); it != it_end; ++it)
@@ -191,32 +189,27 @@ WindowMenu::~WindowMenu()
         delete (*it).second;
     }
     mButtonNames.clear();
-    for (std::vector <Button*>::iterator it = mButtons.begin(),
-         it_end = mButtons.end(); it != it_end; ++it)
+    FOR_EACH (std::vector <Button*>::iterator, it, mButtons)
     {
         Button *const btn = dynamic_cast<Button*>(*it);
         if (!btn)
             continue;
-        if (!btn->isVisible())
-        {
+        if (!btn->isVisibleLocal())
             delete btn;
-        }
     }
     delete_all(mButtonTexts);
     mButtonTexts.clear();
     if (mSkin)
     {
-        Theme *const theme = Theme::instance();
         if (theme)
             theme->unload(mSkin);
         mSkin = nullptr;
     }
 }
 
-void WindowMenu::action(const gcn::ActionEvent &event)
+void WindowMenu::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
-
     const std::map <std::string, ButtonInfo*>::iterator
         it = mButtonNames.find(eventId);
     if (it == mButtonNames.end())
@@ -231,7 +224,9 @@ void WindowMenu::action(const gcn::ActionEvent &event)
 
 void WindowMenu::addButton(const char *const text,
                            const std::string &description,
-                           int &restrict x, int &restrict h, const int key,
+                           int &restrict x,
+                           int &restrict h,
+                           const int key,
                            const bool visible)
 {
     Button *const btn = new Button(this, gettext(text), text, this);
@@ -244,29 +239,33 @@ void WindowMenu::addButton(const char *const text,
     h = btn->getHeight() + 2 * mPadding;
     mButtons.push_back(btn);
     mButtonNames[text] = new ButtonInfo(btn, key, visible);
-    if (key != Input::KEY_SHOW_WINDOWS)
+    if (key != InputAction::SHOW_WINDOWS)
         mButtonTexts.push_back(new ButtonText(description, key));
 }
 
-void WindowMenu::mousePressed(gcn::MouseEvent &event)
+void WindowMenu::mousePressed(MouseEvent &event)
 {
-    if (!viewport)
+    if (!popupManager)
         return;
 
-    if (!mSmallWindow && event.getButton() == gcn::MouseEvent::RIGHT)
+    if (event.getButton() == MouseButton::RIGHT)
     {
+        if (mSmallWindow)
+            return;
+
+        event.consume();
         Button *const btn = dynamic_cast<Button*>(event.getSource());
         if (!btn)
             return;
-        if (viewport)
+        if (popupManager)
         {
-            viewport->showPopup(getX() + event.getX(),
+            popupManager->showPopup(getX() + event.getX(),
                 getY() + event.getY(), btn);
         }
     }
 }
 
-void WindowMenu::mouseMoved(gcn::MouseEvent &event)
+void WindowMenu::mouseMoved(MouseEvent &event)
 {
     mHaveMouse = true;
 
@@ -291,8 +290,8 @@ void WindowMenu::mouseMoved(gcn::MouseEvent &event)
     const int x = event.getX();
     const int y = event.getY();
     const int key = btn->getTag();
-    const gcn::Rectangle &rect = mDimension;
-    if (key != Input::KEY_NO_VALUE)
+    const Rect &rect = mDimension;
+    if (key != InputAction::NO_VALUE)
     {
         mTextPopup->show(x + rect.x, y + rect.y, btn->getDescription(),
             // TRANSLATORS: short key name
@@ -305,7 +304,7 @@ void WindowMenu::mouseMoved(gcn::MouseEvent &event)
     }
 }
 
-void WindowMenu::mouseExited(gcn::MouseEvent& mouseEvent A_UNUSED)
+void WindowMenu::mouseExited(MouseEvent& event A_UNUSED)
 {
     mHaveMouse = false;
     if (!mTextPopup)
@@ -338,7 +337,7 @@ void WindowMenu::updateButtons()
         Button *const btn = dynamic_cast<Button *const>(*it);
         if (!btn)
             continue;
-        if (btn->isVisible())
+        if (btn->isVisibleLocal())
         {
             btn->setPosition(x, mPadding);
             add(btn);
@@ -348,7 +347,7 @@ void WindowMenu::updateButtons()
     }
     x += mPadding - mSpacing;
     if (mainGraphics)
-        setDimension(gcn::Rectangle(mainGraphics->mWidth - x, 0, x, h));
+        setDimension(Rect(mainGraphics->mWidth - x, 0, x, h));
 }
 
 void WindowMenu::loadButtons()
@@ -403,12 +402,10 @@ void WindowMenu::loadButtons()
 void WindowMenu::saveButtons() const
 {
     int i = 0;
-    for (std::vector <Button*>::const_iterator it = mButtons.begin(),
-         it_end = mButtons.end();
-         it != it_end; ++it)
+    FOR_EACH (std::vector <Button*>::const_iterator, it, mButtons)
     {
         const Button *const btn = dynamic_cast<const Button *const>(*it);
-        if (btn && !btn->isVisible())
+        if (btn && !btn->isVisibleLocal())
         {
             config.setValue("windowmenu" + toString(i),
                 btn->getActionEventId());
@@ -419,7 +416,7 @@ void WindowMenu::saveButtons() const
         config.deleteKey("windowmenu" + toString(f));
 }
 
-void WindowMenu::drawChildren(gcn::Graphics* graphics)
+void WindowMenu::drawChildren(Graphics* graphics)
 {
     if (mHaveMouse || !mAutoHide || (mAutoHide == 1
         && mainGraphics && (mSmallWindow || mainGraphics->mWidth > 800)))
