@@ -26,15 +26,20 @@
 #include "shopitem.h"
 #include "units.h"
 
+#include "gui/windows/setupwindow.h"
 #include "gui/windows/tradewindow.h"
 
+#include "gui/models/shopitems.h"
+#include "gui/models/sortlistmodelbuy.h"
+
 #include "gui/widgets/button.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/dropdown.h"
 #include "gui/widgets/inttextfield.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/scrollarea.h"
-#include "gui/widgets/shopitems.h"
 #include "gui/widgets/shoplistbox.h"
 #include "gui/widgets/slider.h"
 
@@ -45,159 +50,131 @@
 
 #include "resources/iteminfo.h"
 
-#include "utils/gettext.h"
+#include "utils/delete2.h"
 
 #include <algorithm>
 
 #include "debug.h"
 
-static const char *const SORT_NAME_BUY[7] =
+namespace
 {
-    // TRANSLATORS: buy dialog sort type.
-    N_("unsorted"),
-    // TRANSLATORS: buy dialog sort type.
-    N_("by price"),
-    // TRANSLATORS: buy dialog sort type.
-    N_("by name"),
-    // TRANSLATORS: buy dialog sort type.
-    N_("by id"),
-    // TRANSLATORS: buy dialog sort type.
-    N_("by weight"),
-    // TRANSLATORS: buy dialog sort type.
-    N_("by amount"),
-    // TRANSLATORS: buy dialog sort type.
-    N_("by type")
-};
-
-class SortListModelBuy final : public gcn::ListModel
-{
-public:
-    ~SortListModelBuy()
-    { }
-
-    int getNumberOfElements()
-    { return 7; }
-
-    std::string getElementAt(int i)
+    class SortItemPriceFunctor final
     {
-        if (i >= getNumberOfElements() || i < 0)
-            return "???";
-        return gettext(SORT_NAME_BUY[i]);
-    }
-};
+        public:
+            bool operator() (const ShopItem *const item1,
+                             const ShopItem *const item2) const
+            {
+                if (!item1 || !item2)
+                    return false;
 
-class SortItemPriceFunctor final
-{
-    public:
-        bool operator() (const ShopItem *const item1,
-                         const ShopItem *const item2) const
-        {
-            if (!item1 || !item2)
-                return false;
+                const int price1 = item1->getPrice();
+                const int price2 = item2->getPrice();
+                if (price1 == price2)
+                    return item1->getDisplayName() < item2->getDisplayName();
+                return price1 < price2;
+            }
+    } itemPriceBuySorter;
 
-            const int price1 = item1->getPrice();
-            const int price2 = item2->getPrice();
-            if (price1 == price2)
-                return item1->getDisplayName() < item2->getDisplayName();
-            return price1 < price2;
-        }
-} itemPriceBuySorter;
+    class SortItemNameFunctor final
+    {
+        public:
+            bool operator() (const ShopItem *const item1,
+                             const ShopItem *const item2) const
+            {
+                if (!item1 || !item2)
+                    return false;
 
-class SortItemNameFunctor final
-{
-    public:
-        bool operator() (const ShopItem *const item1,
-                         const ShopItem *const item2) const
-        {
-            if (!item1 || !item2)
-                return false;
+                const std::string &name1 = item1->getDisplayName();
+                const std::string &name2 = item2->getDisplayName();
+                if (name1 == name2)
+                    return item1->getPrice() < item2->getPrice();
+                return name1 < name2;
+            }
+    } itemNameBuySorter;
 
-            const std::string &name1 = item1->getDisplayName();
-            const std::string &name2 = item2->getDisplayName();
-            if (name1 == name2)
-                return item1->getPrice() < item2->getPrice();
-            return name1 < name2;
-        }
-} itemNameBuySorter;
+    class SortItemIdFunctor final
+    {
+        public:
+            bool operator() (const ShopItem *const item1,
+                             const ShopItem *const item2) const
+            {
+                if (!item1 || !item2)
+                    return false;
 
-class SortItemIdFunctor final
-{
-    public:
-        bool operator() (const ShopItem *const item1,
-                         const ShopItem *const item2) const
-        {
-            if (!item1 || !item2)
-                return false;
+                const int id1 = item1->getId();
+                const int id2 = item2->getId();
+                if (id1 == id2)
+                    return item1->getPrice() < item2->getPrice();
+                return id1 < id2;
+            }
+    } itemIdBuySorter;
 
-            const int id1 = item1->getId();
-            const int id2 = item2->getId();
-            if (id1 == id2)
-                return item1->getPrice() < item2->getPrice();
-            return id1 < id2;
-        }
-} itemIdBuySorter;
+    class SortItemWeightFunctor final
+    {
+        public:
+            bool operator() (const ShopItem *const item1,
+                             const ShopItem *const item2) const
+            {
+                if (!item1 || !item2)
+                    return false;
 
-class SortItemWeightFunctor final
-{
-    public:
-        bool operator() (const ShopItem *const item1,
-                         const ShopItem *const item2) const
-        {
-            if (!item1 || !item2)
-                return false;
+                const int weight1 = item1->getInfo().getWeight();
+                const int weight2 = item2->getInfo().getWeight();
+                if (weight1 == weight2)
+                    return item1->getPrice() < item2->getPrice();
+                return weight1 < weight2;
+            }
+    } itemWeightBuySorter;
 
-            const int weight1 = item1->getInfo().getWeight();
-            const int weight2 = item2->getInfo().getWeight();
-            if (weight1 == weight2)
-                return item1->getPrice() < item2->getPrice();
-            return weight1 < weight2;
-        }
-} itemWeightBuySorter;
+    class SortItemAmountFunctor final
+    {
+        public:
+            bool operator() (const ShopItem *const item1,
+                             const ShopItem *const item2) const
+            {
+                if (!item1 || !item2)
+                    return false;
 
-class SortItemAmountFunctor final
-{
-    public:
-        bool operator() (const ShopItem *const item1,
-                         const ShopItem *const item2) const
-        {
-            if (!item1 || !item2)
-                return false;
+                const int amount1 = item1->getQuantity();
+                const int amount2 = item2->getQuantity();
+                if (amount1 == amount2)
+                    return item1->getPrice() < item2->getPrice();
+                return amount1 < amount2;
+            }
+    } itemAmountBuySorter;
 
-            const int amount1 = item1->getQuantity();
-            const int amount2 = item2->getQuantity();
-            if (amount1 == amount2)
-                return item1->getPrice() < item2->getPrice();
-            return amount1 < amount2;
-        }
-} itemAmountBuySorter;
+    class SortItemTypeFunctor final
+    {
+        public:
+            bool operator() (const ShopItem *const item1,
+                             const ShopItem *const item2) const
+            {
+                if (!item1 || !item2)
+                    return false;
 
-class SortItemTypeFunctor final
-{
-    public:
-        bool operator() (const ShopItem *const item1,
-                         const ShopItem *const item2) const
-        {
-            if (!item1 || !item2)
-                return false;
-
-            const int type1 = item1->getInfo().getType();
-            const int type2 = item2->getInfo().getType();
-            if (type1 == type2)
-                return item1->getPrice() < item2->getPrice();
-            return type1 < type2;
-        }
-} itemTypeBuySorter;
+                const ItemType::Type type1 = item1->getInfo().getType();
+                const ItemType::Type type2 = item2->getInfo().getType();
+                if (type1 == type2)
+                    return item1->getPrice() < item2->getPrice();
+                return type1 < type2;
+            }
+    } itemTypeBuySorter;
+}  // namespace
 
 BuyDialog::DialogList BuyDialog::instances;
 
 BuyDialog::BuyDialog() :
     // TRANSLATORS: buy dialog name
     Window(_("Create items"), false, nullptr, "buy.xml"),
-    gcn::ActionListener(),
-    gcn::SelectionListener(),
-    mNpcId(-2), mMoney(0), mAmountItems(0), mMaxItems(0), mNick(),
+    ActionListener(),
+    SelectionListener(),
     mSortModel(nullptr),
-    mSortDropDown(nullptr)
+    mSortDropDown(nullptr),
+    mNpcId(-2),
+    mMoney(0),
+    mAmountItems(0),
+    mMaxItems(0),
+    mNick()
 {
     init();
 }
@@ -205,11 +182,15 @@ BuyDialog::BuyDialog() :
 BuyDialog::BuyDialog(const int npcId) :
     // TRANSLATORS: buy dialog name
     Window(_("Buy"), false, nullptr, "buy.xml"),
-    gcn::ActionListener(),
-    gcn::SelectionListener(),
-    mNpcId(npcId), mMoney(0), mAmountItems(0), mMaxItems(0), mNick(),
+    ActionListener(),
+    SelectionListener(),
     mSortModel(nullptr),
-    mSortDropDown(nullptr)
+    mSortDropDown(nullptr),
+    mNpcId(npcId),
+    mMoney(0),
+    mAmountItems(0),
+    mMaxItems(0),
+    mNick()
 {
     init();
 }
@@ -217,11 +198,15 @@ BuyDialog::BuyDialog(const int npcId) :
 BuyDialog::BuyDialog(std::string nick) :
     // TRANSLATORS: buy dialog name
     Window(_("Buy"), false, nullptr, "buy.xml"),
-    gcn::ActionListener(),
-    gcn::SelectionListener(),
-    mNpcId(-1), mMoney(0), mAmountItems(0), mMaxItems(0), mNick(nick),
+    ActionListener(),
+    SelectionListener(),
     mSortModel(new SortListModelBuy),
-    mSortDropDown(new DropDown(this, mSortModel, false, false, this, "sort"))
+    mSortDropDown(new DropDown(this, mSortModel, false, false, this, "sort")),
+    mNpcId(-1),
+    mMoney(0),
+    mAmountItems(0),
+    mMaxItems(0),
+    mNick(nick)
 {
     init();
 }
@@ -236,18 +221,21 @@ void BuyDialog::init()
     setMinHeight(220);
     setDefaultSize(260, 230, ImageRect::CENTER);
 
+    if (setupWindow)
+        setupWindow->registerWindowForReset(this);
+
     mShopItems = new ShopItems;
 
     mShopItemList = new ShopListBox(this, mShopItems, mShopItems);
     mShopItemList->postInit();
-    mScrollArea = new ScrollArea(mShopItemList,
+    mScrollArea = new ScrollArea(this, mShopItemList,
         getOptionBool("showbackground"), "buy_background.xml");
-    mScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    mScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
 
-    mSlider = new Slider(1.0);
+    mSlider = new Slider(this, 1.0, 1.0);
     mQuantityLabel = new Label(this, strprintf(
         "%d / %d", mAmountItems, mMaxItems));
-    mQuantityLabel->setAlignment(gcn::Graphics::CENTER);
+    mQuantityLabel->setAlignment(Graphics::CENTER);
     // TRANSLATORS: buy dialog label
     mMoneyLabel = new Label(this, strprintf(
         _("Price: %s / Total: %s"), "", ""));
@@ -308,7 +296,7 @@ void BuyDialog::init()
     placer(8, 8, mQuitButton);
 
     Layout &layout = getLayout();
-    layout.setRowHeight(0, Layout::AUTO_SET);
+    layout.setRowHeight(0, LayoutType::SET);
 
     center();
     loadWindowState();
@@ -323,9 +311,7 @@ void BuyDialog::init()
 
 BuyDialog::~BuyDialog()
 {
-    delete mShopItems;
-    mShopItems = nullptr;
-
+    delete2(mShopItems);
     instances.remove(this);
 }
 
@@ -344,7 +330,7 @@ void BuyDialog::reset()
 
     // Reset previous selected items to prevent failing asserts
     mShopItemList->setSelected(-1);
-    mSlider->setValue2(0);
+    mSlider->setValue(0);
 
     setMoney(0);
 }
@@ -388,7 +374,7 @@ void BuyDialog::sort()
     }
 }
 
-void BuyDialog::action(const gcn::ActionEvent &event)
+void BuyDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "quit")
@@ -419,28 +405,28 @@ void BuyDialog::action(const gcn::ActionEvent &event)
     else if (eventId == "inc" && mAmountItems < mMaxItems)
     {
         mAmountItems++;
-        mSlider->setValue2(mAmountItems);
+        mSlider->setValue(mAmountItems);
         mAmountField->setValue(mAmountItems);
         updateButtonsAndLabels();
     }
     else if (eventId == "dec" && mAmountItems > 1)
     {
         mAmountItems--;
-        mSlider->setValue2(mAmountItems);
+        mSlider->setValue(mAmountItems);
         mAmountField->setValue(mAmountItems);
         updateButtonsAndLabels();
     }
     else if (eventId == "max")
     {
         mAmountItems = mMaxItems;
-        mSlider->setValue2(mAmountItems);
+        mSlider->setValue(mAmountItems);
         mAmountField->setValue(mAmountItems);
         updateButtonsAndLabels();
     }
     else if (eventId == "amount")
     {
         mAmountItems = mAmountField->getValue();
-        mSlider->setValue2(mAmountItems);
+        mSlider->setValue(mAmountItems);
         updateButtonsAndLabels();
     }
     else if (eventId == "buy" && mAmountItems > 0 && mAmountItems <= mMaxItems)
@@ -466,7 +452,7 @@ void BuyDialog::action(const gcn::ActionEvent &event)
             // Reset selection
             mAmountItems = 1;
             mSlider->setScale(1, mMaxItems);
-            mSlider->setValue2(1);
+            mSlider->setValue(1);
         }
         else if (tradeWindow)
         {
@@ -482,11 +468,11 @@ void BuyDialog::action(const gcn::ActionEvent &event)
     }
 }
 
-void BuyDialog::valueChanged(const gcn::SelectionEvent &event A_UNUSED)
+void BuyDialog::valueChanged(const SelectionEvent &event A_UNUSED)
 {
     // Reset amount of items and update labels
     mAmountItems = 1;
-    mSlider->setValue2(1);
+    mSlider->setValue(1);
 
     updateButtonsAndLabels();
     mSlider->setScale(1, mMaxItems);
