@@ -26,6 +26,8 @@
 #include "net/net.h"
 #include "net/npchandler.h"
 
+#include "gui/windows/setupwindow.h"
+
 #include "gui/widgets/button.h"
 
 #include "utils/gettext.h"
@@ -37,7 +39,7 @@ BuySellDialog::DialogList BuySellDialog::dialogInstances;
 BuySellDialog::BuySellDialog(const int npcId) :
     // TRANSLATORS: shop window name
     Window(_("Shop"), false, nullptr, "buysell.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mNpcId(npcId),
     mNick(""),
     mBuyButton(nullptr)
@@ -48,7 +50,7 @@ BuySellDialog::BuySellDialog(const int npcId) :
 BuySellDialog::BuySellDialog(const std::string &nick) :
     // TRANSLATORS: shop window name
     Window(_("Shop"), false, nullptr, "buysell.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mNpcId(-1),
     mNick(nick),
     mBuyButton(nullptr)
@@ -61,6 +63,9 @@ void BuySellDialog::init()
     setWindowName("BuySell");
     setCloseButton(true);
 
+    if (setupWindow)
+        setupWindow->registerWindowForReset(this);
+
     static const char *buttonNames[] =
     {
         // TRANSLATORS: shop window button
@@ -119,7 +124,7 @@ void BuySellDialog::setVisible(bool visible)
     }
 }
 
-void BuySellDialog::action(const gcn::ActionEvent &event)
+void BuySellDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "Buy")
