@@ -24,17 +24,19 @@
 
 #include "client.h"
 
-#include "gui/windows/registerdialog.h"
 #include "gui/windows/okdialog.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/textfield.h"
 
+#include "listeners/wrongdatanoticelistener.h"
+
 #include "net/logindata.h"
 #include "net/loginhandler.h"
 #include "net/net.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include <string>
@@ -42,10 +44,10 @@
 
 #include "debug.h"
 
-ChangeEmailDialog::ChangeEmailDialog(LoginData *const data):
+ChangeEmailDialog::ChangeEmailDialog(LoginData *const data) :
     // TRANSLATORS: change email dialog header
     Window(_("Change Email Address"), true, nullptr, "changeemail.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mFirstEmailField(new TextField(this)),
     mSecondEmailField(new TextField(this)),
     // TRANSLATORS: button in change email dialog
@@ -105,11 +107,10 @@ ChangeEmailDialog::ChangeEmailDialog(LoginData *const data):
 
 ChangeEmailDialog::~ChangeEmailDialog()
 {
-    delete mWrongDataNoticeListener;
-    mWrongDataNoticeListener = nullptr;
+    delete2(mWrongDataNoticeListener);
 }
 
-void ChangeEmailDialog::action(const gcn::ActionEvent &event)
+void ChangeEmailDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "cancel")
@@ -118,9 +119,9 @@ void ChangeEmailDialog::action(const gcn::ActionEvent &event)
     }
     else if (eventId == "change_email")
     {
-        const std::string username = mLoginData->username.c_str();
-        const std::string newFirstEmail = mFirstEmailField->getText();
-        const std::string newSecondEmail = mSecondEmailField->getText();
+        const std::string username = mLoginData->username;
+        const std::string &newFirstEmail = mFirstEmailField->getText();
+        const std::string &newSecondEmail = mSecondEmailField->getText();
         logger->log("ChangeEmailDialog::Email change, Username is %s",
                      username.c_str());
 
@@ -165,7 +166,7 @@ void ChangeEmailDialog::action(const gcn::ActionEvent &event)
 
             // TRANSLATORS: change email error header
             OkDialog *const dlg = new OkDialog(_("Error"),
-                errorMsg.str(), DIALOG_ERROR);
+                errorMsg.str(), DialogType::ERROR);
             dlg->addActionListener(mWrongDataNoticeListener);
         }
         else
