@@ -24,18 +24,20 @@
 
 #include "client.h"
 
-#include "gui/windows/registerdialog.h"
 #include "gui/windows/okdialog.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/passwordfield.h"
 #include "gui/widgets/label.h"
-#include "gui/widgets/layout.h"
+#include "gui/widgets/layoutcell.h"
+
+#include "listeners/wrongdatanoticelistener.h"
 
 #include "net/logindata.h"
 #include "net/loginhandler.h"
 #include "net/net.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include <string>
@@ -43,10 +45,10 @@
 
 #include "debug.h"
 
-ChangePasswordDialog::ChangePasswordDialog(LoginData *const data):
+ChangePasswordDialog::ChangePasswordDialog(LoginData *const data) :
     // TRANSLATORS: change password window name
     Window(_("Change Password"), true, nullptr, "changepassword.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mOldPassField(new PasswordField(this)),
     mFirstPassField(new PasswordField(this)),
     mSecondPassField(new PasswordField(this)),
@@ -84,11 +86,10 @@ ChangePasswordDialog::ChangePasswordDialog(LoginData *const data):
 
 ChangePasswordDialog::~ChangePasswordDialog()
 {
-    delete mWrongDataNoticeListener;
-    mWrongDataNoticeListener = nullptr;
+    delete2(mWrongDataNoticeListener);
 }
 
-void ChangePasswordDialog::action(const gcn::ActionEvent &event)
+void ChangePasswordDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "cancel")
@@ -97,10 +98,10 @@ void ChangePasswordDialog::action(const gcn::ActionEvent &event)
     }
     else if (eventId == "change_password")
     {
-        const std::string username = mLoginData->username.c_str();
-        const std::string oldPassword = mOldPassField->getText();
-        const std::string newFirstPass = mFirstPassField->getText();
-        const std::string newSecondPass = mSecondPassField->getText();
+        const std::string username = mLoginData->username;
+        const std::string &oldPassword = mOldPassField->getText();
+        const std::string &newFirstPass = mFirstPassField->getText();
+        const std::string &newSecondPass = mSecondPassField->getText();
         logger->log("ChangePasswordDialog::Password change, Username is %s",
                      username.c_str());
 
@@ -155,7 +156,7 @@ void ChangePasswordDialog::action(const gcn::ActionEvent &event)
 
             // TRANSLATORS: change password error header
             OkDialog *const dlg = new OkDialog(_("Error"),
-                errorMsg.str(), DIALOG_ERROR);
+                errorMsg.str(), DialogType::ERROR);
             dlg->addActionListener(mWrongDataNoticeListener);
         }
         else
