@@ -24,12 +24,14 @@
 
 #include "main.h"
 
-#include "input/keydata.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
+
+#include "input/inputaction.h"
 
 #include "gui/windows/okdialog.h"
 
 #include "gui/widgets/button.h"
+#include "gui/windows/charselectdialog.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/playerbox.h"
 #include "gui/widgets/radiobutton.h"
@@ -41,30 +43,39 @@
 #include "resources/iteminfo.h"
 
 #include "resources/db/chardb.h"
+#include "resources/db/colordb.h"
 #include "resources/db/itemdb.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include "debug.h"
 
 extern int serverVersion;
 
-static const Being::Action actions[] =
+static const BeingAction::Action actions[] =
 {
-    Being::STAND, Being::SIT, Being::MOVE, Being::ATTACK, Being::DEAD
+    BeingAction::STAND,
+    BeingAction::SIT,
+    BeingAction::MOVE,
+    BeingAction::ATTACK,
+    BeingAction::DEAD
 };
 
 static const uint8_t directions[] =
 {
-    Being::DOWN, Being::RIGHT, Being::UP, Being::LEFT
+    BeingDirection::DOWN,
+    BeingDirection::RIGHT,
+    BeingDirection::UP,
+    BeingDirection::LEFT
 };
 
 CharCreateDialog::CharCreateDialog(CharSelectDialog *const parent,
                                    const int slot) :
     // TRANSLATORS: char create dialog name
     Window(_("New Character"), true, parent, "charcreate.xml"),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
+    ActionListener(),
+    KeyListener(),
     mCharSelectDialog(parent),
     mNameField(new TextField(this, "")),
     // TRANSLATORS: char create dialog label
@@ -111,20 +122,20 @@ CharCreateDialog::CharCreateDialog(CharSelectDialog *const parent,
     mAttributesLeft(new Label(this,
         // TRANSLATORS: char create dialog label
         strprintf(_("Please distribute %d points"), 99))),
-    mMaxPoints(0),
-    mUsedPoints(0),
     // TRANSLATORS: char create dialog button
     mCreateButton(new Button(this, _("Create"), "create", this)),
     // TRANSLATORS: char create dialog button
     mCancelButton(new Button(this, _("Cancel"), "cancel", this)),
+    mPlayer(new Being(0, ActorType::PLAYER, static_cast<uint16_t>(0U),
+            nullptr)),
+    mPlayerBox(new PlayerBox(this, mPlayer, "charcreate_playerbox.xml",
+        "charcreate_selectedplayerbox.xml")),
+    mMaxPoints(0),
+    mUsedPoints(0),
     mRace(0),
     mLook(0),
     mMinLook(CharDB::getMinLook()),
     mMaxLook(CharDB::getMaxLook()),
-    mPlayer(new Being(0, ActorSprite::PLAYER, static_cast<uint16_t>(mRace),
-            nullptr)),
-    mPlayerBox(new PlayerBox(mPlayer, "charcreate_playerbox.xml",
-        "charcreate_selectedplayerbox.xml")),
     mHairStyle(0),
     mHairColor(0),
     mSlot(slot),
@@ -139,7 +150,7 @@ CharCreateDialog::CharCreateDialog(CharSelectDialog *const parent,
     setSticky(true);
     setWindowName("NewCharacter");
 
-    mPlayer->setGender(GENDER_MALE);
+    mPlayer->setGender(Gender::MALE);
     const std::vector<int> &items = CharDB::getDefaultItems();
     int i = 1;
     for (std::vector<int>::const_iterator it = items.begin(),
@@ -155,13 +166,23 @@ CharCreateDialog::CharCreateDialog(CharSelectDialog *const parent,
         maxHairStyle = mPlayer->getNumOfHairstyles();
 
     if (maxHairStyle)
-        mHairStyle = (rand() % maxHairStyle) + minHairStyle;
+    {
+        mHairStyle = (static_cast<unsigned int>(rand())
+            % maxHairStyle) + minHairStyle;
+    }
     else
+    {
         mHairStyle = 0;
+    }
     if (maxHairColor)
-        mHairColor = (rand() % maxHairColor) + minHairColor;
+    {
+        mHairColor = (static_cast<unsigned int>(rand())
+            % maxHairColor) + minHairColor;
+    }
     else
+    {
         mHairColor = 0;
+    }
 
     mNameField->setMaximum(24);
 
@@ -206,13 +227,13 @@ CharCreateDialog::CharCreateDialog(CharSelectDialog *const parent,
     const int h = 350;
 
     setContentSize(w, h);
-    mPlayerBox->setDimension(gcn::Rectangle(360, 0, 110, 90));
+    mPlayerBox->setDimension(Rect(360, 0, 110, 90));
     mActionButton->setPosition(385, 100);
     mRotateButton->setPosition(415, 100);
 
     mNameLabel->setPosition(5, 2);
     mNameField->setDimension(
-            gcn::Rectangle(60, 2, 300, mNameField->getHeight()));
+            Rect(60, 2, 300, mNameField->getHeight()));
 
     const int leftX = 120;
     const int rightX = 300;
@@ -311,23 +332,18 @@ CharCreateDialog::CharCreateDialog(CharSelectDialog *const parent,
 
 CharCreateDialog::~CharCreateDialog()
 {
-    delete mPlayer;
-    mPlayer = nullptr;
+    delete2(mPlayer);
 
     if (Net::getCharServerHandler())
         Net::getCharServerHandler()->setCharCreateDialog(nullptr);
 }
 
-void CharCreateDialog::action(const gcn::ActionEvent &event)
+void CharCreateDialog::action(const ActionEvent &event)
 {
-    const std::string id = event.getId();
+    const std::string &id = event.getId();
     if (id == "create")
     {
-        if (
-#ifdef MANASERV_SUPPORT
-            Net::getNetworkType() == ServerInfo::MANASERV ||
-#endif
-            getName().length() >= 4)
+        if (getName().length() >= 4)
         {
             // Attempt to create the character
             mCreateButton->setEnabled(false);
@@ -339,18 +355,12 @@ void CharCreateDialog::action(const gcn::ActionEvent &event)
                     mAttributeSlider[i]->getValue()));
             }
 
-#ifdef MANASERV_SUPPORT
-            int characterSlot = mSlot;
-            // On Manaserv, the slots start at 1, so we offset them.
-            if (Net::getNetworkType() == ServerInfo::MANASERV)
-                ++characterSlot;
-#else
             const int characterSlot = mSlot;
-#endif
 
             Net::getCharServerHandler()->newCharacter(getName(), characterSlot,
                 mFemale->isSelected(), mHairStyle, mHairColor,
-                static_cast<unsigned char>(mRace), mLook, atts);
+                static_cast<unsigned char>(mRace),
+                static_cast<unsigned char>(mLook), atts);
         }
         else
         {
@@ -358,7 +368,7 @@ void CharCreateDialog::action(const gcn::ActionEvent &event)
             new OkDialog(_("Error"),
                 // TRANSLATORS: char creation error
                 _("Your name needs to be at least 4 characters."),
-                DIALOG_ERROR, true,  this);
+                DialogType::ERROR, true,  this);
         }
     }
     else if (id == "cancel")
@@ -412,9 +422,9 @@ void CharCreateDialog::action(const gcn::ActionEvent &event)
     else if (id == "gender")
     {
         if (mMale->isSelected())
-            mPlayer->setGender(GENDER_MALE);
+            mPlayer->setGender(Gender::MALE);
         else
-            mPlayer->setGender(GENDER_FEMALE);
+            mPlayer->setGender(Gender::FEMALE);
     }
     else if (id == "action")
     {
@@ -497,17 +507,14 @@ void CharCreateDialog::setAttributes(const StringVect &labels,
 {
     mMaxPoints = available;
 
-    for (unsigned i = 0; i < mAttributeLabel.size(); i++)
+    for (size_t i = 0; i < mAttributeLabel.size(); i++)
     {
         remove(mAttributeLabel[i]);
-        delete mAttributeLabel[i];
-        mAttributeLabel[i] = nullptr;
+        delete2(mAttributeLabel[i])
         remove(mAttributeSlider[i]);
-        delete mAttributeSlider[i];
-        mAttributeSlider[i] = nullptr;
+        delete2(mAttributeSlider[i])
         remove(mAttributeValue[i]);
-        delete mAttributeValue[i];
-        mAttributeValue[i] = nullptr;
+        delete2(mAttributeValue[i])
     }
 
     mAttributeLabel.resize(labels.size());
@@ -527,9 +534,8 @@ void CharCreateDialog::setAttributes(const StringVect &labels,
         mAttributeLabel[i]->adjustSize();
         add(mAttributeLabel[i]);
 
-        mAttributeSlider[i] = new Slider(min, max);
-        mAttributeSlider[i]->setDimension(gcn::Rectangle(140, y + i * 24,
-                                                         150, 12));
+        mAttributeSlider[i] = new Slider(this, min, max, 1.0);
+        mAttributeSlider[i]->setDimension(Rect(140, y + i * 24, 150, 12));
         mAttributeSlider[i]->setActionEventId("statslider");
         mAttributeSlider[i]->addActionListener(this);
         add(mAttributeSlider[i]);
@@ -543,15 +549,16 @@ void CharCreateDialog::setAttributes(const StringVect &labels,
     setButtonsPosition(w, h);
 }
 
-void CharCreateDialog::setFixedGender(const bool fixed, const Gender gender)
+void CharCreateDialog::setFixedGender(const bool fixed,
+                                      const Gender::Type gender)
 {
-    if (gender == GENDER_FEMALE)
+    if (gender == Gender::FEMALE)
     {
         mFemale->setSelected(true);
         mMale->setSelected(false);
         mOther->setSelected(false);
     }
-    else if (gender == GENDER_MALE)
+    else if (gender == Gender::MALE)
     {
         mFemale->setSelected(false);
         mMale->setSelected(true);
@@ -634,7 +641,8 @@ void CharCreateDialog::updateLook()
     {
         mLook = 0;
     }
-    mPlayer->setSubtype(static_cast<uint16_t>(mRace), mLook);
+    mPlayer->setSubtype(static_cast<uint16_t>(mRace),
+        static_cast<uint8_t>(mLook));
     if (mRaceNameLabel)
     {
         mRaceNameLabel->setCaption(item.getName());
@@ -664,14 +672,14 @@ void CharCreateDialog::updatePlayer()
     }
 }
 
-void CharCreateDialog::keyPressed(gcn::KeyEvent &keyEvent)
+void CharCreateDialog::keyPressed(KeyEvent &event)
 {
-    const int actionId = static_cast<KeyEvent*>(&keyEvent)->getActionId();
+    const int actionId = event.getActionId();
     switch (actionId)
     {
-        case Input::KEY_GUI_CANCEL:
-            keyEvent.consume();
-            action(gcn::ActionEvent(mCancelButton,
+        case InputAction::GUI_CANCEL:
+            event.consume();
+            action(ActionEvent(mCancelButton,
                 mCancelButton->getActionEventId()));
             break;
 
