@@ -26,11 +26,14 @@
 #include "configuration.h"
 #include "units.h"
 
-#include "input/keydata.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
+
+#include "input/inputaction.h"
+
+#include "being/attributes.h"
 
 #include "gui/windows/charcreatedialog.h"
-#include "gui/windows/confirmdialog.h"
+#include "gui/windows/chardeleteconfirm.h"
 #include "gui/windows/logindialog.h"
 #include "gui/windows/okdialog.h"
 #include "gui/windows/textdialog.h"
@@ -39,57 +42,26 @@
 #include "gui/widgets/characterdisplay.h"
 #include "gui/widgets/characterviewnormal.h"
 #include "gui/widgets/characterviewsmall.h"
-#include "gui/widgets/layout.h"
+#include "gui/widgets/containerplacer.h"
 
+#include "net/character.h"
 #include "net/logindata.h"
 #include "net/loginhandler.h"
-
-#include "utils/gettext.h"
+#include "net/net.h"
+#include "net/registrationoptions.h"
 
 #include "debug.h"
 
 // Character slots per row in the dialog
 static const int SLOTS_PER_ROW = 5;
 
-/**
- * Listener for confirming character deletion.
- */
-class CharDeleteConfirm final : public ConfirmDialog
-{
-    public:
-        CharDeleteConfirm(CharSelectDialog *const m, const int index) :
-            // TRANSLATORS: char deletion message
-            ConfirmDialog(_("Confirm Character Delete"),
-                          // TRANSLATORS: char deletion message
-                          _("Are you sure you want to delete this character?"),
-                          SOUND_REQUEST, false, false, m),
-            mMaster(m),
-            mIndex(index)
-        {
-        }
-
-        A_DELETE_COPY(CharDeleteConfirm)
-
-        void action(const gcn::ActionEvent &event)
-        {
-            if (event.getId() == "yes" && mMaster)
-                mMaster->askPasswordForDeletion(mIndex);
-
-            ConfirmDialog::action(event);
-        }
-
-    private:
-        CharSelectDialog *mMaster;
-        int mIndex;
-};
-
-CharSelectDialog::CharSelectDialog(LoginData *const data):
+CharSelectDialog::CharSelectDialog(LoginData *const data) :
     // TRANSLATORS: char select dialog name
     Window(strprintf(_("Account %s (last login time %s)"),
         data->username.c_str(), data->lastLogin.c_str()),
         false, nullptr, "char.xml"),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
+    ActionListener(),
+    KeyListener(),
     mLoginData(data),
     // TRANSLATORS: char select dialog. button.
     mSwitchLoginButton(new Button(this, _("Switch Login"), "switch", this)),
@@ -125,7 +97,7 @@ CharSelectDialog::CharSelectDialog(LoginData *const data):
     placer(0, 0, mSwitchLoginButton);
 
     int n = 1;
-    if (optionalActions & Net::LoginHandler::Unregister)
+    if (optionalActions & Net::RegistrationOptions::Unregister)
     {
         // TRANSLATORS: char select dialog. button.
         mUnregisterButton = new Button(this, _("Unregister"),
@@ -137,7 +109,7 @@ CharSelectDialog::CharSelectDialog(LoginData *const data):
     placer(n, 0, mChangePasswordButton);
     n ++;
 
-    if (optionalActions & Net::LoginHandler::ChangeEmail)
+    if (optionalActions & Net::RegistrationOptions::ChangeEmail)
     {
         // TRANSLATORS: char select dialog. button.
         mChangeEmailButton = new Button(this, _("Change Email"),
@@ -205,10 +177,10 @@ void CharSelectDialog::postInit()
     requestFocus();
 }
 
-void CharSelectDialog::action(const gcn::ActionEvent &event)
+void CharSelectDialog::action(const ActionEvent &event)
 {
     // Check if a button of a character was pressed
-    const gcn::Widget *const sourceParent = event.getSource()->getParent();
+    const Widget *const sourceParent = event.getSource()->getParent();
     int selected = -1;
     for (unsigned int i = 0, sz = static_cast<unsigned int>(
          mCharacterEntries.size()); i < sz; ++i)
@@ -254,15 +226,21 @@ void CharSelectDialog::action(const gcn::ActionEvent &event)
                 // TRANSLATORS: char select dialog. player info message.
                 _("Hp: %u/%u\nMp: %u/%u\nLevel: %u\n"
                 "Experience: %u\nMoney: %s"),
-                character->data.mAttributes[PlayerInfo::HP],
-                character->data.mAttributes[PlayerInfo::MAX_HP],
-                character->data.mAttributes[PlayerInfo::MP],
-                character->data.mAttributes[PlayerInfo::MAX_MP],
-                character->data.mAttributes[PlayerInfo::LEVEL],
-                character->data.mAttributes[PlayerInfo::EXP],
+                static_cast<uint32_t>(
+                character->data.mAttributes[Attributes::HP]),
+                static_cast<uint32_t>(
+                character->data.mAttributes[Attributes::MAX_HP]),
+                static_cast<uint32_t>(
+                character->data.mAttributes[Attributes::MP]),
+                static_cast<uint32_t>(
+                character->data.mAttributes[Attributes::MAX_MP]),
+                static_cast<uint32_t>(
+                character->data.mAttributes[Attributes::LEVEL]),
+                static_cast<uint32_t>(
+                character->data.mAttributes[Attributes::EXP]),
                 Units::formatCurrency(
-                character->data.mAttributes[PlayerInfo::MONEY]).c_str());
-            new OkDialog(data->getName(), msg, DIALOG_SILENCE);
+                character->data.mAttributes[Attributes::MONEY]).c_str());
+            new OkDialog(data->getName(), msg, DialogType::SILENCE);
         }
     }
     if (eventId == "switch")
@@ -294,7 +272,8 @@ void CharSelectDialog::action(const gcn::ActionEvent &event)
         else
         {
             // TRANSLATORS: error message
-            new OkDialog(_("Error"), _("Incorrect password"), DIALOG_ERROR);
+            new OkDialog(_("Error"), _("Incorrect password"),
+                DialogType::ERROR);
         }
         mDeleteIndex = -1;
     }
@@ -315,20 +294,20 @@ void CharSelectDialog::use(const int selected)
     }
 }
 
-void CharSelectDialog::keyPressed(gcn::KeyEvent &keyEvent)
+void CharSelectDialog::keyPressed(KeyEvent &event)
 {
-    const int actionId = static_cast<KeyEvent*>(&keyEvent)->getActionId();
+    const int actionId = event.getActionId();
     switch (actionId)
     {
-        case Input::KEY_GUI_CANCEL:
-            keyEvent.consume();
-            action(gcn::ActionEvent(mSwitchLoginButton,
+        case InputAction::GUI_CANCEL:
+            event.consume();
+            action(ActionEvent(mSwitchLoginButton,
                 mSwitchLoginButton->getActionEventId()));
             break;
 
-        case Input::KEY_GUI_RIGHT:
+        case InputAction::GUI_RIGHT:
         {
-            keyEvent.consume();
+            event.consume();
             int idx = mCharacterView->getSelected();
             if (idx >= 0)
             {
@@ -341,9 +320,9 @@ void CharSelectDialog::keyPressed(gcn::KeyEvent &keyEvent)
             break;
         }
 
-        case Input::KEY_GUI_LEFT:
+        case InputAction::GUI_LEFT:
         {
-            keyEvent.consume();
+            event.consume();
             int idx = mCharacterView->getSelected();
             if (idx >= 0)
             {
@@ -356,9 +335,9 @@ void CharSelectDialog::keyPressed(gcn::KeyEvent &keyEvent)
             break;
         }
 
-        case Input::KEY_GUI_UP:
+        case InputAction::GUI_UP:
         {
-            keyEvent.consume();
+            event.consume();
             int idx = mCharacterView->getSelected();
             if (idx >= 0)
             {
@@ -371,9 +350,9 @@ void CharSelectDialog::keyPressed(gcn::KeyEvent &keyEvent)
             break;
         }
 
-        case Input::KEY_GUI_DOWN:
+        case InputAction::GUI_DOWN:
         {
-            keyEvent.consume();
+            event.consume();
             int idx = mCharacterView->getSelected();
             if (idx >= 0)
             {
@@ -386,9 +365,9 @@ void CharSelectDialog::keyPressed(gcn::KeyEvent &keyEvent)
             break;
         }
 
-        case Input::KEY_GUI_DELETE:
+        case InputAction::GUI_DELETE:
         {
-            keyEvent.consume();
+            event.consume();
             const int idx = mCharacterView->getSelected();
             if (idx >= 0 && mCharacterEntries[idx]
                 && mCharacterEntries[idx]->getCharacter())
@@ -398,9 +377,9 @@ void CharSelectDialog::keyPressed(gcn::KeyEvent &keyEvent)
             break;
         }
 
-        case Input::KEY_GUI_SELECT:
+        case InputAction::GUI_SELECT:
         {
-            keyEvent.consume();
+            event.consume();
             use(mCharacterView->getSelected());
             break;
         }
@@ -471,15 +450,7 @@ void CharSelectDialog::setCharacters(const Net::Characters &characters)
 
         Net::Character *const character = *i;
 
-        // Slots Number start at 1 for Manaserv, so we offset them by one.
-#ifdef MANASERV_SUPPORT
-        int characterSlot = character->slot;
-        if (Net::getNetworkType() == ServerInfo::MANASERV && characterSlot > 0)
-            --characterSlot;
-#else
         const int characterSlot = character->slot;
-#endif
-
         if (characterSlot >= static_cast<int>(mCharacterEntries.size()))
         {
             logger->log("Warning: slot out of range: %d", character->slot);
@@ -563,7 +534,7 @@ void CharSelectDialog::close()
     Window::close();
 }
 
-void CharSelectDialog::widgetResized(const gcn::Event &event)
+void CharSelectDialog::widgetResized(const Event &event)
 {
     Window::widgetResized(event);
     if (mCharacterView)
