@@ -23,186 +23,75 @@
 #include "gui/windows/chatwindow.h"
 
 #include "actormanager.h"
-#include "client.h"
 #include "commandhandler.h"
 #include "configuration.h"
 #include "game.h"
 #include "guild.h"
 #include "party.h"
+#include "settings.h"
 #include "spellshortcut.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 #include "being/playerrelations.h"
 
-#include "input/inputmanager.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
 
-#include "gui/sdlfont.h"
-#include "gui/sdlinput.h"
-#include "gui/viewport.h"
+#include "input/inputmanager.h"
+
+#include "gui/chatconsts.h"
+#include "gui/focushandler.h"
+#include "gui/gui.h"
+#include "gui/popupmanager.h"
+#include "gui/skin.h"
+
+#include "gui/models/colorlistmodel.h"
 
 #include "gui/windows/emotewindow.h"
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
+#include "gui/widgets/tabbedarea.h"
 #include "gui/windows/whoisonline.h"
 
 #include "gui/widgets/tabs/battletab.h"
 
 #include "gui/widgets/button.h"
+#include "gui/widgets/chatinput.h"
 #include "gui/widgets/dropdown.h"
 #include "gui/widgets/itemlinkhandler.h"
 #include "gui/widgets/scrollarea.h"
-#include "gui/widgets/textfield.h"
 
 #include "gui/widgets/tabs/langtab.h"
 #include "gui/widgets/tabs/tradetab.h"
 #include "gui/widgets/tabs/whispertab.h"
 
+#include "render/opengldebug.h"
+
 #include "net/chathandler.h"
 #include "net/playerhandler.h"
 #include "net/net.h"
 
 #include "utils/copynpaste.h"
+#include "utils/delete2.h"
+#include "utils/files.h"
 #include "utils/gettext.h"
 
-#include "resources/resourcemanager.h"
-
-#include <guichan/focushandler.hpp>
-
 #include <sstream>
 
 #include <sys/stat.h>
 
 #include "debug.h"
 
-/**
- * The chat input hides when it loses focus. It is also invisible by default.
- */
-class ChatInput final : public TextField
-{
-    public:
-        explicit ChatInput(ChatWindow *const window):
-            TextField(window, "", false),
-            mWindow(window),
-            mFocusGaining(false)
-        {
-            setVisible(false);
-            addFocusListener(this);
-        }
-
-        A_DELETE_COPY(ChatInput)
-
-        /**
-         * Called if the chat input loses focus. It will set itself to
-         * invisible as result.
-         */
-        void focusLost(const gcn::Event &event)
-        {
-            TextField::focusLost(event);
-            if (mFocusGaining || !config.getBoolValue("protectChatFocus"))
-            {
-                processVisible(false);
-                if (chatWindow)
-                    chatWindow->updateVisibility();
-                mFocusGaining = false;
-                return;
-            }
-            mFocusGaining = true;
-            requestFocus();
-            mFocusGaining = false;
-        }
-
-        void processVisible(const bool n)
-        {
-            if (!mWindow || isVisible() == n)
-                return;
-
-            if (!n)
-                mFocusGaining = true;
-            setVisible(n);
-            if (config.getBoolValue("hideChatInput")
-                || config.getBoolValue("showEmotesButton"))
-            {
-                mWindow->adjustTabSize();
-            }
-            if (emoteWindow)
-            {
-                emoteWindow->hide();
-            }
-        }
-
-        void unprotectFocus()
-        { mFocusGaining = true; }
-
-        void setVisible(bool visible)
-        {
-            TextField::setVisible(visible);
-        }
-
-    private:
-        ChatWindow *mWindow;
-        bool mFocusGaining;
-};
-
-const char *COLOR_NAME[14] =
-{
-    // TRANSLATORS: chat color
-    N_("default"),
-    // TRANSLATORS: chat color
-    N_("black"),
-    // TRANSLATORS: chat color
-    N_("red"),
-    // TRANSLATORS: chat color
-    N_("green"),
-    // TRANSLATORS: chat color
-    N_("blue"),
-    // TRANSLATORS: chat color
-    N_("gold"),
-    // TRANSLATORS: chat color
-    N_("yellow"),
-    // TRANSLATORS: chat color
-    N_("pink"),
-    // TRANSLATORS: chat color
-    N_("purple"),
-    // TRANSLATORS: chat color
-    N_("grey"),
-    // TRANSLATORS: chat color
-    N_("brown"),
-    // TRANSLATORS: chat color
-    N_("rainbow 1"),
-    // TRANSLATORS: chat color
-    N_("rainbow 2"),
-    // TRANSLATORS: chat color
-    N_("rainbow 3"),
-};
-
-
-class ColorListModel final : public gcn::ListModel
-{
-public:
-    ~ColorListModel()
-    { }
-
-    int getNumberOfElements()
-    {
-        return 14;
-    }
-
-    std::string getElementAt(int i)
-    {
-        if (i >= getNumberOfElements() || i < 0)
-            return "???";
-        return gettext(COLOR_NAME[i]);
-    }
-};
+ChatWindow *chatWindow = nullptr;
 
 static const char *const ACTION_COLOR_PICKER = "color picker";
 
-ChatWindow::ChatWindow():
+ChatWindow::ChatWindow() :
     // TRANSLATORS: chat window name
     Window(_("Chat"), false, nullptr, "chat.xml"),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
+    ActionListener(),
+    KeyListener(),
+    AttributeListener(),
     mItemLinkHandler(new ItemLinkHandler),
     mChatTabs(new TabbedArea(this)),
     mChatInput(new ChatInput(this)),
@@ -212,16 +101,16 @@ ChatWindow::ChatWindow():
     mCurHist(),
     mCommands(),
     mCustomWords(),
-    mReturnToggles(config.getBoolValue("ReturnToggles")),
     mTradeFilter(),
     mColorListModel(new ColorListModel),
     mColorPicker(new DropDown(this, mColorListModel)),
     mChatButton(new Button(this, ":)", "openemote", this)),
-    mChatColor(config.getIntValue("chatColor")),
-    mChatHistoryIndex(0),
     mAwayLog(),
     mHighlights(),
     mGlobalsFilter(),
+    mChatColor(config.getIntValue("chatColor")),
+    mChatHistoryIndex(0),
+    mReturnToggles(config.getBoolValue("ReturnToggles")),
     mGMLoaded(false),
     mHaveMouse(false),
     mAutoHide(config.getBoolValue("autohideChat")),
@@ -231,8 +120,6 @@ ChatWindow::ChatWindow():
 {
     mChatTabs->postInit();
 
-    listen(CHANNEL_ATTRIBUTES);
-
     setWindowName("Chat");
 
     if (setupWindow)
@@ -277,6 +164,7 @@ ChatWindow::ChatWindow():
 
     mChatInput->setActionEventId("chatinput");
     mChatInput->addActionListener(this);
+    mChatInput->setAllowSpecialActions(false);
 
     mColorPicker->setActionEventId(ACTION_COLOR_PICKER);
     mColorPicker->addActionListener(this);
@@ -294,7 +182,7 @@ ChatWindow::ChatWindow():
     updateTabsMargin();
 
     fillCommands();
-    if (player_node && player_node->isGM())
+    if (localPlayer && localPlayer->isGM())
         loadGMCommands();
     initTradeFilter();
     loadCustomList();
@@ -315,12 +203,9 @@ ChatWindow::~ChatWindow()
     saveState();
     config.setValue("ReturnToggles", mReturnToggles);
     removeAllWhispers();
-    delete mItemLinkHandler;
-    mItemLinkHandler = nullptr;
-    delete mColorPicker;
-    mColorPicker = nullptr;
-    delete mColorListModel;
-    mColorListModel = nullptr;
+    delete2(mItemLinkHandler);
+    delete2(mColorPicker);
+    delete2(mColorListModel);
 }
 
 void ChatWindow::postInit()
@@ -335,7 +220,7 @@ void ChatWindow::postInit()
 void ChatWindow::loadCommandsFile(const std::string &name)
 {
     StringVect list;
-    ResourceManager::loadTextFile(name, list);
+    Files::loadTextFile(name, list);
     StringVectCIter it = list.begin();
     const StringVectCIter it_end = list.end();
 
@@ -365,19 +250,15 @@ void ChatWindow::loadGMCommands()
 
 void ChatWindow::updateTabsMargin()
 {
-    if (mColorPicker->isVisible())
-    {
+    if (mColorPicker->isVisibleLocal())
         mChatTabs->setRightMargin(mColorPicker->getWidth() + 16 + 8);
-    }
     else
-    {
         mChatTabs->setRightMargin(8);
-    }
 }
 
 void ChatWindow::adjustTabSize()
 {
-    const gcn::Rectangle area = getChildrenArea();
+    const Rect area = getChildrenArea();
 
     const int aw = area.width;
     const int ah = area.height;
@@ -390,7 +271,7 @@ void ChatWindow::adjustTabSize()
     mChatInput->setPosition(frame, y);
     mChatTabs->setWidth(awFrame2);
     const int height = ah - frame2 - (inputHeight + frame2);
-    if (mChatInput->isVisible() || !config.getBoolValue("hideChatInput"))
+    if (mChatInput->isVisibleLocal() || !config.getBoolValue("hideChatInput"))
         mChatTabs->setHeight(height);
     else
         mChatTabs->setHeight(height + inputHeight);
@@ -400,7 +281,7 @@ void ChatWindow::adjustTabSize()
     {
         const int chatButtonSize = 20;
         int w = awFrame2 - chatButtonSize;
-        int x = aw - frame - chatButtonSize;
+        const int x = aw - frame - chatButtonSize;
         if (mSkin)
         {
             const int ipad = mSkin->getOption("emoteButtonSpacing", 2);
@@ -413,18 +294,19 @@ void ChatWindow::adjustTabSize()
             y -= 2;
         }
         mChatInput->setWidth(w);
-        mChatButton->setVisible(mChatInput->isVisible());
+        mChatButton->setVisible(mChatInput->isVisibleLocal());
         mChatButton->setPosition(x, y);
     }
     else
     {
+        mChatInput->setWidth(awFrame2);
         mChatButton->setVisible(false);
     }
 
     const ChatTab *const tab = getFocused();
     if (tab)
     {
-        gcn::Widget *const content = tab->mScrollArea;
+        Widget *const content = tab->mScrollArea;
         if (content)
         {
             const int contentFrame2 = 2 * content->getFrameSize();
@@ -440,7 +322,7 @@ void ChatWindow::adjustTabSize()
     mChatTabs->adjustSize();
 }
 
-void ChatWindow::widgetResized(const gcn::Event &event)
+void ChatWindow::widgetResized(const Event &event)
 {
     Window::widgetResized(event);
 
@@ -452,7 +334,7 @@ ChatTab *ChatWindow::getFocused() const
     return static_cast<ChatTab*>(mChatTabs->getSelectedTab());
 }
 
-void ChatWindow::clearTab(ChatTab *const tab) const
+void ChatWindow::clearTab(ChatTab *const tab)
 {
     if (tab)
         tab->clearText();
@@ -513,7 +395,7 @@ void ChatWindow::defaultTab()
         mChatTabs->setSelectedTabByIndex(static_cast<unsigned>(0));
 }
 
-void ChatWindow::action(const gcn::ActionEvent &event)
+void ChatWindow::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "chatinput")
@@ -565,7 +447,7 @@ void ChatWindow::action(const gcn::ActionEvent &event)
     {
         if (emoteWindow)
         {
-            if (emoteWindow->isVisible())
+            if (emoteWindow->isVisibleLocal())
                 emoteWindow->hide();
             else
                 emoteWindow->show();
@@ -604,7 +486,7 @@ void ChatWindow::action(const gcn::ActionEvent &event)
         }
     }
 
-    if (mColorPicker && mColorPicker->isVisible()
+    if (mColorPicker && mColorPicker->isVisibleLocal()
         != config.getBoolValue("showChatColorsList"))
     {
         mColorPicker->setVisible(config.getBoolValue(
@@ -695,8 +577,7 @@ void ChatWindow::ignoreAllWhispers()
                                          PlayerRelation::IGNORED);
         }
 
-        delete (iter->second);
-        iter->second = nullptr;
+        delete2(iter->second)
     }
 }
 
@@ -741,7 +622,7 @@ void ChatWindow::doPresent() const
 
     FOR_EACH (ActorSpritesIterator, it, actors)
     {
-        if ((*it)->getType() == ActorSprite::PLAYER)
+        if ((*it)->getType() == ActorType::PLAYER)
         {
             if (!response.empty())
                 response.append(", ");
@@ -756,7 +637,7 @@ void ChatWindow::doPresent() const
         response.c_str(), playercount);
 
     if (getFocused())
-        getFocused()->chatLog(log, BY_SERVER);
+        getFocused()->chatLog(log, ChatMsgType::BY_SERVER);
 }
 
 void ChatWindow::scroll(const int amount) const
@@ -769,20 +650,21 @@ void ChatWindow::scroll(const int amount) const
         tab->scroll(amount);
 }
 
-void ChatWindow::mousePressed(gcn::MouseEvent &event)
+void ChatWindow::mousePressed(MouseEvent &event)
 {
     if (event.isConsumed())
         return;
 
-    if (event.getButton() == gcn::MouseEvent::RIGHT)
+    if (event.getButton() == MouseButton::RIGHT)
     {
-        if (viewport)
+        if (popupManager)
         {
             Tab *const tab = mChatTabs->getSelectedTab();
             if (tab)
             {
+                event.consume();
                 if (inputManager.isActionActive(static_cast<int>(
-                    Input::KEY_CHAT_MOD)))
+                    InputAction::CHAT_MOD)))
                 {
                     ChatTab *const wTab = dynamic_cast<WhisperTab*>(tab);
                     if (wTab)
@@ -792,7 +674,7 @@ void ChatWindow::mousePressed(gcn::MouseEvent &event)
                 {
                     ChatTab *const cTab = dynamic_cast<ChatTab*>(tab);
                     if (cTab)
-                        viewport->showChatPopup(cTab);
+                        popupManager->showChatPopup(cTab);
                 }
             }
         }
@@ -803,7 +685,7 @@ void ChatWindow::mousePressed(gcn::MouseEvent &event)
     if (event.isConsumed())
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         const ChatTab *const tab = getFocused();
         if (tab)
@@ -814,7 +696,7 @@ void ChatWindow::mousePressed(gcn::MouseEvent &event)
     mDragOffsetY = event.getY();
 }
 
-void ChatWindow::mouseDragged(gcn::MouseEvent &event)
+void ChatWindow::mouseDragged(MouseEvent &event)
 {
     Window::mouseDragged(event);
 
@@ -831,20 +713,23 @@ void ChatWindow::mouseDragged(gcn::MouseEvent &event)
     }
 }
 
-#define caseKey(key, str) case key:\
-    temp = str; \
-    break
+#define ifKey(key, str) \
+    else if (actionId == static_cast<int>(key)) \
+    { \
+        temp = str; \
+    }
 
-void ChatWindow::keyPressed(gcn::KeyEvent &event)
+void ChatWindow::keyPressed(KeyEvent &event)
 {
-    const int key = event.getKey().getValue();
-    const int actionId = static_cast<KeyEvent*>(&event)->getActionId();
-    if (actionId == static_cast<int>(Input::KEY_GUI_DOWN))
+    const int actionId = event.getActionId();
+    std::string temp;
+    if (actionId == static_cast<int>(InputAction::GUI_DOWN))
     {
         if (mCurHist != mHistory.end())
         {
             // Move forward through the history
             const HistoryIterator prevHist = mCurHist++;
+            addCurrentToHistory();
 
             if (mCurHist != mHistory.end())
             {
@@ -860,19 +745,22 @@ void ChatWindow::keyPressed(gcn::KeyEvent &event)
         }
         else if (!mChatInput->getText().empty())
         {
+            if (addCurrentToHistory())
+                mCurHist = mHistory.end();
             mChatInput->setText("");
         }
     }
-    else if (actionId == static_cast<int>(Input::KEY_GUI_UP) &&
+    else if (actionId == static_cast<int>(InputAction::GUI_UP) &&
              mCurHist != mHistory.begin() && !mHistory.empty())
     {
         // Move backward through the history
         --mCurHist;
+        addCurrentToHistory();
         mChatInput->setText(*mCurHist);
         mChatInput->setCaretPosition(static_cast<unsigned>(
                 mChatInput->getText().length()));
     }
-    else if (actionId == static_cast<int>(Input::KEY_GUI_INSERT) &&
+    else if (actionId == static_cast<int>(InputAction::GUI_INSERT) &&
              mChatInput->getText() != "")
     {
         // Add the current message to the history and clear the text
@@ -881,19 +769,19 @@ void ChatWindow::keyPressed(gcn::KeyEvent &event)
         mCurHist = mHistory.end();
         mChatInput->setText("");
     }
-    else if (actionId == static_cast<int>(Input::KEY_GUI_TAB) &&
+    else if (actionId == static_cast<int>(InputAction::GUI_TAB) &&
              !mChatInput->getText().empty())
     {
         autoComplete();
         return;
     }
-    else if (actionId == static_cast<int>(Input::KEY_GUI_CANCEL) &&
-             mChatInput->isVisible())
+    else if (actionId == static_cast<int>(InputAction::GUI_CANCEL) &&
+             mChatInput->isVisibleLocal())
     {
         mChatInput->processVisible(false);
     }
-    else if (actionId == static_cast<int>(Input::KEY_CHAT_PREV_HISTORY) &&
-             mChatInput->isVisible())
+    else if (actionId == static_cast<int>(InputAction::CHAT_PREV_HISTORY) &&
+             mChatInput->isVisibleLocal())
     {
         const ChatTab *const tab = getFocused();
         if (tab && tab->hasRows())
@@ -924,19 +812,19 @@ void ChatWindow::keyPressed(gcn::KeyEvent &event)
                 mChatInput->getText().length()));
         }
     }
-    else if (actionId == static_cast<int>(Input::KEY_CHAT_NEXT_HISTORY) &&
-             mChatInput->isVisible())
+    else if (actionId == static_cast<int>(InputAction::CHAT_NEXT_HISTORY) &&
+             mChatInput->isVisibleLocal())
     {
         const ChatTab *const tab = getFocused();
         if (tab && tab->hasRows())
         {
             const std::list<std::string> &rows = tab->getRows();
             const size_t &tabSize = rows.size();
-            if (mChatHistoryIndex + 1 < tabSize)
+            if (static_cast<size_t>(mChatHistoryIndex) + 1 < tabSize)
             {
                 mChatHistoryIndex ++;
             }
-            else if (mChatHistoryIndex < tabSize)
+            else if (static_cast<size_t>(mChatHistoryIndex) < tabSize)
             {
                 mChatHistoryIndex ++;
                 mChatInput->setText("");
@@ -960,104 +848,121 @@ void ChatWindow::keyPressed(gcn::KeyEvent &event)
                     mChatInput->getText().length()));
         }
     }
-
-    std::string temp;
-    switch (key)
+    else if (actionId == static_cast<int>(InputAction::GUI_F1))
     {
-        case Key::F1:
-            if (emoteWindow)
+        if (emoteWindow)
+        {
+            if (emoteWindow->isVisibleLocal())
+                emoteWindow->hide();
+            else
+                emoteWindow->show();
+        }
+    }
+    ifKey(InputAction::GUI_F2, "\u2318")
+    ifKey(InputAction::GUI_F3, "\u263A")
+    ifKey(InputAction::GUI_F4, "\u2665")
+    ifKey(InputAction::GUI_F5, "\u266A")
+    ifKey(InputAction::GUI_F6, "\u266B")
+    ifKey(InputAction::GUI_F7, "\u26A0")
+    ifKey(InputAction::GUI_F8, "\u2622")
+    ifKey(InputAction::GUI_F9, "\u262E")
+    ifKey(InputAction::GUI_F10, "\u2605")
+    ifKey(InputAction::GUI_F11, "\u2618")
+    ifKey(InputAction::GUI_F12, "\u2592")
+
+    if (inputManager.isActionActive(static_cast<int>(InputAction::GUI_CTRL)))
+    {
+        if (actionId == static_cast<int>(InputAction::GUI_B))
+        {
+            std::string inputText = mChatInput->getTextBeforeCaret();
+            toLower(inputText);
+            const size_t idx = inputText.rfind("##b");
+            if (idx == std::string::npos
+                || mChatInput->getTextBeforeCaret().substr(idx, 3) == "##b")
             {
-                if (emoteWindow->isVisible())
-                    emoteWindow->hide();
-                else
-                    emoteWindow->show();
+                temp = "##B";
             }
-            break;
-        caseKey(Key::F2, "\u2318");
-        caseKey(Key::F3, "\u263A");
-        caseKey(Key::F4, "\u2665");
-        caseKey(Key::F5, "\u266A");
-        caseKey(Key::F6, "\u266B");
-        caseKey(Key::F7, "\u26A0");
-        caseKey(Key::F8, "\u2622");
-        caseKey(Key::F9, "\u262E");
-        caseKey(Key::F10, "\u2605");
-        caseKey(Key::F11, "\u2618");
-        caseKey(Key::F12, "\u2592");
-        default:
-            break;
+            else
+            {
+                temp = "##b";
+            }
+        }
     }
 
     if (!temp.empty())
         addInputText(temp, false);
 }
 
-void ChatWindow::processEvent(const Channels channel,
-                              const DepricatedEvent &event)
+#undef ifKey
+
+bool ChatWindow::addCurrentToHistory()
 {
-    if (channel == CHANNEL_ATTRIBUTES)
+    const std::string &str = mChatInput->getText();
+    if (str.empty())
+        return false;
+    FOR_EACH (HistoryIterator, it, mHistory)
     {
-        if (!mShowBattleEvents)
-            return;
-
-        if (event.getName() == EVENT_UPDATEATTRIBUTE)
-        {
-            switch (event.getInt("id"))
-            {
-                case PlayerInfo::EXP:
-                {
-                    if (event.getInt("oldValue") > event.getInt("newValue"))
-                        break;
-
-                    const int change = event.getInt("newValue")
-                        - event.getInt("oldValue");
-
-                    if (change != 0)
-                    {
-                        battleChatLog(std::string("+").append(toString(
-                            change)).append(" xp"));
-                    }
-                    break;
-                }
-                case PlayerInfo::LEVEL:
-                    battleChatLog(std::string("Level: ").append(toString(
-                        event.getInt("newValue"))));
-                    break;
-                default:
-                    break;
-            };
-        }
-        else if (event.getName() == EVENT_UPDATESTAT)
-        {
-            if (!config.getBoolValue("showJobExp"))
-                return;
-
-            const int id = event.getInt("id");
-            if (id == Net::getPlayerHandler()->getJobLocation())
-            {
-                const std::pair<int, int> exp
-                    = PlayerInfo::getStatExperience(id);
-                if (event.getInt("oldValue1") > exp.first
-                    || !event.getInt("oldValue2"))
-                {
-                    return;
-                }
-
-                const int change = exp.first - event.getInt("oldValue1");
-                if (change != 0)
-                {
-                    battleChatLog(std::string("+").append(
-                        toString(change)).append(" job"));
-                }
-            }
-        }
+        if (*it == str)
+            return false;
     }
+    mHistory.push_back(str);
+    return true;
+}
+
+void ChatWindow::statChanged(const int id,
+                             const int oldVal1,
+                             const int oldVal2)
+{
+    if (!mShowBattleEvents || !config.getBoolValue("showJobExp"))
+        return;
+
+    if (id != Net::getPlayerHandler()->getJobLocation())
+        return;
+
+    const std::pair<int, int> exp = PlayerInfo::getStatExperience(id);
+    if (oldVal1 > exp.first || !oldVal2)
+        return;
+
+    const int change = exp.first - oldVal1;
+    if (change != 0)
+    {
+        battleChatLog(std::string("+").append(toString(
+            change)).append(" job"));
+    }
+}
+
+void ChatWindow::attributeChanged(const int id,
+                                  const int oldVal,
+                                  const int newVal)
+{
+    if (!mShowBattleEvents)
+        return;
+    switch (id)
+    {
+        case Attributes::EXP:
+        {
+            if (oldVal > newVal)
+                break;
+            const int change = newVal - oldVal;
+            if (change != 0)
+            {
+                battleChatLog(std::string("+").append(toString(
+                    change)).append(" xp"));
+            }
+            break;
+        }
+        case Attributes::LEVEL:
+            battleChatLog(std::string("Level: ").append(toString(newVal)));
+            break;
+        default:
+            break;
+    };
 }
 
 void ChatWindow::addInputText(const std::string &text, const bool space)
 {
     const int caretPos = mChatInput->getCaretPosition();
-    const std::string inputText = mChatInput->getText();
+    const std::string &inputText = mChatInput->getText();
 
     std::ostringstream ss;
     ss << inputText.substr(0, caretPos) << text;
@@ -1090,12 +995,13 @@ void ChatWindow::setVisible(bool visible)
 }
 
 void ChatWindow::addWhisper(const std::string &restrict nick,
-                            const std::string &restrict mes, const Own own)
+                            const std::string &restrict mes,
+                            const ChatMsgType::Type own)
 {
-    if (mes.empty() || !player_node)
+    if (mes.empty() || !localPlayer)
         return;
 
-    std::string playerName = player_node->getName();
+    std::string playerName = localPlayer->getName();
     std::string tempNick = nick;
 
     toLower(playerName);
@@ -1120,11 +1026,11 @@ void ChatWindow::addWhisper(const std::string &restrict nick,
 
     if (tab)
     {
-        if (own == BY_PLAYER)
+        if (own == ChatMsgType::BY_PLAYER)
         {
             tab->chatInput(mes);
         }
-        else if (own == BY_SERVER)
+        else if (own == ChatMsgType::BY_SERVER)
         {
             tab->chatLog(mes);
         }
@@ -1149,32 +1055,32 @@ void ChatWindow::addWhisper(const std::string &restrict nick,
                 {
                     if (config.getBoolValue("removeColors"))
                         msg = removeColors(msg);
-                    tab->chatLog(msg, BY_SERVER);
+                    tab->chatLog(msg, ChatMsgType::BY_SERVER);
                 }
             }
             else
             {
                 tab->chatLog(nick, mes);
             }
-            player_node->afkRespond(tab, nick);
+            localPlayer->afkRespond(tab, nick);
         }
     }
     else if (localChatTab)
     {
-        if (own == BY_PLAYER)
+        if (own == ChatMsgType::BY_PLAYER)
         {
             Net::getChatHandler()->privateMessage(nick, mes);
 
             // TRANSLATORS: chat message
             localChatTab->chatLog(strprintf(_("Whispering to %s: %s"),
-                nick.c_str(), mes.c_str()), BY_PLAYER);
+                nick.c_str(), mes.c_str()), ChatMsgType::BY_PLAYER);
         }
         else
         {
             localChatTab->chatLog(std::string(nick).append(
-                " : ").append(mes), ACT_WHISPER, false);
-            if (player_node)
-                player_node->afkRespond(nullptr, nick);
+                " : ").append(mes), ChatMsgType::ACT_WHISPER, false);
+            if (localPlayer)
+                localPlayer->afkRespond(nullptr, nick);
         }
     }
 }
@@ -1182,10 +1088,10 @@ void ChatWindow::addWhisper(const std::string &restrict nick,
 WhisperTab *ChatWindow::addWhisperTab(const std::string &nick,
                                       const bool switchTo)
 {
-    if (!player_node)
+    if (!localPlayer)
         return nullptr;
 
-    std::string playerName = player_node->getName();
+    std::string playerName = localPlayer->getName();
     std::string tempNick = nick;
 
     toLower(playerName);
@@ -1219,10 +1125,10 @@ WhisperTab *ChatWindow::addWhisperTab(const std::string &nick,
 
 WhisperTab *ChatWindow::getWhisperTab(const std::string &nick) const
 {
-    if (!player_node)
+    if (!localPlayer)
         return nullptr;
 
-    std::string playerName = player_node->getName();
+    std::string playerName = localPlayer->getName();
     std::string tempNick = nick;
 
     toLower(playerName);
@@ -1311,8 +1217,7 @@ void ChatWindow::autoComplete()
 {
     const int caretPos = mChatInput->getCaretPosition();
     int startName = 0;
-    const std::string inputText = mChatInput->getText();
-    bool needSecure(false);
+    const std::string &inputText = mChatInput->getText();
     std::string name = inputText.substr(0, caretPos);
 
     for (int f = caretPos - 1; f > -1; f --)
@@ -1335,15 +1240,21 @@ void ChatWindow::autoComplete()
     if (cTab)
         cTab->getAutoCompleteList(nameList);
     std::string newName = autoComplete(nameList, name);
-    if (!newName.empty())
-        needSecure = true;
+    if (!newName.empty() && !startName)
+        secureChatCommand(newName);
+
+    if (cTab && newName.empty())
+    {
+        cTab->getAutoCompleteCommands(nameList);
+        newName = autoComplete(nameList, name);
+    }
 
     if (newName.empty() && actorManager)
     {
         actorManager->getPlayerNames(nameList, true);
         newName = autoComplete(nameList, name);
-        if (!newName.empty())
-            needSecure = true;
+        if (!newName.empty() && !startName)
+            secureChatCommand(newName);
     }
     if (newName.empty())
         newName = autoCompleteHistory(name);
@@ -1366,11 +1277,6 @@ void ChatWindow::autoComplete()
 
     if (!newName.empty())
     {
-        if (!startName && needSecure && (newName[0] == '/'
-            || newName[0] == '@' || newName[0] == '#'))
-        {
-            newName = "_" + newName;
-        }
         mChatInput->setText(inputText.substr(0, startName).append(newName)
             .append(inputText.substr(caretPos,
             inputText.length() - caretPos)));
@@ -1385,8 +1291,8 @@ void ChatWindow::autoComplete()
     }
 }
 
-std::string ChatWindow::autoComplete(StringVect &names,
-                                     std::string partName) const
+std::string ChatWindow::autoComplete(const StringVect &names,
+                                     std::string partName)
 {
     StringVectCIter i = names.begin();
     const StringVectCIter i_end = names.end();
@@ -1416,7 +1322,7 @@ std::string ChatWindow::autoComplete(StringVect &names,
 }
 
 std::string ChatWindow::autoComplete(const std::string &partName,
-                                     History *const words) const
+                                     const History *const words) const
 {
     if (!words)
         return "";
@@ -1469,65 +1375,72 @@ std::string ChatWindow::autoCompleteHistory(const std::string &partName) const
     return autoComplete(nameList, partName);
 }
 
-bool ChatWindow::resortChatLog(std::string line, Own own,
+bool ChatWindow::resortChatLog(std::string line, ChatMsgType::Type own,
                                const std::string &channel,
                                const bool ignoreRecord,
                                const bool tryRemoveColors)
 {
-    if (own == -1)
-        own = BY_SERVER;
+    if (own == ChatMsgType::BY_UNKNOWN)
+        own = ChatMsgType::BY_SERVER;
 
     std::string prefix;
     if (!channel.empty())
         prefix = std::string("##3").append(channel).append("##0");
 
-    if (tradeChatTab)
+    if (findI(line, mTradeFilter) != std::string::npos)
     {
-        if (findI(line, mTradeFilter) != std::string::npos)
+        if (tradeChatTab)
         {
             tradeChatTab->chatLog(prefix + line, own,
                 ignoreRecord, tryRemoveColors);
-            return false;
         }
+        return false;
+    }
 
-        size_t idx2 = line.find(": ");
-        if (idx2 != std::string::npos)
+    size_t idx2 = line.find(": ");
+    if (idx2 != std::string::npos)
+    {
+        const size_t idx = line.find(": \302\202");
+        if (idx == idx2)
         {
-            const size_t idx = line.find(": \302\202");
-            if (idx == idx2)
+            if (line.find(": \302\202\302") != std::string::npos)
             {
-                if (line.find(": \302\202\302") != std::string::npos)
+                if (line.find(": \302\202\302e") != std::string::npos)
                 {
-                    if (line.find(": \302\202\302e") != std::string::npos)
-                    {
-                        const std::string nick = line.substr(0, idx2 - 1);
-                        line = line.substr(idx2 + 6);
-                        localPetEmote(nick, atoi(line.c_str()));
-                    }
-                    // ignore other special message formats.
-                    return false;
+                    const std::string nick = line.substr(0, idx2 - 1);
+                    line = line.substr(idx2 + 6);
+                    localPetEmote(nick, static_cast<uint8_t>(
+                        atoi(line.c_str())));
+                }
+                // ignore other special message formats.
+                return false;
+            }
+
+            // pet talk message detected
+            if (line.find(": \302\202\303 ") != std::string::npos)
+            {
+                if (actorManager && idx2 > 1)
+                {
+                    const std::string nick = line.substr(0, idx2 - 1);
+                    line = line.substr(idx2 + 6);
+                    localPetSay(nick, line);
                 }
 
-                // pet talk message detected
-                if (line.find(": \302\202\303 ") != std::string::npos)
-                {
-                    if (actorManager && idx2 > 1)
-                    {
-                        const std::string nick = line.substr(0, idx2 - 1);
-                        line = line.substr(idx2 + 6);
-                        localPetSay(nick, line);
-                    }
-
-                    return false;
-                }
+                return false;
+            }
 
+            if (tradeChatTab)
+            {
                 line = line.erase(idx + 2, 2);
                 tradeChatTab->chatLog(prefix + line, own, ignoreRecord,
                     tryRemoveColors);
-                return false;
             }
+            return false;
         }
+    }
 
+    if (tradeChatTab)
+    {
         const size_t idx1 = line.find("@@");
         if (idx1 != std::string::npos)
         {
@@ -1575,12 +1488,12 @@ bool ChatWindow::resortChatLog(std::string line, Own own,
     return true;
 }
 
-void ChatWindow::battleChatLog(const std::string &line, Own own,
+void ChatWindow::battleChatLog(const std::string &line, ChatMsgType::Type own,
                                const bool ignoreRecord,
                                const bool tryRemoveColors)
 {
-    if (own == -1)
-        own = BY_SERVER;
+    if (own == ChatMsgType::BY_UNKNOWN)
+        own = ChatMsgType::BY_SERVER;
     if (battleChatTab)
         battleChatTab->chatLog(line, own, ignoreRecord, tryRemoveColors);
     else if (debugChatTab)
@@ -1590,7 +1503,7 @@ void ChatWindow::battleChatLog(const std::string &line, Own own,
 void ChatWindow::localPetSay(const std::string &nick, const std::string &text)
 {
     Being *const being = actorManager->findBeingByName(
-        nick, ActorSprite::PLAYER);
+        nick, ActorType::PLAYER);
     Being *pet = nullptr;
     if (being)
     {
@@ -1615,7 +1528,7 @@ void ChatWindow::localPetSay(const std::string &nick, const std::string &text)
 void ChatWindow::localPetEmote(const std::string &nick, const uint8_t emoteId)
 {
     Being *const being = actorManager->findBeingByName(
-        nick, ActorSprite::PLAYER);
+        nick, ActorType::PLAYER);
     if (being)
     {
         Being *const pet = being->getFirstPet();
@@ -1626,7 +1539,7 @@ void ChatWindow::localPetEmote(const std::string &nick, const uint8_t emoteId)
 
 void ChatWindow::initTradeFilter()
 {
-    const std::string tradeListName = client->getServerConfigDirectory()
+    const std::string tradeListName = settings.serverConfigDir
         + "/tradefilter.txt";
 
     std::ifstream tradeFile;
@@ -1649,14 +1562,14 @@ void ChatWindow::initTradeFilter()
     }
 }
 
-void ChatWindow::updateOnline(std::set<std::string> &onlinePlayers) const
+void ChatWindow::updateOnline(const std::set<std::string> &onlinePlayers) const
 {
     const Party *party = nullptr;
     const Guild *guild = nullptr;
-    if (player_node)
+    if (localPlayer)
     {
-        party = player_node->getParty();
-        guild = player_node->getGuild();
+        party = localPlayer->getParty();
+        guild = localPlayer->getGuild();
     }
     FOR_EACH (TabMap::const_iterator, iter, mWhispers)
     {
@@ -1673,11 +1586,11 @@ void ChatWindow::updateOnline(std::set<std::string> &onlinePlayers) const
         }
         else
         {
-            const std::string nick = tab->getNick();
+            const std::string &nick = tab->getNick();
             if (actorManager)
             {
                 const Being *const being = actorManager->findBeingByName(
-                    nick, ActorSprite::PLAYER);
+                    nick, ActorType::PLAYER);
                 if (being)
                 {
                     tab->setWhisperTabColors();
@@ -1764,7 +1677,7 @@ void ChatWindow::saveState() const
     }
 }
 
-std::string ChatWindow::doReplace(const std::string &msg) const
+std::string ChatWindow::doReplace(const std::string &msg)
 {
     std::string str = msg;
     replaceSpecialChars(str);
@@ -1776,7 +1689,7 @@ void ChatWindow::loadCustomList()
     std::ifstream listFile;
     struct stat statbuf;
 
-    std::string listName = client->getServerConfigDirectory()
+    std::string listName = settings.serverConfigDir
         + "/customwords.txt";
 
     if (!stat(listName.c_str(), &statbuf) && S_ISREG(statbuf.st_mode))
@@ -1798,7 +1711,7 @@ void ChatWindow::loadCustomList()
 
 void ChatWindow::addToAwayLog(const std::string &line)
 {
-    if (!player_node || !player_node->getAway())
+    if (!settings.awayMode)
         return;
 
     if (mAwayLog.size() > 20)
@@ -1827,25 +1740,25 @@ void ChatWindow::displayAwayLog() const
 void ChatWindow::parseHighlights()
 {
     mHighlights.clear();
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     splitToStringVector(mHighlights, config.getStringValue(
         "highlightWords"), ',');
 
-    mHighlights.push_back(player_node->getName());
+    mHighlights.push_back(localPlayer->getName());
 }
 
 void ChatWindow::parseGlobalsFilter()
 {
     mGlobalsFilter.clear();
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     splitToStringVector(mGlobalsFilter, config.getStringValue(
         "globalsFilter"), ',');
 
-    mHighlights.push_back(player_node->getName());
+    mHighlights.push_back(localPlayer->getName());
 }
 
 bool ChatWindow::findHighlight(const std::string &str)
@@ -1877,39 +1790,46 @@ void ChatWindow::optionChanged(const std::string &name)
         parseGlobalsFilter();
 }
 
-void ChatWindow::mouseMoved(gcn::MouseEvent &event)
+void ChatWindow::mouseMoved(MouseEvent &event)
 {
     mHaveMouse = true;
     Window::mouseMoved(event);
 }
 
-void ChatWindow::mouseEntered(gcn::MouseEvent& mouseEvent)
+void ChatWindow::mouseEntered(MouseEvent& event)
 {
     mHaveMouse = true;
-    Window::mouseEntered(mouseEvent);
+    Window::mouseEntered(event);
 }
 
-void ChatWindow::mouseExited(gcn::MouseEvent& mouseEvent)
+void ChatWindow::mouseExited(MouseEvent& event)
 {
     updateVisibility();
-    Window::mouseExited(mouseEvent);
+    Window::mouseExited(event);
 }
 
-void ChatWindow::draw(gcn::Graphics* graphics)
+void ChatWindow::draw(Graphics* graphics)
 {
     BLOCK_START("ChatWindow::draw")
     if (!mAutoHide || mHaveMouse)
+    {
+        GLDEBUG_START("ChatWindow::draw");
         Window::draw(graphics);
+        GLDEBUG_END();
+    }
     BLOCK_END("ChatWindow::draw")
 }
 
 void ChatWindow::updateVisibility()
 {
+    if (!gui)
+        return;
+
     int mouseX = 0;
     int mouseY = 0;
     int x = 0;
     int y = 0;
-    SDL_GetMouseState(&mouseX, &mouseY);
+    Gui::getMouseState(&mouseX, &mouseY);
     getAbsolutePosition(x, y);
     if (mChatInput->isVisible())
     {
@@ -1939,7 +1859,12 @@ void ChatWindow::logicChildren()
 void ChatWindow::addGlobalMessage(const std::string &line)
 {
     if (debugChatTab && findI(line, mGlobalsFilter) != std::string::npos)
-        debugChatTab->chatLog(line, BY_OTHER);
+        debugChatTab->chatLog(line, ChatMsgType::BY_OTHER);
     else
-        localChatTab->chatLog(line, BY_GM);
+        localChatTab->chatLog(line, ChatMsgType::BY_GM);
+}
+
+bool ChatWindow::isTabPresent(const ChatTab *const tab) const
+{
+    return mChatTabs->isTabPresent(tab);
 }
