@@ -1,5 +1,5 @@
 /*
- *  The Mana Client
+ *  The ManaPlus Client
  *  Copyright (C) 2011-2012  The Mana Developers
  *  Copyright (C) 2012-2014  The ManaPlus Developers
  *
@@ -21,13 +21,17 @@
 
 #include "gui/windows/editserverdialog.h"
 
-#include "input/keydata.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
+
+#include "input/inputaction.h"
+
+#include "gui/models/typelistmodel.h"
 
 #include "gui/windows/okdialog.h"
 #include "gui/windows/serverdialog.h"
 
 #include "gui/widgets/button.h"
+#include "gui/widgets/checkbox.h"
 #include "gui/widgets/dropdown.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layout.h"
@@ -35,36 +39,15 @@
 
 #include "utils/gettext.h"
 
-std::string TypeListModel::getElementAt(int elementIndex)
-{
-    if (elementIndex == 0)
-        return "TmwAthena";
-    else if (elementIndex == 1)
-        return "Evol";
-#ifdef EATHENA_SUPPORT
-    else if (elementIndex == 2)
-        return "eAthena";
-#ifdef MANASERV_SUPPORT
-    else if (elementIndex == 3)
-        return "ManaServ";
-#endif
-#else
-#ifdef MANASERV_SUPPORT
-    else if (elementIndex == 2)
-        return "ManaServ";
-#endif
-#endif
-    else
-        return "Unknown";
-}
+#include "debug.h"
 
 EditServerDialog::EditServerDialog(ServerDialog *const parent,
                                    ServerInfo server,
                                    const int index) :
     // TRANSLATORS: edit server dialog name
     Window(_("Edit Server"), true, parent),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
+    ActionListener(),
+    KeyListener(),
     mServerAddressField(new TextField(this, std::string())),
     mPortField(new TextField(this, std::string())),
     mNameField(new TextField(this, std::string())),
@@ -76,6 +59,9 @@ EditServerDialog::EditServerDialog(ServerDialog *const parent,
     mOkButton(new Button(this, _("OK"), "addServer", this)),
     // TRANSLATORS: edit server dialog button
     mCancelButton(new Button(this, _("Cancel"), "cancel", this)),
+    // TRANSLATORS: edit server dialog label
+    mPersistentIp(new CheckBox(this, _("Use same ip"),
+                  true, this, "persistentIp")),
     mTypeListModel(new TypeListModel),
     mTypeField(new DropDown(this, mTypeListModel, false, true)),
     mServerDialog(parent),
@@ -116,9 +102,10 @@ EditServerDialog::EditServerDialog(ServerDialog *const parent,
     place(1, 4, mDescriptionField, 4).setPadding(3);
     place(0, 5, onlineListUrlLabel);
     place(1, 5, mOnlineListUrlField, 4).setPadding(3);
-    place(0, 6, mConnectButton);
-    place(4, 6, mOkButton);
-    place(3, 6, mCancelButton);
+    place(0, 6, mPersistentIp, 4).setPadding(3);
+    place(0, 7, mConnectButton);
+    place(4, 7, mOkButton);
+    place(3, 7, mCancelButton);
 
     // Do this manually instead of calling reflowLayout so we can enforce a
     // minimum width.
@@ -152,6 +139,7 @@ EditServerDialog::EditServerDialog(ServerDialog *const parent,
     mOnlineListUrlField->setText(mServer.onlineListUrl);
     mServerAddressField->setText(mServer.hostname);
     mPortField->setText(toString(mServer.port));
+    mPersistentIp->setSelected(mServer.persistentIp);
 
     switch (mServer.type)
     {
@@ -159,17 +147,6 @@ EditServerDialog::EditServerDialog(ServerDialog *const parent,
         case ServerInfo::EATHENA:
             mTypeField->setSelected(2);
             break;
-        case ServerInfo::MANASERV:
-#ifdef MANASERV_SUPPORT
-            mTypeField->setSelected(3);
-            break;
-#endif
-#else
-        case ServerInfo::MANASERV:
-#ifdef MANASERV_SUPPORT
-            mTypeField->setSelected(2);
-            break;
-#endif
 #endif
         default:
         case ServerInfo::UNKNOWN:
@@ -198,7 +175,7 @@ void EditServerDialog::postInit()
     mNameField->requestFocus();
 }
 
-void EditServerDialog::action(const gcn::ActionEvent &event)
+void EditServerDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
 
@@ -217,7 +194,7 @@ void EditServerDialog::action(const gcn::ActionEvent &event)
             OkDialog *const dlg = new OkDialog(_("Error"),
                 // TRANSLATORS: edit server dialog error message
                 _("Please at least type both the address and the port "
-                  "of the server."), DIALOG_ERROR);
+                  "of the server."), DialogType::ERROR);
             dlg->addActionListener(this);
         }
         else
@@ -231,6 +208,7 @@ void EditServerDialog::action(const gcn::ActionEvent &event)
             mServer.hostname = mServerAddressField->getText();
             mServer.port = static_cast<int16_t>(atoi(
                 mPortField->getText().c_str()));
+            mServer.persistentIp = mPersistentIp->isSelected();
 
             if (mTypeField)
             {
@@ -246,17 +224,6 @@ void EditServerDialog::action(const gcn::ActionEvent &event)
                     case 2:
                         mServer.type = ServerInfo::EATHENA;
                         break;
-#ifdef MANASERV_SUPPORT
-                    case 3:
-                        mServer.type = ServerInfo::MANASERV;
-                        break;
-#endif
-#else
-#ifdef MANASERV_SUPPORT
-                    case 2:
-                        mServer.type = ServerInfo::MANASERV;
-                        break;
-#endif
 #endif
                     default:
                         mServer.type = ServerInfo::UNKNOWN;
@@ -284,21 +251,20 @@ void EditServerDialog::action(const gcn::ActionEvent &event)
     }
 }
 
-void EditServerDialog::keyPressed(gcn::KeyEvent &keyEvent)
+void EditServerDialog::keyPressed(KeyEvent &event)
 {
-    if (keyEvent.isConsumed())
+    if (event.isConsumed())
         return;
 
-    const int actionId = static_cast<KeyEvent*>(
-        &keyEvent)->getActionId();
+    const int actionId = event.getActionId();
 
-    if (actionId == static_cast<int>(Input::KEY_GUI_CANCEL))
+    if (actionId == static_cast<int>(InputAction::GUI_CANCEL))
     {
         scheduleDelete();
     }
-    else if (actionId == static_cast<int>(Input::KEY_GUI_SELECT)
-             || actionId == static_cast<int>(Input::KEY_GUI_SELECT2))
+    else if (actionId == static_cast<int>(InputAction::GUI_SELECT)
+             || actionId == static_cast<int>(InputAction::GUI_SELECT2))
     {
-        action(gcn::ActionEvent(nullptr, mOkButton->getActionEventId()));
+        action(ActionEvent(nullptr, mOkButton->getActionEventId()));
     }
 }
