@@ -32,59 +32,63 @@
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 
-#include "gui/viewport.h"
+#include "gui/popupmanager.h"
+
+#include "gui/fonts/font.h"
 
 #include "gui/popups/itempopup.h"
 
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 
 #include "gui/widgets/button.h"
+#include "gui/widgets/equipmentbox.h"
 #include "gui/widgets/playerbox.h"
 
 #include "resources/imageset.h"
+#include "resources/itemslot.h"
 
+#include "utils/delete2.h"
 #include "utils/dtor.h"
 #include "utils/gettext.h"
 
-#include <guichan/font.hpp>
-
-#include <SDL_mouse.h>
-
 #include "debug.h"
 
+EquipmentWindow *equipmentWindow = nullptr;
+EquipmentWindow *beingEquipmentWindow = nullptr;
 static const int BOX_COUNT = 13;
 
 EquipmentWindow::EquipmentWindow(Equipment *const equipment,
                                  Being *const being,
-                                 const bool foring):
+                                 const bool foring) :
     // TRANSLATORS: equipment window name
     Window(_("Equipment"), false, nullptr, "equipment.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mEquipment(equipment),
     mItemPopup(new ItemPopup),
-    mPlayerBox(new PlayerBox("equipment_playerbox.xml",
+    mPlayerBox(new PlayerBox(this,
+        "equipment_playerbox.xml",
         "equipment_selectedplayerbox.xml")),
     // TRANSLATORS: equipment window button
     mUnequip(new Button(this, _("Unequip"), "unequip", this)),
-    mSelected(-1),
-    mForing(foring),
     mImageSet(nullptr),
     mBeing(being),
+    mSlotBackground(),
+    mSlotHighlightedBackground(),
+    mVertexes(new ImageCollection),
     mBoxes(),
     mHighlightColor(getThemeColor(Theme::HIGHLIGHT)),
     mBorderColor(getThemeColor(Theme::BORDER)),
     mLabelsColor(getThemeColor(Theme::LABEL)),
     mLabelsColor2(getThemeColor(Theme::LABEL_OUTLINE)),
-    mSlotBackground(),
-    mSlotHighlightedBackground(),
-    mVertexes(new ImageCollection),
+    mSelected(-1),
     mItemPadding(getOption("itemPadding")),
     mBoxSize(getOption("boxSize")),
     mButtonPadding(getOption("buttonPadding", 5)),
     mMinX(180),
     mMinY(345),
     mMaxX(0),
-    mMaxY(0)
+    mMaxY(0),
+    mForing(foring)
 {
     mItemPopup->postInit();
     if (setupWindow)
@@ -94,7 +98,7 @@ EquipmentWindow::EquipmentWindow(Equipment *const equipment,
         mBoxSize = 36;
 
     // Control that shows the Player
-    mPlayerBox->setDimension(gcn::Rectangle(50, 80, 74, 168));
+    mPlayerBox->setDimension(Rect(50, 80, 74, 168));
     mPlayerBox->setPlayer(being);
 
     if (foring)
@@ -118,13 +122,13 @@ EquipmentWindow::EquipmentWindow(Equipment *const equipment,
 
 void EquipmentWindow::postInit()
 {
-    const gcn::Rectangle &area = getChildrenArea();
+    const Rect &area = getChildrenArea();
     mUnequip->setPosition(area.width  - mUnequip->getWidth() - mButtonPadding,
         area.height - mUnequip->getHeight() - mButtonPadding);
     mUnequip->setEnabled(false);
 
     ImageRect rect;
-    Theme::instance()->loadRect(rect, "equipment_background.xml", "", 0, 1);
+    theme->loadRect(rect, "equipment_background.xml", "", 0, 1);
     mSlotBackground = rect.grid[0];
     mSlotHighlightedBackground = rect.grid[1];
     add(mPlayerBox);
@@ -134,14 +138,12 @@ void EquipmentWindow::postInit()
 
 EquipmentWindow::~EquipmentWindow()
 {
-    delete mItemPopup;
-    mItemPopup = nullptr;
+    delete2(mItemPopup);
     if (this == beingEquipmentWindow)
     {
         if (mEquipment)
             delete mEquipment->getBackend();
-        delete mEquipment;
-        mEquipment = nullptr;
+        delete2(mEquipment)
     }
     delete_all(mBoxes);
     mBoxes.clear();
@@ -154,19 +156,17 @@ EquipmentWindow::~EquipmentWindow()
         mSlotBackground->decRef();
     if (mSlotHighlightedBackground)
         mSlotHighlightedBackground->decRef();
-    delete mVertexes;
-    mVertexes = nullptr;
+    delete2(mVertexes);
 }
 
-void EquipmentWindow::draw(gcn::Graphics *graphics)
+void EquipmentWindow::draw(Graphics *graphics)
 {
     BLOCK_START("EquipmentWindow::draw")
     // Draw window graphics
     Window::draw(graphics);
-    Graphics *const g = static_cast<Graphics*>(graphics);
 
     int i = 0;
-    gcn::Font *const font = getFont();
+    Font *const font = getFont();
     const int fontHeight = font->getHeight();
 
     if (isBatchDrawRenders(openGLMode))
@@ -178,20 +178,27 @@ void EquipmentWindow::draw(gcn::Graphics *graphics)
             {
                 const EquipmentBox *const box = *it;
                 if (!box)
+                {
+                    i ++;
                     continue;
+                }
                 if (i == mSelected)
                 {
-                    g->calcTileCollection(mVertexes,
-                        mSlotHighlightedBackground, box->x, box->y);
+                    graphics->calcTileCollection(mVertexes,
+                        mSlotHighlightedBackground,
+                        box->x, box->y);
                 }
                 else
                 {
-                    g->calcTileCollection(mVertexes, mSlotBackground,
+                    graphics->calcTileCollection(mVertexes,
+                        mSlotBackground,
                         box->x, box->y);
                 }
+                i ++;
             }
+            graphics->finalize(mVertexes);
         }
-        g->drawTileCollection(mVertexes);
+        graphics->drawTileCollection(mVertexes);
     }
     else
     {
@@ -202,9 +209,14 @@ void EquipmentWindow::draw(gcn::Graphics *graphics)
             if (!box)
                 continue;
             if (i == mSelected)
-                g->drawImage2(mSlotHighlightedBackground, box->x, box->y);
+            {
+                graphics->drawImage(mSlotHighlightedBackground,
+                    box->x, box->y);
+            }
             else
-                g->drawImage2(mSlotBackground, box->x, box->y);
+            {
+                graphics->drawImage(mSlotBackground, box->x, box->y);
+            }
         }
     }
 
@@ -230,13 +242,14 @@ void EquipmentWindow::draw(gcn::Graphics *graphics)
             {
                 image->setAlpha(1.0F);  // Ensure the image is drawn
                                         // with maximum opacity
-                g->drawImage2(image, box->x + mItemPadding,
+                graphics->drawImage(image, box->x + mItemPadding,
                     box->y + mItemPadding);
-                if (i == EQUIP_PROJECTILE_SLOT)
+                if (i == ItemSlot::PROJECTILE_SLOT)
                 {
-                    g->setColorAll(mLabelsColor, mLabelsColor2);
+                    graphics->setColorAll(mLabelsColor, mLabelsColor2);
                     const std::string str = toString(item->getQuantity());
-                    font->drawString(g, str,
+                    font->drawString(graphics,
+                        str,
                         box->x + (mBoxSize - font->getWidth(str)) / 2,
                         box->y - fontHeight);
                 }
@@ -244,14 +257,15 @@ void EquipmentWindow::draw(gcn::Graphics *graphics)
         }
         else if (box->image)
         {
-            g->drawImage2(box->image, box->x + mItemPadding,
+            graphics->drawImage(box->image,
+                box->x + mItemPadding,
                 box->y + mItemPadding);
         }
     }
     BLOCK_END("EquipmentWindow::draw")
 }
 
-void EquipmentWindow::action(const gcn::ActionEvent &event)
+void EquipmentWindow::action(const ActionEvent &event)
 {
     if (!mEquipment)
         return;
@@ -277,7 +291,7 @@ Item *EquipmentWindow::getItem(const int x, const int y) const
         const EquipmentBox *const box = *it;
         if (!box)
             continue;
-        const gcn::Rectangle tRect(box->x, box->y, mBoxSize, mBoxSize);
+        const Rect tRect(box->x, box->y, mBoxSize, mBoxSize);
 
         if (tRect.isPointInRect(x, y))
             return mEquipment->getEquipment(i);
@@ -285,22 +299,22 @@ Item *EquipmentWindow::getItem(const int x, const int y) const
     return nullptr;
 }
 
-void EquipmentWindow::mousePressed(gcn::MouseEvent& mouseEvent)
+void EquipmentWindow::mousePressed(MouseEvent& event)
 {
     if (!mEquipment)
     {
-        Window::mousePressed(mouseEvent);
+        Window::mousePressed(event);
         return;
     }
 
-    const int x = mouseEvent.getX();
-    const int y = mouseEvent.getY();
+    const int x = event.getX();
+    const int y = event.getY();
 
-    if (mouseEvent.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         if (mForing)
         {
-            Window::mousePressed(mouseEvent);
+            Window::mousePressed(event);
             return;
         }
         // Checks if any of the presses were in the equip boxes.
@@ -315,13 +329,14 @@ void EquipmentWindow::mousePressed(gcn::MouseEvent& mouseEvent)
             if (!box)
                 continue;
             const Item *const item = mEquipment->getEquipment(i);
-            const gcn::Rectangle tRect(box->x, box->y, mBoxSize, mBoxSize);
+            const Rect tRect(box->x, box->y, mBoxSize, mBoxSize);
 
             if (tRect.isPointInRect(x, y))
             {
                 inBox = true;
                 if (item)
                 {
+                    event.consume();
                     setSelected(i);
                     dragDrop.dragItem(item, DRAGDROP_SOURCE_EQUIPMENT);
                     return;
@@ -331,7 +346,7 @@ void EquipmentWindow::mousePressed(gcn::MouseEvent& mouseEvent)
                 return;
         }
     }
-    else if (mouseEvent.getButton() == gcn::MouseEvent::RIGHT)
+    else if (event.getButton() == MouseButton::RIGHT)
     {
         if (Item *const item = getItem(x, y))
         {
@@ -343,29 +358,30 @@ void EquipmentWindow::mousePressed(gcn::MouseEvent& mouseEvent)
              */
             const int mx = x + getX();
             const int my = y + getY();
-            if (viewport)
+            if (popupManager)
             {
+                event.consume();
                 if (mForing)
-                    viewport->showUndressPopup(mx, my, mBeing, item);
+                    popupManager->showUndressPopup(mx, my, mBeing, item);
                 else
-                    viewport->showPopup(this, mx, my, item, true);
+                    popupManager->showPopup(this, mx, my, item, true);
                 return;
             }
         }
     }
-    Window::mousePressed(mouseEvent);
+    Window::mousePressed(event);
 }
 
-void EquipmentWindow::mouseReleased(gcn::MouseEvent &mouseEvent)
+void EquipmentWindow::mouseReleased(MouseEvent &event)
 {
-    Window::mouseReleased(mouseEvent);
+    Window::mouseReleased(event);
     const DragDropSource src = dragDrop.getSource();
     if (dragDrop.isEmpty() || (src != DRAGDROP_SOURCE_INVENTORY
         && src != DRAGDROP_SOURCE_EQUIPMENT))
     {
         return;
     }
-    Inventory *const inventory = player_node
+    Inventory *const inventory = localPlayer
         ? PlayerInfo::getInventory() : nullptr;
     if (!inventory)
         return;
@@ -387,8 +403,8 @@ void EquipmentWindow::mouseReleased(gcn::MouseEvent &mouseEvent)
     {
         if (item->isEquipment())
         {
-            const int x = mouseEvent.getX();
-            const int y = mouseEvent.getY();
+            const int x = event.getX();
+            const int y = event.getY();
             for (std::vector<EquipmentBox*>::const_iterator
                  it = mBoxes.begin(), it_end = mBoxes.end();
                  it != it_end; ++ it)
@@ -396,7 +412,7 @@ void EquipmentWindow::mouseReleased(gcn::MouseEvent &mouseEvent)
                 const EquipmentBox *const box = *it;
                 if (!box)
                     continue;
-                const gcn::Rectangle tRect(box->x, box->y, mBoxSize, mBoxSize);
+                const Rect tRect(box->x, box->y, mBoxSize, mBoxSize);
 
                 if (tRect.isPointInRect(x, y))
                     return;
@@ -411,7 +427,7 @@ void EquipmentWindow::mouseReleased(gcn::MouseEvent &mouseEvent)
 }
 
 // Show ItemTooltip
-void EquipmentWindow::mouseMoved(gcn::MouseEvent &event)
+void EquipmentWindow::mouseMoved(MouseEvent &event)
 {
     Window::mouseMoved(event);
 
@@ -425,9 +441,6 @@ void EquipmentWindow::mouseMoved(gcn::MouseEvent &event)
 
     if (item)
     {
-        int mouseX, mouseY;
-        SDL_GetMouseState(&mouseX, &mouseY);
-
         mItemPopup->setItem(item);
         mItemPopup->position(x + getX(), y + getY());
     }
@@ -438,7 +451,7 @@ void EquipmentWindow::mouseMoved(gcn::MouseEvent &event)
 }
 
 // Hide ItemTooltip
-void EquipmentWindow::mouseExited(gcn::MouseEvent &event A_UNUSED)
+void EquipmentWindow::mouseExited(MouseEvent &event A_UNUSED)
 {
     if (mItemPopup)
         mItemPopup->setVisible(false);
@@ -511,7 +524,7 @@ void EquipmentWindow::fillBoxes()
 
 void EquipmentWindow::loadPlayerBox(const XmlNodePtr playerBoxNode)
 {
-    mPlayerBox->setDimension(gcn::Rectangle(
+    mPlayerBox->setDimension(Rect(
         XML::getProperty(playerBoxNode, "x", 50),
         XML::getProperty(playerBoxNode, "y", 80),
         XML::getProperty(playerBoxNode, "width", 74),
@@ -557,7 +570,7 @@ void EquipmentWindow::loadSlot(const XmlNodePtr slotNode,
         mMaxY = y + mBoxSize;
 }
 
-int EquipmentWindow::parseSlotName(const std::string &name) const
+int EquipmentWindow::parseSlotName(const std::string &name)
 {
     int id = -1;
     if (name == "shoes" || name == "boot" || name == "boots")
