@@ -30,7 +30,7 @@
 
 #include "utils/xml.h"
 
-#include <guichan/actionlistener.hpp>
+#include "listeners/actionlistener.h"
 
 #include <vector>
 
@@ -42,25 +42,15 @@ class Item;
 class ItemPopup;
 class PlayerBox;
 
-struct EquipmentBox final
-{
-    EquipmentBox(const int x0, const int y0, Image *const img) :
-        x(x0), y(y0), image(img)
-    { }
-
-    A_DELETE_COPY(EquipmentBox)
-
-    int x;
-    int y;
-    Image *image;
-};
+struct EquipmentBox;
 
 /**
  * Equipment dialog.
  *
  * \ingroup Interface
  */
-class EquipmentWindow final : public Window, public gcn::ActionListener
+class EquipmentWindow final : public Window,
+                              public ActionListener
 {
     public:
         /**
@@ -81,11 +71,11 @@ class EquipmentWindow final : public Window, public gcn::ActionListener
         /**
          * Draws the equipment window.
          */
-        void draw(gcn::Graphics *graphics) override final;
+        void draw(Graphics *graphics) override final;
 
-        void action(const gcn::ActionEvent &event) override final;
+        void action(const ActionEvent &event) override final;
 
-        void mousePressed(gcn::MouseEvent& mouseEvent) override final;
+        void mousePressed(MouseEvent& event) override final;
 
         const Item* getEquipment(const int i) const A_WARN_UNUSED
         { return mEquipment ? mEquipment->getEquipment(i) : nullptr; }
@@ -96,11 +86,11 @@ class EquipmentWindow final : public Window, public gcn::ActionListener
 
         void resetBeing(const Being *const being);
 
-        void mouseExited(gcn::MouseEvent &event) override final;
+        void mouseExited(MouseEvent &event) override final;
 
-        void mouseMoved(gcn::MouseEvent &event) override final;
+        void mouseMoved(MouseEvent &event) override final;
 
-        void mouseReleased(gcn::MouseEvent &event) override final;
+        void mouseReleased(MouseEvent &event) override final;
 
         void recalcSize();
 
@@ -122,7 +112,7 @@ class EquipmentWindow final : public Window, public gcn::ActionListener
         void loadSlot(const XmlNodePtr slotNode,
                       const ImageSet *const imageset);
 
-        int parseSlotName(const std::string &name) const A_WARN_UNUSED;
+        static int parseSlotName(const std::string &name) A_WARN_UNUSED;
 
         Equipment *mEquipment;
 
@@ -130,18 +120,17 @@ class EquipmentWindow final : public Window, public gcn::ActionListener
         PlayerBox *mPlayerBox;
         Button *mUnequip;
 
-        int mSelected; /**< Index of selected item. */
-        bool mForing;
         ImageSet *mImageSet;
         Being *mBeing;
-        std::vector<EquipmentBox*> mBoxes;
-        gcn::Color mHighlightColor;
-        gcn::Color mBorderColor;
-        gcn::Color mLabelsColor;
-        gcn::Color mLabelsColor2;
         Image *mSlotBackground;
         Image *mSlotHighlightedBackground;
         ImageCollection *mVertexes;
+        std::vector<EquipmentBox*> mBoxes;
+        Color mHighlightColor;
+        Color mBorderColor;
+        Color mLabelsColor;
+        Color mLabelsColor2;
+        int mSelected; /**< Index of selected item. */
         int mItemPadding;
         int mBoxSize;
         int mButtonPadding;
@@ -149,6 +138,7 @@ class EquipmentWindow final : public Window, public gcn::ActionListener
         int mMinY;
         int mMaxX;
         int mMaxY;
+        bool mForing;
 };
 
 extern EquipmentWindow *equipmentWindow;
