@@ -24,19 +24,19 @@
 
 #include "configuration.h"
 
-#include "gui/sdlfont.h"
+#include "gui/gui.h"
 
 #include "gui/windows/didyouknowwindow.h"
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/browserbox.h"
 #include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/scrollarea.h"
 
-#include "resources/resourcemanager.h"
-
 #include "utils/gettext.h"
+#include "utils/files.h"
 #include "utils/paths.h"
 #include "utils/process.h"
 
@@ -45,15 +45,18 @@
 
 #include "debug.h"
 
+HelpWindow *helpWindow = nullptr;
+
 HelpWindow::HelpWindow() :
     // TRANSLATORS: help window name
     Window(_("Help"), false, nullptr, "help.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     // TRANSLATORS: help window. button.
     mDYKButton(new Button(this, _("Did you know..."), "DYK", this)),
     mBrowserBox(new BrowserBox(this, BrowserBox::AUTO_SIZE, true,
         "browserbox.xml")),
-    mScrollArea(new ScrollArea(mBrowserBox, true, "help_background.xml")),
+    mScrollArea(new ScrollArea(this, mBrowserBox,
+        true, "help_background.xml")),
     mTagFileMap()
 {
     setMinWidth(300);
@@ -72,7 +75,8 @@ HelpWindow::HelpWindow() :
     mBrowserBox->setOpaque(false);
 
     mBrowserBox->setLinkHandler(this);
-    mBrowserBox->setFont(gui->getHelpFont());
+    if (gui)
+        mBrowserBox->setFont(gui->getHelpFont());
     mBrowserBox->setProcessVersion(true);
     mBrowserBox->setEnableImages(true);
     mBrowserBox->setEnableKeys(true);
@@ -82,15 +86,15 @@ HelpWindow::HelpWindow() :
     place(0, 0, mScrollArea, 5, 3).setPadding(3);
 
     Layout &layout = getLayout();
-    layout.setRowHeight(0, Layout::AUTO_SET);
+    layout.setRowHeight(0, LayoutType::SET);
 
     loadWindowState();
     loadTags();
     enableVisibleSound(true);
-    widgetResized(gcn::Event(nullptr));
+    widgetResized(Event(nullptr));
 }
 
-void HelpWindow::action(const gcn::ActionEvent &event)
+void HelpWindow::action(const ActionEvent &event)
 {
     if (event.getId() == "DYK")
     {
@@ -104,7 +108,7 @@ void HelpWindow::action(const gcn::ActionEvent &event)
 }
 
 void HelpWindow::handleLink(const std::string &link,
-                            gcn::MouseEvent *event A_UNUSED)
+                            MouseEvent *event A_UNUSED)
 {
     if (!strStartWith(link, "http://") && !strStartWith(link, "https://"))
     {
@@ -150,7 +154,7 @@ void HelpWindow::loadTags()
     if (helpPath.empty())
         helpPath = paths.getStringValue("help");
     StringVect lines;
-    ResourceManager::loadTextFile(helpPath.append("tags.idx"), lines);
+    Files::loadTextFile(helpPath.append("tags.idx"), lines);
     FOR_EACH (StringVectCIter, it, lines)
     {
         const std::string &str = *it;
