@@ -26,27 +26,34 @@
 #include "item.h"
 #include "units.h"
 
+#include "being/attributes.h"
 #include "being/playerinfo.h"
 
-#include "input/inputmanager.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
 
-#include "gui/viewport.h"
+#include "input/inputmanager.h"
+
+#include "gui/gui.h"
+#include "gui/popupmanager.h"
+
+#include "gui/fonts/font.h"
+
+#include "gui/models/sortlistmodelinv.h"
 
 #include "gui/popups/textpopup.h"
 
 #include "gui/windows/equipmentwindow.h"
 #include "gui/windows/itemamountwindow.h"
 #include "gui/windows/outfitwindow.h"
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 #include "gui/windows/shopwindow.h"
 #include "gui/windows/tradewindow.h"
 
-
 #include "gui/widgets/button.h"
 #include "gui/widgets/dropdown.h"
 #include "gui/widgets/itemcontainer.h"
 #include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/progressbar.h"
 #include "gui/widgets/scrollarea.h"
 #include "gui/widgets/tabstrip.h"
@@ -55,56 +62,24 @@
 #include "net/inventoryhandler.h"
 #include "net/net.h"
 
-#include "utils/gettext.h"
+#include "resources/iteminfo.h"
 
-#include <guichan/font.hpp>
+#include "utils/delete2.h"
 
 #include <string>
 
 #include "debug.h"
 
-static const char *const SORT_NAME_INVENTORY[6] =
-{
-    // TRANSLATORS: inventory sort mode
-    N_("default"),
-    // TRANSLATORS: inventory sort mode
-    N_("by name"),
-    // TRANSLATORS: inventory sort mode
-    N_("by id"),
-    // TRANSLATORS: inventory sort mode
-    N_("by weight"),
-    // TRANSLATORS: inventory sort mode
-    N_("by amount"),
-    // TRANSLATORS: inventory sort mode
-    N_("by type")
-};
-
-class SortListModelInv final : public gcn::ListModel
-{
-public:
-    ~SortListModelInv()
-    { }
-
-    int getNumberOfElements() override final
-    { return 6; }
-
-    std::string getElementAt(int i) override final
-    {
-        if (i >= getNumberOfElements() || i < 0)
-            return "???";
-
-        return gettext(SORT_NAME_INVENTORY[i]);
-    }
-};
-
+InventoryWindow *inventoryWindow = nullptr;
 InventoryWindow::WindowList InventoryWindow::invInstances;
 
-InventoryWindow::InventoryWindow(Inventory *const inventory):
+InventoryWindow::InventoryWindow(Inventory *const inventory) :
     Window("Inventory", false, nullptr, "inventory.xml"),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
-    gcn::SelectionListener(),
+    ActionListener(),
+    KeyListener(),
+    SelectionListener(),
     InventoryListener(),
+    AttributeListener(),
     mInventory(inventory),
     mItems(new ItemContainer(this, mInventory)),
     mUseButton(nullptr),
@@ -132,8 +107,8 @@ InventoryWindow::InventoryWindow(Inventory *const inventory):
     mCompactMode(false)
 {
     mTextPopup->postInit();
-    mSlotsBar->setColor(Theme::getThemeColor(Theme::SLOTS_BAR),
-        Theme::getThemeColor(Theme::SLOTS_BAR_OUTLINE));
+    mSlotsBar->setColor(getThemeColor(Theme::SLOTS_BAR),
+        getThemeColor(Theme::SLOTS_BAR_OUTLINE));
 
     if (inventory)
     {
@@ -163,8 +138,6 @@ InventoryWindow::InventoryWindow(Inventory *const inventory):
         mSortDropDown->setSelected(0);
     }
 
-    listen(CHANNEL_ATTRIBUTES);
-
     if (setupWindow)
         setupWindow->registerWindowForReset(this);
 
@@ -183,9 +156,9 @@ InventoryWindow::InventoryWindow(Inventory *const inventory):
 
     mItems->addSelectionListener(this);
 
-    gcn::ScrollArea *const invenScroll = new ScrollArea(
-        mItems, getOptionBool("showbackground"), "inventory_background.xml");
-    invenScroll->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    ScrollArea *const invenScroll = new ScrollArea(this, mItems,
+        getOptionBool("showbackground"), "inventory_background.xml");
+    invenScroll->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
 
     const int size = config.getIntValue("fontSize");
     mFilter = new TabStrip(this, "filter_" + getWindowName(), size + 16);
@@ -228,8 +201,8 @@ InventoryWindow::InventoryWindow(Inventory *const inventory):
         mEquipmentButton = new Button(this, _("Equipment"), "equipment", this);
         mWeightBar = new ProgressBar(this, 0.0F, 100, 0, Theme::PROG_WEIGHT,
             "weightprogressbar.xml", "weightprogressbar_fill.xml");
-        mWeightBar->setColor(Theme::getThemeColor(Theme::WEIGHT_BAR),
-            Theme::getThemeColor(Theme::WEIGHT_BAR_OUTLINE));
+        mWeightBar->setColor(getThemeColor(Theme::WEIGHT_BAR),
+            getThemeColor(Theme::WEIGHT_BAR_OUTLINE));
 
         place(0, 0, mWeightBar, 4);
         mSlotsBarCell = &place(4, 0, mSlotsBar, 5);
@@ -270,7 +243,7 @@ InventoryWindow::InventoryWindow(Inventory *const inventory):
     }
 
     Layout &layout = getLayout();
-    layout.setRowHeight(2, Layout::AUTO_SET);
+    layout.setRowHeight(2, LayoutType::SET);
 
     mInventory->addInventoyListener(this);
 
@@ -295,7 +268,7 @@ void InventoryWindow::postInit()
     slotsChanged(mInventory);
 
     mItems->setSortType(mSortDropDown->getSelected());
-    widgetResized(gcn::Event(nullptr));
+    widgetResized(Event(nullptr));
     if (!isMainInventory())
         setVisible(true);
 }
@@ -303,17 +276,17 @@ void InventoryWindow::postInit()
 InventoryWindow::~InventoryWindow()
 {
     invInstances.remove(this);
-    mInventory->removeInventoyListener(this);
+    if (mInventory)
+        mInventory->removeInventoyListener(this);
     if (!invInstances.empty())
         invInstances.front()->updateDropButton();
 
     mSortDropDown->hideDrop(false);
-    delete mSortModel;
-    mSortModel = nullptr;
+    delete2(mSortModel);
     mTextPopup = nullptr;
 }
 
-void InventoryWindow::storeSortOrder()
+void InventoryWindow::storeSortOrder() const
 {
     if (mInventory)
     {
@@ -335,7 +308,7 @@ void InventoryWindow::storeSortOrder()
     }
 }
 
-void InventoryWindow::action(const gcn::ActionEvent &event)
+void InventoryWindow::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "outfit")
@@ -425,7 +398,8 @@ void InventoryWindow::action(const gcn::ActionEvent &event)
             if (PlayerInfo::isItemProtected(item->getId()))
                 return;
 
-            if (inputManager.isActionActive(static_cast<int>(Input::KEY_MOD)))
+            if (inputManager.isActionActive(static_cast<int>(
+                InputAction::STOP_ATTACK)))
             {
                 PlayerInfo::dropItem(item, item->getQuantity(), true);
             }
@@ -458,13 +432,13 @@ void InventoryWindow::unselectItem()
     mItems->selectNone();
 }
 
-void InventoryWindow::widgetHidden(const gcn::Event &event)
+void InventoryWindow::widgetHidden(const Event &event)
 {
     Window::widgetHidden(event);
     mItems->hidePopup();
 }
 
-void InventoryWindow::mouseClicked(gcn::MouseEvent &event)
+void InventoryWindow::mouseClicked(MouseEvent &event)
 {
     Window::mouseClicked(event);
 
@@ -474,12 +448,13 @@ void InventoryWindow::mouseClicked(gcn::MouseEvent &event)
         gui->resetClickCount();
 
     const bool mod = (isStorageActive() && inputManager.isActionActive(
-        static_cast<int>(Input::KEY_MOD)));
+        static_cast<int>(InputAction::STOP_ATTACK)));
 
     const bool mod2 = (tradeWindow && tradeWindow->isWindowVisible()
-        && inputManager.isActionActive(static_cast<int>(Input::KEY_MOD)));
+        && inputManager.isActionActive(static_cast<int>(
+        InputAction::STOP_ATTACK)));
 
-    if (!mod && !mod2 && event.getButton() == gcn::MouseEvent::RIGHT)
+    if (!mod && !mod2 && event.getButton() == MouseButton::RIGHT)
     {
         Item *const item = mItems->getSelectedItem();
 
@@ -492,15 +467,15 @@ void InventoryWindow::mouseClicked(gcn::MouseEvent &event)
         const int mx = event.getX() + getX();
         const int my = event.getY() + getY();
 
-        if (viewport)
-            viewport->showPopup(this, mx, my, item, isMainInventory());
+        if (popupManager)
+            popupManager->showPopup(this, mx, my, item, isMainInventory());
     }
 
     if (!mInventory)
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT
-        || event.getButton() == gcn::MouseEvent::RIGHT)
+    if (event.getButton() == MouseButton::LEFT
+        || event.getButton() == MouseButton::RIGHT)
     {
         Item *const item = mItems->getSelectedItem();
 
@@ -511,7 +486,7 @@ void InventoryWindow::mouseClicked(gcn::MouseEvent &event)
         {
             if (mInventory->isMainInventory())
             {
-                if (event.getButton() == gcn::MouseEvent::RIGHT)
+                if (event.getButton() == MouseButton::RIGHT)
                 {
                     ItemAmountWindow::showWindow(ItemAmountWindow::StoreAdd,
                         inventoryWindow, item);
@@ -525,7 +500,7 @@ void InventoryWindow::mouseClicked(gcn::MouseEvent &event)
             }
             else
             {
-                if (event.getButton() == gcn::MouseEvent::RIGHT)
+                if (event.getButton() == MouseButton::RIGHT)
                 {
                     ItemAmountWindow::showWindow(ItemAmountWindow::StoreRemove,
                         inventoryWindow, item);
@@ -542,7 +517,7 @@ void InventoryWindow::mouseClicked(gcn::MouseEvent &event)
         {
             if (PlayerInfo::isItemProtected(item->getId()))
                 return;
-            if (event.getButton() == gcn::MouseEvent::RIGHT)
+            if (event.getButton() == MouseButton::RIGHT)
             {
                 ItemAmountWindow::showWindow(ItemAmountWindow::TradeAdd,
                     tradeWindow, item);
@@ -586,18 +561,18 @@ void InventoryWindow::mouseClicked(gcn::MouseEvent &event)
     }
 }
 
-void InventoryWindow::mouseMoved(gcn::MouseEvent &event)
+void InventoryWindow::mouseMoved(MouseEvent &event)
 {
     Window::mouseMoved(event);
-    const gcn::Widget *const src = event.getSource();
+    const Widget *const src = event.getSource();
     if (src == mSlotsBar || src == mWeightBar)
     {
         const int x = event.getX();
         const int y = event.getY();
-        const gcn::Rectangle &rect = mDimension;
+        const Rect &rect = mDimension;
         mTextPopup->show(rect.x + x, rect.y + y, strprintf(_("Money: %s"),
             Units::formatCurrency(PlayerInfo::getAttribute(
-            PlayerInfo::MONEY)).c_str()));
+            Attributes::MONEY)).c_str()));
     }
     else
     {
@@ -605,30 +580,24 @@ void InventoryWindow::mouseMoved(gcn::MouseEvent &event)
     }
 }
 
-void InventoryWindow::mouseExited(gcn::MouseEvent &event A_UNUSED)
+void InventoryWindow::mouseExited(MouseEvent &event A_UNUSED)
 {
     mTextPopup->hide();
 }
 
-void InventoryWindow::keyPressed(gcn::KeyEvent &event)
+void InventoryWindow::keyPressed(KeyEvent &event)
 {
-    if (static_cast<KeyEvent*>(&event)->getActionId()
-        == static_cast<int>(Input::KEY_GUI_MOD))
-    {
+    if (event.getActionId() == static_cast<int>(InputAction::GUI_MOD))
         mSplit = true;
-    }
 }
 
-void InventoryWindow::keyReleased(gcn::KeyEvent &event)
+void InventoryWindow::keyReleased(KeyEvent &event)
 {
-    if (static_cast<KeyEvent*>(&event)->getActionId()
-        == static_cast<int>(Input::KEY_GUI_MOD))
-    {
+    if (event.getActionId() == static_cast<int>(InputAction::GUI_MOD))
         mSplit = false;
-    }
 }
 
-void InventoryWindow::valueChanged(const gcn::SelectionEvent &event A_UNUSED)
+void InventoryWindow::valueChanged(const SelectionEvent &event A_UNUSED)
 {
     if (!mInventory || !mInventory->isMainInventory())
         return;
@@ -659,36 +628,29 @@ void InventoryWindow::updateButtons(const Item *item)
     if (!item || item->getQuantity() == 0)
     {
         if (mUseButton)
-            mUseButton->setEnabled(true);
+            mUseButton->setEnabled(false);
         if (mDropButton)
-            mDropButton->setEnabled(true);
+            mDropButton->setEnabled(false);
         return;
     }
 
-    if (mUseButton)
-        mUseButton->setEnabled(true);
     if (mDropButton)
         mDropButton->setEnabled(true);
 
     if (mUseButton)
     {
-        if (item->isEquipment())
+        const ItemInfo &info = item->getInfo();
+        const std::string &str = (item->isEquipment() && item->isEquipped())
+            ? info.getUseButton2() : info.getUseButton();
+        if (str.empty())
         {
-            if (item->isEquipped())
-            {
-                // TRANSLATORS: inventory button
-                mUseButton->setCaption(_("Unequip"));
-            }
-            else
-            {
-                // TRANSLATORS: inventory button
-                mUseButton->setCaption(_("Equip"));
-            }
+            mUseButton->setEnabled(false);
+            mUseButton->setCaption(_("Use"));
         }
         else
         {
-            // TRANSLATORS: inventory button
-            mUseButton->setCaption(_("Use"));
+            mUseButton->setEnabled(true);
+            mUseButton->setCaption(str);
         }
     }
 
@@ -705,7 +667,8 @@ void InventoryWindow::updateButtons(const Item *item)
 
 void InventoryWindow::setSplitAllowed(const bool allowed)
 {
-    mSplitButton->setVisible(allowed);
+    if (mSplitButton)
+        mSplitButton->setVisible(allowed);
 }
 
 void InventoryWindow::close()
@@ -717,29 +680,21 @@ void InventoryWindow::close()
     else
     {
         if (Net::getInventoryHandler())
+        {
             Net::getInventoryHandler()->closeStorage(Inventory::STORAGE);
+            Net::getInventoryHandler()->forgotStorage();
+        }
         scheduleDelete();
     }
 }
 
-void InventoryWindow::processEvent(const Channels channel A_UNUSED,
-                                   const DepricatedEvent &event)
-{
-    if (event.getName() == EVENT_UPDATEATTRIBUTE)
-    {
-        const int id = event.getInt("id");
-        if (id == PlayerInfo::TOTAL_WEIGHT || id == PlayerInfo::MAX_WEIGHT)
-            updateWeight();
-    }
-}
-
 void InventoryWindow::updateWeight()
 {
-    if (!isMainInventory())
+    if (!isMainInventory() || !mWeightBar)
         return;
 
-    const int total = PlayerInfo::getAttribute(PlayerInfo::TOTAL_WEIGHT);
-    const int max = PlayerInfo::getAttribute(PlayerInfo::MAX_WEIGHT);
+    const int total = PlayerInfo::getAttribute(Attributes::TOTAL_WEIGHT);
+    const int max = PlayerInfo::getAttribute(Attributes::MAX_WEIGHT);
 
     if (max <= 0)
         return;
@@ -810,7 +765,7 @@ bool InventoryWindow::isAnyInputFocused()
     return false;
 }
 
-void InventoryWindow::widgetResized(const gcn::Event &event)
+void InventoryWindow::widgetResized(const Event &event)
 {
     Window::widgetResized(event);
 
@@ -842,3 +797,22 @@ void InventoryWindow::setVisible(bool visible)
         mSortDropDown->hideDrop();
     Window::setVisible(visible);
 }
+
+void InventoryWindow::unsetInventory()
+{
+    if (mInventory)
+    {
+        mInventory->removeInventoyListener(this);
+        if (mItems)
+            mItems->unsetInventory();
+    }
+    mInventory = nullptr;
+}
+
+void InventoryWindow::attributeChanged(const int id,
+                                       const int oldVal A_UNUSED,
+                                       const int newVal A_UNUSED)
+{
+    if (id == Attributes::TOTAL_WEIGHT || id == Attributes::MAX_WEIGHT)
+        updateWeight();
+}
