@@ -33,74 +33,31 @@
 #include "net/net.h"
 #include "gui/viewport.h"
 
+#include "gui/models/itemsmodel.h"
+
 #include "gui/popups/itempopup.h"
 
 #include "gui/windows/shopwindow.h"
 #include "gui/windows/tradewindow.h"
 
 #include "gui/widgets/button.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/dropdown.h"
 #include "gui/widgets/icon.h"
 #include "gui/widgets/inttextfield.h"
-#include "gui/widgets/layout.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/slider.h"
 
-#include "utils/gettext.h"
+#include "utils/delete2.h"
+
+#include <cmath>
 
 #include "debug.h"
 
-class ItemsModal final : public gcn::ListModel
-{
-public:
-    ItemsModal() :
-        mStrings()
-    {
-        const std::map<int, ItemInfo*> &items = ItemDB::getItemInfos();
-        std::list<std::string> tempStrings;
-
-        for (std::map<int, ItemInfo*>::const_iterator
-             i = items.begin(), i_end = items.end();
-             i != i_end; ++i)
-        {
-            if (i->first < 0)
-                continue;
-
-            const ItemInfo &info = *i->second;
-            const std::string name = info.getName();
-            if (name != "unnamed" && !info.getName().empty()
-                && info.getName() != "unnamed")
-            {
-                tempStrings.push_back(name);
-            }
-        }
-        tempStrings.sort();
-        FOR_EACH (std::list<std::string>::const_iterator, i, tempStrings)
-            mStrings.push_back(*i);
-    }
-
-    A_DELETE_COPY(ItemsModal)
-
-    ~ItemsModal()
-    { }
-
-    int getNumberOfElements() override final
-    {
-        return static_cast<int>(mStrings.size());
-    }
-
-    std::string getElementAt(int i) override final
-    {
-        if (i < 0 || i >= getNumberOfElements())
-            return "???";
-        return mStrings.at(i);
-    }
-private:
-    StringVect mStrings;
-};
-
-void ItemAmountWindow::finish(Item *const item, const int amount,
-                              const int price, const Usage usage)
+void ItemAmountWindow::finish(const Item *const item,
+                              const int amount,
+                              const int price,
+                              const Usage usage)
 {
     switch (usage)
     {
@@ -138,21 +95,21 @@ void ItemAmountWindow::finish(Item *const item, const int amount,
 ItemAmountWindow::ItemAmountWindow(const Usage usage, Window *const parent,
                                    Item *const item, const int maxRange) :
     Window("", false, parent, "amount.xml"),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
+    ActionListener(),
+    KeyListener(),
     mItemAmountTextField(new IntTextField(this, 1)),
     mItemPriceTextField(nullptr),
     mGPLabel(nullptr),
     mItem(item),
     mItemIcon(new Icon(this, item ? item->getImage() : nullptr)),
-    mMax(maxRange),
-    mUsage(usage),
     mItemPopup(new ItemPopup),
-    mItemAmountSlide(new Slider(1.0, mMax)),
+    mItemAmountSlide(new Slider(this, 1.0, maxRange, 1.0)),
     mItemPriceSlide(nullptr),
     mItemDropDown(nullptr),
     mItemsModal(nullptr),
     mPrice(0),
+    mMax(maxRange),
+    mUsage(usage),
     mEnabledKeyboard(keyboard.isEnabled())
 {
     mItemPopup->postInit();
@@ -182,7 +139,7 @@ ItemAmountWindow::ItemAmountWindow(const Usage usage, Window *const parent,
         mItemPriceTextField->setWidth(35);
         mItemPriceTextField->addKeyListener(this);
 
-        mItemPriceSlide = new Slider(1.0, 10000000);
+        mItemPriceSlide = new Slider(this, 1.0, 10000000, 1.0);
         mItemPriceSlide->setHeight(10);
         mItemPriceSlide->setActionEventId("slidePrice");
         mItemPriceSlide->addActionListener(this);
@@ -308,12 +265,11 @@ void ItemAmountWindow::postInit()
 
 ItemAmountWindow::~ItemAmountWindow()
 {
-    delete mItemPopup;
-    mItemPopup = nullptr;
+    delete2(mItemPopup);
 }
 
 // Show ItemTooltip
-void ItemAmountWindow::mouseMoved(gcn::MouseEvent &event)
+void ItemAmountWindow::mouseMoved(MouseEvent &event)
 {
     Window::mouseMoved(event);
 
@@ -328,7 +284,7 @@ void ItemAmountWindow::mouseMoved(gcn::MouseEvent &event)
 }
 
 // Hide ItemTooltip
-void ItemAmountWindow::mouseExited(gcn::MouseEvent &event A_UNUSED)
+void ItemAmountWindow::mouseExited(MouseEvent &event A_UNUSED)
 {
     if (mItemPopup)
         mItemPopup->setVisible(false);
@@ -339,7 +295,7 @@ void ItemAmountWindow::resetAmount()
     mItemAmountTextField->setValue(1);
 }
 
-void ItemAmountWindow::action(const gcn::ActionEvent &event)
+void ItemAmountWindow::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "cancel")
@@ -391,7 +347,7 @@ void ItemAmountWindow::action(const gcn::ActionEvent &event)
     else if (eventId == "slide")
         amount = static_cast<int>(mItemAmountSlide->getValue());
     mItemAmountTextField->setValue(amount);
-    mItemAmountSlide->setValue2(amount);
+    mItemAmountSlide->setValue(amount);
 
     if (mItemPriceTextField && mItemPriceSlide)
     {
@@ -407,14 +363,14 @@ void ItemAmountWindow::action(const gcn::ActionEvent &event)
             mPrice++;
             price = static_cast<int>(pow(10.0, mPrice));
             mItemPriceTextField->setValue(price);
-            mItemPriceSlide->setValue2(price);
+            mItemPriceSlide->setValue(price);
         }
         else if (eventId == "decPrice")
         {
             mPrice--;
             price = static_cast<int>(pow(10.0, mPrice));
             mItemPriceTextField->setValue(price);
-            mItemPriceSlide->setValue2(price);
+            mItemPriceSlide->setValue(price);
         }
         else if (eventId == "slidePrice")
         {
@@ -424,7 +380,7 @@ void ItemAmountWindow::action(const gcn::ActionEvent &event)
             else
                 mPrice = 0;
             mItemPriceTextField->setValue(price);
-            mItemPriceSlide->setValue2(price);
+            mItemPriceSlide->setValue(price);
         }
     }
 }
@@ -435,9 +391,9 @@ void ItemAmountWindow::close()
     scheduleDelete();
 }
 
-void ItemAmountWindow::keyReleased(gcn::KeyEvent &keyEvent A_UNUSED)
+void ItemAmountWindow::keyReleased(KeyEvent &event A_UNUSED)
 {
-    mItemAmountSlide->setValue2(mItemAmountTextField->getValue());
+    mItemAmountSlide->setValue(mItemAmountTextField->getValue());
 }
 
 void ItemAmountWindow::showWindow(const Usage usage, Window *const parent,
