@@ -25,8 +25,14 @@
 #include "client.h"
 #include "configuration.h"
 
-#include "input/keydata.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
+
+#include "input/inputaction.h"
+
+#include "listeners/openurllistener.h"
+
+#include "gui/models/updatelistmodel.h"
+#include "gui/models/updatetypemodel.h"
 
 #include "gui/windows/confirmdialog.h"
 
@@ -34,17 +40,15 @@
 #include "gui/widgets/checkbox.h"
 #include "gui/widgets/dropdown.h"
 #include "gui/widgets/label.h"
-#include "gui/widgets/layout.h"
 #include "gui/widgets/passwordfield.h"
+#include "gui/widgets/layoutcell.h"
 
 #include "net/charserverhandler.h"
-#include "net/logindata.h"
 #include "net/loginhandler.h"
 #include "net/net.h"
 
-#include "utils/gettext.h"
+#include "utils/delete2.h"
 #include "utils/paths.h"
-#include "utils/process.h"
 
 #include "debug.h"
 
@@ -53,98 +57,16 @@ std::string LoginDialog::savedPasswordKey("");
 
 namespace
 {
-    struct OpenUrlListener : public gcn::ActionListener
-    {
-        OpenUrlListener() :
-            gcn::ActionListener(),
-            url()
-        {
-        }
-
-        A_DELETE_COPY(OpenUrlListener)
-
-        void action(const gcn::ActionEvent &event) override final
-        {
-            if (event.getId() == "yes")
-                openBrowser(url);
-        }
-
-        std::string url;
-    } urlListener;
+    OpenUrlListener urlListener;
 }  // namespace
 
-const char *UPDATE_TYPE_TEXT[3] =
-{
-    // TRANSLATORS: update type
-    N_("Normal"),
-    // TRANSLATORS: update type
-    N_("Auto Close"),
-    // TRANSLATORS: update type
-    N_("Skip"),
-};
-
-class UpdateTypeModel final : public gcn::ListModel
-{
-    public:
-        UpdateTypeModel()
-        { }
-
-        A_DELETE_COPY(UpdateTypeModel)
-
-        ~UpdateTypeModel()
-        { }
-
-        int getNumberOfElements() override final
-        {
-            return 3;
-        }
-
-        std::string getElementAt(int i) override final
-        {
-            if (i >= getNumberOfElements() || i < 0)
-                return "???";
-            return gettext(UPDATE_TYPE_TEXT[i]);
-        }
-};
-
-class UpdateListModel final : public gcn::ListModel
-{
-    public:
-        explicit UpdateListModel(LoginData *const data) :
-            gcn::ListModel(),
-            mLoginData(data)
-        {
-        }
-
-        A_DELETE_COPY(UpdateListModel)
-
-        ~UpdateListModel()
-        { }
-
-        int getNumberOfElements() override final
-        {
-            if (!mLoginData)
-                return 0;
-            return static_cast<int>(mLoginData->updateHosts.size());
-        }
-
-        std::string getElementAt(int i) override final
-        {
-            if (!mLoginData || i >= getNumberOfElements() || i < 0)
-                return "???";
-            return mLoginData->updateHosts[i];
-        }
-
-    protected:
-        LoginData *mLoginData;
-};
-
-LoginDialog::LoginDialog(LoginData *const data, std::string serverName,
-                         std::string *const updateHost):
+LoginDialog::LoginDialog(LoginData *const data,
+                         std::string serverName,
+                         std::string *const updateHost) :
     // TRANSLATORS: login dialog name
     Window(_("Login"), false, nullptr, "login.xml"),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
+    ActionListener(),
+    KeyListener(),
     mLoginData(data),
     mUserField(new TextField(this, mLoginData->username)),
     mPassField(new PasswordField(this, mLoginData->password)),
@@ -164,7 +86,7 @@ LoginDialog::LoginDialog(LoginData *const data, std::string serverName,
     mRegisterButton(new Button(this, _("Register"), "register", this)),
     // TRANSLATORS: login dialog checkbox
     mCustomUpdateHost(new CheckBox(this, _("Custom update host"),
-        mLoginData->updateType & LoginData::Upd_Custom, this, "customhost")),
+        mLoginData->updateType & UpdateType::Custom, this, "customhost")),
     mUpdateHostText(new TextField(this, serverConfig.getValue(
         "customUpdateHost", ""))),
     mUpdateListModel(nullptr),
@@ -173,6 +95,7 @@ LoginDialog::LoginDialog(LoginData *const data, std::string serverName,
     mServerName(serverName)
 {
     setCloseButton(true);
+    setWindowName("Login");
 
     Net::getCharServerHandler()->clear();
 
@@ -209,7 +132,7 @@ LoginDialog::LoginDialog(LoginData *const data, std::string serverName,
 
     mUpdateTypeDropDown->setActionEventId("updatetype");
     mUpdateTypeDropDown->setSelected((mLoginData->updateType
-        | LoginData::Upd_Custom) ^ LoginData::Upd_Custom);
+        | UpdateType::Custom) ^ UpdateType::Custom);
 
     if (!mCustomUpdateHost->isSelected())
         mUpdateHostText->setVisible(false);
@@ -245,19 +168,25 @@ LoginDialog::LoginDialog(LoginData *const data, std::string serverName,
     place(3, n + 3, mLoginButton);
 
     addKeyListener(this);
-    if (mUpdateHostLabel)
-        setContentSize(310, 250);
-    else
-        setContentSize(310, 200);
-
-    reflowLayout();
-    center();
 }
 
 void LoginDialog::postInit()
 {
     setVisible(true);
 
+    const int h = 200;
+    if (mUpdateHostLabel)
+        setContentSize(310, 250);
+    setContentSize(310, h);
+#ifdef ANDROID
+    setDefaultSize(310, h, ImageRect::UPPER_CENTER, 0, 0);
+#else
+    setDefaultSize(310, h, ImageRect::CENTER, 0, 0);
+#endif
+    center();
+    loadWindowState();
+    reflowLayout();
+
     if (mUserField->getText().empty())
         mUserField->requestFocus();
     else
@@ -270,13 +199,11 @@ void LoginDialog::postInit()
 
 LoginDialog::~LoginDialog()
 {
-    delete mUpdateTypeModel;
-    mUpdateTypeModel = nullptr;
-    delete mUpdateListModel;
-    mUpdateListModel = nullptr;
+    delete2(mUpdateTypeModel);
+    delete2(mUpdateListModel);
 }
 
-void LoginDialog::action(const gcn::ActionEvent &event)
+void LoginDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "login" && canSubmit())
@@ -318,24 +245,23 @@ void LoginDialog::action(const gcn::ActionEvent &event)
     }
 }
 
-void LoginDialog::keyPressed(gcn::KeyEvent &keyEvent)
+void LoginDialog::keyPressed(KeyEvent &event)
 {
-    if (keyEvent.isConsumed())
+    if (event.isConsumed())
     {
         mLoginButton->setEnabled(canSubmit());
         return;
     }
 
-    const int actionId = static_cast<KeyEvent*>(
-        &keyEvent)->getActionId();
-    if (actionId == static_cast<int>(Input::KEY_GUI_CANCEL))
+    const int actionId = event.getActionId();
+    if (actionId == static_cast<int>(InputAction::GUI_CANCEL))
     {
-        action(gcn::ActionEvent(nullptr, mServerButton->getActionEventId()));
+        action(ActionEvent(nullptr, mServerButton->getActionEventId()));
     }
-    else if (actionId == static_cast<int>(Input::KEY_GUI_SELECT)
-             || actionId == static_cast<int>(Input::KEY_GUI_SELECT2))
+    else if (actionId == static_cast<int>(InputAction::GUI_SELECT)
+             || actionId == static_cast<int>(InputAction::GUI_SELECT2))
     {
-        action(gcn::ActionEvent(nullptr, mLoginButton->getActionEventId()));
+        action(ActionEvent(nullptr, mLoginButton->getActionEventId()));
     }
     else
     {
@@ -360,14 +286,14 @@ void LoginDialog::prepareUpdate()
     if (mCustomUpdateHost->isSelected()
         && !mUpdateHostText->getText().empty())
     {
-        updateType |= LoginData::Upd_Custom;
+        updateType |= UpdateType::Custom;
         serverConfig.setValue("customUpdateHost",
             mUpdateHostText->getText());
 
         if (checkPath(mUpdateHostText->getText()))
         {
             mLoginData->updateHost = mUpdateHostText->getText();
-            *mUpdateHost = mUpdateHostText->getText();
+            *mUpdateHost = mLoginData->updateHost;
         }
         else
         {
