@@ -23,27 +23,34 @@
 #include "gui/windows/minimap.h"
 
 #include "actormanager.h"
-#include "client.h"
 #include "configuration.h"
 #include "party.h"
+#include "settings.h"
 
 #include "being/localplayer.h"
 
+#include "gui/popupmanager.h"
 #include "gui/viewport.h"
 
 #include "gui/popups/textpopup.h"
 
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 
 #include "resources/image.h"
 #include "resources/imagehelper.h"
 #include "resources/resourcemanager.h"
 
+#include "resources/map/map.h"
+#include "resources/map/metatile.h"
+
+#include "utils/delete2.h"
 #include "utils/gettext.h"
+#include "utils/physfstools.h"
 #include "utils/sdlcheckutils.h"
 
 #include "debug.h"
 
+Minimap *minimap = nullptr;
 bool Minimap::mShow = true;
 
 Minimap::Minimap() :
@@ -52,9 +59,9 @@ Minimap::Minimap() :
     mWidthProportion(0.5),
     mHeightProportion(0.5),
     mMapImage(nullptr),
+    mTextPopup(new TextPopup),
     mMapOriginX(0),
     mMapOriginY(0),
-    mTextPopup(new TextPopup),
     mCustomMapImage(false),
     mAutoResize(config.getBoolValue("autoresizeminimaps"))
 {
@@ -88,7 +95,12 @@ Minimap::~Minimap()
     config.setValue(getWindowName() + "Show", mShow);
     config.removeListeners(this);
     CHECKLISTENERS
+    deleteMapImage();
+    delete2(mTextPopup);
+}
 
+void Minimap::deleteMapImage()
+{
     if (mMapImage)
     {
         if (mCustomMapImage)
@@ -97,12 +109,11 @@ Minimap::~Minimap()
             mMapImage->decRef();
         mMapImage = nullptr;
     }
-    delete mTextPopup;
-    mTextPopup = nullptr;
 }
 
 void Minimap::setMap(const Map *const map)
 {
+    BLOCK_START("Minimap::setMap")
     std::string caption;
 
     if (map)
@@ -115,16 +126,7 @@ void Minimap::setMap(const Map *const map)
     }
 
     setCaption(caption);
-
-    // Adapt the image
-    if (mMapImage)
-    {
-        if (mCustomMapImage)
-            delete mMapImage;
-        else
-            mMapImage->decRef();
-        mMapImage = nullptr;
-    }
+    deleteMapImage();
 
     if (map)
     {
@@ -137,6 +139,7 @@ void Minimap::setMap(const Map *const map)
             {
                 if (!isSticky())
                     setVisible(false);
+                BLOCK_END("Minimap::setMap")
                 return;
             }
 
@@ -147,11 +150,12 @@ void Minimap::setMap(const Map *const map)
             {
                 if (!isSticky())
                     setVisible(false);
+                BLOCK_END("Minimap::setMap")
                 return;
             }
             const int size = surface->h * surface->w;
-            const int mask = (Map::BLOCKMASK_WALL | Map::BLOCKMASK_AIR
-                | Map::BLOCKMASK_WATER);
+            const int mask = (BlockMask::WALL | BlockMask::AIR
+                | BlockMask::WATER);
 
             for (int ptr = 0; ptr < size; ptr ++)
                 *(data ++) = -!(map->mMetaTiles[ptr].blockmask & mask);
@@ -159,7 +163,7 @@ void Minimap::setMap(const Map *const map)
             SDL_UnlockSurface(surface);
 
             mMapImage = imageHelper->load(surface);
-            mMapImage->setAlpha(client->getGuiAlpha());
+            mMapImage->setAlpha(settings.guiAlpha);
             mCustomMapImage = true;
             MSDL_FreeSurface(surface);
         }
@@ -171,14 +175,14 @@ void Minimap::setMap(const Map *const map)
 
             std::string minimapName = map->getProperty("minimap");
 
-            if (minimapName.empty() && resman->exists(tempname))
+            if (minimapName.empty() && PhysFs::exists(tempname.c_str()))
                 minimapName = tempname;
 
             if (minimapName.empty())
             {
                 tempname = std::string("graphics/minimaps/").append(
                     map->getFilename()).append(".png");
-                if (resman->exists(tempname))
+                if (PhysFs::exists(tempname.c_str()))
                     minimapName = tempname;
             }
 
@@ -213,7 +217,7 @@ void Minimap::setMap(const Map *const map)
             setHeight(height);
         }
 
-        const gcn::Rectangle &rect = mDimension;
+        const Rect &rect = mDimension;
         setDefaultSize(rect.x, rect.y, rect.width, rect.height);
         resetToDefaultSize();
 
@@ -225,6 +229,7 @@ void Minimap::setMap(const Map *const map)
         if (!isSticky())
             setVisible(false);
     }
+    BLOCK_END("Minimap::setMap")
 }
 
 void Minimap::toggle()
@@ -233,20 +238,18 @@ void Minimap::toggle()
     mShow = isWindowVisible();
 }
 
-void Minimap::draw(gcn::Graphics *graphics)
+void Minimap::draw(Graphics *graphics)
 {
     BLOCK_START("Minimap::draw")
     Window::draw(graphics);
 
-    if (!userPalette || !player_node || !viewport)
+    if (!userPalette || !localPlayer || !viewport)
     {
         BLOCK_END("Minimap::draw")
         return;
     }
 
-    Graphics *const graph = static_cast<Graphics*>(graphics);
-
-    const gcn::Rectangle a = getChildrenArea();
+    const Rect a = getChildrenArea();
 
     graphics->pushClipArea(a);
 
@@ -266,7 +269,7 @@ void Minimap::draw(gcn::Graphics *graphics)
         const int h = rect.h;
         if (w > a.width || h > a.height)
         {
-            const Vector &p = player_node->getPosition();
+            const Vector &p = localPlayer->getPosition();
             mMapOriginX = (a.width / 2) - (p.x + static_cast<float>(
                 viewport->getCameraRelativeX()) * mWidthProportion) / 32;
 
@@ -286,13 +289,13 @@ void Minimap::draw(gcn::Graphics *graphics)
                 mMapOriginY = 0;
         }
 
-        graph->drawImage2(mMapImage, mMapOriginX, mMapOriginY);
+        graphics->drawImage(mMapImage, mMapOriginX, mMapOriginY);
     }
 
     const ActorSprites &actors = actorManager->getAll();
     FOR_EACH (ActorSpritesConstIterator, it, actors)
     {
-        if (!(*it) || (*it)->getType() == ActorSprite::FLOOR_ITEM)
+        if (!(*it) || (*it)->getType() == ActorType::FLOOR_ITEM)
             continue;
 
         const Being *const being = static_cast<const Being *const>(*it);
@@ -302,7 +305,7 @@ void Minimap::draw(gcn::Graphics *graphics)
         int dotSize = 2;
         int type = UserPalette::PC;
 
-        if (being == player_node)
+        if (being == localPlayer)
         {
             type = UserPalette::SELF;
             dotSize = 3;
@@ -311,8 +314,8 @@ void Minimap::draw(gcn::Graphics *graphics)
         {
             type = UserPalette::GM;
         }
-        else if (being->getGuild() == player_node->getGuild()
-                 || being->getGuildName() == player_node->getGuildName())
+        else if (being->getGuild() == localPlayer->getGuild()
+                 || being->getGuildName() == localPlayer->getGuildName())
         {
             type = UserPalette::GUILD;
         }
@@ -320,20 +323,20 @@ void Minimap::draw(gcn::Graphics *graphics)
         {
             switch (being->getType())
             {
-                case ActorSprite::MONSTER:
+                case ActorType::MONSTER:
                     type = UserPalette::MONSTER;
                     break;
 
-                case ActorSprite::NPC:
+                case ActorType::NPC:
                     type = UserPalette::NPC;
                     break;
 
-                case ActorSprite::AVATAR:
-                case ActorSprite::UNKNOWN:
-                case ActorSprite::PLAYER:
-                case ActorSprite::FLOOR_ITEM:
-                case ActorSprite::PORTAL:
-                case ActorSprite::PET:
+                case ActorType::AVATAR:
+                case ActorType::UNKNOWN:
+                case ActorType::PLAYER:
+                case ActorType::FLOOR_ITEM:
+                case ActorType::PORTAL:
+                case ActorType::PET:
                 default:
                     continue;
             }
@@ -348,20 +351,20 @@ void Minimap::draw(gcn::Graphics *graphics)
                 dotSize - 1) * mWidthProportion);
         const Vector &pos = being->getPosition();
 
-        graphics->fillRectangle(gcn::Rectangle(
+        graphics->fillRectangle(Rect(
             static_cast<float>(pos.x * mWidthProportion) / 32
             + mMapOriginX - offsetWidth,
             static_cast<float>(pos.y * mHeightProportion) / 32
             + mMapOriginY - offsetHeight, dotSize, dotSize));
     }
 
-    if (player_node->isInParty())
+    if (localPlayer->isInParty())
     {
-        const Party *const party = player_node->getParty();
+        const Party *const party = localPlayer->getParty();
         if (party)
         {
             const PartyMember *const m = party->getMember(
-                player_node->getName());
+                localPlayer->getName());
             const Party::MemberList *const members = party->getMembers();
             if (m && members)
             {
@@ -386,7 +389,7 @@ void Minimap::draw(gcn::Graphics *graphics)
                         const int offsetWidth = static_cast<int>(
                             mWidthProportion);
 
-                        graphics->fillRectangle(gcn::Rectangle(
+                        graphics->fillRectangle(Rect(
                             static_cast<int>(member->getX()
                             * mWidthProportion) + mMapOriginX - offsetWidth,
                             static_cast<int>(member->getY()
@@ -399,10 +402,10 @@ void Minimap::draw(gcn::Graphics *graphics)
         }
     }
 
-    const Vector &pos = player_node->getPosition();
+    const Vector &pos = localPlayer->getPosition();
 
-    const int gw = graph->getWidth();
-    const int gh = graph->getHeight();
+    const int gw = graphics->getWidth();
+    const int gh = graphics->getHeight();
     int x = static_cast<float>((pos.x - (gw / 2)
         + viewport->getCameraRelativeX())
         * mWidthProportion) / 32 + mMapOriginX;
@@ -431,45 +434,45 @@ void Minimap::draw(gcn::Graphics *graphics)
     }
 
     graphics->setColor(userPalette->getColor(UserPalette::PC));
-    graphics->drawRectangle(gcn::Rectangle(x, y, w, h));
+    graphics->drawRectangle(Rect(x, y, w, h));
     graphics->popClipArea();
     BLOCK_END("Minimap::draw")
 }
 
-void Minimap::mouseReleased(gcn::MouseEvent &event)
+void Minimap::mouseReleased(MouseEvent &event)
 {
     Window::mouseReleased(event);
 
-    if (!player_node || !viewport)
+    if (!localPlayer || !popupManager)
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         int x = event.getX();
         int y = event.getY();
         screenToMap(x, y);
 
-        player_node->navigateTo(x, y);
+        localPlayer->navigateTo(x, y);
     }
-    else if (event.getButton() == gcn::MouseEvent::RIGHT)
+    else if (event.getButton() == MouseButton::RIGHT)
     {
         int x = event.getX();
         int y = event.getY();
         screenToMap(x, y);
-        viewport->showMapPopup(x, y);
+        popupManager->showMapPopup(x, y);
     }
 }
 
-void Minimap::mouseMoved(gcn::MouseEvent &event)
+void Minimap::mouseMoved(MouseEvent &event)
 {
     Window::mouseMoved(event);
     const int x = event.getX();
     const int y = event.getY();
-    const gcn::Rectangle &rect = mDimension;
+    const Rect &rect = mDimension;
     mTextPopup->show(x + rect.x, y + rect.y, mCaption);
 }
 
-void Minimap::mouseExited(gcn::MouseEvent &event)
+void Minimap::mouseExited(MouseEvent &event)
 {
     Window::mouseExited(event);
     mTextPopup->hide();
@@ -477,7 +480,7 @@ void Minimap::mouseExited(gcn::MouseEvent &event)
 
 void Minimap::screenToMap(int &x, int &y)
 {
-    const gcn::Rectangle a = getChildrenArea();
+    const Rect a = getChildrenArea();
     x = (x - a.x - mMapOriginX + mWidthProportion) / mWidthProportion;
     y = (y - a.y - mMapOriginY + mHeightProportion) / mHeightProportion;
 }
