@@ -24,11 +24,14 @@
 
 #include "animatedsprite.h"
 #include "configuration.h"
+#include "inventory.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 
-#include "gui/viewport.h"
+#include "gui/popupmanager.h"
+#include "gui/skin.h"
 
 #include "gui/popups/textpopup.h"
 
@@ -42,25 +45,27 @@
 #include "net/playerhandler.h"
 #include "net/gamehandler.h"
 
+#include "utils/delete2.h"
 #include "utils/dtor.h"
 #include "utils/gettext.h"
 
 #include "debug.h"
 
+MiniStatusWindow *miniStatusWindow = nullptr;
 extern volatile int tick_time;
 
 typedef std::vector <ProgressBar*>::const_iterator ProgressBarVectorCIter;
 
 MiniStatusWindow::MiniStatusWindow() :
-    Popup("MiniStatus", "ministatus.xml"),
+    Window("MiniStatus", false, nullptr, "ministatus.xml"),
     InventoryListener(),
+    AttributeListener(),
+    StatListener(),
+    ArrowsListener(),
+    UpdateStatusListener(),
     mBars(),
     mBarNames(),
     mIcons(),
-    mSpacing(mSkin ? mSkin->getOption("spacing", 3) : 3),
-    mIconPadding(mSkin ? mSkin->getOption("iconPadding", 3) : 3),
-    mIconSpacing(mSkin ? mSkin->getOption("iconSpacing", 2) : 2),
-    mMaxX(0),
     // TRANSLATORS: status bar name
     mHpBar(createBar(0, 100, 0, Theme::HP_BAR, Theme::PROG_HP,
         "hpprogressbar.xml", "hpprogressbar_fill.xml",
@@ -105,17 +110,19 @@ MiniStatusWindow::MiniStatusWindow() :
         // TRANSLATORS: status bar name
         "status bar", _("status bar"))),
     mTextPopup(new TextPopup),
-    mStatusPopup(new StatusPopup)
+    mStatusPopup(new StatusPopup),
+    mSpacing(mSkin ? mSkin->getOption("spacing", 3) : 3),
+    mIconPadding(mSkin ? mSkin->getOption("iconPadding", 3) : 3),
+    mIconSpacing(mSkin ? mSkin->getOption("iconSpacing", 2) : 2),
+    mMaxX(0)
 {
     mTextPopup->postInit();
     mStatusPopup->postInit();
 
-    listen(CHANNEL_ATTRIBUTES);
-
     StatusWindow::updateHPBar(mHpBar);
 
-    if (Net::getGameHandler()->canUseMagicBar())
-        StatusWindow::updateMPBar(mMpBar);
+    if (Net::getGameHandler()->canUseMagicBar() && statusWindow)
+        statusWindow->updateMPBar(mMpBar);
 
     const int job = Net::getPlayerHandler()->getJobLocation()
         && serverConfig.getValueBool("showJob", true);
@@ -134,7 +141,7 @@ MiniStatusWindow::MiniStatusWindow() :
     loadBars();
     updateBars();
 
-    setVisible(config.getValueBool(getPopupName() + "Visible", true));
+    setVisible(true);
     addMouseListener(this);
     Inventory *const inv = PlayerInfo::getInventory();
     if (inv)
@@ -147,10 +154,8 @@ MiniStatusWindow::MiniStatusWindow() :
 
 MiniStatusWindow::~MiniStatusWindow()
 {
-    delete mTextPopup;
-    mTextPopup = nullptr;
-    delete mStatusPopup;
-    mStatusPopup = nullptr;
+    delete2(mTextPopup);
+    delete2(mStatusPopup);
     delete_all(mIcons);
     mIcons.clear();
 
@@ -163,7 +168,7 @@ MiniStatusWindow::~MiniStatusWindow()
         ProgressBar *bar = *it;
         if (!bar)
             continue;
-        if (!bar->isVisible())
+        if (!bar->isVisibleLocal())
             delete bar;
     }
     mBars.clear();
@@ -183,8 +188,7 @@ ProgressBar *MiniStatusWindow::createBar(const float progress,
         progress, width, height, backColor, skin, skinFill);
     bar->setActionEventId(name);
     bar->setId(description);
-    bar->setColor(Theme::getThemeColor(textColor),
-        Theme::getThemeColor(textColor + 1));
+    bar->setColor(getThemeColor(textColor), getThemeColor(textColor + 1));
     mBars.push_back(bar);
     mBarNames[name] = bar;
     return bar;
@@ -195,14 +199,14 @@ void MiniStatusWindow::updateBars()
     int x = 0;
     const ProgressBar *lastBar = nullptr;
     FOR_EACH (ProgressBarVectorCIter, it, mBars)
-        safeRemove(*it);
+        remove(*it);
 
     FOR_EACH (ProgressBarVectorCIter, it, mBars)
     {
         ProgressBar *const bar = *it;
         if (!bar)
             continue;
-        if (bar->isVisible())
+        if (bar->isVisibleLocal())
         {
             bar->setPosition(x, 0);
             add(bar);
@@ -252,44 +256,48 @@ void MiniStatusWindow::drawIcons(Graphics *const graphics)
     }
 }
 
-void MiniStatusWindow::processEvent(const Channels channel A_UNUSED,
-                                    const DepricatedEvent &event)
+void MiniStatusWindow::statChanged(const int id A_UNUSED,
+                                   const int oldVal1 A_UNUSED,
+                                   const int oldVal2 A_UNUSED)
 {
-    if (event.getName() == EVENT_UPDATEATTRIBUTE)
+    if (statusWindow)
+        statusWindow->updateMPBar(mMpBar);
+    StatusWindow::updateJobBar(mJobBar);
+}
+
+void MiniStatusWindow::attributeChanged(const int id,
+                                        const int oldVal A_UNUSED,
+                                        const int newVal A_UNUSED)
+{
+    switch (id)
     {
-        const int id = event.getInt("id");
-        if (id == PlayerInfo::HP || id == PlayerInfo::MAX_HP)
-        {
+        case Attributes::HP:
+        case Attributes::MAX_HP:
             StatusWindow::updateHPBar(mHpBar);
-        }
-        else if (id == PlayerInfo::MP || id == PlayerInfo::MAX_MP)
-        {
-            StatusWindow::updateMPBar(mMpBar);
-        }
-        else if (id == PlayerInfo::EXP || id == PlayerInfo::EXP_NEEDED)
-        {
+            break;
+        case Attributes::MP:
+        case Attributes::MAX_MP:
+            statusWindow->updateMPBar(mMpBar);
+            break;
+        case Attributes::EXP:
+        case Attributes::EXP_NEEDED:
             StatusWindow::updateXPBar(mXpBar);
-        }
-        else if (id == PlayerInfo::TOTAL_WEIGHT
-                 || id == PlayerInfo::MAX_WEIGHT)
-        {
+            break;
+        case Attributes::TOTAL_WEIGHT:
+        case Attributes::MAX_WEIGHT:
             StatusWindow::updateWeightBar(mWeightBar);
-        }
-        else if (id == PlayerInfo::MONEY)
-        {
+            break;
+        case Attributes::MONEY:
             StatusWindow::updateMoneyBar(mMoneyBar);
-        }
-    }
-    else if (event.getName() == EVENT_UPDATESTAT)
-    {
-        StatusWindow::updateMPBar(mMpBar);
-        StatusWindow::updateJobBar(mJobBar);
+            break;
+        default:
+            break;
     }
 }
 
 void MiniStatusWindow::updateStatus()
 {
-    StatusWindow::updateStatusBar(mStatusBar);
+    statusWindow->updateStatusBar(mStatusBar);
     if (mStatusPopup && mStatusPopup->isPopupVisible())
         mStatusPopup->update();
 }
@@ -297,7 +305,7 @@ void MiniStatusWindow::updateStatus()
 void MiniStatusWindow::logic()
 {
     BLOCK_START("MiniStatusWindow::logic")
-    Popup::logic();
+    Window::logic();
 
     for (size_t i = 0, sz = mIcons.size(); i < sz; i++)
     {
@@ -308,21 +316,21 @@ void MiniStatusWindow::logic()
     BLOCK_END("MiniStatusWindow::logic")
 }
 
-void MiniStatusWindow::draw(gcn::Graphics *graphics)
+void MiniStatusWindow::draw(Graphics *graphics)
 {
     BLOCK_START("MiniStatusWindow::draw")
     drawChildren(graphics);
     BLOCK_END("MiniStatusWindow::draw")
 }
 
-void MiniStatusWindow::mouseMoved(gcn::MouseEvent &event)
+void MiniStatusWindow::mouseMoved(MouseEvent &event)
 {
-    Popup::mouseMoved(event);
+    Window::mouseMoved(event);
 
     const int x = event.getX();
     const int y = event.getY();
 
-    const gcn::Rectangle &rect = mDimension;
+    const Rect &rect = mDimension;
     if (event.getSource() == mStatusBar)
     {
         mStatusPopup->view(x + rect.x, y + rect.y);
@@ -331,22 +339,22 @@ void MiniStatusWindow::mouseMoved(gcn::MouseEvent &event)
     else if (event.getSource() == mXpBar)
     {
         std::string level;
-        if (player_node && player_node->isGM())
+        if (localPlayer && localPlayer->isGM())
         {
             // TRANSLATORS: status bar label
             level = strprintf(_("Level: %d (GM %d)"),
-                PlayerInfo::getAttribute(PlayerInfo::LEVEL),
-                player_node->getGMLevel());
+                PlayerInfo::getAttribute(Attributes::LEVEL),
+                localPlayer->getGMLevel());
         }
         else
         {
             // TRANSLATORS: status bar label
             level = strprintf(_("Level: %d"),
-                PlayerInfo::getAttribute(PlayerInfo::LEVEL));
+                PlayerInfo::getAttribute(Attributes::LEVEL));
         }
 
-        const int exp = PlayerInfo::getAttribute(PlayerInfo::EXP);
-        const int expNeed = PlayerInfo::getAttribute(PlayerInfo::EXP_NEEDED);
+        const int exp = PlayerInfo::getAttribute(Attributes::EXP);
+        const int expNeed = PlayerInfo::getAttribute(Attributes::EXP_NEEDED);
         if (exp > expNeed)
         {
             mTextPopup->show(x + rect.x, y + rect.y, level, strprintf("%d/%d",
@@ -364,15 +372,15 @@ void MiniStatusWindow::mouseMoved(gcn::MouseEvent &event)
     else if (event.getSource() == mHpBar)
     {
         mTextPopup->show(x + rect.x, y + rect.y, event.getSource()->getId(),
-            strprintf("%d/%d", PlayerInfo::getAttribute(PlayerInfo::HP),
-            PlayerInfo::getAttribute(PlayerInfo::MAX_HP)));
+            strprintf("%d/%d", PlayerInfo::getAttribute(Attributes::HP),
+            PlayerInfo::getAttribute(Attributes::MAX_HP)));
         mStatusPopup->hide();
     }
     else if (event.getSource() == mMpBar)
     {
         mTextPopup->show(x + rect.x, y + rect.y, event.getSource()->getId(),
-            strprintf("%d/%d", PlayerInfo::getAttribute(PlayerInfo::MP),
-            PlayerInfo::getAttribute(PlayerInfo::MAX_MP)));
+            strprintf("%d/%d", PlayerInfo::getAttribute(Attributes::MP),
+            PlayerInfo::getAttribute(Attributes::MAX_MP)));
         mStatusPopup->hide();
     }
     else if (event.getSource() == mJobBar)
@@ -403,8 +411,8 @@ void MiniStatusWindow::mouseMoved(gcn::MouseEvent &event)
     {
         mTextPopup->show(x + rect.x, y + rect.y, event.getSource()->getId(),
             strprintf("%d/%d", PlayerInfo::getAttribute(
-            PlayerInfo::TOTAL_WEIGHT),
-            PlayerInfo::getAttribute(PlayerInfo::MAX_WEIGHT)));
+            Attributes::TOTAL_WEIGHT),
+            PlayerInfo::getAttribute(Attributes::MAX_WEIGHT)));
         mStatusPopup->hide();
     }
     else if (event.getSource() == mInvSlotsBar)
@@ -424,7 +432,7 @@ void MiniStatusWindow::mouseMoved(gcn::MouseEvent &event)
     {
         mTextPopup->show(x + rect.x, y + rect.y,
             event.getSource()->getId(),
-            toString(PlayerInfo::getAttribute(PlayerInfo::MONEY)));
+            toString(PlayerInfo::getAttribute(Attributes::MONEY)));
     }
     else
     {
@@ -433,28 +441,26 @@ void MiniStatusWindow::mouseMoved(gcn::MouseEvent &event)
     }
 }
 
-void MiniStatusWindow::mousePressed(gcn::MouseEvent &event)
+void MiniStatusWindow::mousePressed(MouseEvent &event)
 {
-    if (!viewport)
-        return;
-
-    if (event.getButton() == gcn::MouseEvent::RIGHT)
+    if (event.getButton() == MouseButton::RIGHT)
     {
         const ProgressBar *const bar = dynamic_cast<ProgressBar*>(
             event.getSource());
         if (!bar)
             return;
-        if (viewport)
+        event.consume();
+        if (popupManager)
         {
-            viewport->showPopup(getX() + event.getX(),
+            popupManager->showPopup(getX() + event.getX(),
                 getY() + event.getY(), bar);
         }
     }
 }
 
-void MiniStatusWindow::mouseExited(gcn::MouseEvent &event)
+void MiniStatusWindow::mouseExited(MouseEvent &event)
 {
-    Popup::mouseExited(event);
+    Window::mouseExited(event);
 
     mTextPopup->hide();
     mStatusPopup->hide();
@@ -508,7 +514,7 @@ void MiniStatusWindow::saveBars() const
     FOR_EACH (ProgressBarVectorCIter, it, mBars)
     {
         const ProgressBar *const bar = *it;
-        if (!bar->isVisible())
+        if (!bar->isVisibleLocal())
         {
             config.setValue("ministatus" + toString(i),
                 bar->getActionEventId());
@@ -530,21 +536,21 @@ void MiniStatusWindow::slotsChanged(Inventory *const inventory)
         StatusWindow::updateInvSlotsBar(mInvSlotsBar);
 }
 
-void MiniStatusWindow::updateArrows()
-{
-    StatusWindow::updateArrowsBar(mArrowsBar);
-}
-
-gcn::Rectangle MiniStatusWindow::getChildrenArea()
+Rect MiniStatusWindow::getChildrenArea()
 {
     const int padding = mPadding;
     const int padding2 = padding * 2;
-    const gcn::Rectangle &rect = mDimension;
-    return gcn::Rectangle(padding, padding,
+    const Rect &rect = mDimension;
+    return Rect(padding, padding,
         rect.width - padding2,
         rect.height - padding2);
 }
 
+void MiniStatusWindow::arrowsChanged()
+{
+    StatusWindow::updateArrowsBar(mArrowsBar);
+}
+
 #ifdef USE_PROFILER
 void MiniStatusWindow::logicChildren()
 {
