@@ -24,18 +24,20 @@
 
 #include "actormanager.h"
 #include "configuration.h"
-#include "client.h"
 #include "inventory.h"
 #include "item.h"
+#include "settings.h"
 #include "soundconsts.h"
 #include "soundmanager.h"
 
 #include "being/being.h"
 
 #include "gui/gui.h"
-#include "gui/sdlfont.h"
+#include "gui/popupmanager.h"
 #include "gui/viewport.h"
 
+#include "gui/fonts/font.h"
+
 #include "gui/windows/inventorywindow.h"
 
 #include "gui/widgets/browserbox.h"
@@ -44,6 +46,7 @@
 #include "gui/widgets/itemcontainer.h"
 #include "gui/widgets/itemlinkhandler.h"
 #include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/extendedlistbox.h"
 #include "gui/widgets/playerbox.h"
 #include "gui/widgets/scrollarea.h"
@@ -55,12 +58,12 @@
 
 #include "net/net.h"
 #include "net/npchandler.h"
+#include "net/packetlimiter.h"
 
 #include "utils/copynpaste.h"
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
-#include <guichan/font.hpp>
-
 #include "debug.h"
 
 // TRANSLATORS: npc dialog button
@@ -80,18 +83,18 @@ typedef std::vector<Image *>::iterator ImageVectorIter;
 NpcDialog::NpcDialog(const int npcId) :
     // TRANSLATORS: npc dialog name
     Window(_("NPC"), false, nullptr, "npc.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mNpcId(npcId),
     mDefaultInt(0),
     mDefaultString(),
     mTextBox(new BrowserBox(this, BrowserBox::AUTO_WRAP, true,
         "browserbox.xml")),
-    mScrollArea(new ScrollArea(mTextBox,
+    mScrollArea(new ScrollArea(this, mTextBox,
         getOptionBool("showtextbackground"), "npc_textbackground.xml")),
     mText(),
     mNewText(),
     mItemList(new ExtendedListBox(this, this, "extendedlistbox.xml")),
-    mListScrollArea(new ScrollArea(mItemList,
+    mListScrollArea(new ScrollArea(this, mItemList,
         getOptionBool("showlistbackground"), "npc_listbackground.xml")),
     mItems(),
     mImages(),
@@ -113,16 +116,16 @@ NpcDialog::NpcDialog(const int npcId) :
     mResetButton(new Button(this, _("Reset"), "reset", this)),
     mInventory(new Inventory(Inventory::NPC, 1)),
     mItemContainer(new ItemContainer(this, mInventory)),
-    mItemScrollArea(new ScrollArea(mItemContainer,
+    mItemScrollArea(new ScrollArea(this, mItemContainer,
         getOptionBool("showitemsbackground"), "npc_listbackground.xml")),
     mInputState(NPC_INPUT_NONE),
     mActionState(NPC_ACTION_WAIT),
+    mPlayerBox(new PlayerBox(nullptr)),
+    mAvatarBeing(nullptr),
     mLastNextTime(0),
     mCameraMode(-1),
     mCameraX(0),
     mCameraY(0),
-    mPlayerBox(new PlayerBox(nullptr)),
-    mAvatarBeing(nullptr),
     mShowAvatar(false),
     mLogInteraction(config.getBoolValue("logNpcInGui"))
 {
@@ -149,8 +152,8 @@ NpcDialog::NpcDialog(const int npcId) :
     mTextBox->setEnableKeys(true);
     mTextBox->setEnableTabs(true);
 
-    mScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
-    mScrollArea->setVerticalScrollPolicy(gcn::ScrollArea::SHOW_ALWAYS);
+    mScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
+    mScrollArea->setVerticalScrollPolicy(ScrollArea::SHOW_ALWAYS);
 
     // Setup listbox
     mItemList->setWrappingEnabled(true);
@@ -164,13 +167,13 @@ NpcDialog::NpcDialog(const int npcId) :
         mItemList->setRowHeight(gui->getNpcFont()->getHeight());
 
     setContentSize(260, 175);
-    mListScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
-    mItemScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    mListScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
+    mItemScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
     mItemList->setVisible(true);
     mTextField->setVisible(true);
     mIntField->setVisible(true);
 
-    const gcn::Font *const fnt = mButton->getFont();
+    const Font *const fnt = mButton->getFont();
     int width = std::max(fnt->getWidth(CAPTION_WAITING),
         fnt->getWidth(CAPTION_NEXT));
     width = std::max(width, fnt->getWidth(CAPTION_CLOSE));
@@ -218,44 +221,23 @@ NpcDialog::~NpcDialog()
         delete mPlayerBox;
     }
 
-    delete mTextBox;
-    mTextBox = nullptr;
-    delete mClearButton;
-    mClearButton = nullptr;
-    delete mButton;
-    mButton = nullptr;
-    delete mButton2;
-    mButton2 = nullptr;
-    delete mButton3;
-    mButton3 = nullptr;
-
-    // These might not actually be in the layout, so lets be safe
-    delete mScrollArea;
-    mScrollArea = nullptr;
-    delete mItemList;
-    mItemList = nullptr;
-    delete mTextField;
-    mTextField = nullptr;
-    delete mIntField;
-    mIntField = nullptr;
-    delete mResetButton;
-    mResetButton = nullptr;
-    delete mPlusButton;
-    mPlusButton = nullptr;
-    delete mMinusButton;
-    mMinusButton = nullptr;
-    delete mItemLinkHandler;
-    mItemLinkHandler = nullptr;
-
-    delete mItemContainer;
-    mItemContainer = nullptr;
-    delete mInventory;
-    mInventory = nullptr;
-    delete mItemScrollArea;
-    mItemScrollArea = nullptr;
-
-    delete mListScrollArea;
-    mListScrollArea = nullptr;
+    delete2(mTextBox);
+    delete2(mClearButton);
+    delete2(mButton);
+    delete2(mButton2);
+    delete2(mButton3);
+    delete2(mScrollArea);
+    delete2(mItemList);
+    delete2(mTextField);
+    delete2(mIntField);
+    delete2(mResetButton);
+    delete2(mPlusButton);
+    delete2(mMinusButton);
+    delete2(mItemLinkHandler);
+    delete2(mItemContainer);
+    delete2(mInventory);
+    delete2(mItemScrollArea);
+    delete2(mListScrollArea);
 
     FOR_EACH (ImageVectorIter, it, mImages)
     {
@@ -295,14 +277,14 @@ void NpcDialog::showCloseButton()
     buildLayout();
 }
 
-void NpcDialog::action(const gcn::ActionEvent &event)
+void NpcDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "ok")
     {
         if (mActionState == NPC_ACTION_NEXT)
         {
-            if (!client->limitPackets(PACKET_NPC_NEXT))
+            if (!PacketLimiter::limitPackets(PACKET_NPC_NEXT))
                 return;
 
             nextDialog();
@@ -327,7 +309,7 @@ void NpcDialog::action(const gcn::ActionEvent &event)
 
                     if (selectedIndex >= static_cast<int>(mItems.size())
                         || selectedIndex < 0
-                        || !client->limitPackets(PACKET_NPC_INPUT))
+                        || !PacketLimiter::limitPackets(PACKET_NPC_INPUT))
                     {
                         return;
                     }
@@ -340,7 +322,7 @@ void NpcDialog::action(const gcn::ActionEvent &event)
                 }
                 case NPC_INPUT_STRING:
                 {
-                    if (!client->limitPackets(PACKET_NPC_INPUT))
+                    if (!PacketLimiter::limitPackets(PACKET_NPC_INPUT))
                         return;
                     printText = mTextField->getText();
                     Net::getNpcHandler()->stringInput(mNpcId, printText);
@@ -348,7 +330,7 @@ void NpcDialog::action(const gcn::ActionEvent &event)
                 }
                 case NPC_INPUT_INTEGER:
                 {
-                    if (!client->limitPackets(PACKET_NPC_INPUT))
+                    if (!PacketLimiter::limitPackets(PACKET_NPC_INPUT))
                         return;
                     printText = strprintf("%d", mIntField->getValue());
                     Net::getNpcHandler()->integerInput(
@@ -357,7 +339,7 @@ void NpcDialog::action(const gcn::ActionEvent &event)
                 }
                 case NPC_INPUT_ITEM:
                 {
-                    if (!client->limitPackets(PACKET_NPC_INPUT))
+                    if (!PacketLimiter::limitPackets(PACKET_NPC_INPUT))
                         return;
 
                     std::string str;
@@ -833,7 +815,7 @@ void NpcDialog::buildLayout()
     }
 
     Layout &layout = getLayout();
-    layout.setRowHeight(1, Layout::AUTO_SET);
+    layout.setRowHeight(1, LayoutType::SET);
     redraw();
     mScrollArea->setVerticalScrollAmount(mScrollArea->getVerticalMaxScroll());
 }
@@ -843,7 +825,7 @@ void NpcDialog::saveCamera()
     if (!viewport || mCameraMode >= 0)
         return;
 
-    mCameraMode = viewport->getCameraMode();
+    mCameraMode = static_cast<int>(settings.cameraMode);
     mCameraX = viewport->getCameraRelativeX();
     mCameraY = viewport->getCameraRelativeY();
 }
@@ -853,15 +835,10 @@ void NpcDialog::restoreCamera()
     if (!viewport || mCameraMode == -1)
         return;
 
-    if (!mCameraMode)
+    if (static_cast<int>(settings.cameraMode) != mCameraMode)
+        viewport->toggleCameraMode();
+    if (mCameraMode)
     {
-        if (viewport->getCameraMode() != mCameraMode)
-            viewport->toggleCameraMode();
-    }
-    else
-    {
-        if (viewport->getCameraMode() != mCameraMode)
-            viewport->toggleCameraMode();
         viewport->setCameraRelativeX(mCameraX);
         viewport->setCameraRelativeY(mCameraY);
     }
@@ -874,7 +851,7 @@ void NpcDialog::showAvatar(const uint16_t avatarId)
     if (needShow)
     {
         delete mAvatarBeing;
-        mAvatarBeing = new Being(0, ActorSprite::AVATAR, avatarId, nullptr);
+        mAvatarBeing = new Being(0, ActorType::AVATAR, avatarId, nullptr);
         mPlayerBox->setPlayer(mAvatarBeing);
         if (!mAvatarBeing->empty())
         {
@@ -898,8 +875,7 @@ void NpcDialog::showAvatar(const uint16_t avatarId)
     }
     else
     {
-        delete mAvatarBeing;
-        mAvatarBeing = nullptr;
+        delete2(mAvatarBeing)
         mPlayerBox->setPlayer(nullptr);
     }
     if (needShow != mShowAvatar)
@@ -924,7 +900,7 @@ void NpcDialog::setAvatarAction(const int actionId)
 {
     Being *const being = mPlayerBox->getBeing();
     if (being)
-        being->setAction(static_cast<Being::Action>(actionId), 0);
+        being->setAction(static_cast<BeingAction::Action>(actionId), 0);
 }
 
 void NpcDialog::logic()
@@ -965,14 +941,15 @@ void NpcDialog::clearDialogs()
     mNpcDialogs.clear();
 }
 
-void NpcDialog::mousePressed(gcn::MouseEvent &event)
+void NpcDialog::mousePressed(MouseEvent &event)
 {
     Window::mousePressed(event);
-    if (event.getButton() == gcn::MouseEvent::RIGHT
+    if (event.getButton() == MouseButton::RIGHT
         && event.getSource() == mTextBox)
     {
-        if (viewport)
-            viewport->showNpcDialogPopup(mNpcId);
+        event.consume();
+        if (popupManager)
+            popupManager->showNpcDialogPopup(mNpcId);
     }
 }
 
