@@ -22,6 +22,8 @@
 
 #include "gui/windows/npcpostdialog.h"
 
+#include "gui/windows/setupwindow.h"
+
 #include "gui/widgets/button.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/textbox.h"
@@ -39,19 +41,22 @@
 
 NpcPostDialog::DialogList NpcPostDialog::instances;
 
-NpcPostDialog::NpcPostDialog(const int npcId):
+NpcPostDialog::NpcPostDialog(const int npcId) :
     // TRANSLATORS: npc post dialog caption
     Window(_("NPC"), false, nullptr, "npcpost.xml"),
-    gcn::ActionListener(),
-    mNpcId(npcId),
+    ActionListener(),
     mText(new TextBox(this)),
-    mSender(new TextField(this))
+    mSender(new TextField(this)),
+    mNpcId(npcId)
 {
     setContentSize(400, 180);
 }
 
 void NpcPostDialog::postInit()
 {
+    if (setupWindow)
+        setupWindow->registerWindowForReset(this);
+
     // create text field for receiver
     // TRANSLATORS: label in npc post dialog
     Label *const senderText = new Label(this, _("To:"));
@@ -74,9 +79,9 @@ void NpcPostDialog::postInit()
     mText->setEditable(true);
 
     // create scroll box for letter text
-    ScrollArea *const scrollArea = new ScrollArea(mText);
-    scrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
-    scrollArea->setDimension(gcn::Rectangle(
+    ScrollArea *const scrollArea = new ScrollArea(this, mText);
+    scrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
+    scrollArea->setDimension(Rect(
                 5, mSender->getHeight() + 5,
                 380, 140 - (mSender->getHeight() + sendButton->getHeight())));
 
@@ -98,7 +103,7 @@ NpcPostDialog::~NpcPostDialog()
     instances.remove(this);
 }
 
-void NpcPostDialog::action(const gcn::ActionEvent &event)
+void NpcPostDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "send")
