@@ -28,19 +28,22 @@
 #include "gui/widgets/button.h"
 #include "gui/widgets/textbox.h"
 
+#include "gui/fonts/font.h"
+
 #include "utils/gettext.h"
 
-#include <guichan/font.hpp>
-
 #include "debug.h"
 
+OkDialog *weightNotice = nullptr;
+OkDialog *deathNotice = nullptr;
+
 OkDialog::OkDialog(const std::string &restrict title,
                    const std::string &restrict msg,
                    const int soundEvent, const bool modal,
                    const bool showCenter, Window *const parent,
                    const int minWidth) :
     Window(title, modal, parent, "ok.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mTextBox(new TextBox(this))
 {
     mTextBox->setEditable(false);
@@ -78,13 +81,13 @@ OkDialog::OkDialog(const std::string &restrict title,
     setVisible(true);
     okButton->requestFocus();
 
-    if (soundEvent == DIALOG_OK)
+    if (soundEvent == DialogType::OK)
         soundManager.playGuiSound(SOUND_INFO);
-    else if (soundEvent == DIALOG_ERROR)
+    else if (soundEvent == DialogType::ERROR)
         soundManager.playGuiSound(SOUND_ERROR);
 }
 
-void OkDialog::action(const gcn::ActionEvent &event)
+void OkDialog::action(const ActionEvent &event)
 {
     setActionEventId(event.getId());
     distributeActionEvent();
