@@ -24,7 +24,7 @@
 
 #include "configuration.h"
 #include "dragdrop.h"
-#include "emoteshortcut.h"
+#include "emoteshortcutconsts.h"
 #include "game.h"
 #include "inventory.h"
 #include "item.h"
@@ -33,12 +33,15 @@
 
 #include "input/inputmanager.h"
 
-#include "gui/viewport.h"
+#include "gui/popupmanager.h"
+
+#include "gui/windows/setupwindow.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/checkbox.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 
 #include "resources/image.h"
 
@@ -48,10 +51,12 @@
 
 #include "debug.h"
 
-OutfitWindow::OutfitWindow():
+OutfitWindow *outfitWindow = nullptr;
+
+OutfitWindow::OutfitWindow() :
     // TRANSLATORS: outfits window name
     Window(_("Outfits"), false, nullptr, "outfits.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     // TRANSLATORS: outfits window button
     mPreviousButton(new Button(this, _("<"), "previous", this)),
     // TRANSLATORS: outfits window button
@@ -66,18 +71,17 @@ OutfitWindow::OutfitWindow():
     // TRANSLATORS: outfits window checkbox
     mAwayOutfitCheck(new CheckBox(this, _("Away outfit"),
         serverConfig.getValue("OutfitAwayIndex", OUTFITS_COUNT - 1))),
-    mCurrentOutfit(0),
     // TRANSLATORS: outfits window label
     mKeyLabel(new Label(this, strprintf(_("Key: %s"),
-        keyName(mCurrentOutfit).c_str()))),
+        keyName(0).c_str()))),
+    mBorderColor(getThemeColor(Theme::BORDER, 64)),
+    mCurrentOutfit(0),
     mBoxWidth(33),
     mBoxHeight(33),
     mGridWidth(4),
     mGridHeight(4),
     mItems(),
     mAwayOutfit(0),
-    mBorderColor(getThemeColor(Theme::BORDER, 64)),
-    mBackgroundColor(getThemeColor(Theme::BACKGROUND, 32)),
     mItemColors(),
     mItemClicked(false),
     mItemsUnequip()
@@ -87,12 +91,17 @@ OutfitWindow::OutfitWindow():
     setCloseButton(true);
     setStickyButtonLock(true);
 
+    mBackgroundColor = getThemeColor(Theme::BACKGROUND, 32);
+
     setDefaultSize(250, 400, 150, 290);
     setMinWidth(145);
     setMinHeight(220);
 
-    mCurrentLabel->setAlignment(gcn::Graphics::CENTER);
-    mKeyLabel->setAlignment(gcn::Graphics::CENTER);
+    if (setupWindow)
+        setupWindow->registerWindowForReset(this);
+
+    mCurrentLabel->setAlignment(Graphics::CENTER);
+    mKeyLabel->setAlignment(Graphics::CENTER);
 
     mUnequipCheck->setActionEventId("unequip");
     mUnequipCheck->addActionListener(this);
@@ -109,7 +118,7 @@ OutfitWindow::OutfitWindow():
     place(0, 7, mAwayOutfitCheck, 4);
 
     Layout &layout = getLayout();
-    layout.setRowHeight(0, Layout::AUTO_SET);
+    layout.setRowHeight(0, LayoutType::SET);
     layout.setColWidth(4, Layout::CENTER);
 
     loadWindowState();
@@ -225,9 +234,9 @@ void OutfitWindow::save() const
     serverConfig.setValue("OutfitAwayIndex", mAwayOutfit);
 }
 
-void OutfitWindow::action(const gcn::ActionEvent &event)
+void OutfitWindow::action(const ActionEvent &event)
 {
-    const std::string eventId = event.getId();
+    const std::string &eventId = event.getId();
     if (eventId == "next")
     {
         next();
@@ -310,11 +319,10 @@ void OutfitWindow::copyOutfit(const int src, const int dst)
     save();
 }
 
-void OutfitWindow::draw(gcn::Graphics *graphics)
+void OutfitWindow::draw(Graphics *graphics)
 {
     BLOCK_START("OutfitWindow::draw")
     Window::draw(graphics);
-    Graphics *const g = static_cast<Graphics*>(graphics);
 
     if (mCurrentOutfit < 0 || mCurrentOutfit
         >= static_cast<signed int>(OUTFITS_COUNT))
@@ -326,12 +334,12 @@ void OutfitWindow::draw(gcn::Graphics *graphics)
     {
         const int itemX = mPadding + ((i % mGridWidth) * mBoxWidth);
         const int itemY = mPadding + mTitleBarHeight
-            + ((i / mGridWidth) * mBoxHeight);
+            + ((i / static_cast<unsigned int>(mGridWidth)) * mBoxHeight);
 
         graphics->setColor(mBorderColor);
-        graphics->drawRectangle(gcn::Rectangle(itemX, itemY, 32, 32));
+        graphics->drawRectangle(Rect(itemX, itemY, 32, 32));
         graphics->setColor(mBackgroundColor);
-        graphics->fillRectangle(gcn::Rectangle(itemX, itemY, 32, 32));
+        graphics->fillRectangle(Rect(itemX, itemY, 32, 32));
 
         if (mItems[mCurrentOutfit][i] < 0)
             continue;
@@ -348,7 +356,7 @@ void OutfitWindow::draw(gcn::Graphics *graphics)
                 const Image *const image = item->getImage();
                 if (image)
                 {
-                    g->drawImage2(image, itemX, itemY);
+                    graphics->drawImage(image, itemX, itemY);
                     foundItem = true;
                 }
             }
@@ -359,7 +367,7 @@ void OutfitWindow::draw(gcn::Graphics *graphics)
                 mItemColors[mCurrentOutfit][i]);
             if (image)
             {
-                g->drawImage2(image, itemX, itemY);
+                graphics->drawImage(image, itemX, itemY);
                 image->decRef();
             }
         }
@@ -367,9 +375,9 @@ void OutfitWindow::draw(gcn::Graphics *graphics)
     BLOCK_END("OutfitWindow::draw")
 }
 
-void OutfitWindow::mouseDragged(gcn::MouseEvent &event)
+void OutfitWindow::mouseDragged(MouseEvent &event)
 {
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         if (dragDrop.isEmpty() && mItemClicked)
         {
@@ -410,14 +418,14 @@ void OutfitWindow::mouseDragged(gcn::MouseEvent &event)
     Window::mouseDragged(event);
 }
 
-void OutfitWindow::mousePressed(gcn::MouseEvent &event)
+void OutfitWindow::mousePressed(MouseEvent &event)
 {
     const int index = getIndexFromGrid(event.getX(), event.getY());
     if (index == -1)
     {
-        if (event.getButton() == gcn::MouseEvent::RIGHT && viewport)
+        if (event.getButton() == MouseButton::RIGHT && popupManager)
         {
-            viewport->showOutfitsPopup();
+            popupManager->showOutfitsPopup();
             event.consume();
         }
         else
@@ -447,9 +455,9 @@ void OutfitWindow::mousePressed(gcn::MouseEvent &event)
     Window::mousePressed(event);
 }
 
-void OutfitWindow::mouseReleased(gcn::MouseEvent &event)
+void OutfitWindow::mouseReleased(MouseEvent &event)
 {
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         if (mCurrentOutfit < 0 || mCurrentOutfit
             >= static_cast<signed int>(OUTFITS_COUNT))
@@ -484,7 +492,7 @@ void OutfitWindow::mouseReleased(gcn::MouseEvent &event)
 
 int OutfitWindow::getIndexFromGrid(const int pointX, const int pointY) const
 {
-    const gcn::Rectangle tRect = gcn::Rectangle(mPadding, mTitleBarHeight,
+    const Rect tRect = Rect(mPadding, mTitleBarHeight,
         mGridWidth * mBoxWidth, mGridHeight * mBoxHeight);
     if (!tRect.isPointInRect(pointX, pointY))
         return -1;
@@ -524,12 +532,12 @@ void OutfitWindow::unequipNotInOutfit(const int outfit) const
     }
 }
 
-std::string OutfitWindow::keyName(const int number) const
+std::string OutfitWindow::keyName(const int number)
 {
     if (number < 0 || number >= SHORTCUT_EMOTES)
         return "";
     return inputManager.getKeyStringLong(static_cast<int>(
-        Input::KEY_EMOTE_1) + number);
+        InputAction::EMOTE_1) + number);
 }
 
 void OutfitWindow::next()
