@@ -26,122 +26,63 @@
 
 #include "being/localplayer.h"
 
-#include "gui/sdlfont.h"
+#include "gui/gui.h"
+
+#include "gui/fonts/font.h"
+
+#include "gui/models/questsmodel.h"
+
+#include "gui/windows/setupwindow.h"
 
 #include "gui/widgets/browserbox.h"
 #include "gui/widgets/button.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/extendedlistbox.h"
-#include "gui/widgets/extendednamesmodel.h"
 #include "gui/widgets/itemlinkhandler.h"
 #include "gui/widgets/scrollarea.h"
 
+#include "utils/delete2.h"
 #include "utils/dtor.h"
 #include "utils/gettext.h"
 
 #include "utils/translation/podict.h"
 
 #include "resources/beingcommon.h"
+#include "resources/questeffect.h"
+#include "resources/questitem.h"
+#include "resources/questtype.h"
+
+#include "resources/map/map.h"
 
 #include "debug.h"
 
-enum QuestType
-{
-    QUEST_TEXT = 0,
-    QUEST_NAME = 1,
-    QUEST_REWARD = 2
-};
-
-struct QuestItemText final
-{
-    QuestItemText(const std::string &text0, const int type0) :
-        text(text0), type(type0)
-    {
-    }
-
-    std::string text;
-    int type;
-};
-
-struct QuestItem final
-{
-    QuestItem() :
-        var(0),
-        name(),
-        group(),
-        incomplete(),
-        complete(),
-        texts(),
-        completeFlag(-1),
-        broken(false)
-    {
-    }
-
-    int var;
-    std::string name;
-    std::string group;
-    std::set<int> incomplete;
-    std::set<int> complete;
-    std::vector<QuestItemText> texts;
-    int completeFlag;
-    bool broken;
-};
-
-class QuestsModel final : public ExtendedNamesModel
-{
-    public:
-        QuestsModel() :
-            ExtendedNamesModel()
-        {
-        }
-
-        A_DELETE_COPY(QuestsModel)
-
-        ~QuestsModel()
-        { }
-};
-
-struct QuestEffect final
-{
-    QuestEffect() :
-        map(),
-        var(0),
-        id(0),
-        effectId(0),
-        values()
-    {
-    }
-
-    std::string map;
-    int var;
-    int id;
-    int effectId;
-    std::set<int> values;
-};
+QuestsWindow *questsWindow = nullptr;
 
 QuestsWindow::QuestsWindow() :
     // TRANSLATORS: quests window name
     Window(_("Quests"), false, nullptr, "quests.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mQuestsModel(new QuestsModel),
     mQuestsListBox(new ExtendedListBox(this,
         mQuestsModel, "extendedlistbox.xml")),
-    mQuestScrollArea(new ScrollArea(mQuestsListBox,
+    mQuestScrollArea(new ScrollArea(this, mQuestsListBox,
         getOptionBool("showlistbackground"), "quests_list_background.xml")),
     mItemLinkHandler(new ItemLinkHandler),
     mText(new BrowserBox(this, BrowserBox::AUTO_WRAP, true, "browserbox.xml")),
-    mTextScrollArea(new ScrollArea(mText,
+    mTextScrollArea(new ScrollArea(this, mText,
         getOptionBool("showtextbackground"), "quests_text_background.xml")),
     // TRANSLATORS: quests window button
     mCloseButton(new Button(this, _("Close"), "close", this)),
+    mCompleteIcon(Theme::getImageFromThemeXml("complete_icon.xml", "")),
+    mIncompleteIcon(Theme::getImageFromThemeXml("incomplete_icon.xml", "")),
     mVars(),
     mQuests(),
     mAllEffects(),
     mMapEffects(),
     mNpcEffects(),
     mQuestLinks(),
-    mCompleteIcon(Theme::getImageFromThemeXml("complete_icon.xml", "")),
-    mIncompleteIcon(Theme::getImageFromThemeXml("incomplete_icon.xml", "")),
     mNewQuestEffectId(paths.getIntValue("newQuestEffectId")),
     mCompleteQuestEffectId(paths.getIntValue("completeQuestEffectId")),
     mMap(nullptr)
@@ -158,15 +99,18 @@ QuestsWindow::QuestsWindow() :
     setMinWidth(310);
     setMinHeight(220);
 
+    if (setupWindow)
+        setupWindow->registerWindowForReset(this);
+
     mQuestsListBox->setActionEventId("select");
     mQuestsListBox->addActionListener(this);
 
-    mQuestScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    mQuestScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
     mText->setOpaque(false);
     mText->setLinkHandler(mItemLinkHandler);
-    mTextScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    mTextScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
     mQuestsListBox->setWidth(500);
-    if (gui->getNpcFont()->getHeight() < 20)
+    if (!gui || gui->getNpcFont()->getHeight() < 20)
         mQuestsListBox->setRowHeight(20);
     else
         mQuestsListBox->setRowHeight(gui->getNpcFont()->getHeight());
@@ -179,7 +123,7 @@ QuestsWindow::QuestsWindow() :
     placer(7, 3, mCloseButton);
 
     Layout &layout = getLayout();
-    layout.setRowHeight(0, Layout::AUTO_SET);
+    layout.setRowHeight(0, LayoutType::SET);
 
     loadWindowState();
     enableVisibleSound(true);
@@ -190,8 +134,7 @@ QuestsWindow::QuestsWindow() :
 
 QuestsWindow::~QuestsWindow()
 {
-    delete mQuestsModel;
-    mQuestsModel = nullptr;
+    delete2(mQuestsModel);
 
     for (std::map<int, std::vector<QuestItem*> >::iterator it
          = mQuests.begin(), it_end = mQuests.end(); it != it_end; ++ it)
@@ -206,8 +149,7 @@ QuestsWindow::~QuestsWindow()
     delete_all(mAllEffects);
     mAllEffects.clear();
 
-    delete mItemLinkHandler;
-    mItemLinkHandler = nullptr;
+    delete2(mItemLinkHandler);
     mQuests.clear();
     mQuestLinks.clear();
     if (mCompleteIcon)
@@ -290,11 +232,11 @@ void QuestsWindow::loadQuest(const int var, const XmlNodePtr node)
         std::string str = translator->getStr(data);
 
         if (xmlNameEqual(dataNode, "text"))
-            quest->texts.push_back(QuestItemText(str, QUEST_TEXT));
+            quest->texts.push_back(QuestItemText(str, QuestType::TEXT));
         else if (xmlNameEqual(dataNode, "name"))
-            quest->texts.push_back(QuestItemText(str, QUEST_NAME));
+            quest->texts.push_back(QuestItemText(str, QuestType::NAME));
         else if (xmlNameEqual(dataNode, "reward"))
-            quest->texts.push_back(QuestItemText(str, QUEST_REWARD));
+            quest->texts.push_back(QuestItemText(str, QuestType::REWARD));
     }
     mQuests[var].push_back(quest);
 }
@@ -318,7 +260,7 @@ void QuestsWindow::loadEffect(const int var, const XmlNodePtr node)
     mAllEffects.push_back(effect);
 }
 
-void QuestsWindow::action(const gcn::ActionEvent &event)
+void QuestsWindow::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "select")
@@ -440,11 +382,11 @@ void QuestsWindow::rebuild(const bool playSound)
             switch (newCompleteStatus)
             {
                 case 0:
-                    effectManager->trigger(mNewQuestEffectId, player_node);
+                    effectManager->trigger(mNewQuestEffectId, localPlayer);
                     break;
                 case 1:
                     effectManager->trigger(mCompleteQuestEffectId,
-                        player_node);
+                        localPlayer);
                     break;
                 default:
                     break;
@@ -466,12 +408,12 @@ void QuestsWindow::showQuest(const QuestItem *const quest)
         const QuestItemText &data = *it;
         switch (data.type)
         {
-            case QUEST_TEXT:
-            case QUEST_REWARD:
+            case QuestType::TEXT:
+            case QuestType::REWARD:
             default:
                 mText->addRow(translator->getStr(data.text));
                 break;
-            case QUEST_NAME:
+            case QuestType::NAME:
                 mText->addRow(std::string("[").append(translator->getStr(
                     data.text)).append("]"));
                 break;
