@@ -24,16 +24,17 @@
 
 #include "client.h"
 #include "configuration.h"
-#include "game.h"
 #include "soundconsts.h"
 #include "soundmanager.h"
 
-#include "input/keydata.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
 
+#include "input/inputaction.h"
+
+#include "gui/dialogsmanager.h"
 #include "gui/viewport.h"
 
-#include "gui/widgets/layout.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/button.h"
 #include "gui/widgets/radiobutton.h"
 
@@ -41,16 +42,19 @@
 #include "net/gamehandler.h"
 #include "net/net.h"
 
+#include "resources/map/map.h"
+
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 #include "utils/process.h"
 
 #include "debug.h"
 
-QuitDialog::QuitDialog(QuitDialog **const pointerToMe):
+QuitDialog::QuitDialog(QuitDialog **const pointerToMe) :
     // TRANSLATORS: quit dialog name
     Window(_("Quit"), true, nullptr, "quit.xml"),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
+    ActionListener(),
+    KeyListener(),
     mOptions(),
     // TRANSLATORS: quit dialog button
     mLogoutQuit(new RadioButton(this, _("Quit"), "quitdialog")),
@@ -130,14 +134,10 @@ QuitDialog::~QuitDialog()
 {
     if (mMyPointer)
         *mMyPointer = nullptr;
-    delete mForceQuit;
-    mForceQuit = nullptr;
-    delete mLogoutQuit;
-    mLogoutQuit = nullptr;
-    delete mSwitchAccountServer;
-    mSwitchAccountServer = nullptr;
-    delete mSwitchCharacter;
-    mSwitchCharacter = nullptr;
+    delete2(mForceQuit);
+    delete2(mLogoutQuit);
+    delete2(mSwitchAccountServer);
+    delete2(mSwitchCharacter);
 }
 
 void QuitDialog::placeOption(ContainerPlacer &placer,
@@ -147,7 +147,7 @@ void QuitDialog::placeOption(ContainerPlacer &placer,
     mOptions.push_back(option);
 }
 
-void QuitDialog::action(const gcn::ActionEvent &event)
+void QuitDialog::action(const ActionEvent &event)
 {
     soundManager.playGuiSound(SOUND_HIDE_WINDOW);
     if (event.getId() == "ok")
@@ -165,7 +165,7 @@ void QuitDialog::action(const gcn::ActionEvent &event)
         }
         else if (mLogoutQuit->isSelected())
         {
-            Game::closeDialogs();
+            DialogsManager::closeDialogs();
             client->setState(STATE_EXIT);
         }
         else if (mRate && mRate->isSelected())
@@ -179,14 +179,14 @@ void QuitDialog::action(const gcn::ActionEvent &event)
             }
             else
             {
-                Game::closeDialogs();
+                DialogsManager::closeDialogs();
                 client->setState(STATE_EXIT);
             }
         }
         else if (Net::getGameHandler()->isConnected()
                  && mSwitchAccountServer->isSelected())
         {
-            Game::closeDialogs();
+            DialogsManager::closeDialogs();
             client->setState(STATE_SWITCH_SERVER);
         }
         else if (mSwitchCharacter->isSelected())
@@ -194,7 +194,7 @@ void QuitDialog::action(const gcn::ActionEvent &event)
             if (client->getState() == STATE_GAME)
             {
                 Net::getCharServerHandler()->switchCharacter();
-                Game::closeDialogs();
+                DialogsManager::closeDialogs();
                 serverConfig.write();
             }
         }
@@ -202,25 +202,24 @@ void QuitDialog::action(const gcn::ActionEvent &event)
     scheduleDelete();
 }
 
-void QuitDialog::keyPressed(gcn::KeyEvent &keyEvent)
+void QuitDialog::keyPressed(KeyEvent &event)
 {
-    const int actionId = static_cast<KeyEvent*>(&keyEvent)->getActionId();
+    const int actionId = event.getActionId();
     int dir = 0;
 
     switch (actionId)
     {
-        case Input::KEY_GUI_SELECT:
-        case Input::KEY_GUI_SELECT2:
-            action(gcn::ActionEvent(nullptr, mOkButton->getActionEventId()));
+        case InputAction::GUI_SELECT:
+        case InputAction::GUI_SELECT2:
+            action(ActionEvent(nullptr, mOkButton->getActionEventId()));
             break;
-        case Input::KEY_GUI_CANCEL:
-            action(gcn::ActionEvent(nullptr,
-                mCancelButton->getActionEventId()));
+        case InputAction::GUI_CANCEL:
+            action(ActionEvent(nullptr, mCancelButton->getActionEventId()));
             break;
-        case Input::KEY_GUI_UP:
+        case InputAction::GUI_UP:
             dir = -1;
             break;
-        case Input::KEY_GUI_DOWN:
+        case InputAction::GUI_DOWN:
             dir = 1;
             break;
         default:
