@@ -26,12 +26,22 @@
 #include "client.h"
 #include "configuration.h"
 #include "main.h"
+#include "settings.h"
 
-#include "input/keydata.h"
-#include "input/keyevent.h"
+#include "events/keyevent.h"
 
-#include "gui/sdlfont.h"
+#include "input/inputaction.h"
 
+#include "net/download.h"
+
+#include "gui/gui.h"
+
+#include "gui/fonts/font.h"
+
+#include "gui/models/serverslistmodel.h"
+
+#include "gui/widgets/checkbox.h"
+#include "gui/widgets/desktop.h"
 #include "gui/windows/editserverdialog.h"
 #include "gui/windows/logindialog.h"
 
@@ -39,13 +49,13 @@
 #include "gui/widgets/label.h"
 #include "gui/widgets/layout.h"
 #include "gui/widgets/listbox.h"
+#include "gui/widgets/serverslistbox.h"
 #include "gui/widgets/scrollarea.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 #include "utils/langs.h"
 
-#include <guichan/font.hpp>
-
 #include <string>
 
 #include "debug.h"
@@ -60,18 +70,9 @@ static std::string serverTypeToString(const ServerInfo::Type type)
             return "TmwAthena";
         case ServerInfo::EVOL:
             return "Evol";
+        case ServerInfo::EATHENA:
 #ifdef EATHENA_SUPPORT
-        case ServerInfo::EATHENA:
             return "eAthena";
-#endif
-#ifdef MANASERV_SUPPORT
-        case ServerInfo::MANASERV:
-            return "ManaServ";
-#else
-        case ServerInfo::MANASERV:
-#endif
-#ifndef EATHENA_SUPPORT
-        case ServerInfo::EATHENA:
 #endif
         default:
         case ServerInfo::UNKNOWN:
@@ -87,168 +88,24 @@ static uint16_t defaultPortForServerType(const ServerInfo::Type type)
         case ServerInfo::EATHENA:
 #ifdef EATHENA_SUPPORT
             return 6900;
-#else
-            return 6901;
 #endif
         case ServerInfo::UNKNOWN:
         case ServerInfo::TMWATHENA:
         case ServerInfo::EVOL:
-#ifdef MANASERV_SUPPORT
             return 6901;
-        case ServerInfo::MANASERV:
-            return 9601;
-#else
-        case ServerInfo::MANASERV:
-            return 6901;
-#endif
     }
 }
 
-ServersListModel::ServersListModel(ServerInfos *const servers,
-                                   ServerDialog *const parent) :
-        mServers(servers),
-        mVersionStrings(servers->size(), VersionString(0, "")),
-        mParent(parent)
-{
-}
-
-int ServersListModel::getNumberOfElements()
-{
-    MutexLocker lock = mParent->lock();
-    return static_cast<int>(mServers->size());
-}
-
-std::string ServersListModel::getElementAt(int elementIndex)
-{
-    MutexLocker lock = mParent->lock();
-    const ServerInfo &server = mServers->at(elementIndex);
-    std::string myServer;
-    myServer.append(server.hostname);
-    return myServer;
-}
-
-void ServersListModel::setVersionString(const int index,
-                                        const std::string &version)
-{
-    if (index < 0 || index >= static_cast<int>(mVersionStrings.size()))
-        return;
-
-    if (version.empty())
-    {
-        mVersionStrings[index] = VersionString(0, "");
-    }
-    else
-    {
-        mVersionStrings[index] = VersionString(
-            gui->getFont()->getWidth(version), version);
-    }
-}
-
-class ServersListBox final : public ListBox
-{
-public:
-    ServersListBox(const Widget2 *const widget,
-                   ServersListModel *const model) :
-        ListBox(widget, model, "serverslistbox.xml"),
-        mNotSupportedColor(getThemeColor(Theme::SERVER_VERSION_NOT_SUPPORTED)),
-        mNotSupportedColor2(getThemeColor(
-            Theme::SERVER_VERSION_NOT_SUPPORTED_OUTLINE))
-    {
-        mHighlightColor = getThemeColor(Theme::HIGHLIGHT);
-    }
-
-    void draw(gcn::Graphics *graphics) override final
-    {
-        if (!mListModel)
-            return;
-
-        ServersListModel *const model = static_cast<ServersListModel *const>(
-            mListModel);
-        Graphics *const g = static_cast<Graphics*>(graphics);
-
-        updateAlpha();
-
-        mHighlightColor.a = static_cast<int>(mAlpha * 255.0F);
-        g->setColor(mHighlightColor);
-
-        const int height = getRowHeight();
-        mNotSupportedColor.a = static_cast<int>(mAlpha * 255.0F);
-
-        // Draw filled rectangle around the selected list element
-        if (mSelected >= 0)
-        {
-            graphics->fillRectangle(gcn::Rectangle(mPadding,
-                height * mSelected + mPadding, getWidth() - 2 * mPadding,
-                height));
-        }
-
-        gcn::Font *const font1 = boldFont;
-        gcn::Font *const font2 = getFont();
-        const int fontHeight = font1->getHeight();
-        const int pad1 = fontHeight + mPadding;
-        const int pad2 = height / 4 + mPadding;
-        const int width = getWidth();
-        // Draw the list elements
-        for (int i = 0, y = 0; i < model->getNumberOfElements();
-             ++i, y += height)
-        {
-            const ServerInfo &info = model->getServer(i);
-
-            if (mSelected == i)
-            {
-                g->setColorAll(mForegroundSelectedColor,
-                    mForegroundSelectedColor2);
-            }
-            else
-            {
-                g->setColorAll(mForegroundColor, mForegroundColor2);
-            }
-
-            int top;
-            int x = mPadding;
-
-            if (!info.name.empty())
-            {
-                x += font1->getWidth(info.name) + 15;
-                font1->drawString(graphics, info.name, mPadding, y + mPadding);
-                top = y + pad1;
-            }
-            else
-            {
-                top = y + pad2;
-            }
-
-            if (!info.description.empty())
-                font2->drawString(graphics, info.description, x, y + mPadding);
-            font2->drawString(graphics, model->getElementAt(i), mPadding, top);
-
-            if (info.version.first > 0)
-            {
-                g->setColorAll(mNotSupportedColor, mNotSupportedColor2);
-                font2->drawString(graphics, info.version.second,
-                    width - info.version.first - mPadding, top);
-            }
-        }
-    }
-
-    unsigned int getRowHeight() const override final
-    {
-        return 2 * getFont()->getHeight() + 5;
-    }
-private:
-    gcn::Color mNotSupportedColor;
-    gcn::Color mNotSupportedColor2;
-};
-
-
 ServerDialog::ServerDialog(ServerInfo *const serverInfo,
                            const std::string &dir) :
     // TRANSLATORS: servers dialog name
     Window(_("Choose Your Server"), false, nullptr, "server.xml"),
-    gcn::ActionListener(),
-    gcn::KeyListener(),
-    gcn::SelectionListener(),
+    ActionListener(),
+    KeyListener(),
+    SelectionListener(),
     mMutex(),
+    mServers(ServerInfos()),
+    mDir(dir),
     mDescription(new Label(this, std::string())),
     // TRANSLATORS: servers dialog button
     mQuitButton(new Button(this, _("Quit"), "quit", this)),
@@ -262,15 +119,13 @@ ServerDialog::ServerDialog(ServerInfo *const serverInfo,
     mDeleteButton(new Button(this, _("Delete"), "remove", this)),
     // TRANSLATORS: servers dialog button
     mLoadButton(new Button(this, _("Load"), "load", this)),
-    mServers(ServerInfos()),
     mServersListModel(new ServersListModel(&mServers, this)),
     mServersList(new ServersListBox(this, mServersListModel)),
-    mDir(dir),
-    mDownloadStatus(DOWNLOADING_UNKNOWN),
     mDownload(nullptr),
-    mDownloadProgress(-1.0F),
     mServerInfo(serverInfo),
-    mPersistentIPCheckBox(nullptr)
+    mPersistentIPCheckBox(nullptr),
+    mDownloadProgress(-1.0F),
+    mDownloadStatus(DOWNLOADING_UNKNOWN)
 {
     mServersList->postInit();
 
@@ -294,9 +149,9 @@ ServerDialog::ServerDialog(ServerInfo *const serverInfo,
 
     mServersList->addMouseListener(this);
 
-    ScrollArea *const usedScroll = new ScrollArea(mServersList,
+    ScrollArea *const usedScroll = new ScrollArea(this, mServersList,
         getOptionBool("showbackground"), "server_background.xml");
-    usedScroll->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    usedScroll->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
 
     mServersList->addSelectionListener(this);
     usedScroll->setVerticalScrollAmount(0);
@@ -351,11 +206,9 @@ ServerDialog::~ServerDialog()
     if (mDownload)
     {
         mDownload->cancel();
-        delete mDownload;
-        mDownload = nullptr;
+        delete2(mDownload)
     }
-    delete mServersListModel;
-    mServersListModel = nullptr;
+    delete2(mServersListModel);
 }
 
 void ServerDialog::connectToSelectedServer()
@@ -385,6 +238,12 @@ void ServerDialog::connectToSelectedServer()
     mServerInfo->onlineListUrl = server.onlineListUrl;
     mServerInfo->supportUrl = server.supportUrl;
     mServerInfo->save = true;
+    mServerInfo->persistentIp = server.persistentIp;
+    mServerInfo->updateMirrors = server.updateMirrors;
+
+    settings.persistentIp = mServerInfo->persistentIp;
+    settings.supportUrl = mServerInfo->supportUrl;
+    settings.updateMirrors = mServerInfo->updateMirrors;
 
     if (chatLogger)
         chatLogger->setServerName(mServerInfo->hostname);
@@ -396,7 +255,8 @@ void ServerDialog::connectToSelectedServer()
         if (mServerInfo->hostname != LoginDialog::savedPasswordKey)
         {
             LoginDialog::savedPassword.clear();
-            client->reloadWallpaper();
+            if (desktop)
+                desktop->reloadWallpaper();
         }
     }
 
@@ -405,7 +265,7 @@ void ServerDialog::connectToSelectedServer()
     client->setState(STATE_CONNECT_SERVER);
 }
 
-void ServerDialog::action(const gcn::ActionEvent &event)
+void ServerDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "connect")
@@ -445,27 +305,27 @@ void ServerDialog::action(const gcn::ActionEvent &event)
     }
 }
 
-void ServerDialog::keyPressed(gcn::KeyEvent &keyEvent)
+void ServerDialog::keyPressed(KeyEvent &event)
 {
-    switch (static_cast<KeyEvent*>(&keyEvent)->getActionId())
+    switch (event.getActionId())
     {
-        case Input::KEY_GUI_CANCEL:
-            keyEvent.consume();
+        case InputAction::GUI_CANCEL:
+            event.consume();
             client->setState(STATE_EXIT);
             return;
 
-        case Input::KEY_GUI_SELECT:
-        case Input::KEY_GUI_SELECT2:
-            keyEvent.consume();
-            action(gcn::ActionEvent(nullptr,
+        case InputAction::GUI_SELECT:
+        case InputAction::GUI_SELECT2:
+            event.consume();
+            action(ActionEvent(nullptr,
                 mConnectButton->getActionEventId()));
             return;
 
-        case Input::KEY_GUI_INSERT:
+        case InputAction::GUI_INSERT:
             (new EditServerDialog(this, ServerInfo(), -1))->postInit();
             return;
 
-        case Input::KEY_GUI_DELETE:
+        case InputAction::GUI_DELETE:
         {
             const int index = mServersList->getSelected();
             if (index >= 0)
@@ -477,7 +337,7 @@ void ServerDialog::keyPressed(gcn::KeyEvent &keyEvent)
             return;
         }
 
-        case Input::KEY_GUI_BACKSPACE:
+        case InputAction::GUI_BACKSPACE:
         {
             const int index = mServersList->getSelected();
             if (index >= 0)
@@ -491,11 +351,11 @@ void ServerDialog::keyPressed(gcn::KeyEvent &keyEvent)
         default:
             break;
     }
-    if (!keyEvent.isConsumed())
-        mServersList->keyPressed(keyEvent);
+    if (!event.isConsumed())
+        mServersList->keyPressed(event);
 }
 
-void ServerDialog::valueChanged(const gcn::SelectionEvent &)
+void ServerDialog::valueChanged(const SelectionEvent &)
 {
     const int index = mServersList->getSelected();
     if (index == -1)
@@ -506,13 +366,17 @@ void ServerDialog::valueChanged(const gcn::SelectionEvent &)
     mDeleteButton->setEnabled(true);
 }
 
-void ServerDialog::mouseClicked(gcn::MouseEvent &mouseEvent)
+void ServerDialog::mouseClicked(MouseEvent &event)
 {
-    if (mouseEvent.getClickCount() == 2 &&
-        mouseEvent.getSource() == mServersList)
+    if (event.getButton() == MouseButton::LEFT)
     {
-        action(gcn::ActionEvent(mConnectButton,
-                                mConnectButton->getActionEventId()));
+        event.consume();
+        if (event.getClickCount() == 2 &&
+            event.getSource() == mServersList)
+        {
+            action(ActionEvent(mConnectButton,
+                mConnectButton->getActionEventId()));
+        }
     }
 }
 
@@ -569,11 +433,11 @@ void ServerDialog::downloadServerList()
     if (mDownload)
     {
         mDownload->cancel();
-        delete mDownload;
-        mDownload = nullptr;
+        delete2(mDownload)
     }
 
-    mDownload = new Net::Download(this, listFile, &downloadUpdate);
+    mDownload = new Net::Download(this, listFile,
+        &downloadUpdate, false, false, true);
     mDownload->setFile(std::string(mDir).append("/").append(
         branding.getStringValue("onlineServerFile")));
     mDownload->start();
@@ -643,7 +507,7 @@ void ServerDialog::loadServers(const bool addNew)
             version = strprintf(_("requires v%s"), version.c_str());
         }
 
-        const gcn::Font *const font = gui->getFont();
+        const Font *const font = gui->getFont();
 
         for_each_xml_child_node(subNode, serverNode)
         {
@@ -683,6 +547,17 @@ void ServerDialog::loadServers(const bool addNew)
                 server.supportUrl = reinterpret_cast<const char*>(
                     subNode->xmlChildrenNode->content);
             }
+            else if (xmlNameEqual(subNode, "persistentIp"))
+            {
+                std::string text = reinterpret_cast<const char*>(
+                    subNode->xmlChildrenNode->content);
+                server.persistentIp = (text == "1" || text == "true");
+            }
+            else if (xmlNameEqual(subNode, "updateMirror"))
+            {
+                server.updateMirrors.push_back(reinterpret_cast<const char*>(
+                    subNode->xmlChildrenNode->content));
+            }
         }
 
         server.version.first = font->getWidth(version);
@@ -704,6 +579,8 @@ void ServerDialog::loadServers(const bool addNew)
                 mServers[i].onlineListUrl = server.onlineListUrl;
                 mServers[i].supportUrl = server.supportUrl;
                 mServers[i].althostname = server.althostname;
+                mServers[i].persistentIp = server.persistentIp;
+                mServers[i].updateMirrors = server.updateMirrors;
                 mServersListModel->setVersionString(i, version);
                 found = true;
                 break;
@@ -728,6 +605,7 @@ void ServerDialog::loadCustomServers()
         const std::string portKey("MostUsedServerPort" + index);
         const std::string onlineListUrlKey
             ("MostUsedServerOnlineList" + index);
+        const std::string persistentIpKey("persistentIp" + index);
 
         ServerInfo server;
         server.name = config.getValue(nameKey, "");
@@ -735,6 +613,8 @@ void ServerDialog::loadCustomServers()
         server.onlineListUrl = config.getValue(onlineListUrlKey, "");
         server.hostname = config.getValue(hostKey, "");
         server.type = ServerInfo::parseType(config.getValue(typeKey, ""));
+        server.persistentIp = config.getValue(
+            persistentIpKey, 0) ? true : false;
 
         const int defaultPort = defaultPortForServerType(server.type);
         server.port = static_cast<uint16_t>(
@@ -755,7 +635,7 @@ void ServerDialog::saveCustomServers(const ServerInfo &currentServer,
     // Make sure the current server is mentioned first
     if (currentServer.isValid())
     {
-        if (index >= 0 && static_cast<unsigned>(index) < mServers.size())
+        if (index >= 0 && static_cast<size_t>(index) < mServers.size())
         {
             mServers[index] = currentServer;
         }
@@ -792,6 +672,7 @@ void ServerDialog::saveCustomServers(const ServerInfo &currentServer,
         const std::string portKey("MostUsedServerPort" + num);
         const std::string onlineListUrlKey
             ("MostUsedServerOnlineList" + num);
+        const std::string persistentIpKey("persistentIp" + num);
 
         config.setValue(nameKey, toString(server.name));
         config.setValue(descKey, toString(server.description));
@@ -799,6 +680,7 @@ void ServerDialog::saveCustomServers(const ServerInfo &currentServer,
         config.setValue(hostKey, toString(server.hostname));
         config.setValue(typeKey, serverTypeToString(server.type));
         config.setValue(portKey, toString(server.port));
+        config.setValue(persistentIpKey, server.persistentIp);
         ++ savedServerCount;
     }
 
@@ -807,10 +689,10 @@ void ServerDialog::saveCustomServers(const ServerInfo &currentServer,
         config.setValue("MostUsedServerName" + toString(savedServerCount), "");
 }
 
-int ServerDialog::downloadUpdate(void *ptr, DownloadStatus status,
+int ServerDialog::downloadUpdate(void *ptr, DownloadStatus::Type status,
                                  size_t total, size_t remaining)
 {
-    if (!ptr || status == DOWNLOAD_STATUS_CANCELLED)
+    if (!ptr || status == DownloadStatus::Cancelled)
         return -1;
 
     ServerDialog *const sd = reinterpret_cast<ServerDialog*>(ptr);
@@ -819,11 +701,11 @@ int ServerDialog::downloadUpdate(void *ptr, DownloadStatus status,
     if (!sd->mDownload)
         return -1;
 
-    if (status == DOWNLOAD_STATUS_COMPLETE)
+    if (status == DownloadStatus::Complete)
     {
         finished = true;
     }
-    else if (status < 0)
+    else if (static_cast<int>(status) < 0)
     {
         logger->log("Error retreiving server list: %s\n",
                     sd->mDownload->getError());
