@@ -20,33 +20,39 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "gui/windows/shopwindow.h"
+
+#include "gui/chatconsts.h"
+
 #include "gui/windows/buydialog.h"
+#include "gui/windows/chatwindow.h"
 #include "gui/windows/confirmdialog.h"
 #include "gui/windows/itemamountwindow.h"
-#include "gui/windows/shopwindow.h"
 #include "gui/windows/selldialog.h"
+#include "gui/windows/setupwindow.h"
 #include "gui/windows/tradewindow.h"
 
+#include "gui/models/shopitems.h"
+
 #include "gui/widgets/button.h"
 #include "gui/widgets/checkbox.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/scrollarea.h"
-#include "gui/widgets/shopitems.h"
 #include "gui/widgets/shoplistbox.h"
 
-#include "gui/widgets/tabs/chattab.h"
-
 #include "actormanager.h"
 #include "auctionmanager.h"
-#include "client.h"
 #include "configuration.h"
 #include "inventory.h"
 #include "item.h"
+#include "settings.h"
 #include "shopitem.h"
-#include "soundconsts.h"
 #include "soundmanager.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 #include "being/playerrelations.h"
@@ -57,6 +63,7 @@
 
 #include "resources/iteminfo.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include <sstream>
@@ -65,23 +72,25 @@
 
 #include "debug.h"
 
+ShopWindow *shopWindow = nullptr;
 extern std::string tradePartnerName;
 ShopWindow::DialogList ShopWindow::instances;
 
-ShopWindow::ShopWindow():
+ShopWindow::ShopWindow() :
     // TRANSLATORS: shop window name
     Window(_("Personal Shop"), false, nullptr, "shop.xml"),
-    gcn::ActionListener(),
-    gcn::SelectionListener(),
+    ActionListener(),
+    SelectionListener(),
     // TRANSLATORS: shop window button
     mCloseButton(new Button(this, _("Close"), "close", this)),
     mBuyShopItems(new ShopItems),
     mSellShopItems(new ShopItems),
+    mTradeItem(nullptr),
     mBuyShopItemList(new ShopListBox(this, mBuyShopItems, mBuyShopItems)),
     mSellShopItemList(new ShopListBox(this, mSellShopItems, mSellShopItems)),
-    mBuyScrollArea(new ScrollArea(mBuyShopItemList,
+    mBuyScrollArea(new ScrollArea(this, mBuyShopItemList,
         getOptionBool("showbuybackground"), "shop_buy_background.xml")),
-    mSellScrollArea(new ScrollArea(mSellShopItemList,
+    mSellScrollArea(new ScrollArea(this, mSellShopItemList,
         getOptionBool("showsellbackground"), "shop_sell_background.xml")),
     // TRANSLATORS: shop window label
     mBuyLabel(new Label(this, _("Buy items"))),
@@ -105,14 +114,13 @@ ShopWindow::ShopWindow():
     // TRANSLATORS: shop window checkbox
     mAnnounceLinks(new CheckBox(this, _("Show links in announce"), false,
         this, "link announce")),
+    mAcceptPlayer(""),
+    mTradeNick(""),
     mSelectedItem(-1),
     mAnnonceTime(0),
     mLastRequestTimeList(0),
     mLastRequestTimeItem(0),
     mRandCounter(0),
-    mAcceptPlayer(""),
-    mTradeItem(nullptr),
-    mTradeNick(""),
     mTradeMoney(0)
 {
     mBuyShopItemList->postInit();
@@ -129,6 +137,9 @@ ShopWindow::ShopWindow():
     else
         setDefaultSize(380, 300, ImageRect::CENTER);
 
+    if (setupWindow)
+        setupWindow->registerWindowForReset(this);
+
     mAnnounceCounter[BUY] = 0;
     mAnnounceCounter[SELL] = 0;
 
@@ -137,8 +148,8 @@ ShopWindow::ShopWindow():
     mBuyShopItemList->setPriceCheck(false);
     mSellShopItemList->setPriceCheck(false);
 
-    mBuyScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
-    mSellScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    mBuyScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
+    mSellScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
 
     mBuyShopItemList->addSelectionListener(this);
     mSellShopItemList->addSelectionListener(this);
@@ -177,7 +188,7 @@ ShopWindow::ShopWindow():
     }
 
     Layout &layout = getLayout();
-    layout.setRowHeight(0, Layout::AUTO_SET);
+    layout.setRowHeight(0, LayoutType::SET);
 
     center();
     loadWindowState();
@@ -196,16 +207,13 @@ ShopWindow::~ShopWindow()
 {
     saveList();
 
-    delete mBuyShopItems;
-    mBuyShopItems = nullptr;
-
-    delete mSellShopItems;
-    mSellShopItems = nullptr;
+    delete2(mBuyShopItems);
+    delete2(mSellShopItems);
 
     instances.remove(this);
 }
 
-void ShopWindow::action(const gcn::ActionEvent &event)
+void ShopWindow::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "close")
@@ -230,15 +238,15 @@ void ShopWindow::action(const gcn::ActionEvent &event)
              && mBuyShopItemList->getSelected() >= 0)
     {
         mBuyShopItems->del(mBuyShopItemList->getSelected());
-        if (isShopEmpty() && player_node)
-            player_node->updateStatus();
+        if (isShopEmpty() && localPlayer)
+            localPlayer->updateStatus();
     }
     else if (eventId == "delete sell" && mSellShopItemList
              && mSellShopItemList->getSelected() >= 0)
     {
         mSellShopItems->del(mSellShopItemList->getSelected());
-        if (isShopEmpty() && player_node)
-            player_node->updateStatus();
+        if (isShopEmpty() && localPlayer)
+            localPlayer->updateStatus();
     }
     else if (eventId == "announce buy" && mBuyShopItems
              && mBuyShopItems->getNumberOfElements() > 0)
@@ -291,7 +299,7 @@ void ShopWindow::startTrade()
         return;
 
     const Being *const being = actorManager->findBeingByName(
-        mTradeNick, Being::PLAYER);
+        mTradeNick, ActorType::PLAYER);
     tradeWindow->clear();
     if (mTradeMoney)
     {
@@ -307,7 +315,7 @@ void ShopWindow::startTrade()
     mTradeNick.clear();
 }
 
-void ShopWindow::valueChanged(const gcn::SelectionEvent &event A_UNUSED)
+void ShopWindow::valueChanged(const SelectionEvent &event A_UNUSED)
 {
     updateButtonsAndLabels();
 }
@@ -337,8 +345,8 @@ void ShopWindow::addBuyItem(const Item *const item, const int amount,
     const bool emp = isShopEmpty();
     mBuyShopItems->addItemNoDup(item->getId(),
         item->getColor(), amount, price);
-    if (emp && player_node)
-        player_node->updateStatus();
+    if (emp && localPlayer)
+        localPlayer->updateStatus();
 
     updateButtonsAndLabels();
 }
@@ -351,8 +359,8 @@ void ShopWindow::addSellItem(const Item *const item, const int amount,
     const bool emp = isShopEmpty();
     mSellShopItems->addItemNoDup(item->getId(),
         item->getColor(), amount, price);
-    if (emp && player_node)
-        player_node->updateStatus();
+    if (emp && localPlayer)
+        localPlayer->updateStatus();
 
     updateButtonsAndLabels();
 }
@@ -368,7 +376,7 @@ void ShopWindow::loadList()
     mBuyShopItems->clear();
     mSellShopItems->clear();
 
-    const std::string shopListName = client->getServerConfigDirectory()
+    const std::string shopListName = settings.serverConfigDir
         + "/shoplist.txt";
 
     if (!stat(shopListName.c_str(), &statbuf) && S_ISREG(statbuf.st_mode))
@@ -418,7 +426,7 @@ void ShopWindow::saveList() const
         return;
 
     std::ofstream shopFile;
-    const std::string shopListName = client->getServerConfigDirectory()
+    const std::string shopListName = settings.serverConfigDir
         + "/shoplist.txt";
     std::map<int, ShopItem*> mapItems;
 
@@ -594,9 +602,9 @@ void ShopWindow::giveList(const std::string &nick, const int mode)
         {
             int amount = item->getQuantity();
             if (item->getPrice() * amount > PlayerInfo::getAttribute(
-                PlayerInfo::MONEY))
+                Attributes::MONEY))
             {
-                amount = PlayerInfo::getAttribute(PlayerInfo::MONEY)
+                amount = PlayerInfo::getAttribute(Attributes::MONEY)
                     / item->getPrice();
             }
 
@@ -629,10 +637,10 @@ void ShopWindow::sendMessage(const std::string &nick,
     if (config.getBoolValue("hideShopMessages"))
         Net::getChatHandler()->privateMessage(nick, data);
     else
-        chatWindow->addWhisper(nick, data, BY_PLAYER);
+        chatWindow->addWhisper(nick, data, ChatMsgType::BY_PLAYER);
 }
 
-void ShopWindow::showList(const std::string &nick, std::string data) const
+void ShopWindow::showList(const std::string &nick, std::string data)
 {
     BuyDialog *buyDialog = nullptr;
     SellDialog *sellDialog = nullptr;
@@ -656,9 +664,9 @@ void ShopWindow::showList(const std::string &nick, std::string data) const
         return;
 
     if (buyDialog)
-        buyDialog->setMoney(PlayerInfo::getAttribute(PlayerInfo::MONEY));
+        buyDialog->setMoney(PlayerInfo::getAttribute(Attributes::MONEY));
     if (sellDialog)
-        sellDialog->setMoney(PlayerInfo::getAttribute(PlayerInfo::MONEY));
+        sellDialog->setMoney(PlayerInfo::getAttribute(Attributes::MONEY));
 
     for (unsigned f = 0; f < data.length(); f += 9)
     {
@@ -693,9 +701,9 @@ void ShopWindow::showList(const std::string &nick, std::string data) const
 void ShopWindow::processRequest(const std::string &nick, std::string data,
                                 const int mode)
 {
-    if (!player_node || !mTradeNick.empty() || PlayerInfo::isTrading()
+    if (!localPlayer || !mTradeNick.empty() || PlayerInfo::isTrading()
         || !actorManager
-        || !actorManager->findBeingByName(nick, Being::PLAYER))
+        || !actorManager->findBeingByName(nick, ActorType::PLAYER))
     {
         return;
     }
@@ -849,7 +857,7 @@ bool ShopWindow::findShopItem(const ShopItem *const shopItem,
 
 int ShopWindow::sumAmount(const Item *const shopItem)
 {
-    if (!player_node || !shopItem)
+    if (!localPlayer || !shopItem)
         return 0;
 
     const Inventory *const inv = PlayerInfo::getInventory();
