@@ -22,42 +22,35 @@
 
 #include "gui/windows/shortcutwindow.h"
 
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
+#include "gui/widgets/tabbedarea.h"
 
 #include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/scrollarea.h"
 #include "gui/widgets/shortcutcontainer.h"
-#include "gui/widgets/tabs/tab.h"
+
+#include "gui/widgets/tabs/shortcuttab.h"
+
+#include "utils/delete2.h"
 
 #include "debug.h"
 
+ShortcutWindow *dropShortcutWindow = nullptr;
+ShortcutWindow *emoteShortcutWindow = nullptr;
+ShortcutWindow *itemShortcutWindow = nullptr;
+ShortcutWindow *spellShortcutWindow = nullptr;
 static const int SCROLL_PADDING = 0;
 
 int ShortcutWindow::mBoxesWidth = 0;
 
-class ShortcutTab final : public Tab
-{
-    public:
-        ShortcutTab(const Widget2 *const widget,
-                    std::string name, ShortcutContainer *const content) :
-            Tab(widget),
-            mContent(content)
-        {
-            setCaption(name);
-        }
-
-        A_DELETE_COPY(ShortcutTab)
-
-        ShortcutContainer* mContent;
-};
-
 ShortcutWindow::ShortcutWindow(const std::string &restrict title,
                                ShortcutContainer *restrict const content,
                                const std::string &restrict skinFile,
                                int width, int height) :
     Window("Window", false, nullptr, skinFile),
     mItems(content),
-    mScrollArea(new ScrollArea(mItems, false)),
+    mScrollArea(new ScrollArea(this, mItems, false)),
     mTabs(nullptr),
     mPages()
 {
@@ -95,12 +88,12 @@ ShortcutWindow::ShortcutWindow(const std::string &restrict title,
     mBoxesWidth += bw + border;
 
     mScrollArea->setPosition(SCROLL_PADDING, SCROLL_PADDING);
-    mScrollArea->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    mScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
 
     place(0, 0, mScrollArea, 5, 5).setPadding(0);
 
     Layout &layout = getLayout();
-    layout.setRowHeight(0, Layout::AUTO_SET);
+    layout.setRowHeight(0, LayoutType::SET);
     layout.setMargin(0);
 
     loadWindowState();
@@ -139,7 +132,7 @@ ShortcutWindow::ShortcutWindow(const std::string &restrict title,
     place(0, 0, mTabs, 5, 5);
 
     Layout &layout = getLayout();
-    layout.setRowHeight(0, Layout::AUTO_SET);
+    layout.setRowHeight(0, LayoutType::SET);
     layout.setMargin(0);
 
     loadWindowState();
@@ -150,18 +143,16 @@ ShortcutWindow::~ShortcutWindow()
 {
     if (mTabs)
         mTabs->removeAll();
-    delete mTabs;
-    mTabs = nullptr;
-    delete mItems;
-    mItems = nullptr;
+    delete2(mTabs);
+    delete2(mItems);
 }
 
 void ShortcutWindow::addTab(const std::string &name,
                             ShortcutContainer *const content)
 {
-    ScrollArea *const scroll = new ScrollArea(content, false);
+    ScrollArea *const scroll = new ScrollArea(this, content, false);
     scroll->setPosition(SCROLL_PADDING, SCROLL_PADDING);
-    scroll->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
+    scroll->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
     content->setWidget2(this);
     Tab *const tab = new ShortcutTab(this, name, content);
     mTabs->addTab(tab, scroll);
@@ -175,7 +166,7 @@ int ShortcutWindow::getTabIndex() const
     return mTabs->getSelectedTabIndex();
 }
 
-void ShortcutWindow::widgetHidden(const gcn::Event &event)
+void ShortcutWindow::widgetHidden(const Event &event)
 {
     if (mItems)
         mItems->widgetHidden(event);
@@ -194,21 +185,21 @@ void ShortcutWindow::widgetHidden(const gcn::Event &event)
     }
 }
 
-void ShortcutWindow::mousePressed(gcn::MouseEvent &event)
+void ShortcutWindow::mousePressed(MouseEvent &event)
 {
     Window::mousePressed(event);
 
     if (event.isConsumed())
         return;
 
-    if (event.getButton() == gcn::MouseEvent::LEFT)
+    if (event.getButton() == MouseButton::LEFT)
     {
         mDragOffsetX = event.getX();
         mDragOffsetY = event.getY();
     }
 }
 
-void ShortcutWindow::mouseDragged(gcn::MouseEvent &event)
+void ShortcutWindow::mouseDragged(MouseEvent &event)
 {
     Window::mouseDragged(event);
 
@@ -225,7 +216,7 @@ void ShortcutWindow::mouseDragged(gcn::MouseEvent &event)
     }
 }
 
-void ShortcutWindow::widgetMoved(const gcn::Event& event)
+void ShortcutWindow::widgetMoved(const Event& event)
 {
     Window::widgetMoved(event);
     if (mItems)
@@ -234,6 +225,18 @@ void ShortcutWindow::widgetMoved(const gcn::Event& event)
         (*it)->setRedraw(true);
 }
 
+void ShortcutWindow::nextTab()
+{
+    if (mTabs)
+        mTabs->selectNextTab();
+}
+
+void ShortcutWindow::prevTab()
+{
+    if (mTabs)
+        mTabs->selectPrevTab();
+}
+
 #ifdef USE_PROFILER
 void ShortcutWindow::logicChildren()
 {
