@@ -45,9 +45,9 @@ class ShortcutWindow final : public Window
                        const std::string &restrict skinFile = "",
                        int width = 0, int height = 0);
 
-        ShortcutWindow(const std::string &restrict title,
-                       const std::string &restrict skinFile = "",
-                       const int width = 0, const int height = 0);
+        explicit ShortcutWindow(const std::string &restrict title,
+                                const std::string &restrict skinFile = "",
+                                const int width = 0, const int height = 0);
 
         A_DELETE_COPY(ShortcutWindow)
 
@@ -60,13 +60,17 @@ class ShortcutWindow final : public Window
 
         int getTabIndex() const A_WARN_UNUSED;
 
-        void widgetHidden(const gcn::Event &event) override final;
+        void widgetHidden(const Event &event) override final;
 
-        void widgetMoved(const gcn::Event& event) override final;
+        void widgetMoved(const Event& event) override final;
 
-        void mousePressed(gcn::MouseEvent &event) override final;
+        void mousePressed(MouseEvent &event) override final;
 
-        void mouseDragged(gcn::MouseEvent &event) override final;
+        void mouseDragged(MouseEvent &event) override final;
+
+        void nextTab();
+
+        void prevTab();
 
 #ifdef USE_PROFILER
         void logicChildren();
@@ -83,8 +87,9 @@ class ShortcutWindow final : public Window
         static int mBoxesWidth;
 };
 
-extern ShortcutWindow *itemShortcutWindow;
-extern ShortcutWindow *emoteShortcutWindow;
 extern ShortcutWindow *dropShortcutWindow;
+extern ShortcutWindow *emoteShortcutWindow;
+extern ShortcutWindow *itemShortcutWindow;
+extern ShortcutWindow *spellShortcutWindow;
 
 #endif  // GUI_WINDOWS_SHORTCUTWINDOW_H
