@@ -24,125 +24,51 @@
 
 #include "configuration.h"
 #include "equipment.h"
+#include "gamemodifiers.h"
 #include "inventory.h"
 #include "item.h"
+#include "settings.h"
 #include "units.h"
 
 #include "gui/windows/chatwindow.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 
-#include "gui/viewport.h"
-
 #include "gui/windows/equipmentwindow.h"
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 
 #include "gui/widgets/button.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layouthelper.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/progressbar.h"
 #include "gui/widgets/scrollarea.h"
+#include "gui/widgets/statuswindowattrs.h"
 #include "gui/widgets/vertcontainer.h"
+#include "gui/widgets/windowcontainer.h"
 
 #include "net/net.h"
 #include "net/playerhandler.h"
 #include "net/gamehandler.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include <SDL_timer.h>
 
 #include "debug.h"
 
-class AttrDisplay : public Container
-{
-    public:
-        enum Type
-        {
-            DERIVED = 0,
-            CHANGEABLE,
-            UNKNOWN
-        };
-
-        A_DELETE_COPY(AttrDisplay)
-
-        virtual ~AttrDisplay();
-
-        virtual std::string update();
-
-        virtual Type getType() const
-        { return UNKNOWN; }
-
-        std::string getValue() const
-        {
-            if (!mValue)
-                return "-";
-            else
-                return mValue->getCaption();
-        }
-
-        const std::string &getShortName() const
-        { return mShortName; }
-
-    protected:
-        AttrDisplay(const Widget2 *const widget,
-                    const int id,
-                    const std::string &restrict name,
-                    const std::string &restrict shortName);
-
-        const int mId;
-        const std::string mName;
-        const std::string mShortName;
-
-        LayoutHelper *mLayout;
-        Label *mLabel;
-        Label *mValue;
-};
-
-class DerDisplay final : public AttrDisplay
-{
-    public:
-        DerDisplay(const Widget2 *const widget,
-                   const int id, const std::string &restrict name,
-                   const std::string &restrict shortName);
-
-        A_DELETE_COPY(DerDisplay)
-
-        Type getType() const override final
-        { return DERIVED; }
-};
-
-class ChangeDisplay final : public AttrDisplay, gcn::ActionListener
-{
-    public:
-        ChangeDisplay(const Widget2 *const widget,
-                      const int id, const std::string &restrict name,
-                      const std::string &restrict shortName);
-
-        A_DELETE_COPY(ChangeDisplay)
-
-        std::string update() override final;
-
-        Type getType() const override final
-        { return CHANGEABLE; }
-
-        void setPointsNeeded(const int needed);
-
-        void action(const gcn::ActionEvent &event) override final;
-
-    private:
-        int mNeeded;
-
-        Label *mPoints;
-        Button *mDec;
-        Button *mInc;
-};
+StatusWindow *statusWindow = nullptr;
 
 StatusWindow::StatusWindow() :
-    Window(player_node ? player_node->getName() :
+    Window(localPlayer ? localPlayer->getName() :
         "?", false, nullptr, "status.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
+    AttributeListener(),
+    StatListener(),
     // TRANSLATORS: status window label
     mLvlLabel(new Label(this, strprintf(_("Level: %d"), 0))),
     // TRANSLATORS: status window label
@@ -159,17 +85,15 @@ StatusWindow::StatusWindow() :
     mJobLabel(nullptr),
     mJobBar(nullptr),
     mAttrCont(new VertContainer(this, 32)),
-    mAttrScroll(new ScrollArea(mAttrCont, false)),
+    mAttrScroll(new ScrollArea(this, mAttrCont, false)),
     mDAttrCont(new VertContainer(this, 32)),
-    mDAttrScroll(new ScrollArea(mDAttrCont, false)),
+    mDAttrScroll(new ScrollArea(this, mDAttrCont, false)),
     mCharacterPointsLabel(new Label(this, "C")),
     mCorrectionPointsLabel(nullptr),
     // TRANSLATORS: status window button
     mCopyButton(new Button(this, _("Copy to chat"), "copy", this)),
     mAttrs()
 {
-    listen(CHANNEL_ATTRIBUTES);
-
     setWindowName("Status");
     if (setupWindow)
         setupWindow->registerWindowForReset(this);
@@ -180,29 +104,29 @@ StatusWindow::StatusWindow() :
     setDefaultSize((windowContainer->getWidth() - 480) / 2,
         (windowContainer->getHeight() - 500) / 2, 480, 500);
 
-    if (player_node && !player_node->getRaceName().empty())
+    if (localPlayer && !localPlayer->getRaceName().empty())
     {
-        setCaption(strprintf("%s (%s)", player_node->getName().c_str(),
-            player_node->getRaceName().c_str()));
+        setCaption(strprintf("%s (%s)", localPlayer->getName().c_str(),
+            localPlayer->getRaceName().c_str()));
     }
 
-    int max = PlayerInfo::getAttribute(PlayerInfo::MAX_HP);
+    int max = PlayerInfo::getAttribute(Attributes::MAX_HP);
     if (!max)
         max = 1;
 
     mHpBar = new ProgressBar(this, static_cast<float>(PlayerInfo::getAttribute(
-        PlayerInfo::HP)) / static_cast<float>(max), 80, 0, Theme::PROG_HP,
+        Attributes::HP)) / static_cast<float>(max), 80, 0, Theme::PROG_HP,
         "hpprogressbar.xml", "hpprogressbar_fill.xml");
-    mHpBar->setColor(Theme::getThemeColor(Theme::HP_BAR),
-        Theme::getThemeColor(Theme::HP_BAR_OUTLINE));
+    mHpBar->setColor(getThemeColor(Theme::HP_BAR),
+        getThemeColor(Theme::HP_BAR_OUTLINE));
 
-    max = PlayerInfo::getAttribute(PlayerInfo::EXP_NEEDED);
+    max = PlayerInfo::getAttribute(Attributes::EXP_NEEDED);
     mXpBar = new ProgressBar(this, max ?
-            static_cast<float>(PlayerInfo::getAttribute(PlayerInfo::EXP))
+            static_cast<float>(PlayerInfo::getAttribute(Attributes::EXP))
             / static_cast<float>(max) : static_cast<float>(0), 80, 0,
             Theme::PROG_EXP, "xpprogressbar.xml", "xpprogressbar_fill.xml");
-    mXpBar->setColor(Theme::getThemeColor(Theme::XP_BAR),
-        Theme::getThemeColor(Theme::XP_BAR_OUTLINE));
+    mXpBar->setColor(getThemeColor(Theme::XP_BAR),
+        getThemeColor(Theme::XP_BAR_OUTLINE));
 
     const bool magicBar = Net::getGameHandler()->canUseMagicBar();
     const int job = Net::getPlayerHandler()->getJobLocation()
@@ -210,25 +134,25 @@ StatusWindow::StatusWindow() :
 
     if (magicBar)
     {
-        max = PlayerInfo::getAttribute(PlayerInfo::MAX_MP);
+        max = PlayerInfo::getAttribute(Attributes::MAX_MP);
         // TRANSLATORS: status window label
         mMpLabel = new Label(this, _("MP:"));
         const bool useMagic = Net::getPlayerHandler()->canUseMagic();
         mMpBar = new ProgressBar(this, max ? static_cast<float>(
-            PlayerInfo::getAttribute(PlayerInfo::MAX_MP))
+            PlayerInfo::getAttribute(Attributes::MAX_MP))
             / static_cast<float>(max) : static_cast<float>(0),
             80, 0, useMagic ? Theme::PROG_MP : Theme::PROG_NO_MP,
             useMagic ? "mpprogressbar.xml" : "nompprogressbar.xml",
             useMagic ? "mpprogressbar_fill.xml" : "nompprogressbar_fill.xml");
         if (useMagic)
         {
-            mMpBar->setColor(Theme::getThemeColor(Theme::MP_BAR),
-                Theme::getThemeColor(Theme::MP_BAR_OUTLINE));
+            mMpBar->setColor(getThemeColor(Theme::MP_BAR),
+                getThemeColor(Theme::MP_BAR_OUTLINE));
         }
         else
         {
-            mMpBar->setColor(Theme::getThemeColor(Theme::NO_MP_BAR),
-                Theme::getThemeColor(Theme::NO_MP_BAR_OUTLINE));
+            mMpBar->setColor(getThemeColor(Theme::NO_MP_BAR),
+                getThemeColor(Theme::NO_MP_BAR_OUTLINE));
         }
     }
     else
@@ -260,8 +184,8 @@ StatusWindow::StatusWindow() :
         mJobLabel = new Label(this, _("Job:"));
         mJobBar = new ProgressBar(this, 0.0F, 80, 0, Theme::PROG_JOB,
             "jobprogressbar.xml", "jobprogressbar_fill.xml");
-        mJobBar->setColor(Theme::getThemeColor(Theme::JOB_BAR),
-            Theme::getThemeColor(Theme::JOB_BAR_OUTLINE));
+        mJobBar->setColor(getThemeColor(Theme::JOB_BAR),
+            getThemeColor(Theme::JOB_BAR_OUTLINE));
 
         place(3, 0, mJobLvlLabel, 3);
         place(5, 2, mJobLabel).setPadding(3);
@@ -288,7 +212,7 @@ StatusWindow::StatusWindow() :
     mDAttrScroll->setVerticalScrollPolicy(ScrollArea::SHOW_AUTO);
     place(6, 3, mDAttrScroll, 5, 3);
 
-    getLayout().setRowHeight(3, Layout::AUTO_SET);
+    getLayout().setRowHeight(3, LayoutType::SET);
 
     place(0, 6, mCharacterPointsLabel, 5);
     place(0, 5, mCopyButton);
@@ -310,157 +234,154 @@ StatusWindow::StatusWindow() :
 
     // TRANSLATORS: status window label
     mMoneyLabel->setCaption(strprintf(_("Money: %s"), Units::formatCurrency(
-        PlayerInfo::getAttribute(PlayerInfo::MONEY)).c_str()));
+        PlayerInfo::getAttribute(Attributes::MONEY)).c_str()));
     mMoneyLabel->adjustSize();
     // TRANSLATORS: status window label
     mCharacterPointsLabel->setCaption(strprintf(_("Character points: %d"),
-        PlayerInfo::getAttribute(PlayerInfo::CHAR_POINTS)));
+        PlayerInfo::getAttribute(Attributes::CHAR_POINTS)));
     mCharacterPointsLabel->adjustSize();
 
-    if (player_node && player_node->isGM())
+    if (localPlayer && localPlayer->isGM())
     {
         // TRANSLATORS: status window label
         mLvlLabel->setCaption(strprintf(_("Level: %d (GM %d)"),
-            PlayerInfo::getAttribute(PlayerInfo::LEVEL),
-            player_node->getGMLevel()));
+            PlayerInfo::getAttribute(Attributes::LEVEL),
+            localPlayer->getGMLevel()));
     }
     else
     {
         // TRANSLATORS: status window label
         mLvlLabel->setCaption(strprintf(_("Level: %d"),
-            PlayerInfo::getAttribute(PlayerInfo::LEVEL)));
+            PlayerInfo::getAttribute(Attributes::LEVEL)));
     }
     mLvlLabel->adjustSize();
 }
 
-void StatusWindow::processEvent(const Channels channel A_UNUSED,
-                                const DepricatedEvent &event)
+void StatusWindow::statChanged(const int id,
+                               const int oldVal1,
+                               const int oldVal2 A_UNUSED)
 {
     static bool blocked = false;
     if (blocked)
         return;
 
-    const DepricatedEvents &eventName = event.getName();
-    if (eventName == EVENT_UPDATEATTRIBUTE)
+    if (id == Net::getPlayerHandler()->getJobLocation())
     {
-        switch (event.getInt("id"))
+        if (mJobLvlLabel)
         {
-            case PlayerInfo::HP:
-            case PlayerInfo::MAX_HP:
-                updateHPBar(mHpBar, true);
-                break;
+            int lvl = PlayerInfo::getStatBase(id);
+            const int oldExp = oldVal1;
+            const std::pair<int, int> exp = PlayerInfo::getStatExperience(id);
 
-            case PlayerInfo::MP:
-            case PlayerInfo::MAX_MP:
-                updateMPBar(mMpBar, true);
-                break;
-
-            case PlayerInfo::EXP:
-            case PlayerInfo::EXP_NEEDED:
-                updateXPBar(mXpBar, false);
-                break;
-
-            case PlayerInfo::MONEY:
-                // TRANSLATORS: status window label
-                mMoneyLabel->setCaption(strprintf(_("Money: %s"),
-                    Units::formatCurrency(event.getInt("newValue")).c_str()));
-                mMoneyLabel->adjustSize();
-                break;
-
-            case PlayerInfo::CHAR_POINTS:
-                mCharacterPointsLabel->setCaption(strprintf(
-                    // TRANSLATORS: status window label
-                    _("Character points: %d"), event.getInt("newValue")));
-
-                mCharacterPointsLabel->adjustSize();
-                // Update all attributes
-                for (Attrs::const_iterator it = mAttrs.begin();
-                     it != mAttrs.end(); ++it)
-                {
-                    if (it->second)
-                        it->second->update();
-                }
-            break;
-
-            case PlayerInfo::CORR_POINTS:
-                mCorrectionPointsLabel->setCaption(strprintf(
-                    // TRANSLATORS: status window label
-                    _("Correction points: %d"), event.getInt("newValue")));
-                mCorrectionPointsLabel->adjustSize();
-                // Update all attributes
-                for (Attrs::const_iterator it = mAttrs.begin();
-                     it != mAttrs.end(); ++it)
-                {
-                    if (it->second)
-                        it->second->update();
-                }
-            break;
-
-            case PlayerInfo::LEVEL:
-                // TRANSLATORS: status window label
-                mLvlLabel->setCaption(strprintf(_("Level: %d"),
-                    event.getInt("newValue")));
-                mLvlLabel->adjustSize();
-            break;
-
-            default:
-                break;
-        }
-    }
-    else if (eventName == EVENT_UPDATESTAT)
-    {
-        const int id = event.getInt("id");
-        if (id == Net::getPlayerHandler()->getJobLocation())
-        {
-            if (mJobLvlLabel)
+            if (!lvl)
             {
-                int lvl = PlayerInfo::getStatBase(id);
-                const int oldExp = event.getInt("oldValue1");
-                const std::pair<int, int> exp
-                    = PlayerInfo::getStatExperience(id);
-
-                if (!lvl)
+                // possible server broken and don't send job level,
+                // then we fixing it :)
+                if (exp.second < 20000)
                 {
-                    // possible server broken and don't send job level,
-                    // then we fixing it :)
-                    if (exp.second < 20000)
-                    {
-                        lvl = 0;
-                    }
-                    else
-                    {
-                        lvl = (exp.second - 20000) / 150;
-                        blocked = true;
-                        PlayerInfo::setStatBase(id, lvl);
-                        blocked = false;
-                    }
+                    lvl = 0;
                 }
-
-                if (exp.first < oldExp && exp.second >= 20000)
-                {   // possible job level up. but server broken and don't send
-                    // new job exp limit, we fixing it
-                    lvl ++;
+                else
+                {
+                    lvl = (exp.second - 20000) / 150;
                     blocked = true;
-                    PlayerInfo::setStatExperience(
-                        id, exp.first, 20000 + lvl * 150);
                     PlayerInfo::setStatBase(id, lvl);
                     blocked = false;
                 }
-
-                // TRANSLATORS: status window label
-                mJobLvlLabel->setCaption(strprintf(_("Job: %d"), lvl));
-                mJobLvlLabel->adjustSize();
-
-                updateProgressBar(mJobBar, id, false);
             }
+
+            if (exp.first < oldExp && exp.second >= 20000)
+            {   // possible job level up. but server broken and don't send
+                // new job exp limit, we fixing it
+                lvl ++;
+                blocked = true;
+                PlayerInfo::setStatExperience(
+                    id, exp.first, 20000 + lvl * 150);
+                PlayerInfo::setStatBase(id, lvl);
+                blocked = false;
+            }
+
+            // TRANSLATORS: status window label
+            mJobLvlLabel->setCaption(strprintf(_("Job: %d"), lvl));
+            mJobLvlLabel->adjustSize();
+
+            updateProgressBar(mJobBar, id, false);
         }
-        else
-        {
+    }
+    else
+    {
+        updateMPBar(mMpBar, true);
+        const Attrs::const_iterator it = mAttrs.find(id);
+        if (it != mAttrs.end() && it->second)
+            it->second->update();
+    }
+}
+
+void StatusWindow::attributeChanged(const int id,
+                                    const int oldVal A_UNUSED,
+                                    const int newVal)
+{
+    switch (id)
+    {
+        case Attributes::HP:
+        case Attributes::MAX_HP:
+            updateHPBar(mHpBar, true);
+            break;
+
+        case Attributes::MP:
+        case Attributes::MAX_MP:
             updateMPBar(mMpBar, true);
-            const Attrs::const_iterator it = mAttrs.find(id);
-            if (it != mAttrs.end() && it->second)
-                it->second->update();
-        }
+            break;
+
+        case Attributes::EXP:
+        case Attributes::EXP_NEEDED:
+            updateXPBar(mXpBar, false);
+            break;
+
+        case Attributes::MONEY:
+            // TRANSLATORS: status window label
+            mMoneyLabel->setCaption(strprintf(_("Money: %s"),
+                Units::formatCurrency(newVal).c_str()));
+            mMoneyLabel->adjustSize();
+            break;
+
+        case Attributes::CHAR_POINTS:
+            mCharacterPointsLabel->setCaption(strprintf(
+                // TRANSLATORS: status window label
+                _("Character points: %d"), newVal));
+
+            mCharacterPointsLabel->adjustSize();
+            // Update all attributes
+            for (Attrs::const_iterator it = mAttrs.begin();
+                 it != mAttrs.end(); ++it)
+            {
+                if (it->second)
+                    it->second->update();
+            }
+            break;
+
+        case Attributes::CORR_POINTS:
+            mCorrectionPointsLabel->setCaption(strprintf(
+                // TRANSLATORS: status window label
+                _("Correction points: %d"), newVal));
+            mCorrectionPointsLabel->adjustSize();
+            // Update all attributes
+            for (Attrs::const_iterator it = mAttrs.begin();
+                 it != mAttrs.end(); ++it)
+            {
+                if (it->second)
+                    it->second->update();
+            }
+            break;
+
+        case Attributes::LEVEL:
+            // TRANSLATORS: status window label
+            mLvlLabel->setCaption(strprintf(_("Level: %d"), newVal));
+            mLvlLabel->adjustSize();
+            break;
+
+        default:
+            break;
     }
 }
 
@@ -511,8 +432,8 @@ void StatusWindow::updateHPBar(ProgressBar *const bar, const bool showMax)
     if (!bar)
         return;
 
-    const int hp = PlayerInfo::getAttribute(PlayerInfo::HP);
-    const int maxHp = PlayerInfo::getAttribute(PlayerInfo::MAX_HP);
+    const int hp = PlayerInfo::getAttribute(Attributes::HP);
+    const int maxHp = PlayerInfo::getAttribute(Attributes::MAX_HP);
     if (showMax)
         bar->setText(toString(hp).append("/").append(toString(maxHp)));
     else
@@ -524,13 +445,14 @@ void StatusWindow::updateHPBar(ProgressBar *const bar, const bool showMax)
     bar->setProgress(prog);
 }
 
-void StatusWindow::updateMPBar(ProgressBar *const bar, const bool showMax)
+void StatusWindow::updateMPBar(ProgressBar *const bar,
+                               const bool showMax) const
 {
     if (!bar)
         return;
 
-    const int mp = PlayerInfo::getAttribute(PlayerInfo::MP);
-    const int maxMp = PlayerInfo::getAttribute(PlayerInfo::MAX_MP);
+    const int mp = PlayerInfo::getAttribute(Attributes::MP);
+    const int maxMp = PlayerInfo::getAttribute(Attributes::MAX_MP);
     if (showMax)
         bar->setText(toString(mp).append("/").append(toString(maxMp)));
     else
@@ -542,14 +464,14 @@ void StatusWindow::updateMPBar(ProgressBar *const bar, const bool showMax)
 
     if (Net::getPlayerHandler()->canUseMagic())
     {
-        bar->setColor(Theme::getThemeColor(Theme::MP_BAR),
-            Theme::getThemeColor(Theme::MP_BAR_OUTLINE));
+        bar->setColor(getThemeColor(Theme::MP_BAR),
+            getThemeColor(Theme::MP_BAR_OUTLINE));
         bar->setProgressPalette(Theme::PROG_MP);
     }
     else
     {
-        bar->setColor(Theme::getThemeColor(Theme::NO_MP_BAR),
-            Theme::getThemeColor(Theme::NO_MP_BAR_OUTLINE));
+        bar->setColor(getThemeColor(Theme::NO_MP_BAR),
+            getThemeColor(Theme::NO_MP_BAR_OUTLINE));
         bar->setProgressPalette(Theme::PROG_NO_MP);
     }
 
@@ -591,8 +513,8 @@ void StatusWindow::updateXPBar(ProgressBar *const bar, const bool percent)
     if (!bar)
         return;
 
-    updateProgressBar(bar, PlayerInfo::getAttribute(PlayerInfo::EXP),
-        PlayerInfo::getAttribute(PlayerInfo::EXP_NEEDED), percent);
+    updateProgressBar(bar, PlayerInfo::getAttribute(Attributes::EXP),
+        PlayerInfo::getAttribute(Attributes::EXP_NEEDED), percent);
 }
 
 void StatusWindow::updateJobBar(ProgressBar *const bar, const bool percent)
@@ -606,7 +528,7 @@ void StatusWindow::updateJobBar(ProgressBar *const bar, const bool percent)
 }
 
 void StatusWindow::updateProgressBar(ProgressBar *const bar, const int id,
-                                     const bool percent) const
+                                     const bool percent)
 {
     const std::pair<int, int> exp =  PlayerInfo::getStatExperience(id);
     updateProgressBar(bar, exp.first, exp.second, percent);
@@ -617,7 +539,7 @@ void StatusWindow::updateWeightBar(ProgressBar *const bar)
     if (!bar)
         return;
 
-    if (PlayerInfo::getAttribute(PlayerInfo::MAX_WEIGHT) == 0)
+    if (PlayerInfo::getAttribute(Attributes::MAX_WEIGHT) == 0)
     {
         // TRANSLATORS: status bar label
         bar->setText(_("Max"));
@@ -626,8 +548,8 @@ void StatusWindow::updateWeightBar(ProgressBar *const bar)
     else
     {
         const int totalWeight = PlayerInfo::getAttribute(
-            PlayerInfo::TOTAL_WEIGHT);
-        const int maxWeight = PlayerInfo::getAttribute(PlayerInfo::MAX_WEIGHT);
+            Attributes::TOTAL_WEIGHT);
+        const int maxWeight = PlayerInfo::getAttribute(Attributes::MAX_WEIGHT);
         float progress = 1.0F;
         if (maxWeight)
         {
@@ -645,8 +567,8 @@ void StatusWindow::updateMoneyBar(ProgressBar *const bar)
     if (!bar)
         return;
 
-    const int money = PlayerInfo::getAttribute(PlayerInfo::MONEY);
-    bar->setText(Units::formatCurrency(money).c_str());
+    const int money = PlayerInfo::getAttribute(Attributes::MONEY);
+    bar->setText(Units::formatCurrency(money));
     if (money > 0)
     {
         const float progress = static_cast<float>(money)
@@ -706,7 +628,7 @@ std::string StatusWindow::translateLetter(const char *const letters)
     return std::string(buf);
 }
 
-std::string StatusWindow::translateLetter2(std::string letters)
+std::string StatusWindow::translateLetter2(const std::string &letters)
 {
     if (letters.size() < 5)
         return "";
@@ -715,38 +637,35 @@ std::string StatusWindow::translateLetter2(std::string letters)
 }
 
 void StatusWindow::updateStatusBar(ProgressBar *const bar,
-                                   const bool percent A_UNUSED)
+                                   const bool percent A_UNUSED) const
 {
-    if (!player_node || !viewport)
-        return;
-
-    bar->setText(translateLetter2(player_node->getInvertDirectionString())
-        .append(translateLetter2(player_node->getCrazyMoveTypeString()))
-        .append(translateLetter2(player_node->getMoveToTargetTypeString()))
-        .append(translateLetter2(player_node->getFollowModeString()))
+    bar->setText(translateLetter2(GameModifiers::getMoveTypeString())
+        .append(translateLetter2(GameModifiers::getCrazyMoveTypeString()))
+        .append(translateLetter2(GameModifiers::getMoveToTargetTypeString()))
+        .append(translateLetter2(GameModifiers::getFollowModeString()))
         .append(" ").append(translateLetter2(
-        player_node->getAttackWeaponTypeString()))
-        .append(translateLetter2(player_node->getAttackTypeString()))
-        .append(translateLetter2(player_node->getMagicAttackString()))
-        .append(translateLetter2(player_node->getPvpAttackString()))
+        GameModifiers::getAttackWeaponTypeString()))
+        .append(translateLetter2(GameModifiers::getAttackTypeString()))
+        .append(translateLetter2(GameModifiers::getMagicAttackTypeString()))
+        .append(translateLetter2(GameModifiers::getPvpAttackTypeString()))
         .append(" ").append(translateLetter2(
-        player_node->getQuickDropCounterString()))
-        .append(translateLetter2(player_node->getPickUpTypeString()))
+        GameModifiers::getQuickDropCounterString()))
+        .append(translateLetter2(GameModifiers::getPickUpTypeString()))
         .append(" ").append(translateLetter2(
-        player_node->getDebugPathString()))
+        GameModifiers::getMapDrawTypeString()))
         .append(" ").append(translateLetter2(
-        player_node->getImitationModeString()))
-        .append(translateLetter2(player_node->getCameraModeString()))
-        .append(translateLetter2(player_node->getAwayModeString())));
+        GameModifiers::getImitationModeString()))
+        .append(translateLetter2(GameModifiers::getCameraModeString()))
+        .append(translateLetter2(GameModifiers::getAwayModeString())));
 
     bar->setProgress(50);
-    if (player_node->getDisableGameModifiers())
-        bar->setBackgroundColor(Theme::getThemeColor(Theme::STATUSBAR_ON));
+    if (settings.disableGameModifiers)
+        bar->setBackgroundColor(getThemeColor(Theme::STATUSBAR_ON));
     else
-        bar->setBackgroundColor(Theme::getThemeColor(Theme::STATUSBAR_OFF));
+        bar->setBackgroundColor(getThemeColor(Theme::STATUSBAR_OFF));
 }
 
-void StatusWindow::action(const gcn::ActionEvent &event)
+void StatusWindow::action(const ActionEvent &event)
 {
     if (!chatWindow)
         return;
@@ -790,8 +709,7 @@ AttrDisplay::AttrDisplay(const Widget2 *const widget,
 
 AttrDisplay::~AttrDisplay()
 {
-    delete mLayout;
-    mLayout = nullptr;
+    delete2(mLayout);
 }
 
 std::string AttrDisplay::update()
@@ -820,7 +738,7 @@ ChangeDisplay::ChangeDisplay(const Widget2 *const widget,
                              const int id, const std::string &restrict name,
                              const std::string &restrict shortName) :
     AttrDisplay(widget, id, name, shortName),
-    gcn::ActionListener(),
+    ActionListener(),
     mNeeded(1),
     // TRANSLATORS: status window label
     mPoints(new Label(this, _("Max"))),
@@ -859,9 +777,9 @@ std::string ChangeDisplay::update()
     }
 
     if (mDec)
-        mDec->setEnabled(PlayerInfo::getAttribute(PlayerInfo::CORR_POINTS));
+        mDec->setEnabled(PlayerInfo::getAttribute(Attributes::CORR_POINTS));
 
-    mInc->setEnabled(PlayerInfo::getAttribute(PlayerInfo::CHAR_POINTS)
+    mInc->setEnabled(PlayerInfo::getAttribute(Attributes::CHAR_POINTS)
         >= mNeeded && mNeeded > 0);
 
     return AttrDisplay::update();
@@ -873,18 +791,18 @@ void ChangeDisplay::setPointsNeeded(const int needed)
     update();
 }
 
-void ChangeDisplay::action(const gcn::ActionEvent &event)
+void ChangeDisplay::action(const ActionEvent &event)
 {
     if (Net::getPlayerHandler()->canCorrectAttributes() &&
         event.getSource() == mDec)
     {
         const int newcorrpoints = PlayerInfo::getAttribute(
-            PlayerInfo::CORR_POINTS);
-        PlayerInfo::setAttribute(PlayerInfo::CORR_POINTS, newcorrpoints - 1);
+            Attributes::CORR_POINTS);
+        PlayerInfo::setAttribute(Attributes::CORR_POINTS, newcorrpoints - 1);
 
         const int newpoints = PlayerInfo::getAttribute(
-            PlayerInfo::CHAR_POINTS) + 1;
-        PlayerInfo::setAttribute(PlayerInfo::CHAR_POINTS, newpoints);
+            Attributes::CHAR_POINTS) + 1;
+        PlayerInfo::setAttribute(Attributes::CHAR_POINTS, newpoints);
 
         const int newbase = PlayerInfo::getStatBase(mId) - 1;
         PlayerInfo::setStatBase(mId, newbase);
@@ -902,8 +820,8 @@ void ChangeDisplay::action(const gcn::ActionEvent &event)
         }
 
         const int newpoints = PlayerInfo::getAttribute(
-            PlayerInfo::CHAR_POINTS) - cnt;
-        PlayerInfo::setAttribute(PlayerInfo::CHAR_POINTS, newpoints);
+            Attributes::CHAR_POINTS) - cnt;
+        PlayerInfo::setAttribute(Attributes::CHAR_POINTS, newpoints);
 
         const int newbase = PlayerInfo::getStatBase(mId) + cnt;
         PlayerInfo::setStatBase(mId, newbase);
