@@ -27,22 +27,27 @@
 #include "item.h"
 #include "units.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 #include "being/playerrelations.h"
 
-#include "gui/sdlfont.h"
+#include "gui/gui.h"
+
+#include "gui/fonts/font.h"
 
 #include "gui/windows/inventorywindow.h"
 #include "gui/windows/itemamountwindow.h"
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 
 #include "gui/widgets/button.h"
+#include "gui/widgets/containerplacer.h"
 #include "gui/widgets/itemcontainer.h"
 #include "gui/widgets/label.h"
+#include "gui/widgets/layout.h"
+#include "gui/widgets/layouttype.h"
 #include "gui/widgets/scrollarea.h"
 #include "gui/widgets/textfield.h"
-#include "gui/widgets/layout.h"
 
 #include "gui/widgets/tabs/chattab.h"
 
@@ -51,10 +56,10 @@
 
 #include "utils/gettext.h"
 
-#include <guichan/font.hpp>
-
 #include "debug.h"
 
+TradeWindow *tradeWindow = nullptr;
+
 // TRANSLATORS: trade window button
 #define CAPTION_PROPOSE _("Propose trade")
 // TRANSLATORS: trade window button
@@ -64,11 +69,11 @@
 // TRANSLATORS: trade window button
 #define CAPTION_ACCEPTED _("Agreed. Waiting...")
 
-TradeWindow::TradeWindow():
+TradeWindow::TradeWindow() :
     // TRANSLATORS: trade window caption
     Window(_("Trade: You"), false, nullptr, "trade.xml"),
-    gcn::ActionListener(),
-    gcn::SelectionListener(),
+    ActionListener(),
+    SelectionListener(),
     mMyInventory(new Inventory(Inventory::TRADE)),
     mPartnerInventory(new Inventory(Inventory::TRADE)),
     mMyItemContainer(new ItemContainer(this, mMyInventory.get())),
@@ -81,13 +86,13 @@ TradeWindow::TradeWindow():
     // TRANSLATORS: trade window money change button
     mMoneyChangeButton(new Button(this, _("Change"), "money", this)),
     mMoneyField(new TextField(this)),
-    mStatus(PROPOSING),
     mAutoAddItem(nullptr),
     mAutoAddToNick(""),
     mGotMoney(0),
     mGotMaxMoney(0),
     mAutoMoney(0),
     mAutoAddAmount(0),
+    mStatus(PROPOSING),
     mOkOther(false),
     mOkMe(false)
 {
@@ -102,7 +107,7 @@ TradeWindow::TradeWindow():
     if (setupWindow)
         setupWindow->registerWindowForReset(this);
 
-    const gcn::Font *const fnt = mOkButton->getFont();
+    const Font *const fnt = mOkButton->getFont();
     int width = std::max(fnt->getWidth(CAPTION_PROPOSE),
         fnt->getWidth(CAPTION_CONFIRMED));
     width = std::max(width, fnt->getWidth(CAPTION_ACCEPT));
@@ -112,13 +117,14 @@ TradeWindow::TradeWindow():
 
     mMyItemContainer->addSelectionListener(this);
 
-    ScrollArea *const myScroll = new ScrollArea(mMyItemContainer,
+    ScrollArea *const myScroll = new ScrollArea(this, mMyItemContainer,
         true, "trade_background.xml");
     myScroll->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
 
     mPartnerItemContainer->addSelectionListener(this);
 
-    ScrollArea *const partnerScroll = new ScrollArea(mPartnerItemContainer,
+    ScrollArea *const partnerScroll = new ScrollArea(this,
+        mPartnerItemContainer,
         true, "trade_background.xml");
     partnerScroll->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
 
@@ -130,8 +136,7 @@ TradeWindow::TradeWindow():
     place(1, 0, mMoneyLabel);
     place(0, 1, myScroll).setPadding(3);
     place(1, 1, partnerScroll).setPadding(3);
-    ContainerPlacer placer;
-    placer = getPlacer(0, 0);
+    ContainerPlacer placer = getPlacer(0, 0);
     placer(0, 0, moneyLabel2);
     placer(1, 0, mMoneyField, 2);
     placer(3, 0, mMoneyChangeButton).setHAlign(LayoutCell::LEFT);
@@ -140,10 +145,10 @@ TradeWindow::TradeWindow():
     placer(1, 0, mOkButton);
     Layout &layout = getLayout();
     layout.extend(0, 2, 2, 1);
-    layout.setRowHeight(1, Layout::AUTO_SET);
+    layout.setRowHeight(1, LayoutType::SET);
     layout.setRowHeight(2, 0);
-    layout.setColWidth(0, Layout::AUTO_SET);
-    layout.setColWidth(1, Layout::AUTO_SET);
+    layout.setColWidth(0, LayoutType::SET);
+    layout.setColWidth(1, LayoutType::SET);
 
     loadWindowState();
     enableVisibleSound(true);
@@ -186,8 +191,11 @@ void TradeWindow::setMoney(const int amount)
     mMoneyLabel->adjustSize();
 }
 
-void TradeWindow::addItem(const int id, const bool own, const int quantity,
-                          const int refine, const unsigned char color) const
+void TradeWindow::addItem(const int id,
+                          const bool own,
+                          const int quantity,
+                          const uint8_t refine,
+                          const unsigned char color) const
 {
     if (own)
         mMyInventory->addItem(id, quantity, refine, color);
@@ -196,7 +204,7 @@ void TradeWindow::addItem(const int id, const bool own, const int quantity,
 }
 
 void TradeWindow::addItem2(const int id, const bool own, const int quantity,
-                           const int refine, const unsigned char color,
+                           const uint8_t refine, const unsigned char color,
                            const bool equipment) const
 {
     if (own)
@@ -268,7 +276,7 @@ void TradeWindow::tradeItem(const Item *const item, const int quantity,
     Net::getTradeHandler()->addItem(item, quantity);
 }
 
-void TradeWindow::valueChanged(const gcn::SelectionEvent &event)
+void TradeWindow::valueChanged(const SelectionEvent &event)
 {
     if (!mMyItemContainer || !mPartnerItemContainer)
         return;
@@ -318,7 +326,7 @@ void TradeWindow::setStatus(const Status s)
     mOkButton->setEnabled((s != PROPOSING && s != ACCEPTED));
 }
 
-void TradeWindow::action(const gcn::ActionEvent &event)
+void TradeWindow::action(const ActionEvent &event)
 {
     if (!inventoryWindow)
         return;
@@ -380,14 +388,14 @@ void TradeWindow::action(const gcn::ActionEvent &event)
             return;
 
         int v = atoi(mMoneyField->getText().c_str());
-        const int curMoney = PlayerInfo::getAttribute(PlayerInfo::MONEY);
+        const int curMoney = PlayerInfo::getAttribute(Attributes::MONEY);
         if (v > curMoney)
         {
             if (localChatTab)
             {
                 // TRANSLATORS: trade error
                 localChatTab->chatLog(_("You don't have enough money."),
-                                      BY_SERVER);
+                                      ChatMsgType::BY_SERVER);
             }
             v = curMoney;
         }
@@ -431,7 +439,7 @@ void TradeWindow::addAutoMoney(const std::string &nick, const int money)
 
 void TradeWindow::initTrade(const std::string &nick)
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     if (!mAutoAddToNick.empty() && mAutoAddToNick == nick)
@@ -441,7 +449,7 @@ void TradeWindow::initTrade(const std::string &nick)
             const Inventory *const inv = PlayerInfo::getInventory();
             if (inv)
             {
-                Item *const item = inv->findItem(mAutoAddItem->getId(),
+                const Item *const item = inv->findItem(mAutoAddItem->getId(),
                     mAutoAddItem->getColor());
                 if (item)
                     tradeItem(item, mAutoAddAmount);
@@ -473,7 +481,8 @@ bool TradeWindow::checkItem(const Item *const item) const
         {
             // TRANSLATORS: trade error
             localChatTab->chatLog(_("Failed adding item. You can not "
-                "overlap one kind of item on the window."), BY_SERVER);
+                "overlap one kind of item on the window."),
+                ChatMsgType::BY_SERVER);
         }
         return false;
     }
