@@ -25,8 +25,8 @@
 
 #include "gui/widgets/window.h"
 
-#include <guichan/actionlistener.hpp>
-#include <guichan/selectionlistener.hpp>
+#include "listeners/actionlistener.h"
+#include "listeners/selectionlistener.h"
 
 #include <memory>
 
@@ -43,8 +43,8 @@ class TextField;
  * \ingroup Interface
  */
 class TradeWindow final : public Window,
-                          private gcn::ActionListener,
-                          private gcn::SelectionListener
+                          private ActionListener,
+                          private SelectionListener
 {
     public:
         /**
@@ -68,7 +68,7 @@ class TradeWindow final : public Window,
          * Add an item to the trade window.
          */
         void addItem(const int id, const bool own, const int quantity,
-                     const int refine, const unsigned char color) const;
+                     const uint8_t refine, const unsigned char color) const;
 
         /**
          * Reset both item containers
@@ -79,7 +79,7 @@ class TradeWindow final : public Window,
          * Add an item to the trade window.
          */
         void addItem2(const int id, const bool own, const int quantity,
-                      const int refine, const unsigned char color,
+                      const uint8_t refine, const unsigned char color,
                       const bool equipment) const;
 
         /**
@@ -109,12 +109,12 @@ class TradeWindow final : public Window,
          * Updates the labels and makes sure only one item is selected in
          * either my inventory or partner inventory.
          */
-        void valueChanged(const gcn::SelectionEvent &event) override final;
+        void valueChanged(const SelectionEvent &event) override final;
 
         /**
          * Called when receiving actions from the widgets.
          */
-        void action(const gcn::ActionEvent &event) override final;
+        void action(const ActionEvent &event) override final;
 
         /**
          * Closes the Trade Window, as well as telling the server that the
@@ -171,13 +171,13 @@ class TradeWindow final : public Window,
         Button  *mMoneyChangeButton;
         TextField *mMoneyField;
 
-        Status mStatus;
         Item* mAutoAddItem;
         std::string mAutoAddToNick;
         int mGotMoney;
         int mGotMaxMoney;
         int mAutoMoney;
         int mAutoAddAmount;
+        Status mStatus;
         bool mOkOther;
         bool mOkMe;
 };
