@@ -25,16 +25,18 @@
 #include "client.h"
 
 #include "gui/windows/okdialog.h"
-#include "gui/windows/registerdialog.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/passwordfield.h"
 
+#include "listeners/wrongdatanoticelistener.h"
+
 #include "net/logindata.h"
 #include "net/loginhandler.h"
 #include "net/net.h"
 
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include <string>
@@ -45,7 +47,7 @@
 UnRegisterDialog::UnRegisterDialog(LoginData *const data) :
     // TRANSLATORS: unregister dialog name
     Window(_("Unregister"), true, nullptr, "unregister.xml"),
-    gcn::ActionListener(),
+    ActionListener(),
     mLoginData(data),
     mPasswordField(new PasswordField(this, mLoginData->password)),
     // TRANSLATORS: unregister dialog. button.
@@ -97,11 +99,10 @@ void UnRegisterDialog::postInit()
 
 UnRegisterDialog::~UnRegisterDialog()
 {
-    delete mWrongDataNoticeListener;
-    mWrongDataNoticeListener = nullptr;
+    delete2(mWrongDataNoticeListener);
 }
 
-void UnRegisterDialog::action(const gcn::ActionEvent &event)
+void UnRegisterDialog::action(const ActionEvent &event)
 {
     const std::string &eventId = event.getId();
     if (eventId == "cancel")
@@ -110,8 +111,8 @@ void UnRegisterDialog::action(const gcn::ActionEvent &event)
     }
     else if (eventId == "unregister")
     {
-        const std::string username = mLoginData->username.c_str();
-        const std::string password = mPasswordField->getText();
+        const std::string username = mLoginData->username;
+        const std::string &password = mPasswordField->getText();
         logger->log("UnregisterDialog::unregistered, Username is %s",
                      username.c_str());
 
@@ -144,7 +145,7 @@ void UnRegisterDialog::action(const gcn::ActionEvent &event)
 
             // TRANSLATORS: unregister dialog. error message.
             OkDialog *const dlg = new OkDialog(_("Error"),
-                errorMsg.str(), DIALOG_ERROR);
+                errorMsg.str(), DialogType::ERROR);
             dlg->addActionListener(mWrongDataNoticeListener);
         }
         else
