@@ -22,28 +22,31 @@
 
 #include "gui/windows/whoisonline.h"
 
+#include "actormanager.h"
+#include "client.h"
+#include "configuration.h"
 #include "guild.h"
 #include "party.h"
+#include "settings.h"
 
-#include "gui/viewport.h"
+#include "gui/onlineplayer.h"
+#include "gui/popupmanager.h"
 
 #include "gui/windows/chatwindow.h"
+#include "gui/windows/setupwindow.h"
 #include "gui/windows/socialwindow.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/browserbox.h"
 #include "gui/widgets/scrollarea.h"
 
-#include "actormanager.h"
-#include "client.h"
-#include "configuration.h"
-#include "main.h"
-
+#include "being/beingflag.h"
 #include "being/localplayer.h"
 #include "being/playerrelations.h"
 
 #include "net/download.h"
 #include "net/net.h"
+#include "net/packetlimiter.h"
 #include "net/playerhandler.h"
 
 #include "utils/gettext.h"
@@ -66,33 +69,42 @@
 #undef malloc
 #endif
 
-class NameFunctuator final
+WhoIsOnline *whoIsOnline = nullptr;
+
+namespace
 {
-    public:
-        bool operator()(const OnlinePlayer *left,
-                        const OnlinePlayer *right) const
-        {
-            return (compareStrI(left->getNick(), right->getNick()) < 0);
-        }
-} nameCompare;
+    class NameFunctuator final
+    {
+        public:
+            bool operator()(const OnlinePlayer *left,
+                            const OnlinePlayer *right) const
+            {
+                return (compareStrI(left->getNick(), right->getNick()) < 0);
+            }
+    } nameCompare;
+}  // namespace
 
 WhoIsOnline::WhoIsOnline() :
     // TRANSLATORS: who is online window name
     Window(_("Who Is Online - Updating"), false, nullptr, "whoisonline.xml"),
+    mUpdateTimer(0),
     mThread(nullptr),
-    mDownloadStatus(UPDATE_LIST),
-    mDownloadComplete(true),
-    mDownloadedBytes(0),
     mMemoryBuffer(nullptr),
     mCurlError(new char[CURL_ERROR_SIZE]),
     mBrowserBox(new BrowserBox(this, BrowserBox::AUTO_SIZE, true,
         "onlinebrowserbox.xml")),
-    mScrollArea(new ScrollArea(mBrowserBox, false)),
-    mUpdateTimer(0),
+    mScrollArea(new ScrollArea(this, mBrowserBox, false)),
     mOnlinePlayers(),
     mOnlineNicks(),
     // TRANSLATORS: who is online. button.
     mUpdateButton(new Button(this, _("Update"), "update", this)),
+    mFriends(),
+    mNeutral(),
+    mDisregard(),
+    mEnemy(),
+    mDownloadedBytes(0),
+    mDownloadStatus(UPDATE_LIST),
+    mDownloadComplete(true),
     mAllowUpdate(true),
     mShowLevel(false),
     mUpdateOnlineList(config.getBoolValue("updateOnlineList")),
@@ -115,11 +127,14 @@ void WhoIsOnline::postInit()
     setStickyButtonLock(true);
     setSaveVisible(true);
 
+    if (setupWindow)
+        setupWindow->registerWindowForReset(this);
+
     mUpdateButton->setEnabled(false);
-    mUpdateButton->setDimension(gcn::Rectangle(5, 5, w - 10, 20 + 5));
+    mUpdateButton->setDimension(Rect(5, 5, w - 10, 20 + 5));
 
     mBrowserBox->setOpaque(false);
-    mScrollArea->setDimension(gcn::Rectangle(5, 20 + 10, w - 10, h - 10 - 30));
+    mScrollArea->setDimension(Rect(5, 20 + 10, w - 10, h - 10 - 30));
     mScrollArea->setSize(w - 10, h - 10 - 30);
     mBrowserBox->setLinkHandler(this);
 
@@ -133,7 +148,7 @@ void WhoIsOnline::postInit()
 
     download();
 
-    widgetResized(gcn::Event(nullptr));
+    widgetResized(Event(nullptr));
     config.addListener("updateOnlineList", this);
     config.addListener("groupFriends", this);
     mGroupFriends = config.getBoolValue("groupFriends");
@@ -159,9 +174,9 @@ WhoIsOnline::~WhoIsOnline()
     mOnlineNicks.clear();
 }
 
-void WhoIsOnline::handleLink(const std::string& link, gcn::MouseEvent *event)
+void WhoIsOnline::handleLink(const std::string& link, MouseEvent *event)
 {
-    if (!event || event->getButton() == gcn::MouseEvent::LEFT)
+    if (!event || event->getButton() == MouseButton::LEFT)
     {
         if (chatWindow)
         {
@@ -177,46 +192,42 @@ void WhoIsOnline::handleLink(const std::string& link, gcn::MouseEvent *event)
             }
         }
     }
-    else if (event->getButton() == gcn::MouseEvent::RIGHT)
+    else if (event->getButton() == MouseButton::RIGHT)
     {
-        if (player_node && link == player_node->getName())
+        if (localPlayer && link == localPlayer->getName())
             return;
 
-        if (viewport)
+        if (popupManager)
         {
             if (actorManager)
             {
                 const std::string text = decodeLinkText(link);
                 Being *const being = actorManager->findBeingByName(
-                    text, Being::PLAYER);
+                    text, ActorType::PLAYER);
 
-                if (being && viewport)
+                if (being && popupManager)
                 {
-                    viewport->showPopup(being);
+                    popupManager->showPopup(being);
                     return;
                 }
             }
-            viewport->showPlayerPopup(link);
+            popupManager->showPlayerPopup(link);
         }
     }
 }
 
-void WhoIsOnline::updateWindow(std::vector<OnlinePlayer*> &restrict friends,
-                               std::vector<OnlinePlayer*> &restrict neutral,
-                               std::vector<OnlinePlayer*> &restrict disregard,
-                               std::vector<OnlinePlayer*> &restrict enemy,
-                               size_t numOnline)
+void WhoIsOnline::updateWindow(size_t numOnline)
 {
     // Set window caption
     // TRANSLATORS: who is online window name
     setCaption(_("Who Is Online - ") + toString(numOnline));
 
     // List the online people
-    std::sort(friends.begin(), friends.end(), nameCompare);
-    std::sort(neutral.begin(), neutral.end(), nameCompare);
-    std::sort(disregard.begin(), disregard.end(), nameCompare);
+    std::sort(mFriends.begin(), mFriends.end(), nameCompare);
+    std::sort(mNeutral.begin(), mNeutral.end(), nameCompare);
+    std::sort(mDisregard.begin(), mDisregard.end(), nameCompare);
     bool addedFromSection(false);
-    FOR_EACH (std::vector<OnlinePlayer*>::const_iterator, it, friends)
+    FOR_EACH (std::vector<OnlinePlayer*>::const_iterator, it, mFriends)
     {
         mBrowserBox->addRow((*it)->getText());
         addedFromSection = true;
@@ -226,7 +237,7 @@ void WhoIsOnline::updateWindow(std::vector<OnlinePlayer*> &restrict friends,
         mBrowserBox->addRow("---");
         addedFromSection = false;
     }
-    FOR_EACH (std::vector<OnlinePlayer*>::const_iterator, it, enemy)
+    FOR_EACH (std::vector<OnlinePlayer*>::const_iterator, it, mEnemy)
     {
         mBrowserBox->addRow((*it)->getText());
         addedFromSection = true;
@@ -236,15 +247,15 @@ void WhoIsOnline::updateWindow(std::vector<OnlinePlayer*> &restrict friends,
         mBrowserBox->addRow("---");
         addedFromSection = false;
     }
-    FOR_EACH (std::vector<OnlinePlayer*>::const_iterator, it, neutral)
+    FOR_EACH (std::vector<OnlinePlayer*>::const_iterator, it, mNeutral)
     {
         mBrowserBox->addRow((*it)->getText());
         addedFromSection = true;
     }
-    if (addedFromSection == true && !disregard.empty())
+    if (addedFromSection == true && !mDisregard.empty())
         mBrowserBox->addRow("---");
 
-    FOR_EACH (std::vector<OnlinePlayer*>::const_iterator, it, disregard)
+    FOR_EACH (std::vector<OnlinePlayer*>::const_iterator, it, mDisregard)
         mBrowserBox->addRow((*it)->getText());
 
     if (mScrollArea->getVerticalMaxScroll() <
@@ -255,14 +266,47 @@ void WhoIsOnline::updateWindow(std::vector<OnlinePlayer*> &restrict friends,
     }
 }
 
-void WhoIsOnline::loadList(std::vector<OnlinePlayer*> &list)
+void WhoIsOnline::handlerPlayerRelation(const std::string &nick,
+                                        OnlinePlayer *const player)
+{
+    switch (player_relations.getRelation(nick))
+    {
+        case PlayerRelation::NEUTRAL:
+        default:
+            setNeutralColor(player);
+            mNeutral.push_back(player);
+            break;
+
+        case PlayerRelation::FRIEND:
+            player->setText("2");
+            if (mGroupFriends)
+                mFriends.push_back(player);
+            else
+                mNeutral.push_back(player);
+            break;
+
+        case PlayerRelation::DISREGARDED:
+        case PlayerRelation::BLACKLISTED:
+            player->setText("8");
+            mDisregard.push_back(player);
+            break;
+
+        case PlayerRelation::ENEMY2:
+            player->setText("1");
+            mEnemy.push_back(player);
+            break;
+
+        case PlayerRelation::IGNORED:
+        case PlayerRelation::ERASED:
+            // Ignore the ignored.
+            break;
+    }
+}
+
+void WhoIsOnline::loadList(const std::vector<OnlinePlayer*> &list)
 {
     mBrowserBox->clearRows();
     const size_t numOnline = list.size();
-    std::vector<OnlinePlayer*> friends;
-    std::vector<OnlinePlayer*> neutral;
-    std::vector<OnlinePlayer*> disregard;
-    std::vector<OnlinePlayer*> enemy;
 
     FOR_EACH (std::set<OnlinePlayer*>::iterator, itd, mOnlinePlayers)
         delete *itd;
@@ -281,41 +325,10 @@ void WhoIsOnline::loadList(std::vector<OnlinePlayer*> &list)
         if (!mShowLevel)
             player->setLevel(0);
 
-        switch (player_relations.getRelation(nick))
-        {
-            case PlayerRelation::NEUTRAL:
-            default:
-                setNeutralColor(player);
-                neutral.push_back(player);
-                break;
-
-            case PlayerRelation::FRIEND:
-                player->setText("2");
-                if (mGroupFriends)
-                    friends.push_back(player);
-                else
-                    neutral.push_back(player);
-                break;
-
-            case PlayerRelation::DISREGARDED:
-            case PlayerRelation::BLACKLISTED:
-                player->setText("8");
-                disregard.push_back(player);
-                break;
-
-            case PlayerRelation::ENEMY2:
-                player->setText("1");
-                enemy.push_back(player);
-                break;
-
-            case PlayerRelation::IGNORED:
-            case PlayerRelation::ERASED:
-                // Ignore the ignored.
-                break;
-        }
+        handlerPlayerRelation(nick, player);
     }
 
-    updateWindow(friends, neutral, disregard, enemy, numOnline);
+    updateWindow(numOnline);
     if (!mOnlineNicks.empty())
     {
         if (chatWindow)
@@ -324,6 +337,10 @@ void WhoIsOnline::loadList(std::vector<OnlinePlayer*> &list)
             socialWindow->updateActiveList();
     }
     updateSize();
+    mFriends.clear();
+    mNeutral.clear();
+    mDisregard.clear();
+    mEnemy.clear();
 }
 
 void WhoIsOnline::loadWebList()
@@ -343,10 +360,6 @@ void WhoIsOnline::loadWebList()
     bool listStarted(false);
     std::string lineStr;
     int numOnline(0);
-    std::vector<OnlinePlayer*> friends;
-    std::vector<OnlinePlayer*> neutral;
-    std::vector<OnlinePlayer*> disregard;
-    std::vector<OnlinePlayer*> enemy;
 
     // Tokenize and add each line separately
     char *line = strtok(mMemoryBuffer, "\n");
@@ -412,7 +425,7 @@ void WhoIsOnline::loadWebList()
                 if (actorManager)
                 {
                     Being *const being = actorManager->findBeingByName(
-                        nick, Being::PLAYER);
+                        nick, ActorType::PLAYER);
                     if (being)
                     {
                         if (level > 0)
@@ -432,8 +445,8 @@ void WhoIsOnline::loadWebList()
                     level = 0;
 
                 OnlinePlayer *const player = new OnlinePlayer(nick,
-                    static_cast<signed char>(255), level,
-                    GENDER_UNSPECIFIED, -1);
+                    static_cast<unsigned char>(255), level,
+                    Gender::UNSPECIFIED, -1);
                 mOnlinePlayers.insert(player);
                 mOnlineNicks.insert(nick);
 
@@ -441,38 +454,7 @@ void WhoIsOnline::loadWebList()
                     player->setIsGM(true);
 
                 numOnline++;
-                switch (player_relations.getRelation(nick))
-                {
-                    case PlayerRelation::NEUTRAL:
-                    default:
-                        setNeutralColor(player);
-                        neutral.push_back(player);
-                        break;
-
-                    case PlayerRelation::FRIEND:
-                        player->setText("2");
-                        if (mGroupFriends)
-                            friends.push_back(player);
-                        else
-                            neutral.push_back(player);
-                        break;
-
-                    case PlayerRelation::DISREGARDED:
-                    case PlayerRelation::BLACKLISTED:
-                        player->setText("8");
-                        disregard.push_back(player);
-                        break;
-
-                    case PlayerRelation::ENEMY2:
-                        player->setText("1");
-                        enemy.push_back(player);
-                        break;
-
-                    case PlayerRelation::IGNORED:
-                    case PlayerRelation::ERASED:
-                        // Ignore the ignored.
-                        break;
-                }
+                handlerPlayerRelation(nick, player);
             }
         }
         else if (lineStr.find("------------------------------")
@@ -483,11 +465,15 @@ void WhoIsOnline::loadWebList()
         line = strtok(nullptr, "\n");
     }
 
-    updateWindow(friends, neutral, disregard, enemy, numOnline);
+    updateWindow(numOnline);
 
     // Free the memory buffer now that we don't need it anymore
     free(mMemoryBuffer);
     mMemoryBuffer = nullptr;
+    mFriends.clear();
+    mNeutral.clear();
+    mDisregard.clear();
+    mEnemy.clear();
 }
 
 size_t WhoIsOnline::memoryWrite(void *ptr, size_t size,
@@ -499,7 +485,7 @@ size_t WhoIsOnline::memoryWrite(void *ptr, size_t size,
     WhoIsOnline *const wio = reinterpret_cast<WhoIsOnline *>(stream);
     const size_t totalMem = size * nmemb;
     wio->mMemoryBuffer = static_cast<char*>(realloc(wio->mMemoryBuffer,
-        wio->mDownloadedBytes + totalMem));
+        static_cast<size_t>(wio->mDownloadedBytes) + totalMem));
     if (wio->mMemoryBuffer)
     {
         memcpy(&(wio->mMemoryBuffer[wio->mDownloadedBytes]), ptr, totalMem);
@@ -514,7 +500,7 @@ int WhoIsOnline::downloadThread(void *ptr)
     int attempts = 0;
     WhoIsOnline *const wio = reinterpret_cast<WhoIsOnline *>(ptr);
     CURLcode res;
-    const std::string url(client->getOnlineUrl() + "/online.txt");
+    const std::string url(settings.onlineListUrl + "/online.txt");
 
     while (attempts < 1 && !wio->mDownloadComplete)
     {
@@ -544,6 +530,7 @@ int WhoIsOnline::downloadThread(void *ptr)
             curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
             curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 7);
             curl_easy_setopt(curl, CURLOPT_TIMEOUT, 30);
+            Net::Download::addHeaders(curl);
             Net::Download::addProxy(curl);
             Net::Download::secureCurl(curl);
 
@@ -608,7 +595,7 @@ void WhoIsOnline::download()
     }
     else
     {
-        if (client->limitPackets(PACKET_ONLINELIST))
+        if (PacketLimiter::limitPackets(PACKET_ONLINELIST))
             Net::getPlayerHandler()->requestOnlineList();
     }
 }
@@ -683,7 +670,7 @@ void WhoIsOnline::slowLogic()
     BLOCK_END("WhoIsOnline::slowLogic")
 }
 
-void WhoIsOnline::action(const gcn::ActionEvent &event)
+void WhoIsOnline::action(const ActionEvent &event)
 {
     if (event.getId() == "update")
     {
@@ -706,7 +693,7 @@ void WhoIsOnline::action(const gcn::ActionEvent &event)
         }
         else
         {
-            if (client->limitPackets(PACKET_ONLINELIST))
+            if (PacketLimiter::limitPackets(PACKET_ONLINELIST))
             {
                 mUpdateTimer = cur_time;
                 Net::getPlayerHandler()->requestOnlineList();
@@ -715,7 +702,7 @@ void WhoIsOnline::action(const gcn::ActionEvent &event)
     }
 }
 
-void WhoIsOnline::widgetResized(const gcn::Event &event)
+void WhoIsOnline::widgetResized(const Event &event)
 {
     Window::widgetResized(event);
     updateSize();
@@ -723,7 +710,7 @@ void WhoIsOnline::widgetResized(const gcn::Event &event)
 
 void WhoIsOnline::updateSize()
 {
-    const gcn::Rectangle area = getChildrenArea();
+    const Rect area = getChildrenArea();
     if (mUpdateButton)
         mUpdateButton->setWidth(area.width - 10);
 
@@ -764,17 +751,17 @@ void WhoIsOnline::setNeutralColor(OnlinePlayer *const player)
     if (!player)
         return;
 
-    if (actorManager && player_node)
+    if (actorManager && localPlayer)
     {
         const std::string &nick = player->getNick();
-        if (nick == player_node->getName())
+        if (nick == localPlayer->getName())
         {
             player->setText("s");
             return;
         }
-        if (player_node->isInParty())
+        if (localPlayer->isInParty())
         {
-            const Party *const party = player_node->getParty();
+            const Party *const party = localPlayer->getParty();
             if (party)
             {
                 if (party->getMember(nick))
@@ -788,7 +775,7 @@ void WhoIsOnline::setNeutralColor(OnlinePlayer *const player)
         const Being *const being = actorManager->findBeingByName(nick);
         if (being)
         {
-            const Guild *const guild2 = player_node->getGuild();
+            const Guild *const guild2 = localPlayer->getGuild();
             if (guild2)
             {
                 const Guild *const guild1 = being->getGuild();
@@ -832,7 +819,7 @@ void OnlinePlayer::setText(std::string color)
     if (mStatus != 255 && actorManager)
     {
         Being *const being = actorManager->findBeingByName(
-            mNick, Being::PLAYER);
+            mNick, ActorType::PLAYER);
         if (being)
         {
             being->setState(mStatus);
@@ -842,33 +829,33 @@ void OnlinePlayer::setText(std::string color)
         }
     }
 
-    if ((mStatus != 255 && mStatus & Being::FLAG_GM) || mIsGM)
+    if ((mStatus != 255 && mStatus & BeingFlag::GM) || mIsGM)
         mText.append("(GM) ");
 
     if (mLevel > 0)
         mText.append(strprintf("%d", mLevel));
 
-    if (mGender == GENDER_FEMALE)
+    if (mGender == Gender::FEMALE)
         mText.append("\u2640");
-    else if (mGender == GENDER_MALE)
+    else if (mGender == Gender::MALE)
         mText.append("\u2642");
 
     if (mStatus > 0 && mStatus != 255)
     {
-        if (mStatus & Being::FLAG_SHOP)
+        if (mStatus & BeingFlag::SHOP)
             mText.append("$");
-        if (mStatus & Being::FLAG_AWAY)
+        if (mStatus & BeingFlag::AWAY)
         {
             // TRANSLATORS: this away status writed in player nick
             mText.append(_("A"));
         }
-        if (mStatus & Being::FLAG_INACTIVE)
+        if (mStatus & BeingFlag::INACTIVE)
         {
             // TRANSLATORS: this inactive status writed in player nick
             mText.append(_("I"));
         }
 
-        if (mStatus & Being::FLAG_GM && color == "0")
+        if (mStatus & BeingFlag::GM && color == "0")
             color = "2";
     }
     else if (mIsGM && color == "0")
