@@ -22,13 +22,14 @@
 
 #include "configuration.h"
 #include "game.h"
+#include "settings.h"
 #include "touchmanager.h"
 
+#include "input/inputactionmap.h"
+#include "input/inputactionsortfunctor.h"
 #include "input/joystick.h"
 #include "input/keyboardconfig.h"
-#include "input/keyboarddata.h"
 #include "being/localplayer.h"
-#include "being/playerinfo.h"
 #ifdef USE_SDL2
 #include "input/multitouchmanager.h"
 #endif
@@ -44,15 +45,13 @@
 #include "gui/windows/npcdialog.h"
 #include "gui/windows/npcpostdialog.h"
 #include "gui/windows/selldialog.h"
-#include "gui/windows/setup.h"
+#include "gui/windows/setupwindow.h"
 #include "gui/windows/textdialog.h"
 #include "gui/windows/tradewindow.h"
-#include "gui/windows/quitdialog.h"
 
 #include "utils/timer.h"
 
-#include <guichan/exception.hpp>
-#include <guichan/focushandler.hpp>
+#include "gui/focushandler.h"
 
 #include <algorithm>
 
@@ -60,23 +59,18 @@
 
 InputManager inputManager;
 
+class QuitDialog;
+
 extern QuitDialog *quitDialog;
 
-static class KeyFunctor final
+namespace
 {
-    public:
-        bool operator() (const int key1, const int key2) const
-        {
-            return keys[key1].priority >= keys[key2].priority;
-        }
-
-        const KeyData *keys;
-} keyDataSorter;
-
+    InputActionSortFunctor inputActionDataSorter;
+}  // namespace
 
 InputManager::InputManager() :
     mSetupInput(nullptr),
-    mNewKeyIndex(Input::KEY_NO_VALUE),
+    mNewKeyIndex(InputAction::NO_VALUE),
     mMask(1),
     mNameMap()
 {
@@ -84,25 +78,25 @@ InputManager::InputManager() :
 
 void InputManager::init()
 {
-    for (unsigned int i = 0; i < Input::KEY_TOTAL; i++)
+    for (unsigned int i = 0; i < InputAction::TOTAL; i++)
     {
-        KeyFunction &kf = mKey[i];
-        for (unsigned int f = 0; f < KeyFunctionSize; f ++)
+        InputFunction &kf = mKey[i];
+        for (unsigned int f = 0; f < inputFunctionSize; f ++)
         {
-            KeyItem &ki = kf.values[f];
-            ki.type = INPUT_UNKNOWN;
+            InputItem &ki = kf.values[f];
+            ki.type = InputType::UNKNOWN;
             ki.value = -1;
         }
     }
 
-    mNewKeyIndex = Input::KEY_NO_VALUE;
+    mNewKeyIndex = InputAction::NO_VALUE;
 
     resetKeys();
     retrieve();
     update();
 }
 
-void InputManager::update() const
+void InputManager::update()
 {
     keyboard.update();
     if (joystick)
@@ -111,19 +105,20 @@ void InputManager::update() const
 
 void InputManager::retrieve()
 {
-    for (int i = 0; i < Input::KEY_TOTAL; i++)
+    for (int i = 0; i < InputAction::TOTAL; i++)
     {
 #ifdef USE_SDL2
-        const std::string cf = std::string("sdl2") + keyData[i].configField;
+        const std::string cf = std::string("sdl2")
+            + inputActionData[i].configField;
 #else
-        const std::string cf = keyData[i].configField;
+        const std::string cf = inputActionData[i].configField;
 #endif
         if (!cf.empty())
         {
             mNameMap[cf] = i;
-            KeyFunction &kf = mKey[i];
+            InputFunction &kf = mKey[i];
             const std::string keyStr = config.getValue(cf, "");
-            const int keyStrSize = keyStr.size();
+            const size_t keyStrSize = keyStr.size();
             if (keyStr.empty())
                 continue;
 
@@ -131,22 +126,22 @@ void InputManager::retrieve()
             splitToStringVector(keys, keyStr, ',');
             unsigned int i2 = 0;
             for (StringVectCIter it = keys.begin(), it_end = keys.end();
-                 it != it_end && i2 < KeyFunctionSize; ++ it)
+                 it != it_end && i2 < inputFunctionSize; ++ it)
             {
                 std::string keyStr2 = *it;
                 if (keyStrSize < 2)
                     continue;
-                int type = INPUT_KEYBOARD;
+                int type = InputType::KEYBOARD;
                 if ((keyStr2[0] < '0' || keyStr2[0] > '9')
                     && keyStr2[0] != '-')
                 {
                     switch (keyStr2[0])
                     {
                         case 'm':
-                            type = INPUT_MOUSE;
+                            type = InputType::MOUSE;
                             break;
                         case 'j':
-                            type = INPUT_JOYSTICK;
+                            type = InputType::JOYSTICK;
                             break;
                         default:
                             break;
@@ -156,7 +151,7 @@ void InputManager::retrieve()
                 const int key = atoi(keyStr2.c_str());
                 if (key >= -255 && key < SDLK_LAST)
                 {
-                    kf.values[i2] = KeyItem(type, key);
+                    kf.values[i2] = InputItem(type, key);
                     i2 ++;
                 }
             }
@@ -166,30 +161,31 @@ void InputManager::retrieve()
 
 void InputManager::store() const
 {
-    for (int i = 0; i < Input::KEY_TOTAL; i++)
+    for (int i = 0; i < InputAction::TOTAL; i++)
     {
 #ifdef USE_SDL2
-        const std::string cf = std::string("sdl2") + keyData[i].configField;
+        const std::string cf = std::string("sdl2")
+            + inputActionData[i].configField;
 #else
-        const std::string cf = keyData[i].configField;
+        const std::string cf = inputActionData[i].configField;
 #endif
         if (!cf.empty())
         {
             std::string keyStr;
-            const KeyFunction &kf = mKey[i];
+            const InputFunction &kf = mKey[i];
 
-            for (size_t i2 = 0; i2 < KeyFunctionSize; i2 ++)
+            for (size_t i2 = 0; i2 < inputFunctionSize; i2 ++)
             {
-                const KeyItem &key = kf.values[i2];
-                if (key.type != INPUT_UNKNOWN)
+                const InputItem &key = kf.values[i2];
+                if (key.type != InputType::UNKNOWN)
                 {
                     std::string tmp("k");
                     switch (key.type)
                     {
-                        case INPUT_MOUSE:
+                        case InputType::MOUSE:
                             tmp = "m";
                             break;
-                        case INPUT_JOYSTICK:
+                        case InputType::JOYSTICK:
                             tmp = "j";
                             break;
                         default:
@@ -217,69 +213,46 @@ void InputManager::store() const
     }
 }
 
+void InputManager::resetKey(const int i)
+{
+    InputFunction &key = mKey[i];
+    for (size_t i2 = 1; i2 < inputFunctionSize; i2 ++)
+    {
+        InputItem &ki2 = key.values[i2];
+        ki2.type = InputType::UNKNOWN;
+        ki2.value = -1;
+    }
+    const InputActionData &kd = inputActionData[i];
+    InputItem &val0 = key.values[0];
+    val0.type = kd.defaultType1;
+    InputItem &val1 = key.values[1];
+    val1.type = kd.defaultType2;
+#ifdef USE_SDL2
+    if (kd.defaultType1 == InputType::KEYBOARD)
+        val0.value = SDL_GetScancodeFromKey(kd.defaultValue1);
+    else
+        val0.value = kd.defaultValue1;
+    if (kd.defaultType2 == InputType::KEYBOARD)
+        val1.value = SDL_GetScancodeFromKey(kd.defaultValue2);
+    else
+        val1.value = kd.defaultValue2;
+#else
+    val0.value = kd.defaultValue1;
+    val1.value = kd.defaultValue2;
+#endif
+}
+
 void InputManager::resetKeys()
 {
-    for (int i = 0; i < Input::KEY_TOTAL; i++)
-    {
-        KeyFunction &key = mKey[i];
-        for (size_t i2 = 1; i2 < KeyFunctionSize; i2 ++)
-        {
-            KeyItem &ki2 = key.values[i2];
-            ki2.type = INPUT_UNKNOWN;
-            ki2.value = -1;
-        }
-        const KeyData &kd = keyData[i];
-        KeyItem &val0 = key.values[0];
-        val0.type = kd.defaultType1;
-        KeyItem &val1 = key.values[1];
-        val1.type = kd.defaultType2;
-#ifdef USE_SDL2
-        if (kd.defaultType1 == INPUT_KEYBOARD)
-            val0.value = SDL_GetScancodeFromKey(kd.defaultValue1);
-        else
-            val0.value = kd.defaultValue1;
-        if (kd.defaultType2 == INPUT_KEYBOARD)
-            val1.value = SDL_GetScancodeFromKey(kd.defaultValue2);
-        else
-            val1.value = kd.defaultValue2;
-#else
-        val0.value = kd.defaultValue1;
-        val1.value = kd.defaultValue2;
-#endif
-    }
+    for (int i = 0; i < InputAction::TOTAL; i++)
+        resetKey(i);
 }
 
 void InputManager::makeDefault(const int i)
 {
-    if (i >= 0 && i < Input::KEY_TOTAL)
+    if (i >= 0 && i < InputAction::TOTAL)
     {
-        KeyFunction &key = mKey[i];
-        for (size_t i2 = 1; i2 < KeyFunctionSize; i2 ++)
-        {
-            KeyItem &ki2 = key.values[i2];
-            ki2.type = INPUT_UNKNOWN;
-            ki2.value = -1;
-        }
-        const KeyData &kd = keyData[i];
-        KeyItem &val0 = key.values[0];
-        val0.type = kd.defaultType1;
-        KeyItem &val1 = key.values[1];
-        val1.type = kd.defaultType2;
-
-#ifdef USE_SDL2
-        if (kd.defaultType1 == INPUT_KEYBOARD)
-            val0.value = SDL_GetScancodeFromKey(kd.defaultValue1);
-        else
-            val0.value = kd.defaultValue1;
-        if (kd.defaultType2 == INPUT_KEYBOARD)
-            val1.value = SDL_GetScancodeFromKey(kd.defaultValue2);
-        else
-            val1.value = kd.defaultValue2;
-#else
-        val0.value = kd.defaultValue1;
-        val1.value = kd.defaultValue2;
-#endif
-
+        resetKey(i);
         update();
     }
 }
@@ -290,31 +263,34 @@ bool InputManager::hasConflicts(int &restrict key1, int &restrict key2) const
      * No need to parse the square matrix: only check one triangle
      * that's enough to detect conflicts
      */
-    for (int i = 0; i < Input::KEY_TOTAL; i++)
+    for (int i = 0; i < InputAction::TOTAL; i++)
     {
-        const KeyData &kdi = keyData[i];
+        const InputActionData &kdi = inputActionData[i];
         if (!*kdi.configField)
             continue;
 
-        const KeyFunction &ki = mKey[i];
-        for (size_t i2 = 0; i2 < KeyFunctionSize; i2 ++)
+        const InputFunction &ki = mKey[i];
+        for (size_t i2 = 0; i2 < inputFunctionSize; i2 ++)
         {
-            const KeyItem &vali2 = ki.values[i2];
-            if (vali2.value == Input::KEY_NO_VALUE)
+            const InputItem &vali2 = ki.values[i2];
+            if (vali2.value == InputAction::NO_VALUE)
                 continue;
 
             size_t j;
-            for (j = i, j++; j < Input::KEY_TOTAL; j++)
+            for (j = i, j++; j < InputAction::TOTAL; j++)
             {
-                if ((kdi.grp & keyData[j].grp) == 0 || !*kdi.configField)
-                    continue;
-
-                for (size_t j2 = 0; j2 < KeyFunctionSize; j2 ++)
+                if ((kdi.grp & inputActionData[j].grp) == 0
+                    || !*kdi.configField)
                 {
-                    const KeyItem &valj2 = mKey[j].values[j2];
+                    continue;
+                }
+
+                for (size_t j2 = 0; j2 < inputFunctionSize; j2 ++)
+                {
+                    const InputItem &valj2 = mKey[j].values[j2];
                     // Allow for item shortcut and emote keys to overlap
                     // as well as emote and ignore keys, but no other keys
-                    if (valj2.type != INPUT_UNKNOWN
+                    if (valj2.type != InputType::UNKNOWN
                         && vali2.value == valj2.value
                         && vali2.type == valj2.type)
                     {
@@ -339,7 +315,7 @@ bool InputManager::isActionActive(const int index) const
     if (!isActionActive0(index))
         return false;
 
-    const KeyData &key = keyData[index];
+    const InputActionData &key = inputActionData[index];
 //    logger->log("isActionActive mask=%d, condition=%d, index=%d",
 //        mMask, key.condition, index);
     if ((key.condition & mMask) != key.condition)
@@ -347,7 +323,7 @@ bool InputManager::isActionActive(const int index) const
     return true;
 }
 
-bool InputManager::isActionActive0(const int index) const
+bool InputManager::isActionActive0(const int index)
 {
     if (keyboard.isActionActive(index))
         return true;
@@ -356,9 +332,9 @@ bool InputManager::isActionActive0(const int index) const
     return touchManager.isActionActive(index);
 }
 
-KeyFunction &InputManager::getKey(int index)
+InputFunction &InputManager::getKey(int index)
 {
-    if (index < 0 || index >= Input::KEY_TOTAL)
+    if (index < 0 || index >= InputAction::TOTAL)
         index = 0;
     return mKey[index];
 }
@@ -366,13 +342,13 @@ KeyFunction &InputManager::getKey(int index)
 std::string InputManager::getKeyStringLong(const int index) const
 {
     std::string keyStr;
-    const KeyFunction &ki = mKey[index];
+    const InputFunction &ki = mKey[index];
 
-    for (size_t i = 0; i < KeyFunctionSize; i ++)
+    for (size_t i = 0; i < inputFunctionSize; i ++)
     {
-        const KeyItem &key = ki.values[i];
+        const InputItem &key = ki.values[i];
         std::string str;
-        if (key.type == INPUT_KEYBOARD)
+        if (key.type == InputType::KEYBOARD)
         {
             if (key.value >= 0)
             {
@@ -384,7 +360,7 @@ std::string InputManager::getKeyStringLong(const int index) const
                 str = strprintf(_("key_%d"), -key.value);
             }
         }
-        else if (key.type == INPUT_JOYSTICK)
+        else if (key.type == InputType::JOYSTICK)
         {
             // TRANSLATORS: long joystick button name. must be short.
             str = strprintf(_("JButton%d"), key.value + 1);
@@ -409,13 +385,13 @@ std::string InputManager::getKeyStringLong(const int index) const
 std::string InputManager::getKeyValueString(const int index) const
 {
     std::string keyStr;
-    const KeyFunction &ki = mKey[index];
+    const InputFunction &ki = mKey[index];
 
-    for (size_t i = 0; i < KeyFunctionSize; i ++)
+    for (size_t i = 0; i < inputFunctionSize; i ++)
     {
-        const KeyItem &key = ki.values[i];
+        const InputItem &key = ki.values[i];
         std::string str;
-        if (key.type == INPUT_KEYBOARD)
+        if (key.type == InputType::KEYBOARD)
         {
             if (key.value >= 0)
             {
@@ -428,7 +404,7 @@ std::string InputManager::getKeyValueString(const int index) const
                 str = strprintf(_("key_%d"), -key.value);
             }
         }
-        else if (key.type == INPUT_JOYSTICK)
+        else if (key.type == InputType::JOYSTICK)
         {
             // TRANSLATORS: short joystick button name. muse be very short
             str = strprintf(_("JB%d"), key.value + 1);
@@ -463,15 +439,15 @@ std::string InputManager::getKeyValueByName(const std::string &keyName)
 void InputManager::addActionKey(const int action, const int type,
                                 const int val)
 {
-    if (action < 0 || action >= Input::KEY_TOTAL)
+    if (action < 0 || action >= InputAction::TOTAL)
         return;
 
     int idx = -1;
-    KeyFunction &key = mKey[action];
-    for (size_t i = 0; i < KeyFunctionSize; i ++)
+    InputFunction &key = mKey[action];
+    for (size_t i = 0; i < inputFunctionSize; i ++)
     {
-        const KeyItem &val2 = key.values[i];
-        if (val2.type == INPUT_UNKNOWN || (val2.type == type
+        const InputItem &val2 = key.values[i];
+        if (val2.type == InputType::UNKNOWN || (val2.type == type
             && val2.value == val))
         {
             idx = static_cast<int>(i);
@@ -480,25 +456,25 @@ void InputManager::addActionKey(const int action, const int type,
     }
     if (idx == -1)
     {
-        for (size_t i = 1; i < KeyFunctionSize; i ++)
+        for (size_t i = 1; i < inputFunctionSize; i ++)
         {
-            KeyItem &val1 = key.values[i - 1];
-            KeyItem &val2 = key.values[i];
+            InputItem &val1 = key.values[i - 1];
+            InputItem &val2 = key.values[i];
             val1.type = val2.type;
             val1.value = val2.value;
         }
-        idx = KeyFunctionSize - 1;
+        idx = inputFunctionSize - 1;
     }
 
-    key.values[idx] = KeyItem(type, val);
+    key.values[idx] = InputItem(type, val);
 }
 
 void InputManager::setNewKey(const SDL_Event &event, const int type)
 {
     int val = -1;
-    if (type == INPUT_KEYBOARD)
-        val = keyboard.getKeyValueFromEvent(event);
-    else if (type == INPUT_JOYSTICK && joystick)
+    if (type == InputType::KEYBOARD)
+        val = KeyboardConfig::getKeyValueFromEvent(event);
+    else if (type == InputType::JOYSTICK && joystick)
         val = joystick->getButtonFromEvent(event);
 
     if (val != -1)
@@ -510,11 +486,11 @@ void InputManager::setNewKey(const SDL_Event &event, const int type)
 
 void InputManager::unassignKey()
 {
-    KeyFunction &key = mKey[mNewKeyIndex];
-    for (size_t i = 0; i < KeyFunctionSize; i ++)
+    InputFunction &key = mKey[mNewKeyIndex];
+    for (size_t i = 0; i < inputFunctionSize; i ++)
     {
-        KeyItem &val = key.values[i];
-        val.type = INPUT_UNKNOWN;
+        InputItem &val = key.values[i];
+        val.type = InputType::UNKNOWN;
         val.value = -1;
     }
     update();
@@ -523,11 +499,11 @@ void InputManager::unassignKey()
 bool InputManager::handleAssignKey(const SDL_Event &event, const int type)
 {
     if (setupWindow && setupWindow->isWindowVisible() &&
-        getNewKeyIndex() > Input::KEY_NO_VALUE)
+        getNewKeyIndex() > InputAction::NO_VALUE)
     {
         setNewKey(event, type);
         callbackNewKey();
-        setNewKeyIndex(Input::KEY_NO_VALUE);
+        setNewKeyIndex(InputAction::NO_VALUE);
         return true;
     }
     return false;
@@ -535,32 +511,29 @@ bool InputManager::handleAssignKey(const SDL_Event &event, const int type)
 
 bool InputManager::handleEvent(const SDL_Event &event)
 {
+    BLOCK_START("InputManager::handleEvent")
     switch (event.type)
     {
         case SDL_KEYDOWN:
         {
             keyboard.refreshActiveKeys();
             updateConditionMask();
-            if (handleAssignKey(event, INPUT_KEYBOARD))
+            if (handleAssignKey(event, InputType::KEYBOARD))
+            {
+                BLOCK_END("InputManager::handleEvent")
                 return true;
+            }
 
             keyboard.handleActicateKey(event);
             // send straight to gui for certain windows
             if (quitDialog || TextDialog::isActive() ||
                 NpcPostDialog::isActive())
             {
-                try
-                {
-                    if (guiInput)
-                        guiInput->pushInput(event);
-                    if (gui)
-                        gui->handleInput();
-                }
-                catch(const gcn::Exception &e)
-                {
-                    const char *const err = e.getMessage().c_str();
-                    logger->log("Warning: guichan input exception: %s", err);
-                }
+                if (guiInput)
+                    guiInput->pushInput(event);
+                if (gui)
+                    gui->handleInput();
+                BLOCK_END("InputManager::handleEvent")
                 return true;
             }
             break;
@@ -576,8 +549,11 @@ bool InputManager::handleEvent(const SDL_Event &event)
         {
             updateConditionMask();
 //            joystick.handleActicateButton(event);
-            if (handleAssignKey(event, INPUT_JOYSTICK))
+            if (handleAssignKey(event, InputType::JOYSTICK))
+            {
+                BLOCK_END("InputManager::handleEvent")
                 return true;
+            }
             break;
         }
         case SDL_JOYBUTTONUP:
@@ -605,35 +581,36 @@ bool InputManager::handleEvent(const SDL_Event &event)
             break;
     }
 
-    try
-    {
-        if (guiInput)
-            guiInput->pushInput(event);
-    }
-    catch(const gcn::Exception &e)
-    {
-        const char *const err = e.getMessage().c_str();
-        logger->log("Warning: guichan input exception: %s", err);
-    }
+    if (guiInput)
+        guiInput->pushInput(event);
     if (gui)
     {
         const bool res = gui->handleInput();
         if (res && event.type == SDL_KEYDOWN)
+        {
+            BLOCK_END("InputManager::handleEvent")
             return true;
+        }
     }
 
     switch (event.type)
     {
         case SDL_KEYDOWN:
             if (triggerAction(keyboard.getActionVector(event)))
+            {
+                BLOCK_END("InputManager::handleEvent")
                 return true;
+            }
             break;
 
         case SDL_JOYBUTTONDOWN:
             if (joystick && joystick->validate())
             {
                 if (triggerAction(joystick->getActionVector(event)))
+                {
+                    BLOCK_END("InputManager::handleEvent")
                     return true;
+                }
             }
             break;
 #ifdef ANDROID
@@ -648,10 +625,11 @@ bool InputManager::handleEvent(const SDL_Event &event)
             break;
     }
 
+    BLOCK_END("InputManager::handleEvent")
     return false;
 }
 
-void InputManager::handleRepeat() const
+void InputManager::handleRepeat()
 {
     const int time = tick_time;
     keyboard.handleRepeat(time);
@@ -663,73 +641,73 @@ void InputManager::updateConditionMask()
 {
     mMask = 1;
     if (keyboard.isEnabled())
-        mMask |= COND_ENABLED;
+        mMask |= InputCondition::ENABLED;
     if ((!chatWindow || !chatWindow->isInputFocused()) &&
         !NpcDialog::isAnyInputFocused() &&
         !InventoryWindow::isAnyInputFocused() &&
         (!tradeWindow || !tradeWindow->isInpupFocused()))
     {
-        mMask |= COND_NOINPUT;
+        mMask |= InputCondition::NOINPUT;
     }
 
     if (!BuyDialog::isActive() && !SellDialog::isActive())
-        mMask |= COND_NOBUYSELL;
+        mMask |= InputCondition::NOBUYSELL;
 
-    if (!player_node || !player_node->getAway())
-        mMask |= COND_NOAWAY;
+    if (!settings.awayMode)
+        mMask |= InputCondition::NOAWAY;
 
     if (!setupWindow || !setupWindow->isWindowVisible())
-        mMask |= COND_NOSETUP;
+        mMask |= InputCondition::NOSETUP;
 
     if (Game::instance() && Game::instance()->getValidSpeed())
-        mMask |= COND_VALIDSPEED;
+        mMask |= InputCondition::VALIDSPEED;
 
     if (gui && !gui->getFocusHandler()->getModalFocused())
-        mMask |= COND_NOMODAL;
+        mMask |= InputCondition::NOMODAL;
 
     const NpcDialog *const dialog = NpcDialog::getActive();
     if (!dialog || !dialog->isTextInputFocused())
-        mMask |= COND_NONPCINPUT;
+        mMask |= InputCondition::NONPCINPUT;
     if (!dialog || dialog->isCloseState())
     {
-        mMask |= COND_NONPCDIALOG;
+        mMask |= InputCondition::NONPCDIALOG;
         if (!InventoryWindow::isStorageActive())
-            mMask |= COND_NOTALKING;
+            mMask |= InputCondition::NOTALKING;
     }
 
-    if (!player_node || !player_node->getDisableGameModifiers())
-        mMask |= COND_EMODS;
+    if (!settings.disableGameModifiers)
+        mMask |= InputCondition::EMODS;
 
-    if (!isActionActive0(Input::KEY_STOP_ATTACK)
-        && !isActionActive0(Input::KEY_UNTARGET))
+    if (!isActionActive0(InputAction::STOP_ATTACK)
+        && !isActionActive0(InputAction::UNTARGET))
     {
-        mMask |= COND_NOTARGET;
+        mMask |= InputCondition::NOTARGET;
     }
     if (Game::instance())
-        mMask |= COND_INGAME;
+        mMask |= InputCondition::INGAME;
 
-    if (!player_node || player_node->getFollow().empty())
-        mMask |= COND_NOFOLLOW;
+    if (!localPlayer || localPlayer->getFollow().empty())
+        mMask |= InputCondition::NOFOLLOW;
 }
 
-bool InputManager::checkKey(const KeyData *const key) const
+bool InputManager::checkKey(const InputActionData *const key) const
 {
     // logger->log("checkKey mask=%d, condition=%d", mMask, key->condition);
     if (!key || (key->condition & mMask) != key->condition)
         return false;
 
-    return (key->modKeyIndex == Input::KEY_NO_VALUE
+    return (key->modKeyIndex == InputAction::NO_VALUE
         || isActionActive0(key->modKeyIndex));
 }
 
-bool InputManager::invokeKey(const KeyData *const key, const int keyNum)
+bool InputManager::invokeKey(const InputActionData *const key, const int keyNum)
 {
     // no validation to keyNum because it validated in caller
 
     if (checkKey(key))
     {
         InputEvent evt(keyNum, mMask);
-        ActionFuncPtr func = *(keyData[keyNum].action);
+        ActionFuncPtr func = *(inputActionData[keyNum].action);
         if (func && func(evt))
             return true;
     }
@@ -738,11 +716,11 @@ bool InputManager::invokeKey(const KeyData *const key, const int keyNum)
 
 void InputManager::executeAction(const int keyNum)
 {
-    if (keyNum < 0 || keyNum >= Input::KEY_TOTAL)
+    if (keyNum < 0 || keyNum >= InputAction::TOTAL)
         return;
 
     InputEvent evt(keyNum, mMask);
-    ActionFuncPtr func = *(keyData[keyNum].action);
+    ActionFuncPtr func = *(inputActionData[keyNum].action);
     if (func)
         func(evt);
 }
@@ -755,45 +733,45 @@ void InputManager::updateKeyActionMap(KeyToActionMap &actionMap,
     actionMap.clear();
     keyTimeMap.clear();
 
-    for (size_t i = 0; i < Input::KEY_TOTAL; i ++)
+    for (size_t i = 0; i < InputAction::TOTAL; i ++)
     {
-        const KeyFunction &key = mKey[i];
-        const KeyData &kd = keyData[i];
+        const InputFunction &key = mKey[i];
+        const InputActionData &kd = inputActionData[i];
         if (kd.action)
         {
-            for (size_t i2 = 0; i2 < KeyFunctionSize; i2 ++)
+            for (size_t i2 = 0; i2 < inputFunctionSize; i2 ++)
             {
-                const KeyItem &ki = key.values[i2];
+                const InputItem &ki = key.values[i2];
                 if (ki.type == type && ki.value != -1)
                     actionMap[ki.value].push_back(static_cast<int>(i));
             }
         }
         if (kd.configField && (kd.grp & Input::GRP_GUICHAN))
         {
-            for (size_t i2 = 0; i2 < KeyFunctionSize; i2 ++)
+            for (size_t i2 = 0; i2 < inputFunctionSize; i2 ++)
             {
-                const KeyItem &ki = key.values[i2];
+                const InputItem &ki = key.values[i2];
                 if (ki.type == type && ki.value != -1)
                     idMap[ki.value] = static_cast<int>(i);
             }
         }
         if (kd.configField && (kd.grp & Input::GRP_REPEAT))
         {
-            for (size_t i2 = 0; i2 < KeyFunctionSize; i2 ++)
+            for (size_t i2 = 0; i2 < inputFunctionSize; i2 ++)
             {
-                const KeyItem &ki = key.values[i2];
+                const InputItem &ki = key.values[i2];
                 if (ki.type == type && ki.value != -1)
                     keyTimeMap[ki.value] = 0;
             }
         }
     }
 
-    keyDataSorter.keys = &keyData[0];
+    inputActionDataSorter.keys = &inputActionData[0];
     FOR_EACH (KeyToActionMapIter, it, actionMap)
     {
         KeysVector *const keys = &it->second;
         if (keys && keys->size() > 1)
-            std::sort(keys->begin(), keys->end(), keyDataSorter);
+            std::sort(keys->begin(), keys->end(), inputActionDataSorter);
     }
 }
 
@@ -807,10 +785,10 @@ bool InputManager::triggerAction(const KeysVector *const ptrs)
     FOR_EACHP (KeysVectorCIter, it, ptrs)
     {
         const int keyNum = *it;
-        if (keyNum < 0 || keyNum >= Input::KEY_TOTAL)
+        if (keyNum < 0 || keyNum >= InputAction::TOTAL)
             continue;
 
-        if (invokeKey(&keyData[keyNum], keyNum))
+        if (invokeKey(&inputActionData[keyNum], keyNum))
             return true;
     }
     return false;
@@ -819,13 +797,13 @@ bool InputManager::triggerAction(const KeysVector *const ptrs)
 int InputManager::getKeyIndex(const int value, const int grp,
                               const int type) const
 {
-    for (size_t i = 0; i < Input::KEY_TOTAL; i++)
+    for (size_t i = 0; i < InputAction::TOTAL; i++)
     {
-        const KeyFunction &key = mKey[i];
-        const KeyData &kd = keyData[i];
-        for (size_t i2 = 0; i2 < KeyFunctionSize; i2 ++)
+        const InputFunction &key = mKey[i];
+        const InputActionData &kd = inputActionData[i];
+        for (size_t i2 = 0; i2 < inputFunctionSize; i2 ++)
         {
-            const KeyItem &vali2 = key.values[i2];
+            const InputItem &vali2 = key.values[i2];
             if (value == vali2.value && (grp & kd.grp) != 0
                 && vali2.type == type)
             {
@@ -833,7 +811,7 @@ int InputManager::getKeyIndex(const int value, const int grp,
             }
         }
     }
-    return Input::KEY_NO_VALUE;
+    return InputAction::NO_VALUE;
 }
 
 int InputManager::getActionByKey(const SDL_Event &event) const
@@ -842,8 +820,8 @@ int InputManager::getActionByKey(const SDL_Event &event) const
     if (event.type == SDL_KEYDOWN || event.type == SDL_KEYUP)
     {
         const int idx = keyboard.getActionId(event);
-        if (idx >= 0 && checkKey(&keyData[idx]))
+        if (idx >= 0 && checkKey(&inputActionData[idx]))
             return idx;
     }
-    return Input::KEY_NO_VALUE;
+    return InputAction::NO_VALUE;
 }
