@@ -21,80 +21,19 @@
 #ifndef INPUT_INPUTMANAGER_H
 #define INPUT_INPUTMANAGER_H
 
-#include "input/inputevent.h"
-#include "input/keydata.h"
+#include "input/inputaction.h"
+#include "input/inputfunction.h"
+
+#include "events/inputevent.h"
 
 #include <string>
 #include <map>
 
 #include <SDL_events.h>
 
-const unsigned int KeyFunctionSize = 3;
-
-// hack to avoid conflicts with windows headers.
-#ifdef INPUT_KEYBOARD
-#undef INPUT_KEYBOARD
-#endif
-#ifdef INPUT_MOUSE
-#undef INPUT_MOUSE
-#endif
-
 class Setup_Input;
 
-enum KeyTypes
-{
-    INPUT_UNKNOWN = 0,
-    INPUT_KEYBOARD = 1,
-    INPUT_MOUSE = 2,
-    INPUT_JOYSTICK = 3
-};
-
-struct KeyItem final
-{
-    KeyItem() :
-        type(-1), value(-1)
-    { }
-
-    KeyItem(const int type0, const int value0) :
-        type(type0), value(value0)
-    { }
-
-    int type;
-
-    int value;
-};
-
-struct KeyFunction final
-{
-    KeyItem values[KeyFunctionSize];
-};
-
-enum KeyCondition
-{
-    COND_DEFAULT = 1,                         // default condition
-    COND_ENABLED = 2,                         // keyboard must be enabled
-    COND_NOINPUT = 4,                         // input items must be unfocused
-    COND_NOAWAY = 8,                          // player not in away mode
-    COND_NOSETUP = 16,                        // setup window is hidde
-    COND_VALIDSPEED = 32,                     // valid speed
-    COND_NOMODAL = 64,                        // modal windows inactive
-    COND_NONPCINPUT = 128,                    // npc input field inactive
-    COND_EMODS = 256,                         // game modifiers enabled
-    COND_NOTARGET = 512,                      // no target/untarget keys
-                                              // pressed
-    COND_NOFOLLOW = 1024,                     // follow mode disabled
-    COND_INGAME = 2048,                       // game must be started
-    COND_NOBUYSELL = 4096,                    // no active buy or sell dialogs
-    COND_NONPCDIALOG = 8192,                  // no active npc dialog or
-                                              // dialog almost closed
-    COND_NOTALKING = 16384,                   // player have no opened
-                                              // dialogs what prevent moving
-    COND_SHORTCUT = 2 + 4 + 16 + 512 + 2048,  // flags for shortcut keys
-    COND_SHORTCUT0 = 2 + 4 + 16 + 512,        // flags for shortcut keys
-    COND_GAME = 2 + 4 + 8 + 16 + 64 + 2048,   // main game key
-    COND_GAME2 = 2 + 8 + 16 + 64 + 2048,
-    COND_ARROWKEYS = 2 + 8 + 16 + 64 + 2048 + 4096 + 16384
-};
+struct InputActionData;
 
 class InputManager final
 {
@@ -107,7 +46,7 @@ class InputManager final
 
         bool handleEvent(const SDL_Event &event);
 
-        bool checkKey(const KeyData *const key) const A_WARN_UNUSED;
+        bool checkKey(const InputActionData *const key) const A_WARN_UNUSED;
 
         void retrieve();
 
@@ -122,7 +61,7 @@ class InputManager final
 
         void callbackNewKey();
 
-        KeyFunction &getKey(int index) A_WARN_UNUSED;
+        InputFunction &getKey(int index) A_WARN_UNUSED;
 
         std::string getKeyValueString(const int index) const A_WARN_UNUSED;
 
@@ -160,18 +99,18 @@ class InputManager final
                                 KeyToIdMap &idMap, KeyTimeMap &keyTimeMap,
                                 const int type) const;
 
-        bool invokeKey(const KeyData *const key, const int keyNum);
+        bool invokeKey(const InputActionData *const key, const int keyNum);
 
         bool handleAssignKey(const SDL_Event &event, const int type);
 
-        void handleRepeat() const;
+        static void handleRepeat();
 
         bool triggerAction(const KeysVector *const ptrs);
 
         int getKeyIndex(const int value, const int grp,
                         const int type) const A_WARN_UNUSED;
 
-        void update() const;
+        static void update();
 
         void updateConditionMask();
 
@@ -180,7 +119,9 @@ class InputManager final
         void executeAction(const int keyNum);
 
     protected:
-        bool isActionActive0(const int index) const A_WARN_UNUSED;
+        void resetKey(const int i);
+
+        static bool isActionActive0(const int index) A_WARN_UNUSED;
 
         Setup_Input *mSetupInput;      /**< Reference to setup window */
 
@@ -190,7 +131,7 @@ class InputManager final
 
         std::map<std::string, int> mNameMap;
 
-        KeyFunction mKey[Input::KEY_TOTAL]; /**< Pointer to all the key data */
+        InputFunction mKey[InputAction::TOTAL];
 };
 
 extern InputManager inputManager;
