@@ -30,6 +30,9 @@
 
 #include "resources/iteminfo.h"
 
+#include "listeners/inventorylistener.h"
+
+#include "utils/delete2.h"
 #include "utils/gettext.h"
 
 #include <algorithm>
@@ -38,14 +41,17 @@
 
 #include "debug.h"
 
-struct SlotUsed final
+namespace
 {
-    bool operator()(const Item *const item) const
+    struct SlotUsed final
     {
-        return item && item->mId >= 0 && item->mQuantity > 0;
-    }
-    typedef Item *argument_type;
-};
+        bool operator()(const Item *const item) const
+        {
+            return item && item->mId >= 0 && item->mQuantity > 0;
+        }
+        typedef Item *argument_type;
+    };
+}  // namespace
 
 Inventory::Inventory(const int type, const int size1) :
     mInventoryListeners(),
@@ -97,14 +103,14 @@ Item *Inventory::findItem(const int itemId, const unsigned char color) const
     return nullptr;
 }
 
-void Inventory::addItem(const int id, const int quantity, const int refine,
-                        const unsigned char color, const bool equipment)
+void Inventory::addItem(const int id, const int quantity, const uint8_t refine,
+                        const uint8_t color, const bool equipment)
 {
     setItem(getFreeSlot(), id, quantity, refine, color, equipment);
 }
 
 void Inventory::setItem(const int index, const int id, const int quantity,
-                        const int refine, const unsigned char color,
+                        const uint8_t refine, const unsigned char color,
                         const bool equipment)
 {
     if (index < 0 || index >= static_cast<int>(mSize))
@@ -153,8 +159,7 @@ void Inventory::removeItem(const int id)
 
 void Inventory::removeItemAt(const int index)
 {
-    delete mItems[index];
-    mItems[index] = nullptr;
+    delete2(mItems[index]);
     mUsed--;
     if (mUsed < 0)  // Already at 0, no need to distribute event
         mUsed = 0;
@@ -182,7 +187,7 @@ int Inventory::getFreeSlot() const
 {
     Item **const i = std::find_if(mItems, mItems + mSize,
         std::not1(SlotUsed()));
-    return (i == mItems + static_cast<int>(mSize)) ? -1
+    return (i == mItems + mSize) ? -1
         : static_cast<int>(i - mItems);
 }
 
@@ -214,7 +219,7 @@ void Inventory::distributeSlotsChangedEvent()
 }
 
 const Item *Inventory::findItemBySprite(std::string spritePath,
-                                        const Gender gender,
+                                        const Gender::Type gender,
                                         const int race) const
 {
     spritePath = removeSpriteIndex(spritePath);
@@ -280,6 +285,6 @@ void Inventory::resize(const unsigned int newSize)
     delete [] mItems;
 
     mSize = newSize;
-    mItems = new Item*[mSize];
+    mItems = new Item*[static_cast<size_t>(mSize)];
     std::fill_n(mItems, mSize, static_cast<Item*>(nullptr));
 }
