@@ -37,9 +37,9 @@ extern int serverVersion;
 
 DragDrop dragDrop(nullptr, DRAGDROP_SOURCE_EMPTY);
 
-Item::Item(const int id, const int quantity, const int refine,
+Item::Item(const int id, const int quantity, const uint8_t refine,
            const unsigned char color, const bool equipment,
-           const bool equipped):
+           const bool equipped) :
     mId(0),
     mColor(0),
     mQuantity(quantity),
@@ -71,7 +71,7 @@ void Item::setId(const int id, const unsigned char color)
     mColor = color;
 
     // Types 0 and 1 are not equippable items.
-    mEquipment = id && getInfo().getType() >= 2;
+    mEquipment = id && static_cast<int>(getInfo().getType()) >= 2;
 
     if (mImage)
         mImage->decRef();
@@ -92,11 +92,12 @@ void Item::setId(const int id, const unsigned char color)
     }
 }
 
-bool Item::isHaveTag(const int tagId)
+bool Item::isHaveTag(const int tagId) const
 {
-    if (mTags.find(tagId) == mTags.end())
+    const std::map <int, int>::const_iterator it = mTags.find(tagId);
+    if (it == mTags.end())
         return false;
-    return mTags[tagId] > 0;
+    return (*it).second > 0;
 }
 
 Image *Item::getImage(const int id, const unsigned char color)
@@ -112,7 +113,7 @@ Image *Item::getImage(const int id, const unsigned char color)
     return image;
 }
 
-std::string Item::getName()
+std::string Item::getName() const
 {
     const ItemInfo &info = ItemDB::get(mId);
     if (serverVersion > 0)
