@@ -20,15 +20,18 @@
 
 #include "navigationmanager.h"
 
-#include "map.h"
-#include "walklayer.h"
+#include "resources/map/map.h"
+#include "resources/map/metatile.h"
+#include "resources/map/walklayer.h"
 
-static const int walkMask = (Map::BLOCKMASK_WALL | Map::BLOCKMASK_AIR
-    | Map::BLOCKMASK_WATER);
+#include "debug.h"
+
+static const int walkMask = (BlockMask::WALL | BlockMask::AIR
+    | BlockMask::WATER);
 
 namespace
 {
-    struct Cell
+    struct Cell final
     {
         Cell(const int x0, const int y0) :
             x(x0),
