@@ -24,7 +24,6 @@
 #define NET_CHARSERVERHANDLER_H
 
 #include "being/localplayer.h"
-#include "being/playerinfo.h"
 
 #include <string>
 #include <vector>
@@ -35,34 +34,11 @@ class CharSelectDialog;
 namespace Net
 {
 
-/**
- * A structure to hold information about a character.
- */
-struct Character final
-{
-    Character() :
-        slot(0),
-        dummy(nullptr),
-        data()
-    {
-    }
-
-    A_DELETE_COPY(Character)
-
-    ~Character()
-    {
-        delete dummy;
-        dummy = nullptr;
-    }
-
-    int slot;            /**< The index in the list of characters */
-    LocalPlayer *dummy;  /**< A dummy representing this character */
-    PlayerInfoBackend data;
-};
+struct Character;
 
 typedef std::list<Character*> Characters;
 
-class CharServerHandler
+class CharServerHandler notfinal
 {
     public:
         A_DELETE_COPY(CharServerHandler)
