@@ -42,7 +42,6 @@
 #include "gui/windows/outfitwindow.h"
 #include "gui/windows/socialwindow.h"
 #include "gui/windows/killstats.h"
-#include "gui/windows/questswindow.h"
 
 #include "utils/timer.h"
 
@@ -50,9 +49,12 @@
 
 #include "resources/db/itemdb.h"
 
+#include "resources/map/map.h"
+
 #include "debug.h"
 
 extern int serverVersion;
+extern unsigned int tmwServerVersion;
 
 namespace Ea
 {
@@ -68,37 +70,17 @@ Being *BeingHandler::createBeing(const int id, const int16_t job) const
     if (!actorManager)
         return nullptr;
 
-    ActorSprite::Type type = ActorSprite::UNKNOWN;
+    ActorType::Type type = ActorType::UNKNOWN;
     if (job <= 25 || (job >= 4001 && job <= 4049))
-        type = ActorSprite::PLAYER;
+        type = ActorType::PLAYER;
     else if (job >= 46 && job <= 1000)
-        type = ActorSprite::NPC;
+        type = ActorType::NPC;
     else if (job > 1000 && job <= 2000)
-        type = ActorSprite::MONSTER;
+        type = ActorType::MONSTER;
     else if (job == 45)
-        type = ActorSprite::PORTAL;
+        type = ActorType::PORTAL;
 
     Being *const being = actorManager->createBeing(id, type, job);
-
-    if (type == ActorSprite::PLAYER || type == ActorSprite::NPC)
-    {
-        being->updateFromCache();
-        requestNameById(id);
-        if (player_node)
-            player_node->checkNewName(being);
-    }
-    if (type == Being::PLAYER)
-    {
-        if (botCheckerWindow)
-            botCheckerWindow->updateList();
-        if (socialWindow)
-            socialWindow->updateActiveList();
-    }
-    else if (type == Being::NPC)
-    {
-        if (questsWindow)
-            questsWindow->addEffect(being);
-    }
     return being;
 }
 
@@ -116,8 +98,12 @@ void BeingHandler::setSprite(Being *const being, const unsigned int slot,
 void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
                                              const bool visible)
 {
+    BLOCK_START("BeingHandler::processBeingVisibleOrMove")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processBeingVisibleOrMove")
         return;
+    }
 
     int spawnId;
 
@@ -134,12 +120,12 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
     statusEffects |= (static_cast<uint32_t>(msg.readInt16())) << 16;  // option
     const int16_t job = msg.readInt16();  // class
     int disguiseId = 0;
-    if (id == player_node->getId() && job >= 1000)
+    if (id == localPlayer->getId() && job >= 1000)
         disguiseId = job;
 
     Being *dstBeing = actorManager->findBeing(id);
 
-    if (dstBeing && dstBeing->getType() == Being::MONSTER
+    if (dstBeing && dstBeing->getType() == ActorType::MONSTER
         && !dstBeing->isAlive())
     {
         actorManager->destroy(dstBeing);
@@ -152,22 +138,31 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         // Being with id >= 110000000 and job 0 are better
         // known as ghosts, so don't create those.
         if (job == 0 && id >= 110000000)
+        {
+            BLOCK_END("BeingHandler::processBeingVisibleOrMove")
             return;
+        }
 
         if (actorManager->isBlocked(id) == true)
+        {
+            BLOCK_END("BeingHandler::processBeingVisibleOrMove")
             return;
+        }
 
         dstBeing = createBeing(id, job);
 
         if (!dstBeing)
+        {
+            BLOCK_END("BeingHandler::processBeingVisibleOrMove")
             return;
+        }
 
         if (job == 1022 && killStats)
             killStats->jackoAlive(dstBeing->getId());
     }
     else
     {
-        if (dstBeing->getType() == Being::NPC)
+        if (dstBeing->getType() == ActorType::NPC)
         {
             actorManager->undelete(dstBeing);
             if (serverVersion < 1)
@@ -175,29 +170,29 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         }
     }
 
-    if (dstBeing->getType() == Being::PLAYER)
+    if (dstBeing->getType() == ActorType::PLAYER)
         dstBeing->setMoveTime();
 
     if (spawnId)
     {
-        dstBeing->setAction(Being::SPAWN, 0);
+        dstBeing->setAction(BeingAction::SPAWN, 0);
     }
     else if (visible)
     {
         dstBeing->clearPath();
         dstBeing->setActionTime(tick_time);
-        dstBeing->setAction(Being::STAND, 0);
+        dstBeing->setAction(BeingAction::STAND, 0);
     }
 
     // Prevent division by 0 when calculating frame
     if (speed == 0)
         speed = 150;
 
-    const int hairStyle = msg.readInt8();
-    const int look = msg.readInt8();
+    const uint8_t hairStyle = msg.readUInt8();
+    const uint8_t look = msg.readUInt8();
     dstBeing->setSubtype(job, look);
-    if (dstBeing->getType() == ActorSprite::MONSTER && player_node)
-        player_node->checkNewName(dstBeing);
+    if (dstBeing->getType() == ActorType::MONSTER && localPlayer)
+        localPlayer->checkNewName(dstBeing);
     dstBeing->setWalkSpeed(Vector(speed, speed, 0));
     const uint16_t weapon = msg.readInt16();
     const uint16_t headBottom = msg.readInt16();
@@ -208,14 +203,14 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
     const uint16_t shield = msg.readInt16();
     const uint16_t headTop = msg.readInt16();
     const uint16_t headMid = msg.readInt16();
-    const int hairColor = msg.readInt8();
-    msg.readInt8();  // free
+    const uint8_t hairColor = msg.readUInt8();
+    msg.readUInt8();  // free
     const uint16_t shoes = msg.readInt16();  // clothes color
 
     uint16_t gloves;
-    if (dstBeing->getType() == ActorSprite::MONSTER)
+    if (dstBeing->getType() == ActorType::MONSTER)
     {
-        if (serverVersion > 0)
+        if (serverVersion > 0 || tmwServerVersion >= 0x0E0701)
         {
             const int hp = msg.readInt32();
             const int maxHP = msg.readInt32();
@@ -244,18 +239,18 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
 
     msg.readInt16();  // manner
     dstBeing->setStatusEffectBlock(32, msg.readInt16());  // opt3
-    if (serverVersion > 0 && dstBeing->getType() == ActorSprite::MONSTER)
+    if (serverVersion > 0 && dstBeing->getType() == ActorType::MONSTER)
     {
-        const int attackRange = msg.readInt8();   // karma
+        const int attackRange = static_cast<int>(msg.readUInt8());   // karma
         dstBeing->setAttackRange(attackRange);
     }
     else
     {
-        msg.readInt8();   // karma
+        msg.readUInt8();   // karma
     }
-    uint8_t gender = msg.readInt8();
+    uint8_t gender = msg.readUInt8();
 
-    if (!disguiseId && dstBeing->getType() == ActorSprite::PLAYER)
+    if (!disguiseId && dstBeing->getType() == ActorType::PLAYER)
     {
         // reserving bits for future usage
         gender &= 3;
@@ -273,21 +268,21 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         if (!mHideShield)
             setSprite(dstBeing, EA_SPRITE_SHIELD, shield);
     }
-    else if (dstBeing->getType() == ActorSprite::NPC)
+    else if (dstBeing->getType() == ActorType::NPC)
     {
         switch (gender)
         {
             case 2:
-                dstBeing->setGender(GENDER_FEMALE);
+                dstBeing->setGender(Gender::FEMALE);
                 break;
             case 3:
-                dstBeing->setGender(GENDER_MALE);
+                dstBeing->setGender(Gender::MALE);
                 break;
             case 4:
-                dstBeing->setGender(GENDER_OTHER);
+                dstBeing->setGender(Gender::OTHER);
                 break;
             default:
-                dstBeing->setGender(GENDER_UNSPECIFIED);
+                dstBeing->setGender(Gender::UNSPECIFIED);
                 break;
         }
     }
@@ -298,7 +293,7 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         msg.readCoordinatePair(srcX, srcY, dstX, dstY);
         if (!disguiseId)
         {
-            dstBeing->setAction(Being::STAND, 0);
+            dstBeing->setAction(BeingAction::STAND, 0);
             dstBeing->setTileCoords(srcX, srcY);
             if (serverVersion < 10)
                 dstBeing->setDestination(dstX, dstY);
@@ -328,8 +323,8 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         dstBeing->setDirection(dir);
     }
 
-    msg.readInt8();   // unknown
-    msg.readInt8();   // unknown
+    msg.readUInt8();   // unknown
+    msg.readUInt8();   // unknown
     msg.readInt16();
 
     dstBeing->setStunMode(stunMode);
@@ -337,12 +332,17 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         (statusEffects >> 16) & 0xffff));
     dstBeing->setStatusEffectBlock(16, static_cast<uint16_t>(
         statusEffects & 0xffff));
+    BLOCK_END("BeingHandler::processBeingVisibleOrMove")
 }
 
 void BeingHandler::processBeingMove2(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processBeingMove2")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processBeingMove2")
         return;
+    }
 
     /*
       * A simplified movement packet, used by the
@@ -358,21 +358,26 @@ void BeingHandler::processBeingMove2(Net::MessageIn &msg) const
       */
 
     if (!dstBeing)
+    {
+        BLOCK_END("BeingHandler::processBeingMove2")
         return;
+    }
 
     uint16_t srcX, srcY, dstX, dstY;
     msg.readCoordinatePair(srcX, srcY, dstX, dstY);
     msg.readInt32();  // Server tick
 
-    dstBeing->setAction(Being::STAND, 0);
+    dstBeing->setAction(BeingAction::STAND, 0);
     dstBeing->setTileCoords(srcX, srcY);
     dstBeing->setDestination(dstX, dstY);
-    if (dstBeing->getType() == Being::PLAYER)
+    if (dstBeing->getType() == ActorType::PLAYER)
         dstBeing->setMoveTime();
+    BLOCK_END("BeingHandler::processBeingMove2")
 }
 
 void BeingHandler::processBeingSpawn(Net::MessageIn &msg)
 {
+    BLOCK_START("BeingHandler::processBeingSpawn")
     // skipping this packet
     mSpawnId = msg.readInt32();    // id
     msg.readInt16();    // speed
@@ -380,32 +385,40 @@ void BeingHandler::processBeingSpawn(Net::MessageIn &msg)
     msg.readInt16();  // opt2
     msg.readInt16();  // option
     msg.readInt16();    // disguise
+    BLOCK_END("BeingHandler::processBeingSpawn")
 }
 
 void BeingHandler::processBeingRemove(Net::MessageIn &msg) const
 {
-    if (!actorManager || !player_node)
+    BLOCK_START("BeingHandler::processBeingRemove")
+    if (!actorManager || !localPlayer)
+    {
+        BLOCK_END("BeingHandler::processBeingRemove")
         return;
+    }
 
     // A being should be removed or has died
 
     const int id = msg.readInt32();
     Being *const dstBeing = actorManager->findBeing(id);
     if (!dstBeing)
+    {
+        BLOCK_END("BeingHandler::processBeingRemove")
         return;
+    }
 
-    player_node->followMoveTo(dstBeing, player_node->getNextDestX(),
-        player_node->getNextDestY());
+    localPlayer->followMoveTo(dstBeing, localPlayer->getNextDestX(),
+        localPlayer->getNextDestY());
 
     // If this is player's current target, clear it.
-    if (dstBeing == player_node->getTarget())
-        player_node->stopAttack(true);
+    if (dstBeing == localPlayer->getTarget())
+        localPlayer->stopAttack(true);
 
-    if (msg.readInt8() == 1)
+    if (msg.readUInt8() == 1U)
     {
-        if (dstBeing->getCurrentAction() != Being::DEAD)
+        if (dstBeing->getCurrentAction() != BeingAction::DEAD)
         {
-            dstBeing->setAction(Being::DEAD, 0);
+            dstBeing->setAction(BeingAction::DEAD, 0);
             dstBeing->recalcSpritesOrder();
         }
         if (dstBeing->getName() == "Jack O" && killStats)
@@ -413,7 +426,7 @@ void BeingHandler::processBeingRemove(Net::MessageIn &msg) const
     }
     else
     {
-        if (dstBeing->getType() == Being::PLAYER)
+        if (dstBeing->getType() == ActorType::PLAYER)
         {
             if (botCheckerWindow)
                 botCheckerWindow->updateList();
@@ -422,33 +435,45 @@ void BeingHandler::processBeingRemove(Net::MessageIn &msg) const
         }
         actorManager->destroy(dstBeing);
     }
+    BLOCK_END("BeingHandler::processBeingRemove")
 }
 
 void BeingHandler::processBeingResurrect(Net::MessageIn &msg) const
 {
-    if (!actorManager || !player_node)
+    BLOCK_START("BeingHandler::processBeingResurrect")
+    if (!actorManager || !localPlayer)
+    {
+        BLOCK_END("BeingHandler::processBeingResurrect")
         return;
+    }
 
     // A being changed mortality status
 
     const int id = msg.readInt32();
     Being *const dstBeing = actorManager->findBeing(id);
     if (!dstBeing)
+    {
+        BLOCK_END("BeingHandler::processBeingResurrect")
         return;
+    }
 
     // If this is player's current target, clear it.
-    if (dstBeing == player_node->getTarget())
-        player_node->stopAttack();
+    if (dstBeing == localPlayer->getTarget())
+        localPlayer->stopAttack();
 
-    if (msg.readInt8() == 1)
-        dstBeing->setAction(Being::STAND, 0);
+    if (msg.readUInt8() == 1U)
+        dstBeing->setAction(BeingAction::STAND, 0);
+    BLOCK_END("BeingHandler::processBeingResurrect")
 }
 
-
 void BeingHandler::processSkillDamage(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processSkillDamage")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processSkillDamage")
         return;
+    }
 
     const int id = msg.readInt16();  // Skill Id
     Being *const srcBeing = actorManager->findBeing(msg.readInt32());
@@ -459,17 +484,22 @@ void BeingHandler::processSkillDamage(Net::MessageIn &msg) const
     const int param1 = msg.readInt32();  // Damage
     const int level = msg.readInt16();  // Skill level
     msg.readInt16();  // Div
-    msg.readInt8();  // Skill hit/type (?)
+    msg.readUInt8();  // Skill hit/type (?)
     if (srcBeing)
         srcBeing->handleSkill(dstBeing, param1, id, level);
     if (dstBeing)
         dstBeing->takeDamage(srcBeing, param1, Being::SKILL, id);
+    BLOCK_END("BeingHandler::processSkillDamage")
 }
 
 void BeingHandler::processBeingAction(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processBeingAction")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processBeingAction")
         return;
+    }
 
     Being *const srcBeing = actorManager->findBeing(msg.readInt32());
     Being *const dstBeing = actorManager->findBeing(msg.readInt32());
@@ -479,7 +509,7 @@ void BeingHandler::processBeingAction(Net::MessageIn &msg) const
     msg.readInt32();   // dst speed
     const int param1 = msg.readInt16();
     msg.readInt16();  // param 2
-    const int type = msg.readInt8();
+    const uint8_t type = msg.readUInt8();
     msg.readInt16();  // param 3
 
     switch (type)
@@ -491,11 +521,11 @@ void BeingHandler::processBeingAction(Net::MessageIn &msg) const
         case Being::FLEE:  // Lucky Dodge
             if (srcBeing)
             {
-                if (srcSpeed && srcBeing->getType() == Being::PLAYER)
+                if (srcSpeed && srcBeing->getType() == ActorType::PLAYER)
                     srcBeing->setAttackDelay(srcSpeed);
                 // attackid=1, type
                 srcBeing->handleAttack(dstBeing, param1, 1);
-                if (srcBeing->getType() == Being::PLAYER)
+                if (srcBeing->getType() == ActorType::PLAYER)
                     srcBeing->setAttackTime();
             }
             if (dstBeing)
@@ -509,17 +539,17 @@ void BeingHandler::processBeingAction(Net::MessageIn &msg) const
             break;
             // tmw server can send here garbage?
 //            if (srcBeing)
-//                srcBeing->setAction(Being::DEAD, 0);
+//                srcBeing->setAction(BeingAction::DEAD, 0);
 
         case 0x02:  // Sit
             if (srcBeing)
             {
-                srcBeing->setAction(Being::SIT, 0);
-                if (srcBeing->getType() == Being::PLAYER)
+                srcBeing->setAction(BeingAction::SIT, 0);
+                if (srcBeing->getType() == ActorType::PLAYER)
                 {
                     srcBeing->setMoveTime();
-                    if (player_node)
-                        player_node->imitateAction(srcBeing, Being::SIT);
+                    if (localPlayer)
+                        localPlayer->imitateAction(srcBeing, BeingAction::SIT);
                 }
             }
             break;
@@ -527,12 +557,15 @@ void BeingHandler::processBeingAction(Net::MessageIn &msg) const
         case 0x03:  // Stand up
             if (srcBeing)
             {
-                srcBeing->setAction(Being::STAND, 0);
-                if (srcBeing->getType() == Being::PLAYER)
+                srcBeing->setAction(BeingAction::STAND, 0);
+                if (srcBeing->getType() == ActorType::PLAYER)
                 {
                     srcBeing->setMoveTime();
-                    if (player_node)
-                        player_node->imitateAction(srcBeing, Being::STAND);
+                    if (localPlayer)
+                    {
+                        localPlayer->imitateAction(srcBeing,
+                            BeingAction::STAND);
+                    }
                 }
             }
             break;
@@ -545,17 +578,25 @@ void BeingHandler::processBeingAction(Net::MessageIn &msg) const
             logger->log("type: " + toString(type));
             break;
     }
+    BLOCK_END("BeingHandler::processBeingAction")
 }
 
 void BeingHandler::processBeingSelfEffect(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processBeingSelfEffect")
     if (!effectManager || !actorManager)
+    {
+        BLOCK_END("BeingHandler::processBeingSelfEffect")
         return;
+    }
 
     const int id = static_cast<uint32_t>(msg.readInt32());
     Being *const being = actorManager->findBeing(id);
     if (!being)
+    {
+        BLOCK_END("BeingHandler::processBeingSelfEffect")
         return;
+    }
 
     const int effectType = msg.readInt32();
 
@@ -563,48 +604,61 @@ void BeingHandler::processBeingSelfEffect(Net::MessageIn &msg) const
         effectManager->trigger(effectType, being);
 
     // +++ need dehard code effectType == 3
-    if (effectType == 3 && being->getType() == Being::PLAYER
+    if (effectType == 3 && being->getType() == ActorType::PLAYER
         && socialWindow)
     {   // reset received damage
         socialWindow->resetDamage(being->getName());
     }
+    BLOCK_END("BeingHandler::processBeingSelfEffect")
 }
 
 void BeingHandler::processBeingEmotion(Net::MessageIn &msg) const
 {
-    if (!player_node || !actorManager)
+    BLOCK_START("BeingHandler::processBeingEmotion")
+    if (!localPlayer || !actorManager)
+    {
+        BLOCK_END("BeingHandler::processBeingEmotion")
         return;
+    }
 
     Being *const dstBeing = actorManager->findBeing(msg.readInt32());
     if (!dstBeing)
+    {
+        BLOCK_END("BeingHandler::processBeingEmotion")
         return;
+    }
 
     if (player_relations.hasPermission(dstBeing, PlayerRelation::EMOTE))
     {
-        const unsigned char emote = msg.readInt8();
+        const uint8_t emote = msg.readUInt8();
         if (emote)
         {
             dstBeing->setEmote(emote, 0);
-            player_node->imitateEmote(dstBeing, emote);
+            localPlayer->imitateEmote(dstBeing, emote);
         }
     }
-    if (dstBeing->getType() == Being::PLAYER)
+    if (dstBeing->getType() == ActorType::PLAYER)
         dstBeing->setOtherTime();
+    BLOCK_END("BeingHandler::processBeingEmotion")
 }
 
 void BeingHandler::processNameResponse(Net::MessageIn &msg) const
 {
-    if (!player_node || !actorManager)
+    BLOCK_START("BeingHandler::processNameResponse")
+    if (!localPlayer || !actorManager)
+    {
+        BLOCK_END("BeingHandler::processNameResponse")
         return;
+    }
 
     const int beingId = msg.readInt32();
     Being *const dstBeing = actorManager->findBeing(beingId);
 
     if (dstBeing)
     {
-        if (beingId == player_node->getId())
+        if (beingId == localPlayer->getId())
         {
-            player_node->pingResponse();
+            localPlayer->pingResponse();
         }
         else
         {
@@ -612,12 +666,12 @@ void BeingHandler::processNameResponse(Net::MessageIn &msg) const
             dstBeing->updateGuild();
             dstBeing->addToCache();
 
-            if (dstBeing->getType() == Being::PLAYER)
+            if (dstBeing->getType() == ActorType::PLAYER)
                 dstBeing->updateColors();
 
-            if (player_node)
+            if (localPlayer)
             {
-                const Party *const party = player_node->getParty();
+                const Party *const party = localPlayer->getParty();
                 if (party && party->isMember(dstBeing->getId()))
                 {
                     PartyMember *const member = party->getMember(
@@ -626,26 +680,36 @@ void BeingHandler::processNameResponse(Net::MessageIn &msg) const
                     if (member)
                         member->setName(dstBeing->getName());
                 }
-                player_node->checkNewName(dstBeing);
+                localPlayer->checkNewName(dstBeing);
             }
         }
     }
+    BLOCK_END("BeingHandler::processNameResponse")
 }
 
 void BeingHandler::processIpResponse(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processIpResponse")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processIpResponse")
         return;
+    }
 
     Being *const dstBeing = actorManager->findBeing(msg.readInt32());
     if (dstBeing)
         dstBeing->setIp(ipToString(msg.readInt32()));
+    BLOCK_END("BeingHandler::processIpResponse")
 }
 
 void BeingHandler::processPlayerGuilPartyInfo(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processPlayerGuilPartyInfo")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processPlayerGuilPartyInfo")
         return;
+    }
 
     Being *const dstBeing = actorManager->findBeing(msg.readInt32());
 
@@ -664,46 +728,47 @@ void BeingHandler::processPlayerGuilPartyInfo(Net::MessageIn &msg) const
         dstBeing->addToCache();
         msg.readString(24);  // Discard this
     }
+    BLOCK_END("BeingHandler::processPlayerGuilPartyInfo")
 }
 
 void BeingHandler::processBeingChangeDirection(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processBeingChangeDirection")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processBeingChangeDirection")
         return;
+    }
 
     Being *const dstBeing = actorManager->findBeing(msg.readInt32());
 
     if (!dstBeing)
+    {
+        BLOCK_END("BeingHandler::processBeingChangeDirection")
         return;
+    }
 
     msg.readInt16();  // unused
 
-    const unsigned char dir = msg.readInt8() & 0x0f;
+    const uint8_t dir = static_cast<uint8_t>(msg.readUInt8() & 0x0FU);
     dstBeing->setDirection(dir);
-    if (player_node)
-        player_node->imitateDirection(dstBeing, dir);
+    if (localPlayer)
+        localPlayer->imitateDirection(dstBeing, dir);
+    BLOCK_END("BeingHandler::processBeingChangeDirection")
 }
 
 void BeingHandler::processPlayerStop(Net::MessageIn &msg) const
 {
-    if (!actorManager || !player_node)
+    BLOCK_START("BeingHandler::processPlayerStop")
+    if (!actorManager || !localPlayer)
+    {
+        BLOCK_END("BeingHandler::processPlayerStop")
         return;
-
-    /*
-      *  Instruction from server to stop walking at x, y.
-      *
-      *  Some people like having this enabled.  Others absolutely
-      *  despise it.  So I'm setting to so that it only affects the
-      *  local player if the person has set a key "EnableSync" to "1"
-      *  in their config.xml file.
-      *
-      *  This packet will be honored for all other beings, regardless
-      *  of the config setting.
-      */
+    }
 
     const int id = msg.readInt32();
 
-    if (mSync || id != player_node->getId())
+    if (mSync || id != localPlayer->getId())
     {
         Being *const dstBeing = actorManager->findBeing(id);
         if (dstBeing)
@@ -711,29 +776,36 @@ void BeingHandler::processPlayerStop(Net::MessageIn &msg) const
             const uint16_t x = msg.readInt16();
             const uint16_t y = msg.readInt16();
             dstBeing->setTileCoords(x, y);
-            if (dstBeing->getCurrentAction() == Being::MOVE)
-                dstBeing->setAction(Being::STAND, 0);
+            if (dstBeing->getCurrentAction() == BeingAction::MOVE)
+                dstBeing->setAction(BeingAction::STAND, 0);
         }
     }
+    BLOCK_END("BeingHandler::processPlayerStop")
 }
 
 void BeingHandler::processPlayerMoveToAttack(Net::MessageIn &msg A_UNUSED)
                                              const
 {
+    BLOCK_START("BeingHandler::processPlayerStop")
     /*
       * This is an *advisory* message, telling the client that
       * it needs to move the character before attacking
       * a target (out of range, obstruction in line of fire).
       * We can safely ignore this...
       */
-    if (player_node)
-        player_node->fixAttackTarget();
+    if (localPlayer)
+        localPlayer->fixAttackTarget();
+    BLOCK_END("BeingHandler::processPlayerStop")
 }
 
 void BeingHandler::processPlaterStatusChange(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processPlayerStop")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processPlayerStop")
         return;
+    }
 
     // Change in players' flags
     const int id = msg.readInt32();
@@ -744,28 +816,34 @@ void BeingHandler::processPlaterStatusChange(Net::MessageIn &msg) const
     const uint16_t stunMode = msg.readInt16();
     uint32_t statusEffects = msg.readInt16();
     statusEffects |= (static_cast<uint32_t>(msg.readInt16())) << 16;
-    msg.readInt8();  // Unused?
+    msg.readUInt8();  // Unused?
 
     dstBeing->setStunMode(stunMode);
     dstBeing->setStatusEffectBlock(0, static_cast<uint16_t>(
         (statusEffects >> 16) & 0xffff));
     dstBeing->setStatusEffectBlock(16, static_cast<uint16_t>(
         statusEffects & 0xffff));
+    BLOCK_END("BeingHandler::processPlayerStop")
 }
 
 void BeingHandler::processBeingStatusChange(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processBeingStatusChange")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processBeingStatusChange")
         return;
+    }
 
     // Status change
     const uint16_t status = msg.readInt16();
     const int id = msg.readInt32();
-    const int flag = msg.readInt8();  // 0: stop, 1: start
+    const bool flag = msg.readUInt8();  // 0: stop, 1: start
 
     Being *const dstBeing = actorManager->findBeing(id);
     if (dstBeing)
         dstBeing->setStatusEffect(status, flag);
+    BLOCK_END("BeingHandler::processBeingStatusChange")
 }
 
 void BeingHandler::processSkilCasting(Net::MessageIn &msg) const
@@ -785,22 +863,28 @@ void BeingHandler::processSkillNoDamage(Net::MessageIn &msg) const
     msg.readInt16();    // heal
     msg.readInt32();    // dst id
     msg.readInt32();    // src id
-    msg.readInt8();     // fail
+    msg.readUInt8();    // fail
 }
 
 void BeingHandler::processPvpMapMode(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processPvpMapMode")
     const Game *const game = Game::instance();
     if (!game)
+    {
+        BLOCK_END("BeingHandler::processPvpMapMode")
         return;
+    }
 
     Map *const map = game->getCurrentMap();
     if (map)
         map->setPvpMode(msg.readInt16());
+    BLOCK_END("BeingHandler::processPvpMapMode")
 }
 
 void BeingHandler::processPvpSet(Net::MessageIn &msg) const
 {
+    BLOCK_START("BeingHandler::processPvpSet")
     const int id = msg.readInt32();    // id
     const int rank = msg.readInt32();  // rank
     msg.readInt32();             // num
@@ -810,6 +894,7 @@ void BeingHandler::processPvpSet(Net::MessageIn &msg) const
         if (dstBeing)
             dstBeing->setPvpRank(rank);
     }
+    BLOCK_END("BeingHandler::processPvpSet")
 }
 
 }  // namespace Ea
