@@ -24,9 +24,9 @@
 
 #include "configuration.h"
 #include "inventory.h"
-#include "notifications.h"
 #include "notifymanager.h"
 
+#include "being/attributes.h"
 #include "being/playerinfo.h"
 
 #include "gui/windows/chatwindow.h"
@@ -39,10 +39,10 @@
 #include "net/chathandler.h"
 #include "net/net.h"
 
-#include "net/ea/eaprotocol.h"
-
 #include "utils/timer.h"
 
+#include "resources/notifytypes.h"
+
 #include "debug.h"
 
 namespace Ea
@@ -69,7 +69,7 @@ void BuySellHandler::requestSellList(const std::string &nick) const
     else
     {
         if (chatWindow)
-            chatWindow->addWhisper(nick, data, BY_PLAYER);
+            chatWindow->addWhisper(nick, data, ChatMsgType::BY_PLAYER);
     }
 }
 
@@ -88,7 +88,7 @@ void BuySellHandler::requestBuyList(const std::string &nick) const
     else
     {
         if (chatWindow)
-            chatWindow->addWhisper(nick, data, BY_PLAYER);
+            chatWindow->addWhisper(nick, data, ChatMsgType::BY_PLAYER);
     }
 }
 
@@ -107,7 +107,7 @@ void BuySellHandler::sendBuyRequest(const std::string &nick,
     if (config.getBoolValue("hideShopMessages"))
         Net::getChatHandler()->privateMessage(nick, data);
     else
-        chatWindow->addWhisper(nick, data, BY_PLAYER);
+        chatWindow->addWhisper(nick, data, ChatMsgType::BY_PLAYER);
 }
 
 void BuySellHandler::sendSellRequest(const std::string &nick,
@@ -126,7 +126,7 @@ void BuySellHandler::sendSellRequest(const std::string &nick,
     if (config.getBoolValue("hideShopMessages"))
         Net::getChatHandler()->privateMessage(nick, data);
     else
-        chatWindow->addWhisper(nick, data, BY_PLAYER);
+        chatWindow->addWhisper(nick, data, ChatMsgType::BY_PLAYER);
 }
 
 void BuySellHandler::processNpcBuySellChoice(Net::MessageIn &msg)
@@ -146,7 +146,7 @@ void BuySellHandler::processNpcSell(Net::MessageIn &msg,
     if (n_items > 0)
     {
         SellDialog *const dialog = new SellDialog(mNpcId);
-        dialog->setMoney(PlayerInfo::getAttribute(PlayerInfo::MONEY));
+        dialog->setMoney(PlayerInfo::getAttribute(Attributes::MONEY));
 
         for (int k = 0; k < n_items; k++)
         {
@@ -163,23 +163,23 @@ void BuySellHandler::processNpcSell(Net::MessageIn &msg,
     }
     else
     {
-        NotifyManager::notify(NotifyManager::SELL_LIST_EMPTY);
+        NotifyManager::notify(NotifyTypes::SELL_LIST_EMPTY);
     }
 }
 
 void BuySellHandler::processNpcBuyResponse(Net::MessageIn &msg) const
 {
-    if (msg.readInt8() == 0)
+    if (msg.readUInt8() == 0U)
     {
-        NotifyManager::notify(NotifyManager::BUY_DONE);
+        NotifyManager::notify(NotifyTypes::BUY_DONE);
     }
     else
     {
         // Reset player money since buy dialog already assumed purchase
         // would go fine
         if (mBuyDialog)
-            mBuyDialog->setMoney(PlayerInfo::getAttribute(PlayerInfo::MONEY));
-        NotifyManager::notify(NotifyManager::BUY_FAILED);
+            mBuyDialog->setMoney(PlayerInfo::getAttribute(Attributes::MONEY));
+        NotifyManager::notify(NotifyTypes::BUY_FAILED);
     }
 }
 
