@@ -24,6 +24,7 @@
 
 #include "client.h"
 #include "configuration.h"
+#include "settings.h"
 
 #include "gui/windows/charcreatedialog.h"
 #include "gui/windows/okdialog.h"
@@ -33,6 +34,8 @@
 #include "net/ea/gamehandler.h"
 #include "net/ea/network.h"
 
+#include "net/character.h"
+#include "net/messagein.h"
 #include "net/net.h"
 
 #include "utils/dtor.h"
@@ -121,7 +124,7 @@ unsigned int CharServerHandler::maxSprite() const
 void CharServerHandler::processCharLoginError(Net::MessageIn &msg) const
 {
     BLOCK_START("CharServerHandler::processCharLoginError")
-    switch (msg.readInt8())
+    switch (msg.readUInt8())
     {
         case 0:
             // TRANSLATORS: error message
@@ -163,7 +166,7 @@ void CharServerHandler::processCharCreate(Net::MessageIn &msg,
 void CharServerHandler::processCharCreateFailed(Net::MessageIn &msg)
 {
     BLOCK_START("CharServerHandler::processCharCreateFailed")
-    switch (msg.readInt8())
+    switch (msg.readUInt8())
     {
         case 1:
         case 0:
@@ -198,7 +201,7 @@ void CharServerHandler::processCharCreateFailed(Net::MessageIn &msg)
             break;
     }
     // TRANSLATORS: error message header
-    new OkDialog(_("Error"), errorMessage, DIALOG_ERROR);
+    new OkDialog(_("Error"), errorMessage, DialogType::ERROR);
     if (mCharCreateDialog)
         mCharCreateDialog->unlock();
     BLOCK_END("CharServerHandler::processCharCreateFailed")
@@ -222,7 +225,8 @@ void CharServerHandler::processCharDeleteFailed(Net::MessageIn &msg A_UNUSED)
     BLOCK_START("CharServerHandler::processCharDeleteFailed")
     unlockCharSelectDialog();
     // TRANSLATORS: error message
-    new OkDialog(_("Error"), _("Failed to delete character."), DIALOG_ERROR);
+    new OkDialog(_("Error"), _("Failed to delete character."),
+        DialogType::ERROR);
     BLOCK_END("CharServerHandler::processCharDeleteFailed")
 }
 
@@ -237,14 +241,14 @@ void CharServerHandler::processCharMapInfo(Net::MessageIn &restrict msg,
                                            ServerInfo &restrict server)
 {
     BLOCK_START("CharServerHandler::processCharMapInfo")
-//    msg.skip(4); // CharID, must be the same as player_node->charID
+//    msg.skip(4); // CharID, must be the same as localPlayer->charID
     PlayerInfo::setCharId(msg.readInt32());
     GameHandler *const gh = static_cast<GameHandler*>(Net::getGameHandler());
     gh->setMap(msg.readString(16));
-    if (config.getBoolValue("usePersistentIP"))
+    if (config.getBoolValue("usePersistentIP") || settings.persistentIp)
     {
         msg.readInt32();
-        server.hostname = client->getServerName();
+        server.hostname = settings.serverName;
     }
     else
     {
@@ -253,7 +257,7 @@ void CharServerHandler::processCharMapInfo(Net::MessageIn &restrict msg,
     server.port = msg.readInt16();
 
     // Prevent the selected local player from being deleted
-    player_node = mSelectedCharacter->dummy;
+    localPlayer = mSelectedCharacter->dummy;
     PlayerInfo::setBackend(mSelectedCharacter->data);
 
     mSelectedCharacter->dummy = nullptr;
@@ -287,10 +291,10 @@ void CharServerHandler::processChangeMapServer(Net::MessageIn &restrict msg,
 
     network->disconnect();
     client->setState(STATE_CHANGE_MAP);
-    if (player_node)
+    if (localPlayer)
     {
-        player_node->setTileCoords(x, y);
-        player_node->setMap(nullptr);
+        localPlayer->setTileCoords(x, y);
+        localPlayer->setMap(nullptr);
     }
     BLOCK_END("CharServerHandler::processChangeMapServer")
 }
