@@ -24,13 +24,18 @@
 
 #include "client.h"
 #include "game.h"
-#include "notifications.h"
 #include "notifymanager.h"
 
 #include "being/localplayer.h"
 
 #include "gui/windows/okdialog.h"
 
+#include "net/messagein.h"
+
+#include "resources/notifytypes.h"
+
+#include "utils/gettext.h"
+
 #include "debug.h"
 
 namespace Ea
@@ -60,27 +65,33 @@ void GameHandler::processMapLogin(Net::MessageIn &msg) const
     msg.skip(2);      // 0x0505
     logger->log("Protocol: Player start position: (%d, %d),"
                 " Direction: %d", x, y, direction);
+
+    mLastHost &= 0xffffff;
+
     // Switch now or we'll have problems
     client->setState(STATE_GAME);
-    if (player_node)
-        player_node->setTileCoords(x, y);
+    if (localPlayer)
+        localPlayer->setTileCoords(x, y);
 }
 
 void GameHandler::processWhoAnswer(Net::MessageIn &msg) const
 {
-    NotifyManager::notify(NotifyManager::ONLINE_USERS, msg.readInt32());
+    NotifyManager::notify(NotifyTypes::ONLINE_USERS, msg.readInt32());
 }
 
 void GameHandler::processCharSwitchResponse(Net::MessageIn &msg) const
 {
-    if (msg.readInt8())
+    if (msg.readUInt8())
         client->setState(STATE_SWITCH_CHARACTER);
 }
 
 void GameHandler::processMapQuitResponse(Net::MessageIn &msg) const
 {
-    if (msg.readInt8())
-        new OkDialog(_("Game"), _("Request to quit denied!"), DIALOG_ERROR);
+    if (msg.readUInt8())
+    {
+        new OkDialog(_("Game"), _("Request to quit denied!"),
+            DialogType::ERROR);
+    }
 }
 
 void GameHandler::clear()
