@@ -22,15 +22,22 @@
 
 #include "net/ea/inventoryhandler.h"
 
-#include "notifications.h"
 #include "notifymanager.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
+#include "being/pickup.h"
 
-#include "gui/windows/ministatuswindow.h"
+#include "net/messagein.h"
 
 #include "net/ea/eaprotocol.h"
 
+#include "utils/delete2.h"
+
+#include "listeners/arrowslistener.h"
+
+#include "resources/notifytypes.h"
+
 #include "debug.h"
 
 extern int serverVersion;
@@ -93,14 +100,12 @@ InventoryHandler::~InventoryHandler()
         mStorageWindow = nullptr;
     }
 
-    delete mStorage;
-    mStorage = nullptr;
+    delete2(mStorage);
 }
 
 void InventoryHandler::clear()
 {
-    delete mStorage;
-    mStorage = nullptr;
+    delete2(mStorage);
 }
 
 bool InventoryHandler::canSplit(const Item *const item A_UNUSED) const
@@ -147,10 +152,10 @@ int InventoryHandler::convertFromServerSlot(const int serverSlot) const
     if (serverSlot < 0 || serverSlot > 13)
         return 0;
 
-    return EQUIP_CONVERT[serverSlot];
+    return static_cast<int>(EQUIP_CONVERT[serverSlot]);
 }
 
-int InventoryHandler::getSlot(const int eAthenaSlot) const
+int InventoryHandler::getSlot(const int eAthenaSlot)
 {
     if (eAthenaSlot == 0)
         return Equipment::EQUIP_VECTOREND;
@@ -158,20 +163,21 @@ int InventoryHandler::getSlot(const int eAthenaSlot) const
     if (eAthenaSlot & 0x8000)
         return Equipment::EQUIP_PROJECTILE_SLOT;
 
-    int mask = 1;
+    unsigned int mask = 1;
     int position = 0;
     while (!(eAthenaSlot & mask))
     {
         mask <<= 1;
         position++;
     }
-    return EQUIP_POINTS[position];
+    return static_cast<int>(EQUIP_POINTS[position]);
 }
 
 void InventoryHandler::processPlayerInventory(Net::MessageIn &msg,
                                               const bool playerInvintory)
 {
-    Inventory *const inventory = player_node
+    BLOCK_START("InventoryHandler::processPlayerInventory")
+    Inventory *const inventory = localPlayer
         ? PlayerInfo::getInventory() : nullptr;
     if (playerInvintory)
     {
@@ -199,8 +205,8 @@ void InventoryHandler::processPlayerInventory(Net::MessageIn &msg,
         const int index = msg.readInt16() - (playerInvintory
             ? INVENTORY_OFFSET : STORAGE_OFFSET);
         const int itemId = msg.readInt16();
-        const int itemType = msg.readInt8();
-        unsigned char identified = msg.readInt8();
+        const uint8_t itemType = msg.readUInt8();
+        uint8_t identified = msg.readUInt8();
         const int amount = msg.readInt16();
         const int arrow = msg.readInt16();
         for (int i = 0; i < 4; i++)
@@ -234,10 +240,12 @@ void InventoryHandler::processPlayerInventory(Net::MessageIn &msg,
                 amount, 0, identified, false));
         }
     }
+    BLOCK_END("InventoryHandler::processPlayerInventory")
 }
 
 void InventoryHandler::processPlayerStorageEquip(Net::MessageIn &msg)
 {
+    BLOCK_START("InventoryHandler::processPlayerStorageEquip")
     msg.readInt16();  // length
     const int number = (msg.getLength() - 4) / 20;
 
@@ -246,36 +254,39 @@ void InventoryHandler::processPlayerStorageEquip(Net::MessageIn &msg)
         int cards[4];
         const int index = msg.readInt16() - STORAGE_OFFSET;
         const int itemId = msg.readInt16();
-        const int itemType = msg.readInt8();
-        unsigned char identified = msg.readInt8();
+        const uint8_t itemType = msg.readUInt8();
+        uint8_t identified = msg.readUInt8();
         const int amount = 1;
         msg.readInt16();    // Equip Point?
         msg.readInt16();    // Another Equip Point?
-        msg.readInt8();   // Attribute (broken)
-        const int refine = msg.readInt8();
+        msg.readUInt8();   // Attribute (broken)
+        const uint8_t refine = msg.readUInt8();
         for (int i = 0; i < 4; i++)
             cards[i] = msg.readInt16();
 
         if (mDebugInventory)
         {
-            logger->log("Index: %d, ID: %d, Type: %d, Identified: %d, "
-                        "Qty: %d, Cards: %d, %d, %d, %d, Refine: %d",
-                        index, itemId, itemType, identified, amount,
-                        cards[0], cards[1], cards[2], cards[3],
-                        refine);
+            logger->log("Index: %d, ID: %d, Type: %d, Identified: %u, "
+                "Qty: %d, Cards: %d, %d, %d, %d, Refine: %u",
+                index, itemId, itemType,
+                static_cast<unsigned int>(identified), amount,
+                cards[0], cards[1], cards[2], cards[3],
+                static_cast<unsigned int>(refine));
         }
 
-        if (serverVersion < 1 && identified > 1)
-            identified = 1;
+        if (serverVersion < 1 && identified > 1U)
+            identified = 1U;
 
         mInventoryItems.push_back(Ea::InventoryItem(index,
             itemId, amount, refine, identified, false));
     }
+    BLOCK_END("InventoryHandler::processPlayerStorageEquip")
 }
 
 void InventoryHandler::processPlayerInventoryAdd(Net::MessageIn &msg)
 {
-    Inventory *const inventory = player_node
+    BLOCK_START("InventoryHandler::processPlayerInventoryAdd")
+    Inventory *const inventory = localPlayer
         ? PlayerInfo::getInventory() : nullptr;
 
     if (PlayerInfo::getEquipment()
@@ -287,16 +298,16 @@ void InventoryHandler::processPlayerInventoryAdd(Net::MessageIn &msg)
     const int index = msg.readInt16() - INVENTORY_OFFSET;
     int amount = msg.readInt16();
     const int itemId = msg.readInt16();
-    unsigned char identified = msg.readInt8();
-    msg.readInt8();  // attribute
-    const int refine = msg.readInt8();
+    uint8_t identified = msg.readUInt8();
+    msg.readUInt8();  // attribute
+    const uint8_t refine = msg.readUInt8();
     for (int i = 0; i < 4; i++)
         msg.readInt16();  // cards[i]
     const int equipType = msg.readInt16();
-    msg.readInt8();  // itemType
+    msg.readUInt8();  // itemType
 
     const ItemInfo &itemInfo = ItemDB::get(itemId);
-    const unsigned char err = msg.readInt8();
+    const unsigned char err = msg.readUInt8();
     int floorId;
     if (mSentPickups.empty())
     {
@@ -310,15 +321,15 @@ void InventoryHandler::processPlayerInventoryAdd(Net::MessageIn &msg)
 
     if (err)
     {
-        if (player_node)
-            player_node->pickedUp(itemInfo, 0, identified, floorId, err);
+        if (localPlayer)
+            localPlayer->pickedUp(itemInfo, 0, identified, floorId, err);
     }
     else
     {
-        if (player_node)
+        if (localPlayer)
         {
-            player_node->pickedUp(itemInfo, amount,
-                identified, floorId, PICKUP_OKAY);
+            localPlayer->pickedUp(itemInfo, amount,
+                identified, floorId, Pickup::OKAY);
         }
 
         if (inventory)
@@ -334,12 +345,15 @@ void InventoryHandler::processPlayerInventoryAdd(Net::MessageIn &msg)
             inventory->setItem(index, itemId, amount, refine,
                 identified, equipType != 0);
         }
+        ArrowsListener::distributeEvent();
     }
+    BLOCK_END("InventoryHandler::processPlayerInventoryAdd")
 }
 
-void InventoryHandler::processPlayerInventoryRemove(Net::MessageIn &msg) const
+void InventoryHandler::processPlayerInventoryRemove(Net::MessageIn &msg)
 {
-    Inventory *const inventory = player_node
+    BLOCK_START("InventoryHandler::processPlayerInventoryRemove")
+    Inventory *const inventory = localPlayer
         ? PlayerInfo::getInventory() : nullptr;
 
     const int index = msg.readInt16() - INVENTORY_OFFSET;
@@ -351,22 +365,23 @@ void InventoryHandler::processPlayerInventoryRemove(Net::MessageIn &msg) const
             item->increaseQuantity(-amount);
             if (item->getQuantity() == 0)
                 inventory->removeItemAt(index);
-            if (miniStatusWindow)
-                miniStatusWindow->updateArrows();
+            ArrowsListener::distributeEvent();
         }
     }
+    BLOCK_END("InventoryHandler::processPlayerInventoryRemove")
 }
 
-void InventoryHandler::processPlayerInventoryUse(Net::MessageIn &msg) const
+void InventoryHandler::processPlayerInventoryUse(Net::MessageIn &msg)
 {
-    Inventory *const inventory = player_node
+    BLOCK_START("InventoryHandler::processPlayerInventoryUse")
+    Inventory *const inventory = localPlayer
         ? PlayerInfo::getInventory() : nullptr;
 
     const int index = msg.readInt16() - INVENTORY_OFFSET;
     msg.readInt16();  // item id
     msg.readInt32();  // id
     const int amount = msg.readInt16();
-    msg.readInt8();  // type
+    msg.readUInt8();  // type
 
     if (inventory)
     {
@@ -378,19 +393,21 @@ void InventoryHandler::processPlayerInventoryUse(Net::MessageIn &msg) const
                 inventory->removeItemAt(index);
         }
     }
+    BLOCK_END("InventoryHandler::processPlayerInventoryUse")
 }
 
-void InventoryHandler::processItemUseResponse(Net::MessageIn &msg) const
+void InventoryHandler::processItemUseResponse(Net::MessageIn &msg)
 {
-    Inventory *const inventory = player_node
+    BLOCK_START("InventoryHandler::processItemUseResponse")
+    Inventory *const inventory = localPlayer
         ? PlayerInfo::getInventory() : nullptr;
 
     const int index = msg.readInt16() - INVENTORY_OFFSET;
     const int amount = msg.readInt16();
 
-    if (msg.readInt8() == 0)
+    if (msg.readUInt8() == 0)
     {
-        NotifyManager::notify(NotifyManager::USE_FAILED);
+        NotifyManager::notify(NotifyTypes::USE_FAILED);
     }
     else
     {
@@ -405,10 +422,12 @@ void InventoryHandler::processItemUseResponse(Net::MessageIn &msg) const
             }
         }
     }
+    BLOCK_END("InventoryHandler::processItemUseResponse")
 }
 
 void InventoryHandler::processPlayerStorageStatus(Net::MessageIn &msg)
 {
+    BLOCK_START("InventoryHandler::processPlayerStorageStatus")
     /*
       * This is the closest we get to an "Open Storage" packet from the
       * server. It always comes after the two SMSG_PLAYER_STORAGE_...
@@ -432,17 +451,19 @@ void InventoryHandler::processPlayerStorageStatus(Net::MessageIn &msg)
         mStorageWindow = new InventoryWindow(mStorage);
         mStorageWindow->postInit();
     }
+    BLOCK_END("InventoryHandler::processPlayerStorageStatus")
 }
 
 void InventoryHandler::processPlayerStorageAdd(Net::MessageIn &msg)
 {
+    BLOCK_START("InventoryHandler::processPlayerStorageAdd")
     // Move an item into storage
     const int index = msg.readInt16() - STORAGE_OFFSET;
     const int amount = msg.readInt32();
     const int itemId = msg.readInt16();
-    unsigned char identified = msg.readInt8();
-    msg.readInt8();  // attribute
-    const int refine = msg.readInt8();
+    unsigned char identified = msg.readUInt8();
+    msg.readUInt8();  // attribute
+    const uint8_t refine = msg.readUInt8();
     for (int i = 0; i < 4; i++)
         msg.readInt16();  // card i
 
@@ -462,10 +483,12 @@ void InventoryHandler::processPlayerStorageAdd(Net::MessageIn &msg)
                 refine, identified, false);
         }
     }
+    BLOCK_END("InventoryHandler::processPlayerStorageAdd")
 }
 
 void InventoryHandler::processPlayerStorageRemove(Net::MessageIn &msg)
 {
+    BLOCK_START("InventoryHandler::processPlayerStorageRemove")
     // Move an item out of storage
     const int index = msg.readInt16() - STORAGE_OFFSET;
     const int amount = msg.readInt16();
@@ -478,24 +501,32 @@ void InventoryHandler::processPlayerStorageRemove(Net::MessageIn &msg)
                 mStorage->removeItemAt(index);
         }
     }
+    BLOCK_END("InventoryHandler::processPlayerStorageRemove")
 }
 
 void InventoryHandler::processPlayerStorageClose(Net::MessageIn &msg A_UNUSED)
 {
+    BLOCK_START("InventoryHandler::processPlayerStorageClose")
     // Storage access has been closed
     // Storage window deletes itself
+    if (mStorageWindow)
+    {
+        mStorageWindow->unsetInventory();
+        mStorageWindow->close();
+    }
     mStorageWindow = nullptr;
 
     if (mStorage)
         mStorage->clear();
 
-    delete mStorage;
-    mStorage = nullptr;
+    delete2(mStorage);
+    BLOCK_END("InventoryHandler::processPlayerStorageClose")
 }
 
 void InventoryHandler::processPlayerEquipment(Net::MessageIn &msg)
 {
-    Inventory *const inventory = player_node
+    BLOCK_START("InventoryHandler::processPlayerEquipment")
+    Inventory *const inventory = localPlayer
         ? PlayerInfo::getInventory() : nullptr;
 
     msg.readInt16();  // length
@@ -511,13 +542,13 @@ void InventoryHandler::processPlayerEquipment(Net::MessageIn &msg)
     {
         const int index = msg.readInt16() - INVENTORY_OFFSET;
         const int itemId = msg.readInt16();
-        const int itemType = msg.readInt8();  // type
-        unsigned char identified = msg.readInt8();  // identify flag
+        const uint8_t itemType = msg.readUInt8();  // type
+        uint8_t identified = msg.readUInt8();      // identify flag
 
         msg.readInt16();  // equip type
         const int equipType = msg.readInt16();
-        msg.readInt8();  // attribute
-        const int refine = msg.readInt8();
+        msg.readUInt8();  // attribute
+        const uint8_t refine = msg.readUInt8();
         msg.skip(8);  // card
 
         if (mDebugInventory)
@@ -538,62 +569,76 @@ void InventoryHandler::processPlayerEquipment(Net::MessageIn &msg)
         if (equipType)
             mEquips.setEquipment(getSlot(equipType), index);
     }
+    BLOCK_END("InventoryHandler::processPlayerEquipment")
 }
 
 void InventoryHandler::processPlayerEquip(Net::MessageIn &msg)
 {
+    BLOCK_START("InventoryHandler::processPlayerEquip")
     const int index = msg.readInt16() - INVENTORY_OFFSET;
     const int equipType = msg.readInt16();
-    const int flag = msg.readInt8();
+    const uint8_t flag = msg.readUInt8();
 
     if (!flag)
-        NotifyManager::notify(NotifyManager::EQUIP_FAILED);
+        NotifyManager::notify(NotifyTypes::EQUIP_FAILED);
     else
         mEquips.setEquipment(getSlot(equipType), index);
+    BLOCK_END("InventoryHandler::processPlayerEquip")
 }
 
 void InventoryHandler::processPlayerUnEquip(Net::MessageIn &msg)
 {
+    BLOCK_START("InventoryHandler::processPlayerUnEquip")
     msg.readInt16();  // inder val - INVENTORY_OFFSET;
     const int equipType = msg.readInt16();
-    const int flag = msg.readInt8();
+    const uint8_t flag = msg.readUInt8();
 
     if (flag)
         mEquips.setEquipment(getSlot(equipType), -1);
-    if (miniStatusWindow && equipType & 0x8000)
-        miniStatusWindow->updateArrows();
+    if (equipType & 0x8000)
+        ArrowsListener::distributeEvent();
+    BLOCK_END("InventoryHandler::processPlayerUnEquip")
 }
 
-void InventoryHandler::processPlayerAttackRange(Net::MessageIn &msg) const
+void InventoryHandler::processPlayerAttackRange(Net::MessageIn &msg)
 {
+    BLOCK_START("InventoryHandler::processPlayerAttackRange")
     const int range = msg.readInt16();
-    if (player_node)
-        player_node->setAttackRange(range);
-    PlayerInfo::setStatBase(PlayerInfo::ATTACK_RANGE, range);
-    PlayerInfo::setStatMod(PlayerInfo::ATTACK_RANGE, 0);
+    if (localPlayer)
+        localPlayer->setAttackRange(range);
+    PlayerInfo::setStatBase(Attributes::ATTACK_RANGE, range);
+    PlayerInfo::setStatMod(Attributes::ATTACK_RANGE, 0);
+    BLOCK_END("InventoryHandler::processPlayerAttackRange")
 }
 
 void InventoryHandler::processPlayerArrowEquip(Net::MessageIn &msg)
 {
+    BLOCK_START("InventoryHandler::processPlayerArrowEquip")
     int index = msg.readInt16();
     if (index <= 1)
         return;
 
     index -= INVENTORY_OFFSET;
-
     mEquips.setEquipment(Equipment::EQUIP_PROJECTILE_SLOT, index);
-
-    if (miniStatusWindow)
-        miniStatusWindow->updateArrows();
+    ArrowsListener::distributeEvent();
+    BLOCK_END("InventoryHandler::processPlayerArrowEquip")
 }
 
 void InventoryHandler::closeStorage()
 {
+    BLOCK_START("InventoryHandler::closeStorage")
     if (mStorageWindow)
     {
+        mStorageWindow->unsetInventory();
         mStorageWindow->close();
         mStorageWindow = nullptr;
     }
+    BLOCK_END("InventoryHandler::closeStorage")
+}
+
+void InventoryHandler::forgotStorage()
+{
+    mStorageWindow = nullptr;
 }
 
 }  // namespace Ea
