@@ -25,10 +25,17 @@
 
 #include "net/messagein.h"
 
+#include "localconsts.h"
+
+namespace Net
+{
+    class MessageIn;
+}
+
 namespace Ea
 {
 
-class ItemHandler
+class ItemHandler notfinal
 {
     public:
         ItemHandler();
@@ -37,11 +44,11 @@ class ItemHandler
 
         virtual ~ItemHandler();
 
-        void processItemVisible(Net::MessageIn &msg) const;
+        static void processItemVisible(Net::MessageIn &msg);
 
-        void processItemRemove(Net::MessageIn &msg) const;
+        static void processItemRemove(Net::MessageIn &msg);
 
-        void processItemDropped(Net::MessageIn &msg) const;
+        static void processItemDropped(Net::MessageIn &msg);
 };
 
 }  // namespace Ea
