@@ -25,6 +25,9 @@
 #include "client.h"
 #include "configuration.h"
 #include "logger.h"
+#include "settings.h"
+
+#include "being/being.h"
 
 #include "gui/windows/logindialog.h"
 
@@ -32,6 +35,8 @@
 #include "utils/gettext.h"
 #include "utils/paths.h"
 
+#include "net/messagein.h"
+
 #include "debug.h"
 
 namespace Ea
@@ -68,26 +73,22 @@ void LoginHandler::loginAccount(LoginData *const loginData1) const
 {
     if (loginData1)
     {
-        // Since we're attempting to use the tAthena protocol,
-        // let's reset the character slots to the good value,
-        // in case we just logged out a Manaserv server
-        // with a different config.
         loginData1->resetCharacterSlots();
-
         sendLoginRegister(loginData1->username, loginData1->password, "");
     }
 }
 
-void LoginHandler::chooseServer(const unsigned int server) const
+void LoginHandler::chooseServer(const unsigned int server,
+                                const bool persistentIp) const
 {
-    if (server >= mWorlds.size() || !mWorlds[server])
+    if (static_cast<size_t>(server) >= mWorlds.size() || !mWorlds[server])
         return;
 
     ServerInfo *const charServer = getCharServer();
     if (charServer)
     {
-        if (config.getBoolValue("usePersistentIP"))
-            charServer->hostname = client->getServerName();
+        if (config.getBoolValue("usePersistentIP") || persistentIp)
+            charServer->hostname = settings.serverName;
         else
             charServer->hostname = ipToString(mWorlds[server]->address);
         charServer->port = mWorlds[server]->port;
@@ -96,7 +97,7 @@ void LoginHandler::chooseServer(const unsigned int server) const
     client->setState(STATE_UPDATE);
 }
 
-void LoginHandler::registerAccount(LoginData *const loginData1) const
+void LoginHandler::registerAccount(const LoginData *const loginData1) const
 {
     if (!loginData1)
         return;
@@ -104,16 +105,16 @@ void LoginHandler::registerAccount(LoginData *const loginData1) const
     std::string username = loginData1->username;
     switch (loginData1->gender)
     {
-        case GENDER_FEMALE:
+        case Gender::FEMALE:
             username.append("_F");
             break;
-        case GENDER_MALE:
+        case Gender::MALE:
             username.append("_M");
             break;
-        case GENDER_OTHER:
+        case Gender::OTHER:
             username.append("_O");
             break;
-        case GENDER_UNSPECIFIED:
+        case Gender::UNSPECIFIED:
         default:
             break;
     }
@@ -135,7 +136,7 @@ void LoginHandler::clearWorlds()
 void LoginHandler::procecessCharPasswordResponse(Net::MessageIn &msg) const
 {
     // 0: acc not found, 1: success, 2: password mismatch, 3: pass too short
-    const int errMsg = msg.readInt8();
+    const uint8_t errMsg = msg.readUInt8();
     // Successful pass change
     if (errMsg == 1)
     {
@@ -202,7 +203,8 @@ void LoginHandler::processLoginData(Net::MessageIn &msg)
 
 //    msg.skip(30);                           // unknown
     // reserve bits for future usage
-    mToken.sex = Being::intToGender(static_cast<uint8_t>(msg.readInt8() & 3));
+    mToken.sex = Being::intToGender(static_cast<uint8_t>(
+        msg.readUInt8() & 3U));
 
     for (int i = 0; i < worldCount; i++)
     {
@@ -227,8 +229,8 @@ void LoginHandler::processLoginData(Net::MessageIn &msg)
 
 void LoginHandler::processLoginError(Net::MessageIn &msg) const
 {
-    const int code = msg.readInt8();
-    logger->log("Login::error code: %i", code);
+    const uint8_t code = msg.readUInt8();
+    logger->log("Login::error code: %u", static_cast<unsigned int>(code));
 
     switch (code)
     {
@@ -294,6 +296,30 @@ void LoginHandler::processLoginError(Net::MessageIn &msg) const
     client->setState(STATE_ERROR);
 }
 
+void LoginHandler::loginOrRegister(LoginData *const data) const
+{
+    if (!data)
+        return;
+
+    logger->log("Username is %s", data->username.c_str());
+
+    // Send login infos
+    if (data->registerLogin)
+        registerAccount(data);
+    else
+        loginAccount(data);
+
+    // Clear the password, avoids auto login when returning to login
+    data->password.clear();
+
+    const bool remember = data->remember;
+    if (remember)
+        serverConfig.setValue("username", data->username);
+    else
+        serverConfig.setValue("username", "");
+    serverConfig.setValue("remember", remember);
+}
+
 void LoginHandler::logout() const
 {
 }
