@@ -32,6 +32,8 @@
 
 #include "debug.h"
 
+extern unsigned int mLastHost;
+
 namespace Ea
 {
 
@@ -84,12 +86,11 @@ Network::~Network()
     TcpNet::quit();
 }
 
-bool Network::connect(ServerInfo server)
+bool Network::connect(const ServerInfo &server)
 {
     if (mState != IDLE && mState != NET_ERROR)
     {
         logger->log1("Tried to connect an already connected socket!");
-//        assert(false);
         return false;
     }
 
@@ -175,7 +176,7 @@ void Network::skip(const int len)
     if (mInSize >= mToSkip)
     {
         mInSize -= mToSkip;
-        memmove(mInBuffer, mInBuffer + mToSkip, mInSize);
+        memmove(mInBuffer, mInBuffer + static_cast<size_t>(mToSkip), mInSize);
         mToSkip = 0;
     }
     else
@@ -217,6 +218,7 @@ bool Network::realConnect()
         return false;
     }
 
+    mLastHost = ipAddress.host;
     logger->log_r("Network::Started session with %s:%i",
         ipToString(ipAddress.host), ipAddress.port);
 
@@ -265,7 +267,8 @@ void Network::receive()
                     continue;
                 }
 
-                const int ret = TcpNet::recv(mSocket, mInBuffer + mInSize,
+                const int ret = TcpNet::recv(mSocket,
+                    mInBuffer + static_cast<size_t>(mInSize),
                     BUFFER_SIZE - mInSize);
 
                 if (!ret)
@@ -289,7 +292,9 @@ void Network::receive()
                         if (mInSize >= mToSkip)
                         {
                             mInSize -= mToSkip;
-                            memmove(mInBuffer, mInBuffer + mToSkip, mInSize);
+                            memmove(mInBuffer,
+                                mInBuffer + static_cast<size_t>(mToSkip),
+                                mInSize);
                             mToSkip = 0;
                         }
                         else
@@ -330,9 +335,11 @@ void Network::setError(const std::string &error)
 uint16_t Network::readWord(const int pos) const
 {
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
-    return SDL_Swap16(*reinterpret_cast<uint16_t*>(mInBuffer + (pos)));
+    return SDL_Swap16(*reinterpret_cast<uint16_t*>(
+        mInBuffer + static_cast<size_t>(pos)));
 #else
-    return (*reinterpret_cast<uint16_t*>(mInBuffer + (pos)));
+    return (*reinterpret_cast<uint16_t*>(
+        mInBuffer + static_cast<size_t>(pos)));
 #endif
 }
 
