@@ -25,30 +25,35 @@
 #include "configuration.h"
 #include "game.h"
 #include "party.h"
-#include "notifications.h"
 #include "notifymanager.h"
 #include "units.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
 
 #include "gui/viewport.h"
 
-#include "gui/windows/ministatuswindow.h"
 #include "gui/windows/okdialog.h"
-#include "gui/windows/npcdialog.h"
 #include "gui/windows/skilldialog.h"
 #include "gui/windows/statuswindow.h"
 
+#include "resources/notifytypes.h"
+
 #include "resources/db/deaddb.h"
 
-#include "net/net.h"
+#include "resources/map/map.h"
+
+#include "listeners/playerdeathlistener.h"
+#include "listeners/weightlistener.h"
+
+#include "net/messagein.h"
 
 #include "net/ea/eaprotocol.h"
 
+#include "utils/gettext.h"
+
 #include "debug.h"
 
-extern OkDialog *weightNotice;
-extern OkDialog *deathNotice;
 extern int weightNoticeTime;
 
 // Max. distance we are willing to scroll after a teleport;
@@ -57,39 +62,8 @@ static const int MAP_TELEPORT_SCROLL_DISTANCE = 8;
 
 namespace
 {
-    /**
-     * Listener used for handling the overweigth message.
-     */
-    struct WeightListener final : public gcn::ActionListener
-    {
-        void action(const gcn::ActionEvent &event A_UNUSED)
-        {
-            weightNotice = nullptr;
-        }
-    } weightListener;
-
-    /**
-     * Listener used for handling death message.
-     */
-    struct DeathListener final : public gcn::ActionListener
-    {
-        void action(const gcn::ActionEvent &event A_UNUSED)
-        {
-            if (Net::getPlayerHandler())
-                Net::getPlayerHandler()->respawn();
-            deathNotice = nullptr;
-
-            Game::closeDialogs();
-
-            if (viewport)
-                viewport->closePopupMenu();
-
-            NpcDialog::clearDialogs();
-            if (player_node)
-                player_node->respawn();
-        }
-    } deathListener;
-
+    WeightListener weightListener;
+    PlayerDeathListener deathListener;
 }  // anonymous namespace
 
 namespace Ea
@@ -124,7 +98,7 @@ Vector PlayerHandler::getDefaultWalkSpeed() const
     return Vector(150, 150, 0);
 }
 
-void PlayerHandler::processWalkResponse(Net::MessageIn &msg) const
+void PlayerHandler::processWalkResponse(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processWalkResponse")
     /*
@@ -135,12 +109,12 @@ void PlayerHandler::processWalkResponse(Net::MessageIn &msg) const
     uint16_t srcX, srcY, dstX, dstY;
     msg.readInt32();  // tick
     msg.readCoordinatePair(srcX, srcY, dstX, dstY);
-    if (player_node)
-        player_node->setRealPos(dstX, dstY);
+    if (localPlayer)
+        localPlayer->setRealPos(dstX, dstY);
     BLOCK_END("PlayerHandler::processWalkResponse")
 }
 
-void PlayerHandler::processPlayerWarp(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerWarp(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processPlayerWarp")
     std::string mapPath = msg.readString(16);
@@ -149,15 +123,15 @@ void PlayerHandler::processPlayerWarp(Net::MessageIn &msg) const
 
     logger->log("Warping to %s (%d, %d)", mapPath.c_str(), x, y);
 
-    if (!player_node)
-        logger->log1("SMSG_PLAYER_WARP player_node null");
+    if (!localPlayer)
+        logger->log1("SMSG_PLAYER_WARP localPlayer null");
 
     /*
       * We must clear the local player's target *before* the call
       * to changeMap, as it deletes all beings.
       */
-    if (player_node)
-        player_node->stopAttack();
+    if (localPlayer)
+        localPlayer->stopAttack();
 
     Game *const game = Game::instance();
 
@@ -171,7 +145,7 @@ void PlayerHandler::processPlayerWarp(Net::MessageIn &msg) const
     int scrollOffsetX = 0;
     int scrollOffsetY = 0;
 
-    if (player_node)
+    if (localPlayer)
     {
         const Map *const map = game->getCurrentMap();
         if (map)
@@ -186,22 +160,22 @@ void PlayerHandler::processPlayerWarp(Net::MessageIn &msg) const
                 y = 0;
             /* Scroll if neccessary */
             if (!sameMap
-                || (abs(x - player_node->getTileX())
+                || (abs(x - localPlayer->getTileX())
                 > MAP_TELEPORT_SCROLL_DISTANCE)
-                || (abs(y - player_node->getTileY())
+                || (abs(y - localPlayer->getTileY())
                 > MAP_TELEPORT_SCROLL_DISTANCE))
             {
-                scrollOffsetX = (x - player_node->getTileX())
+                scrollOffsetX = (x - localPlayer->getTileX())
                     * map->getTileWidth();
-                scrollOffsetY = (y - player_node->getTileY())
+                scrollOffsetY = (y - localPlayer->getTileY())
                     * map->getTileHeight();
             }
         }
 
-        player_node->setAction(Being::STAND, 0);
-        player_node->setTileCoords(x, y);
-        player_node->updatePets();
-        player_node->navigateClean();
+        localPlayer->setAction(BeingAction::STAND, 0);
+        localPlayer->setTileCoords(x, y);
+        localPlayer->updatePets();
+        localPlayer->navigateClean();
     }
 
     logger->log("Adjust scrolling by %d:%d", scrollOffsetX, scrollOffsetY);
@@ -209,19 +183,17 @@ void PlayerHandler::processPlayerWarp(Net::MessageIn &msg) const
     if (viewport)
     {
         viewport->returnCamera();
-        if (miniStatusWindow)
-            miniStatusWindow->updateStatus();
         viewport->scrollBy(scrollOffsetX, scrollOffsetY);
     }
     BLOCK_END("PlayerHandler::processPlayerWarp")
 }
 
-void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processPlayerStatUpdate1")
     const int type = msg.readInt16();
     const int value = msg.readInt32();
-    if (!player_node)
+    if (!localPlayer)
     {
         BLOCK_END("PlayerHandler::processPlayerStatUpdate1")
         return;
@@ -230,59 +202,59 @@ void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg) const
     switch (type)
     {
         case 0x0000:
-            player_node->setWalkSpeed(Vector(static_cast<float>(
+            localPlayer->setWalkSpeed(Vector(static_cast<float>(
                 value), static_cast<float>(value), 0));
-            PlayerInfo::setStatBase(PlayerInfo::WALK_SPEED, value);
-            PlayerInfo::setStatMod(PlayerInfo::WALK_SPEED, 0);
+            PlayerInfo::setStatBase(Attributes::WALK_SPEED, value);
+            PlayerInfo::setStatMod(Attributes::WALK_SPEED, 0);
         break;
         case 0x0004:
             break;  // manner
         case 0x0005:
-            PlayerInfo::setAttribute(PlayerInfo::HP, value);
-            if (player_node->isInParty() && Party::getParty(1))
+            PlayerInfo::setAttribute(Attributes::HP, value);
+            if (localPlayer->isInParty() && Party::getParty(1))
             {
                 PartyMember *const m = Party::getParty(1)
-                    ->getMember(player_node->getId());
+                    ->getMember(localPlayer->getId());
                 if (m)
                 {
                     m->setHp(value);
-                    m->setMaxHp(PlayerInfo::getAttribute(PlayerInfo::MAX_HP));
+                    m->setMaxHp(PlayerInfo::getAttribute(Attributes::MAX_HP));
                 }
             }
             break;
         case 0x0006:
-            PlayerInfo::setAttribute(PlayerInfo::MAX_HP, value);
+            PlayerInfo::setAttribute(Attributes::MAX_HP, value);
 
-            if (player_node->isInParty() && Party::getParty(1))
+            if (localPlayer->isInParty() && Party::getParty(1))
             {
                 PartyMember *const m = Party::getParty(1)->getMember(
-                    player_node->getId());
+                    localPlayer->getId());
                 if (m)
                 {
-                    m->setHp(PlayerInfo::getAttribute(PlayerInfo::HP));
+                    m->setHp(PlayerInfo::getAttribute(Attributes::HP));
                     m->setMaxHp(value);
                 }
             }
             break;
         case 0x0007:
-            PlayerInfo::setAttribute(PlayerInfo::MP, value);
+            PlayerInfo::setAttribute(Attributes::MP, value);
             break;
         case 0x0008:
-            PlayerInfo::setAttribute(PlayerInfo::MAX_MP, value);
+            PlayerInfo::setAttribute(Attributes::MAX_MP, value);
             break;
         case 0x0009:
-            PlayerInfo::setAttribute(PlayerInfo::CHAR_POINTS, value);
+            PlayerInfo::setAttribute(Attributes::CHAR_POINTS, value);
             break;
         case 0x000b:
-            PlayerInfo::setAttribute(PlayerInfo::LEVEL, value);
-            if (player_node)
+            PlayerInfo::setAttribute(Attributes::LEVEL, value);
+            if (localPlayer)
             {
-                player_node->setLevel(value);
-                player_node->updateName();
+                localPlayer->setLevel(value);
+                localPlayer->updateName();
             }
             break;
         case 0x000c:
-            PlayerInfo::setAttribute(PlayerInfo::SKILL_POINTS, value);
+            PlayerInfo::setAttribute(Attributes::SKILL_POINTS, value);
             if (skillDialog)
                 skillDialog->update();
             break;
@@ -290,9 +262,9 @@ void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg) const
             if (!weightNotice && config.getBoolValue("weightMsg"))
             {
                 const int max = PlayerInfo::getAttribute(
-                    PlayerInfo::MAX_WEIGHT) / 2;
+                    Attributes::MAX_WEIGHT) / 2;
                 const int total = PlayerInfo::getAttribute(
-                    PlayerInfo::TOTAL_WEIGHT);
+                    Attributes::TOTAL_WEIGHT);
                 if (value >= max && total < max)
                 {
                     weightNoticeTime = cur_time + 5;
@@ -301,7 +273,7 @@ void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg) const
                         // TRANSLATORS: weight message
                         _("You are carrying more than "
                         "half your weight. You are "
-                        "unable to regain health."), DIALOG_OK, false);
+                        "unable to regain health."), DialogType::OK, false);
                     weightNotice->addActionListener(
                         &weightListener);
                 }
@@ -313,15 +285,15 @@ void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg) const
                         // TRANSLATORS: weight message
                         _("You are carrying less than "
                         "half your weight. You "
-                        "can regain health."), DIALOG_OK, false);
+                        "can regain health."), DialogType::OK, false);
                     weightNotice->addActionListener(
                         &weightListener);
                 }
             }
-            PlayerInfo::setAttribute(PlayerInfo::TOTAL_WEIGHT, value);
+            PlayerInfo::setAttribute(Attributes::TOTAL_WEIGHT, value);
             break;
         case 0x0019:
-            PlayerInfo::setAttribute(PlayerInfo::MAX_WEIGHT, value);
+            PlayerInfo::setAttribute(Attributes::MAX_WEIGHT, value);
             break;
 
         case 0x0029:
@@ -370,9 +342,9 @@ void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg) const
             break;
 
         case 0x0035:
-            player_node->setAttackSpeed(value);
-            PlayerInfo::setStatBase(PlayerInfo::ATTACK_DELAY, value);
-            PlayerInfo::setStatMod(PlayerInfo::ATTACK_DELAY, 0);
+            localPlayer->setAttackSpeed(value);
+            PlayerInfo::setStatBase(Attributes::ATTACK_DELAY, value);
+            PlayerInfo::setStatMod(Attributes::ATTACK_DELAY, 0);
             PlayerInfo::updateAttrs();
             break;
 
@@ -381,7 +353,7 @@ void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg) const
             break;
 
         case 500:
-            player_node->setGMLevel(value);
+            localPlayer->setGMLevel(value);
             break;
 
         default:
@@ -390,29 +362,29 @@ void PlayerHandler::processPlayerStatUpdate1(Net::MessageIn &msg) const
             break;
     }
 
-    if (PlayerInfo::getAttribute(PlayerInfo::HP) == 0 && !deathNotice)
+    if (PlayerInfo::getAttribute(Attributes::HP) == 0 && !deathNotice)
     {
         // TRANSLATORS: message header
         deathNotice = new OkDialog(_("Message"),
-            DeadDB::getRandomString(), DIALOG_OK, false);
+            DeadDB::getRandomString(), DialogType::OK, false);
         deathNotice->addActionListener(&deathListener);
-        if (player_node->getCurrentAction() != Being::DEAD)
+        if (localPlayer->getCurrentAction() != BeingAction::DEAD)
         {
-            player_node->setAction(Being::DEAD, 0);
-            player_node->recalcSpritesOrder();
+            localPlayer->setAction(BeingAction::DEAD, 0);
+            localPlayer->recalcSpritesOrder();
         }
     }
     BLOCK_END("PlayerHandler::processPlayerStatUpdate1")
 }
 
-void PlayerHandler::processPlayerStatUpdate2(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerStatUpdate2(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processPlayerStatUpdate2")
     const int type = msg.readInt16();
     switch (type)
     {
         case 0x0001:
-            PlayerInfo::setAttribute(PlayerInfo::EXP, msg.readInt32());
+            PlayerInfo::setAttribute(Attributes::EXP, msg.readInt32());
             break;
         case 0x0002:
             PlayerInfo::setStatExperience(EA_JOB, msg.readInt32(),
@@ -420,24 +392,24 @@ void PlayerHandler::processPlayerStatUpdate2(Net::MessageIn &msg) const
             break;
         case 0x0014:
         {
-            const int oldMoney = PlayerInfo::getAttribute(PlayerInfo::MONEY);
+            const int oldMoney = PlayerInfo::getAttribute(Attributes::MONEY);
             const int newMoney = msg.readInt32();
             if (newMoney > oldMoney)
             {
-                NotifyManager::notify(NotifyManager::MONEY_GET,
+                NotifyManager::notify(NotifyTypes::MONEY_GET,
                     Units::formatCurrency(newMoney - oldMoney));
             }
             else if (newMoney < oldMoney)
             {
-                NotifyManager::notify(NotifyManager::MONEY_SPENT,
+                NotifyManager::notify(NotifyTypes::MONEY_SPENT,
                     Units::formatCurrency(oldMoney - newMoney).c_str());
             }
 
-            PlayerInfo::setAttribute(PlayerInfo::MONEY, newMoney);
+            PlayerInfo::setAttribute(Attributes::MONEY, newMoney);
             break;
         }
         case 0x0016:
-            PlayerInfo::setAttribute(PlayerInfo::EXP_NEEDED, msg.readInt32());
+            PlayerInfo::setAttribute(Attributes::EXP_NEEDED, msg.readInt32());
             break;
         case 0x0017:
             PlayerInfo::setStatExperience(EA_JOB,
@@ -450,7 +422,7 @@ void PlayerHandler::processPlayerStatUpdate2(Net::MessageIn &msg) const
     BLOCK_END("PlayerHandler::processPlayerStatUpdate2")
 }
 
-void PlayerHandler::processPlayerStatUpdate3(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerStatUpdate3(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processPlayerStatUpdate3")
     const int type = msg.readInt32();
@@ -459,77 +431,77 @@ void PlayerHandler::processPlayerStatUpdate3(Net::MessageIn &msg) const
 
     PlayerInfo::setStatBase(type, base, false);
     PlayerInfo::setStatMod(type, bonus);
-    if (type == EA_ATK || type == PlayerInfo::ATTACK_DELAY)
+    if (type == EA_ATK || type == Attributes::ATTACK_DELAY)
         PlayerInfo::updateAttrs();
     BLOCK_END("PlayerHandler::processPlayerStatUpdate3")
 }
 
-void PlayerHandler::processPlayerStatUpdate4(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerStatUpdate4(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processPlayerStatUpdate4")
     const int type = msg.readInt16();
-    const int ok = msg.readInt8();
-    const int value = msg.readInt8();
+    const uint8_t ok = msg.readUInt8();
+    const int value = msg.readUInt8();
 
     if (ok != 1)
     {
         const int oldValue = PlayerInfo::getStatBase(type);
-        const int points = PlayerInfo::getAttribute(PlayerInfo::CHAR_POINTS)
+        const int points = PlayerInfo::getAttribute(Attributes::CHAR_POINTS)
             + oldValue - value;
-        PlayerInfo::setAttribute(PlayerInfo::CHAR_POINTS, points);
-        NotifyManager::notify(NotifyManager::SKILL_RAISE_ERROR);
+        PlayerInfo::setAttribute(Attributes::CHAR_POINTS, points);
+        NotifyManager::notify(NotifyTypes::SKILL_RAISE_ERROR);
     }
 
     PlayerInfo::setStatBase(type, value);
     BLOCK_END("PlayerHandler::processPlayerStatUpdate4")
 }
 
-void PlayerHandler::processPlayerStatUpdate5(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerStatUpdate5(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processPlayerStatUpdate5")
-    PlayerInfo::setAttribute(PlayerInfo::CHAR_POINTS, msg.readInt16());
+    PlayerInfo::setAttribute(Attributes::CHAR_POINTS, msg.readInt16());
 
-    int val = msg.readInt8();
+    unsigned int val = msg.readUInt8();
     PlayerInfo::setStatBase(EA_STR, val);
     if (statusWindow)
-        statusWindow->setPointsNeeded(EA_STR, msg.readInt8());
+        statusWindow->setPointsNeeded(EA_STR, msg.readUInt8());
     else
-        msg.readInt8();
+        msg.readUInt8();
 
-    val = msg.readInt8();
+    val = msg.readUInt8();
     PlayerInfo::setStatBase(EA_AGI, val);
     if (statusWindow)
-        statusWindow->setPointsNeeded(EA_AGI, msg.readInt8());
+        statusWindow->setPointsNeeded(EA_AGI, msg.readUInt8());
     else
-        msg.readInt8();
+        msg.readUInt8();
 
-    val = msg.readInt8();
+    val = msg.readUInt8();
     PlayerInfo::setStatBase(EA_VIT, val);
     if (statusWindow)
-        statusWindow->setPointsNeeded(EA_VIT, msg.readInt8());
+        statusWindow->setPointsNeeded(EA_VIT, msg.readUInt8());
     else
-        msg.readInt8();
+        msg.readUInt8();
 
-    val = msg.readInt8();
+    val = msg.readUInt8();
     PlayerInfo::setStatBase(EA_INT, val);
     if (statusWindow)
-        statusWindow->setPointsNeeded(EA_INT, msg.readInt8());
+        statusWindow->setPointsNeeded(EA_INT, msg.readUInt8());
     else
-        msg.readInt8();
+        msg.readUInt8();
 
-    val = msg.readInt8();
+    val = msg.readUInt8();
     PlayerInfo::setStatBase(EA_DEX, val);
     if (statusWindow)
-        statusWindow->setPointsNeeded(EA_DEX, msg.readInt8());
+        statusWindow->setPointsNeeded(EA_DEX, msg.readUInt8());
     else
-        msg.readInt8();
+        msg.readUInt8();
 
-    val = msg.readInt8();
+    val = msg.readUInt8();
     PlayerInfo::setStatBase(EA_LUK, val);
     if (statusWindow)
-        statusWindow->setPointsNeeded(EA_LUK, msg.readInt8());
+        statusWindow->setPointsNeeded(EA_LUK, msg.readUInt8());
     else
-        msg.readInt8();
+        msg.readUInt8();
 
     PlayerInfo::setStatBase(EA_ATK, msg.readInt16(), false);
     PlayerInfo::setStatMod(EA_ATK, msg.readInt16());
@@ -558,7 +530,7 @@ void PlayerHandler::processPlayerStatUpdate5(Net::MessageIn &msg) const
     BLOCK_END("PlayerHandler::processPlayerStatUpdate5")
 }
 
-void PlayerHandler::processPlayerStatUpdate6(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerStatUpdate6(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processPlayerStatUpdate6")
     const int type = msg.readInt16();
@@ -567,22 +539,22 @@ void PlayerHandler::processPlayerStatUpdate6(Net::MessageIn &msg) const
         switch (type)
         {
             case 0x0020:
-                statusWindow->setPointsNeeded(EA_STR, msg.readInt8());
+                statusWindow->setPointsNeeded(EA_STR, msg.readUInt8());
                 break;
             case 0x0021:
-                statusWindow->setPointsNeeded(EA_AGI, msg.readInt8());
+                statusWindow->setPointsNeeded(EA_AGI, msg.readUInt8());
                 break;
             case 0x0022:
-                statusWindow->setPointsNeeded(EA_VIT, msg.readInt8());
+                statusWindow->setPointsNeeded(EA_VIT, msg.readUInt8());
                 break;
             case 0x0023:
-                statusWindow->setPointsNeeded(EA_INT, msg.readInt8());
+                statusWindow->setPointsNeeded(EA_INT, msg.readUInt8());
                 break;
             case 0x0024:
-                statusWindow->setPointsNeeded(EA_DEX, msg.readInt8());
+                statusWindow->setPointsNeeded(EA_DEX, msg.readUInt8());
                 break;
             case 0x0025:
-                statusWindow->setPointsNeeded(EA_LUK, msg.readInt8());
+                statusWindow->setPointsNeeded(EA_LUK, msg.readUInt8());
                 break;
             default:
                 logger->log("QQQQ PLAYER_STAT_UPDATE_6 "
@@ -593,14 +565,14 @@ void PlayerHandler::processPlayerStatUpdate6(Net::MessageIn &msg) const
     BLOCK_END("PlayerHandler::processPlayerStatUpdate6")
 }
 
-void PlayerHandler::processPlayerArrowMessage(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerArrowMessage(Net::MessageIn &msg)
 {
     BLOCK_START("PlayerHandler::processPlayerArrowMessage")
     const int type = msg.readInt16();
     switch (type)
     {
         case 0:
-            NotifyManager::notify(NotifyManager::ARROWS_EQUIP_NEEDED);
+            NotifyManager::notify(NotifyTypes::ARROWS_EQUIP_NEEDED);
             break;
         case 3:
             // arrows equiped
