@@ -23,7 +23,6 @@
 #include "net/ea/skillhandler.h"
 
 #include "logger.h"
-#include "notifications.h"
 #include "notifymanager.h"
 
 #include "being/localplayer.h"
@@ -31,8 +30,13 @@
 
 #include "gui/windows/skilldialog.h"
 
+#include "utils/gettext.h"
 #include "utils/stringutils.h"
 
+#include "net/messagein.h"
+
+#include "resources/notifytypes.h"
+
 #include "debug.h"
 
 /** job dependend identifiers (?)  */
@@ -71,7 +75,7 @@ SkillHandler::SkillHandler()
 {
 }
 
-void SkillHandler::processPlayerSkills(Net::MessageIn &msg) const
+void SkillHandler::processPlayerSkills(Net::MessageIn &msg)
 {
     msg.readInt16();  // length
     const int skillCount = (msg.getLength() - 4) / 37;
@@ -86,7 +90,7 @@ void SkillHandler::processPlayerSkills(Net::MessageIn &msg) const
         msg.readInt16();  // sp
         const int range = msg.readInt16();
         msg.skip(24);  // 0 unused
-        const int up = msg.readInt8();
+        const int up = msg.readUInt8();
         const int oldLevel = PlayerInfo::getSkillLevel(skillId);
         if (oldLevel && oldLevel != level)
             updateSkill = skillId;
@@ -101,13 +105,13 @@ void SkillHandler::processPlayerSkills(Net::MessageIn &msg) const
         skillDialog->playUpdateEffect(updateSkill);
 }
 
-void SkillHandler::processPlayerSkillUp(Net::MessageIn &msg) const
+void SkillHandler::processPlayerSkillUp(Net::MessageIn &msg)
 {
     const int skillId = msg.readInt16();
     const int level = msg.readInt16();
     msg.readInt16();  // sp
     const int range = msg.readInt16();
-    const int up = msg.readInt8();
+    const int up = msg.readUInt8();
 
     if (skillDialog && PlayerInfo::getSkillLevel(skillId) != level)
         skillDialog->playUpdateEffect(skillId);
@@ -119,15 +123,15 @@ void SkillHandler::processPlayerSkillUp(Net::MessageIn &msg) const
     }
 }
 
-void SkillHandler::processSkillFailed(Net::MessageIn &msg) const
+void SkillHandler::processSkillFailed(Net::MessageIn &msg)
 {
     // Action failed (ex. sit because you have not reached the
     // right level)
     const int skillId   = msg.readInt16();
     const int16_t bskill  = msg.readInt16();
     msg.readInt16();  // btype
-    const signed char success = msg.readInt8();
-    const signed char reason  = msg.readInt8();
+    const signed char success = msg.readUInt8();
+    const signed char reason  = msg.readUInt8();
     if (success != static_cast<int>(SKILL_FAILED)
         && bskill == static_cast<int>(BSKILL_EMOTE))
     {
@@ -138,10 +142,10 @@ void SkillHandler::processSkillFailed(Net::MessageIn &msg) const
     if (success == static_cast<int>(SKILL_FAILED)
         && skillId == static_cast<int>(SKILL_BASIC))
     {
-        if (player_node && bskill == static_cast<int>(BSKILL_EMOTE)
+        if (localPlayer && bskill == static_cast<int>(BSKILL_EMOTE)
             && reason == static_cast<int>(RFAIL_SKILLDEP))
         {
-            player_node->stopAdvert();
+            localPlayer->stopAdvert();
         }
 
         switch (bskill)
@@ -252,6 +256,6 @@ void SkillHandler::processSkillFailed(Net::MessageIn &msg) const
         }
     }
 
-    NotifyManager::notify(NotifyManager::SKILL_FAIL_MESSAGE, txt);
+    NotifyManager::notify(NotifyTypes::SKILL_FAIL_MESSAGE, txt);
 }
 }  // namespace Ea
