@@ -23,24 +23,30 @@
 #ifndef NET_EA_SKILLHANDLER_H
 #define NET_EA_SKILLHANDLER_H
 
-#include "net/messagein.h"
 #include "net/skillhandler.h"
 
+#include "localconsts.h"
+
+namespace Net
+{
+    class MessageIn;
+}
+
 namespace Ea
 {
 
-class SkillHandler : public Net::SkillHandler
+class SkillHandler notfinal : public Net::SkillHandler
 {
     public:
         SkillHandler();
 
         A_DELETE_COPY(SkillHandler)
 
-        void processPlayerSkills(Net::MessageIn &msg) const;
+        static void processPlayerSkills(Net::MessageIn &msg);
 
-        void processPlayerSkillUp(Net::MessageIn &msg) const;
+        static void processPlayerSkillUp(Net::MessageIn &msg);
 
-        void processSkillFailed(Net::MessageIn &msg) const;
+        static void processSkillFailed(Net::MessageIn &msg);
 };
 
 }  // namespace Ea
