@@ -25,7 +25,6 @@
 #include "inventory.h"
 #include "item.h"
 #include "logger.h"
-#include "notifications.h"
 #include "notifymanager.h"
 
 #include "being/playerinfo.h"
@@ -34,10 +33,16 @@
 #include "gui/windows/confirmdialog.h"
 #include "gui/windows/tradewindow.h"
 
-#include "net/net.h"
+#include "net/messagein.h"
 
 #include "net/ea/eaprotocol.h"
 
+#include "utils/gettext.h"
+
+#include "resources/notifytypes.h"
+
+#include "listeners/requesttradelistener.h"
+
 #include "debug.h"
 
 extern std::string tradePartnerName;
@@ -48,17 +53,7 @@ ConfirmDialog *confirmDlg = nullptr;
  */
 namespace
 {
-    struct RequestTradeListener : public gcn::ActionListener
-    {
-        void action(const gcn::ActionEvent &event)
-        {
-            confirmDlg = nullptr;
-            const std::string &eventId = event.getId();
-            if (eventId == "ignore")
-                player_relations.ignoreTrade(tradePartnerName);
-            Net::getTradeHandler()->respond(eventId == "yes");
-        }
-    } listener;
+    RequestTradeListener listener;
 }  // namespace
 
 namespace Ea
@@ -133,18 +128,18 @@ void TradeHandler::processTradeResponse(Net::MessageIn &msg) const
         return;
     }
 
-    switch (msg.readInt8())
+    switch (msg.readUInt8())
     {
         case 0:  // Too far away
-            NotifyManager::notify(NotifyManager::TRADE_FAIL_FAR_AWAY,
+            NotifyManager::notify(NotifyTypes::TRADE_FAIL_FAR_AWAY,
                 tradePartnerName);
             break;
         case 1:  // Character doesn't exist
-            NotifyManager::notify(NotifyManager::TRADE_FAIL_CHAR_NOT_EXISTS,
+            NotifyManager::notify(NotifyTypes::TRADE_FAIL_CHAR_NOT_EXISTS,
                 tradePartnerName);
             break;
         case 2:  // Invite request check failed...
-            NotifyManager::notify(NotifyManager::TRADE_CANCELLED_ERROR);
+            NotifyManager::notify(NotifyTypes::TRADE_CANCELLED_ERROR);
             break;
         case 3:  // Trade accepted
             if (tradeWindow)
@@ -161,7 +156,7 @@ void TradeHandler::processTradeResponse(Net::MessageIn &msg) const
             if (player_relations.hasPermission(tradePartnerName,
                 PlayerRelation::SPEECH_LOG))
             {
-                NotifyManager::notify(NotifyManager::TRADE_CANCELLED_NAME,
+                NotifyManager::notify(NotifyTypes::TRADE_CANCELLED_NAME,
                     tradePartnerName);
             }
             // otherwise ignore silently
@@ -174,7 +169,7 @@ void TradeHandler::processTradeResponse(Net::MessageIn &msg) const
             PlayerInfo::setTrading(false);
             break;
         default:  // Shouldn't happen as well, but to be sure
-            NotifyManager::notify(NotifyManager::TRADE_ERROR_UNKNOWN,
+            NotifyManager::notify(NotifyTypes::TRADE_ERROR_UNKNOWN,
                 tradePartnerName);
             if (tradeWindow)
                 tradeWindow->clear();
@@ -182,14 +177,14 @@ void TradeHandler::processTradeResponse(Net::MessageIn &msg) const
     }
 }
 
-void TradeHandler::processTradeItemAdd(Net::MessageIn &msg) const
+void TradeHandler::processTradeItemAdd(Net::MessageIn &msg)
 {
     const int amount = msg.readInt32();
     const int type = msg.readInt16();
-    const int identify = msg.readInt8();  // identified flag
-    msg.readInt8();  // attribute
-    const int refine = msg.readInt8();  // refine
-    msg.skip(8);     // card (4 shorts)
+    const uint8_t identify = msg.readUInt8();  // identified flag
+    msg.readUInt8();  // attribute
+    const uint8_t refine = msg.readUInt8();  // refine
+    msg.skip(8);      // card (4 shorts)
 
     if (tradeWindow)
     {
@@ -199,13 +194,13 @@ void TradeHandler::processTradeItemAdd(Net::MessageIn &msg) const
         }
         else
         {
-            tradeWindow->addItem2(type, false, amount, refine,
-                static_cast<unsigned char>(identify), false);
+            tradeWindow->addItem2(type, false, amount,
+                refine, identify, false);
         }
     }
 }
 
-void TradeHandler::processTradeItemAddResponse(Net::MessageIn &msg) const
+void TradeHandler::processTradeItemAddResponse(Net::MessageIn &msg)
 {
     // Trade: New Item add response (was 0x00ea, now 01b1)
     const int index = msg.readInt16() - INVENTORY_OFFSET;
@@ -221,7 +216,7 @@ void TradeHandler::processTradeItemAddResponse(Net::MessageIn &msg) const
     }
     const int quantity = msg.readInt16();
 
-    const int res = msg.readInt8();
+    const uint8_t res = msg.readUInt8();
     switch (res)
     {
         case 0:
@@ -236,37 +231,37 @@ void TradeHandler::processTradeItemAddResponse(Net::MessageIn &msg) const
             break;
         case 1:
             // Add item failed - player overweighted
-            NotifyManager::notify(NotifyManager::
+            NotifyManager::notify(NotifyTypes::
                 TRADE_ADD_PARTNER_OVER_WEIGHT);
             break;
         case 2:
             // Add item failed - player has no free slot
-            NotifyManager::notify(NotifyManager::TRADE_ADD_PARTNER_NO_SLOTS);
+            NotifyManager::notify(NotifyTypes::TRADE_ADD_PARTNER_NO_SLOTS);
             break;
         case 3:
             // Add item failed - non tradable item
-            NotifyManager::notify(NotifyManager::TRADE_ADD_UNTRADABLE_ITEM);
+            NotifyManager::notify(NotifyTypes::TRADE_ADD_UNTRADABLE_ITEM);
             break;
         default:
-            NotifyManager::notify(NotifyManager::TRADE_ADD_ERROR);
+            NotifyManager::notify(NotifyTypes::TRADE_ADD_ERROR);
             logger->log("QQQ SMSG_TRADE_ITEM_ADD_RESPONSE: "
                         + toString(res));
             break;
     }
 }
 
-void TradeHandler::processTradeOk(Net::MessageIn &msg) const
+void TradeHandler::processTradeOk(Net::MessageIn &msg)
 {
     // 0 means ok from myself, 1 means ok from other;
     if (tradeWindow)
-        tradeWindow->receivedOk(msg.readInt8() == 0);
+        tradeWindow->receivedOk(msg.readUInt8() == 0U);
     else
-        msg.readInt8();
+        msg.readUInt8();
 }
 
-void TradeHandler::processTradeCancel(Net::MessageIn &msg A_UNUSED) const
+void TradeHandler::processTradeCancel(Net::MessageIn &msg A_UNUSED)
 {
-    NotifyManager::notify(NotifyManager::TRADE_CANCELLED);
+    NotifyManager::notify(NotifyTypes::TRADE_CANCELLED);
     if (tradeWindow)
     {
         tradeWindow->setVisible(false);
@@ -275,9 +270,9 @@ void TradeHandler::processTradeCancel(Net::MessageIn &msg A_UNUSED) const
     PlayerInfo::setTrading(false);
 }
 
-void TradeHandler::processTradeComplete(Net::MessageIn &msg A_UNUSED) const
+void TradeHandler::processTradeComplete(Net::MessageIn &msg A_UNUSED)
 {
-    NotifyManager::notify(NotifyManager::TRADE_COMPLETE);
+    NotifyManager::notify(NotifyTypes::TRADE_COMPLETE);
     if (tradeWindow)
     {
         tradeWindow->setVisible(false);
