@@ -22,12 +22,13 @@
 
 #include "net/eathena/adminhandler.h"
 
-#include "notifications.h"
 #include "notifymanager.h"
 
 #include "net/eathena/messageout.h"
 #include "net/eathena/protocol.h"
 
+#include "resources/notifytypes.h"
+
 #include <string>
 
 #include "debug.h"
@@ -56,9 +57,9 @@ void AdminHandler::handleMessage(Net::MessageIn &msg)
     {
         case SMSG_ADMIN_KICK_ACK:
             if (msg.readInt32() == 0)
-                NotifyManager::notify(NotifyManager::KICK_FAIL);
+                NotifyManager::notify(NotifyTypes::KICK_FAIL);
             else
-                NotifyManager::notify(NotifyManager::KICK_SUCCEED);
+                NotifyManager::notify(NotifyTypes::KICK_SUCCEED);
             break;
         default:
             break;
