@@ -38,6 +38,7 @@
 
 #include "net/eathena/messageout.h"
 #include "net/eathena/protocol.h"
+#include "net/eathena/sprite.h"
 
 #include "resources/iteminfo.h"
 
@@ -54,7 +55,7 @@ extern int serverVersion;
 namespace EAthena
 {
 
-BeingHandler::BeingHandler(const bool enableSync):
+BeingHandler::BeingHandler(const bool enableSync) :
     MessageHandler(),
     Ea::BeingHandler(enableSync)
 {
@@ -98,7 +99,7 @@ BeingHandler::BeingHandler(const bool enableSync):
 
 void BeingHandler::requestNameById(const int id) const
 {
-    MessageOut outMsg(0x0094);
+    MessageOut outMsg(CMSG_NAME_REQUEST);
     outMsg.writeInt32(id);  // readLong(2));
 }
 
@@ -266,15 +267,15 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
     if (!dstBeing)
         return;
 
-    const int type = msg.readInt8();
+    const uint8_t type = msg.readUInt8();
     int id = 0;
-    int id2 = 0;
+    unsigned int id2 = 0U;
     const std::string color;
 
     if (!look2)
     {
-        id = msg.readInt8();
-        id2 = 1;    // default color
+        id = static_cast<int>(msg.readUInt8());
+        id2 = 1U;    // default color
     }
     else
     {        // SMSG_BEING_CHANGE_LOOKS2
@@ -286,10 +287,10 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
 //        color.clear();
     }
 
-    if (dstBeing->getType() == Being::PLAYER)
+    if (dstBeing->getType() == ActorType::PLAYER)
         dstBeing->setOtherTime();
 
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     switch (type)
@@ -304,22 +305,22 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
             dstBeing->setSprite(SPRITE_WEAPON, id, "", 1, true);
             if (!mHideShield)
                 dstBeing->setSprite(SPRITE_SHIELD, id2);
-            player_node->imitateOutfit(dstBeing, SPRITE_SHIELD);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_SHIELD);
             break;
         case 3:     // Change lower headgear for eAthena, pants for us
             dstBeing->setSprite(SPRITE_BOTTOMCLOTHES, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_BOTTOMCLOTHES);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_BOTTOMCLOTHES);
             break;
         case 4:     // Change upper headgear for eAthena, hat for us
             dstBeing->setSprite(SPRITE_HAT, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_HAT);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_HAT);
             break;
         case 5:     // Change middle headgear for eathena, armor for us
             dstBeing->setSprite(SPRITE_TOPCLOTHES, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_TOPCLOTHES);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_TOPCLOTHES);
             break;
         case 6:     // eAthena LOOK_HAIR_COLOR
             dstBeing->setSpriteColor(SPRITE_HAIR, ItemDB::get(
@@ -334,42 +335,42 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
                 dstBeing->setSprite(SPRITE_SHIELD, id, color,
                     static_cast<unsigned char>(id2));
             }
-            player_node->imitateOutfit(dstBeing, SPRITE_SHIELD);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_SHIELD);
             break;
         case 9:     // eAthena LOOK_SHOES
             dstBeing->setSprite(SPRITE_SHOE, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_SHOE);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_SHOE);
             break;
         case 10:   // LOOK_GLOVES
             dstBeing->setSprite(SPRITE_GLOVES, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_GLOVES);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_GLOVES);
             break;
         case 11:  // LOOK_CAPE
             dstBeing->setSprite(SPRITE_CAPE, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_CAPE);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_CAPE);
             break;
         case 12:
             dstBeing->setSprite(SPRITE_MISC1, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_MISC1);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_MISC1);
             break;
         case 13:
             dstBeing->setSprite(SPRITE_MISC2, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_MISC2);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_MISC2);
             break;
         case 14:
             dstBeing->setSprite(SPRITE_EVOL1, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_EVOL1);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_EVOL1);
             break;
         case 15:
             dstBeing->setSprite(SPRITE_EVOL2, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_EVOL2);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_EVOL2);
             break;
         default:
             logger->log("QQQ3 CHANGE_LOOKS: unsupported type: "
@@ -380,9 +381,9 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
     }
 }
 
-void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
+void BeingHandler::processNameResponse2(Net::MessageIn &msg)
 {
-    if (!actorManager || !player_node)
+    if (!actorManager || !localPlayer)
         return;
 
     const int len = msg.readInt16();
@@ -391,9 +392,9 @@ void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
     Being *const dstBeing = actorManager->findBeing(beingId);
     if (dstBeing)
     {
-        if (beingId == player_node->getId())
+        if (beingId == localPlayer->getId())
         {
-            player_node->pingResponse();
+            localPlayer->pingResponse();
         }
         else
         {
@@ -401,12 +402,12 @@ void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
             dstBeing->updateGuild();
             dstBeing->addToCache();
 
-            if (dstBeing->getType() == Being::PLAYER)
+            if (dstBeing->getType() == ActorType::PLAYER)
                 dstBeing->updateColors();
 
-            if (player_node)
+            if (localPlayer)
             {
-                const Party *const party = player_node->getParty();
+                const Party *const party = localPlayer->getParty();
                 if (party && party->isMember(dstBeing->getId()))
                 {
                     PartyMember *const member = party->getMember(
@@ -415,7 +416,7 @@ void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
                     if (member)
                         member->setName(dstBeing->getName());
                 }
-                player_node->checkNewName(dstBeing);
+                localPlayer->checkNewName(dstBeing);
             }
         }
     }
@@ -424,7 +425,7 @@ void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
 void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
                                            const int msgType) const
 {
-    if (!actorManager || !player_node)
+    if (!actorManager || !localPlayer)
         return;
 
     // An update about a player, potentially including movement.
@@ -432,8 +433,8 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
     const int16_t speed = msg.readInt16();
     const uint16_t stunMode = msg.readInt16();  // opt1
     uint32_t statusEffects = msg.readInt16();   // opt2
-    statusEffects |= (static_cast<uint32_t>(msg.readInt16()))
-        << 16;  // status.options; Aethyra uses this as misc2
+    statusEffects |= (static_cast<uint16_t>(msg.readInt16()))
+        << 16U;  // status.options; Aethyra uses this as misc2
     const int16_t job = msg.readInt16();
     int disguiseId = 0;
     if (id < 110000000 && job >= 1000)
@@ -465,7 +466,7 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
             dstBeing->setDirection(dir);
     }
 
-    if (Party *const party = player_node->getParty())
+    if (Party *const party = localPlayer->getParty())
     {
         if (party->isMember(id))
             dstBeing->setParty(party);
@@ -486,10 +487,10 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
     const uint16_t headMid = msg.readInt16();
     const int hairColor = msg.readInt16();
 
-    msg.readInt8();
-    msg.readInt8();
-    msg.readInt8();
-    msg.readInt8();  // unused
+    msg.readUInt8();
+    msg.readUInt8();
+    msg.readUInt8();
+    msg.readUInt8();  // unused
 
     const int guild = msg.readInt32();  // guild
 
@@ -504,9 +505,10 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
     msg.readInt16();  // emblem
     msg.readInt16();  // manner
     dstBeing->setStatusEffectBlock(32, msg.readInt16());  // opt3
-    msg.readInt8();   // karma
+    msg.readUInt8();  // karma
     // reserving bit for future usage
-    dstBeing->setGender(Being::intToGender(msg.readInt8() & 3));
+    dstBeing->setGender(Being::intToGender(
+        static_cast<uint8_t>(msg.readUInt8() & 3U)));
 
     if (!disguiseId)
     {
@@ -520,14 +522,14 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
         dstBeing->updateSprite(SPRITE_HAIR, hairStyle * -1,
             ItemDB::get(-hairStyle).getDyeColorsString(hairColor));
     }
-    player_node->imitateOutfit(dstBeing);
+    localPlayer->imitateOutfit(dstBeing);
 
     if (msgType == 3)
     {
         uint16_t srcX, srcY, dstX, dstY;
         msg.readCoordinatePair(srcX, srcY, dstX, dstY);
 
-        player_node->followMoveTo(dstBeing, srcX, srcY, dstX, dstY);
+        localPlayer->followMoveTo(dstBeing, srcX, srcY, dstX, dstY);
 
         dstBeing->setTileCoords(srcX, srcY);
         dstBeing->setDestination(dstX, dstY);
@@ -543,11 +545,11 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
                 dstBeing->setDirectionDelayed(static_cast<uint8_t>(d));
         }
 
-        if (player_node->getCurrentAction() != Being::STAND)
-            player_node->imitateAction(dstBeing, Being::STAND);
-        if (player_node->getDirection() != dstBeing->getDirection())
+        if (localPlayer->getCurrentAction() != BeingAction::STAND)
+            localPlayer->imitateAction(dstBeing, BeingAction::STAND);
+        if (localPlayer->getDirection() != dstBeing->getDirection())
         {
-            player_node->imitateDirection(dstBeing,
+            localPlayer->imitateDirection(dstBeing,
                 dstBeing->getDirection());
         }
     }
@@ -558,7 +560,7 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
         dstBeing->setTileCoords(x, y);
         dstBeing->setDirection(dir);
 
-        player_node->imitateDirection(dstBeing, dir);
+        localPlayer->imitateDirection(dstBeing, dir);
     }
 
     const uint16_t gmstatus = msg.readInt16();
@@ -568,25 +570,25 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
 
     if (msgType == 1 || msgType == 2)
     {
-        const int type = msg.readInt8();
+        const uint8_t type = msg.readUInt8();
         switch (type)
         {
             case 0:
-                dstBeing->setAction(Being::STAND, 0);
-                player_node->imitateAction(dstBeing, Being::STAND);
+                dstBeing->setAction(BeingAction::STAND, 0);
+                localPlayer->imitateAction(dstBeing, BeingAction::STAND);
                 break;
 
             case 1:
-                if (dstBeing->getCurrentAction() != Being::DEAD)
+                if (dstBeing->getCurrentAction() != BeingAction::DEAD)
                 {
-                    dstBeing->setAction(Being::DEAD, 0);
+                    dstBeing->setAction(BeingAction::DEAD, 0);
                     dstBeing->recalcSpritesOrder();
                 }
                 break;
 
             case 2:
-                dstBeing->setAction(Being::SIT, 0);
-                player_node->imitateAction(dstBeing, Being::SIT);
+                dstBeing->setAction(BeingAction::SIT, 0);
+                localPlayer->imitateAction(dstBeing, BeingAction::SIT);
                 break;
 
             default:
@@ -600,17 +602,17 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
     }
     else if (msgType == 3)
     {
-        msg.readInt8();  // unknown
+        msg.readUInt8();  // unknown
     }
 
-    const int level = msg.readInt8();  // Lv
+    const int level = static_cast<int>(msg.readUInt8());  // Lv
 
     if (level)
         dstBeing->setLevel(level);
 
-    msg.readInt8();  // unknown
+    msg.readUInt8();  // unknown
 
-    if (dstBeing->getType() != Being::PLAYER
+    if (dstBeing->getType() != ActorType::PLAYER
         || msgType != 3)
     {
         dstBeing->setActionTime(tick_time);
@@ -618,10 +620,11 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
 
     dstBeing->setStunMode(stunMode);
     dstBeing->setStatusEffectBlock(0, static_cast<uint16_t>(
-        (statusEffects >> 16) & 0xffff));
-    dstBeing->setStatusEffectBlock(16, statusEffects & 0xffff);
+        (statusEffects >> 16) & 0xffffU));
+    dstBeing->setStatusEffectBlock(16, static_cast<uint16_t>(
+        statusEffects & 0xffffU));
 
-    if (msgType == 3 && dstBeing->getType() == Being::PLAYER)
+    if (msgType == 3 && dstBeing->getType() == ActorType::PLAYER)
         dstBeing->setMoveTime();
 }
 
@@ -632,7 +635,7 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         return;
 
     if (visible)
-        msg.readInt8();  // padding?
+        msg.readUInt8();  // padding?
 
     // Information about a being in range
     const int id = msg.readInt32();
@@ -645,12 +648,13 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
     int16_t speed = msg.readInt16();
     const uint16_t stunMode = msg.readInt16();  // opt1
     uint32_t statusEffects = msg.readInt16();   // opt2
-    statusEffects |= (static_cast<uint32_t>(msg.readInt16())) << 16;  // option
+    statusEffects |= (static_cast<uint16_t>(
+        msg.readInt16())) << 16U;  // option
     const int16_t job = msg.readInt16();  // class
 
     Being *dstBeing = actorManager->findBeing(id);
 
-    if (dstBeing && dstBeing->getType() == Being::MONSTER
+    if (dstBeing && dstBeing->getType() == ActorType::MONSTER
         && !dstBeing->isAlive())
     {
         actorManager->destroy(dstBeing);
@@ -679,22 +683,22 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
     else
     {
         // undeleting marked for deletion being
-        if (dstBeing->getType() == Being::NPC)
+        if (dstBeing->getType() == ActorType::NPC)
             actorManager->undelete(dstBeing);
     }
 
-    if (dstBeing->getType() == Being::PLAYER)
+    if (dstBeing->getType() == ActorType::PLAYER)
         dstBeing->setMoveTime();
 
     if (spawnId)
     {
-        dstBeing->setAction(Being::SPAWN, 0);
+        dstBeing->setAction(BeingAction::SPAWN, 0);
     }
     else if (visible)
     {
         dstBeing->clearPath();
         dstBeing->setActionTime(tick_time);
-        dstBeing->setAction(Being::STAND, 0);
+        dstBeing->setAction(BeingAction::STAND, 0);
     }
 
     // Prevent division by 0 when calculating frame
@@ -703,8 +707,8 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
 
     dstBeing->setWalkSpeed(Vector(speed, speed, 0));
     dstBeing->setSubtype(job, 0);
-    if (dstBeing->getType() == ActorSprite::MONSTER && player_node)
-        player_node->checkNewName(dstBeing);
+    if (dstBeing->getType() == ActorType::MONSTER && localPlayer)
+        localPlayer->checkNewName(dstBeing);
 
     const int hairStyle = msg.readInt16();
     const uint16_t weapon = msg.readInt16();
@@ -717,7 +721,7 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
     const uint16_t shoes = msg.readInt16();  // clothes color
 
     uint16_t gloves;
-    if (dstBeing->getType() == ActorSprite::MONSTER)
+    if (dstBeing->getType() == ActorType::MONSTER)
     {
         msg.readInt32();
         msg.readInt32();
@@ -732,10 +736,10 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
 
     msg.readInt16();  // manner
     dstBeing->setStatusEffectBlock(32, msg.readInt16());  // opt3
-    msg.readInt8();   // karma
-    int16_t gender = msg.readInt8();
+    msg.readUInt8();   // karma
+    uint8_t gender = msg.readUInt8();
 
-    if (dstBeing->getType() == ActorSprite::PLAYER)
+    if (dstBeing->getType() == ActorType::PLAYER)
     {
         gender &= 3;
         dstBeing->setGender(Being::intToGender(gender));
@@ -751,21 +755,21 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         if (!mHideShield)
             setSprite(dstBeing, SPRITE_SHIELD, shield);
     }
-    else if (dstBeing->getType() == ActorSprite::NPC)
+    else if (dstBeing->getType() == ActorType::NPC)
     {
         switch (gender)
         {
             case 2:
-                dstBeing->setGender(GENDER_FEMALE);
+                dstBeing->setGender(Gender::FEMALE);
                 break;
             case 3:
-                dstBeing->setGender(GENDER_MALE);
+                dstBeing->setGender(Gender::MALE);
                 break;
             case 4:
-                dstBeing->setGender(GENDER_OTHER);
+                dstBeing->setGender(Gender::OTHER);
                 break;
             default:
-                dstBeing->setGender(GENDER_UNSPECIFIED);
+                dstBeing->setGender(Gender::UNSPECIFIED);
                 break;
         }
     }
@@ -774,7 +778,7 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
     {
         uint16_t srcX, srcY, dstX, dstY;
         msg.readCoordinatePair(srcX, srcY, dstX, dstY);
-        dstBeing->setAction(Being::STAND, 0);
+        dstBeing->setAction(BeingAction::STAND, 0);
         dstBeing->setTileCoords(srcX, srcY);
         dstBeing->setDestination(dstX, dstY);
     }
@@ -802,14 +806,15 @@ void BeingHandler::processBeingVisibleOrMove(Net::MessageIn &msg,
         dstBeing->setDirection(dir);
     }
 
-    msg.readInt8();   // unknown
-    msg.readInt8();   // state / sit
+    msg.readUInt8();  // unknown
+    msg.readUInt8();  // state / sit
     msg.readInt16();  // level
 
     dstBeing->setStunMode(stunMode);
     dstBeing->setStatusEffectBlock(0, static_cast<uint16_t>(
-        (statusEffects >> 16) & 0xffff));
-    dstBeing->setStatusEffectBlock(16, statusEffects & 0xffff);
+        (statusEffects >> 16) & 0xffffU));
+    dstBeing->setStatusEffectBlock(16, static_cast<uint16_t>(
+        statusEffects & 0xffffU));
 }
 
 }  // namespace EAthena
