@@ -50,7 +50,7 @@ class BeingHandler final : public MessageHandler, public Ea::BeingHandler
         void processBeingChangeLook(Net::MessageIn &msg,
                                     const bool look2) const;
 
-        void processNameResponse2(Net::MessageIn &msg) const;
+        static void processNameResponse2(Net::MessageIn &msg);
 
         void processPlayerMoveUpdate(Net::MessageIn &msg,
                                      const int type) const;
