@@ -38,12 +38,12 @@
 namespace EAthena
 {
 
-MessageOut::MessageOut(const int16_t id):
+MessageOut::MessageOut(const int16_t id) :
     Net::MessageOut(id),
     mNetwork(EAthena::Network::instance())
 {
     mNetwork->fixSendBuffer();
-    mData = mNetwork->mOutBuffer + mNetwork->mOutSize;
+    mData = mNetwork->mOutBuffer + static_cast<size_t>(mNetwork->mOutSize);
 
     // +++ can be issue. call to virtual member
     writeInt16(id);
@@ -61,9 +61,9 @@ void MessageOut::writeInt16(const int16_t value)
     expand(2);
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
     int16_t swap = SDL_Swap16(value);
-    memcpy(mData + mPos, &swap, sizeof(int16_t));
+    memcpy(mData + static_cast<size_t>(mPos), &swap, sizeof(int16_t));
 #else
-    memcpy(mData + mPos, &value, sizeof(int16_t));
+    memcpy(mData + static_cast<size_t>(mPos), &value, sizeof(int16_t));
 #endif
     mPos += 2;
     PacketCounters::incOutBytes(2);
@@ -75,9 +75,9 @@ void MessageOut::writeInt32(const int32_t value)
     expand(4);
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
     int32_t swap = SDL_Swap32(value);
-    memcpy(mData + mPos, &swap, sizeof(int32_t));
+    memcpy(mData + static_cast<size_t>(mPos), &swap, sizeof(int32_t));
 #else
-    memcpy(mData + mPos, &value, sizeof(int32_t));
+    memcpy(mData + static_cast<size_t>(mPos), &value, sizeof(int32_t));
 #endif
     mPos += 4;
     PacketCounters::incOutBytes(4);
@@ -91,12 +91,16 @@ void MessageOut::writeCoordinates(const uint16_t x,
                                   const uint16_t y,
                                   unsigned char direction)
 {
-    DEBUGLOG(strprintf("writeCoordinates: %u,%u %u", x, y, direction));
-    char *const data = mData + mPos;
+    DEBUGLOG(strprintf("writeCoordinates: %u,%u %u",
+        static_cast<unsigned>(x),
+        static_cast<unsigned>(y),
+        static_cast<unsigned>(direction)));
+    unsigned char *const data = reinterpret_cast<unsigned char*>(mData)
+        + static_cast<size_t>(mPos);
     mNetwork->mOutSize += 3;
     mPos += 3;
 
-    int16_t temp = x;
+    uint16_t temp = x;
     temp <<= 6;
     data[0] = 0;
     data[1] = 1;
