@@ -224,8 +224,8 @@ int NpcHandler::getNpc(Net::MessageIn &msg, const bool haveLength)
             mDialog = new NpcDialog(npcId);
             mDialog->postInit();
             mDialog->saveCamera();
-            if (player_node)
-                player_node->stopWalking(false);
+            if (localPlayer)
+                localPlayer->stopWalking(false);
             NpcDialog::mNpcDialogs[npcId] = mDialog;
         }
     }
@@ -242,19 +242,19 @@ int NpcHandler::getNpc(Net::MessageIn &msg, const bool haveLength)
 }
 
 void NpcHandler::processNpcCutin(Net::MessageIn &msg A_UNUSED,
-                                 const int npcId A_UNUSED) const
+                                 const int npcId A_UNUSED)
 {
     msg.readString(64);  // image name
-    msg.readInt8();      // type
+    msg.readUInt8();     // type
 }
 
 void NpcHandler::processNpcViewPoint(Net::MessageIn &msg A_UNUSED,
-                                     const int npcId A_UNUSED) const
+                                     const int npcId A_UNUSED)
 {
     msg.readInt32();  // type
     msg.readInt32();  // x
     msg.readInt32();  // y
-    msg.readInt8();   // byte
+    msg.readUInt8();  // byte
     msg.readInt32();  // color
 }
 
