@@ -22,8 +22,11 @@
 
 #include "net/eathena/playerhandler.h"
 
+#include "being/attributes.h"
+
 #include "net/net.h"
 
+#include "net/eathena/attrs.h"
 #include "net/eathena/messageout.h"
 #include "net/eathena/protocol.h"
 #include "net/eathena/inventoryhandler.h"
@@ -145,7 +148,7 @@ void PlayerHandler::increaseAttribute(const int attr) const
 
 void PlayerHandler::increaseSkill(const uint16_t skillId) const
 {
-    if (PlayerInfo::getAttribute(PlayerInfo::SKILL_POINTS) <= 0)
+    if (PlayerInfo::getAttribute(Attributes::SKILL_POINTS) <= 0)
         return;
 
     MessageOut outMsg(CMSG_SKILL_LEVELUP_REQUEST);
@@ -181,23 +184,23 @@ void PlayerHandler::setDestination(const int x, const int y,
         static_cast<unsigned char>(direction));
 }
 
-void PlayerHandler::changeAction(const Being::Action &action) const
+void PlayerHandler::changeAction(const BeingAction::Action &action) const
 {
     unsigned char type;
     switch (action)
     {
-        case Being::SIT:
+        case BeingAction::SIT:
             type = 2;
             break;
-        case Being::STAND:
+        case BeingAction::STAND:
             type = 3;
             break;
         default:
-        case Being::MOVE:
-        case Being::ATTACK:
-        case Being::DEAD:
-        case Being::HURT:
-        case Being::SPAWN:
+        case BeingAction::MOVE:
+        case BeingAction::ATTACK:
+        case BeingAction::DEAD:
+        case BeingAction::HURT:
+        case BeingAction::SPAWN:
             return;
     }
 
@@ -223,19 +226,19 @@ void PlayerHandler::updateStatus(const uint8_t status) const
     outMsg.writeInt8(0);
 }
 
-void PlayerHandler::processPlayerShortcuts(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerShortcuts(Net::MessageIn &msg)
 {
     for (int f = 0; f < 27; f ++)
     {
-        msg.readInt8();   // type 0: item, 1: skill
+        msg.readUInt8();  // type 0: item, 1: skill
         msg.readInt32();  // item or skill id
         msg.readInt16();  // lvl
     }
 }
 
-void PlayerHandler::processPlayerShowEquip(Net::MessageIn &msg) const
+void PlayerHandler::processPlayerShowEquip(Net::MessageIn &msg)
 {
-    msg.readInt8();  // show equip
+    msg.readUInt8();  // show equip
 }
 
 }  // namespace EAthena
