@@ -23,56 +23,13 @@
 #ifndef NET_EATHENA_PROTOCOL_H
 #define NET_EATHENA_PROTOCOL_H
 
-namespace EAthena
-{
-    enum
-    {
-        JOB = 0xa,
-
-        STR = 0xd,
-        AGI,
-        VIT,
-        INT,
-        DEX,
-        LUK,
-
-        ATK,
-        DEF,
-        MATK,
-        MDEF,
-        HIT,
-        FLEE,
-        CRIT
-
-//        KARMA,
-//        MANNER
-    };
-}  // namespace EAthena
-
-enum
-{
-    SPRITE_BASE = 0,
-    SPRITE_SHOE,
-    SPRITE_BOTTOMCLOTHES,
-    SPRITE_TOPCLOTHES,
-    SPRITE_MISC1,
-    SPRITE_MISC2,
-    SPRITE_HAIR,
-    SPRITE_HAT,
-    SPRITE_CAPE,
-    SPRITE_GLOVES,
-    SPRITE_WEAPON,
-    SPRITE_SHIELD,
-    SPRITE_EVOL1,
-    SPRITE_EVOL2,
-    SPRITE_VECTOREND
-};
-
 /*********************************
  * Packets from server to client *
  *********************************/
 #define SMSG_SERVER_VERSION_RESPONSE 0x7531
 
+#define CMSG_LOGIN_REGISTER          0x0064
+
 #define SMSG_SERVER_PING             0x007f /**< Contains server tick */
 #define SMSG_CONNECTION_PROBLEM      0x0081
 
@@ -140,6 +97,7 @@ enum
 #define SMSG_BEING_EMOTION           0x00c0
 #define SMSG_BEING_ACTION            0x008a /**< Attack, sit, stand up, ... */
 #define SMSG_BEING_CHAT              0x008d /**< A being talks */
+#define CMSG_NAME_REQUEST            0x0094
 #define SMSG_BEING_NAME_RESPONSE     0x0095 /**< Has to be requested */
 #define SMSG_BEING_NAME_RESPONSE2    0x0220 /**< Has to be requested */
 #define SMSG_BEING_CHANGE_DIRECTION  0x009c
