@@ -37,7 +37,7 @@
 namespace Net
 {
 
-MessageIn::MessageIn(const char *const data, const unsigned int length):
+MessageIn::MessageIn(const char *const data, const unsigned int length) :
     mData(data),
     mLength(length),
     mId(0),
@@ -47,7 +47,7 @@ MessageIn::MessageIn(const char *const data, const unsigned int length):
     DEBUGLOG("MessageIn");
 }
 
-unsigned char MessageIn::readInt8()
+unsigned char MessageIn::readUInt8()
 {
     unsigned char value = static_cast<unsigned char>(-1);
     if (mPos < mLength)
@@ -55,24 +55,20 @@ unsigned char MessageIn::readInt8()
 
     mPos += 1;
     PacketCounters::incInBytes(1);
-    DEBUGLOG("readInt8: " + toStringPrint(static_cast<int>(value)));
+    DEBUGLOG("readUInt8: " + toStringPrint(static_cast<int>(value)));
     return value;
 }
 
-void MessageIn::readCoordinates(uint16_t &restrict x, uint16_t &restrict y)
+signed char MessageIn::readInt8()
 {
-    if (mPos + 3 <= mLength)
-    {
-        const unsigned char *const p
-            = reinterpret_cast<unsigned char const *const>(mData + mPos);
-        x = static_cast<uint16_t>(p[0] | ((p[1] & 0x07) << 8));
-        y = static_cast<uint16_t>((p[1] >> 3) | ((p[2] & 0x3F) << 5));
-    }
-    mPos += 3;
-    PacketCounters::incInBytes(3);
-    DEBUGLOG(std::string("readCoordinates: ").append(toString(
-        static_cast<int>(x))).append(",").append(toString(
-        static_cast<int>(y))));
+    signed char value = static_cast<signed char>(-1);
+    if (mPos < mLength)
+        value = static_cast<signed char>(mData[mPos]);
+
+    mPos += 1;
+    PacketCounters::incInBytes(1);
+    DEBUGLOG("readInt8: " + toStringPrint(static_cast<int>(value)));
+    return value;
 }
 
 uint8_t MessageIn::fromServerDirection(const uint8_t serverDir)
@@ -97,10 +93,7 @@ uint8_t MessageIn::fromServerDirection(const uint8_t serverDir)
         case 7:
             return 9;
         case 8:
-#ifdef MANASERV_SUPPORT
-            if (Net::getNetworkType() != ServerInfo::MANASERV)
-#endif
-                return 8;
+            return 8;
         default:
             logger->log("incorrect direction: %d",
                 static_cast<int>(serverDir));
@@ -111,16 +104,15 @@ uint8_t MessageIn::fromServerDirection(const uint8_t serverDir)
 void MessageIn::readCoordinates(uint16_t &restrict x, uint16_t &restrict y,
                                 uint8_t &restrict direction)
 {
-    uint8_t serverDir = 0;
     if (mPos + 3 <= mLength)
     {
-        const char *const data = mData + mPos;
-        int16_t temp = MAKEWORD(data[1] & 0x00c0, data[0] & 0x00ff);
+        const char *const data = mData + static_cast<size_t>(mPos);
+        uint16_t temp = MAKEWORD(data[1] & 0x00c0, data[0] & 0x00ff);
         x = static_cast<uint16_t>(temp >> 6);
         temp = MAKEWORD(data[2] & 0x00f0, data[1] & 0x003f);
         y = static_cast<uint16_t>(temp >> 4);
 
-        serverDir = static_cast<uint8_t>(data[2] & 0x000f);
+        const uint8_t serverDir = static_cast<uint8_t>(data[2] & 0x000f);
         direction = fromServerDirection(serverDir);
 
         DEBUGLOG(std::string("readCoordinates: ").append(toString(
@@ -146,8 +138,8 @@ void MessageIn::readCoordinatePair(uint16_t &restrict srcX,
 {
     if (mPos + 5 <= mLength)
     {
-        const char *const data = mData + mPos;
-        int16_t temp = MAKEWORD(data[3], data[2] & 0x000f);
+        const char *const data = mData + static_cast<size_t>(mPos);
+        uint16_t temp = MAKEWORD(data[3], data[2] & 0x000f);
         dstX = static_cast<uint16_t>(temp >> 2);
 
         dstY = MAKEWORD(data[4], data[3] & 0x0003);
@@ -198,12 +190,12 @@ std::string MessageIn::readString(int length)
     }
 
     // Read the string
-    const char *const stringBeg = mData + mPos;
+    const char *const stringBeg = mData + static_cast<size_t>(mPos);
     const char *const stringEnd
         = static_cast<const char *const>(memchr(stringBeg, '\0', length));
 
     const std::string str(stringBeg, stringEnd
-        ? stringEnd - stringBeg : length);
+        ? stringEnd - stringBeg : static_cast<size_t>(length));
     mPos += length;
     PacketCounters::incInBytes(length);
     DEBUGLOG("readString: " + str);
@@ -224,11 +216,11 @@ std::string MessageIn::readRawString(int length)
     }
 
     // Read the string
-    const char *const stringBeg = mData + mPos;
+    const char *const stringBeg = mData + static_cast<size_t>(mPos);
     const char *const stringEnd
         = static_cast<const char *const>(memchr(stringBeg, '\0', length));
     std::string str(stringBeg, stringEnd
-        ? stringEnd - stringBeg : length);
+        ? stringEnd - stringBeg : static_cast<size_t>(length));
 
     mPos += length;
     PacketCounters::incInBytes(length);
@@ -236,7 +228,8 @@ std::string MessageIn::readRawString(int length)
 
     if (stringEnd)
     {
-        const size_t len2 = length - (stringEnd - stringBeg) - 1;
+        const size_t len2 = static_cast<size_t>(length)
+            - (stringEnd - stringBeg) - 1;
         const char *const stringBeg2 = stringEnd + 1;
         const char *const stringEnd2
             = static_cast<const char *const>(memchr(stringBeg2, '\0', len2));
@@ -266,9 +259,10 @@ unsigned char *MessageIn::readBytes(int length)
         return nullptr;
     }
 
-    unsigned char *const buf = new unsigned char[length + 2];
+    unsigned char *const buf
+        = new unsigned char[static_cast<size_t>(length + 2)];
 
-    memcpy(buf, mData + mPos, length);
+    memcpy(buf, mData + static_cast<size_t>(mPos), length);
     buf[length] = 0;
     buf[length + 1] = 0;
     mPos += length;
