@@ -32,10 +32,6 @@
 #include "net/eathena/generalhandler.h"
 #endif
 
-#ifdef MANASERV_SUPPORT
-#include "net/manaserv/generalhandler.h"
-#endif
-
 #include "debug.h"
 
 Net::AdminHandler *adminHandler = nullptr;
@@ -141,6 +137,7 @@ ServerInfo::Type networkType = ServerInfo::UNKNOWN;
 
 void connectToServer(const ServerInfo &server)
 {
+    BLOCK_START("Net::connectToServer")
     if (networkType == server.type && getGeneralHandler())
     {
         getGeneralHandler()->reload();
@@ -160,13 +157,6 @@ void connectToServer(const ServerInfo &server)
                 new EAthena::GeneralHandler;
 #else
                 new TmwAthena::GeneralHandler;
-#endif
-                break;
-            case ServerInfo::MANASERV:
-#ifdef MANASERV_SUPPORT
-                new ManaServ::GeneralHandler;
-#else
-                new TmwAthena::GeneralHandler;
 #endif
                 break;
             case ServerInfo::TMWATHENA:
@@ -186,6 +176,7 @@ void connectToServer(const ServerInfo &server)
         getLoginHandler()->setServer(server);
         getLoginHandler()->connect();
     }
+    BLOCK_END("Net::connectToServer")
 }
 
 void unload()
