@@ -27,22 +27,16 @@
 
 #include "gui/widgets/tabs/chattab.h"
 
+#include "net/partyshare.h"
+
 #include <string>
 
 class Being;
 
-enum PartyShare
-{
-    PARTY_SHARE_UNKNOWN = -1,
-    PARTY_SHARE_NO,
-    PARTY_SHARE,
-    PARTY_SHARE_NOT_POSSIBLE = 2
-};
-
 namespace Net
 {
 
-class PartyHandler
+class PartyHandler notfinal
 {
     public:
         virtual ~PartyHandler()
@@ -52,7 +46,7 @@ class PartyHandler
 
         virtual void join(const int partyId) const = 0;
 
-        virtual void invite(Being *const player) const = 0;
+        virtual void invite(const Being *const player) const = 0;
 
         virtual void invite(const std::string &name) const = 0;
 
@@ -61,19 +55,22 @@ class PartyHandler
 
         virtual void leave() const = 0;
 
-        virtual void kick(Being *const player) const = 0;
+        virtual void kick(const Being *const player) const = 0;
 
         virtual void kick(const std::string &name) const = 0;
 
         virtual void chat(const std::string &text) const = 0;
 
-        virtual PartyShare getShareExperience() const A_WARN_UNUSED = 0;
+        virtual Net::PartyShare::Type getShareExperience()
+                                      const A_WARN_UNUSED = 0;
 
-        virtual void setShareExperience(const PartyShare share) const = 0;
+        virtual void setShareExperience(const Net::PartyShare::Type
+                                        share) const = 0;
 
-        virtual PartyShare getShareItems() const A_WARN_UNUSED = 0;
+        virtual Net::PartyShare::Type getShareItems() const A_WARN_UNUSED = 0;
 
-        virtual void setShareItems(const PartyShare share) const = 0;
+        virtual void setShareItems(const Net::PartyShare::Type
+                                   share) const = 0;
 
         virtual void clear() const = 0;
 
