@@ -31,7 +31,7 @@
 namespace Net
 {
 
-class PlayerHandler
+class PlayerHandler notfinal
 {
     public:
         virtual ~PlayerHandler()
@@ -56,7 +56,7 @@ class PlayerHandler
         virtual void setDestination(const int x, const int y,
                                     const int direction) const = 0;
 
-        virtual void changeAction(const Being::Action &action) const = 0;
+        virtual void changeAction(const BeingAction::Action &action) const = 0;
 
         virtual void respawn() const = 0;
 
