@@ -32,6 +32,7 @@
 
 #include "net/tmwa/messageout.h"
 #include "net/tmwa/protocol.h"
+#include "net/tmwa/sprite.h"
 
 #include "resources/iteminfo.h"
 
@@ -94,7 +95,7 @@ BeingHandler::BeingHandler(const bool enableSync) :
 
 void BeingHandler::requestNameById(const int id) const
 {
-    MessageOut outMsg(0x0094);
+    MessageOut outMsg(CMSG_NAME_REQUEST);
     outMsg.writeInt32(id);  // readLong(2));
 }
 
@@ -249,8 +250,12 @@ void BeingHandler::undress(Being *const being) const
 void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
                                           const bool look2) const
 {
+    BLOCK_START("BeingHandler::processBeingChangeLook")
     if (!actorManager)
+    {
+        BLOCK_END("BeingHandler::processBeingChangeLook")
         return;
+    }
 
     /*
       * SMSG_BEING_CHANGE_LOOKS (0x00c3) and
@@ -266,16 +271,19 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
 
     Being *const dstBeing = actorManager->findBeing(msg.readInt32());
     if (!dstBeing)
+    {
+        BLOCK_END("BeingHandler::processBeingChangeLook")
         return;
+    }
 
-    const int type = msg.readInt8();
-    int id = 0;
+    const uint8_t type = msg.readUInt8();
+    int16_t id = 0;
     int id2 = 0;
     const std::string color;
 
     if (!look2)
     {
-        id = msg.readInt8();
+        id = static_cast<int16_t>(msg.readUInt8());
         id2 = 1;    // default color
     }
     else
@@ -287,11 +295,14 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
             id2 = 1;
     }
 
-    if (dstBeing->getType() == Being::PLAYER)
+    if (dstBeing->getType() == ActorType::PLAYER)
         dstBeing->setOtherTime();
 
-    if (!player_node)
+    if (!localPlayer)
+    {
+        BLOCK_END("BeingHandler::processBeingChangeLook")
         return;
+    }
 
     switch (type)
     {
@@ -301,7 +312,7 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
             break;
         case 1:     // eAthena LOOK_HAIR
         {
-            const int look = id / 256;
+            const uint8_t look = static_cast<uint8_t>(id / 256);
             const int hair = id % 256;
             dstBeing->setHairStyle(SPRITE_HAIR, hair * -1);
             dstBeing->setLook(look);
@@ -311,25 +322,25 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
             dstBeing->setSprite(SPRITE_WEAPON, id, "", 1, true);
             if (!mHideShield)
                 dstBeing->setSprite(SPRITE_SHIELD, id2);
-            player_node->imitateOutfit(dstBeing, SPRITE_SHIELD);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_SHIELD);
             break;
         case 3:     // Change lower headgear for eAthena, pants for us
             dstBeing->setSprite(SPRITE_BOTTOMCLOTHES, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_BOTTOMCLOTHES);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_BOTTOMCLOTHES);
             break;
         case 4:     // Change upper headgear for eAthena, hat for us
             dstBeing->setSprite(SPRITE_HAT, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_HAT);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_HAT);
             break;
         case 5:     // Change middle headgear for eathena, armor for us
             dstBeing->setSprite(SPRITE_TOPCLOTHES, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_TOPCLOTHES);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_TOPCLOTHES);
             break;
         case 6:     // eAthena LOOK_HAIR_COLOR
-            dstBeing->setHairColor(SPRITE_HAIR, id);
+            dstBeing->setHairColor(SPRITE_HAIR, static_cast<uint8_t>(id));
             break;
         case 7:     // Clothes color
             // ignoring it
@@ -340,45 +351,45 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
                 dstBeing->setSprite(SPRITE_SHIELD, id, color,
                     static_cast<unsigned char>(id2));
             }
-            player_node->imitateOutfit(dstBeing, SPRITE_SHIELD);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_SHIELD);
             break;
         case 9:     // eAthena LOOK_SHOES
             dstBeing->setSprite(SPRITE_SHOE, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_SHOE);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_SHOE);
             break;
         case 10:   // LOOK_GLOVES
             dstBeing->setSprite(SPRITE_GLOVES, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_GLOVES);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_GLOVES);
             break;
         case 11:  // LOOK_CAPE
             dstBeing->setSprite(SPRITE_CAPE, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_CAPE);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_CAPE);
             break;
         case 12:
             dstBeing->setSprite(SPRITE_MISC1, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_MISC1);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_MISC1);
             break;
         case 13:
             dstBeing->setSprite(SPRITE_MISC2, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_MISC2);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_MISC2);
             break;
         case 14:
             dstBeing->setSprite(SPRITE_EVOL1, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_EVOL1);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_EVOL1);
             break;
         case 15:
             dstBeing->setSprite(SPRITE_EVOL2, id, color,
                 static_cast<unsigned char>(id2));
-            player_node->imitateOutfit(dstBeing, SPRITE_EVOL2);
+            localPlayer->imitateOutfit(dstBeing, SPRITE_EVOL2);
             break;
         case 16:
-            dstBeing->setLook(id);
+            dstBeing->setLook(static_cast<uint8_t>(id));
             break;
         default:
             logger->log("QQQ3 CHANGE_LOOKS: unsupported type: "
@@ -387,13 +398,17 @@ void BeingHandler::processBeingChangeLook(Net::MessageIn &msg,
             logger->log("name: " + toString(dstBeing->getName()));
             break;
     }
+    BLOCK_END("BeingHandler::processBeingChangeLook")
 }
 
-void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
+void BeingHandler::processNameResponse2(Net::MessageIn &msg)
 {
-    if (!actorManager || !player_node)
+    BLOCK_START("BeingHandler::processNameResponse2")
+    if (!actorManager || !localPlayer)
+    {
+        BLOCK_END("BeingHandler::processNameResponse2")
         return;
-
+    }
 
     const int len = msg.readInt16();
     const int beingId = msg.readInt32();
@@ -401,9 +416,9 @@ void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
     Being *const dstBeing = actorManager->findBeing(beingId);
     if (dstBeing)
     {
-        if (beingId == player_node->getId())
+        if (beingId == localPlayer->getId())
         {
-            player_node->pingResponse();
+            localPlayer->pingResponse();
         }
         else
         {
@@ -411,12 +426,12 @@ void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
             dstBeing->updateGuild();
             dstBeing->addToCache();
 
-            if (dstBeing->getType() == Being::PLAYER)
+            if (dstBeing->getType() == ActorType::PLAYER)
                 dstBeing->updateColors();
 
-            if (player_node)
+            if (localPlayer)
             {
-                const Party *const party = player_node->getParty();
+                const Party *const party = localPlayer->getParty();
                 if (party && party->isMember(dstBeing->getId()))
                 {
                     PartyMember *const member = party->getMember(
@@ -425,17 +440,22 @@ void BeingHandler::processNameResponse2(Net::MessageIn &msg) const
                     if (member)
                         member->setName(dstBeing->getName());
                 }
-                player_node->checkNewName(dstBeing);
+                localPlayer->checkNewName(dstBeing);
             }
         }
     }
+    BLOCK_END("BeingHandler::processNameResponse2")
 }
 
 void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
                                            const int msgType) const
 {
-    if (!actorManager || !player_node)
+    BLOCK_START("BeingHandler::processPlayerMoveUpdate")
+    if (!actorManager || !localPlayer)
+    {
+        BLOCK_END("BeingHandler::processPlayerMoveUpdate")
         return;
+    }
 
     // An update about a player, potentially including movement.
     const int id = msg.readInt32();
@@ -453,12 +473,18 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
     if (!dstBeing)
     {
         if (actorManager->isBlocked(id) == true)
+        {
+            BLOCK_END("BeingHandler::processPlayerMoveUpdate")
             return;
+        }
 
         dstBeing = createBeing(id, job);
 
         if (!dstBeing)
+        {
+            BLOCK_END("BeingHandler::processPlayerMoveUpdate")
             return;
+        }
     }
     else if (disguiseId)
     {
@@ -474,7 +500,7 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
             dstBeing->setDirection(dir);
     }
 
-    if (Party *const party = player_node->getParty())
+    if (Party *const party = localPlayer->getParty())
     {
         if (party->isMember(id))
             dstBeing->setParty(party);
@@ -482,8 +508,8 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
 
     dstBeing->setWalkSpeed(Vector(speed, speed, 0));
 
-    const int hairStyle = msg.readInt8();
-    const int look = msg.readInt8();
+    const uint8_t hairStyle = msg.readUInt8();
+    const uint8_t look = msg.readUInt8();
     dstBeing->setSubtype(job, look);
     const uint16_t weapon = msg.readInt16();
     const uint16_t shield = msg.readInt16();
@@ -494,15 +520,15 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
 
     const uint16_t headTop = msg.readInt16();
     const uint16_t headMid = msg.readInt16();
-    const int hairColor = msg.readInt8();
-    msg.readInt8();  // free
+    const uint8_t hairColor = msg.readUInt8();
+    msg.readUInt8();  // free
 
-    unsigned char colors[9];
-    colors[0] = msg.readInt8();
-    colors[1] = msg.readInt8();
-    colors[2] = msg.readInt8();
+    uint8_t colors[9];
+    colors[0] = msg.readUInt8();
+    colors[1] = msg.readUInt8();
+    colors[2] = msg.readUInt8();
 
-    msg.readInt8();  // unused
+    msg.readUInt8();  // unused
 
     const int guild = msg.readInt32();  // guild
 
@@ -517,10 +543,10 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
     msg.readInt16();  // emblem
     msg.readInt16();  // manner
     dstBeing->setStatusEffectBlock(32, msg.readInt16());  // opt3
-    msg.readInt8();   // karma
+    msg.readUInt8();   // karma
     // reserving bit for future usage
     dstBeing->setGender(Being::intToGender(
-        static_cast<uint8_t>(msg.readInt8() & 3)));
+        static_cast<uint8_t>(msg.readUInt8() & 3)));
 
     if (!disguiseId)
     {
@@ -545,14 +571,14 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
             ItemDB::get(-hairStyle).getDyeColorsString(hairColor));
         dstBeing->setHairColor(hairColor);
     }
-    player_node->imitateOutfit(dstBeing);
+    localPlayer->imitateOutfit(dstBeing);
 
     if (msgType == 3)
     {
         uint16_t srcX, srcY, dstX, dstY;
         msg.readCoordinatePair(srcX, srcY, dstX, dstY);
 
-        player_node->followMoveTo(dstBeing, srcX, srcY, dstX, dstY);
+        localPlayer->followMoveTo(dstBeing, srcX, srcY, dstX, dstY);
 
         dstBeing->setTileCoords(srcX, srcY);
         dstBeing->setDestination(dstX, dstY);
@@ -568,11 +594,11 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
                 dstBeing->setDirectionDelayed(static_cast<uint8_t>(d));
         }
 
-        if (player_node->getCurrentAction() != Being::STAND)
-            player_node->imitateAction(dstBeing, Being::STAND);
-        if (player_node->getDirection() != dstBeing->getDirection())
+        if (localPlayer->getCurrentAction() != BeingAction::STAND)
+            localPlayer->imitateAction(dstBeing, BeingAction::STAND);
+        if (localPlayer->getDirection() != dstBeing->getDirection())
         {
-            player_node->imitateDirection(dstBeing,
+            localPlayer->imitateDirection(dstBeing,
                 dstBeing->getDirection());
         }
     }
@@ -584,7 +610,7 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
         dstBeing->setTileCoords(x, y);
         dstBeing->setDirection(dir);
 
-        player_node->imitateDirection(dstBeing, dir);
+        localPlayer->imitateDirection(dstBeing, dir);
     }
 
     const uint16_t gmstatus = msg.readInt16();
@@ -594,25 +620,25 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
 
     if (msgType == 1 || msgType == 2)
     {
-        const int type = msg.readInt8();
+        const uint8_t type = msg.readUInt8();
         switch (type)
         {
             case 0:
-                dstBeing->setAction(Being::STAND, 0);
-                player_node->imitateAction(dstBeing, Being::STAND);
+                dstBeing->setAction(BeingAction::STAND, 0);
+                localPlayer->imitateAction(dstBeing, BeingAction::STAND);
                 break;
 
             case 1:
-                if (dstBeing->getCurrentAction() != Being::DEAD)
+                if (dstBeing->getCurrentAction() != BeingAction::DEAD)
                 {
-                    dstBeing->setAction(Being::DEAD, 0);
+                    dstBeing->setAction(BeingAction::DEAD, 0);
                     dstBeing->recalcSpritesOrder();
                 }
                 break;
 
             case 2:
-                dstBeing->setAction(Being::SIT, 0);
-                player_node->imitateAction(dstBeing, Being::SIT);
+                dstBeing->setAction(BeingAction::SIT, 0);
+                localPlayer->imitateAction(dstBeing, BeingAction::SIT);
                 break;
 
             default:
@@ -628,16 +654,16 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
     }
     else if (msgType == 3)
     {
-        msg.readInt8();  // unknown
+        msg.readUInt8();  // unknown
     }
 
-    const int level = msg.readInt8();   // Lv
+    const int level = static_cast<int>(msg.readUInt8());   // Lv
     if (level)
         dstBeing->setLevel(level);
 
-    msg.readInt8();  // unknown
+    msg.readUInt8();  // unknown
 
-    if (dstBeing->getType() != Being::PLAYER
+    if (dstBeing->getType() != ActorType::PLAYER
         || msgType != 3)
     {
         dstBeing->setActionTime(tick_time);
@@ -649,22 +675,30 @@ void BeingHandler::processPlayerMoveUpdate(Net::MessageIn &msg,
     dstBeing->setStatusEffectBlock(16, static_cast<uint16_t>(
         statusEffects & 0xffff));
 
-    if (msgType == 3 && dstBeing->getType() == Being::PLAYER)
+    if (msgType == 3 && dstBeing->getType() == ActorType::PLAYER)
         dstBeing->setMoveTime();
+    BLOCK_END("BeingHandler::processPlayerMoveUpdate")
 }
 
-void BeingHandler::processBeingMove3(Net::MessageIn &msg) const
+void BeingHandler::processBeingMove3(Net::MessageIn &msg)
 {
+    BLOCK_START("BeingHandler::processBeingMove3")
     if (serverVersion < 10)
+    {
+        BLOCK_END("BeingHandler::processBeingMove3")
         return;
+    }
 
-    static const int dirx[8] = {0, -1, -1, -1,  0,  1, 1, 1};
-    static const int diry[8] = {1,  1,  0, -1, -1, -1, 0, 1};
+    static const int16_t dirx[8] = {0, -1, -1, -1,  0,  1, 1, 1};
+    static const int16_t diry[8] = {1,  1,  0, -1, -1, -1, 0, 1};
 
     const int len = msg.readInt16() - 14;
     Being *const dstBeing = actorManager->findBeing(msg.readInt32());
     if (!dstBeing)
+    {
+        BLOCK_END("BeingHandler::processBeingMove3")
         return;
+    }
     const int16_t speed = msg.readInt16();
     dstBeing->setWalkSpeed(Vector(speed, speed, 0));
     int16_t x = msg.readInt16();
@@ -690,6 +724,7 @@ void BeingHandler::processBeingMove3(Net::MessageIn &msg) const
         delete [] moves;
     }
     dstBeing->setPath(path);
+    BLOCK_END("BeingHandler::processBeingMove3")
 }
 
 }  // namespace TmwAthena
