@@ -24,8 +24,7 @@
 
 #include "being/localplayer.h"
 
-#include "gui/widgets/tabs/chattab.h"
-
+#include "net/tmwa/messageout.h"
 #include "net/tmwa/protocol.h"
 
 #include <string>
@@ -110,10 +109,10 @@ void ChatHandler::handleMessage(Net::MessageIn &msg)
 void ChatHandler::talk(const std::string &restrict text,
                        const std::string &restrict channel) const
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
-    const std::string mes = std::string(player_node->getName()).append(
+    const std::string mes = std::string(localPlayer->getName()).append(
         " : ").append(text);
 
     if (serverVersion >= 8 && channel.size() == 3)
