@@ -22,14 +22,16 @@
 #include "net/tmwa/partyhandler.h"
 
 #include "actormanager.h"
-#include "notifications.h"
 #include "notifymanager.h"
+#include "party.h"
 
 #include "being/localplayer.h"
 
 #include "net/tmwa/messageout.h"
 #include "net/tmwa/protocol.h"
 
+#include "resources/notifytypes.h"
+
 #include "debug.h"
 
 extern Net::PartyHandler *partyHandler;
@@ -111,7 +113,7 @@ void PartyHandler::create(const std::string &name) const
     outMsg.writeString(name.substr(0, 23), 24);
 }
 
-void PartyHandler::invite(Being *const being) const
+void PartyHandler::invite(const Being *const being) const
 {
     if (being)
     {
@@ -126,7 +128,7 @@ void PartyHandler::invite(const std::string &name) const
         return;
 
     const Being *const being = actorManager->findBeingByName(
-        name, Being::PLAYER);
+        name, ActorType::PLAYER);
     if (being)
     {
         MessageOut outMsg(CMSG_PARTY_INVITE);
@@ -137,10 +139,10 @@ void PartyHandler::invite(const std::string &name) const
 void PartyHandler::inviteResponse(const std::string &inviter A_UNUSED,
                                   const bool accept) const
 {
-    if (player_node)
+    if (localPlayer)
     {
         MessageOut outMsg(CMSG_PARTY_INVITED);
-        outMsg.writeInt32(player_node->getId());
+        outMsg.writeInt32(localPlayer->getId());
         outMsg.writeInt32(accept ? 1 : 0);
     }
 }
@@ -150,7 +152,7 @@ void PartyHandler::leave() const
     MessageOut outMsg(CMSG_PARTY_LEAVE);
 }
 
-void PartyHandler::kick(Being *const being) const
+void PartyHandler::kick(const Being *const being) const
 {
     if (being)
     {
@@ -168,7 +170,7 @@ void PartyHandler::kick(const std::string &name) const
     const PartyMember *const m = Ea::taParty->getMember(name);
     if (!m)
     {
-        NotifyManager::notify(NotifyManager::PARTY_USER_NOT_IN_PARTY, name);
+        NotifyManager::notify(NotifyTypes::PARTY_USER_NOT_IN_PARTY, name);
         return;
     }
 
@@ -184,9 +186,9 @@ void PartyHandler::chat(const std::string &text) const
     outMsg.writeString(text, static_cast<int>(text.length()));
 }
 
-void PartyHandler::setShareExperience(const PartyShare share) const
+void PartyHandler::setShareExperience(const Net::PartyShare::Type share) const
 {
-    if (share == PARTY_SHARE_NOT_POSSIBLE)
+    if (share == Net::PartyShare::NOT_POSSIBLE)
         return;
 
     MessageOut outMsg(CMSG_PARTY_SETTINGS);
@@ -194,9 +196,9 @@ void PartyHandler::setShareExperience(const PartyShare share) const
     outMsg.writeInt16(static_cast<int16_t>(mShareItems));
 }
 
-void PartyHandler::setShareItems(const PartyShare share) const
+void PartyHandler::setShareItems(const Net::PartyShare::Type share) const
 {
-    if (share == PARTY_SHARE_NOT_POSSIBLE)
+    if (share == Net::PartyShare::NOT_POSSIBLE)
         return;
 
     MessageOut outMsg(CMSG_PARTY_SETTINGS);
