@@ -26,16 +26,23 @@
 #include "game.h"
 #include "soundmanager.h"
 
+#include "being/attributes.h"
+#include "being/beingflag.h"
+
 #include "net/net.h"
 
+#include "net/tmwa/attrs.h"
 #include "net/tmwa/inventoryhandler.h"
 #include "net/tmwa/messageout.h"
 #include "net/tmwa/protocol.h"
 
 #include "gui/windows/whoisonline.h"
 
+#include "gui/onlineplayer.h"
 #include "gui/viewport.h"
 
+#include "resources/map/map.h"
+
 #include "debug.h"
 
 extern Net::PlayerHandler *playerHandler;
@@ -161,7 +168,7 @@ void PlayerHandler::increaseAttribute(const int attr) const
 
 void PlayerHandler::increaseSkill(const uint16_t skillId) const
 {
-    if (PlayerInfo::getAttribute(PlayerInfo::SKILL_POINTS) <= 0)
+    if (PlayerInfo::getAttribute(Attributes::SKILL_POINTS) <= 0)
         return;
 
     MessageOut outMsg(CMSG_SKILL_LEVELUP_REQUEST);
@@ -197,23 +204,23 @@ void PlayerHandler::setDestination(const int x, const int y,
         static_cast<unsigned char>(direction));
 }
 
-void PlayerHandler::changeAction(const Being::Action &action) const
+void PlayerHandler::changeAction(const BeingAction::Action &action) const
 {
     char type;
     switch (action)
     {
-        case Being::SIT:
+        case BeingAction::SIT:
             type = 2;
             break;
-        case Being::STAND:
+        case BeingAction::STAND:
             type = 3;
             break;
         default:
-        case Being::MOVE:
-        case Being::ATTACK:
-        case Being::DEAD:
-        case Being::HURT:
-        case Being::SPAWN:
+        case BeingAction::MOVE:
+        case BeingAction::ATTACK:
+        case BeingAction::DEAD:
+        case BeingAction::HURT:
+        case BeingAction::SPAWN:
             return;
     }
 
@@ -233,7 +240,7 @@ void PlayerHandler::requestOnlineList() const
     MessageOut outMsg(CMSG_ONLINE_LIST);
 }
 
-void PlayerHandler::processOnlineList(Net::MessageIn &msg) const
+void PlayerHandler::processOnlineList(Net::MessageIn &msg)
 {
     if (!whoIsOnline)
         return;
@@ -263,7 +270,8 @@ void PlayerHandler::processOnlineList(Net::MessageIn &msg) const
     if (serverVersion >= 4)
         addVal = 3;
 
-    while (buf - start + 1 < size && *(buf + addVal))
+    while (buf - start + 1 < size
+           && *(buf + static_cast<size_t>(addVal)))
     {
         unsigned char status = 255;
         unsigned char ver = 0;
@@ -278,17 +286,17 @@ void PlayerHandler::processOnlineList(Net::MessageIn &msg) const
         }
         buf ++;
 
-        int gender = GENDER_UNSPECIFIED;
+        unsigned char gender = Gender::UNSPECIFIED;
         if (serverVersion >= 4)
         {
             if (config.getBoolValue("showgender"))
             {
-                if (status & Being::FLAG_GENDER_MALE)
-                    gender = GENDER_MALE;
-                else if (status & Being::FLAG_GENDER_OTHER)
-                    gender = GENDER_OTHER;
+                if (status & BeingFlag::GENDER_MALE)
+                    gender = Gender::MALE;
+                else if (status & BeingFlag::GENDER_OTHER)
+                    gender = Gender::OTHER;
                 else
-                    gender = GENDER_FEMALE;
+                    gender = Gender::FEMALE;
             }
         }
         arr.push_back(new OnlinePlayer(static_cast<const char*>(buf),
@@ -309,7 +317,7 @@ void PlayerHandler::updateStatus(const uint8_t status) const
     outMsg.writeInt8(0);
 }
 
-void PlayerHandler::processMapMask(Net::MessageIn &msg) const
+void PlayerHandler::processMapMask(Net::MessageIn &msg)
 {
     const int mask = msg.readInt32();
     msg.readInt32();  // unused
@@ -318,7 +326,7 @@ void PlayerHandler::processMapMask(Net::MessageIn &msg) const
         map->setMask(mask);
 }
 
-void PlayerHandler::processMapMusic(Net::MessageIn &msg) const
+void PlayerHandler::processMapMusic(Net::MessageIn &msg)
 {
     const int size = msg.readInt16() - 5;
     const std::string music = msg.readString(size);
