@@ -29,9 +29,11 @@
 
 #include "localconsts.h"
 
+class Item;
+
 namespace Net
 {
-class TradeHandler
+class TradeHandler notfinal
 {
     public:
         virtual ~TradeHandler()
