@@ -21,7 +21,6 @@
 #include "notifymanager.h"
 
 #include "guildmanager.h"
-#include "notifications.h"
 #include "soundmanager.h"
 
 #include "being/localplayer.h"
@@ -32,13 +31,18 @@
 #include "net/net.h"
 #include "net/partyhandler.h"
 
+#include "resources/notifications.h"
+#include "resources/notifytypes.h"
+
 #include "resources/db/sounddb.h"
 
+#include "debug.h"
+
 namespace NotifyManager
 {
     static ChatTab *getGuildTab()
     {
-        const Guild *const guild = player_node->getGuild();
+        const Guild *const guild = localPlayer->getGuild();
         if (guild)
         {
             if (guild->getServerGuild())
@@ -52,50 +56,50 @@ namespace NotifyManager
     static void chatLog(ChatTab *const tab, const std::string &str)
     {
         if (tab)
-            tab->chatLog(str, BY_SERVER);
+            tab->chatLog(str, ChatMsgType::BY_SERVER);
         else if (debugChatTab)
-            debugChatTab->chatLog(str, BY_SERVER);
+            debugChatTab->chatLog(str, ChatMsgType::BY_SERVER);
     }
 
     void notify(const unsigned int message)
     {
-        if (message >= TYPE_END || !localChatTab)
+        if (message >= NotifyTypes::TYPE_END || !localChatTab)
             return;
         const NotificationInfo &info = notifications[message];
         switch (info.flags)
         {
-            case EMPTY:
+            case NotifyFlags::EMPTY:
                 localChatTab->chatLog(gettext(info.text),
-                    BY_SERVER);
+                    ChatMsgType::BY_SERVER);
                 break;
 
-            case GUILD:
+            case NotifyFlags::GUILD:
             {
-                if (!player_node)
+                if (!localPlayer)
                     return;
                 ChatTab *const tab = getGuildTab();
                 chatLog(tab, gettext(info.text));
                 break;
             }
 
-            case PARTY:
+            case NotifyFlags::PARTY:
             {
                 ChatTab *const tab = Net::getPartyHandler()->getTab();
                 chatLog(tab, gettext(info.text));
                 break;
             }
 
-            case SPEECH:
+            case NotifyFlags::SPEECH:
             {
-                if (player_node)
-                    player_node->setSpeech(gettext(info.text));
+                if (localPlayer)
+                    localPlayer->setSpeech(gettext(info.text));
                 break;
             }
 
-            case INT:
-            case STRING:
-            case GUILD_STRING:
-            case PARTY_STRING:
+            case NotifyFlags::INT:
+            case NotifyFlags::STRING:
+            case NotifyFlags::GUILD_STRING:
+            case NotifyFlags::PARTY_STRING:
             default:
                 break;
         }
@@ -104,47 +108,47 @@ namespace NotifyManager
 
     void notify(const unsigned int message, const int num)
     {
-        if (message >= TYPE_END || !localChatTab)
+        if (message >= NotifyTypes::TYPE_END || !localChatTab)
             return;
         const NotificationInfo &info = notifications[message];
-        if (info.flags == INT)
+        if (info.flags == NotifyFlags::INT)
         {
             localChatTab->chatLog(strprintf(gettext(info.text),
-                num), BY_SERVER);
+                num), ChatMsgType::BY_SERVER);
         }
         soundManager.playSfx(SoundDB::getSound(message));
     }
 
     void notify(const unsigned int message, const std::string &str)
     {
-        if (message >= TYPE_END || !localChatTab)
+        if (message >= NotifyTypes::TYPE_END || !localChatTab)
             return;
         const NotificationInfo &info = notifications[message];
         switch (info.flags)
         {
-            case STRING:
+            case NotifyFlags::STRING:
             {
                 localChatTab->chatLog(strprintf(gettext(info.text),
-                    str.c_str()), BY_SERVER);
+                    str.c_str()), ChatMsgType::BY_SERVER);
                 break;
             }
-            case GUILD_STRING:
+            case NotifyFlags::GUILD_STRING:
             {
                 ChatTab *const tab = getGuildTab();
                 chatLog(tab, strprintf(gettext(info.text), str.c_str()));
                 break;
             }
-            case PARTY_STRING:
+            case NotifyFlags::PARTY_STRING:
             {
                 ChatTab *const tab = Net::getPartyHandler()->getTab();
                 chatLog(tab, strprintf(gettext(info.text), str.c_str()));
                 break;
             }
-            case EMPTY:
-            case INT:
-            case GUILD:
-            case PARTY:
-            case SPEECH:
+            case NotifyFlags::EMPTY:
+            case NotifyFlags::INT:
+            case NotifyFlags::GUILD:
+            case NotifyFlags::PARTY:
+            case NotifyFlags::SPEECH:
             default:
                 break;
         }
@@ -153,7 +157,7 @@ namespace NotifyManager
 
     int getIndexBySound(const std::string &sound)
     {
-        for (int f = 0; f < TYPE_END; f ++)
+        for (int f = 0; f < NotifyTypes::TYPE_END; f ++)
         {
             if (notifications[f].sound == sound)
                 return f;
