@@ -25,6 +25,15 @@
 #include "configuration.h"
 #include "logger.h"
 
+#include "resources/attack.h"
+#include "resources/spriteaction.h"
+#include "resources/spritereference.h"
+
+#include "resources/db/colordb.h"
+
+#include "resources/map/blockmask.h"
+
+#include "utils/delete2.h"
 #include "utils/dtor.h"
 #include "utils/gettext.h"
 
@@ -39,14 +48,14 @@ BeingInfo::BeingInfo() :
     mDisplay(),
     // TRANSLATORS: being info default name
     mName(_("unnamed")),
-    mTargetCursorSize(ActorSprite::TC_MEDIUM),
+    mTargetCursorSize(TargetCursorSize::MEDIUM),
     mHoverCursor(Cursor::CURSOR_POINTER),
     mSounds(),
     mAttacks(),
-    mWalkMask(Map::BLOCKMASK_WALL | Map::BLOCKMASK_CHARACTER
-              | Map::BLOCKMASK_MONSTER | Map::BLOCKMASK_AIR
-              | Map::BLOCKMASK_WATER),
-    mBlockType(Map::BLOCKTYPE_CHARACTER),
+    mWalkMask(BlockMask::WALL | BlockMask::CHARACTER
+              | BlockMask::MONSTER | BlockMask::AIR
+              | BlockMask::WATER),
+    mBlockType(BlockType::CHARACTER),
     mColors(nullptr),
     mTargetOffsetX(0),
     mTargetOffsetY(0),
@@ -70,10 +79,13 @@ BeingInfo::BeingInfo() :
     mMoveOffsetY(0),
     mDeadOffsetX(0),
     mDeadOffsetY(0),
+    mAttackOffsetX(0),
+    mAttackOffsetY(0),
     mThinkTime(50),
     mDirectionType(1),
     mSitDirectionType(1),
     mDeadDirectionType(1),
+    mAttackDirectionType(1),
     mStaticMaxHP(false),
     mTargetSelection(true)
 {
@@ -100,25 +112,25 @@ void BeingInfo::setTargetCursorSize(const std::string &size)
 {
     if (size == "small")
     {
-        setTargetCursorSize(ActorSprite::TC_SMALL);
+        setTargetCursorSize(TargetCursorSize::SMALL);
     }
     else if (size == "medium")
     {
-        setTargetCursorSize(ActorSprite::TC_MEDIUM);
+        setTargetCursorSize(TargetCursorSize::MEDIUM);
     }
     else if (size == "large")
     {
-        setTargetCursorSize(ActorSprite::TC_LARGE);
+        setTargetCursorSize(TargetCursorSize::LARGE);
     }
     else
     {
         logger->log("Unknown target cursor type \"%s\" for %s - using medium "
                     "sized one", size.c_str(), getName().c_str());
-        setTargetCursorSize(ActorSprite::TC_MEDIUM);
+        setTargetCursorSize(TargetCursorSize::MEDIUM);
     }
 }
 
-void BeingInfo::addSound(const ItemSoundEvent event,
+void BeingInfo::addSound(const ItemSoundEvent::Type event,
                          const std::string &filename,
                          const int delay)
 {
@@ -129,7 +141,7 @@ void BeingInfo::addSound(const ItemSoundEvent event,
         mSounds[event]->push_back(SoundInfo("sfx/" + filename, delay));
 }
 
-const SoundInfo &BeingInfo::getSound(const ItemSoundEvent event) const
+const SoundInfo &BeingInfo::getSound(const ItemSoundEvent::Type event) const
 {
     static SoundInfo emptySound("", 0);
 
@@ -142,7 +154,7 @@ const SoundInfo &BeingInfo::getSound(const ItemSoundEvent event) const
     if (!vect || vect->empty())
         return emptySound;
     else
-        return vect->at(rand() % vect->size());
+        return vect->at(static_cast<unsigned int>(rand()) % vect->size());
 }
 
 const Attack *BeingInfo::getAttack(const int id) const
@@ -151,9 +163,12 @@ const Attack *BeingInfo::getAttack(const int id) const
     return (i == mAttacks.end()) ? empty : (*i).second;
 }
 
-void BeingInfo::addAttack(const int id, std::string action,
-                          std::string skyAction, std::string waterAction,
-                          const int effectId, const int hitEffectId,
+void BeingInfo::addAttack(const int id,
+                          const std::string &action,
+                          const std::string &skyAction,
+                          const std::string &waterAction,
+                          const int effectId,
+                          const int hitEffectId,
                           const int criticalHitEffectId,
                           const int missEffectId,
                           const std::string &missileParticle)
@@ -165,10 +180,8 @@ void BeingInfo::addAttack(const int id, std::string action,
 
 void BeingInfo::clear()
 {
-    delete unknown;
-    unknown = nullptr;
-    delete empty;
-    empty = nullptr;
+    delete2(unknown);
+    delete2(empty);
 }
 
 void BeingInfo::init()
