@@ -23,43 +23,23 @@
 #ifndef RESOURCES_BEINGINFO_H
 #define RESOURCES_BEINGINFO_H
 
-#include "being/actorsprite.h"
-
 #include "resources/cursor.h"
-#include "resources/soundinfo.h"
 
-#include "resources/db/colordb.h"
+#include "being/targetcursorsize.h"
+
+#include "resources/soundinfo.h"
+#include "resources/spritedisplay.h"
+
+#include "resources/map/blocktype.h"
 
 #include <map>
 
-struct Attack final
+struct Attack;
+
+namespace ColorDB
 {
-    std::string mAction;
-    std::string mSkyAction;
-    std::string mWaterAction;
-    int mEffectId;
-    int mHitEffectId;
-    int mCriticalHitEffectId;
-    int mMissEffectId;
-    std::string mMissileParticle;
-
-    Attack(const std::string &action, const std::string &skyAction,
-           const std::string &waterAction, const int effectId,
-           const int hitEffectId, const int criticalHitEffectId,
-           const int missEffectId, const std::string &missileParticle) :
-        mAction(action),
-        mSkyAction(skyAction),
-        mWaterAction(waterAction),
-        mEffectId(effectId),
-        mHitEffectId(hitEffectId),
-        mCriticalHitEffectId(criticalHitEffectId),
-        mMissEffectId(missEffectId),
-        mMissileParticle(missileParticle)
-    {
-    }
-
-    A_DELETE_COPY(Attack)
-};
+    class ItemColor;
+}
 
 typedef std::map<int, Attack*> Attacks;
 
@@ -95,7 +75,7 @@ class BeingInfo final
 
         void setTargetCursorSize(const std::string &size);
 
-        void setTargetCursorSize(const ActorSprite::TargetCursorSize
+        void setTargetCursorSize(const TargetCursorSize::Size
                                  &targetSize)
         { mTargetCursorSize = targetSize; }
 
@@ -108,18 +88,23 @@ class BeingInfo final
         Cursor::Cursor getHoverCursor() const A_WARN_UNUSED
         { return mHoverCursor; }
 
-        ActorSprite::TargetCursorSize getTargetCursorSize() const A_WARN_UNUSED
+        TargetCursorSize::Size getTargetCursorSize() const A_WARN_UNUSED
         { return mTargetCursorSize; }
 
-        void addSound(const ItemSoundEvent event, const std::string &filename,
+        void addSound(const ItemSoundEvent::Type event,
+                      const std::string &filename,
                       const int delay);
 
-        const SoundInfo &getSound(const ItemSoundEvent event)
+        const SoundInfo &getSound(const ItemSoundEvent::Type event)
                                   const A_WARN_UNUSED;
 
-        void addAttack(const int id, std::string action, std::string skyAttack,
-                       std::string waterAttack, const int effectId,
-                       const int hitEffectId, const int criticalHitEffectId,
+        void addAttack(const int id,
+                       const std::string &action,
+                       const std::string &skyAttack,
+                       const std::string &waterAttack,
+                       const int effectId,
+                       const int hitEffectId,
+                       const int criticalHitEffectId,
                        const int missEffectId,
                        const std::string &missileParticle);
 
@@ -134,10 +119,10 @@ class BeingInfo final
         unsigned char getWalkMask() const A_WARN_UNUSED
         { return mWalkMask; }
 
-        void setBlockType(const Map::BlockType &blockType)
+        void setBlockType(const BlockType::BlockType &blockType)
         { mBlockType = blockType; }
 
-        Map::BlockType getBlockType() const A_WARN_UNUSED
+        BlockType::BlockType getBlockType() const A_WARN_UNUSED
         { return mBlockType; }
 
         void setTargetOffsetX(const int n)
@@ -284,6 +269,18 @@ class BeingInfo final
         int getDeadOffsetY() const A_WARN_UNUSED
         { return mDeadOffsetY; }
 
+        void setAttackOffsetX(const int n)
+        { mAttackOffsetX = n; }
+
+        int getAttackOffsetX() const A_WARN_UNUSED
+        { return mAttackOffsetX; }
+
+        void setAttackOffsetY(const int n)
+        { mAttackOffsetY = n; }
+
+        int getAttackOffsetY() const A_WARN_UNUSED
+        { return mAttackOffsetY; }
+
         void setThinkTime(const int n)
         { mThinkTime = n; }
 
@@ -308,6 +305,12 @@ class BeingInfo final
         int getDeadDirectionType() const A_WARN_UNUSED
         { return mDeadDirectionType; }
 
+        void setAttackDirectionType(const int n)
+        { mAttackDirectionType = n; }
+
+        int getAttackDirectionType() const A_WARN_UNUSED
+        { return mAttackDirectionType; }
+
         void setColorsList(const std::string &name);
 
         std::string getColor(const int idx) const A_WARN_UNUSED;
@@ -319,12 +322,12 @@ class BeingInfo final
     private:
         SpriteDisplay mDisplay;
         std::string mName;
-        ActorSprite::TargetCursorSize mTargetCursorSize;
+        TargetCursorSize::Size mTargetCursorSize;
         Cursor::Cursor mHoverCursor;
         ItemSoundEvents mSounds;
         Attacks mAttacks;
         unsigned char mWalkMask;
-        Map::BlockType mBlockType;
+        BlockType::BlockType mBlockType;
         const std::map <int, ColorDB::ItemColor> *mColors;
         int mTargetOffsetX;
         int mTargetOffsetY;
@@ -348,10 +351,13 @@ class BeingInfo final
         int mMoveOffsetY;
         int mDeadOffsetX;
         int mDeadOffsetY;
+        int mAttackOffsetX;
+        int mAttackOffsetY;
         int mThinkTime;
         int mDirectionType;
         int mSitDirectionType;
         int mDeadDirectionType;
+        int mAttackDirectionType;
         bool mStaticMaxHP;
         bool mTargetSelection;
 };
