@@ -1,5 +1,5 @@
 /*
- *  Emote database
+ *  The ManaPlus Client
  *  Copyright (C) 2009  Aethyra Development Team
  *  Copyright (C) 2011-2014  The ManaPlus Developers
  *
@@ -28,6 +28,8 @@
 #include "configuration.h"
 
 #include "resources/beingcommon.h"
+#include "resources/emoteinfo.h"
+#include "resources/emotesprite.h"
 
 #include "debug.h"
 
@@ -49,6 +51,7 @@ void EmoteDB::load()
     unknownSprite->sprite = AnimatedSprite::load(
         paths.getStringValue("spriteErrorFile"));
     unknownSprite->name = "unknown";
+    unknownSprite->id = 0;
     mUnknown.sprites.push_back(unknownSprite);
 
     logger->log1("Initializing emote database...");
@@ -122,6 +125,7 @@ void EmoteDB::loadXmlFile(const std::string &fileName)
                     XML::getProperty(spriteNode, "variant", 0));
                 currentSprite->name = XML::langProperty(
                     spriteNode, "name", "");
+                currentSprite->id = id;
                 currentInfo->sprites.push_back(currentSprite);
             }
             else if (xmlNameEqual(spriteNode, "particlefx"))
@@ -194,6 +198,7 @@ void EmoteDB::loadSpecialXmlFile(const std::string &fileName)
                     XML::getProperty(spriteNode, "variant", 0));
                 currentSprite->name = XML::langProperty(
                     spriteNode, "name", "");
+                currentSprite->id = id;
                 currentInfo->sprites.push_back(currentSprite);
             }
             else if (xmlNameEqual(spriteNode, "particlefx"))
@@ -292,3 +297,18 @@ const int &EmoteDB::getLast()
 {
     return mLastEmote;
 }
+
+int EmoteDB::size()
+{
+    return static_cast<signed int>(mEmoteInfos.size());
+}
+
+int EmoteDB::getIdByIndex(const int index)
+{
+    if (index < 0 || index >= static_cast<signed int>(mEmoteInfos.size()))
+        return 0;
+    EmoteInfos::const_iterator it = mEmoteInfos.begin();
+    for (int f = 0; f < index; f ++)
+        ++ it;
+    return (*it).first;
+}
