@@ -22,14 +22,23 @@
 
 #include "resources/db/itemdb.h"
 
+#include "resources/map/mapconsts.h"
+
 #include "configuration.h"
 #include "logger.h"
 
 #include "resources/beingcommon.h"
 #include "resources/iteminfo.h"
+#include "resources/itemtypemapdata.h"
+#include "resources/spritedirection.h"
+#include "resources/spritereference.h"
 
+#include "resources/db/itemdbstat.h"
+
+#include "utils/delete2.h"
 #include "utils/dtor.h"
-#include "utils/gettext.h"
+
+#include "utils/translation/podict.h"
 
 #include "debug.h"
 
@@ -42,7 +51,7 @@ namespace
     bool mConstructed = false;
     StringVect mTagNames;
     std::map<std::string, int> mTags;
-    std::map<std::string, ItemSoundEvent> mSoundNames;
+    std::map<std::string, ItemSoundEvent::Type> mSoundNames;
 }  // namespace
 
 extern int serverVersion;
@@ -64,8 +73,20 @@ static int parseDirectionName(const std::string &name);
 
 namespace
 {
-    struct FieldType
+    struct FieldType final
     {
+#ifdef ADVGCC
+        FieldType(const char *const name0,
+                  const char *const description0,
+                  const bool sign0) :
+            name(name0),
+            description(description0),
+            sign(sign0)
+        { }
+
+        A_DELETE_COPY(FieldType)
+#endif
+
         const char *name;
         const char *description;
         const bool sign;
@@ -73,16 +94,30 @@ namespace
 
     static const FieldType fields[] =
     {
-        // TRANSLATORS: item info label
-        { "attack",  N_("Attack %s"),  true },
-        // TRANSLATORS: item info label
-        { "defense", N_("Defense %s"), true },
-        // TRANSLATORS: item info label
-        { "hp",      N_("HP %s"),      true },
-        // TRANSLATORS: item info label
-        { "mp",      N_("MP %s"),      true },
-        // TRANSLATORS: item info label
-        { "level",   N_("Level %s"),   false }
+        // TRANSLATORS: item info label (attack)
+        { "attack",   N_("Attack %s"),     true },
+        // TRANSLATORS: item info label (attack)
+        { "criticalattack", N_("Critical attack %s"), true },
+        // TRANSLATORS: item info label (magic attack)
+        { "mattack",  N_("M. Attack %s"),  true },
+        // TRANSLATORS: item info label (defence)
+        { "defense",  N_("Defense %s"),    true },
+        // TRANSLATORS: item info label (defence)
+        { "criticaldefense", N_("Critical defense %s"), true },
+        // TRANSLATORS: item info label (magic defence)
+        { "mdefense", N_("M. Defense %s"), true },
+        // TRANSLATORS: item info label (health)
+        { "hp",       N_("HP %s"),         true },
+        // TRANSLATORS: item info label (mana)
+        { "mp",       N_("MP %s"),         true },
+        // TRANSLATORS: item info label (level)
+        { "level",    N_("Level %s"),      false },
+        // TRANSLATORS: item info label (moving speed)
+        { "speed",    N_("Speed %s"),      true },
+        // TRANSLATORS: item info label (health)
+        { "range",    N_("Range %s"),      true },
+        // TRANSLATORS: item info label (health)
+        { "flee",     N_("Flee %s"),       true }
     };
 }  // namespace
 
@@ -93,92 +128,58 @@ void ItemDB::setStatsList(const std::vector<ItemDB::Stat> &stats)
     extraStats = stats;
 }
 
-static ItemType itemTypeFromString(const std::string &name)
+static ItemType::Type itemTypeFromString(const std::string &name)
 {
-    if (name == "generic" || name == "other")
+    const size_t sz = sizeof(itemTypeMap) / sizeof(itemTypeMap[0]);
+    for (size_t f = 0; f < sz; f ++)
     {
-        return ITEM_UNUSABLE;
+        const ItemTypeMap &type = itemTypeMap[f];
+        if (type.name == name)
+            return type.type;
     }
-    else if (name == "usable")
+    logger->log("Unknown item type: " + name);
+    return ItemType::UNUSABLE;
+}
+
+static std::string useButtonFromItemType(const ItemType::Type &type)
+{
+    const size_t sz = sizeof(itemTypeMap) / sizeof(itemTypeMap[0]);
+    for (size_t f = 0; f < sz; f ++)
     {
-        return ITEM_USABLE;
+        const ItemTypeMap &item = itemTypeMap[f];
+        if (item.type == type)
+            return translator->getStr(item.useButton.c_str());
     }
-    else if (name == "equip-1hand")
+    logger->log("Unknown item type");
+    return std::string();
+}
+
+static std::string useButton2FromItemType(const ItemType::Type &type)
+{
+    const size_t sz = sizeof(itemTypeMap) / sizeof(itemTypeMap[0]);
+    for (size_t f = 0; f < sz; f ++)
     {
-        return ITEM_EQUIPMENT_ONE_HAND_WEAPON;
-    }
-    else if (name == "equip-2hand")
-    {
-        return ITEM_EQUIPMENT_TWO_HANDS_WEAPON;
-    }
-    else if (name == "equip-torso")
-    {
-        return ITEM_EQUIPMENT_TORSO;
-    }
-    else if (name == "equip-arms")
-    {
-        return ITEM_EQUIPMENT_ARMS;
-    }
-    else if (name == "equip-head")
-    {
-        return ITEM_EQUIPMENT_HEAD;
-    }
-    else if (name == "equip-legs")
-    {
-        return ITEM_EQUIPMENT_LEGS;
-    }
-    else if (name == "equip-shield")
-    {
-        return ITEM_EQUIPMENT_SHIELD;
-    }
-    else if (name == "equip-ring")
-    {
-        return ITEM_EQUIPMENT_RING;
-    }
-    else if (name == "equip-charm")
-    {
-        return ITEM_EQUIPMENT_CHARM;
-    }
-    else if (name == "equip-necklace" || name == "equip-neck")
-    {
-        return ITEM_EQUIPMENT_NECKLACE;
-    }
-    else if (name == "equip-feet")
-    {
-        return ITEM_EQUIPMENT_FEET;
-    }
-    else if (name == "equip-ammo")
-    {
-        return ITEM_EQUIPMENT_AMMO;
-    }
-    else if (name == "racesprite")
-    {
-        return ITEM_SPRITE_RACE;
-    }
-    else if (name == "hairsprite")
-    {
-        return ITEM_SPRITE_HAIR;
-    }
-    else
-    {
-        logger->log("Unknown item type: " + name);
-        return ITEM_UNUSABLE;
+        const ItemTypeMap &item = itemTypeMap[f];
+        if (item.type == type)
+            return translator->getStr(item.useButton2.c_str());
     }
+    logger->log("Unknown item type");
+    return std::string();
 }
 
 static void initStatic()
 {
     mConstructed = true;
-    mSoundNames["hit"] = SOUND_EVENT_HIT;
-    mSoundNames["strike"] = SOUND_EVENT_MISS;
-    mSoundNames["miss"] = SOUND_EVENT_MISS;
-    mSoundNames["use"] = SOUND_EVENT_USE;
-    mSoundNames["equip"] = SOUND_EVENT_EQUIP;
-    mSoundNames["unequip"] = SOUND_EVENT_UNEQUIP;
-    mSoundNames["drop"] = SOUND_EVENT_DROP;
-    mSoundNames["pickup"] = SOUND_EVENT_PICKUP;
-    mSoundNames["take"] = SOUND_EVENT_TAKE;
-    mSoundNames["put"] = SOUND_EVENT_PUT;
+    mSoundNames["hit"] = ItemSoundEvent::HIT;
+    mSoundNames["strike"] = ItemSoundEvent::MISS;
+    mSoundNames["miss"] = ItemSoundEvent::MISS;
+    mSoundNames["use"] = ItemSoundEvent::USE;
+    mSoundNames["equip"] = ItemSoundEvent::EQUIP;
+    mSoundNames["unequip"] = ItemSoundEvent::UNEQUIP;
+    mSoundNames["drop"] = ItemSoundEvent::DROP;
+    mSoundNames["pickup"] = ItemSoundEvent::PICKUP;
+    mSoundNames["take"] = ItemSoundEvent::TAKE;
+    mSoundNames["put"] = ItemSoundEvent::PUT;
 }
 
 void ItemDB::load()
@@ -209,15 +210,17 @@ void ItemDB::load()
     mUnknown->setName(_("Unknown item"));
     mUnknown->setDisplay(SpriteDisplay());
     std::string errFile = paths.getStringValue("spriteErrorFile");
-    mUnknown->setSprite(errFile, GENDER_MALE, 0);
-    mUnknown->setSprite(errFile, GENDER_FEMALE, 0);
-    mUnknown->setSprite(errFile, GENDER_OTHER, 0);
+    mUnknown->setSprite(errFile, Gender::MALE, 0);
+    mUnknown->setSprite(errFile, Gender::FEMALE, 0);
+    mUnknown->setSprite(errFile, Gender::OTHER, 0);
     mUnknown->addTag(mTags["All"]);
     loadXmlFile(paths.getStringValue("itemsFile"), tagNum);
     loadXmlFile(paths.getStringValue("itemsPatchFile"), tagNum);
 
     StringVect list;
-    BeingCommon::getIncludeFiles(paths.getStringValue("itemsPatchDir"), list);
+    BeingCommon::getIncludeFiles(paths.getStringValue("itemsPatchDir"),
+        list,
+        ".xml");
     FOR_EACH (StringVectCIter, it, list)
         loadXmlFile(*it, tagNum);
 }
@@ -281,6 +284,8 @@ void ItemDB::loadXmlFile(const std::string &fileName, int &tagNum)
         const int pet = XML::getProperty(node, "pet", 0);
         const int maxFloorOffset = XML::getIntProperty(
             node, "maxFloorOffset", mapTileSize, 0, mapTileSize);
+        std::string useButton = XML::getProperty(node, "useButton", "");
+        std::string useButton2 = XML::getProperty(node, "useButton2", "");
         std::string colors;
         if (serverVersion >= 1)
         {
@@ -329,6 +334,12 @@ void ItemDB::loadXmlFile(const std::string &fileName, int &tagNum)
         itemInfo->setName(name.empty() ? _("unnamed") : name);
         itemInfo->setDescription(description);
         itemInfo->setType(itemTypeFromString(typeStr));
+        if (useButton.empty())
+            useButton = useButtonFromItemType(itemInfo->getType());
+        itemInfo->setUseButton(useButton);
+        if (useButton2.empty())
+            useButton2 = useButton2FromItemType(itemInfo->getType());
+        itemInfo->setUseButton2(useButton2);
         itemInfo->addTag(mTags["All"]);
         itemInfo->setPet(pet);
         itemInfo->setProtected(XML::getBoolProperty(
@@ -336,27 +347,27 @@ void ItemDB::loadXmlFile(const std::string &fileName, int &tagNum)
 
         switch (itemInfo->getType())
         {
-            case ITEM_USABLE:
+            case ItemType::USABLE:
                 itemInfo->addTag(mTags["Usable"]);
                 break;
-            case ITEM_UNUSABLE:
+            case ItemType::UNUSABLE:
                 itemInfo->addTag(mTags["Unusable"]);
                 break;
             default:
-            case ITEM_EQUIPMENT_ONE_HAND_WEAPON:
-            case ITEM_EQUIPMENT_TWO_HANDS_WEAPON:
-            case ITEM_EQUIPMENT_TORSO:
-            case ITEM_EQUIPMENT_ARMS:
-            case ITEM_EQUIPMENT_HEAD:
-            case ITEM_EQUIPMENT_LEGS:
-            case ITEM_EQUIPMENT_SHIELD:
-            case ITEM_EQUIPMENT_RING:
-            case ITEM_EQUIPMENT_NECKLACE:
-            case ITEM_EQUIPMENT_FEET:
-            case ITEM_EQUIPMENT_AMMO:
-            case ITEM_EQUIPMENT_CHARM:
-            case ITEM_SPRITE_RACE:
-            case ITEM_SPRITE_HAIR:
+            case ItemType::EQUIPMENT_ONE_HAND_WEAPON:
+            case ItemType::EQUIPMENT_TWO_HANDS_WEAPON:
+            case ItemType::EQUIPMENT_TORSO:
+            case ItemType::EQUIPMENT_ARMS:
+            case ItemType::EQUIPMENT_HEAD:
+            case ItemType::EQUIPMENT_LEGS:
+            case ItemType::EQUIPMENT_SHIELD:
+            case ItemType::EQUIPMENT_RING:
+            case ItemType::EQUIPMENT_NECKLACE:
+            case ItemType::EQUIPMENT_FEET:
+            case ItemType::EQUIPMENT_AMMO:
+            case ItemType::EQUIPMENT_CHARM:
+            case ItemType::SPRITE_RACE:
+            case ItemType::SPRITE_HAIR:
                 itemInfo->addTag(mTags["Equipment"]);
                 break;
         }
@@ -541,8 +552,7 @@ void ItemDB::unload()
 {
     logger->log1("Unloading item database...");
 
-    delete mUnknown;
-    mUnknown = nullptr;
+    delete2(mUnknown);
 
     delete_all(mItemInfos);
     mItemInfos.clear();
@@ -673,40 +683,40 @@ int parseDirectionName(const std::string &name)
     if (name == "down")
     {
         if (serverVersion > 0)
-            id = DIRECTION_DOWN;
+            id = SpriteDirection::DOWN;
         else
             id = -2;
     }
     else if (name == "downleft" || name == "leftdown")
     {
-        id = DIRECTION_DOWNLEFT;
+        id = SpriteDirection::DOWNLEFT;
     }
     else if (name == "left")
     {
-        id = DIRECTION_LEFT;
+        id = SpriteDirection::LEFT;
     }
     else if (name == "upleft" || name == "leftup")
     {
-        id = DIRECTION_UPLEFT;
+        id = SpriteDirection::UPLEFT;
     }
     else if (name == "up")
     {
         if (serverVersion > 0)
-            id = DIRECTION_UP;
+            id = SpriteDirection::UP;
         else
             id = -3;
     }
     else if (name == "upright" || name == "rightup")
     {
-        id = DIRECTION_UPRIGHT;
+        id = SpriteDirection::UPRIGHT;
     }
     else if (name == "right")
     {
-        id = DIRECTION_RIGHT;
+        id = SpriteDirection::RIGHT;
     }
     else if (name == "downright" || name == "rightdown")
     {
-        id = DIRECTION_DOWNRIGHT;
+        id = SpriteDirection::DOWNRIGHT;
     }
     else if (name == "downall")
     {
@@ -733,11 +743,11 @@ void loadSpriteRef(ItemInfo *const itemInfo, const XmlNodePtr node)
 
     const int race = XML::getProperty(node, "race", 0);
     if (gender == "male" || gender == "unisex")
-        itemInfo->setSprite(filename, GENDER_MALE, race);
+        itemInfo->setSprite(filename, Gender::MALE, race);
     if (gender == "female" || gender == "unisex")
-        itemInfo->setSprite(filename, GENDER_FEMALE, race);
+        itemInfo->setSprite(filename, Gender::FEMALE, race);
     if (gender == "other" || gender == "unisex")
-        itemInfo->setSprite(filename, GENDER_OTHER, race);
+        itemInfo->setSprite(filename, Gender::OTHER, race);
 }
 
 void loadSoundRef(ItemInfo *const itemInfo, const XmlNodePtr node)
@@ -747,7 +757,7 @@ void loadSoundRef(ItemInfo *const itemInfo, const XmlNodePtr node)
         node->xmlChildrenNode->content);
     const int delay = XML::getProperty(node, "delay", 0);
 
-    const std::map<std::string, ItemSoundEvent>::const_iterator
+    const std::map<std::string, ItemSoundEvent::Type>::const_iterator
         it = mSoundNames.find(event);
     if (it != mSoundNames.end())
     {
@@ -831,11 +841,11 @@ void loadReplaceSprite(ItemInfo *const itemInfo,
         case -2:
         {
             itemInfo->addReplaceSprite(parseSpriteName(
-                removeSprite), DIRECTION_DOWN);
+                removeSprite), SpriteDirection::DOWN);
             itemInfo->addReplaceSprite(parseSpriteName(
-                removeSprite), DIRECTION_DOWNLEFT);
+                removeSprite), SpriteDirection::DOWNLEFT);
             itemInfo->addReplaceSprite(parseSpriteName(
-                removeSprite), DIRECTION_DOWNRIGHT);
+                removeSprite), SpriteDirection::DOWNRIGHT);
 
             for_each_xml_child_node(itemNode, replaceNode)
             {
@@ -844,17 +854,17 @@ void loadReplaceSprite(ItemInfo *const itemInfo,
                     const int from = XML::getProperty(itemNode, "from", 0);
                     const int to = XML::getProperty(itemNode, "to", 1);
                     std::map<int, int> *mapList = itemInfo->addReplaceSprite(
-                        parseSpriteName(removeSprite), DIRECTION_DOWN);
+                        parseSpriteName(removeSprite), SpriteDirection::DOWN);
                     if (mapList)
                         (*mapList)[from] = to;
 
                     mapList = itemInfo->addReplaceSprite(parseSpriteName(
-                        removeSprite), DIRECTION_DOWNLEFT);
+                        removeSprite), SpriteDirection::DOWNLEFT);
                     if (mapList)
                         (*mapList)[from] = to;
 
                     mapList = itemInfo->addReplaceSprite(parseSpriteName(
-                        removeSprite), DIRECTION_DOWNRIGHT);
+                        removeSprite), SpriteDirection::DOWNRIGHT);
                     if (mapList)
                         (*mapList)[from] = to;
                 }
@@ -864,11 +874,11 @@ void loadReplaceSprite(ItemInfo *const itemInfo,
         case -3:
         {
             itemInfo->addReplaceSprite(parseSpriteName(
-                removeSprite), DIRECTION_UP);
+                removeSprite), SpriteDirection::UP);
             itemInfo->addReplaceSprite(parseSpriteName(
-                removeSprite), DIRECTION_UPLEFT);
+                removeSprite), SpriteDirection::UPLEFT);
             itemInfo->addReplaceSprite(parseSpriteName(
-                removeSprite), DIRECTION_UPRIGHT);
+                removeSprite), SpriteDirection::UPRIGHT);
 
             for_each_xml_child_node(itemNode, replaceNode)
             {
@@ -877,17 +887,17 @@ void loadReplaceSprite(ItemInfo *const itemInfo,
                     const int from = XML::getProperty(itemNode, "from", 0);
                     const int to = XML::getProperty(itemNode, "to", 1);
                     std::map<int, int> *mapList = itemInfo->addReplaceSprite(
-                        parseSpriteName(removeSprite), DIRECTION_UP);
+                        parseSpriteName(removeSprite), SpriteDirection::UP);
                     if (mapList)
                         (*mapList)[from] = to;
 
                     mapList = itemInfo->addReplaceSprite(parseSpriteName(
-                        removeSprite), DIRECTION_UPLEFT);
+                        removeSprite), SpriteDirection::UPLEFT);
                     if (mapList)
                         (*mapList)[from] = to;
 
                     mapList = itemInfo->addReplaceSprite(parseSpriteName(
-                        removeSprite), DIRECTION_UPRIGHT);
+                        removeSprite), SpriteDirection::UPRIGHT);
                     if (mapList)
                         (*mapList)[from] = to;
                 }
