@@ -25,6 +25,9 @@
 #include "logger.h"
 #include "main.h"
 
+#include "resources/dye.h"
+#include "resources/dyepalette.h"
+
 #include "utils/sdlcheckutils.h"
 
 #include <SDL_image.h>
@@ -37,7 +40,7 @@ ImageHelper *surfaceImageHelper = nullptr;
 bool ImageHelper::mEnableAlpha = true;
 RenderType ImageHelper::mUseOpenGL = RENDER_SOFTWARE;
 
-Image *ImageHelper::load(SDL_RWops *const rw) const
+Image *ImageHelper::load(SDL_RWops *const rw)
 {
     SDL_Surface *const tmpImage = loadPng(rw);
     if (!tmpImage)
@@ -52,6 +55,71 @@ Image *ImageHelper::load(SDL_RWops *const rw) const
     return image;
 }
 
+Image *ImageHelper::load(SDL_RWops *const rw, Dye const &dye)
+{
+    BLOCK_START("ImageHelper::load")
+    SDL_Surface *const tmpImage = loadPng(rw);
+    if (!tmpImage)
+    {
+        logger->log("Error, image load failed: %s", IMG_GetError());
+        BLOCK_END("ImageHelper::load")
+        return nullptr;
+    }
+
+    SDL_PixelFormat rgba;
+    rgba.palette = nullptr;
+    rgba.BitsPerPixel = 32;
+    rgba.BytesPerPixel = 4;
+
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+    rgba.Rmask = 0x000000FF;
+    rgba.Gmask = 0x0000FF00;
+    rgba.Bmask = 0x00FF0000;
+    rgba.Amask = 0xFF000000;
+#else
+    rgba.Rmask = 0xFF000000;
+    rgba.Gmask = 0x00FF0000;
+    rgba.Bmask = 0x0000FF00;
+    rgba.Amask = 0x000000FF;
+#endif
+
+    SDL_Surface *const surf = MSDL_ConvertSurface(
+        tmpImage, &rgba, SDL_SWSURFACE);
+    MSDL_FreeSurface(tmpImage);
+
+    uint32_t *const pixels = static_cast<uint32_t *const>(surf->pixels);
+    const int type = dye.getType();
+
+    switch (type)
+    {
+        case 1:
+        {
+            const DyePalette *const pal = dye.getSPalete();
+            if (pal)
+                pal->replaceSColor(pixels, surf->w * surf->h);
+            break;
+        }
+        case 2:
+        {
+            const DyePalette *const pal = dye.getAPalete();
+            if (pal)
+                pal->replaceAColor(pixels, surf->w * surf->h);
+            break;
+        }
+        case 0:
+        default:
+        {
+            dye.normalDye(pixels, surf->w * surf->h);
+            break;
+        }
+    }
+
+    Image *const image = load(surf);
+    MSDL_FreeSurface(surf);
+    BLOCK_END("ImageHelper::load")
+    return image;
+}
+
 SDL_Surface* ImageHelper::convertTo32Bit(SDL_Surface *const tmpImage)
 {
     if (!tmpImage)
@@ -94,7 +162,7 @@ SDL_Surface* ImageHelper::convertTo32Bit(SDL_Surface *const tmpImage)
     return MSDL_ConvertSurface(tmpImage, &RGBAFormat, SDL_SWSURFACE);
 }
 
-void ImageHelper::dumpSurfaceFormat(const SDL_Surface *const image) const
+void ImageHelper::dumpSurfaceFormat(const SDL_Surface *const image)
 {
     if (image->format)
     {
@@ -104,12 +172,15 @@ void ImageHelper::dumpSurfaceFormat(const SDL_Surface *const image) const
         logger->log("Alpha: %d", format->alpha);
 #endif
         logger->log("Loss: %02x, %02x, %02x, %02x",
-            static_cast<int>(format->Rloss), static_cast<int>(format->Gloss),
-            static_cast<int>(format->Bloss), static_cast<int>(format->Aloss));
+            static_cast<unsigned int>(format->Rloss),
+            static_cast<unsigned int>(format->Gloss),
+            static_cast<unsigned int>(format->Bloss),
+            static_cast<unsigned int>(format->Aloss));
         logger->log("Shift: %02x, %02x, %02x, %02x",
-            static_cast<int>(format->Rshift), static_cast<int>(format->Gshift),
-            static_cast<int>(format->Bshift),
-            static_cast<int>(format->Ashift));
+            static_cast<unsigned int>(format->Rshift),
+            static_cast<unsigned int>(format->Gshift),
+            static_cast<unsigned int>(format->Bshift),
+            static_cast<unsigned int>(format->Ashift));
         logger->log("Mask: %08x, %08x, %08x, %08x", format->Rmask,
             format->Gmask, format->Bmask, format->Amask);
     }
@@ -138,3 +209,22 @@ SDL_Surface *ImageHelper::loadPng(SDL_RWops *const rw)
     SDL_RWclose(rw);
     return nullptr;
 }
+
+SDL_Surface *ImageHelper::create32BitSurface(int width,
+                                             int height) const
+{
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+    const int rmask = 0xff000000;
+    const int gmask = 0x00ff0000;
+    const int bmask = 0x0000ff00;
+    const int amask = 0x000000ff;
+#else
+    const int rmask = 0x000000ff;
+    const int gmask = 0x0000ff00;
+    const int bmask = 0x00ff0000;
+    const int amask = 0xff000000;
+#endif
+
+    return MSDL_CreateRGBSurface(SDL_SWSURFACE,
+        width, height, 32, rmask, gmask, bmask, amask);
+}
