@@ -25,7 +25,7 @@
 
 #include "localconsts.h"
 
-#include "render/renderers.h"
+#include "render/rendertype.h"
 
 #include "resources/resource.h"
 
@@ -37,7 +37,7 @@ class Image;
 /**
  * Defines a class for loading and storing images.
  */
-class ImageHelper
+class ImageHelper notfinal
 {
     friend class CompoundSprite;
     friend class Image;
@@ -56,22 +56,18 @@ class ImageHelper
          * @return <code>NULL</code> if an error occurred, a valid pointer
          *         otherwise.
          */
-        Image *load(SDL_RWops *const rw) const A_WARN_UNUSED;
+        Image *load(SDL_RWops *const rw) A_WARN_UNUSED;
+
+        virtual Image *load(SDL_RWops *const rw, Dye const &dye) A_WARN_UNUSED;
 
 #ifdef __GNUC__
-        virtual Image *load(SDL_RWops *const rw, Dye
-                            const &dye) const A_WARN_UNUSED = 0;
-
-        virtual Image *load(SDL_Surface *const) const A_WARN_UNUSED = 0;
+        virtual Image *load(SDL_Surface *const) A_WARN_UNUSED = 0;
 
         virtual Image *createTextSurface(SDL_Surface *const tmpImage,
                                          const int width, const int height,
-                                         float alpha) const A_WARN_UNUSED = 0;
+                                         float alpha) A_WARN_UNUSED = 0;
 #else
-        virtual Image *load(SDL_RWops *rw, Dye const &dye) const A_WARN_UNUSED
-        { return nullptr; }
-
-        virtual Image *load(SDL_Surface *) const A_WARN_UNUSED
+        virtual Image *load(SDL_Surface *) A_WARN_UNUSED
         { return nullptr; }
 
         virtual Image *createTextSurface(SDL_Surface *const tmpImage,
@@ -79,13 +75,21 @@ class ImageHelper
         { return nullptr; }
 #endif
 
+        virtual SDL_Surface *create32BitSurface(int width,
+                                                int height)
+                                                const A_WARN_UNUSED;
+
+        virtual void copySurfaceToImage(Image *const image A_UNUSED,
+                                        const int x A_UNUSED,
+                                        const int y A_UNUSED,
+                                        SDL_Surface *const surface A_UNUSED)
+                                        const
+        { }
+
         static SDL_Surface *convertTo32Bit(SDL_Surface *const tmpImage)
                                            A_WARN_UNUSED;
 
-        void dumpSurfaceFormat(const SDL_Surface *const image) const;
-
-        virtual SDL_Surface *create32BitSurface(int width, int height)
-                                                const A_WARN_UNUSED = 0;
+        static void dumpSurfaceFormat(const SDL_Surface *const image);
 
         static void setEnableAlpha(const bool n)
         { mEnableAlpha = n; }
@@ -98,6 +102,9 @@ class ImageHelper
         virtual RenderType useOpenGL() const A_WARN_UNUSED
         { return mUseOpenGL; }
 
+        virtual void postInit()
+        { }
+
     protected:
         ImageHelper()
         { }
