@@ -28,12 +28,14 @@
 #include "logger.h"
 
 #include "render/mgl.h"
+#include "render/mglcheck.h"
 #include "render/mobileopenglgraphics.h"
+#include "render/modernopenglgraphics.h"
 #include "render/normalopenglgraphics.h"
-#include "render/renderers.h"
 #include "render/safeopenglgraphics.h"
 
 #include "resources/dye.h"
+#include "resources/dyepalette.h"
 #include "resources/image.h"
 
 #include "utils/sdlcheckutils.h"
@@ -48,7 +50,13 @@ int OpenGLImageHelper::mTextureSize = 0;
 bool OpenGLImageHelper::mBlur = true;
 bool OpenGLImageHelper::mUseTextureSampler = false;
 
-Image *OpenGLImageHelper::load(SDL_RWops *const rw, Dye const &dye) const
+OpenGLImageHelper::~OpenGLImageHelper()
+{
+    glDeleteTextures(static_cast<GLsizei>(texturesSize - mFreeTextureIndex),
+        &mTextures[mFreeTextureIndex]);
+}
+
+Image *OpenGLImageHelper::load(SDL_RWops *const rw, Dye const &dye)
 {
     SDL_Surface *const tmpImage = loadPng(rw);
     if (!tmpImage)
@@ -92,14 +100,14 @@ Image *OpenGLImageHelper::load(SDL_RWops *const rw, Dye const &dye) const
     return image;
 }
 
-Image *OpenGLImageHelper::load(SDL_Surface *const tmpImage) const
+Image *OpenGLImageHelper::load(SDL_Surface *const tmpImage)
 {
     return glLoad(tmpImage);
 }
 
 Image *OpenGLImageHelper::createTextSurface(SDL_Surface *const tmpImage,
                                             const int width, const int height,
-                                            const float alpha) const
+                                            const float alpha)
 {
     if (!tmpImage)
         return nullptr;
@@ -110,7 +118,7 @@ Image *OpenGLImageHelper::createTextSurface(SDL_Surface *const tmpImage,
     return img;
 }
 
-int OpenGLImageHelper::powerOfTwo(const int input) const
+int OpenGLImageHelper::powerOfTwo(const int input)
 {
     int value;
     if (mTextureType == GL_TEXTURE_2D)
@@ -126,19 +134,12 @@ int OpenGLImageHelper::powerOfTwo(const int input) const
     return value >= mTextureSize ? mTextureSize : value;
 }
 
-Image *OpenGLImageHelper::glLoad(SDL_Surface *tmpImage,
-                                 int width, int height) const
+SDL_Surface *OpenGLImageHelper::convertSurface(SDL_Surface *tmpImage,
+                                               int width, int height)
 {
     if (!tmpImage)
         return nullptr;
 
-    // Flush current error flag.
-    glGetError();
-
-    if (!width)
-        width = tmpImage->w;
-    if (!height)
-        height = tmpImage->h;
     int realWidth = powerOfTwo(width);
     int realHeight = powerOfTwo(height);
 
@@ -169,14 +170,13 @@ Image *OpenGLImageHelper::glLoad(SDL_Surface *tmpImage,
     amask = 0xff000000;
 #endif
 
-    SDL_Surface *oldImage = nullptr;
     if (tmpImage->format->BitsPerPixel != 32
         || realWidth != width || realHeight != height
         || rmask != tmpImage->format->Rmask
         || gmask != tmpImage->format->Gmask
         || amask != tmpImage->format->Amask)
     {
-        oldImage = tmpImage;
+        SDL_Surface *oldImage = tmpImage;
 #ifdef USE_SDL2
         SDL_SetSurfaceBlendMode(oldImage, SDL_BLENDMODE_NONE);
 #endif
@@ -190,21 +190,27 @@ Image *OpenGLImageHelper::glLoad(SDL_Surface *tmpImage,
         }
         SDL_BlitSurface(oldImage, nullptr, tmpImage, nullptr);
     }
+    return tmpImage;
+}
 
-    GLuint texture;
-    glGenTextures(1, &texture);
+void OpenGLImageHelper::bindTexture(const GLuint texture)
+{
     switch (mUseOpenGL)
     {
 #ifndef ANDROID
         case RENDER_NORMAL_OPENGL:
             NormalOpenGLGraphics::bindTexture(mTextureType, texture);
             break;
+        case RENDER_MODERN_OPENGL:
+            ModernOpenGLGraphics::bindTexture(mTextureType, texture);
+            break;
         case RENDER_SAFE_OPENGL:
             SafeOpenGLGraphics::bindTexture(mTextureType, texture);
             break;
 #else
         case RENDER_NORMAL_OPENGL:
         case RENDER_SAFE_OPENGL:
+        case RENDER_MODERN_OPENGL:
 #endif
         case RENDER_GLES_OPENGL:
             MobileOpenGLGraphics::bindTexture(mTextureType, texture);
@@ -217,11 +223,36 @@ Image *OpenGLImageHelper::glLoad(SDL_Surface *tmpImage,
             logger->log("Unknown OpenGL backend: %d", mUseOpenGL);
             break;
     }
+}
+
+Image *OpenGLImageHelper::glLoad(SDL_Surface *tmpImage,
+                                 int width, int height)
+{
+    if (!tmpImage)
+        return nullptr;
+
+    // Flush current error flag.
+    graphicsManager.getLastError();
+
+    if (!width)
+        width = tmpImage->w;
+    if (!height)
+        height = tmpImage->h;
+
+    SDL_Surface *oldImage = tmpImage;
+    tmpImage = convertSurface(tmpImage, width, height);
+
+    const int realWidth = tmpImage->w;
+    const int realHeight = tmpImage->h;
+
+    const GLuint texture = getNewTexture();
+    bindTexture(texture);
 
     if (SDL_MUSTLOCK(tmpImage))
         SDL_LockSurface(tmpImage);
 
-    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+    if (mUseOpenGL != RENDER_MODERN_OPENGL && mUseOpenGL != RENDER_GLES_OPENGL)
+        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
     if (!mUseTextureSampler)
     {
@@ -244,6 +275,15 @@ Image *OpenGLImageHelper::glLoad(SDL_Surface *tmpImage,
         tmpImage->w, tmpImage->h,
         0, GL_RGBA, GL_UNSIGNED_BYTE, tmpImage->pixels);
 
+#ifdef DEBUG_OPENGL
+//  disabled for now, because debugger cant show it
+//    if (isGLNotNull(mglLabelObject))
+//    {
+//        const char *const text = "image text";
+//        mglLabelObject(GL_TEXTURE, texture, strlen(text), text);
+//    }
+#endif
+
 /*
     GLint compressed;
     glGetTexLevelParameteriv(mTextureType, 0,
@@ -261,16 +301,16 @@ Image *OpenGLImageHelper::glLoad(SDL_Surface *tmpImage,
     if (SDL_MUSTLOCK(tmpImage))
         SDL_UnlockSurface(tmpImage);
 
-    if (oldImage)
+    if (oldImage != tmpImage)
         MSDL_FreeSurface(tmpImage);
 
-    GLenum error = glGetError();
+    GLenum error = graphicsManager.getLastError();
     if (error)
     {
         std::string errmsg = GraphicsManager::errorToString(error);
-        logger->log("Error: Image GL import failed: %s (%d)",
+        logger->log("Error: Image GL import failed: %s (%u)",
             errmsg.c_str(), error);
-        return nullptr;
+//        return nullptr;
     }
 
     return new Image(texture, width, height, realWidth, realHeight);
@@ -290,7 +330,8 @@ void OpenGLImageHelper::initTextureSampler(const GLint id)
     }
 }
 
-SDL_Surface *OpenGLImageHelper::create32BitSurface(int width, int height) const
+SDL_Surface *OpenGLImageHelper::create32BitSurface(int width,
+                                                   int height) const
 {
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
     const int rmask = 0xff000000;
@@ -311,4 +352,50 @@ SDL_Surface *OpenGLImageHelper::create32BitSurface(int width, int height) const
         width, height, 32, rmask, gmask, bmask, amask);
 }
 
+GLuint OpenGLImageHelper::getNewTexture()
+{
+    GLuint texture = mTextures[mFreeTextureIndex];
+    mFreeTextureIndex ++;
+    if (mFreeTextureIndex == texturesSize)
+    {
+        mFreeTextureIndex = 0;
+        postInit();
+    }
+    return texture;
+}
+
+void OpenGLImageHelper::postInit()
+{
+    glGenTextures(texturesSize, &mTextures[mFreeTextureIndex]);
+}
+
+void OpenGLImageHelper::invalidate(const GLuint textureId)
+{
+    if (isGLNotNull(mglInvalidateTexImage))
+    {
+        logger->log("invalidate: %u", textureId);
+        mglInvalidateTexImage(textureId, 0);
+    }
+}
+
+void OpenGLImageHelper::copySurfaceToImage(Image *const image,
+                                           const int x, const int y,
+                                           SDL_Surface *surface) const
+{
+    if (!surface || !image)
+        return;
+
+    SDL_Surface *const oldSurface = surface;
+    surface = convertSurface(surface, surface->w, surface->h);
+
+    mglTextureSubImage2D(image->mGLImage,
+        mTextureType, 0,
+        x, y,
+        surface->w, surface->h,
+        GL_RGBA, GL_UNSIGNED_BYTE, surface->pixels);
+
+    if (surface != oldSurface)
+        MSDL_FreeSurface(surface);
+}
+
 #endif
