@@ -58,14 +58,14 @@ class OpenGLImageHelper final : public ImageHelper
     friend class Image;
 
     public:
-        OpenGLImageHelper()
+        OpenGLImageHelper() :
+            mFreeTextureIndex(0)
         {
         }
 
         A_DELETE_COPY(OpenGLImageHelper)
 
-        ~OpenGLImageHelper()
-        { }
+        ~OpenGLImageHelper();
 
         /**
          * Loads an image from an SDL_RWops structure and recolors it.
@@ -77,18 +77,18 @@ class OpenGLImageHelper final : public ImageHelper
          *         otherwise.
          */
         Image *load(SDL_RWops *const rw,
-                    Dye const &dye) const override final A_WARN_UNUSED;
+                    Dye const &dye) override final A_WARN_UNUSED;
 
         /**
          * Loads an image from an SDL surface.
          */
-        Image *load(SDL_Surface *const tmpImage) const
+        Image *load(SDL_Surface *const tmpImage)
                     override final A_WARN_UNUSED;
 
         Image *createTextSurface(SDL_Surface *const tmpImage,
                                  const int width, const int height,
                                  const float alpha)
-                                 const override final A_WARN_UNUSED;
+                                 override final A_WARN_UNUSED;
 
         // OpenGL only public functions
 
@@ -116,17 +116,36 @@ class OpenGLImageHelper final : public ImageHelper
         static void setUseTextureSampler(const bool b)
         { mUseTextureSampler = b; }
 
+        static void invalidate(const GLuint textureId);
+
+        static void bindTexture(const GLuint texture);
+
         SDL_Surface *create32BitSurface(int width,
                                         int height) const override final;
 
+        void postInit() override final;
+
+        void copySurfaceToImage(Image *const image,
+                                const int x, const int y,
+                                SDL_Surface *surface) const;
+
     protected:
         /**
          * Returns the first power of two equal or bigger than the input.
          */
-        int powerOfTwo(const int input) const A_WARN_UNUSED;
+        static int powerOfTwo(const int input) A_WARN_UNUSED;
+
+        static SDL_Surface *convertSurface(SDL_Surface *tmpImage,
+                                           int width, int height);
 
         Image *glLoad(SDL_Surface *tmpImage,
-                      int width = 0, int height = 0) const A_WARN_UNUSED;
+                      int width = 0, int height = 0) A_WARN_UNUSED;
+
+        GLuint getNewTexture();
+
+        static const size_t texturesSize = 10;
+        size_t mFreeTextureIndex;
+        GLuint mTextures[texturesSize];
 
         static int mTextureSize;
         static bool mBlur;
