@@ -25,6 +25,7 @@
 #include "resources/sdl2imagehelper.h"
 
 #include "resources/dye.h"
+#include "resources/dyepalette.h"
 #include "resources/resourcemanager.h"
 
 #include "logger.h"
@@ -43,69 +44,7 @@ bool SDLImageHelper::mEnableAlphaCache = false;
 SDL_Renderer *SDLImageHelper::mRenderer = nullptr;
 #endif
 
-Image *SDLImageHelper::load(SDL_RWops *const rw, Dye const &dye) const
-{
-    SDL_Surface *const tmpImage = loadPng(rw);
-    if (!tmpImage)
-    {
-        logger->log("Error, image load failed: %s", IMG_GetError());
-        return nullptr;
-    }
-
-    SDL_PixelFormat rgba;
-    rgba.palette = nullptr;
-    rgba.BitsPerPixel = 32;
-    rgba.BytesPerPixel = 4;
-
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-    rgba.Rmask = 0x000000FF;
-    rgba.Gmask = 0x0000FF00;
-    rgba.Bmask = 0x00FF0000;
-    rgba.Amask = 0xFF000000;
-#else
-    rgba.Rmask = 0xFF000000;
-    rgba.Gmask = 0x00FF0000;
-    rgba.Bmask = 0x0000FF00;
-    rgba.Amask = 0x000000FF;
-#endif
-
-    SDL_Surface *const surf = MSDL_ConvertSurface(
-        tmpImage, &rgba, SDL_SWSURFACE);
-    MSDL_FreeSurface(tmpImage);
-
-    uint32_t *pixels = static_cast<uint32_t *>(surf->pixels);
-    const int type = dye.getType();
-
-    switch (type)
-    {
-        case 1:
-        {
-            const DyePalette *const pal = dye.getSPalete();
-            if (pal)
-                pal->replaceSColor(pixels, surf->w * surf->h);
-            break;
-        }
-        case 2:
-        {
-            const DyePalette *const pal = dye.getAPalete();
-            if (pal)
-                pal->replaceAColor(pixels, surf->w * surf->h);
-            break;
-        }
-        case 0:
-        default:
-        {
-            dye.normalDye(pixels, surf->w * surf->h);
-            break;
-        }
-    }
-
-    Image *const image = load(surf);
-    MSDL_FreeSurface(surf);
-    return image;
-}
-
-Image *SDLImageHelper::load(SDL_Surface *const tmpImage) const
+Image *SDLImageHelper::load(SDL_Surface *const tmpImage)
 {
     return _SDLload(tmpImage);
 }
@@ -113,7 +52,7 @@ Image *SDLImageHelper::load(SDL_Surface *const tmpImage) const
 Image *SDLImageHelper::createTextSurface(SDL_Surface *const tmpImage,
                                          const int width A_UNUSED,
                                          const int height A_UNUSED,
-                                         const float alpha) const
+                                         const float alpha)
 {
     if (!tmpImage)
         return nullptr;
@@ -132,7 +71,7 @@ SDL_Surface* SDLImageHelper::SDLDuplicateSurface(SDL_Surface *const tmpImage)
     return MSDL_ConvertSurface(tmpImage, tmpImage->format, SDL_SWSURFACE);
 }
 
-Image *SDLImageHelper::_SDLload(SDL_Surface *tmpImage) const
+Image *SDLImageHelper::_SDLload(SDL_Surface *tmpImage)
 {
     if (!tmpImage)
         return nullptr;
@@ -145,24 +84,6 @@ Image *SDLImageHelper::_SDLload(SDL_Surface *tmpImage) const
     return new Image(texture, tmpImage->w, tmpImage->h);
 }
 
-SDL_Surface *SDLImageHelper::create32BitSurface(int width, int height) const
-{
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-    const int rmask = 0xff000000;
-    const int gmask = 0x00ff0000;
-    const int bmask = 0x0000ff00;
-    const int amask = 0x000000ff;
-#else
-    const int rmask = 0x000000ff;
-    const int gmask = 0x0000ff00;
-    const int bmask = 0x00ff0000;
-    const int amask = 0xff000000;
-#endif
-
-    return MSDL_CreateRGBSurface(SDL_SWSURFACE,
-        width, height, 32, rmask, gmask, bmask, amask);
-}
-
 int SDLImageHelper::combineSurface(SDL_Surface *restrict const src,
                                    SDL_Rect *restrict const srcrect,
                                    SDL_Surface *restrict const dst,
@@ -173,4 +94,22 @@ int SDLImageHelper::combineSurface(SDL_Surface *restrict const src,
     return 1;
 }
 
+void SDLImageHelper::copySurfaceToImage(Image *const image,
+                                        const int x, const int y,
+                                        SDL_Surface *const surface) const
+{
+    if (!image || !surface)
+        return;
+
+    SDL_SetSurfaceAlphaMod(surface, SDL_ALPHA_OPAQUE);
+    SDL_SetSurfaceBlendMode(surface, SDL_BLENDMODE_NONE);
+
+    SDL_Rect rect =
+    {
+        x, y,
+        surface->w, surface->h
+    };
+
+    SDL_BlitSurface(surface, nullptr, image->mSDLSurface, &rect);
+}
 #endif  // USE_SDL2
