@@ -51,28 +51,21 @@ class SDLImageHelper final : public ImageHelper
         ~SDLImageHelper()
         { }
 
-        /**
-         * Loads an image from an SDL_RWops structure and recolors it.
-         *
-         * @param rw         The SDL_RWops to load the image from.
-         * @param dye        The dye used to recolor the image.
-         *
-         * @return <code>NULL</code> if an error occurred, a valid pointer
-         *         otherwise.
-         */
-        Image *load(SDL_RWops *const rw,
-                    Dye const &dye) const override final A_WARN_UNUSED;
-
         /**
          * Loads an image from an SDL surface.
          */
-        Image *load(SDL_Surface *const tmpImage) const
+        Image *load(SDL_Surface *const tmpImage)
                     override final A_WARN_UNUSED;
 
         Image *createTextSurface(SDL_Surface *const tmpImage,
                                  const int width, const int height,
                                  const float alpha)
-                                 const override final A_WARN_UNUSED;
+                                 override final A_WARN_UNUSED;
+
+        void copySurfaceToImage(Image *const image,
+                                const int x, const int y,
+                                SDL_Surface *const surface)
+                                const override final;
 
         static void SDLSetEnableAlphaCache(const bool n)
         { mEnableAlphaCache = n; }
@@ -83,9 +76,6 @@ class SDLImageHelper final : public ImageHelper
         static SDL_Surface* SDLDuplicateSurface(SDL_Surface *const tmpImage)
                                                 A_WARN_UNUSED;
 
-        SDL_Surface *create32BitSurface(int width, int height) const
-                                        override final;
-
         static int combineSurface(SDL_Surface *restrict const src,
                                   SDL_Rect *restrict const srcrect,
                                   SDL_Surface *restrict const dst,
@@ -98,7 +88,7 @@ class SDLImageHelper final : public ImageHelper
 
     protected:
         /** SDL_Surface to SDL_Surface Image loader */
-        Image *_SDLload(SDL_Surface *tmpImage) const A_WARN_UNUSED;
+        Image *_SDLload(SDL_Surface *tmpImage) A_WARN_UNUSED;
 
         static bool mEnableAlphaCache;
 #ifdef USE_SDL2
