@@ -24,11 +24,11 @@
 
 #include "resources/sdlimagehelper.h"
 
-#include "resources/dye.h"
-
 #include "logger.h"
 #include "main.h"
 
+#include "resources/dye.h"
+#include "resources/dyepalette.h"
 #include "resources/image.h"
 
 #include "utils/sdlcheckutils.h"
@@ -40,7 +40,7 @@
 
 bool SDLImageHelper::mEnableAlphaCache = false;
 
-Image *SDLImageHelper::load(SDL_RWops *const rw, Dye const &dye) const
+Image *SDLImageHelper::load(SDL_RWops *const rw, Dye const &dye)
 {
     SDL_Surface *const tmpImage = loadPng(rw);
     if (!tmpImage)
@@ -104,7 +104,7 @@ Image *SDLImageHelper::load(SDL_RWops *const rw, Dye const &dye) const
     return image;
 }
 
-Image *SDLImageHelper::load(SDL_Surface *const tmpImage) const
+Image *SDLImageHelper::load(SDL_Surface *const tmpImage)
 {
     return _SDLload(tmpImage);
 }
@@ -112,13 +112,13 @@ Image *SDLImageHelper::load(SDL_Surface *const tmpImage) const
 Image *SDLImageHelper::createTextSurface(SDL_Surface *const tmpImage,
                                          const int width A_UNUSED,
                                          const int height A_UNUSED,
-                                         const float alpha) const
+                                         const float alpha)
 {
     if (!tmpImage)
         return nullptr;
 
     bool hasAlpha = false;
-    const int sz = tmpImage->w * tmpImage->h;
+    const size_t sz = tmpImage->w * tmpImage->h;
 
     // The alpha channel to be filled with alpha values
     uint8_t *alphaChannel = new uint8_t[sz];
@@ -126,7 +126,7 @@ Image *SDLImageHelper::createTextSurface(SDL_Surface *const tmpImage,
     const SDL_PixelFormat *const fmt = tmpImage->format;
     if (fmt->Amask)
     {
-        for (int i = 0; i < sz; ++ i)
+        for (size_t i = 0; i < sz; ++ i)
         {
             uint32_t c = (static_cast<uint32_t*>(tmpImage->pixels))[i];
 
@@ -185,7 +185,7 @@ SDL_Surface* SDLImageHelper::SDLDuplicateSurface(SDL_Surface *const tmpImage)
     return MSDL_ConvertSurface(tmpImage, tmpImage->format, SDL_SWSURFACE);
 }
 
-Image *SDLImageHelper::_SDLload(SDL_Surface *tmpImage) const
+Image *SDLImageHelper::_SDLload(SDL_Surface *tmpImage)
 {
     if (!tmpImage)
         return nullptr;
@@ -202,7 +202,7 @@ Image *SDLImageHelper::_SDLload(SDL_Surface *tmpImage) const
         converted = true;
     }
 
-    const int sz = tmpImage->w * tmpImage->h;
+    const size_t sz = tmpImage->w * tmpImage->h;
 
     // The alpha channel to be filled with alpha values
     uint8_t *alphaChannel = new uint8_t[sz];
@@ -217,7 +217,7 @@ Image *SDLImageHelper::_SDLload(SDL_Surface *tmpImage) const
             const uint8_t ashift = fmt->Ashift;
             const uint8_t aloss = fmt->Aloss;
             const uint32_t *pixels = static_cast<uint32_t*>(tmpImage->pixels);
-            for (int i = 0; i < sz; ++ i)
+            for (size_t i = 0; i < sz; ++ i)
             {
                 const unsigned v = (pixels[i] & amask) >> ashift;
                 const uint8_t a = static_cast<const uint8_t>((v << aloss)
@@ -276,24 +276,6 @@ Image *SDLImageHelper::_SDLload(SDL_Surface *tmpImage) const
     return new Image(image, hasAlpha, alphaChannel);
 }
 
-SDL_Surface *SDLImageHelper::create32BitSurface(int width, int height) const
-{
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-    const int rmask = 0xff000000;
-    const int gmask = 0x00ff0000;
-    const int bmask = 0x0000ff00;
-    const int amask = 0x000000ff;
-#else
-    const int rmask = 0x000000ff;
-    const int gmask = 0x0000ff00;
-    const int bmask = 0x00ff0000;
-    const int amask = 0xff000000;
-#endif
-
-    return MSDL_CreateRGBSurface(SDL_SWSURFACE,
-        width, height, 32, rmask, gmask, bmask, amask);
-}
-
 int SDLImageHelper::combineSurface(SDL_Surface *restrict const src,
                                    SDL_Rect *restrict const srcrect,
                                    SDL_Surface *restrict const dst,
@@ -302,4 +284,21 @@ int SDLImageHelper::combineSurface(SDL_Surface *restrict const src,
     return SDL_gfxBlitRGBA(src, srcrect, dst, dstrect);
 }
 
+void SDLImageHelper::copySurfaceToImage(Image *const image,
+                                        const int x, const int y,
+                                        SDL_Surface *const surface) const
+{
+    if (!image || !surface)
+        return;
+
+    SDL_SetAlpha(surface, 0, SDL_ALPHA_OPAQUE);
+    SDL_Rect rect =
+    {
+        static_cast<int16_t>(x), static_cast<int16_t>(y),
+        static_cast<uint16_t>(surface->w), static_cast<uint16_t>(surface->h)
+    };
+
+    SDL_BlitSurface(surface, nullptr, image->mSDLSurface, &rect);
+}
+
 #endif  // USE_SDL2
