@@ -24,7 +24,7 @@
 #define RESOURCES_SDLIMAGEHELPER_H
 
 #ifdef USE_SDL2
-#include "sdl2imagehelper.h"
+#include "resources/sdl2imagehelper.h"
 
 #else
 
@@ -62,18 +62,22 @@ class SDLImageHelper final : public ImageHelper
          *         otherwise.
          */
         Image *load(SDL_RWops *const rw,
-                    Dye const &dye) const override final A_WARN_UNUSED;
+                    Dye const &dye) override final A_WARN_UNUSED;
 
         /**
          * Loads an image from an SDL surface.
          */
-        Image *load(SDL_Surface *const tmpImage) const
-                    override final A_WARN_UNUSED;
+        Image *load(SDL_Surface *const tmpImage) override final A_WARN_UNUSED;
 
         Image *createTextSurface(SDL_Surface *const tmpImage,
                                  const int width, const int height,
                                  const float alpha)
-                                 const override final A_WARN_UNUSED;
+                                 override final A_WARN_UNUSED;
+
+        void copySurfaceToImage(Image *const image,
+                                const int x, const int y,
+                                SDL_Surface *const surface)
+                                const override final;
 
         static void SDLSetEnableAlphaCache(const bool n)
         { mEnableAlphaCache = n; }
@@ -84,9 +88,6 @@ class SDLImageHelper final : public ImageHelper
         static SDL_Surface* SDLDuplicateSurface(SDL_Surface *const tmpImage)
                                                 A_WARN_UNUSED;
 
-        SDL_Surface *create32BitSurface(int width,
-                                        int height) const override final;
-
         static int combineSurface(SDL_Surface *restrict const src,
                                   SDL_Rect *restrict const srcrect,
                                   SDL_Surface *restrict const dst,
@@ -94,7 +95,7 @@ class SDLImageHelper final : public ImageHelper
 
     protected:
         /** SDL_Surface to SDL_Surface Image loader */
-        Image *_SDLload(SDL_Surface *tmpImage) const A_WARN_UNUSED;
+        Image *_SDLload(SDL_Surface *tmpImage) A_WARN_UNUSED;
 
         static bool mEnableAlphaCache;
 };
