@@ -136,7 +136,7 @@ SubImage::SubImage(Image *const parent, SDL_Surface *const image,
 #ifdef USE_OPENGL
 SubImage::SubImage(Image *const parent, const GLuint image,
                    const int x, const int y, const int width, const int height,
-                   const int texWidth, const int texHeight):
+                   const int texWidth, const int texHeight) :
     Image(image, width, height, texWidth, texHeight),
     mInternalBounds(),
     mParent(parent)
@@ -211,3 +211,10 @@ Image *SubImage::getSubImage(const int x, const int y,
     else
         return nullptr;
 }
+
+#ifdef USE_OPENGL
+void SubImage::decRef()
+{
+    Resource::decRef();
+}
+#endif
