@@ -24,6 +24,7 @@
 
 #include "configuration.h"
 
+#include "being/attributes.h"
 #include "being/localplayer.h"
 #include "being/playerinfo.h"
 
@@ -36,6 +37,8 @@
 
 #include "debug.h"
 
+SpellManager *spellManager = nullptr;
+
 SpellManager::SpellManager() :
     mSpells(),
     mSpellsVector()
@@ -52,7 +55,7 @@ SpellManager::~SpellManager()
 
 TextCommand* SpellManager::getSpell(const int spellId) const
 {
-    if (spellId < 0 || static_cast<unsigned int>(spellId) >= mSpells.size())
+    if (spellId < 0 || static_cast<size_t>(spellId) >= mSpells.size())
         return nullptr;
 
     const std::map<unsigned int, TextCommand*>::const_iterator
@@ -128,7 +131,7 @@ void SpellManager::useItem(const int itemId) const
 
 void SpellManager::invoke(const int spellId) const
 {
-    if (!player_node)
+    if (!localPlayer)
         return;
 
     const TextCommand *const spell = getSpell(spellId);
@@ -142,16 +145,16 @@ void SpellManager::invoke(const int spellId) const
         || (Net::getPlayerHandler()->canUseMagic()
         && PlayerInfo::getSkillLevel(SKILL_MAGIC)
         >= static_cast<signed>(spell->getBaseLvl())
-        && PlayerInfo::getSkillLevel(
-        spell->getSchool()) >= static_cast<signed>(spell->getSchoolLvl())
-        && PlayerInfo::getAttribute(PlayerInfo::MP) >= spell->getMana()))
+        && PlayerInfo::getSkillLevel(static_cast<int>(
+        spell->getSchool())) >= static_cast<signed>(spell->getSchoolLvl())
+        && PlayerInfo::getAttribute(Attributes::MP) >= spell->getMana()))
     {
-        const Being *const target = player_node->getTarget();
+        const Being *const target = localPlayer->getTarget();
         if (spell->getTargetType() == NOTARGET)
         {
             invokeSpell(spell);
         }
-        if ((target && (target->getType() != Being::MONSTER
+        if ((target && (target->getType() != ActorType::MONSTER
             || spell->getCommandType() == TEXT_COMMAND_TEXT))
             && (spell->getTargetType() == ALLOWTARGET
             || spell->getTargetType() == NEEDTARGET))
@@ -165,7 +168,7 @@ void SpellManager::invoke(const int spellId) const
     }
 }
 
-void SpellManager::invokeSpell(const TextCommand *const spell) const
+void SpellManager::invokeSpell(const TextCommand *const spell)
 {
     if (!chatWindow || !spell)
         return;
@@ -173,7 +176,7 @@ void SpellManager::invokeSpell(const TextCommand *const spell) const
 }
 
 void SpellManager::invokeSpell(const TextCommand *const spell,
-                               const Being *const target) const
+                               const Being *const target)
 {
     if (!chatWindow || !spell || !target)
         return;
@@ -181,9 +184,9 @@ void SpellManager::invokeSpell(const TextCommand *const spell,
 }
 
 std::string SpellManager::parseCommand(std::string command,
-                                       const Being *const target) const
+                                       const Being *const target)
 {
-    if (!player_node)
+    if (!localPlayer)
         return command;
 
     std::string name;
@@ -193,12 +196,12 @@ std::string SpellManager::parseCommand(std::string command,
     if (target)
     {
         name = target->getName();
-        name2 = target->getName();
+        name2 = name;
         id = toString(target->getId());
     }
     else
     {
-        name2 = player_node->getName();
+        name2 = localPlayer->getName();
     }
 
     bool found = false;
@@ -399,6 +402,9 @@ void SpellManager::swap(const int id1, const int id2)
 {
     TextCommand *const spell1 = mSpells[id1];
     TextCommand *const spell2 = mSpells[id2];
+    if (!spell1 || !spell2)
+        return;
+
     // swap in map
     mSpells[id1] = spell2;
     mSpells[id2] = spell1;
