@@ -1,5 +1,5 @@
 /*
- *  Retrieve string pasted depending on OS mechanisms.
+ *  The ManaPlus Client
  *  Copyright (C) 2001-2010  Wormux Team
  *  Copyright (C) 2011-2014  The ManaPlus Developers
  *
@@ -77,21 +77,21 @@ bool retrieveBuffer(std::string& text, size_t& pos)
     HANDLE h = GetClipboardData(CF_UNICODETEXT);
     if (h)
     {
-        LPCWSTR data = (LPCWSTR)GlobalLock(h);
+        LPCWSTR data = static_cast<LPCWSTR>(GlobalLock(h));
 
         if (data)
         {
-            const int len = WideCharToMultiByte(CP_UTF8, 0, data, -1,
+            const size_t len = WideCharToMultiByte(CP_UTF8, 0, data, -1,
                 nullptr, 0, nullptr, nullptr);
             if (len > 0)
             {
                 // Convert from UTF-16 to UTF-8
                 void *temp = calloc(len, 1);
                 if (WideCharToMultiByte(CP_UTF8, 0, data, -1,
-                    (LPSTR)temp, len, nullptr, nullptr))
+                    static_cast<LPSTR>(temp), len, nullptr, nullptr))
                 {
-                    text.insert(pos, (char*)temp);
-                    pos += len-1;
+                    text.insert(pos, static_cast<char*>(temp));
+                    pos += len - 1;
                 }
                 free(temp);
                 ret = true;
@@ -105,7 +105,7 @@ bool retrieveBuffer(std::string& text, size_t& pos)
 
         if (h)
         {
-            const char *const data = (char*)GlobalLock(h);
+            const char *const data = static_cast<char*>(GlobalLock(h));
             if (data)
             {
                 text.insert(pos, data);
@@ -128,8 +128,8 @@ bool sendBuffer(std::string& text)
         return false;
 
     HANDLE h = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE,
-        wCharsLen * sizeof(WCHAR));
-    WCHAR *const out = (WCHAR*)GlobalLock(h);
+        static_cast<size_t>(wCharsLen) * sizeof(WCHAR));
+    WCHAR *const out = static_cast<WCHAR*>(GlobalLock(h));
 
     MultiByteToWideChar(CP_UTF8, 0, text.c_str(), -1, out, wCharsLen);
 
@@ -441,14 +441,21 @@ static bool runxsel(std::string& text, const char *p1, const char *p2)
             }
             close(fd[0]);
         }
+
+        const char *const xselPath =
+#if defined __OpenBSD__ || defined __FreeBSD__ || defined __DragonFly__
+            "/usr/local/bin/xsel";
+#else
+            "/usr/bin/xsel";
+#endif
         if (p2)
         {
-            execl("/usr/bin/xsel", "xsel", p1, p2,
+            execl(xselPath, "xsel", p1, p2,
                 static_cast<char *>(nullptr));
         }
         else
         {
-            execl("/usr/bin/xsel", "xsel", p1,
+            execl(xselPath, "xsel", p1,
                 static_cast<char *>(nullptr));
         }
 
