@@ -24,6 +24,7 @@
 
 #include "client.h"
 #include "logger.h"
+#include "settings.h"
 
 #include "utils/stringutils.h"
 
@@ -38,7 +39,7 @@ namespace
 void Fuzzer::init()
 {
     fuzz = new Logger;
-    fuzz->setLogFile(client->getLocalDataDirectory() + "/fuzzer.log");
+    fuzz->setLogFile(settings.localDataDir + "/fuzzer.log");
     unsigned int sr = time(nullptr);
 
     fuzz->log("Srand: %u", sr);
