@@ -32,7 +32,7 @@
 
 #include "debug.h"
 
-static int UTF8_MAX_SIZE = 10;
+static size_t UTF8_MAX_SIZE = 10;
 
 std::string &trim(std::string &str)
 {
@@ -73,7 +73,7 @@ unsigned int atox(const std::string &str)
     return value;
 }
 
-const char *ipToString(const int address)
+const char *ipToString(const uint32_t address)
 {
     static char asciiIP[18];
 
@@ -92,7 +92,7 @@ std::string strprintf(const char *const format, ...)
     char buf[257];
     va_list(args);
     va_start(args, format);
-    int nb = vsnprintf(buf, 256, format, args);
+    size_t nb = vsnprintf(buf, 256, format, args);
     buf[256] = 0;
     va_end(args);
     if (nb < 256)
@@ -189,7 +189,7 @@ size_t findI(std::string str, std::string subStr)
     return str.find(subStr);
 }
 
-size_t findI(std::string text, StringVect &list)
+size_t findI(std::string text, const StringVect &list)
 {
     toLower(text);
     FOR_EACH (StringVectCIter, i, list)
@@ -203,8 +203,8 @@ size_t findI(std::string text, StringVect &list)
     return std::string::npos;
 }
 
-int base = 94;
-int start = 33;
+unsigned int base = 94;
+unsigned int start = 33;
 
 const std::string encodeStr(unsigned int value, const unsigned int size)
 {
@@ -283,7 +283,7 @@ std::string removeSpriteIndex(std::string str)
     return str;
 }
 
-const char* getSafeUtf8String(std::string text)
+const char* getSafeUtf8String(const std::string &text)
 {
     const size_t sz = text.size();
     const size_t size = sz + UTF8_MAX_SIZE;
@@ -573,9 +573,12 @@ std::string stringToHexPath(const std::string &str)
     if (str.empty())
         return "";
 
-    std::string hex = strprintf("%%%2x/", static_cast<int>(str[0]));
-    for (unsigned f = 1, sz = static_cast<unsigned>(str.size()); f < sz; f ++)
-        hex.append(strprintf("%%%2x", static_cast<int>(str[f])));
+    std::string hex = strprintf("%%%2x/", static_cast<unsigned int>(str[0]));
+    for (unsigned f = 1, sz = static_cast<unsigned int>(str.size());
+         f < sz; f ++)
+    {
+        hex.append(strprintf("%%%2x", static_cast<unsigned int>(str[f])));
+    }
     return hex;
 }
 
@@ -721,3 +724,9 @@ bool isDigit(const std::string &str)
     }
     return true;
 }
+
+void secureChatCommand(std::string &str)
+{
+    if (str[0] == '/' || str[0] == '@' || str[0] == '#')
+        str = "_" + str;
+}
