@@ -90,7 +90,7 @@ std::string toStringPrint(const unsigned int val);
  * @param address the address to convert to a string
  * @return the string representation of the address
  */
-const char *ipToString(const int address) A_WARN_UNUSED;
+const char *ipToString(const uint32_t address) A_WARN_UNUSED;
 
 /**
  * A safe version of sprintf that returns a std::string of the result.
@@ -138,7 +138,7 @@ bool isWordSeparator(const signed char chr) A_WARN_UNUSED;
 
 size_t findI(std::string str, std::string subStr) A_WARN_UNUSED;
 
-size_t findI(std::string text, StringVect &list) A_WARN_UNUSED;
+size_t findI(std::string text, const StringVect &list) A_WARN_UNUSED;
 
 const std::string encodeStr(unsigned int value,
                             const unsigned int size = 0) A_WARN_UNUSED;
@@ -149,7 +149,7 @@ std::string extractNameFromSprite(std::string str) A_WARN_UNUSED;
 
 std::string removeSpriteIndex(std::string str) A_WARN_UNUSED;
 
-const char* getSafeUtf8String(std::string text) A_WARN_UNUSED;
+const char* getSafeUtf8String(const std::string &text) A_WARN_UNUSED;
 
 void getSafeUtf8String(std::string text, char *const buf);
 
@@ -236,4 +236,6 @@ std::string decodeLinkText(std::string data);
 
 bool isDigit(const std::string &str);
 
+void secureChatCommand(std::string &str);
+
 #endif  // UTILS_STRINGUTILS_H
